@@ -22,9 +22,9 @@
  *
  * Contributor(s): Miguel Rodriguez Perez
  *
- * $Revision: 29230 $
+ * $Revision: 29536 $
  * $Author: rjongbloed $
- * $Date: 2013-03-10 20:20:22 -0500 (Sun, 10 Mar 2013) $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef OPAL_CODEC_ECHOCANCEL_H
@@ -34,7 +34,9 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
+
+#if OPAL_AEC
 
 #include <rtp/rtp.h>
 #include <ptclib/qchannel.h>
@@ -115,6 +117,8 @@ protected:
 };
 
 
+#endif // OPAL_AEC
+
 #endif // OPAL_CODEC_ECHOCANCEL_H
 
 
