@@ -31,9 +31,9 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Revision: 29223 $
+ * $Revision: 34175 $
  * $Author: rjongbloed $
- * $Date: 2013-03-07 18:00:29 -0600 (Thu, 07 Mar 2013) $
+ * $Date: 2015-10-10 11:27:32 +0100 (Sat, 10 Oct 2015) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_HPP
@@ -44,6 +44,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
+#include <stdio.h>
 
 #include <map>
 #include <string>
@@ -101,9 +102,9 @@
 class PluginCodec_RTP
 {
     unsigned char * m_packet;
-    unsigned        m_maxSize;
-    unsigned        m_headerSize;
-    unsigned        m_payloadSize;
+    size_t          m_maxSize;
+    size_t          m_headerSize;
+    size_t          m_payloadSize;
 
   public:
     PluginCodec_RTP(const void * packet, unsigned size)
@@ -114,15 +115,15 @@ class PluginCodec_RTP
     {
     }
 
-    __inline unsigned GetMaxSize() const           { return m_maxSize; }
-    __inline unsigned GetPacketSize() const        { return m_headerSize+m_payloadSize; }
-    __inline unsigned GetHeaderSize() const        { return m_headerSize; }
-    __inline unsigned GetPayloadSize() const       { return m_payloadSize; }
-    __inline bool     SetPayloadSize(unsigned size)
+    __inline size_t GetMaxSize() const           { return m_maxSize; }
+    __inline size_t GetPacketSize() const        { return m_headerSize+m_payloadSize; }
+    __inline size_t GetHeaderSize() const        { return m_headerSize; }
+    __inline size_t GetPayloadSize() const       { return m_payloadSize; }
+    __inline bool   SetPayloadSize(size_t size)
     {
       if (m_headerSize+size > m_maxSize)
         return false;
-      m_payloadSize = size;
+      m_payloadSize = (unsigned int)size;
       return true;
     }
 
@@ -298,16 +299,24 @@ class PluginCodec_Utilities
         unsigned m_macroblocks;
       } MaxVideoResolutions[] = {
         #define OPAL_PLUGIN_CLAMPED_RESOLUTION(width, height) { width, height, ((width+15)/16) * ((height+15)/16) }
-        OPAL_PLUGIN_CLAMPED_RESOLUTION(2816, 2304),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION(1920, 1080),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION(1408, 1152),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION(1280,  720),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION( 704,  576),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION( 640,  480),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION( 352,  288),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION( 320,  240),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION( 176,  144),
-        OPAL_PLUGIN_CLAMPED_RESOLUTION( 128,   96)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(4096, 2304),   // 36864 - Cinema 16:9    (H.264 Level 5.1 - 5.2)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(4096, 2160),   // 34560 - DCI HD
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(3840, 2160),   // 32400 -  4k UHDTV
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(3672, 1536),   // 22080 - Cinema 2.39:1 (H.264 Level 5)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(2048, 1080),   //  8704 - 2Kx1080       (H.264 Level 4.2)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(2048, 1024),   //  8192 - 2Kx1K         (H.264 Level 4 - 4.1)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(1920, 1080),   //  8100 - 1080p HD
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(1408, 1152),   //  6336 - 16CIF
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(1280, 1024),   //  5120 - SXGA          (H.264 Level 3.2)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION(1280,  720),   //  3600 - 720p HD       (H.264 Level 3.1)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 720,  576),   //  1620 - 625 SD        (H.264 Level 2.2 - 3)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 704,  576),   //  1584 - 4CIF
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 640,  480),   //  1200 - VGA
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 352,  576),   //   792 - 625 HHR       (H.264 Level 2.1)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 352,  288),   //   396 - CIF           (H.264 Level 1.1 - 2)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 320,  240),   //   300 - QVGA
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 176,  144),   //    99 - QCIF          (H.264 Level 1, 1b)
+        OPAL_PLUGIN_CLAMPED_RESOLUTION( 128,   96)    //    48 - SQCIF
       };
       static size_t const LastMaxVideoResolutions = sizeof(MaxVideoResolutions)/sizeof(MaxVideoResolutions[0]) - 1;
 
@@ -334,11 +343,41 @@ class PluginCodec_Utilities
           ++index;
       }
 
+      PTRACE(4, "Plugin", "ClampResolution: idx=" << index << ' '
+             << width << 'x' << height << " > " << maxFrameSize << "mb reduced to "
+             << MaxVideoResolutions[index].m_width << 'x' << MaxVideoResolutions[index].m_height
+             << '=' << MaxVideoResolutions[index].m_macroblocks << "mb");
       width = MaxVideoResolutions[index].m_width;
       height = MaxVideoResolutions[index].m_height;
       maxFrameSize = MaxVideoResolutions[index].m_macroblocks;
       return true;
     }
+
+    static bool ClampResolution(
+			PluginCodec_OptionMapBase & original,
+			PluginCodec_OptionMapBase & changed,
+			unsigned maxWidth,
+			unsigned maxHeight,
+      unsigned & maxMacroBlocks,
+      const char * widthKey = PLUGINCODEC_OPTION_FRAME_WIDTH,
+      const char * heightKey = PLUGINCODEC_OPTION_FRAME_HEIGHT,
+      const char * maxWidthKey = PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH,
+      const char * maxHeightKey = PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT,
+      const char * minWidthKey = PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH,
+      const char * minHeightKey = PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT
+    )
+		{
+      if (!ClampResolution(maxWidth, maxHeight, maxMacroBlocks))
+				return false;
+
+      ClampMax(maxWidth,  original, changed, widthKey);
+      ClampMax(maxHeight, original, changed, heightKey);
+      ClampMax(maxWidth,  original, changed, maxWidthKey);
+      ClampMax(maxHeight, original, changed, maxHeightKey);
+      ClampMax(maxWidth,  original, changed, minWidthKey);
+      ClampMax(maxHeight, original, changed, minHeightKey);
+			return true;
+		}
 };
 
 
@@ -393,6 +432,7 @@ class PluginCodec_MediaFormat : public PluginCodec_Utilities
     typedef PluginCodec_OptionMap OptionMap;
 
   protected:
+    const char * m_rawFormat;
     const char * m_formatName;
     const char * m_payloadName;
     unsigned     m_payloadType;
@@ -405,19 +445,22 @@ class PluginCodec_MediaFormat : public PluginCodec_Utilities
 
   protected:
     PluginCodec_MediaFormat(
+        const char * rawFormat,
         const char * formatName,
         const char * payloadName,
         const char * description,
         unsigned     maxBandwidth,
-        OptionsTable options)
-      : m_formatName(formatName)
+        OptionsTable options,
+        unsigned flags = PluginCodec_RTPTypeDynamic
+    ) : m_rawFormat(rawFormat)
+      , m_formatName(formatName)
       , m_payloadName(payloadName)
       , m_payloadType(0)
       , m_description(description)
       , m_maxBandwidth(maxBandwidth)
       , m_h323CapabilityType(PluginCodec_H323Codec_NoH323)
       , m_h323CapabilityData(NULL)
-      , m_flags(PluginCodec_RTPTypeDynamic)
+      , m_flags(flags)
       , m_options(options)
     {
     }
@@ -428,6 +471,7 @@ class PluginCodec_MediaFormat : public PluginCodec_Utilities
     }
 
 
+    __inline const char *  GetRawFormat() const    { return this->m_rawFormat; }
     __inline const char *  GetFormatName() const   { return this->m_formatName; }
     __inline const char *  GetPayloadName() const  { return this->m_payloadName; }
     __inline unsigned char GetPayloadType() const  { return (unsigned char)this->m_payloadType; }
@@ -438,16 +482,16 @@ class PluginCodec_MediaFormat : public PluginCodec_Utilities
     __inline unsigned      GetFlags() const { return this->m_flags; }
     __inline const void *  GetOptionsTable() const { return this->m_options; }
 
+    __inline void SetFlags(unsigned b, unsigned m = 0) { this->m_flags = (this->m_flags&~m) | b; }
+    __inline void SetPayloadType(unsigned p) { this->m_payloadType = p; this->m_flags |= PluginCodec_RTPTypeExplicit; }
+    __inline void SetH323Capability(unsigned type, const void * data = NULL) { this->m_h323CapabilityType = type; this->m_h323CapabilityData = data; }
+
 
     /// Determine if codec is valid for the specified protocol
-    virtual bool IsValidForProtocol(const char * /*protocol*/)
-    {
-      return true;
-    }
-
+    virtual bool IsValidForProtocol(const char * /*protocol*/) const = 0;
 
     /// Utility function to adjust option strings, used by ToNormalised()/ToCustomised().
-    bool AdjustOptions(void * parm, unsigned * parmLen, bool (PluginCodec_MediaFormat:: * adjuster)(OptionMap & original, OptionMap & changed))
+    bool AdjustOptions(void * parm, unsigned * parmLen, bool (PluginCodec_MediaFormat:: * adjuster)(OptionMap & original, OptionMap & changed) const) const
     {
       if (parmLen == NULL || parm == NULL || *parmLen != sizeof(char ***)) {
         PTRACE(1, "Plugin", "Invalid parameters to AdjustOptions.");
@@ -466,20 +510,12 @@ class PluginCodec_MediaFormat : public PluginCodec_Utilities
 
 
     /// Adjust normalised options calculated from codec specific options.
-    virtual bool ToNormalised(OptionMap & /*original*/, OptionMap & /*changed*/)
-    {
-      return true;
-    }
-
+    virtual bool ToNormalised(OptionMap & /*original*/, OptionMap & /*changed*/) const = 0;
 
     // Adjust codec specific options calculated from normalised options.
-    virtual bool ToCustomised(OptionMap & /*original*/, OptionMap & /*changed*/)
-    {
-      return true;
-    }
+    virtual bool ToCustomised(OptionMap & /*original*/, OptionMap & /*changed*/) const = 0;
 
-
-    virtual void AdjustForVersion(unsigned version, const PluginCodec_Definition * /*definition*/)
+    virtual void AdjustForVersion(unsigned version, const PluginCodec_Definition * /*definition*/) const
     {
       if (version < PLUGIN_CODEC_VERSION_INTERSECT) {
         for (PluginCodec_Option ** options = (PluginCodec_Option **)m_options; *options != NULL; ++options) {
@@ -507,16 +543,18 @@ class PluginCodec_MediaFormat : public PluginCodec_Utilities
 template<typename NAME>
 class PluginCodec_AudioFormat : public PluginCodec_MediaFormat<NAME>
 {
+  public:
+    typedef PluginCodec_MediaFormat<NAME> Parent;
+    typedef typename Parent::OptionsTable OptionsTable;
+
   protected:
     unsigned m_samplesPerFrame;
     unsigned m_bytesPerFrame;
     unsigned m_sampleRate;
     unsigned m_recommendedFramesPerPacket;
     unsigned m_maxFramesPerPacket;
+    unsigned m_frameTime;
 
-  public:
-    typedef PluginCodec_MediaFormat<NAME> Parent;
-    typedef typename Parent::OptionsTable OptionsTable;
 
     PluginCodec_AudioFormat(
       const char * formatName,
@@ -524,26 +562,32 @@ class PluginCodec_AudioFormat : public PluginCodec_MediaFormat<NAME>
       const char * description,
       unsigned     samplesPerFrame,
       unsigned     bytesPerFrame,
-      unsigned     sampleRate,
-      OptionsTable options
-    ) : Parent(formatName, payloadName, description, bytesPerFrame*8 * samplesPerFrame*1000000/sampleRate, options)
+      unsigned     sampleRate = 8000,
+      OptionsTable options = NULL
+    ) : Parent(PLUGINCODEC_RAW_AUDIO, formatName, payloadName, description, bytesPerFrame*8 * samplesPerFrame*1000000/sampleRate, options)
       , m_samplesPerFrame(samplesPerFrame)
       , m_bytesPerFrame(bytesPerFrame)
       , m_sampleRate(sampleRate)
       , m_recommendedFramesPerPacket((50*sampleRate)/(1000*samplesPerFrame))
       , m_maxFramesPerPacket((120*sampleRate)/(1000*samplesPerFrame))
+      , m_frameTime(samplesPerFrame * 1000000 / sampleRate)
     {
       this->m_flags = PluginCodec_MediaTypeAudio /* audio codec */
                     | PluginCodec_InputTypeRaw   /* raw input data */
                     | PluginCodec_OutputTypeRaw; /* raw output data */
-
     }
 
+    __inline PluginCodec_AudioFormat & SetFlags(unsigned b, unsigned m = 0) { this->Parent::SetFlags(b,m); return *this; }
+    __inline PluginCodec_AudioFormat & SetPayloadType(unsigned p) { this->Parent::SetPayloadType(p); return *this; }
+    __inline PluginCodec_AudioFormat & SetH323Capability(unsigned type, const void * data = NULL) { this->Parent::SetH323Capability(type, data); return *this; }
+    __inline PluginCodec_AudioFormat & SetMaxFramesPerPacket(unsigned p) { this->m_maxFramesPerPacket = p; return *this; }
+    __inline PluginCodec_AudioFormat & SetChannels(unsigned n)    { this->m_flags |= PluginCodec_SetChannels(n); return *this; }
 
+  public:
     __inline unsigned GetSamplesPerFrame() const { return this->m_samplesPerFrame; }
     __inline unsigned GetBytesPerFrame() const { return this->m_bytesPerFrame; }
     __inline unsigned GetSampleRate() const { return this->m_sampleRate; }
-    __inline unsigned GetFrameTime() const { return this->m_samplesPerFrame*1000000/this->m_sampleRate; }
+    __inline unsigned GetFrameTime() const { return this->m_frameTime; }
     __inline unsigned GetRecommendedFramesPerPacket() const { return this->m_recommendedFramesPerPacket; }
     __inline unsigned GetMaxFramesPerPacket() const { return this->m_maxFramesPerPacket; }
 };
@@ -552,21 +596,22 @@ class PluginCodec_AudioFormat : public PluginCodec_MediaFormat<NAME>
 template<typename NAME>
 class PluginCodec_VideoFormat : public PluginCodec_MediaFormat<NAME>
 {
+  public:
+    typedef PluginCodec_MediaFormat<NAME> Parent;
+    typedef typename Parent::OptionsTable OptionsTable;
+
   protected:
     unsigned m_maxWidth;
     unsigned m_maxHeight;
 
-  public:
-    typedef PluginCodec_MediaFormat<NAME> Parent;
-    typedef typename Parent::OptionsTable OptionsTable;
 
     PluginCodec_VideoFormat(
       const char * formatName,
       const char * payloadName,
       const char * description,
       unsigned     maxBandwidth,
-      OptionsTable options
-    ) : Parent(formatName, payloadName, description, maxBandwidth, options)
+      OptionsTable options = NULL
+    ) : Parent(PLUGINCODEC_RAW_VIDEO, formatName, payloadName, description, maxBandwidth, options)
       , m_maxWidth(1920)
       , m_maxHeight(1200)
     {
@@ -575,7 +620,11 @@ class PluginCodec_VideoFormat : public PluginCodec_MediaFormat<NAME>
                     | PluginCodec_OutputTypeRTP; /* raw output data */
     }
 
+    __inline PluginCodec_VideoFormat & SetFlags(unsigned b, unsigned m = 0) { this->Parent::SetFlags(b,m); return *this; }
+    __inline PluginCodec_VideoFormat & SetPayloadType(unsigned p) { this->Parent::SetPayloadType(p); return *this; }
+    __inline PluginCodec_VideoFormat & SetH323Capability(unsigned type, const void * data = NULL) { this->Parent::SetH323Capability(type, data); return *this; }
 
+  public:
     __inline unsigned GetMaxWidth() const { return this->m_maxWidth; }
     __inline unsigned GetMaxHeight() const { return this->m_maxHeight; }
 };
@@ -590,7 +639,7 @@ class PluginCodec : public PluginCodec_Utilities
     PluginCodec(const PluginCodec_Definition * defn)
       : m_definition(defn)
       , m_optionsSame(false)
-      , m_maxBitRate(defn->bitsPerSec)
+      , m_maxBitRate(defn->bitsPerSec > 0 ? defn->bitsPerSec : 4*1024*1024)
       , m_frameTime((defn->sampleRate/1000*defn->usPerFrame)/1000) // Odd way of calculation to avoid 32 bit integer overflow
     {
       PTRACE(3, "Plugin", "Codec created: \"" << defn->descr
@@ -630,9 +679,9 @@ class PluginCodec : public PluginCodec_Utilities
 
 
     /// Gather any statistics as a string into the provide buffer.
-    virtual bool GetStatistics(char * /*bufferPtr*/, unsigned /*bufferSize*/)
+    virtual int GetStatistics(char * /*bufferPtr*/, unsigned /*bufferSize*/)
     {
-      return true;
+      return -1;
     }
 
 
@@ -693,11 +742,19 @@ class PluginCodec : public PluginCodec_Utilities
     virtual bool SetOption(const char * optionName, const char * optionValue)
     {
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_TARGET_BIT_RATE) == 0)
-        return this->SetOptionUnsigned(this->m_maxBitRate, optionValue, 1, this->m_definition->bitsPerSec);
+        return this->SetOptionUnsigned(this->m_maxBitRate, optionValue, 1);
+
+      if (strcasecmp(optionName, PLUGINCODEC_OPTION_MAX_BIT_RATE) == 0) {
+        unsigned bitRate = this->m_maxBitRate;
+        if (!this->SetOptionUnsigned(bitRate, optionValue, 1))
+          return false;
+        if (m_maxBitRate > bitRate)
+          m_maxBitRate = bitRate;
+        return true;
+      }
 
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_FRAME_TIME) == 0)
-        return this->SetOptionUnsigned(this->m_frameTime, optionValue,
-                                       this->m_definition->sampleRate/1000, this->m_definition->sampleRate); // 1ms to 1 second
+        return this->SetOptionUnsigned(this->m_frameTime, optionValue, 1);
 
       return true;
     }
@@ -837,7 +894,7 @@ class PluginCodec : public PluginCodec_Utilities
 
     static int GetOutputDataSize_s(const PluginCodec_Definition *, void * context, const char *, void *, unsigned *)
     {
-      return context != NULL ? ((PluginCodec *)context)->GetOutputDataSize() : 0;
+      return context != NULL ? (int)((PluginCodec *)context)->GetOutputDataSize() : 0;
     }
 
     typedef PluginCodec_MediaFormat<NAME> MediaFormat;
@@ -916,8 +973,7 @@ class PluginCodec : public PluginCodec_Utilities
     static int GetStatistics_s(const PluginCodec_Definition *, void * context, const char *, void * parm, unsigned * len)
     {
       PluginCodec * codec = (PluginCodec *)context;
-      return len != NULL && parm != NULL &&
-             codec != NULL && codec->GetStatistics((char *)parm, *len);
+      return len != NULL && parm != NULL && codec != NULL ? codec->GetStatistics((char *)parm, *len) : -1;
     }
 
     static int Terminate_s(const PluginCodec_Definition *, void * context, const char *, void *, unsigned *)
@@ -962,25 +1018,45 @@ class PluginVideoCodec : public PluginCodec<NAME>
 {
     typedef PluginCodec<NAME> BaseClass;
 
+  protected:
+    unsigned m_maxWidth;
+    unsigned m_maxHeight;
+    unsigned m_width;
+    unsigned m_height;
+
   public:
     enum {
       DefaultWidth = 352, // CIF size
-      DefaultHeight = 288,
-      MaxWidth = DefaultWidth*8,
-      MaxHeight = DefaultHeight*8
+      DefaultHeight = 288
     };
 
 
     PluginVideoCodec(const PluginCodec_Definition * defn)
       : BaseClass(defn)
+      , m_maxWidth(DefaultWidth*8)
+      , m_maxHeight(DefaultHeight*8)
+      , m_width(DefaultWidth)
+      , m_height(DefaultHeight)
     {
     }
 
 
+    virtual int GetStatistics(char * bufferPtr, unsigned bufferSize)
+    {
+      return ::snprintf(bufferPtr, bufferSize, "Width=%u\nHeight=%u\n", this->m_width, this->m_height);
+    }
+
+
     virtual size_t GetRawFrameSize(unsigned width, unsigned height)
     {
       return width*height*3/2; // YUV420P
     }
+
+
+    __inline unsigned GetMaxWidth() const { return m_maxWidth; }
+    __inline unsigned GetMaxHeight() const { return m_maxHeight; }
+    __inline unsigned GetWidth() const { return m_width; }
+    __inline unsigned GetHeight() const { return m_height; }
 };
 
 
@@ -992,8 +1068,6 @@ class PluginVideoEncoder : public PluginVideoCodec<NAME>
     typedef PluginVideoCodec<NAME> BaseClass;
 
   protected:
-    unsigned m_width;
-    unsigned m_height;
     unsigned m_maxRTPSize;
     unsigned m_tsto;
     unsigned m_keyFramePeriod;
@@ -1001,8 +1075,6 @@ class PluginVideoEncoder : public PluginVideoCodec<NAME>
   public:
     PluginVideoEncoder(const PluginCodec_Definition * defn)
       : BaseClass(defn)
-      , m_width(BaseClass::DefaultWidth)
-      , m_height(BaseClass::DefaultHeight)
       , m_maxRTPSize(PluginCodec_RTP_MaxPacketSize)
       , m_tsto(31)
       , m_keyFramePeriod(0) // Indicates auto/default
@@ -1010,16 +1082,22 @@ class PluginVideoEncoder : public PluginVideoCodec<NAME>
     }
 
 
+    virtual size_t GetOutputDataSize()
+    {
+      return this->m_maxRTPSize;
+    }
+
+
     virtual bool SetOption(const char * optionName, const char * optionValue)
     {
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_FRAME_WIDTH) == 0)
-        return this->SetOptionUnsigned(this->m_width, optionValue, 16, BaseClass::MaxWidth);
+        return this->SetOptionUnsigned(this->m_width, optionValue, 16, this->m_maxWidth);
 
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_FRAME_HEIGHT) == 0)
-        return this->SetOptionUnsigned(this->m_height, optionValue, 16, BaseClass::MaxHeight);
+        return this->SetOptionUnsigned(this->m_height, optionValue, 16, this->m_maxHeight);
 
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE) == 0)
-        return this->SetOptionUnsigned(this->m_maxRTPSize, optionValue, 256, 8192);
+        return this->SetOptionUnsigned(this->m_maxRTPSize, optionValue, 256);
 
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF) == 0)
         return this->SetOptionUnsigned(this->m_tsto, optionValue, 1, 31);
@@ -1071,6 +1149,19 @@ class PluginVideoDecoder : public PluginVideoCodec<NAME>
     }
 
 
+    virtual bool SetOption(const char * optionName, const char * optionValue)
+    {
+      if (strcasecmp(optionName, PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH) == 0)
+        return this->SetOptionUnsigned(this->m_width, optionValue, 16, this->m_maxWidth);
+
+      if (strcasecmp(optionName, PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT) == 0)
+        return this->SetOptionUnsigned(this->m_height, optionValue, 16, this->m_maxHeight);
+
+      // Base class sets bit rate and frame time
+      return BaseClass::SetOption(optionName, optionValue);
+    }
+
+
     virtual size_t GetOutputDataSize()
     {
       return this->m_outputSize;
@@ -1079,9 +1170,12 @@ class PluginVideoDecoder : public PluginVideoCodec<NAME>
 
     virtual bool CanOutputImage(unsigned width, unsigned height, PluginCodec_RTP & rtp, unsigned & flags)
     {
-      size_t newSize = this->GetRawFrameSize(width, height) + sizeof(PluginCodec_Video_FrameHeader) + rtp.GetHeaderSize();
-      if (newSize > rtp.GetMaxSize() || !rtp.SetPayloadSize(newSize)) {
-        m_outputSize = newSize;
+      if (width == 0 || height == 0)
+        return false;
+
+      size_t newSize = this->GetRawFrameSize(width, height) + sizeof(PluginCodec_Video_FrameHeader);
+      if (!rtp.SetPayloadSize(newSize)) {
+        m_outputSize = newSize + rtp.GetHeaderSize();
         flags |= PluginCodec_ReturnCoderBufferTooSmall;
         return false;
       }
@@ -1089,8 +1183,8 @@ class PluginVideoDecoder : public PluginVideoCodec<NAME>
       PluginCodec_Video_FrameHeader * videoHeader = rtp.GetVideoHeader();
       videoHeader->x = 0;
       videoHeader->y = 0;
-      videoHeader->width = width;
-      videoHeader->height = height;
+      videoHeader->width = this->m_width = width;
+      videoHeader->height = this->m_height = height;
 
       flags |= PluginCodec_ReturnCoderLastFrame;
       rtp.SetMarker(true);
@@ -1102,7 +1196,7 @@ class PluginVideoDecoder : public PluginVideoCodec<NAME>
     {
       unsigned        m_width;
       unsigned        m_height;
-      unsigned        m_raster;
+      int             m_raster;
       unsigned char * m_source;
       unsigned char * m_destination;
 
@@ -1137,13 +1231,44 @@ class PluginVideoDecoder : public PluginVideoCodec<NAME>
           planeInfo[plane].Copy();
       }
 
-      return rtp.GetPacketSize();
+      return (unsigned)rtp.GetPacketSize();
     }
 };
 
 
 /////////////////////////////////////////////////////////////////////////////
 
+/// Declare a "known by OPAL" codec using C++ support classes
+#define PLUGINCODEC_KNOWN_CODEC_CXX(MediaType,       /**< Media type (AUDIO or VIDEO) */ \
+                                    Name,            /**< Media format name */ \
+                                    Description,     /**< Description of codec */ \
+                                    EncoderClass,    /**< Encoder class name */ \
+                                    DecoderClass     /**< Decoder class name */ \
+                                    ) \
+             PLUGINCODEC_CODEC_PAIR(Name, \
+                                    NULL, \
+                                    Description, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    0, \
+                                    EncoderClass::Create_s<EncoderClass>, \
+                                    EncoderClass::Destroy_s, \
+                                    EncoderClass::Transcode_s, \
+                                    DecoderClass::Create_s<DecoderClass>, \
+                                    DecoderClass::Destroy_s, \
+                                    DecoderClass::Transcode_s, \
+                                    DecoderClass::GetControls(), /* Note doesn't matter if encoder or decoder */ \
+                                    PluginCodec_MediaTypeKnown, \
+                                    PLUGINCODEC_RAW_##MediaType, \
+                                    NULL)
+
 /// Declare an audio codec using C++ support classes
 #define PLUGINCODEC_AUDIO_CODEC_CXX(MediaFormat,     /**< PluginCodec_VideoFormat instance */ \
                                     EncoderClass,    /**< Encoder class name */ \
@@ -1170,7 +1295,7 @@ class PluginVideoDecoder : public PluginVideoCodec<NAME>
                                     DecoderClass::Transcode_s, \
                                     DecoderClass::GetControls(), /* Note doesn't matter if encoder or decoder */ \
                                     MediaFormat.GetFlags(), \
-                                    PLUGINCODEC_RAW_AUDIO, \
+                                    MediaFormat.GetRawFormat(), \
                                     &MediaFormat)
 
 /// Declare a video codec using C++ support classes
@@ -1198,7 +1323,7 @@ class PluginVideoDecoder : public PluginVideoCodec<NAME>
                                     DecoderClass::Transcode_s, \
                                     DecoderClass::GetControls(), /* Note doesn't matter if encoder or decoder */ \
                                     MediaFormat.GetFlags(), \
-                                    PLUGINCODEC_RAW_VIDEO, \
+                                    MediaFormat.GetRawFormat(), \
                                     &MediaFormat)
 
 
