@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29929 $
+ * $Revision: 34427 $
  * $Author: rjongbloed $
- * $Date: 2013-06-12 02:11:18 -0500 (Wed, 12 Jun 2013) $
+ * $Date: 2015-12-13 16:51:42 +0000 (Sun, 13 Dec 2015) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGINMGR_H
@@ -37,7 +37,7 @@
 
 #include <ptlib/object.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <ptlib/pluginmgr.h>
 #include <ptlib/pfactory.h>
@@ -90,6 +90,7 @@ class OpalPluginCodecHandler : public PObject
                                                                   unsigned frameTime,
                                                                   unsigned timeUnits,
                                                                     time_t timeStamp);
+    virtual void RegisterAudioTranscoder(const PString & src, const PString & dst, const PluginCodec_Definition * codec, bool isEnc);
 
 #if OPAL_VIDEO
     virtual OpalMediaFormatInternal * OnCreateVideoFormat(OpalPluginCodecManager & mgr,
@@ -97,7 +98,7 @@ class OpalPluginCodecHandler : public PObject
                                                               const char * fmtName,
                                                               const char * rtpEncodingName,
                                                                     time_t timeStamp);
-    virtual void RegisterVideoTranscoder(const PString & src, const PString & dst, const PluginCodec_Definition * codec, bool v);
+    virtual void RegisterVideoTranscoder(const PString & src, const PString & dst, const PluginCodec_Definition * codec, bool isEnc);
 #endif
 
 #if OPAL_FAX
@@ -108,6 +109,7 @@ class OpalPluginCodecHandler : public PObject
                                                                 unsigned frameTime,
                                                                 unsigned timeUnits,
                                                                   time_t timeStamp);
+    virtual void RegisterFaxTranscoder(const PString & src, const PString & dst, const PluginCodec_Definition * codec, bool isEnc);
 #endif
 };
 
@@ -125,8 +127,9 @@ class OpalPluginCodecManager : public PPluginModuleManager
                              PluginCodec_GetAPIVersionFunction getApiVerFn,
                              PluginCodec_GetCodecFunction getCodecFn);
 
-    void OnLoadPlugin(PDynaLink & dll, INT code);
+    void OnLoadPlugin(PDynaLink & dll, P_INT_PTR code);
 
+    virtual void OnStartup();
     virtual void OnShutdown();
 
 #if OPAL_H323
@@ -145,7 +148,13 @@ class OpalPluginCodecManager : public PPluginModuleManager
     void RegisterCodecPlugins  (unsigned int count, const PluginCodec_Definition * codecList, OpalPluginCodecHandler * handler);
     void UnregisterCodecPlugins(unsigned int count, const PluginCodec_Definition * codecList, OpalPluginCodecHandler * handler);
 
-    bool AddMediaFormat(OpalPluginCodecHandler * handler, const PTime & timeNow, const PluginCodec_Definition * codecDefn, const char * fmtName);
+    bool AddMediaFormat(
+      OpalPluginCodecHandler * handler,
+      const PTime & timeNow,
+      const PluginCodec_Definition * codecDefn,
+      const char * fmtName,
+      OpalMediaFormat & mediaFormat
+    );
 #if OPAL_H323
     void RegisterCapability(const PluginCodec_Definition * codecDefn);
 #endif
@@ -204,18 +213,6 @@ class OpalPluginMediaFormatInternal
 };
 
 
-class OpalPluginMediaFormat : public OpalMediaFormat
-{
-  public:
-    OpalPluginMediaFormat(OpalMediaFormatInternal * info)
-      : OpalMediaFormat(info)
-    {
-    }
-
-    OpalPluginMediaFormatInternal * GetInfo() const { return dynamic_cast<OpalPluginMediaFormatInternal *>(m_info); }
-};
-
-
 class OpalPluginTranscoder
 {
   public:
@@ -240,6 +237,9 @@ class OpalPluginTranscoder
     OpalPluginControl freeOptionsControl;
     OpalPluginControl getOutputDataSizeControl;
     OpalPluginControl getCodecStatistics;
+#if PTRACING
+    bool m_firstLoggedUpdateOptions[2];
+#endif
 };
 
 
@@ -269,9 +269,10 @@ class OpalPluginFramedAudioTranscoder : public OpalFramedTranscoder, public Opal
 {
   PCLASSINFO(OpalPluginFramedAudioTranscoder, OpalFramedTranscoder);
   public:
-    OpalPluginFramedAudioTranscoder(const PluginCodec_Definition * codecDefn, bool isEncoder);
+    OpalPluginFramedAudioTranscoder(const OpalTranscoderKey & key, const PluginCodec_Definition * codecDefn, bool isEncoder);
     bool UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output);
     PBoolean ExecuteCommand(const OpalMediaCommand & command);
+    void GetStatistics(OpalMediaStatistics & statistics) const;
     PBoolean ConvertFrame(const BYTE * input, PINDEX & consumed, BYTE * output, PINDEX & created);
     virtual PBoolean ConvertSilentFrame(BYTE * buffer);
     virtual bool AcceptComfortNoise() const { return comfortNoise; }
@@ -284,7 +285,7 @@ class OpalPluginStreamedAudioTranscoder : public OpalStreamedTranscoder, public
 {
   PCLASSINFO(OpalPluginStreamedAudioTranscoder, OpalStreamedTranscoder);
   public:
-    OpalPluginStreamedAudioTranscoder(const PluginCodec_Definition * codec, bool isEncoder);
+    OpalPluginStreamedAudioTranscoder(const OpalTranscoderKey & key, const PluginCodec_Definition * codec, bool isEncoder);
     bool UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output);
     PBoolean ExecuteCommand(const OpalMediaCommand & command);
     virtual bool AcceptComfortNoise() const { return comfortNoise; }
@@ -318,7 +319,7 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
 {
   PCLASSINFO(OpalPluginVideoTranscoder, OpalVideoTranscoder);
   public:
-    OpalPluginVideoTranscoder(const PluginCodec_Definition * codec, bool isEncoder);
+    OpalPluginVideoTranscoder(const OpalTranscoderKey & key, const PluginCodec_Definition * codec, bool isEncoder);
     ~OpalPluginVideoTranscoder();
 
 #if OPAL_STATISTICS
@@ -335,10 +336,11 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
     bool DecodeFrame(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
 
     RTP_DataFrame * m_bufferRTP;
-    DWORD           m_lastSequenceNumber;
     DWORD           m_lastDecodedTimestamp; // For missing marker bit detection
     DWORD           m_lastMarkerTimestamp;  // For continuous marker bit detection
+    unsigned        m_consecutiveMarkers;
     bool            m_badMarkers;
+    unsigned        m_totalFrames;
 
 #if PTRACING
     unsigned m_consecutiveIntraFrames;
@@ -435,21 +437,38 @@ class OpalFactoryCodec : public PObject {
 template<class TranscoderClass>
 class OpalPluginTranscoderFactory : public OpalTranscoderFactory
 {
-  public:
+  protected:
     class Worker : public OpalTranscoderFactory::WorkerBase 
     {
       public:
         Worker(const OpalTranscoderKey & key, const PluginCodec_Definition * codec, bool enc)
-          : OpalTranscoderFactory::WorkerBase(), codecDefn(codec), isEncoder(enc)
-        { OpalTranscoderFactory::Register(key, this); }
+          : OpalTranscoderFactory::WorkerBase()
+          , m_key(key)
+          , m_codecDefn(codec)
+          , m_isEncoder(enc)
+        {
+        }
 
       protected:
         virtual OpalTranscoder * Create(const OpalTranscoderKey &) const
-        { return new TranscoderClass(codecDefn, isEncoder); }
+        {
+          return new TranscoderClass(m_key, m_codecDefn, m_isEncoder);
+        }
 
-        const PluginCodec_Definition * codecDefn;
-        bool isEncoder;
+        OpalTranscoderKey              m_key;
+        const PluginCodec_Definition * m_codecDefn;
+        bool                           m_isEncoder;
     };
+
+  public:
+    static void Register(const OpalTranscoderKey & key, const PluginCodec_Definition * codec, bool enc)
+    {
+      Worker * worker = PNEW Worker(key, codec, enc);
+      if (!OpalTranscoderFactory::Register(key, worker, true)) {
+        delete worker;
+        PTRACE(3, "OpalTranscoderFactory worker for " << key.first << '/' << key.second << " already registered.");
+      }
+    }
 };
 
 //////////////////////////////////////////////////////////////////////////////
@@ -677,7 +696,7 @@ class H323H263Capability : public H323VideoCapability
       const H245_VideoCapability & pdu  /// PDU to get information from
     );
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PASN_Object & subTypePDU,     ///<  sub-type PDU of H323Capability
       const PString & mediaPacketization  ///< Media packetization used
     ) const;
 
