@@ -23,9 +23,9 @@
  *
  * Contributor(s): 
  *
- * $Revision: 29334 $
+ * $Revision: 33984 $
  * $Author: rjongbloed $
- * $Date: 2013-03-26 22:08:07 -0500 (Tue, 26 Mar 2013) $
+ * $Date: 2015-09-07 11:59:59 +0100 (Mon, 07 Sep 2015) $
  */
 
 #ifndef OPAL_CODEC_VIDCODEC_H
@@ -35,7 +35,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/transcoders.h>
 
@@ -46,20 +46,94 @@
 #endif
 
 #include <codec/opalplugin.h>
+#include <codec/known.h>
 
 
-#define OPAL_RGB24   "RGB24"
-#define OPAL_RGB32   "RGB32"
-#define OPAL_YUV420P "YUV420P"
-
 extern const OpalVideoFormat & GetOpalRGB24();
 extern const OpalVideoFormat & GetOpalRGB32();
+extern const OpalVideoFormat & GetOpalBGR24();
+extern const OpalVideoFormat & GetOpalBGR32();
 extern const OpalVideoFormat & GetOpalYUV420P();
 
 #define OpalRGB24   GetOpalRGB24()
 #define OpalRGB32   GetOpalRGB32()
+#define OpalBGR24   GetOpalBGR24()
+#define OpalBGR32   GetOpalBGR32()
 #define OpalYUV420P GetOpalYUV420P()
 
+#define OPAL_REGISTER_RAW_VIDEO() \
+              GetOpalRGB24(); \
+              GetOpalRGB32(); \
+              GetOpalBGR24(); \
+              GetOpalBGR32(); \
+              GetOpalYUV420P()
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+/**Handle logic for Intra frame control.
+    This class deals with issues of too many and too few Intra frames being
+    sent in response to requests from various sources.
+  */
+class OpalIntraFrameControl : public PObject
+{
+    PCLASSINFO(OpalIntraFrameControl, PObject);
+  public:
+    static PTimeInterval const DefaultMinThrottle;
+    static PTimeInterval const DefaultMaxThrottle;
+    static PTimeInterval const DefaultPeriodic;
+
+    OpalIntraFrameControl(
+      const PTimeInterval & minThrottle = DefaultMinThrottle, ///< Minimum time between intra frames
+      const PTimeInterval & maxThrottle = DefaultMaxThrottle, ///< If many requests, throttle period increases to this
+      const PTimeInterval & periodic = DefaultPeriodic        ///< Make sure Intra frames are at least this often
+    );
+
+    /**Handling incoming request for Intra frame.
+      */
+    void IntraFrameRequest();
+
+    /**Indicate that now is the time for an Intra frame to be generated.
+      */
+    bool RequireIntraFrame();
+
+    /**Detected Intra frame, so can cease requests.
+      */
+    void IntraFrameDetected();
+
+    /**Set throttle times for controlling Intra Frame.
+       This is 
+      */
+    void SetTimes(
+      const PTimeInterval & minThrottle, ///< Minimum time between intra frames
+      const PTimeInterval & maxThrottle, ///< If many requests, throttle period increases to this
+      const PTimeInterval & periodic,    ///< Make sure Intra frames are at least this often
+      const PTimeInterval & retry        ///< Retry request, typically the media round trip time, plus a bit.
+    );
+
+  protected:
+    PTimeInterval m_minThrottleTime;
+    PTimeInterval m_maxThrottleTime;
+    PTimeInterval m_periodicTime;
+    PTimeInterval m_retryTime;
+    PTimeInterval m_currentThrottleTime;
+
+    enum State
+    {
+      e_Idle,
+      e_Periodic,
+      e_Throttled,
+      e_Requesting,
+      e_Requested
+    } m_state;
+
+    unsigned m_stuckCount;
+
+   PDECLARE_MUTEX(m_mutex);
+    PTime  m_lastRequest;
+    PTimer m_requestTimer;
+    PDECLARE_NOTIFIER(PTimer, OpalIntraFrameControl, OnTimedRequest);
+};
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -143,30 +217,49 @@ class OpalVideoTranscoder : public OpalTranscoder
       RTP_DataFrame & output        ///<  Output data
     );
 
-#if OPAL_STATISTICS
-    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
-#endif
+    static OpalVideoFormat::VideoFrameType GetVideoFrameType(
+      const PCaselessString & rtpEncodingName,
+      const BYTE * payloadPtr,
+      PINDEX payloadSize,
+      PBYTEArray & context
+    );
 
     virtual bool HasErrorConcealment() const  { return m_errorConcealment; }
-    bool WasLastFrameIFrame() const { return lastFrameWasIFrame; }
-    void ForceIFrame()              { forceIFrame = true; }
+    bool WasLastFrameIFrame() const { return m_lastFrameWasIFrame; }
+    virtual void SendIFrameRequest(unsigned sequenceNumber, unsigned timestamp);
+    virtual bool HandleIFrameRequest();
+
+    void SetEncodingIntraFrameControlTimes(
+      const PTimeInterval & minThrottle, ///< Minimum time between intra frames
+      const PTimeInterval & maxThrottle, ///< If many requests, throttle period increases to this
+      const PTimeInterval & periodic,    ///< Make sure Intra frames are at least this often
+      const PTimeInterval & retry        ///< Retry request, typically the media round trip time, plus a bit.
+    ) { m_encodingIntraFrameControl.SetTimes(minThrottle, maxThrottle, periodic, retry); }
+
+    void SetDecodingIntraFrameControlTimes(
+      const PTimeInterval & minThrottle, ///< Minimum time between intra frames
+      const PTimeInterval & maxThrottle, ///< If many requests, throttle period increases to this
+      const PTimeInterval & periodic,    ///< Make sure Intra frames are at least this often
+      const PTimeInterval & retry        ///< Retry request, typically the media round trip time, plus a bit.
+    ) { m_decodingIntraFrameControl.SetTimes(minThrottle, maxThrottle, periodic, retry); }
   //@}
 
   protected:
+    PINDEX m_inDataSize;
+    PINDEX m_outDataSize;
     bool   m_errorConcealment;
-    PINDEX inDataSize;
-    PINDEX outDataSize;
-    bool   forceIFrame;
-    bool   lastFrameWasIFrame;
+    bool   m_freezeTillIFrame;
+    bool   m_frozenTillIFrame;
+    bool   m_lastFrameWasIFrame;
 
-    DWORD m_totalFrames;
-    DWORD m_keyFrames;
+    OpalIntraFrameControl m_encodingIntraFrameControl;
+    OpalIntraFrameControl m_decodingIntraFrameControl;
 };
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
-OPAL_DEFINE_MEDIA_COMMAND(OpalVideoFreezePicture, "Freeze Picture");
+OPAL_DEFINE_MEDIA_COMMAND(OpalVideoFreezePicture, "Freeze Picture", OpalMediaType::Video());
 
 /**This indicates that a force video picture update is required, that
    is an Intra or Key frame MUST be sent. This may be called when the
@@ -177,6 +270,10 @@ class OpalVideoUpdatePicture : public OpalMediaCommand
 {
     PCLASSINFO_WITH_CLONE(OpalVideoUpdatePicture, OpalMediaCommand);
   public:
+    OpalVideoUpdatePicture(
+      unsigned sessionID = 0,          ///< Session for media stream, 0 is use first \p mediaType stream
+      unsigned ssrc = 0                ///< Sync Source for media stream (if RTP)
+    );
     virtual PString GetName() const;
 };
 
@@ -189,7 +286,12 @@ class OpalVideoPictureLoss : public OpalVideoUpdatePicture
 {
     PCLASSINFO_WITH_CLONE(OpalVideoPictureLoss, OpalVideoUpdatePicture);
   public:
-    OpalVideoPictureLoss(unsigned sequenceNumber = 0, unsigned timestamp = 0);
+    OpalVideoPictureLoss(
+      unsigned sequenceNumber = 0,
+      unsigned timestamp = 0,
+      unsigned sessionID = 0,          ///< Session for media stream, 0 is use first \p mediaType stream
+      unsigned ssrc = 0                ///< Sync Source for media stream (if RTP)
+    );
 
     virtual PString GetName() const;
 
@@ -214,7 +316,11 @@ class OpalTemporalSpatialTradeOff : public OpalMediaCommand
 {
     PCLASSINFO_WITH_CLONE(OpalTemporalSpatialTradeOff, OpalMediaCommand);
   public:
-    OpalTemporalSpatialTradeOff(int tradeoff) : m_tradeOff(tradeoff) { }
+    OpalTemporalSpatialTradeOff(
+      unsigned tradeoff,
+      unsigned sessionID = 0,          ///< Session for media stream, 0 is use first \p mediaType stream
+      unsigned ssrc = 0                ///< Sync Source for media stream (if RTP)
+    );
 
     virtual PString GetName() const;
 
