@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29937 $
+ * $Revision: 33238 $
  * $Author: rjongbloed $
- * $Date: 2013-06-12 21:03:10 -0500 (Wed, 12 Jun 2013) $
+ * $Date: 2015-01-11 03:07:45 +0000 (Sun, 11 Jan 2015) $
  */
 
 #ifndef OPAL_OPAL_LOCALEP_H
@@ -36,11 +36,17 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/endpoint.h>
 
+
 class OpalLocalConnection;
+class OpalH224Handler;
+class OpalFarEndCameraControl;
+
+
+#define OPAL_LOCAL_PREFIX "local"
 
 
 /** Local EndPoint.
@@ -56,8 +62,9 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Create a new endpoint.
      */
     OpalLocalEndPoint(
-      OpalManager & manager,        ///<  Manager of all endpoints.
-      const char * prefix = "local" ///<  Prefix for URL style address strings
+      OpalManager & manager,         ///<  Manager of all endpoints.
+      const char * prefix = OPAL_LOCAL_PREFIX, ///<  Prefix for URL style address strings
+      bool useCallbacks = true       ///<  Indicate all media is directed to the callback virtual functions
     );
 
     /**Destroy endpoint.
@@ -207,11 +214,32 @@ class OpalLocalEndPoint : public OpalEndPoint
       const OpalLocalConnection & connection, ///<  Connection having event
       const PString & indication    ///< User input received
     );
+  //@}
+
+  /**@name Media call backs */
+  //@{
+    /// Enumeration for usage of media callback direction in CallbackMap
+    P_DECLARE_BITWISE_ENUM(CallbackUsage, 2, (NoCallbacks, UseSourceCallback, UseSinkCallback));
+
+    /// Map for indicating if a media type is to use callback virtual functions.
+    typedef map<OpalMediaType, CallbackUsage> CallbackMap;
+
+    /// Indicate media type and direction is to use virtual callback functions.
+    bool UseCallback(
+      const OpalMediaFormat & mediaFormat,  ///< Media format to check
+      bool isSource                         ///< Direction of media stream
+    ) const;
+
+    /// Set callback usage for media type
+    bool SetCallbackUsage(
+      const OpalMediaType & mediaType,
+      CallbackUsage usage
+    );
 
     /**Call back to get media data for transmission.
        If false is returned then OnReadMediaData() is called.
 
-       Care with the handling of real time is rqeuired, see GetSynchronicity
+       Care with the handling of real time is required, see GetSynchronicity
        for more details.
 
        The default implementation returns false.
@@ -225,9 +253,12 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Call back to handle received media data.
        If false is returned then OnWriteMediaData() is called.
 
-       Care with the handling of real time is rqeuired, see GetSynchronicity
+       Care with the handling of real time is required, see GetSynchronicity
        for more details.
 
+       Note it is the responsibility of this function to update the \p frame
+       timestamp for correct operation of the jitter buffer.
+
        The default implementation returns false.
       */
     virtual bool OnWriteMediaFrame(
@@ -239,10 +270,10 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Call back to get media data for transmission.
        If false is returned the media stream will be closed.
 
-       Care with the handling of real time is rqeuired, see GetSynchronicity
+       Care with the handling of real time is required, see GetSynchronicity
        for more details.
 
-       The default implementation returns false.
+       The default implementation fills the buffer with zeros and returns true.
       */
     virtual bool OnReadMediaData(
       const OpalLocalConnection & connection, ///<  Connection for media
@@ -255,21 +286,19 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Call back to handle received media data.
        If false is returned the media stream will be closed.
 
-       Note: For audio media, if \p data is NULL then that indicates there is
-       no incoming audio available from the jitter buffer. The application
-       should output silence for a time. The \p written value should still
-       contain the bytes of silence emitted, even though it ewill be larger
-       that \p length.
-       
-       Also, it is expected that this function be real time. That is if 320
-       bytes of PCM-16 are written, this function should take 20ms to execute.
-       If not then the jitter buffer will not operate correctly and audio will
-       not be of high quality.
+       It is expected that this function be real time. That is if 320 bytes of
+       PCM-16 are written, this function should take 20ms to execute. If not
+       then the jitter buffer will not operate correctly and audio will not be
+       of high quality. This timing can be simulated if required, see
+       GetSynchronicity for more details.
 
-       Care with the handling of real time is rqeuired, see GetSynchronicity
-       for more details.
+       Note: For encoded audio media, if \p data is NULL then that indicates
+       there is no incoming audio available from the jitter buffer. The
+       application should output silence for a time. The \p written value
+       should still contain the bytes of silence emitted, even though it will
+       be larger that \p length. This does not occcur for raw (PCM-16) audio.
 
-       The default implementation returns false.
+       The default implementation ignores the media data and returns true.
       */
     virtual bool OnWriteMediaData(
       const OpalLocalConnection & connection, ///<  Connection for media
@@ -279,6 +308,28 @@ class OpalLocalEndPoint : public OpalEndPoint
       PINDEX & written                        ///<  Amount of data written
     );
 
+#if OPAL_VIDEO
+    /**Create an PVideoInputDevice for a source media stream.
+      */
+    virtual bool CreateVideoInputDevice(
+      const OpalConnection & connection,    ///<  Connection needing created video device
+      const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
+      PVideoInputDevice * & device,         ///<  Created device
+      bool & autoDelete                     ///<  Flag for auto delete device
+    );
+
+    /**Create an PVideoOutputDevice for a sink media stream or the preview
+       display for a source media stream.
+      */
+    virtual bool CreateVideoOutputDevice(
+      const OpalConnection & connection,    ///<  Connection needing created video device
+      const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
+      bool preview,                         ///<  Flag indicating is a preview output
+      PVideoOutputDevice * & device,        ///<  Created device
+      bool & autoDelete                     ///<  Flag for auto delete device
+    );
+#endif
+
     /**Indicate the synchronous mode for I/O.
        This indicates that the OnReadMediaXXX and OnWriteMediaXXX functions
        will execute blocking to the correct real time synchronisation.
@@ -286,7 +337,7 @@ class OpalLocalEndPoint : public OpalEndPoint
        OnWriteMediaData() is sent 320 bytes of PCM data, it will block for
        20 milliseconds.
 
-       If GetSynchronicity() returns e_SimulateSyncronous, then the system will try and
+       If GetSynchronicity() returns e_SimulateSynchronous, then the system will try and
        simulate the correct timing using the operating system sleep function.
        This is not desirable as this function is notoriously inaccurate, and
        OPAL does it's best to compensate, but very often there is no other
@@ -303,7 +354,7 @@ class OpalLocalEndPoint : public OpalEndPoint
     enum Synchronicity {
       e_Synchronous,        ///< Functions will block for correct real time
       e_Asynchronous,       ///< Functions will not block, and do not require any real time handling.
-      e_SimulateSyncronous  ///< Functions wlll not block, but do require real time handling.
+      e_SimulateSynchronous ///< Functions wlll not block, but do require real time handling.
     };
 
     /**Indicate the I/O synchronous mode.
@@ -318,7 +369,10 @@ class OpalLocalEndPoint : public OpalEndPoint
       const OpalMediaFormat & mediaFormat,  ///< Media format for stream being opened.
       bool isSource                         ///< Stream is a a source
     ) const;
+  //@}
 
+  /**@name Member access */
+  //@{
     /**Get default synchronous mode for audio sources and sinks.
       */
     Synchronicity GetDefaultAudioSynchronicity() const { return m_defaultAudioSynchronicity; }
@@ -350,15 +404,47 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Indicate AcceptIncomingCall() execution is be deferred or immediate on OnIncomingCall().
       */
     void SetDeferredAnswer(bool defer) { m_deferredAnswer = defer; }
+
+    /**Indicate when we put remote on hold, our media is also paused.
+       If system will send music on hold or similar, then this should be set to false.
+      */
+    bool WillPauseTransmitMediaOnHold() const { return m_pauseTransmitMediaOnHold; }
+
+    /**Indicate when we put remote on hold, our media is also paused.
+       If system will send music on hold or similar, then this should be set to false.
+      */
+    void SetPauseTransmitMediaOnHold(bool pause) { m_pauseTransmitMediaOnHold = pause; }
+
+#if OPAL_HAS_H281
+    /// Set a callback for when the far end camera control capabilities change.
+    void SetFarEndCameraCapabilityChangedNotifier(const PNotifier & notifier) { m_farEndCameraCapabilityChangedNotifier = notifier; }
+
+    /// Get a callback for when the far end camera control capabilities change.
+    const PNotifier & GetFarEndCameraCapabilityChangedNotifier() const { return m_farEndCameraCapabilityChangedNotifier; }
+
+    /// Set a callback for when a far end camera control action comes from remote.
+    void SetFarEndCameraActionNotifier(const PNotifier & notifier) { m_farEndCameraActionNotifier = notifier; }
+
+    /// Get a callback for when a far end camera control action comes from remote.
+    const PNotifier & GetFarEndCameraActionNotifier() const { return m_farEndCameraActionNotifier; }
+#endif // OPAL_HAS_H281
   //@}
 
   protected:
     bool m_deferredAlerting;
     bool m_deferredAnswer;
+    bool m_pauseTransmitMediaOnHold;
+
+    CallbackMap m_useCallback;
 
     Synchronicity m_defaultAudioSynchronicity;
     Synchronicity m_defaultVideoSourceSynchronicity;
 
+#if OPAL_HAS_H281
+    PNotifier m_farEndCameraCapabilityChangedNotifier;
+    PNotifier m_farEndCameraActionNotifier;
+#endif // OPAL_HAS_H281
+
   private:
     P_REMOVE_VIRTUAL(OpalLocalConnection *, CreateConnection(OpalCall &, void *), 0);
     P_REMOVE_VIRTUAL(bool, IsSynchronous() const, false);
@@ -414,7 +500,7 @@ class OpalLocalConnection : public OpalConnection
 
        The default behaviour calls the base class then OnOutgoingSetUp().
 
-       Note that the most explicit version of this override is made pure, so as to force 
+       Note that the most explicit version of this override is made pure, so as to force
        descendant classes to implement it. This will only affect code that implements new
        descendants of OpalConnection - code that uses existing descendants will be unaffected
      */
@@ -471,6 +557,24 @@ class OpalLocalConnection : public OpalConnection
       */
     virtual PBoolean SetConnected();
 
+    /**Put the current connection on hold, suspending media streams.
+       The streams from the remote are always paused. The streams from the
+       local to the remote are conditionally paused depending on underlying
+       logic for "music on hold" functionality.
+
+       The \p fromRemote parameter indicates if we a putting the remote on
+       hold (false) or it is a request for the remote to put us on hold (true).
+
+       The /p placeOnHold parameter indicates of the command/request is for
+       going on hold or retrieving from hold.
+
+       @return true if hold request successfully initiated. The OnHold() call
+               back must be monitored for final confirmation of hold state.
+     */
+    virtual bool HoldRemote(
+      bool placeOnHold  ///< Flag for setting on or off hold
+    );
+
     /**Open a new media stream.
        This will create a media stream of an appropriate subclass as required
        by the underlying connection protocol. For instance H.323 would create
@@ -491,12 +595,12 @@ class OpalLocalConnection : public OpalConnection
       PBoolean isSource                    ///<  Is a source stream
     );
 
-    /**Open source or sink media stream for session.
+    /**Call back for closed a media stream.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
       */
-    virtual OpalMediaStreamPtr OpenMediaStream(
-      const OpalMediaFormat & mediaFormat, ///<  Media format to open
-      unsigned sessionID,                  ///<  Session to start stream on
-      bool isSource                        ///< Stream is a source/sink
+    virtual void OnClosedMediaStream(
+      const OpalMediaStream & stream     ///<  Media stream being closed
     );
 
     /**Send a user input indication to the remote endpoint.
@@ -548,6 +652,155 @@ class OpalLocalConnection : public OpalConnection
     /**Accept the incoming connection.
       */
     virtual void AcceptIncoming();
+
+
+  /**@name Media call backs */
+  //@{
+    /**Call back to get media data for transmission.
+       If false is returned then OnReadMediaData() is called.
+
+       Care with the handling of real time is required, see GetSynchronicity
+       for more details.
+
+       The default implementation returns false.
+      */
+    virtual bool OnReadMediaFrame(
+      const OpalMediaStream & mediaStream,    ///<  Media stream data is required for
+      RTP_DataFrame & frame                   ///<  RTP frame for data
+    );
+
+    /**Call back to handle received media data.
+       If false is returned then OnWriteMediaData() is called.
+
+       Care with the handling of real time is required, see GetSynchronicity
+       for more details.
+
+       Note it is the responsibility of this function to update the \p frame
+       timestamp for correct operation of the jitter buffer.
+
+       The default implementation returns false.
+      */
+    virtual bool OnWriteMediaFrame(
+      const OpalMediaStream & mediaStream,    ///<  Media stream data is required for
+      RTP_DataFrame & frame                   ///<  RTP frame for data
+    );
+
+    /**Call back to get media data for transmission.
+       If false is returned the media stream will be closed.
+
+       Care with the handling of real time is required, see GetSynchronicity
+       for more details.
+
+       The default implementation fills the buffer with zeros and returns true.
+      */
+    virtual bool OnReadMediaData(
+      const OpalMediaStream & mediaStream,    ///<  Media stream data is required for
+      void * data,                            ///<  Data to send
+      PINDEX size,                            ///<  Maximum size of data buffer
+      PINDEX & length                         ///<  Number of bytes placed in buffer
+    );
+
+    /**Call back to handle received media data.
+       If false is returned the media stream will be closed.
+
+       It is expected that this function be real time. That is if 320 bytes of
+       PCM-16 are written, this function should take 20ms to execute. If not
+       then the jitter buffer will not operate correctly and audio will not be
+       of high quality. This timing can be simulated if required, see
+       GetSynchronicity for more details.
+
+       Note: For encoded audio media, if \p data is NULL then that indicates
+       there is no incoming audio available from the jitter buffer. The
+       application should output silence for a time. The \p written value
+       should still contain the bytes of silence emitted, even though it will
+       be larger that \p length. This does not occcur for raw (PCM-16) audio.
+
+       The default implementation ignores the media data and returns true.
+      */
+    virtual bool OnWriteMediaData(
+      const OpalMediaStream & mediaStream,    ///<  Media stream data is required for
+      const void * data,                      ///<  Data received
+      PINDEX length,                          ///<  Amount of data available to write
+      PINDEX & written                        ///<  Amount of data written
+    );
+
+    /**Indicate the I/O synchronous mode.
+       See Synchronicity for more details.
+
+       Default behaviour returns m_defaultAudioSynchronicity (initially
+       e_Synchronous) when is audio source or sink,
+       m_defaultVideoSourceSynchronicity (initially e_Synchronous) when a
+       video source, e_Asynchronous in all other cases.
+      */
+    virtual OpalLocalEndPoint::Synchronicity GetSynchronicity(
+      const OpalMediaFormat & mediaFormat,  ///< Media format for stream being opened.
+      bool isSource                         ///< Stream is a a source
+    ) const;
+
+#if OPAL_VIDEO
+    /**Create an PVideoInputDevice for a source media stream.
+      */
+    virtual bool CreateVideoInputDevice(
+      const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
+      PVideoInputDevice * & device,         ///<  Created device
+      bool & autoDelete                     ///<  Flag for auto delete device
+    );
+
+    /**Create an PVideoOutputDevice for a sink media stream or the preview
+       display for a source media stream.
+      */
+    virtual bool CreateVideoOutputDevice(
+      const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
+      bool preview,                         ///<  Flag indicating is a preview output
+      PVideoOutputDevice * & device,        ///<  Created device
+      bool & autoDelete                     ///<  Flag for auto delete device
+    );
+
+    /**Change a PVideoInputDevice for a source media stream.
+      */
+    virtual bool ChangeVideoInputDevice(
+      const PVideoDevice::OpenArgs & device,  ///< Device to change to
+      unsigned sessionID = 0                  ///< Session for media stream, 0 indicates first video stream
+    );
+
+    /**Create an PVideoOutputDevice for a sink media stream or the preview
+       display for a source media stream.
+      */
+    virtual bool ChangeVideoOutputDevice(
+      const PVideoDevice::OpenArgs & device,  ///< Device to change to
+      unsigned sessionID = 0,                 ///< Session for media stream, 0 indicates first video stream
+      bool preview = false                    ///< Flag indicating is a preview output device
+    );
+#endif // OPAL_VIDEO
+
+#if OPAL_HAS_H281
+    /**Start/Stop Far End Camera Control on remote system.
+       If \p direction is negative then starts panning left, tilting down,
+       zoom out or focus out.
+
+       If \p direction is positive then starts panning right, tilting up, zoom
+       in or focus in.
+
+       If \p direction is zero then the operation is stopped. This can also be
+       used to determine if the remote is capabile of the operation.
+
+       If \p duration is non-zero then the actionis automatically stopped after
+       that period. Zero indicated continue until explicitly stopped.
+
+       @returns false if the operation cannot be performed.
+      */
+    bool FarEndCameraControl(
+      PVideoControlInfo::Types what,      ///< What to control, pan, tilt, zoom or focus
+      int direction = 0,                  ///< Direction to move
+      const PTimeInterval & duration = 0  ///< Duration of move
+    );
+
+    /// Set a callback for when the far end camera control capabilities change.
+    void SetFarEndCameraCapabilityChangedNotifier(const PNotifier & notifier);
+
+    /// Set a callback for when a far end camera control action comes from remote.
+    void SetFarEndCameraActionNotifier(const PNotifier & notifier);
+#endif // OPAL_HAS_H281
   //@}
 
   /**@name Member variable access */
@@ -563,8 +816,15 @@ class OpalLocalConnection : public OpalConnection
     friend class PSafeWorkNoArg<OpalLocalConnection>;
     void InternalAcceptIncoming();
 
-    OpalLocalEndPoint & endpoint;
-    void * m_userData;
+    OpalLocalEndPoint & m_endpoint;
+    void              * m_userData;
+
+#if OPAL_HAS_H224
+    OpalH224Handler * m_h224Handler;
+#endif
+#if OPAL_HAS_H281
+    OpalFarEndCameraControl * m_farEndCameraControl;
+#endif
 };
 
 
@@ -636,7 +896,9 @@ class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacin
   protected:
     virtual void InternalClose() { }
 
+    OpalLocalConnection            & m_connection;
     OpalLocalEndPoint::Synchronicity m_synchronicity;
+    PBYTEArray                       m_silence;
 };
 
 
