@@ -25,9 +25,9 @@
  * Contributor(s): Craig Southeren (craigs@postincrement.com)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 29867 $
+ * $Revision: 34318 $
  * $Author: rjongbloed $
- * $Date: 2013-06-02 22:06:44 -0500 (Sun, 02 Jun 2013) $
+ * $Date: 2015-11-20 20:57:53 +0000 (Fri, 20 Nov 2015) $
  */
 
 
@@ -38,7 +38,7 @@
 #include <ptlib.h>
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_HAS_MIXER
 
@@ -56,6 +56,8 @@ class OpalMixerConnection;
 //#define OPAL_MIXER_AUDIO_DEBUG 1
 
 
+#define OPAL_MIXER_PREFIX "mcu"
+
 #define OPAL_OPT_LISTEN_ONLY "Listen-Only"      ///< String option for listen only mixer connection
 #define OPAL_OPT_CONF_OWNER  "Conference-Owner" ///< String option for listen only mixer connection
 
@@ -71,8 +73,9 @@ class OpalMixerConnection;
     Note the timestamps of the input media are extremely important as they are
     used so that breaks or too fast data in the input media is dealt with correctly.
   */
-class OpalBaseMixer : public PObject
+class OpalBaseMixer : public PSmartObject
 {
+    PCLASSINFO(OpalBaseMixer, PSmartObject);
   public:
     OpalBaseMixer(
       bool pushThread,    ///< Indicate if the push thread should be started
@@ -175,7 +178,7 @@ class OpalBaseMixer : public PObject
     RTP_DataFrame * m_pushFrame;        // Cached frame for pushing RTP
     PThread *       m_workerThread;     // reader thread handle
     bool            m_threadRunning;    // used to stop reader thread
-    PMutex          m_mutex;            // mutex for list of streams and thread handle
+   PDECLARE_MUTEX(m_mutex);             // mutex for list of streams and thread handle
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -190,6 +193,7 @@ class OpalBaseMixer : public PObject
   */
 class OpalAudioMixer : public OpalBaseMixer
 {
+    PCLASSINFO(OpalAudioMixer, OpalBaseMixer);
   public:
     OpalAudioMixer(
       bool stereo = false,    ///< Indicate stero or mixed mono mode
@@ -285,6 +289,7 @@ class OpalAudioMixer : public OpalBaseMixer
   */
 class OpalVideoMixer : public OpalBaseMixer
 {
+    PCLASSINFO(OpalVideoMixer, OpalBaseMixer);
   public:
     enum Styles {
       eSideBySideLetterbox, /**< Two images side by side with black bars top and bottom.
@@ -305,6 +310,7 @@ class OpalVideoMixer : public OpalBaseMixer
                                  ratio. e.g. for CIF inputs, output would be 352x576. */
       eGrid,                /**< Standard 2x2, 3x3, 4x4 grid pattern. Size of grid is
                                  dependent on the number of video streams. */
+      eUser                 /**< User defined */
     };
 
     OpalVideoMixer(
@@ -332,7 +338,7 @@ class OpalVideoMixer : public OpalBaseMixer
     /**Set output video frame rate.
        May be dynamically changed at any time.
       */
-    bool SetFrameRate(
+    virtual bool SetFrameRate(
       unsigned rate   // New frames per second.
     );
 
@@ -360,6 +366,11 @@ class OpalVideoMixer : public OpalBaseMixer
     virtual bool MixStreams(RTP_DataFrame & frame);
     virtual size_t GetOutputSize() const;
 
+    virtual bool MixVideo();
+    virtual bool StartMix(unsigned & x, unsigned & y, unsigned & w, unsigned & h, unsigned & left);
+    virtual bool NextMix(unsigned & x, unsigned & y, unsigned & w, unsigned & h, unsigned & left);
+    void InsertVideoFrame(const StreamMap_T::iterator & it, unsigned x, unsigned y, unsigned w, unsigned h);
+
   protected:
     Styles     m_style;
     unsigned   m_width, m_height;
@@ -424,6 +435,8 @@ struct OpalMixerNodeInfo
 ///////////////////////////////////////////////////////////////////////////////
 
 class OpalMixerNode;
+class OpalAudioStreamMixer;
+class OpalVideoStreamMixer;
 
 
 /** Mixer node manager.
@@ -506,7 +519,7 @@ class OpalMixerNodeManager
 
     /**Add node name to association list.
       */
-    void AddNodeName(
+    bool AddNodeName(
       PString name,        ///< alias name for node
       OpalMixerNode * node ///< node associated with name
     );
@@ -539,6 +552,14 @@ class OpalMixerNodeManager
       OpalConferenceState::ChangeType change ///< Change that occurred
     );
 
+    /// Create the instance of the audio mixer
+    virtual OpalAudioStreamMixer * CreateAudioMixer(const OpalMixerNodeInfo & info);
+
+#if OPAL_VIDEO
+    /// Create the instance of the video mixer
+    virtual OpalVideoStreamMixer * CreateVideoMixer(const OpalMixerNodeInfo & info);
+#endif
+
     /// Get manager
     OpalManager & GetManager() const { return m_manager; }
   //@}
@@ -567,7 +588,7 @@ class OpalMixerEndPoint : public OpalLocalEndPoint, public OpalMixerNodeManager
      */
     OpalMixerEndPoint(
       OpalManager & manager,  ///<  Manager of all endpoints.
-      const char * prefix     ///<  Prefix for URL style address strings
+      const char * prefix = OPAL_MIXER_PREFIX  ///<  Prefix for URL style address strings
     );
 
     /**Destroy endpoint.
@@ -763,8 +784,8 @@ class OpalMixerEndPoint : public OpalLocalEndPoint, public OpalMixerNodeManager
 
     OpalMixerNodeInfo  * m_adHocNodeInfo;
     OpalMixerNodeInfo  * m_factoryNodeInfo;
-    PMutex               m_infoMutex; // For above two fields
-    PAtomicInteger       m_factoryIndex;
+    PDECLARE_MUTEX(m_infoMutex); // For above two fields
+    atomic<uint32_t>     m_factoryIndex;
 };
 
 
@@ -955,6 +976,13 @@ class OpalMixerMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
+    /**Start the media stream.
+
+       The default behaviour calls Resume() on the associated OpalMediaPatch
+       thread if it was suspended.
+      */
+    virtual PBoolean Start();
+
     /**Write an RTP frame of data to the sink media stream.
        The default behaviour simply calls WriteData() on the data portion of the
        RTP_DataFrame and and sets the internal timestamp and marker from the
@@ -989,18 +1017,99 @@ class OpalMixerMediaStream : public OpalMediaStream
     PSafePtr<OpalMixerNode> GetNode() { return m_node; }
   //@}
 
+#if OPAL_VIDEO
+    bool CheckMixedVideoSize(unsigned width, unsigned height);
+#endif
+
   protected:
     virtual void InternalClose();
-    virtual bool InternalSetJitterBuffer(const OpalJitterBuffer::Init & init) const;
+    virtual bool InternalSetJitterBuffer(const OpalJitterBuffer::Init & init);
 
     PSafePtr<OpalMixerNode> m_node;
     bool m_listenOnly;
 #if OPAL_VIDEO
-    bool m_video;
+    unsigned m_mixedVideoWidth;
+    unsigned m_mixedVideoHeight;
 #endif
 };
 
 
+class OpalMediaStreamMixer
+{
+  public:
+    OpalMediaStreamMixer();
+    void Append(const PSafePtr<OpalMixerMediaStream> & stream);
+    void Remove(const PSafePtr<OpalMixerMediaStream> & stream) { m_outputStreams.Remove(stream); }
+    void CloseOne(const PSafePtr<OpalMixerMediaStream> & stream);
+
+  protected:
+    PSafeList<OpalMixerMediaStream> m_outputStreams;
+};
+
+/** Audio mixer.
+    This class represents the store for the mixed audio.
+*/
+class OpalAudioStreamMixer : public OpalAudioMixer, public OpalMediaStreamMixer
+{
+    PCLASSINFO(OpalAudioStreamMixer, OpalAudioMixer);
+  public:
+    OpalAudioStreamMixer(const OpalMixerNodeInfo & info);
+    ~OpalAudioStreamMixer();
+
+    virtual bool OnPush();
+
+  protected:
+    struct CachedAudio
+    {
+      CachedAudio();
+      ~CachedAudio();
+      enum
+      {
+        Collecting, Collected, Completed
+      } m_state;
+      RTP_DataFrame    m_raw;
+      RTP_DataFrame    m_encoded;
+      OpalTranscoder * m_transcoder;
+    };
+    std::map<PString, CachedAudio> m_cache;
+
+    void PushOne(
+      PSafePtr<OpalMixerMediaStream> & stream,
+      CachedAudio & cache,
+      const short * audioToSubtract
+    );
+
+#ifdef OPAL_MIXER_AUDIO_DEBUG
+    class PAudioMixerDebug * m_audioDebug;
+#endif
+};
+
+
+#if OPAL_VIDEO
+/** Video mixer.
+    This class represents the frame store for the mixed video.
+
+    A user would typically create a derived class to override the MixVideo()
+    or the StartMix()/NextMix() virtual functions for new screen
+    patterns/algorithms.
+*/
+class OpalVideoStreamMixer : public OpalVideoMixer, public OpalMediaStreamMixer
+{
+    PCLASSINFO(OpalVideoStreamMixer, OpalVideoMixer);
+  public:
+    OpalVideoStreamMixer(const OpalMixerNodeInfo & info);
+    ~OpalVideoStreamMixer();
+
+    virtual bool SetFrameRate(unsigned rate);
+    virtual bool OnMixed(RTP_DataFrame * & output);
+
+  protected:
+    typedef PDictionary<PString, OpalTranscoder> TranscoderMap;
+    TranscoderMap m_transcoders;
+};
+#endif // OPAL_VIDEO
+
+
 /** Mixer node.
     This class represents a group of connections that are being mixed.
   */
@@ -1081,23 +1190,14 @@ class OpalMixerNode : public PSafeObject
     bool SetJitterBufferSize(
       const OpalBaseMixer::Key_T & key,     ///< key for mixer stream
       const OpalJitterBuffer::Init & init   ///< Initialisation information
-    ) { return m_audioMixer.SetJitterBufferSize(key, init); }
+    );
 
     /**Write data to mixer.
       */
-    bool WriteAudio(
-      const OpalBaseMixer::Key_T & key, ///< key for mixer stream
-      const RTP_DataFrame & input       ///< Input RTP data for media
-    ) { return m_audioMixer.WriteStream(key, input); }
-
-#if OPAL_VIDEO
-    /**Write data to mixer.
-      */
-    bool WriteVideo(
-      const OpalBaseMixer::Key_T & key, ///< key for mixer stream
-      const RTP_DataFrame & input       ///< Input RTP data for media
-    ) { return m_videoMixer.WriteStream(key, input); }
-#endif // OPAL_VIDEO
+    bool WritePacket(
+      const OpalMixerMediaStream & stream,  ///< key for mixer stream
+      const RTP_DataFrame & input           ///< Input RTP data for media
+    );
 
     /**Send a user input indication to all connections.
       */
@@ -1135,7 +1235,7 @@ class OpalMixerNode : public PSafeObject
 
     /**Add a name for this node.
       */
-    void AddName(
+    bool AddName(
       const PString & name
     );
 
@@ -1190,59 +1290,19 @@ class OpalMixerNode : public PSafeObject
     PStringSet             m_names;
     OpalMixerNodeInfo    * m_info;
     PTime                  m_creationTime;
-    PAtomicBoolean         m_shuttingDown;
+    atomic<bool>           m_shuttingDown;
 
     PSafeList<OpalConnection> m_connections;
     PString                   m_ownerConnection;
 
-    struct MediaMixer
-    {
-      MediaMixer();
-      void CloseOne(const PSafePtr<OpalMixerMediaStream> & stream);
-
-      PSafeList<OpalMixerMediaStream> m_outputStreams;
-    };
-
-    struct AudioMixer : public OpalAudioMixer, public MediaMixer
-    {
-      AudioMixer(const OpalMixerNodeInfo & info);
-      ~AudioMixer();
-
-      virtual bool OnPush();
-
-      struct CachedAudio {
-        CachedAudio();
-        ~CachedAudio();
-        enum { Collecting, Collected, Completed } m_state;
-        RTP_DataFrame    m_raw;
-        RTP_DataFrame    m_encoded;
-        OpalTranscoder * m_transcoder;
-      };
-      std::map<PString, CachedAudio> m_cache;
-
-      void PushOne(
-        PSafePtr<OpalMixerMediaStream> & stream,
-        CachedAudio & cache,
-        const short * audioToSubtract
-      );
-#ifdef OPAL_MIXER_AUDIO_DEBUG
-      class PAudioMixerDebug * m_audioDebug;
-#endif
-    };
-    AudioMixer m_audioMixer;
-
+    OpalAudioStreamMixer * m_audioMixer;
 #if OPAL_VIDEO
-    struct VideoMixer : public OpalVideoMixer, public MediaMixer
-    {
-      VideoMixer(const OpalMixerNodeInfo & info);
-      ~VideoMixer();
-
-      virtual bool OnMixed(RTP_DataFrame * & output);
-
-      PDictionary<PString, OpalTranscoder> m_transcoders;
-    };
-    VideoMixer m_videoMixer;
+    typedef std::map<OpalVideoFormat::ContentRole, OpalVideoStreamMixer *> VideoMixerMap;
+    VideoMixerMap m_videoMixers;
 #endif // OPAL_VIDEO
+
+    typedef std::map<PString, OpalBaseMixer *> MixerByIdMap;
+    MixerByIdMap m_mixerById;
 };
 
 
