@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28481 $
+ * $Revision: 34097 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:13:17 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2015-09-27 19:11:34 +0100 (Sun, 27 Sep 2015) $
  */
 
 #ifndef OPAL_OPAL_PCSS_H
@@ -37,7 +37,7 @@
 #endif
 
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_HAS_PCSS
 
@@ -47,6 +47,8 @@
 
 class OpalPCSSConnection;
 
+#define OPAL_PCSS_PREFIX "pc"
+
 
 /** PC Sound System endpoint.
  */
@@ -60,7 +62,7 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
      */
     OpalPCSSEndPoint(
       OpalManager & manager,  ///<  Manager of all endpoints.
-      const char * prefix = "pc" ///<  Prefix for URL style address strings
+      const char * prefix = OPAL_PCSS_PREFIX ///<  Prefix for URL style address strings
     );
 
     /**Destroy endpoint.
@@ -141,6 +143,17 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
       const OpalLocalConnection & connection, ///<  Connection having event
       const PString & indication              ///<  Received user input indications
     );
+
+#if OPAL_VIDEO
+    /**Create an PVideoInputDevice for a source media stream.
+      */
+    virtual bool CreateVideoInputDevice(
+      const OpalConnection & connection,    ///<  Connection needing created video device
+      const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
+      PVideoInputDevice * & device,         ///<  Created device
+      bool & autoDelete                     ///<  Flag for auto delete device
+    );
+#endif
   //@}
 
   /**@name Customisation call backs */
@@ -164,6 +177,15 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
       const OpalMediaFormat & mediaFormat,   ///<  Media format for the connection
       PBoolean isSource                          ///<  Direction for channel
     );
+
+    /**Create an PSoundChannel based media stream.
+      */
+    virtual PSoundChannel * CreateSoundChannel(
+      const OpalPCSSConnection & connection, ///<  Connection needing created sound channel
+      const OpalMediaFormat & mediaFormat,   ///<  Media format for the connection
+      const PString & device,                ///<  Name of audio device to create
+      bool isSource                          ///<  Direction for channel
+    );
   //@}
 
   /**@name User Interface operations */
@@ -183,13 +205,13 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
        Note this function should not block or it will impede the operation of
        the stack.
 
-       The default implementation is pure.
+       The default implementation calls OpalPCSSEndPoint::OnIncomingCall().
 
        @return false is returned the call is aborted with status of EndedByLocalBusy.
       */
     virtual PBoolean OnShowIncoming(
       const OpalPCSSConnection & connection ///<  Connection having event
-    ) = 0;
+    );
 
     /**Accept the incoming connection.
        Returns false if the connection token does not correspond to a valid
@@ -215,7 +237,7 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
       */
     virtual PBoolean OnShowOutgoing(
       const OpalPCSSConnection & connection ///<  Connection having event
-    ) = 0;
+    );
 
     /**Call back to indicate that the remote user has indicated something.
        If false is returned the call is aborted.
@@ -230,6 +252,23 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
 
   /**@name Member variable access */
   //@{
+    /**Set the local ringback tone to be played when remote indicates it is
+       ringing.
+
+       This string may be one of several forms:
+          A filename for an existing WAV file.
+          A two letter country code, or the full country name.
+          A tone specification as per PTones class.
+      */
+    bool SetLocalRingbackTone(
+      const PString & tone    ///< Tone specification string
+    );
+
+    /**Get the local ringback tone to be played when remote indicates it is
+       ringing.
+      */
+    const PString & GetLocalRingbackTone() const { return m_localRingbackTone; }
+
     /**Set the name for the sound channel to be used for output.
        If the name is not suitable for use with the PSoundChannel class then
        the function will return false and not change the device.
@@ -243,7 +282,7 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
        This defaults to the value of the PSoundChannel::GetDefaultDevice()
        function.
      */
-    const PString & GetSoundChannelPlayDevice() const { return soundChannelPlayDevice; }
+    const PString & GetSoundChannelPlayDevice() const { return m_soundChannelPlayDevice; }
 
     /**Set the name for the sound channel to be used for input.
        If the name is not suitable for use with the PSoundChannel class then
@@ -258,13 +297,83 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
        This defaults to the value of the PSoundChannel::GetDefaultDevice()
        function.
      */
-    const PString & GetSoundChannelRecordDevice() const { return soundChannelRecordDevice; }
+    const PString & GetSoundChannelRecordDevice() const { return m_soundChannelRecordDevice; }
+
+    /**Set the name for the sound channel to be used for input when on hold.
+       If the name is not suitable for use with the PSoundChannel class then
+       the function will return false and not change the device.
+
+       This defaults to the "Null Audio".
+     */
+    virtual bool SetSoundChannelOnHoldDevice(const PString & name);
+
+    /**Get the name for the sound channel to be used for input when on hold.
+       This defaults to the "Null Audio".
+     */
+    const PString & GetSoundChannelOnHoldDevice() const { return m_soundChannelOnHoldDevice; }
+
+    /**Set the name for the sound channel to be used for input when ringing.
+
+       This defaults to an empty string which does not send video on ringing.
+     */
+    virtual bool SetSoundChannelOnRingDevice(const PString & name);
+
+    /**Get the name for the sound channel to be used for input when on hold.
+       This defaults to an empty string which does not send video on ringing.
+     */
+    const PString & GetSoundChannelOnRingDevice() const { return m_soundChannelOnRingDevice; }
+
+#if OPAL_VIDEO
+    /**Set the name for the video device to be used for sent video.
+     */
+    virtual bool SetVideoGrabberDevice(const PVideoDevice::OpenArgs & args);
+
+    /**Get the name for the video device to be used for sent video.
+     */
+    const PVideoDevice::OpenArgs & GetVideoGrabberDevice() const { return GetManager().GetVideoInputDevice(); }
+
+    /**Set the name for the video device to be used for displaying sent video.
+     */
+    virtual bool SetVideoPreviewDevice(const PVideoDevice::OpenArgs & args);
+
+    /**Get the name for the video device to be used for displaying sent video.
+     */
+    const PVideoDevice::OpenArgs & GetVideoPreviewDevice() const { return GetManager().GetVideoPreviewDevice(); }
+
+    /**Set the name for the video device to be used for displaying received video.
+     */
+    virtual bool SetVideoDisplayDevice(const PVideoDevice::OpenArgs & args);
+
+    /**Get the name for the video device to be used for displaying received video.
+     */
+    const PVideoDevice::OpenArgs & GetVideoDisplayDevice() const { return GetManager().GetVideoOutputDevice(); }
+
+    /**Set the name for the video device to be used for input when on hold.
+       This defaults to the "Null Video Out".
+     */
+    virtual bool SetVideoOnHoldDevice(const PVideoDevice::OpenArgs & args);
+
+    /**Get the name for the video device to be used for input when on hold.
+       This defaults to the "Null Video Out".
+     */
+    const PVideoDevice::OpenArgs & GetVideoOnHoldDevice() const { return m_videoOnHoldDevice; }
+
+    /**Set the name for the video device to be used for input when ringing.
+       This defaults to an empty string which does not send video on ringing.
+     */
+    virtual bool SetVideoOnRingDevice(const PVideoDevice::OpenArgs & args);
+
+    /**Get the name for the video device to be used for input when ringing.
+       This defaults to an empty string which does not send video on ringing.
+     */
+    const PVideoDevice::OpenArgs & GetVideoOnRingDevice() const { return m_videoOnRingDevice; }
+#endif
 
     /**Get default the sound channel buffer depth.
        Note the largest of the depth in frames and the depth in milliseconds
        as returned by GetSoundBufferTime() is used.
       */
-    unsigned GetSoundChannelBufferDepth() const { return soundChannelBuffers; }
+    unsigned GetSoundChannelBufferDepth() const { return m_soundChannelBuffers; }
 
     /**Set the default sound channel buffer depth.
        Note the largest of the depth in frames and the depth in milliseconds
@@ -290,10 +399,17 @@ class OpalPCSSEndPoint : public OpalLocalEndPoint
   //@}
 
   protected:
-    PString  soundChannelPlayDevice;
-    PString  soundChannelRecordDevice;
-    unsigned soundChannelBuffers;
+    PString  m_localRingbackTone;
+    PString  m_soundChannelPlayDevice;
+    PString  m_soundChannelRecordDevice;
+    PString  m_soundChannelOnHoldDevice;
+    PString  m_soundChannelOnRingDevice;
+    unsigned m_soundChannelBuffers;
     unsigned m_soundChannelBufferTime;
+#if OPAL_VIDEO
+    PVideoDevice::OpenArgs m_videoOnHoldDevice;
+    PVideoDevice::OpenArgs m_videoOnRingDevice;
+#endif
 
   private:
     P_REMOVE_VIRTUAL(OpalPCSSConnection *, CreateConnection(OpalCall &, const PString &, const PString &, void *), 0)
@@ -326,6 +442,40 @@ class OpalPCSSConnection : public OpalLocalConnection
 
   /**@name Overrides from OpalConnection */
   //@{
+    /**Clean up the termination of the connection.
+       This function can do any internal cleaning up and waiting on background
+       threads that may be using the connection object.
+
+       Note that there is not a one to one relationship with the
+       OnEstablishedConnection() function. This function may be called without
+       that function being called. For example if SetUpConnection() was used
+       but the call never completed.
+
+       Classes that override this function should make sure they call the
+       ancestor version for correct operation.
+
+       An application will not typically call this function as it is used by
+       the OpalManager during a release of the connection.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
+      */
+    virtual void OnReleased();
+
+    /**Indicate to remote endpoint an alert is in progress.
+       If this is an incoming connection and the AnswerCallResponse is in a
+       AnswerCallDeferred or AnswerCallPending state, then this function is
+       used to indicate to that endpoint that an alert is in progress. This is
+       usually due to another connection which is in the call (the B party)
+       has received an OnAlerting() indicating that its remote endpoint is
+       "ringing".
+
+       The default behaviour does nothing.
+      */
+    virtual PBoolean SetAlerting(
+      const PString & calleeName,   ///<  Name of endpoint being alerted.
+      PBoolean withMedia            ///<  Open media with alerting
+    );
+
     /**Initiate the transfer of an existing call (connection) to a new remote 
        party.
 
@@ -336,6 +486,15 @@ class OpalPCSSConnection : public OpalLocalConnection
       const PString & remoteParty   ///<  Remote party to transfer the existing call to
     );
 
+    /**Call back indicating result of last hold/retrieve operation.
+       This also indicates if the local connection has been put on hold by the
+       remote connection.
+     */
+    virtual void OnHold(
+      bool fromRemote,               ///<  Indicates remote has held local connection
+      bool onHold                    ///<  Indicates have just been held/retrieved.
+    );
+
     /**Open a new media stream.
        This will create a media stream of an appropriate subclass as required
        by the underlying connection protocol. For instance H.323 would create
@@ -356,6 +515,14 @@ class OpalPCSSConnection : public OpalLocalConnection
       PBoolean isSource                        ///<  Is a source stream
     );
 
+    /**Call back when media stream patch thread starts.
+
+       Default behaviour calls OpalManager function of same name.
+      */
+    virtual void OnStartMediaPatch(
+      OpalMediaPatch & patch    ///< Patch being started
+    );
+
     /**Set  the volume (gain) for the audio media channel.
        The volume range is 0 == muted, 100 == LOUDEST.
       */
@@ -392,6 +559,18 @@ class OpalPCSSConnection : public OpalLocalConnection
     virtual unsigned GetAudioSignalLevel(
       PBoolean source                   ///< true for source (microphone), false for sink (speaker)
     );
+
+    /**Indicate alerting for the incoming connection.
+       If GetSoundChannelOnRingDevice() is non-empty, then the \p withMedia
+       is overridden to true.
+      */
+    virtual void AlertingIncoming(
+      bool withMedia = false  ///< Indicate media should be started
+    );
+
+    /**Accept the incoming connection.
+      */
+    virtual void AcceptIncoming();
   //@}
 
   /**@name New operations */
@@ -402,27 +581,76 @@ class OpalPCSSConnection : public OpalLocalConnection
       const OpalMediaFormat & mediaFormat, ///<  Media format for the connection
       PBoolean isSource                        ///<  Direction for channel
     );
+
+    /**Change a PVideoInputDevice for a source media stream.
+      */
+    virtual bool ChangeSoundChannel(
+      const PString & device,   ///< Device to change to
+      bool isSource,            ///< Change source (recorder) or sink (player)
+      unsigned sessionID = 0    ///< Session for media stream, 0 indicates first audio stream
+    );
+
+    /**Read characters from the channel and produce User Input Indications.
+       Note that this will start a background thread to read the characters, so
+       the lifetime of the connection.
+      */
+    bool StartReadUserInput(
+      PChannel * channel,
+      bool autoDelete = true
+    );
+
+    /**Stop the background thread reading user input.
+       See StartReadUserInput().
+      */
+    void StopReadUserInput();
   //@}
 
   /**@name Member variable access */
   //@{
+    /**Get the local ringback tone to be played when remote indicates it is
+       ringing.
+      */
+    const PString & GetLocalRingbackTone() const { return m_localRingbackTone; }
+
     /**Get the name for the sound channel to be used for output.
        This defaults to the value of the PSoundChannel::GetDefaultDevice()
        function.
      */
-    const PString & GetSoundChannelPlayDevice() const { return soundChannelPlayDevice; }
+    const PString & GetSoundChannelPlayDevice() const { return m_soundChannelPlayDevice; }
 
     /**Get the name for the sound channel to be used for input.
        This defaults to the value of the PSoundChannel::GetDefaultDevice()
        function.
      */
-    const PString & GetSoundChannelRecordDevice() const { return soundChannelRecordDevice; }
+    const PString & GetSoundChannelRecordDevice() const { return m_soundChannelRecordDevice; }
+
+    /**Get the name for the sound channel to be used for input when on hold.
+       This defaults to the "Null Audio".
+     */
+    const PString & GetSoundChannelOnHoldDevice() const { return m_soundChannelOnHoldDevice; }
+
+    /**Get the name for the sound channel to be used for input when on hold.
+       This defaults to an empty string which does not send video on ringing.
+     */
+    const PString & GetSoundChannelOnRingDevice() const { return m_soundChannelOnRingDevice; }
+
+#if OPAL_VIDEO
+    /**Get the name for the video device to be used for input when on hold.
+       This defaults to the "Null Video Out".
+     */
+    const PVideoDevice::OpenArgs & GetVideoOnHoldDevice() const { return m_videoOnHoldDevice; }
+
+    /**Get the name for the video device to be used for input when ringing.
+       This defaults to an empty string which does not send video on ringing.
+     */
+    const PVideoDevice::OpenArgs & GetVideoOnRingDevice() const { return m_videoOnRingDevice; }
+#endif
 
     /**Get default the sound channel buffer depth.
        Note the largest of the depth in frames and the depth in milliseconds
        as returned by GetSoundBufferTime() is used.
       */
-    unsigned GetSoundChannelBufferDepth() const { return soundChannelBuffers; }
+    unsigned GetSoundChannelBufferDepth() const { return m_soundChannelBuffers; }
 
     /**Get default the sound channel buffer time in milliseconds.
        Note the largest of the depth in frames and the depth in milliseconds
@@ -433,25 +661,27 @@ class OpalPCSSConnection : public OpalLocalConnection
 
 
   protected:
-    OpalPCSSEndPoint & endpoint;
-    PString            soundChannelPlayDevice;
-    PString            soundChannelRecordDevice;
-    unsigned           soundChannelBuffers;
+    OpalPCSSEndPoint & m_endpoint;
+    PString            m_localRingbackTone;
+    PString            m_soundChannelPlayDevice;
+    PString            m_soundChannelRecordDevice;
+    PString            m_soundChannelOnHoldDevice;
+    PString            m_soundChannelOnRingDevice;
+    unsigned           m_soundChannelBuffers;
     unsigned           m_soundChannelBufferTime;
-};
-
-#else
-
-#ifdef _MSC_VER
-#pragma message("PTLib soundcard support not available")
-#else
-#warning "PTLib soundcard support not available"
+#if OPAL_VIDEO
+    PVideoDevice::OpenArgs m_videoOnHoldDevice;
+    PVideoDevice::OpenArgs m_videoOnRingDevice;
 #endif
 
+    PThread  * m_ringbackThread;
+    PSyncPoint m_ringbackStop;
+    void RingbackMain();
 
+    PThread  * m_userInputThread;
+    PChannel * m_userInputChannel;
+    bool       m_userInputAutoDelete;
+    void UserInputMain();
+};
 #endif // OPAL_HAS_PCSS
-
 #endif // OPAL_OPAL_PCSS_H
-
-
-// End of File ///////////////////////////////////////////////////////////////
