@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28351 $
+ * $Revision: 32828 $
  * $Author: rjongbloed $
- * $Date: 2012-09-13 19:58:35 -0500 (Thu, 13 Sep 2012) $
+ * $Date: 2014-09-26 04:52:33 +0100 (Fri, 26 Sep 2014) $
  */
 
 #ifndef OPAL_H323_CHANNELS_H
@@ -38,7 +38,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_H323
 
@@ -220,7 +220,7 @@ class H323Channel : public PObject
      */
     virtual PBoolean OnSendingPDU(
       H245_OpenLogicalChannel & openPDU  ///<  Open PDU to send. 
-    ) const = 0;
+    ) const;
 
     /**This is called when request to create a channel is received from a
        remote machine and is about to be acknowledged.
@@ -315,6 +315,12 @@ class H323Channel : public PObject
     /**Get the capability that created this channel.
       */
     const H323Capability & GetCapability() const { return *capability; }
+
+    /// Get the media format (from capability)
+    OpalMediaFormat GetMediaFormat() const;
+
+    /// Update media format options
+    virtual bool UpdateMediaFormat(const OpalMediaFormat & mediaFormat);
   //@}
 
   protected:
@@ -326,7 +332,7 @@ class H323Channel : public PObject
     H323ChannelNumber      number;
     H323ChannelNumber      reverseChannel;
     bool                   opened;
-    PAtomicInteger         m_terminating;
+    atomic<bool>           m_terminating;
 
   private:
     OpalBandwidth m_bandwidthUsed;
@@ -391,12 +397,13 @@ class H323UnidirectionalChannel : public H323Channel
     /**Open the channel.
       */
     virtual PBoolean Open();
+
+    /// Update media format options
+    virtual bool UpdateMediaFormat(const OpalMediaFormat & mediaFormat);
   //@}
 
   /**@name Member variable access */
   //@{
-    OpalMediaFormat GetMediaFormat() const { return m_mediaFormat; }
-
     /**Get the media stream associated with this logical channel.
        The default behaviour returns m_mediaStream.
       */
@@ -413,7 +420,6 @@ class H323UnidirectionalChannel : public H323Channel
     virtual void InternalClose();
 
     bool               receiver;
-    OpalMediaFormat    m_mediaFormat;
     OpalMediaStreamPtr m_mediaStream;
 };
 
@@ -554,12 +560,6 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       const H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
     );
 
-    /**Alternate RTP port information for Same NAT
-      */
-    virtual PBoolean OnReceivedAckAltPDU(
-      const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-    ) = 0;
-
     RTP_DataFrame::PayloadTypes GetDynamicRTPPayloadType() const;
   //@}
 };
