@@ -26,20 +26,20 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 29536 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef OPAL_H323_H323_H
 #define OPAL_H323_H323_H
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <h323/h323ep.h>
 #include <h323/h323con.h>
 #include <h323/gkclient.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 PString  OpalGetVersion();
 unsigned OpalGetMajorVersion();
