@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29960 $
+ * $Revision: 33270 $
  * $Author: rjongbloed $
- * $Date: 2013-06-13 23:26:53 -0500 (Thu, 13 Jun 2013) $
+ * $Date: 2015-01-22 05:15:09 +0000 (Thu, 22 Jan 2015) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -38,14 +38,16 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_H323
 
 #include <rtp/rtpconn.h>
 #include <opal/guid.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #include <h323/h323caps.h>
+#include <h323/h235dh.h>
+#include <h460/h460.h>
 #include <ptclib/dtmf.h>
 
 
@@ -56,6 +58,7 @@
  */
 class PPER_Stream;
 class PASN_OctetString;
+class PASN_Sequence;
 
 class H225_EndpointType;
 class H225_TransportAddress;
@@ -67,6 +70,7 @@ class H225_AdmissionReject;
 class H225_InfoRequestResponse;
 class H225_DisengageRequest;
 class H225_FeatureSet;
+class H225_H245Security;
 
 class H245_TerminalCapabilitySet;
 class H245_TerminalCapabilitySetReject;
@@ -112,9 +116,7 @@ class H45011Handler;
 
 class OpalCall;
 
-#if OPAL_H460
 class H460_FeatureSet;
-#endif
 
 
 /**OpalConnection::StringOption key to an . Default true.
@@ -210,7 +212,7 @@ class H323Connection : public OpalRTPConnection
     /** Called when a connection is established.
         This indicates that a connection to an endpoint was established. This
         usually occurs after OnConnected() and indicates that the connection
-        is both connected and has media flowing.
+        is both connected and media can flow.
 
         Default behaviour is to call H323EndPoint::OnConnectionEstablished
       */
@@ -292,6 +294,12 @@ class H323Connection : public OpalRTPConnection
       */
     virtual OpalMediaFormatList GetMediaFormats() const;
 
+    /**Get media security methods in priority order.
+       Returns an array of names for security methods,
+       e.g. { "Clear", "AES_CM_128_HMAC_SHA1_80", "AES_CM_128_HMAC_SHA1_32" }.
+      */
+    virtual PStringArray GetMediaCryptoSuites() const;
+
     /**Get next available session ID for the media type.
       */
     virtual unsigned GetNextSessionID(
@@ -333,6 +341,8 @@ class H323Connection : public OpalRTPConnection
                on to a remote system for direct access.
      */
     virtual bool GetMediaTransportAddresses(
+      OpalConnection & otherConnection,      ///< Other half of call needing media transport addresses
+      unsigned sessionId,                    ///< Session identifier
       const OpalMediaType & mediaType,       ///< Media type for session to return information
       OpalTransportAddressArray & transports ///<  Information on media session
     ) const;
@@ -620,15 +630,21 @@ class H323Connection : public OpalRTPConnection
       const PString & remoteParty   ///<  Remote party to transfer the existing call to
     );
 
-    /**Put the current connection on hold, suspending all media streams.
+    /**Put the current connection on hold, suspending media streams.
+       The streams from the remote are always paused. The streams from the
+       local to the remote are conditionally paused depending on underlying
+       logic for "music on hold" functionality.
+
        The \p fromRemote parameter indicates if we a putting the remote on
        hold (false) or it is a request for the remote to put us on hold (true).
 
-       The /p placeOnHold parameter indicates of teh command/request is for
+       The /p placeOnHold parameter indicates of the command/request is for
        going on hold or retrieving from hold.
+
+       @return true if hold request successfully initiated. The OnHold() call
+               back must be monitored for final confirmation of hold state.
      */
-    virtual bool Hold(
-      bool fromRemote,  ///< Flag for if remote has us on hold, or we have them
+    virtual bool HoldRemote(
       bool placeOnHold  ///< Flag for setting on or off hold
     );
 
@@ -638,7 +654,7 @@ class H323Connection : public OpalRTPConnection
      */
     virtual bool IsOnHold(
       bool fromRemote  ///< Flag for if remote has us on hold, or we have them
-    );
+    ) const;
 
 #if OPAL_H450
 
@@ -991,7 +1007,11 @@ class H323Connection : public OpalRTPConnection
        channel.
      */
     virtual PBoolean CreateOutgoingControlChannel(
-      const H225_TransportAddress & h245Address   ///<  H245 address
+      const PASN_Sequence & enclosingPDU,         ///< Enclosing PDU, e.g. H225_Connect_UUIE
+      const H225_TransportAddress & h245Address,  ///< H245 address
+      unsigned h245AddressField,                  ///< Optional field code for address
+      const H225_H245Security & h245Security,     ///< Optional Security mode for H.245 channel
+      unsigned h245SecurityField            ///< Optional field code for address
     );
 
     /**Start a separate control channel.
@@ -1007,7 +1027,11 @@ class H323Connection : public OpalRTPConnection
        channel.
       */
     virtual PBoolean CreateIncomingControlChannel(
-      H225_TransportAddress & h245Address  ///<  PDU transport address to set
+      PASN_Sequence & enclosingPDU,         ///< Enclosing PDU, e.g. H225_Connect_UUIE
+      H225_TransportAddress & h245Address,  ///< PDU transport address to set
+      unsigned h245AddressField,            ///< Optional field code for address
+      H225_H245Security & h245Security,     ///< Optional Security mode for H.245 channel
+      unsigned h245SecurityField            ///< Optional field code for address
     );
 
     /**Write a PDU to the control channel.
@@ -1728,9 +1752,8 @@ class H323Connection : public OpalRTPConnection
        OpalRTPConnection::DetermineRTPNAT() from being called.
       */
     virtual void DetermineRTPNAT(
-      const PIPSocket::Address & localAddr,   ///< Local physical address of connection
-      const PIPSocket::Address & peerAddr,    ///< Remote physical address of connection
-      const PIPSocket::Address & signalAddr   ///< Remotes signaling address as indicated by protocol of connection
+      const OpalTransport & transport,          ///< Transport to get physical address of connection
+      const OpalTransportAddress & signalAddr   ///< Remotes signaling address as indicated by protocol of connection
     );
 
   /**@name Request Mode Changes */
@@ -1927,7 +1950,7 @@ class H323Connection : public OpalRTPConnection
 
     /**Get the signalling channel being used.
       */
-    const H323Transport * GetSignallingChannel() const { return m_signallingChannel; }
+    OpalTransportPtr GetSignallingChannel() const { return m_signallingChannel; }
 
     /**Get the signalling channel protocol version number.
       */
@@ -2012,9 +2035,11 @@ class H323Connection : public OpalRTPConnection
     OpalMediaFormatList GetRemoteH239Formats() const;
 #endif
 
-    virtual PBoolean OnSendFeatureSet(unsigned, H225_FeatureSet &) const;
-    
-    virtual void OnReceiveFeatureSet(unsigned, const H225_FeatureSet &) const;
+#if OPAL_H235_6
+    /// Get the Diffie-Hellman handler
+    const H235DiffieHellman & GetDiffieHellman() const { return m_dh; }
+          H235DiffieHellman & GetDiffieHellman()       { return m_dh; }
+#endif
 
 #if OPAL_H450
     /**
@@ -2027,77 +2052,31 @@ class H323Connection : public OpalRTPConnection
 #if OPAL_H460
     /** Get the connection FeatureSet
      */
-    virtual H460_FeatureSet * GetFeatureSet();
+    H460_FeatureSet * GetFeatureSet() const { return m_features; }
 
-    /** Call to set the direction of call establishment
-      */
-    void H46019SetCallReceiver();
-
-    /** Enable H46019 for this call
-      */
-    void H46019Enabled();
+    virtual bool OnSendFeatureSet(H460_MessageType pduType, H225_FeatureSet &) const;
+    virtual void OnReceiveFeatureSet(H460_MessageType pduType, const H225_FeatureSet &) const;
+#endif
 
     /**Received OLC Generic Information. This is used to supply alternate RTP
           destination information in the generic information field in the OLC for the
           purpose of probing for an alternate route to the remote party.
     */
-    virtual PBoolean OnReceiveOLCGenericInformation(
+    virtual void OnReceiveOLCGenericInformation(
       unsigned sessionID,
-      const H245_ArrayOf_GenericInformation & alternate
+      const H245_ArrayOf_GenericInformation & info,
+      bool isAck
     ) const;
 
     /**Send Generic Information in the OLC. This is used to include generic
           information in the openlogicalchannel
     */
-   virtual PBoolean OnSendingOLCGenericInformation(
-     const unsigned & sessionID,              ///< Session Information
-     H245_ArrayOf_GenericInformation & gen,   ///< Generic OLC/OLCack message
-     PBoolean isAck
+   virtual bool OnSendingOLCGenericInformation(
+     unsigned sessionID,                      ///< Session Information
+     H245_ArrayOf_GenericInformation & info,  ///< Generic OLC/OLCack message
+     bool isAck
    ) const;
-#endif
     
-#if OPAL_H460_NAT
-    virtual PUDPSocket * GetNatSocket(unsigned session, PBoolean rtp);
-
-    /** Set RTP NAT information callback
-      */
-    virtual void SetRTPNAT(unsigned sessionid, PUDPSocket * _rtp, PUDPSocket * _rtcp);
-
-
-    /**Session Information 
-       This contains session information which is passed to the socket handler
-       when creating RTP socket pairs.
-      */
-    class SessionInformation : public PObject
-    {
-      public:
-        SessionInformation(const OpalGloballyUniqueID & id, const PString & token, unsigned session);
-
-        const PString & GetCallToken();
-
-        unsigned GetSessionID() const;
-
-        const OpalGloballyUniqueID & GetCallIdentifer();
-
-        const PString & GetCUI();
-
-      protected:
-        OpalGloballyUniqueID m_callID;
-        PString              m_callToken;
-        unsigned             m_sessionID;
-        PString              m_CUI;
-    };
-
-    SessionInformation * BuildSessionInformation(unsigned sessionID) const;
-
-    struct NAT_Sockets 
-    {
-        PUDPSocket * rtp;
-        PUDPSocket * rtcp;
-    };
-#endif // OPAL_H460_NAT
-
-
     /** Callback for media commands.
         Calls the SendIntraFrameRequest on the rtp session
 
@@ -2110,12 +2089,24 @@ class H323Connection : public OpalRTPConnection
 
     /**Compatibility workarounds.
       */
-    enum CompatibilityIssues {
+    P_DECLARE_ENUM(CompatibilityIssues,
       e_NoMultipleTunnelledH245, ///< Cannot have more than one tunnelled H.245 message
       e_BadMasterSlaveConflict,  ///< Endpoint does not repropose on OLC master/slave conflict refect
       e_NoUserInputCapability,   ///< Endpoint ignores TCS if has User Input Capability
-      NumCompatibilityIssues
-    };
+      e_H224MustBeSession3,      ///< H.224 channel must be session number 3, do not use H.245 negotiation
+      e_NeedTCSAfterNonEmptyTCS, ///< Must send a TCS after get a retrieve from hold (non-empty) TCS
+      e_NeedMSDAfterNonEmptyTCS, ///< Must send a MSD after get a retrieve from hold (non-empty) TCS
+      e_ForceMaintainConnection  ///< Server will re-use TCP connection for another without setting flag in PDU
+    );
+    P_DECLARE_ENUM_NAMES(CompatibilityIssues,
+      "No-Multiple-Tunnelled-H245",
+      "Bad-Master-Slave-Conflict",
+      "No-User-Input-Capability",
+      "H.224-Must-Be-Session-3",
+      "Need-TCS-after-Non-Empty-TCS",
+      "Need-MSD-after-Non-Empty-TCS",
+      "Force-Maintain-TCP-Connection"
+    );
 
     /// Determine if we must compensate for remote endpoint.
     bool HasCompatibilityIssue(
@@ -2133,11 +2124,12 @@ class H323Connection : public OpalRTPConnection
        the fast start algorithm.
     */
     virtual void InternalEstablishedConnectionCheck();
-    PBoolean InternalEndSessionCheck(PPER_Stream & strm);
+    bool InternalEndSessionCheck(PPER_Stream & strm);
+    bool SendReleaseComplete();
     void SetRemoteVersions(const H225_ProtocolIdentifier & id);
-    void SetBearerCapabilities(H323SignalPDU & pdu) const;
+    void SetOutgoingBearerCapabilities(H323SignalPDU & pdu) const;
+    void SetIncomingBearerCapabilities(const H323SignalPDU & pdu);
     void MonitorCallStatus();
-    PDECLARE_NOTIFIER(PThread, H323Connection, StartOutgoing);
     PDECLARE_NOTIFIER(PThread, H323Connection, NewOutgoingControlChannel);
     PDECLARE_AcceptHandlerNotifier(H323Connection, NewIncomingControlChannel);
 
@@ -2196,16 +2188,31 @@ class H323Connection : public OpalRTPConnection
 
     bool       mustSendDRQ;
     bool       mediaWaitForConnect;
-    bool       transmitterSidePaused;
-    bool       remoteTransmitPaused;
-    bool       earlyStart;
+    bool       m_holdToRemote;
+    bool       m_earlyStart;
     PString    t38ModeChangeCapabilities;
     PSyncPoint digitsWaitFlag;
-    bool       endSessionSent;
-    bool       endSessionNeeded;
+    bool       m_releaseCompleteNeeded;
+    bool       m_endSessionNeeded;
     PSyncPoint endSessionReceived;
     PTimer     enforcedDurationLimit;
     bool       isConsultationTransfer;
+    bool       m_maintainConnection;
+
+    template <class PDU> void SetMaintainConnectionFlag(const PDU & pdu)
+    {
+      if (pdu.HasOptionalField(PDU::e_maintainConnection) && pdu.m_maintainConnection)
+        m_maintainConnection = true;
+    }
+
+    PSafeDictionary<POrdinalKey, OpalMediaStream> m_conflictingChannels;
+
+    P_DECLARE_TRACED_ENUM(RemoteHoldStates,
+      eOffHoldFromRemote,
+      eOnHoldFromRemote,
+      eRetrieveFromRemote
+    );
+    RemoteHoldStates m_holdFromRemote;
 
     ConferenceGoal m_conferenceGoal;
 
@@ -2251,29 +2258,27 @@ class H323Connection : public OpalRTPConnection
 #endif
 
 #if OPAL_H460
-    bool disableH460;
-    H460_FeatureSet * features;
-
-    bool m_H46019CallReceiver;
-    bool m_H46019enabled;
-    bool m_h245Connect;
-
-    bool m_H46024Aenabled;
-    bool m_H46024Ainitator;
-    PINDEX m_H46024Astate;
-
-    bool m_H46024Benabled;
-    PINDEX m_H46024Bstate;
+    H460_FeatureSet * m_features;
 #endif
 
 #if OPAL_VIDEO
     PSimpleTimer m_h245FastUpdatePictureTimer;
 #endif
 
-#if OPAL_H460_NAT
-    PMutex NATSocketMutex;
-    std::map<unsigned, NAT_Sockets> m_NATSockets;
-#endif // OPAL_H460_NAT
+#if OPAL_H235_6
+    H235DiffieHellman m_dh;
+
+    template <class PDU> void SetDiffieHellman(const PDU & pdu)
+    {
+      if (pdu.HasOptionalField(PDU::e_tokens))
+        m_dh.FromTokens(pdu.m_tokens);
+    }
+#endif
+
+    char         m_lastUserInputIndication;
+    PSimpleTimer m_lastUserInputIndicationStart;
+    PTimer       m_UserInputIndicationTimer;
+    PDECLARE_NOTIFIER(PTimer, H323Connection, UserInputIndicationTimeout);
 
   private:
     P_REMOVE_VIRTUAL_VOID(CleanUpOnCallEnd());
