@@ -23,19 +23,21 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 33711 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2015-07-04 23:04:34 +0100 (Sat, 04 Jul 2015) $
  */
 
-#ifndef __OPAL_H323TRANS_H
-#define __OPAL_H323TRANS_H
+#ifndef OPAL_H323_H323TRANS_H
+#define OPAL_H323_H323TRANS_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
+
+#if OPAL_H323
 
 #include <h323/transaddr.h>
 #include <h323/h235auth.h>
@@ -74,19 +76,17 @@ class H323TransactionPDU {
       const H235Authenticators & auth
     ) { authenticators = auth; }
 
-    H235Authenticator::ValidationResult Validate(
-      const PASN_Array & clearTokens,
-      unsigned clearOptionalField,
-      const PASN_Array & cryptoTokens,
-      unsigned cryptoOptionalField
-    ) const { return authenticators.ValidatePDU(*this, clearTokens, clearOptionalField, cryptoTokens, cryptoOptionalField, rawPDU); }
+    template <class RAS> H235Authenticator::ValidationResult Validate(const RAS & ras) const
+    {
+      return authenticators.ValidatePDU(*this, ras, rawPDU);
+    }
 
-    void Prepare(
-      PASN_Array & clearTokens,
-      unsigned clearOptionalField,
-      PASN_Array & cryptoTokens,
-      unsigned cryptoOptionalField
-    ) { authenticators.PreparePDU(*this, clearTokens, clearOptionalField, cryptoTokens, cryptoOptionalField); }
+    template <class PDU> void Prepare(PDU & pdu)
+    {
+      authenticators.PreparePDU(*this, pdu);
+    }
+
+    const PBYTEArray & GetRawPDU() const { return rawPDU; }
 
   protected:
     mutable H235Authenticators authenticators;
@@ -140,14 +140,6 @@ class H323Transactor : public PObject
       const H323TransportAddress & iface ///<  Local interface for transport
     );
 
-    /**Return the list of addresses used for this peer element
-      */
-    H323TransportAddressArray GetInterfaceAddresses(
-      PBoolean excludeLocalHost = PTrue,       ///<  Flag to exclude 127.0.0.1
-      H323Transport * associatedTransport = NULL
-                          ///<  Associated transport for precedence and translation
-    );
-
     /**Start the channel processing transactions
       */
     virtual PBoolean StartChannel();
@@ -184,7 +176,7 @@ class H323Transactor : public PObject
     virtual PBoolean WriteTo(
       H323TransactionPDU & pdu,
       const H323TransportAddressArray & addresses,
-      PBoolean callback = PTrue
+      PBoolean callback = true
     );
   //@}
 
@@ -224,14 +216,10 @@ class H323Transactor : public PObject
     {
         PCLASSINFO(Request, PObject);
       public:
-        Request(
-          unsigned seqNum,
-          H323TransactionPDU & pdu
-        );
         Request(
           unsigned seqNum,
           H323TransactionPDU & pdu,
-          const H323TransportAddressArray & addresses
+          const H323TransportAddressArray & addresses = H323TransportAddressArray()
         );
 
         PBoolean Poll(H323Transactor &,
@@ -240,18 +228,14 @@ class H323Transactor : public PObject
         void CheckResponse(unsigned, const PASN_Choice *);
         void OnReceiveRIP(unsigned milliseconds);
 
+        unsigned                  sequenceNumber;
+        H323TransactionPDU      & requestPDU;
+        H323TransportAddressArray requestAddresses;
+
         // Inter-thread transfer variables
         unsigned rejectReason;
         void   * responseInfo;
 
-        H323TransportAddressArray requestAddresses;
-
-        unsigned             sequenceNumber;
-        H323TransactionPDU & requestPDU;
-        PTimeInterval        whenResponseExpected;
-        PSyncPoint           responseHandled;
-        PMutex               responseMutex;
-
         enum {
           AwaitingResponse,
           ConfirmReceived,
@@ -261,6 +245,10 @@ class H323Transactor : public PObject
           RequestInProgress,
           NoResponseReceived
         } responseResult;
+
+        PTimeInterval whenResponseExpected;
+        PSyncPoint    responseHandled;
+        PMutex        responseMutex;
     };
 
     virtual PBoolean MakeRequest(
@@ -275,13 +263,12 @@ class H323Transactor : public PObject
       const H323TransactionPDU & pdu,
       unsigned delay
     );
-    PBoolean CheckCryptoTokens(
+    bool CheckCryptoTokens1(const H323TransactionPDU & pdu);
+    bool CheckCryptoTokens2();
+    template <class RAS> bool CheckCryptoTokens(
       const H323TransactionPDU & pdu,
-      const PASN_Array & clearTokens,
-      unsigned clearOptionalField,
-      const PASN_Array & cryptoTokens,
-      unsigned cryptoOptionalField
-    );
+      const RAS & ras
+    ) { return CheckCryptoTokens1(pdu) || (pdu.Validate(ras) == H235Authenticator::e_OK || CheckCryptoTokens2()); }
 
     void AgeResponses();
     PBoolean SendCachedResponse(
@@ -308,10 +295,9 @@ class H323Transactor : public PObject
     WORD            defaultLocalPort;
     WORD            defaultRemotePort;
     H323Transport * transport;
-    PBoolean            checkResponseCryptoTokens;
+    bool            checkResponseCryptoTokens;
 
-    unsigned  nextSequenceNumber;
-    PMutex    nextSequenceNumberMutex;
+    atomic<uint16_t> m_nextSequenceNumber;
 
     PDictionary<POrdinalKey, Request> requests;
     PMutex                            requestsMutex;
@@ -359,14 +345,15 @@ class H323Transaction : public PObject
       H323TransactionPDU & pdu
     );
 
-    PBoolean CheckCryptoTokens(
-      const H235Authenticators & authenticators
-    );
+    virtual void PrepareConfirm() { }
+
+    virtual bool CheckCryptoTokens();
 
 #if PTRACING
     virtual const char * GetName() const = 0;
 #endif
     virtual H235Authenticator::ValidationResult ValidatePDU() const = 0;
+    virtual unsigned GetSecurityRejectTag() const { return 0; }
     virtual void SetRejectReason(
       unsigned reasonCode
     ) = 0;
@@ -378,6 +365,7 @@ class H323Transaction : public PObject
     PBoolean IsBehindNAT() const { return isBehindNAT; }
     H323Transactor & GetTransactor() const { return transactor; }
     H235Authenticator::ValidationResult GetAuthenticatorResult() const { return authenticatorResult; }
+    void SetAuthenticators(const H235Authenticators & auth) { authenticators = auth; }
 
   protected:
     virtual Response OnHandlePDU() = 0;
@@ -386,15 +374,15 @@ class H323Transaction : public PObject
     H323Transactor         & transactor;
     unsigned                 requestSequenceNumber;
     H323TransportAddressArray replyAddresses;
-    PBoolean                     fastResponseRequired;
+    bool                     fastResponseRequired;
     H323TransactionPDU     * request;
     H323TransactionPDU     * confirm;
     H323TransactionPDU     * reject;
 
     H235Authenticators                  authenticators;
     H235Authenticator::ValidationResult authenticatorResult;
-    PBoolean                                isBehindNAT;
-    PBoolean                                canSendRIP;
+    bool                                isBehindNAT;
+    bool                                canSendRIP;
 };
 
 
@@ -436,7 +424,7 @@ class H323TransactionServer : public PObject
        If the array is empty then the string "*" is assumed which will listen
        on the standard UDP port on INADDR_ANY.
 
-       Returns PTrue if at least one interface was successfully started.
+       Returns true if at least one interface was successfully started.
       */
     PBoolean AddListeners(
       const H323TransportAddressArray & ifaces ///<  Interfaces to listen on.
@@ -452,7 +440,7 @@ class H323TransactionServer : public PObject
     /**Add a gatekeeper listener to this gatekeeper server given the transport.
        Note that the transport is then owned by the listener and will be
        deleted automatically when the listener is destroyed. Note also the
-       transport is deleted if this function returns PFalse and no listener was
+       transport is deleted if this function returns false and no listener was
        created.
       */
     PBoolean AddListener(
@@ -462,7 +450,7 @@ class H323TransactionServer : public PObject
     /**Add a gatekeeper listener to this gatekeeper server.
        Note that the gatekeeper listener is then owned by the gatekeeper
        server and will be deleted automatically when the listener is removed.
-       Note also the listener is deleted if this function returns PFalse and
+       Note also the listener is deleted if this function returns false and
        the listener was not used.
       */
     PBoolean AddListener(
@@ -503,7 +491,9 @@ class H323TransactionServer : public PObject
 };
 
 
-#endif // __OPAL_H323TRANS_H
+#endif // OPAL_H323
+
+#endif // OPAL_H323_H323TRANS_H
 
 
 /////////////////////////////////////////////////////////////////////////////
