@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 27079 $
+ * $Revision: 32817 $
  * $Author: rjongbloed $
- * $Date: 2012-02-29 23:12:01 -0600 (Wed, 29 Feb 2012) $
+ * $Date: 2014-09-25 05:00:38 +0100 (Thu, 25 Sep 2014) $
  */
 
 #ifndef OPAL_IAX2_CALLPROCESSOR_H
@@ -36,7 +36,7 @@
 #include <ptlib.h>
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
@@ -461,16 +461,16 @@ class IAX2CallProcessor : public IAX2Processor
   IAX2FullFrameProtocol *BuildNewFrameForSending(IAX2FullFrameProtocol *inReplyTo = NULL);
   
   /**Count of the number of sound frames sent */
-  PAtomicInteger audioFramesSent;
+  atomic<uint32_t> audioFramesSent;
   
   /**Count of the number of sound frames received */
-  PAtomicInteger audioFramesRcvd;
+  atomic<uint32_t> audioFramesRcvd;
   
   /**Count of the number of video frames sent */
-  PAtomicInteger videoFramesSent;
+  atomic<uint32_t> videoFramesSent;
   
   /**Count of the number of video frames received */
-  PAtomicInteger videoFramesRcvd;
+  atomic<uint32_t> videoFramesRcvd;
   
   /**Phone number of the remote  endpoint */
   SafeString remotePhoneNumber;
