@@ -27,15 +27,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 29536 $
+ * $Author: rjongbloed $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef OPAL_IAX2_IAX2_H
 #define OPAL_IAX2_IAX2_H
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
