@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 29658 $
+ * $Revision: 33270 $
  * $Author: rjongbloed $
- * $Date: 2013-05-07 21:16:26 -0500 (Tue, 07 May 2013) $
+ * $Date: 2015-01-22 05:15:09 +0000 (Thu, 22 Jan 2015) $
  */
 
 #ifndef OPAL_IAX2_IAX2EP_H
@@ -40,7 +40,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
@@ -266,7 +266,7 @@ class IAX2EndPoint : public OpalEndPoint
     /**A call back function whenever a connection is established.
        This indicates that a connection to an endpoint was established. This
        usually occurs after OnConnected() and indicates that the connection
-       is both connected and has media flowing.
+       is both connected and media can flow.
       */
     void OnEstablished(
        OpalConnection & con
@@ -499,7 +499,7 @@ class IAX2EndPoint : public OpalEndPoint
 
   /**Thread safe counter which keeps track of the calls created by this endpoint.
      This value is used when giving outgoing calls a unique ID */
-  PAtomicInteger callsEstablished;
+  atomic<uint32_t> callsEstablished;
 
   /**Local copy of the media types we can handle*/
   OpalMediaFormatList localMediaFormats;
