@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 27154 $
+ * $Revision: 30574 $
  * $Author: rjongbloed $
- * $Date: 2012-03-07 19:21:33 -0600 (Wed, 07 Mar 2012) $
+ * $Date: 2013-09-26 04:26:16 +0100 (Thu, 26 Sep 2013) $
  */
 
 #ifndef OPAL_IAX2_MEDIASTRM_H
@@ -37,7 +37,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
@@ -75,19 +75,6 @@ class OpalIAX2MediaStream : public OpalMediaStream
  public:
   /**@name Overrides of OpalMediaStream class */
   //@{
-    /**Open the media stream.
- 
- 
-      */
-    virtual PBoolean Open();
- 
-    /**Start the media stream.
-       
-    The default behaviour calls Resume() on the associated
-    OpalMediaPatch thread if it was suspended.
-    */
-    virtual PBoolean Start();
-
     /**
        Goes to the IAX2Connection class, and removes a packet from the connection. The connection class turned the media 
        packet into a RTP_DataFrame class, and jitter buffered it.
