@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 25515 $
+ * $Revision: 29536 $
  * $Author: rjongbloed $
- * $Date: 2011-04-06 01:51:53 -0500 (Wed, 06 Apr 2011) $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef OPAL_IAX2_IES_H
@@ -36,7 +36,7 @@
 #include <ptlib.h>
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
