@@ -23,15 +23,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28208 $
+ * $Revision: 29536 $
  * $Author: rjongbloed $
- * $Date: 2012-08-16 20:54:46 -0500 (Thu, 16 Aug 2012) $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef OPAL_IM_RFC4103_H
 #define OPAL_IM_RFC4103_H
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_HAS_RFC4103
 
