@@ -23,16 +23,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28394 $
+ * $Revision: 29536 $
  * $Author: rjongbloed $
- * $Date: 2012-09-24 00:17:07 -0500 (Mon, 24 Sep 2012) $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef OPAL_IM_SIPIM_H
 #define OPAL_IM_SIPIM_H
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_HAS_SIPIM
 
