@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21328 $
+ * $Revision: 31817 $
  * $Author: rjongbloed $
- * $Date: 2008-10-14 05:50:19 -0500 (Tue, 14 Oct 2008) $
+ * $Date: 2014-04-30 06:56:11 +0100 (Wed, 30 Apr 2014) $
  */
 
 #ifndef OPAL_LIDS_LID_H
@@ -38,7 +38,9 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
+
+#if OPAL_LID
 
 #include <opal/mediafmt.h>
 
@@ -112,13 +114,13 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean IsLinePresent(
       unsigned line,      ///<  Number of line
-      PBoolean force = PFalse  ///<  Force test, do not optimise
+      PBoolean force = false  ///<  Force test, do not optimise
     );
 
 
     /**Determine if line is currently off hook.
        This function implies that the state is debounced and that a return
-       value of PTrue indicates that the phone is really off hook. That is
+       value of true indicates that the phone is really off hook. That is
        hook flashes and winks are masked out.
       */
     virtual PBoolean IsLineOffHook(
@@ -132,7 +134,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean SetLineOffHook(
       unsigned line,        ///<  Number of line
-      PBoolean newState = PTrue  ///<  New state to set
+      PBoolean newState = true  ///<  New state to set
     ) = 0;
 
     /**Set the hook state of the line.
@@ -140,7 +142,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean SetLineOnHook(
       unsigned line        ///<  Number of line
-    ) { return SetLineOffHook(line, PFalse); }
+    ) { return SetLineOffHook(line, false); }
 
     /**Set the hook state off then straight back on again.
        This will only operate if the line is currently off hook.
@@ -150,14 +152,14 @@ class OpalLineInterfaceDevice : public PObject
       unsigned flashTime = 200    ///<  Time for hook flash in milliseconds
     );
 
-    /**Return PTrue if a hook flash has been detected
+    /**Return true if a hook flash has been detected
       */
     virtual PBoolean HasHookFlash(unsigned line);
 
 
     /**Determine if line is ringing.
        This function implies that the state is "debounced" and that a return
-       value of PTrue indicates that the phone is still ringing and it is not
+       value of true indicates that the phone is still ringing and it is not
        simply a pause in the ring cadence.
 
        If cadence is not NULL then it is set with the bit pattern for the
@@ -237,7 +239,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean IsLineDisconnected(
       unsigned line,   ///<  Number of line
-      PBoolean checkForWink = PTrue
+      PBoolean checkForWink = true ///< Flag to check for remote hang up
     );
 
 
@@ -384,14 +386,14 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean EnableAudio(
       unsigned line,      ///<  Number of line
-      PBoolean enable = PTrue
+      PBoolean enable = true ///< Enable/disable audio
     );
 
     /**Disable audio for the line.
       */
     PBoolean DisableAudio(
       unsigned line   ///<  Number of line
-    ) { return EnableAudio(line, PFalse); }
+    ) { return EnableAudio(line, false); }
 
     /**Determine if audio for the line is enabled.
       */
@@ -487,7 +489,7 @@ class OpalLineInterfaceDevice : public PObject
        ID data, or if full is true, all of the fields in the caller ID data.
 
        The full data of the caller ID string consists fields separated by tab
-       characters ('\t'), the first three are always the Calling Line Identity
+       characters ('\\t'), the first three are always the Calling Line Identity
        (CLI or calling number), the date and the Calling Line Name field. Other
        fields may follow and are the of the form name=value. The values are
        LID dependent.
@@ -498,7 +500,7 @@ class OpalLineInterfaceDevice : public PObject
     virtual PBoolean GetCallerID(
       unsigned line,      ///<  Number of line
       PString & idString, ///<  ID string returned
-      PBoolean full = PFalse   ///<  Get full information in idString
+      PBoolean full = false   ///<  Get full information in idString
     );
 
     /**Set Caller ID information.
@@ -506,7 +508,7 @@ class OpalLineInterfaceDevice : public PObject
        Identity.
 
        The full data of the caller ID string consists fields separated by tab
-       characters ('\t'), the first three are always the Calling Line Identity
+       characters ('\\t'), the first three are always the Calling Line Identity
        (CLI or calling number), the date and the Calling Line Name field. Other
        fields may follow and are the of the form name=value. The values are
        LID dependent.
@@ -531,7 +533,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean SendVisualMessageWaitingIndicator(
       unsigned line,            ///<  Number of line
-      PBoolean on
+      PBoolean on               ///< Flag for VMWI on/off
     );
 
 
@@ -551,7 +553,7 @@ class OpalLineInterfaceDevice : public PObject
     );
 
     /**Read a DTMF digit detected.
-       This may be characters from the set 0-9, A-D, * or #. A null ('\0')
+       This may be characters from the set 0-9, A-D, * or #. A null ('\\0')
        character indicates that there are no tones in the queue.
        Characters E through P indicate the following tones:
 
@@ -710,7 +712,7 @@ class OpalLineInterfaceDevice : public PObject
        The takes the line off hook, waits for dial tone, and transmits the
        specified number as DTMF tones.
 
-       If the requireTones flag is PTrue the call is aborted of the call
+       If the requireTones flag is true the call is aborted of the call
        progress tones are not detected. Otherwise the call proceeds with short
        delays while it tries to detect the call progress tones.
 
@@ -788,11 +790,28 @@ class OpalLineInterfaceDevice : public PObject
       */
     PString GetCountryCodeName() const;
 
-    /**Get the country code set for the device as a string.
+    /**Get the country code as a string.
       */
     static PString GetCountryCodeName(T35CountryCodes code);
+
+    /**Get the country code from a string.
+      */
     static T35CountryCodes GetCountryCode(const PString & name);
 
+    struct T35CountryInfo
+    {
+      const char *    m_isoName;
+      unsigned        m_dialCode;
+      T35CountryCodes m_t35Code;
+      const char    * m_fullName;
+      const char    * m_tone[NumTones];
+    };
+
+    /**Get the T.35 country information.
+    */
+    static const T35CountryInfo & GetCountryInfo(T35CountryCodes code);
+
+
     /**Set the country code set for the device.
        This may change the line analogue coefficients, ring detect, call
        disconnect detect and call progress tones to fit the countries
@@ -831,8 +850,8 @@ class OpalLineInterfaceDevice : public PObject
       * start recording audio
       */
     virtual PBoolean RecordAudioStart(
-      unsigned line,            /// line
-      const PString & filename  /// File Name
+      unsigned line,            ///< line
+      const PString & filename  ///< File Name
     );
     
     /**
@@ -840,7 +859,7 @@ class OpalLineInterfaceDevice : public PObject
      */
         
     virtual PBoolean RecordAudioStop(
-      unsigned line            /// line
+      unsigned line            ///< line
     );
     
 
@@ -943,13 +962,13 @@ class OpalLine : public PObject
     /**Determine if a physical line is present on the logical line.
       */
     virtual PBoolean IsPresent(
-      PBoolean force = PFalse  ///<  Force test, do not optimise
+      PBoolean force = false  ///<  Force test, do not optimise
     ) { return device.IsLinePresent(lineNumber, force); }
 
 
     /**Determine if line is currently off hook.
        This function implies that the state is debounced and that a return
-       value of PTrue indicates that the phone is really off hook. That is
+       value of true indicates that the phone is really off hook. That is
        hook flashes and winks are masked out.
       */
     virtual PBoolean IsOffHook() { return device.IsLineOffHook(lineNumber); }
@@ -959,12 +978,12 @@ class OpalLine : public PObject
        a standard telephone handset. The hook state is determined by external
        hardware and cannot be changed by the software.
       */
-    virtual PBoolean SetOffHook() { return device.SetLineOffHook(lineNumber, PTrue); }
+    virtual PBoolean SetOffHook() { return device.SetLineOffHook(lineNumber, true); }
 
     /**Set the hook state of the line.
        This is the complement of SetLineOffHook().
       */
-    virtual PBoolean SetOnHook() { return device.SetLineOffHook(lineNumber, PFalse); }
+    virtual PBoolean SetOnHook() { return device.SetLineOffHook(lineNumber, false); }
 
     /**Set the hook state off then straight back on again.
        This will only operate if the line is currently off hook.
@@ -973,14 +992,14 @@ class OpalLine : public PObject
       unsigned flashTime = 200    ///<  Time for hook flash in milliseconds
     ) { return device.HookFlash(lineNumber, flashTime); }
 
-    /**Return PTrue if a hook flash has been detected
+    /**Return true if a hook flash has been detected
       */
     virtual PBoolean HasHookFlash() { return device.HasHookFlash(lineNumber); }
 
 
     /**Determine if line is ringing.
        This function implies that the state is "debounced" and that a return
-       value of PTrue indicates that the phone is still ringing and it is not
+       value of true indicates that the phone is still ringing and it is not
        simply a pause in the ring cadence.
 
        If cadence is not NULL then it is set with the bit pattern for the
@@ -1145,12 +1164,12 @@ class OpalLine : public PObject
     /**Enable audio for the line.
       */
     virtual PBoolean EnableAudio(
-      PBoolean enable = PTrue
+      PBoolean enable = true
     ) { return device.EnableAudio(lineNumber, enable); }
 
     /**Disable audio for the line.
       */
-    PBoolean DisableAudio() { return EnableAudio(PFalse); }
+    PBoolean DisableAudio() { return EnableAudio(false); }
 
     /**Determine if audio is ebabled for the line.
       */
@@ -1221,7 +1240,7 @@ class OpalLine : public PObject
        ID data, or if full is true, all of the fields in the caller ID data.
 
        The full data of the caller ID string consists fields separated by tab
-       characters ('\t'), the first three are always the Calling Line Identity
+       characters ('\\t'), the first three are always the Calling Line Identity
        (CLI or calling number), the date and the Calling Line Name field. Other
        fields may follow and are the of the form name=value. The values are
        LID dependent.
@@ -1231,7 +1250,7 @@ class OpalLine : public PObject
       */
     virtual PBoolean GetCallerID(
       PString & idString, ///<  ID string returned
-      PBoolean full = PFalse   ///<  Get full information in idString
+      PBoolean full = false   ///<  Get full information in idString
     ) { return device.GetCallerID(lineNumber, idString, full); }
 
     /**Set Caller ID information.
@@ -1239,7 +1258,7 @@ class OpalLine : public PObject
        Identity.
 
        The full data of the caller ID string consists fields separated by tab
-       characters ('\t'), the first three are always the Calling Line Identity
+       characters ('\\t'), the first three are always the Calling Line Identity
        (CLI or calling number), the date and the Calling Line Name field. Other
        fields may follow and are the of the form name=value. The values are
        LID dependent.
@@ -1276,7 +1295,7 @@ class OpalLine : public PObject
     ) { return device.PlayDTMF(lineNumber, digits, onTime, offTime); }
 
     /**Read a DTMF digit detected.
-       This may be characters from the set 0-9, A-D, * or #. A null ('\0')
+       This may be characters from the set 0-9, A-D, * or #. A null ('\\0')
        character indicates that there are no tones in the queue.
        Characters E through P indicate the following tones:
 
@@ -1337,7 +1356,7 @@ class OpalLine : public PObject
        The takes the line off hook, waits for dial tone, and transmits the
        specified number as DTMF tones.
 
-       If the requireTones flag is PTrue the call is aborted of the call
+       If the requireTones flag is true the call is aborted of the call
        progress tones are not detected. Otherwise the call proceeds with short
        delays while it tries to detect the call progress tones.
 
@@ -1449,6 +1468,7 @@ OpalLineInterfaceDevice * cls##_Registration::Create(void * param) const
   OPAL_REGISTER_LID_FUNCTION(cls, type, parameter) \
   { return new cls(parameter); }
 
+#endif // OPAL_LID
 
 #endif // OPAL_LIDS_LID_H
 
