@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27154 $
+ * $Revision: 29536 $
  * $Author: rjongbloed $
- * $Date: 2012-03-07 19:21:33 -0600 (Wed, 07 Mar 2012) $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #ifndef OPAL_LIDS_LIDEP_H
@@ -38,7 +38,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_LID
 
