@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29572 $
+ * $Revision: 33929 $
  * $Author: rjongbloed $
- * $Date: 2013-04-24 23:28:21 -0500 (Wed, 24 Apr 2013) $
+ * $Date: 2015-08-27 13:45:19 +0100 (Thu, 27 Aug 2015) $
  */
 
 #ifndef OPAL_OPAL_CALL_H
@@ -36,7 +36,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/connection.h>
 #include <opal/recording.h>
@@ -46,6 +46,7 @@
 
 
 class OpalManager;
+class OpalRTPSession;
 
 
 /**This class manages a call.
@@ -192,8 +193,6 @@ class OpalCall : public PSafeObject
        remote endpoint is "ringing". Generally some time after the
        MakeConnection() function was called, this is function is called.
 
-       If false is returned the connection is aborted.
-
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
        only intercept this function if it wishes to do some form of logging.
@@ -203,9 +202,11 @@ class OpalCall : public PSafeObject
        The default behaviour is to call SetAlerting() on all the other
        connections in the call.
       */
-    virtual PBoolean OnAlerting(
-      OpalConnection & connection   ///<  Connection that indicates it is alerting
+    virtual void OnAlerting(
+      OpalConnection & connection,   ///<  Connection that indicates it is alerting
+      bool withMedia                 ///<  Indicated media should be started, if possible
     );
+    virtual PBoolean OnAlerting(OpalConnection & connection); // For backward compatibility
 
     /**Call back for answering an incoming call.
        This function is called after the connection has been acknowledged
@@ -246,7 +247,7 @@ class OpalCall : public PSafeObject
     /**A call back function whenever a connection is "established".
        This indicates that a connection to an endpoint was established. This
        usually occurs after OnConnected() and indicates that the connection
-       is both connected and has media flowing.
+       is both connected and media can flow.
 
        In the context of H.323 this means that the CONNECT pdu has been
        received and either fast start was in operation or the subsequent Open
@@ -326,19 +327,21 @@ class OpalCall : public PSafeObject
        This function places the remote user of the network connection in the
        call on hold.
       */
-    bool Hold();
+    bool Hold(bool hold = true);
 
     /**Retrieve call from hold.
        This function retrieves the remote user of the network connection in
        the call from hold.
       */
-    bool Retrieve();
+    bool Retrieve() { return Hold(false); }
 
     /**Indicate if call is in hold.
-       This function determines if the remote user of the network connection
-       in the call is currently on hold.
+       This function determines if the local or remote user of the network
+       connection in the call is currently on hold.
       */
-    bool IsOnHold() const;
+    bool IsOnHold(
+      bool fromRemote = false
+    ) const;
 
     /**Transfer connection.
        There are several scenarios for this function.
@@ -423,7 +426,8 @@ class OpalCall : public PSafeObject
 #if OPAL_VIDEO
       OpalVideoFormat::ContentRole contentRole = OpalVideoFormat::eNoRole, ///< Content role for video
 #endif
-      bool transfer = false
+      bool transfer = false,
+      bool startPaused = false
     );
 
     /**Select media format pair from the source/destination list.
@@ -434,15 +438,11 @@ class OpalCall : public PSafeObject
       const OpalMediaType & mediaType,        ///< Media type for selection.
       const OpalMediaFormatList & srcFormats, ///<  Names of possible source formats
       const OpalMediaFormatList & dstFormats, ///<  Names of possible destination formats
-      const OpalMediaFormatList & allFormats, ///<  Master list of formats for merging options
+      const OpalMediaFormatList & masterFormats, ///<  Master list of formats for merging options
       OpalMediaFormat & srcFormat,            ///<  Selected source format to be used
       OpalMediaFormat & dstFormat             ///<  Selected destination format to be used
     ) const;
 
-    /**Start the media streams on the connections.
-     */
-    virtual void StartMediaStreams();
-
     /**Close the media streams on the connections.
      */
     virtual void CloseMediaStreams();
@@ -461,8 +461,16 @@ class OpalCall : public PSafeObject
     );
 
     /**Call back for remote enpoint has sent user input as tones.
-       If duration is zero then this indicates the beginning of the tone. If
-       duration is non-zero then it indicates the end of the tone output.
+       If \p duration is zero then this indicates the beginning of the tone.
+       If \p duration is greater than zero then it indicates the end of the
+       tone output and how long the tone had run.
+
+       Note, there is no guarantee a zero value (start tone) will occur. There
+       is also no guarantee this function is called at all, given how the
+       remote may send user indications. For simple, "event" based, user
+       indications the OnUserInputString() should be used. THis function is
+       only for when a more precise representation of the tone, and it's
+       duration, is required.
 
        The default behaviour calls connection.OnUserInputString(tone) if there
        are no other connections in the call, otherwise it calls
@@ -632,7 +640,8 @@ class OpalCall : public PSafeObject
     PTime   m_establishedTime;
     bool    m_isEstablished;
     bool    m_isClearing;
-    PAtomicBoolean m_isCleared;
+    bool    m_handlingHold;
+    atomic<bool> m_isCleared;
 
     OpalConnection::CallEndReason callEndReason;
     std::list<PSyncPoint *> m_endCallSyncPoint;
