@@ -32,7 +32,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -44,6 +44,12 @@ class OpalMediaCommand : public PObject
 {
   PCLASSINFO(OpalMediaCommand, PObject);
   public:
+    OpalMediaCommand(
+      const OpalMediaType & mediaType, ///< Media type to search for in open streams
+      unsigned sessionID = 0,          ///< Session for media stream, 0 is use first \p mediaType stream
+      unsigned ssrc = 0                ///< Sync Source for media stream (if RTP)
+    );
+
   /**@name Overrides from PObject */
   //@{
     /**Standard stream print function.
@@ -87,36 +93,75 @@ class OpalMediaCommand : public PObject
       */
     virtual unsigned * GetPlugInSize() const;
   //@}
+
+  /**@name Memebers */
+  //@{
+    const OpalMediaType & GetMediaType() const { return m_mediaType; }
+    unsigned GetSessionID() const { return m_sessionID; }
+    unsigned GetSyncSource() const { return m_ssrc; }
+  //@}
+
+  protected:
+    OpalMediaType m_mediaType;
+    unsigned      m_sessionID;
+    unsigned      m_ssrc;
 };
 
 
-#define OPAL_DEFINE_MEDIA_COMMAND(cls, name) \
+#define OPAL_DEFINE_MEDIA_COMMAND(cls, name, mediaType) \
   class cls : public OpalMediaCommand \
   { \
       PCLASSINFO_WITH_CLONE(cls, OpalMediaCommand) \
     public: \
-      cls() { } \
+      cls(unsigned id = 0, unsigned ssrc = 0) : OpalMediaCommand(mediaType, id, ssrc) { } \
       virtual PString GetName() const { return name; } \
   }
 
 
-/**This indicates that the media flow (bit rate) is to be adjsuted.
+/**This indicates that the media flow (bit rate) is to be adjusted.
   */
 class OpalMediaFlowControl : public OpalMediaCommand
 {
     PCLASSINFO_WITH_CLONE(OpalMediaFlowControl, OpalMediaCommand);
   public:
-    OpalMediaFlowControl(unsigned maxBitRate)
-      : m_maxBitRate(maxBitRate)  { }
+    OpalMediaFlowControl(
+      OpalBandwidth bitRate,           ///< Bandwidth to use
+      const OpalMediaType & mediaType, ///< Media type to search for in open streams
+      unsigned sessionID = 0,          ///< Session for media stream, 0 is use first \p mediaType stream
+      unsigned ssrc = 0                ///< Sync Source for media stream (if RTP)
+    );
 
     virtual PString GetName() const;
 
-    unsigned GetMaxBitRate() const { return m_maxBitRate; }
+    const OpalBandwidth & GetMaxBitRate() const { return m_bitRate; }
 
   protected:
-    unsigned m_maxBitRate;
+    OpalBandwidth m_bitRate;
 };
 
+
+/**This indicates the packet loss as percentage over the last arbitrary period.
+  */
+class OpalMediaPacketLoss : public OpalMediaCommand
+{
+    PCLASSINFO_WITH_CLONE(OpalMediaPacketLoss, OpalMediaCommand);
+  public:
+    OpalMediaPacketLoss(
+      unsigned packetLoss,             ///< Packet loss as percentage
+      const OpalMediaType & mediaType, ///< Media type to search for in open streams
+      unsigned sessionID = 0,          ///< Session for media stream, 0 is use first \p mediaType stream
+      unsigned ssrc = 0                ///< Sync Source for media stream (if RTP)
+    );
+
+    virtual PString GetName() const;
+
+    unsigned GetPacketLoss() const { return m_packetLoss; }
+
+  protected:
+    unsigned m_packetLoss;
+};
+
+
 #endif // OPAL_OPAL_MEDIACMD_H
 
 
