@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29330 $
+ * $Revision: 34064 $
  * $Author: rjongbloed $
- * $Date: 2013-03-26 19:55:21 -0500 (Tue, 26 Mar 2013) $
+ * $Date: 2015-09-22 20:33:19 +0100 (Tue, 22 Sep 2015) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -36,7 +36,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <ptclib/delaychan.h>
 
@@ -47,11 +47,9 @@
 #include <ptclib/guid.h>
 
 
-class OpalRTPSession;
 class OpalMediaPatch;
 class OpalLine;
 class OpalConnection;
-class OpalRTPConnection;
 class OpalMediaStatistics;
 
 
@@ -171,14 +169,15 @@ class OpalMediaStream : public PSafeObject
     /**Update the media format. This can be used to adjust the
        parameters of a codec at run time. Note you cannot change the basic
        media format, eg change GSM0610 to G.711, only options for that
-       format, eg 6k3 mode to 5k3 mode in G.723.1. If the formats are
-       different then a OpalMediaFormat::Merge() is performed.
-
-       The default behaviour updates the mediaFormat member variable and
-       pases the value on to the OpalMediaPatch.
+       format, eg 6k3 mode to 5k3 mode in G.723.1.
+       
+       If the formats are different, or the \p mergeOnly parameter is true,
+       then a OpalMediaFormat::Merge() rather than OpalMediaFormat::Update()
+       is performed.
       */
     bool UpdateMediaFormat(
-      const OpalMediaFormat & mediaFormat   ///<  New media format
+      const OpalMediaFormat & mediaFormat,    ///< New media format
+      bool mergeOnly = false                  ///< Indicate merge parameters
     );
 
     /**Execute the command specified to the transcoder. The commands are
@@ -189,9 +188,9 @@ class OpalMediaStream : public PSafeObject
 
        @returns true if command is handled.
       */
-    virtual PBoolean ExecuteCommand(
+    bool ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
-    );
+    ) const;
 
     /**Open the media stream using the media format.
 
@@ -203,6 +202,10 @@ class OpalMediaStream : public PSafeObject
       */
     virtual bool IsOpen() const;
 
+    /**Returns true if the media stream is established.
+    */
+    virtual bool IsEstablished() const;
+
     /**Start the media stream.
 
        The default behaviour calls Resume() on the associated OpalMediaPatch
@@ -229,6 +232,14 @@ class OpalMediaStream : public PSafeObject
       OpalMediaPatch & patch    ///< Media patch that is stopping
     );
 
+    /** Bypass media patch.
+        Send media directly to/from the media streams.
+      */
+    virtual bool SetMediaPassThrough(
+      OpalMediaStream & otherStream,  ///< Stream to bypass media data to/from
+      bool bypass                     ///< Turn bypass on or off
+    );
+
     /**Write a list of RTP frames of data to the sink media stream.
        The default behaviour simply calls WritePacket() on each of the
        elements in the list.
@@ -279,7 +290,7 @@ class OpalMediaStream : public PSafeObject
 
     /**Pushes a frame to the patch
       */
-    bool PushPacket(
+    virtual bool PushPacket(
       RTP_DataFrame & packet
     );
 
@@ -296,7 +307,7 @@ class OpalMediaStream : public PSafeObject
     /**Get the data size in bytes that is expected to be used. Some media
        streams can make use of this information to perform optimisations.
       */
-    PINDEX GetDataSize() const { return defaultDataSize; }
+    PINDEX GetDataSize() const { return m_defaultDataSize; }
 
     /**Indicate if the media stream is synchronous.
        If this returns true then the media stream will block of the amount of
@@ -321,12 +332,12 @@ class OpalMediaStream : public PSafeObject
     virtual PBoolean RequiresPatchThread() const; // For backward compatibility
 
     /**Enable jitter buffer for the media stream.
-       Returns true if a jitter buffer is enabled/disabled. Returns false if
-       no jitter buffer exists for the media stream.
+       Returns true if a jitter buffer is enabled. Returns false if the jitter
+       buffer is disabled or no jitter buffer can be used on the media stream.
 
        The default behaviour does nothing and returns false.
       */
-    virtual bool EnableJitterBuffer(bool enab = true) const;
+    virtual bool EnableJitterBuffer(bool enab = true);
   //@}
 
   /**@name Member variable access */
@@ -380,10 +391,9 @@ class OpalMediaStream : public PSafeObject
        This will stop reading/writing data from the stream.
        Returns true if the pause state was changed
       */
-    virtual bool SetPaused(
-      bool pause,             ///< Indicate that the stream should be paused
-      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
-    );
+    bool SetPaused(
+      bool pause  ///< Indicate that the stream should be paused
+    ) { return InternalSetPaused(pause, true, false); }
     
     /**Set the patch thread that is using this stream.
       */
@@ -412,16 +422,32 @@ class OpalMediaStream : public PSafeObject
 #if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
 #endif
+
+    P_DECLARE_BITWISE_ENUM(Details, 6,(
+      DetailMinimum,
+      DetailEOL, // end of line
+      DetailNAT,
+      DetailSecured,
+      DetailFEC,
+      DetailAudio,
+      DetailAddresses
+    ));
+    virtual void PrintDetail(
+      ostream & strm,
+      const char * prefix = NULL,
+      Details details = Details::All()
+    ) const;
   //@}
 
+    virtual PString GetPatchThreadName() const;
     virtual bool InternalUpdateMediaFormat(const OpalMediaFormat & mediaFormat);
+    virtual bool InternalSetPaused(bool pause, bool fromUser, bool fromPatch);
+    virtual bool InternalExecuteCommand(const OpalMediaCommand & command);
 
   protected:
-    void IncrementTimestamp(PINDEX size);
-    bool InternalWriteData(const BYTE * data, PINDEX length, PINDEX & written);
     OpalMediaPatchPtr InternalSetPatchPart1(OpalMediaPatch * newPatch);
     void InternalSetPatchPart2(const OpalMediaPatchPtr & oldPatch);
-    virtual bool InternalSetJitterBuffer(const OpalJitterBuffer::Init & init) const;
+    virtual bool InternalSetJitterBuffer(const OpalJitterBuffer::Init & init);
 
     /**Close any internal components of the stream.
        This should be used in preference to overriding the Close() function as
@@ -432,12 +458,13 @@ class OpalMediaStream : public PSafeObject
 
     OpalConnection & connection;
     unsigned         sessionID;
+    WORD             m_sequenceNumber;
     PString          identifier;
     OpalMediaFormat  mediaFormat;
     bool             m_paused;
     bool             m_isSource;
     bool             m_isOpen;
-    PINDEX           defaultDataSize;
+    PINDEX           m_defaultDataSize;
     unsigned         timestamp;
     bool             marker;
 
@@ -472,7 +499,7 @@ class OpalMediaStreamPacing
 
     /// Delay appropriate time for the written bytes
     void Pace(
-      bool reading,     ///< Are reading from medium
+      bool generated,   ///< Are generating data, set marker true
       PINDEX bytes,     ///< Bytes read/written
       bool & marker     ///< RTP Marker
     );
@@ -487,6 +514,9 @@ class OpalMediaStreamPacing
     PINDEX         m_frameSize;
     unsigned       m_timeUnits;
     PAdaptiveDelay m_delay;
+    unsigned       m_previousDelay;
+
+    PTRACE_THROTTLE(m_throttleLog, 4, 5000);
 };
 
 
@@ -537,15 +567,6 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
       PINDEX & written     ///<  Length of data actually written
     );
 	
-    /**Set the paused state for stream.
-       This will stop reading/writing data from the stream.
-       Returns true if the pause state was changed
-      */
-    virtual bool SetPaused(
-      bool pause,             ///< Indicate that the stream should be paused
-      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
-    );
-
     /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
        The default behaviour returns the value of m_isSynchronous.
       */
@@ -560,121 +581,13 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
   protected:
     virtual void InternalClose() { }
     virtual bool InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat);
+    virtual bool InternalSetPaused(bool pause, bool fromUser, bool fromPatch);
 
     bool m_isSynchronous;
     bool m_requiresPatchThread;
 };
 
 
-/**This class describes a media stream that transfers data to/from a RTP
-   session.
-  */
-class OpalRTPMediaStream : public OpalMediaStream
-{
-    PCLASSINFO(OpalRTPMediaStream, OpalMediaStream);
-  public:
-  /**@name Construction */
-  //@{
-    /**Construct a new media stream for RTP sessions.
-       This will add a reference to the rtpSession passed in.
-      */
-    OpalRTPMediaStream(
-      OpalRTPConnection & conn,            ///<  Connection that owns the stream
-      const OpalMediaFormat & mediaFormat, ///<  Media format for stream
-      bool isSource,                       ///<  Is a source stream
-      OpalRTPSession & rtpSession          ///<  RTP session to stream to/from
-    );
-
-    /**Destroy the media stream for RTP sessions.
-       This will release the reference to the rtpSession passed into the constructor.
-      */
-    ~OpalRTPMediaStream();
-  //@}
-
-  /**@name Overrides of OpalMediaStream class */
-  //@{
-    /**Open the media stream using the media format.
-
-       The default behaviour simply sets the isOpen variable to true.
-      */
-    virtual PBoolean Open();
-
-    /**Returns true if the media stream is open.
-      */
-    virtual bool IsOpen() const;
-
-    /**Set the paused state for stream.
-       This will stop reading/writing data from the stream.
-      */
-    virtual bool SetPaused(
-      bool pause,             ///< Indicate that the stream should be paused
-      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
-    );
-
-    /**Read an RTP frame of data from the source media stream.
-       The new behaviour simply calls OpalRTPSession::ReadData().
-      */
-    virtual PBoolean ReadPacket(
-      RTP_DataFrame & packet
-    );
-
-    /**Write an RTP frame of data to the sink media stream.
-       The new behaviour simply calls OpalRTPSession::WriteData().
-      */
-    virtual PBoolean WritePacket(
-      RTP_DataFrame & packet
-    );
-
-    /**Set the data size in bytes that is expected to be used.
-      */
-    virtual PBoolean SetDataSize(
-      PINDEX dataSize,  ///< New data size (in total)
-      PINDEX frameTime  ///< Individual frame time (if applicable)
-    );
-
-    /**Indicate if the media stream is synchronous.
-       Returns false for RTP streams.
-      */
-    virtual PBoolean IsSynchronous() const;
-
-    /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
-       The default behaviour dermines if the media will be flowing between two
-       RTP sessions within the same process. If so the
-       OpalRTPConnection::OnLocalRTP() is called, and if it returns true
-       indicating local handling then this function returns faklse to disable
-       the patch thread.
-      */
-    virtual PBoolean RequiresPatchThread() const;
-
-    /**Set the patch thread that is using this stream.
-      */
-    virtual PBoolean SetPatch(
-      OpalMediaPatch * patch  ///<  Media patch thread
-    );
-
-    /** Return current RTP session
-      */
-    virtual OpalRTPSession & GetRtpSession() const
-    { return rtpSession; }
-
-#if OPAL_STATISTICS
-    virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
-#endif
-  //@}
-
-  protected:
-    virtual void InternalClose();
-    virtual bool InternalSetJitterBuffer(const OpalJitterBuffer::Init & init) const;
-
-    OpalRTPSession & rtpSession;
-#if OPAL_VIDEO
-    bool             m_forceIntraFrameFlag;
-    PSimpleTimer     m_forceIntraFrameTimer;
-#endif
-};
-
-
-
 /**This class describes a media stream that transfers PCM-16 data to/from a PChannel.
   */
 class OpalRawMediaStream : public OpalMediaStream
@@ -741,7 +654,7 @@ class OpalRawMediaStream : public OpalMediaStream
 
     PChannel * m_channel;
     bool       m_autoDelete;
-    PMutex     m_channelMutex;
+    PDECLARE_MUTEX(m_channelMutex);
 
     PBYTEArray m_silence;
 
@@ -916,17 +829,6 @@ class OpalVideoMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Execute the command specified to the transcoder. The commands are
-       highly context sensitive, for example OpalVideoUpdatePicture would only
-       apply to a video transcoder.
-
-       The default behaviour checks for video fast update and then
-       passes the command on to the OpalMediaPatch.
-      */
-    virtual PBoolean ExecuteCommand(
-      const OpalMediaCommand & command    ///<  Command to execute.
-    );
-
     /**Read raw media data from the source media stream.
        The default behaviour simply calls ReadPacket() on the data portion of the
        RTP_DataFrame and sets the frames timestamp and marker from the internal
@@ -989,20 +891,38 @@ class OpalVideoMediaStream : public OpalMediaStream
       return m_outputDevice;
     }
 
+    /** Set the watermark device
+      */
+    virtual void SetVideoWatermarkDevice(
+      PVideoInputDevice * device,
+      bool autoDelete = true
+    );
+
+    /** Get the watermark device
+      */
+    virtual PVideoInputDevice * GetVideoWatermarkDevice() const
+    {
+      return m_watermarkDevice;
+    }
   //@}
 
   protected:
     virtual void InternalClose();
     virtual bool InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat);
+    virtual bool InternalExecuteCommand(const OpalMediaCommand & command);
     bool InternalAdjustDevices();
+    virtual void ApplyWatermark(unsigned width, unsigned height, BYTE * frame);
 
     PVideoInputDevice  * m_inputDevice;
+    PVideoInputDevice  * m_watermarkDevice;
     PVideoOutputDevice * m_outputDevice;
     bool                 m_autoDeleteInput;
+    bool                 m_autoDeleteWatermark;
     bool                 m_autoDeleteOutput;
     PTimeInterval        m_lastGrabTime;
     bool                 m_needKeyFrame;
-    PMutex               m_devicesMutex;
+    PDECLARE_MUTEX(m_devicesMutex);
+    PBYTEArray           m_watermarkData;
 };
 
 #endif // OPAL_VIDEO
