@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29334 $
+ * $Revision: 34467 $
  * $Author: rjongbloed $
- * $Date: 2013-03-26 22:08:07 -0500 (Tue, 26 Mar 2013) $
+ * $Date: 2015-12-30 18:17:31 +0000 (Wed, 30 Dec 2015) $
  */
 
 #ifndef OPAL_OPAL_PATCH_H
@@ -36,9 +36,10 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/mediastrm.h>
+#include <opal/mediasession.h>
 #include <opal/mediacmd.h>
 #include <codec/ratectl.h>
 
@@ -128,7 +129,7 @@ class OpalMediaPatch : public PSafeObject
 
     /**Get the current source stream for patch.
       */
-    OpalMediaStream & GetSource() const { return source; }
+    OpalMediaStream & GetSource() const { return m_source; }
 
     /**Get the media stream for a sink stream
       */
@@ -183,15 +184,15 @@ class OpalMediaPatch : public PSafeObject
 
        @returns true if command is handled.
       */
-    virtual PBoolean ExecuteCommand(
-      const OpalMediaCommand & command,   ///<  Command to execute.
-      PBoolean fromSink                       ///<  Flag for source or sink
+    virtual bool ExecuteCommand(
+      const OpalMediaCommand & command   ///<  Command to execute.
     );
 
     /**Set streams to paused.
       */
-    virtual bool SetPaused(
-      bool pause    ///< Indicate that the stream should be paused
+    virtual bool InternalSetPaused(
+      bool pause,    ///< Indicate that the stream should be paused
+      bool fromUser  ///< Pause is from user, not internal logic
     );
 
     /**Push a frame out to all the sink streams, transcoding as necessary.
@@ -223,6 +224,8 @@ class OpalMediaPatch : public PSafeObject
       const OpalMediaPatchPtr & patch
     );
 
+    bool IsBypassed() const { return m_bypassToPatch != NULL || m_bypassFromPatch != NULL; }
+
     /**Get the transcoder used within a sink stream
       */
     virtual OpalTranscoder * GetAndLockSinkTranscoder(PINDEX i = 0) const;
@@ -239,13 +242,15 @@ class OpalMediaPatch : public PSafeObject
   protected:
     PDECLARE_NOTIFIER(OpalMediaCommand, OpalMediaPatch, InternalOnMediaCommand1);
     void InternalOnMediaCommand2(OpalMediaCommand * command);
-                
+
+    bool CanStart() const;
+
     /**Called from the associated patch thread */
     virtual void Main();
     void StopThread();
     bool DispatchFrame(RTP_DataFrame & frame);
 
-    OpalMediaStream & source;
+    OpalMediaStream & m_source;
 
     class Sink : public PObject {
         PCLASSINFO(Sink, PObject);
@@ -254,47 +259,57 @@ class OpalMediaPatch : public PSafeObject
         ~Sink();
         bool CreateTranscoders();
         bool UpdateMediaFormat(const OpalMediaFormat & mediaFormat);
-        bool ExecuteCommand(const OpalMediaCommand & command);
-        bool WriteFrame(RTP_DataFrame & sourceFrame);
+        bool ExecuteCommand(const OpalMediaCommand & command, bool atLeastOne);
+        bool WriteFrame(RTP_DataFrame & sourceFrame, bool bypassing);
 #if OPAL_STATISTICS
         void GetStatistics(OpalMediaStatistics & statistics, bool fromSource) const;
 #endif
 
-        OpalMediaPatch  &  patch;
-        OpalMediaStreamPtr stream;
-        OpalTranscoder   * primaryCodec;
-        OpalTranscoder   * secondaryCodec;
-        RTP_DataFrameList  intermediateFrames;
-        RTP_DataFrameList  finalFrames;
-        bool               writeSuccessful;
-
-        RTP_DataFrame::PayloadTypes m_lastPayloadType;
-        unsigned                    m_consecutivePayloadTypeMismatches;
-        bool CannotTranscodeFrame(OpalTranscoder & codec, RTP_DataFrame & frame);
+        OpalMediaPatch  &  m_patch;
+        OpalMediaStreamPtr m_stream;
+        OpalTranscoder   * m_primaryCodec;
+        OpalTranscoder   * m_secondaryCodec;
+        RTP_DataFrameList  m_intermediateFrames;
+        RTP_DataFrameList  m_finalFrames;
 
 #if OPAL_VIDEO
         void SetRateControlParameters(const OpalMediaFormat & mediaFormat);
         bool RateControlExceeded(bool & forceIFrame);
-        OpalVideoRateController * rateController;
+        OpalVideoRateController * m_rateController;
+
+#if OPAL_STATISTICS
+        OpalVideoFormat     m_videoFormat;
+        PBYTEArray          m_keyFrameDetectContext;
+        typedef map<RTP_SyncSourceId, OpalVideoStatistics> VideoStatsMap;
+        VideoStatsMap m_videoStatistics;
+        PDECLARE_MUTEX(m_videoStatsMutex);
+#endif
 #endif
     };
-    PList<Sink> sinks;
+    PList<Sink> m_sinks;
 
     class Filter : public PObject {
         PCLASSINFO(Filter, PObject);
       public:
-        Filter(const PNotifier & n, const OpalMediaFormat & s) : notifier(n), stage(s) { }
-        PNotifier notifier;
-        OpalMediaFormat stage;
+        Filter(const PNotifier & n, const OpalMediaFormat & s) : m_notifier(n), m_stage(s) { }
+
+        PNotifier       m_notifier;
+        OpalMediaFormat m_stage;
     };
-    PList<Filter> filters;
+    PList<Filter> m_filters;
 
     OpalMediaPatchPtr m_bypassToPatch;
     OpalMediaPatchPtr m_bypassFromPatch;
     PSyncPoint        m_bypassEnded;
 
-    PThread * patchThread;
-    PMutex patchThreadMutex;
+    PThread * m_patchThread;
+    PDECLARE_MUTEX(m_patchThreadMutex);
+#if OPAL_STATISTICS
+    PThreadIdentifier m_patchThreadId;
+#endif
+
+
+    bool m_transcoderChanged;
 
   private:
     P_REMOVE_VIRTUAL(bool, OnPatchStart(), false);
@@ -317,6 +332,7 @@ class OpalPassiveMediaPatch : public OpalMediaPatch
     );
 
     virtual void Start();
+    virtual void Close();
 
   protected:
     bool m_started;
