@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 29047 $
+ * $Revision: 34323 $
  * $Author: rjongbloed $
- * $Date: 2013-01-29 20:23:19 -0600 (Tue, 29 Jan 2013) $
+ * $Date: 2015-11-21 15:25:51 +0000 (Sat, 21 Nov 2015) $
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -40,10 +40,11 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <ptlib/sockets.h>
 #include <ptclib/psockbun.h>
+#include <ptclib/http.h>
 
 
 class OpalManager;
@@ -160,6 +161,10 @@ class OpalTransportAddress : public PCaselessString
     static const PCaselessString & TcpPrefix();
 #if OPAL_PTLIB_SSL
     static const PCaselessString & TlsPrefix();
+#if OPAL_PTLIB_HTTP
+    static const PCaselessString & WsPrefix();  // RFC6455 WebSocket (yes, still needs SSL)
+    static const PCaselessString & WssPrefix(); // RFC6455 WebSocket over TLS
+#endif
 #endif
 
   /**@name Construction */
@@ -177,9 +182,13 @@ class OpalTransportAddress : public PCaselessString
     );
     OpalTransportAddress(
       const PIPSocket::Address & ip,  ///<  IP address for transport
-      WORD port,                      ///<  Default port number
+      WORD port = 0,                  ///<  Default port number
       const char * proto = NULL       ///<  Default is "tcp"
     );
+    OpalTransportAddress(
+      const PIPSocket::AddressAndPort & ap,  ///<  IP address and port for transport
+      const char * proto = NULL              ///<  Default is "tcp"
+    );
 
     /// Clone function
     virtual PObject * Clone() const { return new OpalTransportAddress(*this); }
@@ -321,47 +330,68 @@ class OpalTransportAddress : public PCaselessString
 class OpalTransportAddressArray : public PArray<OpalTransportAddress>
 {
     typedef PArray<OpalTransportAddress> ParentClass;
-    PCLASSINFO(OpalTransportAddressArray, ParentClass);
-
-  protected:
-    inline OpalTransportAddressArray(int dummy, const OpalTransportAddressArray * c)
-      : ParentClass(dummy, c) { }
+    PCLASSINFO_WITH_CLONE(OpalTransportAddressArray, ParentClass);
 
   public:
     OpalTransportAddressArray(PINDEX initialSize = 0)
       : ParentClass(initialSize) { }
+    OpalTransportAddressArray(
+      const PString & address
+    ) { AppendString(address); }
     OpalTransportAddressArray(
       const OpalTransportAddress & address
     ) { AppendAddress(address); }
     OpalTransportAddressArray(
-      const PStringArray & array
-    ) { AppendStringCollection(array); }
-    OpalTransportAddressArray(
-      const PStringList & list
-    ) { AppendStringCollection(list); }
-    OpalTransportAddressArray(
-      const PSortedStringList & list
-    ) { AppendStringCollection(list); }
+      const PCollection & coll
+    ) { AppendStringCollection(coll); }
 
-    void AppendString(
-      const char * address
-    );
-    void AppendString(
-      const PString & address
-    );
-    void AppendAddress(
+    /// Append one address to array
+    bool AppendAddress(
       const OpalTransportAddress & address
     );
 
-    virtual PObject * Clone() const
-    {
-      return new OpalTransportAddressArray(0, this);
-    }
+    /** Append a string with special cases.
+        If address is a string containing ',', ';', tab or newline separated
+        sub-strings, each one is added using AppendStringCollection().
 
-  protected:
-    void AppendStringCollection(
+        This may contain wild card strings matching interfaces that are
+        availabe in the system. For example "192.168.*" will match any
+        interfaces starting with 192.168. The string "0.0.0.0" will match all
+        IPv4 interfaces. The string "[::]" will map to all IPv6 interfaces.
+        Note the loopback interface will only be included if explictly
+        indicated.
+      */
+    bool AppendString(
+      const PString & address
+    );
+
+    /** Append all the strings in the collection.
+        Note the collection must contain PString objects.
+        See AppendString() for how each string in the collection is
+        interpreted.
+      */
+    bool AppendStringCollection(
       const PCollection & coll
     );
+
+    /// Set the array to 1 or 2 entries.
+    bool SetAddressPair(
+      const OpalTransportAddress & addr1,
+      const OpalTransportAddress & addr2
+    );
+
+    /** Get IP addres and port at the index, modulus the size of the array.
+        If the ap is already valid, then it is returned as is. If the array
+        is empty or the entry is invalid, then IPv4 INADDR_ANY and port zero
+        is returned.
+      */
+    void GetIpAndPort(
+      PINDEX index,
+      PIPAddressAndPort & ap
+    );
+
+  protected:
+    bool AddInterfaces(const PIPAddress & network, const PIPAddress & mask, const PString & portStr);
 };
 
 
@@ -414,7 +444,6 @@ class OpalListener : public PObject
     typedef PNotifierTemplate<const OpalTransportPtr &> AcceptHandler;
     #define PDECLARE_AcceptHandlerNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalListener, cls, fn, const OpalTransportPtr &)
 
-
     /** Open the listener.
         A thread is spawned to listen for incoming connections. The notifier
         function acceptHandler is called when a new connection is created. The
@@ -461,7 +490,8 @@ class OpalListener : public PObject
        external address and port.
       */
     virtual OpalTransportAddress GetLocalAddress(
-      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress(),
+      const OpalTransportAddress & defaultAddress = OpalTransportAddress() // If INADDR_ANY
     ) const = 0;
 
     /**Close channel and wait for associated thread to terminate.
@@ -474,6 +504,7 @@ class OpalListener : public PObject
     void CleanUpOnTermination() { CloseWait(); }
   //@}
 
+    virtual const PCaselessString & GetProtoPrefix() const = 0;
 
   protected:
     /**Handle incoming connections and dispatch them in new threads based on
@@ -487,10 +518,16 @@ class OpalListener : public PObject
     void ListenForConnections();
     void TransportThreadMain(OpalTransportPtr transport);
 
-    OpalEndPoint & endpoint;
-    PThread      * thread;
-    AcceptHandler  acceptHandler;
-    ThreadMode     threadMode;
+    OpalEndPoint & m_endpoint;
+    PThread      * m_thread;
+    AcceptHandler  m_acceptHandler;
+    ThreadMode     m_threadMode;
+
+  private:
+    OpalListener(const OpalListener & other) : PObject(other), m_endpoint(other.m_endpoint) { }
+    void operator=(const OpalListener &) { }
+
+  friend class OpalHTTPConnector;
 };
 
 
@@ -526,15 +563,14 @@ class OpalListenerIP : public OpalListener
        external address and port.
       */
     virtual OpalTransportAddress GetLocalAddress(
-      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress(),
+      const OpalTransportAddress & defaultAddress = OpalTransportAddress() // If INADDR_ANY
     ) const;
   //@}
 
   /**@name Operations */
   //@{
-    WORD GetListenerPort() const { return listenerPort; }
-
-    virtual const PCaselessString & GetProtoPrefix() const = 0;
+    WORD GetListenerPort() const { return m_binding.GetPort(); }
   //@}
 
 
@@ -544,9 +580,8 @@ class OpalListenerIP : public OpalListener
       const OpalTransportAddress & remoteAddress
     ) const;
 
-    PIPSocket::Address localAddress;
-    WORD               listenerPort;
-    bool               exclusiveListener;
+    PIPSocketAddressAndPort m_binding;
+    bool                    m_exclusiveListener;
 };
 
 
@@ -627,8 +662,9 @@ class OpalListenerTCP : public OpalListenerIP
 
   protected:
     virtual const PCaselessString & GetProtoPrefix() const;
+    virtual OpalTransport * OnAccept(PTCPSocket * socket);
 
-    PTCPSocket listener;
+    PTCPSocket m_listener;
 };
 
 
@@ -711,7 +747,8 @@ class OpalListenerUDP : public OpalListenerIP
        external address and port.
       */
     virtual OpalTransportAddress GetLocalAddress(
-      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress(),
+      const OpalTransportAddress & defaultAddress = OpalTransportAddress() // If INADDR_ANY
     ) const;
   //@}
 
@@ -728,7 +765,7 @@ class OpalListenerUDP : public OpalListenerIP
   protected:
     virtual const PCaselessString & GetProtoPrefix() const;
 
-    PMonitoredSocketsPtr listenerBundle;
+    PMonitoredSocketsPtr m_listenerBundle;
     PINDEX               m_bufferSize;
 };
 
@@ -802,7 +839,7 @@ class OpalTransport : public PSafeObject
 
         The default behaviour returns the local address via GetLocalAddress()
       */
-    virtual PString GetInterface() const;
+    virtual PString GetInterface() const = 0;
 
     /**Bind this transport to an interface.
         This is generally only relevant for datagram based transports such as
@@ -816,17 +853,7 @@ class OpalTransport : public PSafeObject
 
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress(
-      bool allowNAT = true   ///< Allow translation if remote needs NAT
-    ) const = 0;
-
-    /**Set local address to connect from.
-       Note that this may not work for all transport types or may work only
-       before Connect() has been called.
-      */
-    virtual PBoolean SetLocalAddress(
-      const OpalTransportAddress & address
-    ) = 0;
+    virtual OpalTransportAddress GetLocalAddress() const = 0;
 
     /**Get the transport address of the remote endpoint.
       */
@@ -954,6 +981,9 @@ class OpalTransport : public PSafeObject
       const PBYTEArray & data
     );
 
+    /// Indicate keep alive is active
+    bool HasKeepAlive() const { return !m_keepAliveData.IsEmpty() && m_keepAliveTimer.IsRunning(); }
+
     /**Attach a thread to the transport.
       */
     virtual void AttachThread(
@@ -965,41 +995,40 @@ class OpalTransport : public PSafeObject
     virtual PBoolean IsRunning() const;
   //@}
 
-    OpalEndPoint & GetEndPoint() const { return endpoint; }
+    OpalEndPoint & GetEndPoint() const { return m_endpoint; }
 
     /**Get the prefix for this transports protocol type.
       */
     virtual const PCaselessString & GetProtoPrefix() const = 0;
 
-    bool IsOpen() const
-      { return m_channel != NULL && m_channel->IsOpen(); }
-
-    bool IsGood() const
-      { return IsOpen() && !m_channel->bad() && !m_channel->eof(); }
-
-    PChannel::Errors GetErrorCode(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const
-      { return m_channel != NULL ? m_channel->GetErrorCode(group) : PChannel::NotOpen; }
-
-    PString GetErrorText(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const
-      { return m_channel != NULL ? m_channel->GetErrorText(group) : PString::Empty(); }
-
-    int GetErrorNumber(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const
-      { return m_channel != NULL ? m_channel->GetErrorNumber(group) : -1; }
-
-    void SetReadTimeout(const PTimeInterval & t)
-      { if (m_channel != NULL) m_channel->SetReadTimeout(t); }
+    bool IsOpen() const;
+    bool IsGood() const;
+    PChannel::Errors GetErrorCode(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const;
+    PString GetErrorText(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const;
+    int GetErrorNumber(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const;
+    void SetReadTimeout(const PTimeInterval & t);
 
     PChannel * GetChannel() const { return m_channel; }
-    void SetChannel(PChannel * chan) { m_channel = chan; }
+
+    bool IsIdle() const { return m_referenceCount == 0 && m_idleTimer.HasExpired(); }
+
+    void Reference()   { ++m_referenceCount; }
+    void Dereference() { --m_referenceCount; }
 
   protected:
     PDECLARE_NOTIFIER(PTimer, OpalTransport, KeepAlive);
 
-    OpalEndPoint & endpoint;
+    OpalEndPoint & m_endpoint;
     PChannel     * m_channel;
     PThread      * m_thread;      ///<  Thread handling the transport
     PTimer         m_keepAliveTimer;
     PBYTEArray     m_keepAliveData;
+    PSimpleTimer   m_idleTimer;
+    atomic<unsigned> m_referenceCount;
+
+  private:
+    OpalTransport(const OpalTransport & other) : PSafeObject(other), m_endpoint(other.m_endpoint), m_referenceCount(0) { }
+    void operator=(const OpalTransport &) { }
 };
 
 
@@ -1022,19 +1051,13 @@ class OpalTransportIP : public OpalTransport
   public:
   /**@name Operations */
   //@{
+    /** Get the interface this transport is bound to.
+      */
+    virtual PString GetInterface() const;
+
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress(
-      bool allowNAT = true   ///< Allow translation if remote needs NAT
-    ) const;
-
-    /**Set local address to connect from.
-       Note that this may not work for all transport types or may work only
-       before Connect() has been called.
-      */
-    virtual PBoolean SetLocalAddress(
-      const OpalTransportAddress & address
-    );
+    virtual OpalTransportAddress GetLocalAddress() const;
 
     /**Get the transport dependent name of the remote endpoint.
       */
@@ -1056,10 +1079,9 @@ class OpalTransportIP : public OpalTransport
       */
     virtual const PCaselessString & GetProtoPrefix() const = 0;
 
-    PIPSocket::Address localAddress;  // Address of the local interface
-    WORD               localPort;
-    PIPSocket::Address remoteAddress; // Address of the remote host
-    WORD               remotePort;
+    PIPAddress              m_binding;
+    PIPSocketAddressAndPort m_localAP;  // Address of the local (NAT adjusted)
+    PIPSocketAddressAndPort m_remoteAP; // Address of the remote host
 };
 
 
@@ -1075,7 +1097,7 @@ class OpalTransportTCP : public OpalTransportIP
       OpalEndPoint & endpoint,    ///<  Endpoint object
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
       WORD port = 0,              ///<  Local port to bind to
-      PBoolean reuseAddr = false      ///<  Flag for binding to already bound interface
+      bool dummy = false          ///<  Place holder for template reuseAddr param
     );
     OpalTransportTCP(
       OpalEndPoint & endpoint,    ///<  Endpoint object
@@ -1125,6 +1147,24 @@ class OpalTransportTCP : public OpalTransportIP
     );
   //@}
 
+    /** Set PDU length format.
+        Zero indicates TPKT format (default)
+        Negative number is number of little endian bytes.
+        Positive number is number of big endian bytes.
+
+        off is added to the length after the above is read.
+
+        Examples:
+        16 bit big endian length header which include the length itself, would be:
+           2, -2 -> 00 05 AA BB CC
+        32 bit little endian length header would be:
+           -4, 0 -> 03 00 00 00 AA BB CC
+      */
+    void SetPDULengthFormat(int fmt,int off)
+    {
+      m_pduLengthFormat = fmt;
+      m_pduLengthOffset = off;
+    }
 
   protected:
     /**Get the prefix for this transports protocol type.
@@ -1133,7 +1173,8 @@ class OpalTransportTCP : public OpalTransportIP
 
     bool OnConnectedSocket(PTCPSocket * socket);
 
-    PBoolean reuseAddressFlag;
+    int  m_pduLengthFormat;
+    int  m_pduLengthOffset;
 };
 
 
@@ -1210,17 +1251,7 @@ class OpalTransportUDP : public OpalTransportIP
 
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress(
-      bool allowNAT = true   ///< Allow translation if remote needs NAT
-    ) const;
-
-    /**Set local address to connect from.
-       Note that this may not work for all transport types or may work only
-       before Connect() has been called.
-      */
-    virtual PBoolean SetLocalAddress(
-      const OpalTransportAddress & address
-    );
+    virtual OpalTransportAddress GetLocalAddress() const;
 
     /**Set remote address to connect to.
        Note that this does not necessarily initiate a transport level
@@ -1306,7 +1337,7 @@ class OpalTransportUDP : public OpalTransportIP
       */
     virtual const PCaselessString & GetProtoPrefix() const;
 
-    OpalManager & manager;
+    OpalManager & m_manager;
     PINDEX        m_bufferSize;
     PBYTEArray    m_preReadPacket;
     bool          m_preReadOK;
@@ -1374,8 +1405,7 @@ class OpalInternalIPTransport : public OpalInternalTransport
     static PBoolean GetAdjustedIpAndPort(const OpalTransportAddress & address,
                                      OpalEndPoint & endpoint,
                                      OpalTransportAddress::BindOptions option,
-                                     PIPSocket::Address & ip,
-                                     WORD & port,
+                                     PIPSocketAddressAndPort & ap,
                                      PBoolean & reuseAddr);
 };
 
@@ -1398,14 +1428,13 @@ class OpalInternalIPTransportTemplate : public OpalInternalIPTransport
       OpalTransportAddress::BindOptions options
     ) const
     {
-      PIPSocket::Address ip;
-      WORD port;
+      PIPSocketAddressAndPort ap;
       PBoolean reuseAddr;
-      if (GetAdjustedIpAndPort(address, endpoint, options, ip, port, reuseAddr)) {
+      if (GetAdjustedIpAndPort(address, endpoint, options, ap, reuseAddr)) {
         if (options == AltTypeOption)
-          return new AltTypeClass(endpoint, ip, 0, reuseAddr);
+          return new AltTypeClass(endpoint, ap.GetAddress(), 0, reuseAddr);
         else
-          return new TransportType(endpoint, ip, 0, reuseAddr);
+          return new TransportType(endpoint, ap.GetAddress(), 0, reuseAddr);
       }
       return NULL;
     }
@@ -1414,6 +1443,10 @@ class OpalInternalIPTransportTemplate : public OpalInternalIPTransport
 typedef OpalInternalIPTransportTemplate<OpalListenerTCP, OpalTransportTCP, OpalTransportAddress::Datagram, OpalTransportUDP> OpalInternalTCPTransport;
 typedef OpalInternalIPTransportTemplate<OpalListenerUDP, OpalTransportUDP, OpalTransportAddress::Streamed, OpalTransportTCP> OpalInternalUDPTransport;
 
+
+////////////////////////////////////////////////////////////////
+
+
 #if OPAL_PTLIB_SSL
 
 class PSSLContext;
@@ -1440,7 +1473,6 @@ class OpalListenerTLS : public OpalListenerTCP
     ~OpalListenerTLS();
 
     virtual PBoolean Open(const AcceptHandler & acceptHandler, ThreadMode mode = SpawnNewThreadMode);
-    virtual OpalTransport * Accept(const PTimeInterval & timeout);
     virtual const PCaselessString & GetProtoPrefix() const;
     virtual OpalTransport * CreateTransport(
       const OpalTransportAddress & localAddress,
@@ -1448,6 +1480,8 @@ class OpalListenerTLS : public OpalListenerTCP
     ) const;
 
   protected:
+    virtual OpalTransport * OnAccept(PTCPSocket * socket);
+
     PSSLContext * m_sslContext;
 };
 
@@ -1458,14 +1492,14 @@ class OpalTransportTLS : public OpalTransportTCP
     public:
       OpalTransportTLS(
         OpalEndPoint & endpoint,    ///<  Endpoint object
-        PSSLChannel * ssl
+        PChannel * ssl
       );
 
       OpalTransportTLS(
         OpalEndPoint & endpoint,    ///<  Endpoint object
         PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
         WORD port = 0,              ///<  Local port to bind to
-        PBoolean reuseAddr = false      ///<  Flag for binding to already bound interface
+        bool dummy = false          ///<  Place holder for template reuseAddr param
       );
 
       /// Destroy the TCPS channel
@@ -1478,12 +1512,191 @@ class OpalTransportTLS : public OpalTransportTCP
       virtual bool IsAuthenticated(const PString & domain) const;
 };
 
+
 typedef OpalInternalIPTransportTemplate<OpalListenerTLS, OpalTransportTLS, OpalTransportAddress::Datagram, OpalTransportUDP> OpalInternalTLSTransport;
 
 typedef OpalTransportTLS OpalTransportTCPS; // For backward compatibility
 typedef OpalListenerTLS OpalListenerTCPS;
 
 
+////////////////////////////////////////////////////////////////
+
+#if OPAL_PTLIB_HTTP
+
+class OpalListenerWS : public OpalListenerTCP
+{
+  PCLASSINFO(OpalListenerWS, OpalListenerTCP);
+public:
+  /**@name Construction */
+  //@{
+  /**Create a new listener.
+  */
+  OpalListenerWS(
+    OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
+    PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
+    WORD port = 0,                           ///<  TCP port to listen for connections
+    bool exclusive = true                    ///< Exclusive listening mode, no other process can accept on the port
+    );
+  OpalListenerWS(
+    OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
+    const OpalTransportAddress & binding,     ///<  Local interface to listen on
+    OpalTransportAddress::BindOptions option  ///< OPtions for binding
+    );
+
+  /**Create a transport compatible with this listener.
+  */
+  virtual OpalTransport * CreateTransport(
+    const OpalTransportAddress & localAddress,
+    const OpalTransportAddress & remoteAddress
+    ) const;
+
+protected:
+  virtual const PCaselessString & GetProtoPrefix() const;
+  virtual OpalTransport * OnAccept(PTCPSocket * socket);
+};
+
+
+class OpalListenerWSS : public OpalListenerTLS
+{
+  PCLASSINFO(OpalListenerWSS, OpalListenerTLS);
+public:
+  OpalListenerWSS(
+    OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
+    PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
+    WORD port = 0,                           ///<  TCP port to listen for connections
+    PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
+    );
+  OpalListenerWSS(
+    OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
+    const OpalTransportAddress & binding,     ///<  Local interface to listen on
+    OpalTransportAddress::BindOptions option  ///< OPtions for binding
+    );
+
+  virtual const PCaselessString & GetProtoPrefix() const;
+
+protected:
+  virtual OpalTransport * OnAccept(PTCPSocket * socket);
+};
+
+
+class OpalTransportWS : public OpalTransportTCP
+{
+  PCLASSINFO(OpalTransportWS, OpalTransportTCP);
+public:
+  OpalTransportWS(
+    OpalEndPoint & endpoint,    ///<  Endpoint object
+    PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
+    WORD port = 0,              ///<  Local port to bind to
+    bool dummy = false          ///<  Place holder for template reuseAddr param
+    );
+  OpalTransportWS(
+    OpalEndPoint & endpoint,    ///<  Endpoint object
+    PChannel * socket           ///<  Socket to use
+    );
+
+  /**Connect to the remote address.
+  */
+  virtual PBoolean Connect();
+
+  /** Read a packet from the transport.
+  This will read using the transports mechanism for PDU boundaries, for
+  example UDP is a single Read() call, while for TCP there is a TPKT
+  header that indicates the size of the PDU.
+
+  If false is returned but there is data returned in the \p packet
+  that indicates that the available buffer space was too small, e.g. an
+  EMSGSIZE error was returned by recvfrom.
+  */
+  virtual PBoolean ReadPDU(
+    PBYTEArray & pdu  ///<  PDU read from transport
+    );
+
+  /**Write a packet to the transport.
+  This will write using the transports mechanism for PDU boundaries, for
+  example UDP is a single Write() call, while for TCP there is a TPKT
+  header that indicates the size of the PDU.
+  */
+  virtual PBoolean WritePDU(
+    const PBYTEArray & pdu     ///<  Packet to write
+    );
+
+  virtual PBoolean IsCompatibleTransport(const OpalTransportAddress & address) const;
+  virtual const PCaselessString & GetProtoPrefix() const;
+};
+
+
+class OpalTransportWSS : public OpalTransportTLS
+{
+  PCLASSINFO(OpalTransportWSS, OpalTransportTLS);
+public:
+  OpalTransportWSS(
+    OpalEndPoint & endpoint,    ///<  Endpoint object
+    PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
+    WORD port = 0,              ///<  Local port to bind to
+    bool dummy = false          ///<  Place holder for template reuseAddr param
+    );
+  OpalTransportWSS(
+    OpalEndPoint & endpoint,    ///<  Endpoint object
+    PChannel * socket           ///<  Socket to use
+    );
+
+  /**Connect to the remote address.
+  */
+  virtual PBoolean Connect();
+
+  /** Read a packet from the transport.
+  This will read using the transports mechanism for PDU boundaries, for
+  example UDP is a single Read() call, while for TCP there is a TPKT
+  header that indicates the size of the PDU.
+
+  If false is returned but there is data returned in the \p packet
+  that indicates that the available buffer space was too small, e.g. an
+  EMSGSIZE error was returned by recvfrom.
+  */
+  virtual PBoolean ReadPDU(
+    PBYTEArray & pdu  ///<  PDU read from transport
+    );
+
+  /**Write a packet to the transport.
+  This will write using the transports mechanism for PDU boundaries, for
+  example UDP is a single Write() call, while for TCP there is a TPKT
+  header that indicates the size of the PDU.
+  */
+  virtual PBoolean WritePDU(
+    const PBYTEArray & pdu     ///<  Packet to write
+    );
+
+  virtual PBoolean IsCompatibleTransport(const OpalTransportAddress & address) const;
+  virtual const PCaselessString & GetProtoPrefix() const;
+};
+
+
+typedef OpalInternalIPTransportTemplate<OpalListenerWS, OpalTransportWS, OpalTransportAddress::Datagram, OpalTransportUDP> OpalInternalWSTransport;
+typedef OpalInternalIPTransportTemplate<OpalListenerWSS, OpalTransportWSS, OpalTransportAddress::Datagram, OpalTransportUDP> OpalInternalWSSTransport;
+
+
+class OpalHTTPConnector : public PHTTPResource
+{
+    PCLASSINFO(OpalHTTPConnector, PHTTPResource)
+  public:
+    OpalHTTPConnector(
+      OpalManager & manager,         ///< Opal manager
+      const PURL & url               ///< Name of the resource in URL space.
+    );
+    OpalHTTPConnector(
+      OpalManager & manager,         ///< Opal manager
+      const PURL & url,              ///< Name of the resource in URL space.
+      const PHTTPAuthority & auth    ///< Authorisation for the resource.
+    );
+
+    virtual bool OnWebSocket(PHTTPServer & server, PHTTPConnectionInfo & connectInfo);
+
+  protected:
+    OpalManager & m_manager;
+};
+
+
+#endif // OPAL_PTLIB_HTTP
 #endif // OPAL_PTLIB_SSL
 
 
