@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29641 $
+ * $Revision: 34723 $
  * $Author: rjongbloed $
- * $Date: 2013-05-03 07:14:54 -0500 (Fri, 03 May 2013) $
+ * $Date: 2016-03-28 15:36:44 +0100 (Mon, 28 Mar 2016) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -36,13 +36,14 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_SIP
 
 #include <rtp/rtpconn.h>
 #include <sip/sippdu.h>
 #include <sip/handlers.h>
+#include <sdp/sdpep.h>
 
 
 class OpalSIPIMContext;
@@ -59,6 +60,11 @@ class OpalSIPIMContext;
   */
 #define OPAL_OPT_REFER_SUB       "Refer-Sub"
 
+/**OpalConnection::StringOption key to a boolean indicating whether to not 
+use RFC4488 ("Refer-Sub" header) at all, in the REFER request. Default false.
+*/
+#define OPAL_OPT_NO_REFER_SUB    "No-Refer-Sub"
+
 /**OpalConnection::StringOption key to an integer indicating the the mode
    for the reliable provisional response system. See PRACKMode for more
    information. Default is from SIPEndPoint::GetDefaultPRACKMode() which
@@ -71,23 +77,6 @@ class OpalSIPIMContext;
   */
 #define OPAL_OPT_INITIAL_OFFER "Initial-Offer"
 
-/**OpalConnection::StringOption key to a string for a regular expression to
-   match the product information, which if matching the remote system, will
-   indicate the remote does not support asymmetric hold as required by the
-   standard.
-   
-   This fault is when SDP sendonly is sent (us putting them on hold), and
-   they reply inactive, which implies them putting us on hold. When we
-   subsequently send recvonly to release our hold to them, they continue to
-   send inactive, and hold is never released.
-
-   Note the OpalProductInfo vendor, name & version strings are concatenated
-   before comparison with the regular expression.
-
-   Defaults to empty string.
-  */
-#define OPAL_OPT_SYMMETRIC_HOLD_PRODUCT "Symmetric-Hold-Product"
-
 /**Allow INVITE with Replaces to operate on "early" dialog.
    While RFC 3891 explicitly forbids and incoming INVITE with Replaces to
    replace a call which was incoming to the UA. It is allowed for a dialog
@@ -112,6 +101,16 @@ class OpalSIPIMContext;
   */
 #define OPAL_OPT_EXTERNAL_SDP "External-SDP"
 
+/**Enable SRTP when not using secure signalling (sips)
+   Normally SRTP and SDES only applies to secure signalling, sips or
+   transport=tls, however this flag will allow it on unsecure connections
+   which means the key exchange can be observed.
+
+   Defaults to false.
+*/
+#define OPAL_OPT_UNSECURE_SRTP "Unsecure-SRTP"
+
+
 #define SIP_HEADER_PREFIX      "SIP-Header:"
 #define SIP_HEADER_REPLACES    SIP_HEADER_PREFIX"Replaces"
 #define SIP_HEADER_REFERRED_BY SIP_HEADER_PREFIX"Referred-By"
@@ -125,9 +124,9 @@ class OpalSIPIMContext;
 /**Session Initiation Protocol connection.
  */
 
-class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
+class SIPConnection : public OpalSDPConnection, public SIPTransactionOwner
 {
-    PCLASSINFO(SIPConnection, OpalRTPConnection);
+    PCLASSINFO(SIPConnection, OpalSDPConnection);
   public:
 
   /**@name Construction */
@@ -165,16 +164,6 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
 
   /**@name Overrides from OpalConnection */
   //@{
-    /**Get indication of connection being to a "network".
-       This indicates the if the connection may be regarded as a "network"
-       connection. The distinction is about if there is a concept of a "remote"
-       party being connected to and is best described by example: sip, h323,
-       iax and pstn are all "network" connections as they connect to something
-       "remote". While pc, pots and ivr are not as the entity being connected
-       to is intrinsically local.
-      */
-    virtual bool IsNetworkConnection() const { return true; }
-
     /**Get this connections protocol prefix for URLs.
       */
     virtual PString GetPrefixName() const;
@@ -198,6 +187,13 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
       */
     virtual OpalTransportAddress GetRemoteAddress() const { return m_remoteAddress; }
 
+    /** Get the remote identity.
+        Under some circumstances the "identity" of the remote party, may be
+        different from the name, number or URL for that user. For example, this
+        would be the P-Asserted-Identity field in SIP.
+      */
+    virtual PString GetRemoteIdentity() const;
+
     /**Get the destination address of an incoming connection.
        This will, for example, collect a phone number from a POTS line, or
        get the fields from the H.225 SETUP pdu in a H.323 connection.
@@ -215,6 +211,14 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
       */
     virtual PString GetCalledPartyURL();
 
+    /**Get any extra call information.
+      */
+    virtual PMultiPartList GetExtraCallInfo() const { return m_multiPartMIME; }
+
+    /**Get the local name/alias.
+      */
+    virtual PString GetLocalPartyURL() const;
+
     /**Get alerting type information of an incoming call.
        The type of "distinctive ringing" for the call. The string is protocol
        dependent, so the caller would need to be aware of the type of call
@@ -278,26 +282,6 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
       const PString & remoteParty   ///<  Remote party to transfer the existing call to
     );
 
-    /**Put the current connection on hold, suspending all media streams.
-       The \p fromRemote parameter indicates if we a putting the remote on
-       hold (false) or it is a request for the remote to put us on hold (true).
-
-       The /p placeOnHold parameter indicates of teh command/request is for
-       going on hold or retrieving from hold.
-     */
-    virtual bool Hold(
-      bool fromRemote,  ///< Flag for if remote has us on hold, or we have them
-      bool placeOnHold  ///< Flag for setting on or off hold
-    );
-
-    /**Return true if the current connection is on hold.
-       The \p fromRemote parameter indicates if we are testing if the remote
-       system has us on hold, or we have them on hold.
-     */
-    virtual bool IsOnHold(
-      bool fromRemote  ///< Flag for if remote has us on hold, or we have them
-    );
-
     /**Indicate to remote endpoint an alert is in progress.
        If this is an incoming connection and the AnswerCallResponse is in a
        AnswerCallDeferred or AnswerCallPending state, then this function is
@@ -322,7 +306,7 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
     /**Get the data formats this endpoint is capable of operating in.
       */
     virtual OpalMediaFormatList GetMediaFormats() const;
-    
+
     /**Indicate connection requires symmetric media.
        Default behaviour returns false.
       */
@@ -336,6 +320,10 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
     );
 #endif
 
+    /**Indicate security modes available in media negotiation.
+    */
+    virtual OpalMediaCryptoSuite::KeyExchangeModes GetMediaCryptoKeyExchangeModes() const;
+
     /**Create a new media stream.
      */
     virtual OpalMediaStream * CreateMediaStream(
@@ -370,6 +358,8 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
                on to a remote system for direct access.
      */
     virtual bool GetMediaTransportAddresses(
+      OpalConnection & otherConnection,      ///< Other half of call needing media transport addresses
+      unsigned sessionId,                    ///< Session identifier
       const OpalMediaType & mediaType,       ///< Media type for session to return information
       OpalTransportAddressArray & transports ///<  Information on media session
     ) const;
@@ -671,69 +661,43 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
     virtual void OnReceivedMESSAGE(SIP_PDU & pdu);
     virtual void OnReceivedSUBSCRIBE(SIP_PDU & pdu);
 
-    PString GetLocalPartyURL() const;
+    virtual PString GetMediaInterface();
+    virtual OpalTransportAddress GetRemoteMediaAddress();
 
-    virtual void PrintOn(ostream & strm) const { OpalRTPConnection::PrintOn(strm); }
 
   protected:
     virtual bool GarbageCollection();
-    typedef SIPPoolTimer<SIPConnection> PoolTimer;
     void OnSessionTimeout();
     void OnInviteResponseRetry();
     void OnInviteResponseTimeout();
     void OnInviteCollision();
+    void OnDelayedRefer();
+    virtual bool OnHoldStateChanged(bool placeOnHold);
+    virtual void OnMediaStreamOpenFailed(bool rx);
 
-    virtual bool OnSendOfferSDP(
-      SDPSessionDescription & sdpOut,
-      bool offerCurrentOnly
+    void OnReceivedAlertingResponse(
+      SIPTransaction & transaction,
+      SIP_PDU & response
     );
-    virtual bool OnSendOfferSDPSession(
-      unsigned sessionID,
-      SDPSessionDescription & sdpOut,
-      bool offerOpenMediaStreamOnly
+    bool InternalSetConnected(bool transfer);
+    virtual bool OnSendAnswer(
+      SIP_PDU::StatusCodes response,
+      bool transfer
     );
-
-    virtual bool OnSendAnswerSDP(
-      SDPSessionDescription & sdpOut
-    );
-    virtual bool OnSendAnswerSDP(
-      const SDPSessionDescription & sdpOffer,
-      SDPSessionDescription & sdpAnswer
-    );
-    virtual bool OnSendAnswerSDPSession(
-      const SDPSessionDescription & sdpIn,
-      unsigned sessionIndex,
-      SDPSessionDescription & sdpOut
-    );
-
-    virtual void OnReceivedAnswerSDP(
+    virtual bool OnReceivedAnswer(
       SIP_PDU & response,
       SIPTransaction * transaction
     );
-    virtual bool OnReceivedAnswerSDPSession(
-      SDPSessionDescription & sdp,
-      unsigned sessionId,
-      bool & multipleFormats
-    );
 
-    virtual OpalMediaSession * SetUpMediaSession(
-      const unsigned rtpSessionId,
-      const OpalMediaType & mediaType,
-      const SDPMediaDescription & mediaDescription,
-      OpalTransportAddress & localAddress,
-      bool & remoteChanged
-    );
-
-    bool SendReINVITE(PTRACE_PARAM(const char * msg));
+    bool SendReINVITE(PTRACE_PARAM(const char * msg, ) int operation = 0);
     bool StartPendingReINVITE();
 
     friend class SIPInvite;
     PDECLARE_WriteConnectCallback(SIPConnection, WriteINVITE);
 
-    virtual void SendDelayedACK(bool force);
+    virtual bool SendDelayedACK(bool force);
     void OnDelayedAckTimeout();
 
-    virtual bool SendInviteOK();
     virtual PBoolean SendInviteResponse(
       SIP_PDU::StatusCodes code,
       const SDPSessionDescription * sdp = NULL
@@ -743,9 +707,6 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
     );
 
     void UpdateRemoteAddresses();
-#if OPAL_SRTP
-    virtual bool CanDoSRTP() const { return m_dialog.GetRemoteTransportAddress(m_dnsEntry).GetProtoPrefix() == OpalTransportAddress::TlsPrefix(); }
-#endif
 
     void NotifyDialogState(
       SIPDialogNotification::States state,
@@ -754,33 +715,29 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
     );
 
     virtual bool InviteConferenceParticipant(const PString & conf, const PString & dest);
+    PSafePtr<SIPConnection> GetB2BUA();
+
+
+    typedef SIPPoolTimer<SIPConnection> PoolTimer;
 
     // Member variables
     unsigned              m_allowedMethods;
     PStringSet            m_allowedEvents;
 
-    enum HoldState {
-      eHoldOff,
-      eRetrieveInProgress,
-
-      // Order is important!
-      eHoldOn,
-      eHoldInProgress
-    };
-    HoldState             m_holdToRemote;
-    bool                  m_holdFromRemote;
     PString               m_forwardParty;
     OpalTransportAddress  m_remoteAddress;
+    SIPURL                m_assertedIdentity;
+    SIPURL                m_remoteIdentity;
     SIPURL                m_contactAddress;
     SIPURL                m_ciscoRemotePartyID;
+    PMultiPartList        m_multiPartMIME;
 
     SIP_PDU             * m_lastReceivedINVITE;
     SIP_PDU             * m_delayedAckInviteResponse;
     PoolTimer             m_delayedAckTimer;
-    PTimeInterval         m_delayedAckTimeout;
-    SIP_PDU             * m_lastSentAck;
-    time_t                m_sdpSessionId;
-    unsigned              m_sdpVersion; // Really a sequence number
+    PTimeInterval         m_delayedAckTimeout1;
+    PTimeInterval         m_delayedAckTimeout2;
+    SIP_PDU             * m_delayedAckPDU;
     bool                  m_needReINVITE;
     bool                  m_handlingINVITE;
     bool                  m_resolveMultipleFormatReINVITE;
@@ -788,20 +745,23 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
     OpalGloballyUniqueID  m_dialogNotifyId;
     int                   m_appearanceCode;
     PString               m_alertInfo;
+#if OPAL_VIDEO
+    bool                  m_canDoVideoFastUpdateINFO;
+#endif
     PoolTimer             m_sessionTimer;
+    PRACKMode             m_prackMode;
+    bool                  m_prackEnabled;
+    unsigned              m_prackSequenceNumber;
+    std::queue<SIP_PDU>   m_responsePackets;
+    PoolTimer             m_responseFailTimer;
+    PoolTimer             m_responseRetryTimer;
+    unsigned              m_responseRetryCount;
+    PoolTimer             m_inviteCollisionTimer;
+    bool                  m_referOfRemoteInProgress;
+    PoolTimer             m_delayedReferTimer;
+    SIPURL                m_delayedReferTo;
+    SIPURL                m_sentReferTo;
 
-    std::map<SIP_PDU::Methods, unsigned> m_lastRxCSeq;
-
-    PRACKMode      m_prackMode;
-    bool           m_prackEnabled;
-    unsigned       m_prackSequenceNumber;
-    std::queue<SIP_PDU> m_responsePackets;
-    PoolTimer      m_responseFailTimer;
-    PoolTimer      m_responseRetryTimer;
-    unsigned       m_responseRetryCount;
-    PoolTimer      m_inviteCollisionTimer;
-
-    bool                      m_referInProgress;
     PSafeList<SIPTransaction> m_forkedInvitations; // Not for re-INVITE
     PSafeList<SIPTransaction> m_pendingInvitations; // For re-INVITE
 
@@ -812,9 +772,7 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
       ReleaseWithNothing,
     } releaseMethod;
 
-    OpalMediaFormatList m_remoteFormatList;
-    OpalMediaFormatList m_answerFormatList;
-    bool SetRemoteMediaFormats(SIP_PDU * pdu);
+    int SetRemoteMediaFormats(SIP_PDU & pdu);
 
     std::map<std::string, SIP_PDU *> m_responses;
 
@@ -822,10 +780,6 @@ class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
     PSafePtr<OpalSIPIMContext> m_imContext;
 #endif
 
-#if OPAL_VIDEO
-    PSimpleTimer m_infoPictureFastUpdateTimer;
-#endif
-
     enum {
       UserInputMethodUnknown,
       ReceivedRFC2833,
