@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29947 $
+ * $Revision: 33984 $
  * $Author: rjongbloed $
- * $Date: 2013-06-13 05:59:17 -0500 (Thu, 13 Jun 2013) $
+ * $Date: 2015-09-07 11:59:59 +0100 (Mon, 07 Sep 2015) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -40,11 +40,11 @@
 #include <ptlib.h>
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_SIP
 
-#include <rtp/rtpep.h>
+#include <sdp/sdpep.h>
 #include <sip/sipcon.h>
 #include <sip/handlers.h> 
 
@@ -53,9 +53,9 @@
 
 /**Session Initiation Protocol endpoint.
  */
-class SIPEndPoint : public OpalRTPEndPoint
+class SIPEndPoint : public OpalSDPEndPoint
 {
-    PCLASSINFO(SIPEndPoint, OpalRTPEndPoint);
+    PCLASSINFO(SIPEndPoint, OpalSDPEndPoint);
   public:
   /**@name Construction */
   //@{
@@ -180,6 +180,10 @@ class SIPEndPoint : public OpalRTPEndPoint
         Default behaviour deletes the objects in the connectionsActive list.
       */
     virtual PBoolean GarbageCollection();
+
+    /** Get available string option names.
+      */
+    virtual PStringList GetAvailableStringOptions() const;
   //@}
 
   /**@name Customisation call backs */
@@ -191,7 +195,7 @@ class SIPEndPoint : public OpalRTPEndPoint
       const SIPConnection::Init & init     ///< Initialisation parameters
 
     );
-    
+
     /**Setup a connection transfer a connection for the SIP endpoint.
       */
     virtual PBoolean SetupTransfer(
@@ -244,17 +248,12 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     /**Handle an incoming SIP PDU that has been full decoded
+       @return true if ownership of \p pdu is taken, false will delete it.
       */
     virtual bool OnReceivedPDU(
       SIP_PDU * pdu
     );
 
-    /**Handle an incoming SIP PDU not assigned to any connection
-      */
-    virtual bool OnReceivedConnectionlessPDU(
-      SIP_PDU * pdu
-    );
-
     /**Handle an incoming response PDU.
       */
     virtual void OnReceivedResponse(
@@ -263,24 +262,28 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     /**Handle an incoming INVITE request.
+       @return true if ownership of \p pdu is taken, false will delete it.
       */
     virtual bool OnReceivedINVITE(
       SIP_PDU * pdu
     );
 
     /**Handle an incoming NOTIFY PDU.
+       @return true if handled and response sent.
       */
     virtual bool OnReceivedNOTIFY(
       SIP_PDU & request
     );
 
     /**Handle an incoming REGISTER PDU.
+       @return true if handled and response sent.
       */
     virtual bool OnReceivedREGISTER(
       SIP_PDU & request
     );
 
     /**Handle an incoming SUBSCRIBE PDU.
+       @return true if handled and response sent.
       */
     virtual bool OnReceivedSUBSCRIBE(
       SIP_PDU & request,
@@ -288,12 +291,14 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     /**Handle an incoming MESSAGE PDU.
+       @return true if handled and response sent.
       */
     virtual bool OnReceivedMESSAGE(
       SIP_PDU & request
     );
 
     /**Handle an incoming OPTIONS PDU.
+       @return true if handled and response sent.
       */
     virtual bool OnReceivedOPTIONS(
       SIP_PDU & request
@@ -438,6 +443,7 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU::StatusCodes m_reason;            ///< Reason for status change
       OpalProductInfo      m_productInfo;       ///< Server product info from registrar if available.
       void               * m_userData;          ///< User data corresponding to this registration
+      friend ostream & operator<<(ostream & strm, const RegistrationStatus & status);
     };
 
     /** Get current registration status.
@@ -787,6 +793,46 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PURL & to
     );
 
+    // Registrar
+    class RegistrarAoR : public PSafeObject
+    {
+      PCLASSINFO(RegistrarAoR, PSafeObject)
+      public:
+        RegistrarAoR(const PURL & aor);
+
+        virtual Comparison Compare(const PObject & obj) const;
+        virtual void PrintOn(ostream & strm) const;
+
+        virtual SIP_PDU::StatusCodes OnReceivedREGISTER(SIPEndPoint & endpoint, const SIP_PDU & request);
+        virtual bool ExpireBindings();
+
+        const PURL & GetAoR() const { return m_aor; }
+        SIPURLList GetContacts() const;
+
+        bool HasBindings() const { return !m_bindings.empty(); }
+
+      protected:
+        PURL m_aor;
+
+        struct Binding {
+          PString m_id;
+          PTime   m_lastUpdate;
+        };
+        typedef std::map<SIPURL, Binding> BindingMap;
+        BindingMap m_bindings;
+
+        std::map<PString, unsigned> m_cseq;
+    };
+    friend class RegistrarAoR;
+
+    virtual RegistrarAoR * CreateRegistrarAoR(const SIP_PDU & request);
+    virtual PSafePtr<RegistrarAoR> FindRegistrarAoR(const SIPURL & aor) { return m_registeredUAs.FindWithLock(RegistrarAoR(aor)); }
+    virtual SIPURLList GetRegistrarAoRs() const;
+    virtual void OnChangedRegistrarAoR(RegistrarAoR & ua);
+
+    void SetRegistrarDomains(const PStringSet & domains) { m_registrarDomains = domains; }
+    const PStringSet & GetRegistrarDomains() const { return m_registrarDomains; }
+
     /** Get the allowed events for SUBSCRIBE commands.
       */
     const PStringSet & GetAllowedEvents() const { return m_allowedEvents; }
@@ -874,7 +920,7 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIPTransaction * transaction
     ) { m_transactions.Append(transaction); }
 
-    PSafePtr<SIPTransaction> GetTransaction(const PString & transactionID, PSafetyMode mode = PSafeReadWrite)
+    PSafePtr<SIPTransaction> GetTransaction(const PString & transactionID, PSafetyMode mode)
     { return PSafePtrCast<SIPTransactionBase, SIPTransaction>(m_transactions.FindWithLock(transactionID, mode)); }
     
     /**Return the next CSEQ for the next transaction.
@@ -978,8 +1024,9 @@ class SIPEndPoint : public OpalRTPEndPoint
 
 
   protected:
-    void AddTransport(const OpalTransportPtr & transport);
+    void AddTransport(const OpalTransportPtr & transport, KeepAliveType keepAliveType);
     void TransportThreadMain(OpalTransportPtr transport);
+    SIP_PDU::StatusCodes InternalHandleREGISTER(SIP_PDU & request, SIP_PDU * response);
 
     SIPURL        m_proxy;
     PString       m_userAgentString;
@@ -998,7 +1045,6 @@ class SIPEndPoint : public OpalRTPEndPoint
     PTimeInterval ackTimeout;
     PTimeInterval registrarTimeToLive;
     PTimeInterval notifierTimeToLive;
-    PTimeInterval natBindingTimeout;
     PTimeInterval m_keepAliveTimeout;
     KeepAliveType m_keepAliveType;
     bool          m_registeredUserMode;
@@ -1012,12 +1058,12 @@ class SIPEndPoint : public OpalRTPEndPoint
     PSafePtr<SIPHandler> FindHandlerByPDU(const SIP_PDU & pdu, PSafetyMode mode);
 
     PStringToString   m_receivedConnectionTokens;
-    PMutex            m_receivedConnectionMutex;
+    PDECLARE_MUTEX(m_receivedConnectionMutex);
 
     PSafeSortedList<SIPTransactionBase> m_transactions;
 
-    PAtomicInteger m_lastSentCSeq;
-    int            m_defaultAppearanceCode;
+    atomic<unsigned> m_lastSentCSeq;
+    int              m_defaultAppearanceCode;
 
     struct RegistrationCompletion {
       PSyncPoint           m_sync;
@@ -1030,12 +1076,19 @@ class SIPEndPoint : public OpalRTPEndPoint
     typedef std::multimap<PString, SIPURL> ConferenceMap;
     ConferenceMap m_conferenceAOR;
 
+    // Registrar
+    typedef PSafeSortedList<RegistrarAoR> RegistrarList;
+    RegistrarList m_registeredUAs;
+    PStringSet    m_registrarDomains;
+
     // Thread pooling
     SIPThreadPool m_threadPool;
 
     // Network interface checking
     PDECLARE_InterfaceNotifier(SIPEndPoint, OnHighPriorityInterfaceChange);
     PDECLARE_InterfaceNotifier(SIPEndPoint, OnLowPriorityInterfaceChange);
+    PInterfaceMonitor::Notifier m_onHighPriorityInterfaceChange;
+    PInterfaceMonitor::Notifier m_onLowPriorityInterfaceChange;
 
     bool m_disableTrying;
 
@@ -1046,6 +1099,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     P_REMOVE_VIRTUAL(SIPConnection *,CreateConnection(OpalCall &, const PString &, void *, const SIPURL &, OpalTransport *, SIP_PDU *, unsigned, OpalConnection::StringOptions *), NULL);
     P_REMOVE_VIRTUAL(PBoolean, OnReceivedPDU(OpalTransport &, SIP_PDU *), false);
     P_REMOVE_VIRTUAL(bool, OnReceivedConnectionlessPDU(OpalTransport &,  SIP_PDU *), false);
+    P_REMOVE_VIRTUAL(bool, OnReceivedConnectionlessPDU(SIP_PDU *), false);
     P_REMOVE_VIRTUAL(PBoolean, OnReceivedINVITE(OpalTransport &, SIP_PDU *), false);
     P_REMOVE_VIRTUAL(PBoolean, OnReceivedNOTIFY(OpalTransport &, SIP_PDU &), false);
     P_REMOVE_VIRTUAL(PBoolean, OnReceivedREGISTER(OpalTransport &, SIP_PDU &), false);
