@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29947 $
+ * $Revision: 34403 $
  * $Author: rjongbloed $
- * $Date: 2013-06-13 05:59:17 -0500 (Thu, 13 Jun 2013) $
+ * $Date: 2015-12-10 09:08:21 +0000 (Thu, 10 Dec 2015) $
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -36,7 +36,7 @@
 #pragma interface
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_SIP
 
@@ -46,6 +46,7 @@
 #include <ptclib/pxml.h>
 #include <ptclib/threadpool.h>
 #include <opal/transports.h>
+#include <im/im.h>
 #include <rtp/rtpconn.h>
 
  
@@ -251,7 +252,11 @@ class SIPURLList : public std::list<SIPURL>
     );
     PString ToString() const;
     SIPURL FindCompatible(const OpalTransportAddress & addr PTRACE_PARAM(, const char * listType)) const;
+
     friend ostream & operator<<(ostream & strm, const SIPURLList & urls);
+
+  protected:
+    void InternalFromString(const PString & str, SIPURL::UsageContext context, bool reversed);
 };
 
 
@@ -303,6 +308,8 @@ class SIPMIMEInfo : public PMIMEInfo
     PCaselessString GetContentType(bool includeParameters = false) const;
     void SetContentType(const PString & v);
 
+    bool GetSDP(const PString & entityBody, PString & sdp, PMultiPartList & parts) const;
+
     PCaselessString GetContentEncoding() const;
     void SetContentEncoding(const PString & v);
 
@@ -332,7 +339,7 @@ class SIPMIMEInfo : public PMIMEInfo
     void SetCallID(const PString & v);
 
     SIPURL GetContact() const;
-    bool GetContacts(SIPURLList & contacts) const;
+    bool GetContacts(SIPURLList & contacts, unsigned defaultExpires) const;
     void SetContact(const PString & v);
 
     PString GetSubject() const;
@@ -613,7 +620,8 @@ class SIP_PDU : public PSafeObject
 
     SIP_PDU(
       Methods method = SIP_PDU::NumMethods,
-      const OpalTransportPtr & transport = NULL
+      const OpalTransportPtr & transport = NULL,
+      const PString & transactionID = PString::Empty()
     );
 
     /** Construct a Response message
@@ -673,7 +681,7 @@ class SIP_PDU : public PSafeObject
 
     /**Write the PDU to the transport.
       */
-    bool Send();
+    virtual bool Send();
 
     /**Write PDU as a response to a request.
     */
@@ -705,10 +713,12 @@ class SIP_PDU : public PSafeObject
           SIPMIMEInfo & GetMIME()            { return m_mime; }
     SDPSessionDescription * GetSDP()         { return m_SDP; }
     void SetSDP(SDPSessionDescription * sdp);
-    bool DecodeSDP(const OpalMediaFormatList & masterList);
+    bool DecodeSDP(SIPConnection & connection, PMultiPartList & parts);
+    bool IsContentSDP(bool emptyOK = false) const;
 
     const PString & GetExternalTransportAddress() const { return m_externalTransportAddress; }
     OpalTransportPtr GetTransport()  const    { return m_transport; }
+    void SetTransport(const OpalTransportPtr & transport);
 
   protected:
     void CalculateVia();
@@ -726,9 +736,9 @@ class SIP_PDU : public PSafeObject
 
     SDPSessionDescription * m_SDP;
 
-    OpalTransportPtr     m_transport;
-    OpalTransportAddress m_viaAddress;
-    OpalTransportAddress m_externalTransportAddress;
+    const OpalTransportPtr m_transport;
+    OpalTransportAddress   m_viaAddress;
+    OpalTransportAddress   m_externalTransportAddress;
 };
 
 
@@ -811,7 +821,7 @@ class SIPDialogContext
     SIPURLList  m_routeSet;
     unsigned    m_lastSentCSeq;
     unsigned    m_lastReceivedCSeq;
-    OpalTransportAddress m_externalTransportAddress;
+    OpalTransportAddress m_fixedTransportAddress;
     bool        m_forking;
     SIPURL      m_proxy;
     PString     m_interface;
@@ -972,7 +982,7 @@ class SIPTransactionOwner
       const OpalTransport::WriteConnectCallback & function
     );
 
-    SIP_PDU::StatusCodes SwitchTransportProto(const char * proto, OpalTransportPtr * transport);
+    SIP_PDU::StatusCodes SwitchTransportProto(const char * proto, SIP_PDU * pdu);
 
     SIP_PDU::StatusCodes HandleAuthentication(const SIP_PDU & response);
 
@@ -982,7 +992,6 @@ class SIPTransactionOwner
     const SIPURL & GetRemoteURI() const { return m_dialog.GetRemoteURI(); }
     const SIPURL & GetProxy() const { return m_dialog.GetProxy(); }
     const PString & GetInterface() const { return m_dialog.GetInterface(); }
-    void ResetInterface() { m_dialog.SetInterface(PString::Empty()); }
     PINDEX GetDNSEntry() const { return m_dnsEntry; }
     SIPAuthentication * GetAuthenticator() const { return m_authentication; }
     SIPDialogContext & GetDialog() { return m_dialog; }
@@ -997,7 +1006,7 @@ class SIPTransactionOwner
     unsigned            m_authenticateErrors;
 
     PSafeList<SIPTransaction> m_transactions;
-    PMutex                    m_forkMutex;
+    PDECLARE_MUTEX(m_forkMutex);
 
   friend class SIPTransaction;
 };
@@ -1008,8 +1017,10 @@ class SIPTransactionBase : public SIP_PDU
     PCLASSINFO(SIPTransactionBase, SIP_PDU);
   protected:
     SIPTransactionBase(
-      Methods method
-    ) : SIP_PDU(method) { }
+      Methods method,
+      const OpalTransportPtr & transport,
+      const PString & transactionID
+    ) : SIP_PDU(method, transport, transactionID) { }
 
   public:
     SIPTransactionBase(
@@ -1042,7 +1053,8 @@ class SIPTransaction : public SIPTransactionBase
       Methods method,
       SIPTransactionOwner * owner,
       OpalTransport * transport,
-      bool deleteOwner = false
+      bool deleteOwner = false,
+      const PString & transactionID = PString::Empty()
     );
   public:
     ~SIPTransaction();
@@ -1137,7 +1149,7 @@ class SIPResponse : public SIPTransaction
 
     virtual SIPTransaction * CreateDuplicate() const;
 
-    bool Resend(SIP_PDU & command);
+    virtual bool Send();
 };
 
 
@@ -1157,6 +1169,7 @@ class SIPInvite : public SIPTransaction
       SIPConnection & connection,
       OpalTransport * transport = NULL
     );
+    ~SIPInvite();
 
     virtual SIPTransaction * CreateDuplicate() const;
 
@@ -1325,6 +1338,7 @@ class SIPSubscribe : public SIPTransaction
       SIP_PDU::StatusCodes  m_reason;            ///< Reason for status change
       OpalProductInfo       m_productInfo;       ///< Server product info from registrar if available.
       void                * m_userData;          ///< User data corresponding to this registration
+      friend ostream & operator<<(ostream & strm, const SubscriptionStatus & status);
     };
 
     struct NotifyCallbackInfo {
@@ -1470,11 +1484,19 @@ class SIPRefer : public SIPTransaction
 {
   PCLASSINFO(SIPRefer, SIPTransaction);
   public:
+    enum ReferSubMode
+    {
+      e_NoSubMode,
+      e_DisableSubMode,
+      e_EnableSubMode,
+    };
+    static ReferSubMode SubModeFromBooleans(bool avoid, bool sub) { return avoid ? e_NoSubMode : (sub ? e_EnableSubMode : e_DisableSubMode); }
+
     SIPRefer(
       SIPConnection & connection,
       const SIPURL & referTo,
       const SIPURL & referred_by,
-      bool referSub
+      ReferSubMode referSubMode
     );
 
     virtual SIPTransaction * CreateDuplicate() const;
@@ -1515,10 +1537,12 @@ class SIPMessage : public SIPTransaction
         m_expire = 3600; // Try to get mesage through for an hour
       }
 
-      PCaselessString             m_contentType;
-      PString                     m_id;
-      PString                     m_body;
-      PAtomicInteger::IntegerType m_messageId;
+      PCaselessString   m_contentType;
+      PString           m_id;
+      PString           m_body;
+#if OPAL_HAS_IM
+      OpalIM::MessageID m_messageId;
+#endif
     };
 
     SIPMessage(
