@@ -24,20 +24,20 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22675 $
+ * $Revision: 31918 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
+ * $Date: 2014-05-20 06:49:03 +0100 (Tue, 20 May 2014) $
  */
 #include "easycodecs.h"
 
-#include "EasyG7231/EasyG7231.h"
+#include "EasyG7231.h"
 
 
 
 /////////////////////////////////////////////////////////////////////////////
 
 //#define Ecodec EasyG7231
-static const char		EasyG7231_63_Desc[] = { "Easy G7231-6.3" };	// text decription  
+static const char		EasyG7231_63_Desc[] = { "Easy G.723.1 6.3kbps" };	// text decription  
 static const char		EasyG7231_63_L16Desc[]= { "L16" };			// source format 
 static const char		EasyG7231_63_MediaFmt[] = { "G.7231-6.3k[e]" };	// destination format  
 static unsigned int		EasyG7231_63_SamplePerSec = 8000;          // samples per second  
@@ -90,7 +90,7 @@ class G7231_63_EasyCodec : public G7231_EasyCodec
 
 
 #pragma data_seg(".G7231_SHARED")				// define the segment
-#pragma comment(lib,"EasyG7231/EasyG7231.lib")
+#pragma comment(lib,"EasyG7231.lib")
 static G7231_63_EasyCodec * m_G7231_63_codec = NULL;	// specify the data and initialise at Instantiation
 #pragma data_seg()
 #pragma comment(linker, "/section:.G7231_SHARED,rws") // Link the segment
\ No newline at end of file
