@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22675 $
+ * $Revision: 31918 $
  * $Author: rjongbloed $
- * $Date: 2009-05-20 04:23:29 +0000 (Wed, 20 May 2009) $
+ * $Date: 2014-05-20 06:49:03 +0100 (Tue, 20 May 2014) $
  */
 #include "easycodecs.h"
 
-#include "EasyG728/EasyG728.h"
+#include "EasyG728.h"
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -68,7 +68,7 @@ class G728_EasyCodec : public EasyCodec
 }; 
 
 #pragma data_seg(".G728_SHARED")				// define the segment
-#pragma comment(lib,"EasyG728/EasyG728.lib")
+#pragma comment(lib,"EasyG728.lib")
 static G728_EasyCodec * m_G728codec = NULL; 	// specify the data and initialise at Instantiation
 #pragma data_seg()
 #pragma comment(linker, "/section:.G728_SHARED,rws") // Link the segment
