@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22675 $
+ * $Revision: 31918 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
+ * $Date: 2014-05-20 06:49:03 +0100 (Tue, 20 May 2014) $
  */
 #include "easycodecs.h"
 
-#include "EasyG729A/EasyG729A.h"
+#include "EasyG729A.h"
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -68,7 +68,7 @@ class G729A_EasyCodec : public EasyCodec
 }; 
 
 #pragma data_seg(".G729A_SHARED")				// define the segment
-#pragma comment(lib,"EasyG729A/EasyG729A.lib")
+#pragma comment(lib,"EasyG729A.lib")
 static G729A_EasyCodec * m_G729Acodec = NULL; 	// specify the data and initialise at Instantiation
 #pragma data_seg()
 #pragma comment(linker, "/section:.G729A_SHARED,rws") // Link the segment
\ No newline at end of file
