@@ -49,7 +49,7 @@
 #include <stdlib.h>
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.h>
@@ -59,7 +59,7 @@
 
 /////////////////////////////////////////////////////////////////////////////
 
-static const char G7222Description[]  = "G.722.2 (AMR-WB, Wideband Adaptive Multirate Codec)";
+static const char G7222Description[]  = "ITU-T G.722.2 (AMR-WB) - Wideband Adaptive Multirate Codec";
 
 PLUGINCODEC_LICENSE(
   "Ted Szoczei, Nimajin Software Consulting",                  // source code author
@@ -115,7 +115,7 @@ const UWord16 AMRWB_core_block_bits[16]= { 132, 177, 253, 285, 317, 365, 397, 46
 const UWord16 AMRWB_efficient_ToC_bits = 6;
 const UWord16 AMRWB_efficient_CMR_bits = 4;
 
-const UWord8 AMRWB_block_size_octet[16]= { 18, 24, 33, 37, 41, 47, 51, 59, 61, 6, 6, 0, 0, 0, 1, 1 };
+const unsigned AMRWB_block_size_octet[16]= { 18, 24, 33, 37, 41, 47, 51, 59, 61, 6, 6, 0, 0, 0, 1, 1 };
 
 // RFC 3267 octet-aligned ToC adds 8 bits to each core speech bits frame and 8 per packet for header
 // RFC 3267 bandwidth-efficient adds 6 bits to each core speech bits frame and 4 per packet for header
@@ -189,7 +189,7 @@ static int MyEncodeAudio(const struct PluginCodec_Definition * codec,
     //PTRACE(2, "Codec\tAMR-WB encoder: Audio data of size " << *fromLen << " did not match expected " << G7222_SAMPLES_PER_FRAME * sizeof(short));
     return 0;
   }
-  if (*toLen < (unsigned) AMRWB_block_size_octet[Context->mode] + 1)
+  if (*toLen < AMRWB_block_size_octet[Context->mode] + 1)
   {
     //PTRACE(2,"Codec\tAMR-WB encoder: Output buffer of size " << *toLen << " too short for mode " << mode);
     return 0;
@@ -363,7 +363,7 @@ static int MyDecodeAudio(const struct PluginCodec_Definition * codec,
       return 0;
 
     case -2:
-      //PTRACE(2, "Codec\tAMR-WB decoder: Packet size " << *fromLen << " did not match expected " << (unsigned)(AMRWB_block_size_octet[FrameType] + 1) << " for frame type " << FrameType);
+      //PTRACE(2, "Codec\tAMR-WB decoder: Packet size " << *fromLen << " did not match expected " << (AMRWB_block_size_octet[FrameType] + 1) << " for frame type " << FrameType);
       return 0;
 
     case -3:
@@ -452,6 +452,7 @@ static struct PluginCodec_Option const OptionOctetAlign =
   G7222_H245_OCTET_ALIGNED            // H.245 generic capability code and bit mask
 };
 
+#if USE_MODE_SET
 static struct PluginCodec_Option const ModeSetG7222 =
 {
   PluginCodec_IntegerOption,          // Option type
@@ -463,6 +464,7 @@ static struct PluginCodec_Option const ModeSetG7222 =
   NULL,                               // FMTP default value
   G7222_H245_MODE_SET                 // H.245 generic capability code and bit mask
 };
+#endif
 
 static struct PluginCodec_Option const ModeChangePeriodG7222 =
 {
@@ -555,7 +557,9 @@ static struct PluginCodec_Option const * const OptionsTable[] = {
   &MediaPacketizationsRFC3267,
 #endif
   &OptionOctetAlign,
-  //&ModeSetG7222,
+#if USE_MODE_SET
+  &ModeSetG7222,
+#endif
   &ModeChangePeriodG7222,
   &ModeChangeNeighbourG7222,
   &CRC_G7222,
