@@ -19,13 +19,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29451 $
+ * $Revision: 33537 $
  * $Author: rjongbloed $
- * $Date: 2013-04-07 21:18:19 -0500 (Sun, 07 Apr 2013) $
+ * $Date: 2015-04-28 04:23:13 +0100 (Tue, 28 Apr 2015) $
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.h>
@@ -44,14 +44,15 @@
    see RFC3551/4.5.2. So some of these values have to be lied about so that
    the OPAL system gets it right.
   */
-#define CLOCK_RATE              8000  // Clock rate is not samples/second in this case!
+#define RTP_CLOCK_RATE          8000  // Clock rate is not samples/second in this case!
 #define BITS_PER_SECOND         64000 // raw bits per second
-#define FRAME_TIME              1000  // Microseconds in a millisecond
-#define SAMPLES_PER_FRAME       32    // Samples in a millisecond
+#define FRAME_TIME_MS           1000  // Microseconds in a millisecond
+#define FRAME_TIME_RTP          8     // Normally, samples in a millisecond, but here it is RTP clock rate cycles
 #define	BYTES_PER_FRAME         8     // Bytes in a millisecond
 #define MAX_FRAMES_PER_PACKET   90    // 90 milliseconds, which means RTP packets smaller than 1500 bytes typical LAN maximum
 #define PREF_FRAMES_PER_PACKET  20    // 20 milliseconds
 
+static const char desc[] = "ITU-T G.722 - 64kbps Wideband Audio Codec";
 static const char L16Desc[]  = "PCM-16-16kHz"; // Cannot use "L16" as usual, force 16kHz PCM
 static const char g722[]     = "G.722-64k";
 static const char sdpG722[]  = "G722";
@@ -165,7 +166,7 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // specified RTP type
 
-    g722_16,                              // text decription
+    desc,                                 // text decription
     L16Desc,                              // source format
     g722_16,                              // destination format
 
@@ -173,11 +174,13 @@ static struct PluginCodec_Definition g722CodecDefn[] =
 
     16000,                                // samples per second
     BITS_PER_SECOND,                      // raw bits per second
-    FRAME_TIME,                           // microseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    BYTES_PER_FRAME,                      // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    FRAME_TIME_MS,                        // microseconds per frame
+    {{
+      FRAME_TIME_RTP,                     // samples per frame
+      BYTES_PER_FRAME,                    // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG722,                              // RTP payload name
 
@@ -200,7 +203,7 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // specified RTP type
 
-    g722_16,                              // text decription
+    desc,                                 // text decription
     g722_16,                              // source format
     L16Desc,                              // destination format
 
@@ -208,11 +211,13 @@ static struct PluginCodec_Definition g722CodecDefn[] =
 
     16000,                                // samples per second
     BITS_PER_SECOND,                      // raw bits per second
-    FRAME_TIME,                           // microseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    BYTES_PER_FRAME,                      // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    FRAME_TIME_MS,                        // microseconds per frame
+    {{
+      FRAME_TIME_RTP,                     // samples per frame
+      BYTES_PER_FRAME,                    // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG722,                              // RTP payload name
 
@@ -237,19 +242,21 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeExplicit,          // specified RTP type
 
-    g722,                                 // text decription
+    desc,                                 // text decription
     L16Desc,                              // source format
     g722,                                 // destination format
 
     0,                                    // user data
 
-    CLOCK_RATE,                           // samples per second
+    RTP_CLOCK_RATE,                       // samples per second (not really)
     BITS_PER_SECOND,                      // raw bits per second
-    FRAME_TIME,                           // microseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    BYTES_PER_FRAME,                      // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    FRAME_TIME_MS,                        // microseconds per frame
+    {{
+      FRAME_TIME_RTP,                     // samples per frame
+      BYTES_PER_FRAME,                    // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG722,                              // RTP payload name
 
@@ -270,22 +277,23 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PluginCodec_MediaTypeAudio |          // audio codec
     PluginCodec_InputTypeRaw |            // raw input data
     PluginCodec_OutputTypeRaw |           // raw output data
-    PluginCodec_EmptyPayload |            // Can accept and empty payload
     PluginCodec_RTPTypeExplicit,          // specified RTP type
 
-    g722,                                 // text decription
+    desc,                                 // text decription
     g722,                                 // source format
     L16Desc,                              // destination format
 
     0,                                    // user data
 
-    CLOCK_RATE,                           // samples per second
+    RTP_CLOCK_RATE,                       // samples per second (not really)
     BITS_PER_SECOND,                      // raw bits per second
-    FRAME_TIME,                           // microseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    BYTES_PER_FRAME,                      // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    FRAME_TIME_MS,                        // microseconds per frame
+    {{
+      FRAME_TIME_RTP,                     // samples per frame
+      BYTES_PER_FRAME,                    // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG722,                              // RTP payload name
 
