@@ -23,26 +23,23 @@
  * 2550 Garcia Avenue
  * Mountain View, California  94043
  */
+/* 16kbps version created, used 24kbps code and changing as little as possible.
+ * G.726 specs are available from ITU's gopher or WWW site (http://www.itu.ch)
+ * If any errors are found, please contact me at mrand@tamu.edu
+ *      -Marc Randolph
+ */
 
 /*
- * g726_40.c
+ * g726_16.c
  *
  * Description:
  *
- * g723_40_encoder(), g723_40_decoder()
+ * g723_16_encoder(), g723_16_decoder()
  *
- * These routines comprise an implementation of the CCITT G.723 40Kbps
+ * These routines comprise an implementation of the CCITT G.726 16 Kbps
  * ADPCM coding algorithm.  Essentially, this implementation is identical to
- * the bit level description except for a few deviations which
- * take advantage of workstation attributes, such as hardware 2's
- * complement arithmetic.
- *
- * The deviation from the bit level specification (lookup tables),
- * preserves the bit level performance specifications.
- *
- * As outlined in the G.723 Recommendation, the algorithm is broken
- * down into modules.  Each section of code below is preceded by
- * the name of the module which it is implementing.
+ * the bit level description except for a few deviations which take advantage
+ * of workstation attributes, such as hardware 2's complement arithmetic.
  *
  * The ITU-T G.726 coder is an adaptive differential pulse code modulation
  * (ADPCM) waveform coding algorithm, suitable for coding of digitized
@@ -54,51 +51,48 @@
  * 2:1 over the G.711 format. For 24 kbps 40 kbps operation, the quantized
  * difference signal is 3 bits and 5 bits, respectively.
  *
- * $Revision: 22675 $
+ * $Revision: 22678 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
+ * $Date: 2009-05-21 01:35:59 +0100 (Thu, 21 May 2009) $
  */
-
 #include "g72x.h"
 #include "private.h"
 
 /*
- * Maps G.723_40 code word to ructeconstructed scale factor normalized log
- * magnitude values.
+ * Maps G.723_16 code word to reconstructed scale factor normalized log
+ * magnitude values.  Comes from Table 11/G.726
  */
-static short	_dqlntab[32] = {-2048, -66, 28, 104, 169, 224, 274, 318,
-				358, 395, 429, 459, 488, 514, 539, 566,
-				566, 539, 514, 488, 459, 429, 395, 358,
-				318, 274, 224, 169, 104, 28, -66, -2048};
+static short	_dqlntab[4] = { 116, 365, 365, 116}; 
 
-/* Maps G.723_40 code word to log of scale factor multiplier. */
-static short	_witab[32] = {448, 448, 768, 1248, 1280, 1312, 1856, 3200,
-			4512, 5728, 7008, 8960, 11456, 14080, 16928, 22272,
-			22272, 16928, 14080, 11456, 8960, 7008, 5728, 4512,
-			3200, 1856, 1312, 1280, 1248, 768, 448, 448};
+/* Maps G.723_16 code word to log of scale factor multiplier.
+ *
+ * _witab[4] is actually {-22 , 439, 439, -22}, but FILTD wants it
+ * as WI << 5  (multiplied by 32), so we'll do that here 
+ */
+static short	_witab[4] = {-704, 14048, 14048, -704};
 
 /*
- * Maps G.723_40 code words to a set of values whose long and short
+ * Maps G.723_16 code words to a set of values whose long and short
  * term averages are computed and then compared to give an indication
  * how stationary (steady state) the signal is.
  */
-static short	_fitab[32] = {0, 0, 0, 0, 0, 0x200, 0x200, 0x200,
-			0x200, 0x200, 0x400, 0x600, 0x800, 0xA00, 0xC00, 0xC00,
-			0xC00, 0xC00, 0xA00, 0x800, 0x600, 0x400, 0x200, 0x200,
-			0x200, 0x200, 0x200, 0, 0, 0, 0, 0};
 
-static int qtab_723_40[15] = {-122, -16, 68, 139, 198, 250, 298, 339,
-				378, 413, 445, 475, 502, 528, 553};
+/* Comes from FUNCTF */
+static short	_fitab[4] = {0, 0xE00, 0xE00, 0};
+
+/* Comes from quantizer decision level tables (Table 7/G.726)
+ */
+static int qtab_723_16[1] = {261};
+
 
 /*
- * g723_40_encoder()
+ * g723_16_encoder()
  *
- * Encodes a 16-bit linear PCM, A-law or u-law input sample and returns
- * the resulting 5-bit CCITT G.723 40Kbps code.
- * Returns -1 if the input coding value is invalid.
+ * Encodes a linear PCM, A-law or u-law input sample and returns its 2-bit code.
+ * Returns -1 if invalid input coding value.
  */
 int
-g726_40_encoder(
+g726_16_encoder(
 	int		sl,
 	int		in_coding,
 	g726_state *state_ptr)
@@ -133,32 +127,39 @@ g726_40_encoder(
 	sei = sezi + predictor_pole(state_ptr);
 	se = sei >> 1;			/* se = estimated signal */
 
-	d = sl - se;			/* d = estimation difference */
+	d = sl - se;			/* d = estimation diff. */
 
-	/* quantize prediction difference */
-	y = step_size(state_ptr);	/* adaptive quantizer step size */
-	i = quantize(d, y, qtab_723_40, 15);	/* i = ADPCM code */
+	/* quantize prediction difference d */
+	y = step_size(state_ptr);	/* quantizer step size */
+	i = quantize(d, y, qtab_723_16, 1);  /* i = ADPCM code */
 
-	dq = reconstruct(i & 0x10, _dqlntab[i], y);	/* quantized diff */
+	      /* Since quantize() only produces a three level output
+	       * (1, 2, or 3), we must create the fourth one on our own
+	       */
+	if (i == 3)                          /* i code for the zero region */
+	  if ((d & 0x8000) == 0)             /* If d > 0, i=3 isn't right... */
+	    i = 0;
+	    
+	dq = reconstruct(i & 2, _dqlntab[i], y); /* quantized diff. */
 
-	sr = (dq < 0) ? se - (dq & 0x7FFF) : se + dq; /* reconstructed signal */
+	sr = (dq < 0) ? se - (dq & 0x3FFF) : se + dq; /* reconstructed signal */
 
-	dqsez = sr + sez - se;		/* dqsez = pole prediction diff. */
+	dqsez = sr + sez - se;		/* pole prediction diff. */
 
-	update(5, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
+	update(2, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
 
 	return (i);
 }
 
 /*
- * g723_40_decoder()
+ * g723_16_decoder()
  *
- * Decodes a 5-bit CCITT G.723 40Kbps code and returns
+ * Decodes a 2-bit CCITT G.723_16 ADPCM code and returns
  * the resulting 16-bit linear PCM, A-law or u-law sample value.
  * -1 is returned if the output coding is unknown.
  */
 int
-g726_40_decoder(
+g726_16_decoder(
 	int		i,
 	int		out_coding,
 	g726_state *state_ptr)
@@ -172,26 +173,26 @@ g726_40_decoder(
 	int		sr;				/* ADDB */
 	int		dqsez;
 
-	i &= 0x1f;				/* mask to get proper bits */
+	i &= 0x03;			/* mask to get proper bits */
 	sezi = predictor_zero(state_ptr);
 	sez = sezi >> 1;
 	sei = sezi + predictor_pole(state_ptr);
 	se = sei >> 1;			/* se = estimated signal */
 
 	y = step_size(state_ptr);	/* adaptive quantizer step size */
-	dq = reconstruct(i & 0x10, _dqlntab[i], y);	/* estimation diff. */
+	dq = reconstruct(i & 0x02, _dqlntab[i], y); /* unquantize pred diff */
 
-	sr = (dq < 0) ? (se - (dq & 0x7FFF)) : (se + dq); /* reconst. signal */
+	sr = (dq < 0) ? (se - (dq & 0x3FFF)) : (se + dq); /* reconst. signal */
 
-	dqsez = sr - se + sez;		/* pole prediction diff. */
+	dqsez = sr - se + sez;			/* pole prediction diff. */
 
-	update(5, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
+	update(2, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
 
 	switch (out_coding) {
 	case AUDIO_ENCODING_ALAW:
-		return (tandem_adjust_alaw(sr, se, y, i, 0x10, qtab_723_40));
+		return (tandem_adjust_alaw(sr, se, y, i, 2, qtab_723_16));
 	case AUDIO_ENCODING_ULAW:
-		return (tandem_adjust_ulaw(sr, se, y, i, 0x10, qtab_723_40));
+		return (tandem_adjust_ulaw(sr, se, y, i, 2, qtab_723_16));
 	case AUDIO_ENCODING_LINEAR:
 		return (sr << 2);	/* sr was of 14-bit dynamic range */
 	default:
