@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22817 $
+ * $Revision: 33537 $
  * $Author: rjongbloed $
- * $Date: 2009-06-04 21:37:23 -0500 (Thu, 04 Jun 2009) $
+ * $Date: 2015-04-28 04:23:13 +0100 (Tue, 28 Apr 2015) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -32,7 +32,7 @@
 #include <string.h>
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.h>
@@ -113,6 +113,8 @@ static struct PluginCodec_information licenseInfo = {
 
 /////////////////////////////////////////////////////////////////////////////
 
+static const char desc[] = "ITU-T G.726 - ADPCM Codec";
+
 static const char L16Desc[]  = { "L16" };
 
 static const char sdpG726_16[]  = { "G726-16" };
@@ -185,7 +187,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_40,                              // text decription
+    desc,                                 // text decription
     L16Desc,                              // source format
     g726_40,                              // destination format
 
@@ -225,7 +227,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_40,                              // text decription
+    desc,                                 // text decription
     g726_40,                              // source format
     L16Desc,                              // destination format
 
@@ -267,7 +269,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_32,                              // text decription
+    desc,                                 // text decription
     L16Desc,                              // source format
     g726_32,                              // destination format
 
@@ -276,10 +278,12 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     8000,                                 // samples per second
     32000,                                // raw bits per second
     1000,                                 // nanoseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    4,                                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                  // samples per frame
+      4,                                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG726_32,                           // RTP payload name
 
@@ -303,7 +307,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_32,                              // text decription
+    desc,                                 // text decription
     g726_32,                              // source format
     L16Desc,                              // destination format
 
@@ -312,10 +316,12 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     8000,                                 // samples per second
     32000,                                // raw bits per second
     1000,                                 // nanoseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    4,                                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                  // samples per frame
+      4,                                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG726_32,                           // RTP payload name
 
@@ -341,7 +347,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_24,                              // text decription
+    desc,                                 // text decription
     L16Desc,                              // source format
     g726_24,                              // destination format
 
@@ -350,10 +356,12 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     8000,                                 // samples per second
     24000,                                // raw bits per second
     1000,                                 // nanoseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    3,                                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                  // samples per frame
+      3,                                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG726_24,                           // RTP payload name
 
@@ -377,7 +385,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_24,                              // text decription
+    desc,                                 // text decription
     g726_24,                              // source format
     L16Desc,                              // destination format
 
@@ -386,10 +394,12 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     8000,                                 // samples per second
     24000,                                // raw bits per second
     1000,                                 // nanoseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    3,                                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                  // samples per frame
+      3,                                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG726_32,                           // RTP payload name
 
@@ -415,7 +425,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_16,                              // text decription
+    desc,                                 // text decription
     L16Desc,                              // source format
     g726_16,                              // destination format
 
@@ -424,10 +434,12 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     8000,                                 // samples per second
     16000,                                // raw bits per second
     1000,                                 // nanoseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    2,                                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                  // samples per frame
+      2,                                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG726_16,                           // RTP payload name
 
@@ -451,7 +463,7 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // dynamic RTP type
 
-    g726_16,                              // text decription
+    desc,                                 // text decription
     g726_16,                              // source format
     L16Desc,                              // destination format
 
@@ -460,10 +472,12 @@ static struct PluginCodec_Definition g726CodecDefn[] =
     8000,                                 // samples per second
     16000,                                // raw bits per second
     1000,                                 // nanoseconds per frame
-    SAMPLES_PER_FRAME,                    // samples per frame
-    2,                                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,               // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,                // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                  // samples per frame
+      2,                                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                         // IANA RTP payload code
     sdpG726_16,                           // RTP payload name
 
