@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22746 $
+ * $Revision: 33537 $
  * $Author: rjongbloed $
- * $Date: 2009-05-28 01:34:50 +0000 (Thu, 28 May 2009) $
+ * $Date: 2015-04-28 04:23:13 +0100 (Tue, 28 Apr 2015) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -32,12 +32,12 @@
 #include <string.h>
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.h>
 
-#include "./inc/gsm.h"
+#include <gsm.h>
 
 #define	BITS_PER_SECOND         13200
 
@@ -241,6 +241,8 @@ static struct PluginCodec_information licenseInfo = {
 
 /////////////////////////////////////////////////////////////////////////////
 
+static const char desc[] = "GSM 06.10 - 13kbps Audio Codec";
+
 static const char L16Desc[]  = { "L16" };
 
 static const char gsm0610[]  = { "GSM-06.10" };
@@ -360,7 +362,7 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeExplicit,        // specified RTP type
 
-    gsm0610,                            // text decription
+    desc,                               // text decription
     L16Desc,                            // source format
     gsm0610,                            // destination format
 
@@ -369,10 +371,12 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     8000,                               // samples per second
     BITS_PER_SECOND,                    // raw bits per second
     20000,                              // nanoseconds per frame
-    SAMPLES_PER_FRAME,                  // samples per frame
-    BYTES_PER_FRAME,                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                // samples per frame
+      BYTES_PER_FRAME,                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,           // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,            // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                       // IANA RTP payload code
     sdpGSM,                             // RTP payload name
 
@@ -395,7 +399,7 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeExplicit,        // dynamic RTP type
 
-    gsm0610,                            // text decription
+    desc,                               // text decription
     gsm0610,                            // source format
     L16Desc,                            // destination format
 
@@ -404,10 +408,12 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     8000,                               // samples per second
     BITS_PER_SECOND,                    // raw bits per second
     20000,                              // nanoseconds per frame
-    SAMPLES_PER_FRAME,                  // samples per frame
-    BYTES_PER_FRAME,                    // bytes per frame
-    PREF_FRAMES_PER_PACKET,             // recommended number of frames per packet
-    MAX_FRAMES_PER_PACKET,              // maximum number of frames per packe
+    {{
+      SAMPLES_PER_FRAME,                // samples per frame
+      BYTES_PER_FRAME,                  // bytes per frame
+      PREF_FRAMES_PER_PACKET,           // recommended number of frames per packet
+      MAX_FRAMES_PER_PACKET,            // maximum number of frames per packe
+    }},
     PAYLOAD_CODE,                       // IANA RTP payload code
     sdpGSM,                             // RTP payload name
 
@@ -433,7 +439,7 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeDynamic,         // specified RTP type
 
-    msGSM,                              // text decription
+    desc,                               // text decription
     L16Desc,                            // source format
     msGSM,                              // destination format
 
@@ -442,10 +448,12 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     8000,                               // samples per second
     BITS_PER_SECOND,                    // raw bits per second
     40000,                              // nanoseconds per frame
-    MSGSM_SAMPLES_PER_FRAME,            // samples per frame
-    MSGSM_BYTES_PER_FRAME,              // bytes per frame
-    1,                                  // recommended number of frames per packet
-    1,                                  // maximum number of frames per packe
+    {{
+      MSGSM_SAMPLES_PER_FRAME,          // samples per frame
+      MSGSM_BYTES_PER_FRAME,            // bytes per frame
+      1,                                // recommended number of frames per packet
+      1,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpMSGSM,                           // RTP payload name
 
@@ -468,7 +476,7 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    msGSM,                              // text decription
+    desc,                               // text decription
     msGSM,                              // source format
     L16Desc,                            // destination format
 
@@ -477,10 +485,12 @@ static struct PluginCodec_Definition gsmCodecDefn[4] =
     8000,                               // samples per second
     BITS_PER_SECOND,                    // raw bits per second
     40000,                              // nanoseconds per frame
-    MSGSM_SAMPLES_PER_FRAME,            // samples per frame
-    MSGSM_BYTES_PER_FRAME,              // bytes per frame
-    1,                                  // recommended number of frames per packet
-    1,                                  // maximum number of frames per packe
+    {{
+      MSGSM_SAMPLES_PER_FRAME,          // samples per frame
+      MSGSM_BYTES_PER_FRAME,            // bytes per frame
+      1,                                // recommended number of frames per packet
+      1,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpMSGSM,                           // RTP payload name
 
