@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29803 $
+ * $Revision: 34175 $
  * $Author: rjongbloed $
- * $Date: 2013-05-26 23:58:43 -0500 (Sun, 26 May 2013) $
+ * $Date: 2015-10-10 11:27:32 +0100 (Sat, 10 Oct 2015) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -36,7 +36,7 @@
 #endif
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.h>
@@ -48,7 +48,11 @@
 #endif
 
 extern "C" {
-#include "libspeex/speex.h" 
+  #if OPAL_SYSTEM_SPEEX
+    #include <speex/speex.h>
+  #else
+    #include "libspeex/speex.h" 
+  #endif
 }
 
 #define NARROW_SAMPLES_PER_FRAME       160
@@ -57,9 +61,6 @@ extern "C" {
 #define NS_PER_FRAME                   20000
 
 
-const float MaxSampleValue   = 32767.0;
-const float MinSampleValue   = -32767.0;
-
 struct PluginSpeexContext {
   SpeexBits speexBits;
   void      * coderState;
@@ -79,17 +80,17 @@ static int Speex_Bits_Per_Second(int mode, int sampleRate) {
     return bitrate;
 }
 
+/*
 static int Speex_Bytes_Per_Frame(int mode, int sampleRate) {
     int bits_per_frame = Speex_Bits_Per_Second(mode, sampleRate) / 50; // (20ms frame size)
     return ((bits_per_frame+7)/8); // round up
 }
+*/
 
 /////////////////////////////////////////////////////////////////////////////
 
 static void * create_encoder(const struct PluginCodec_Definition * codec)
 {
-  intptr_t mode = (intptr_t)codec->userData;
-
   struct PluginSpeexContext * context = new PluginSpeexContext;
 
   if (codec->sampleRate == 16000)
@@ -98,7 +99,12 @@ static void * create_encoder(const struct PluginCodec_Definition * codec)
     context->coderState = speex_encoder_init(&speex_nb_mode);
 
   speex_encoder_ctl(context->coderState, SPEEX_GET_FRAME_SIZE, &context->encoderFrameSize);
-  speex_encoder_ctl(context->coderState, SPEEX_SET_QUALITY,    &mode);
+
+  int value = 6;
+  speex_encoder_ctl(context->coderState, SPEEX_SET_QUALITY, &value);
+
+  value = 5;
+  speex_encoder_ctl(context->coderState, SPEEX_SET_COMPLEXITY, &value);
 
   return context;
 }
@@ -141,7 +147,7 @@ static void destroy_encoder(const struct PluginCodec_Definition * /*codec*/, voi
   PluginSpeexContext * context = (PluginSpeexContext *)_context;
 
   speex_encoder_destroy(context->coderState); 
-  free(context);
+  delete context;
 }
 
 static void * create_decoder(const struct PluginCodec_Definition * codec)
@@ -208,22 +214,7 @@ static void destroy_decoder(const struct PluginCodec_Definition * /*codec*/, voi
 
   //speex_bits_destroy(&context->speexBits);  // speex_decoder_destroy does this
   speex_decoder_destroy(context->coderState); 
-  free(context);
-}
-
-static int encoder_get_options(
-      const PluginCodec_Definition * , 
-      void * /*context*/, 
-      const char * , 
-      void * parm , 
-      unsigned * parmLen)
-{
-  if (parmLen == NULL || parm == NULL || *parmLen != sizeof(char **))
-    return 0;
-
-  const char ***options = (const char ***)parm;
-
-  return 0;
+  delete context;
 }
 
 static int valid_for_sip(
@@ -254,12 +245,12 @@ static int valid_for_h323(
 }
 
 static const char * default_narrow_sip_options[][3] = {
-  { "fmtp",                       "sr=8000,mode=any" ,      "s" },
+  { "fmtp", "mode=any", "s" },
   { NULL, NULL, NULL }
 };
 
 static const char * default_wide_sip_options[][3] = {
-  { "fmtp",                       "sr=16000,mode=any" ,     "s" },
+  { "fmtp", "mode=any", "s" },
   { NULL, NULL, NULL }
 };
 
@@ -373,6 +364,8 @@ static struct PluginCodec_information licenseInfo = {
   PluginCodec_License_BSD                                      // codec license code
 };
 
+static const char desc[] = "Xiph Speex CELP Audio Codec";
+
 static const char L16Desc[]  = { "L16" };
 
 static const char sdpSpeex[]  = { "Speex" };
@@ -462,7 +455,7 @@ CREATE_IETFSPEEX_CAP_DATA(desc, suffix, ordinal, 8000) \
   PluginCodec_RTPTypeShared |         /* share RTP code */ \
   PluginCodec_RTPTypeDynamic |        /* dynamic RTP type */ \
   PluginCodec_DecodeSilence,          /* can encode silence frames */ \
-  prefix##suffix,                     /* text decription */ \
+  desc,                               /* text decription */ \
   L16Desc,                            /* source format */ \
   prefix##suffix,                     /* destination format */ \
   (void *)ordinal,                    /* user data */ \
@@ -494,7 +487,7 @@ CREATE_IETFSPEEX_CAP_DATA(desc, suffix, ordinal, 8000) \
   PluginCodec_RTPTypeShared |         /* share RTP code */ \
   PluginCodec_RTPTypeDynamic |        /* dynamic RTP type */ \
   PluginCodec_DecodeSilence,          /* can encode silence frames */ \
-  prefix##suffix,                     /* text decription */ \
+  desc,                               /* text decription */ \
   prefix##suffix,                     /* source format */ \
   L16Desc,                            /* destination format */ \
   (void *)ordinal,                    /* user data */ \
@@ -517,6 +510,11 @@ CREATE_IETFSPEEX_CAP_DATA(desc, suffix, ordinal, 8000) \
   &prefix##suffix##Cap                /* h323CapabilityData */ \
 } \
 
+#if __GNUC__
+#pragma GCC diagnostic ignored "-Wunused-const-variable"
+#pragma GCC diagnostic ignored "-Wunused-variable"
+#endif
+
 CREATE_NARROW_SPEEX_CAP_DATA(Narrow-5.95k, Narrow5k95,  2)
 CREATE_NARROW_SPEEX_CAP_DATA(Narrow-8k,    Narrow8k,    3)
 CREATE_NARROW_SPEEX_CAP_DATA(Narrow-11k,   Narrow11k,   4)
@@ -550,7 +548,7 @@ CREATE_IETFSPEEX_CAP_DATA(desc, suffix, ordinal, 16000)
   PluginCodec_RTPTypeShared |         /* share RTP code */ \
   PluginCodec_RTPTypeDynamic |        /* dynamic RTP type */ \
   PluginCodec_DecodeSilence,          /* can encode silence frames */ \
-  prefix##suffix,                     /* text decription */ \
+  desc,                               /* text decription */ \
   L16Desc,                            /* source format */ \
   prefix##suffix,                     /* destination format */ \
   (void *)ordinal,                    /* user data */ \
@@ -582,7 +580,7 @@ CREATE_IETFSPEEX_CAP_DATA(desc, suffix, ordinal, 16000)
   PluginCodec_RTPTypeShared |         /* share RTP code */ \
   PluginCodec_RTPTypeDynamic |        /* dynamic RTP type */ \
   PluginCodec_DecodeSilence,          /* can encode silence frames */ \
-  prefix##suffix,                     /* text decription */ \
+  desc,                               /* text decription */ \
   prefix##suffix,                     /* source format */ \
   L16Desc,                            /* destination format */ \
   (void *)ordinal,                    /* user data */ \
@@ -634,7 +632,7 @@ static struct PluginCodec_H323NonStandardCodecData speexW##suffix##Cap = \
   PluginCodec_RTPTypeShared |         /* share RTP code */ \
   PluginCodec_RTPTypeDynamic |        /* dynamic RTP type */ \
   PluginCodec_DecodeSilence,          /* can encode silence frames */ \
-  speexW##suffix,                     /* text decription */ \
+  desc,                               /* text decription */ \
   L16Desc,                            /* source format */ \
   speexW##suffix,                     /* destination format */ \
   (void *)ordinal,                    /* user data */ \
@@ -665,7 +663,7 @@ static struct PluginCodec_H323NonStandardCodecData speexW##suffix##Cap = \
   PluginCodec_OutputTypeRaw |         /* raw output data */ \
   PluginCodec_RTPTypeShared |         /* share RTP code */ \
   PluginCodec_RTPTypeDynamic,         /* dynamic RTP type */ \
-  speexW##suffix,                     /* text decription */ \
+  desc,                               /* text decription */ \
   speexW##suffix,                     /* source format */ \
   L16Desc,                            /* destination format */ \
   (void *)ordinal,                    /* user data */ \
@@ -703,7 +701,7 @@ CREATE_NARROW_SPEEXW_CAP_DATA(Narrow-8k,    Narrow8k,    3)
   PluginCodec_RTPTypeShared |         /* share RTP code */ \
   PluginCodec_RTPTypeDynamic |        /* dynamic RTP type */ \
   PluginCodec_DecodeSilence,          /* can encode silence frames */ \
-  name,                               /* text decription */ \
+  desc,                               /* text decription */ \
   L16Desc,                            /* source format */ \
   name,                               /* destination format */ \
   (void *)ordinal,                    /* user data */ \
@@ -854,7 +852,7 @@ extern "C" {
 
   void OutputInfo(int mode, int rate)
   {
-    int bps = Speex_Bits_Per_Second(mode, rate);
+    //int bps = Speex_Bits_Per_Second(mode, rate);
     //printf("mode = %i, rate = %i, bps = %i\n", mode, rate, bps);
   }
 
