@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22678 $
+ * $Revision: 30035 $
  * $Author: rjongbloed $
- * $Date: 2009-05-20 19:35:59 -0500 (Wed, 20 May 2009) $
+ * $Date: 2013-06-24 09:32:14 +0100 (Mon, 24 Jun 2013) $
  */
 
 #include <codec/opalplugin.h>
@@ -101,12 +101,10 @@ static const char ianaName[]   = { "G729" };
 #define IANACODE  18
 
 
-static const char VADStr[] = "VAD";
-
 static struct PluginCodec_Option const NoVADOption =
 {
   PluginCodec_EnumOption,     // PluginCodec_OptionTypes
-  VADStr,                     // Generic (human readable) option name
+  PLUGINCODEC_OPTION_VOICE_ACTIVITY_DETECT,                     // Generic (human readable) option name
   0,                          // Read Only flag
   PluginCodec_MinMerge,       // Merge mode
   "no",                       // Initial value
@@ -126,7 +124,7 @@ static struct PluginCodec_Option const * const NoVADOptionTable[] = {
 static struct PluginCodec_Option const VADOption =
 {
   PluginCodec_EnumOption,     // PluginCodec_OptionTypes
-  VADStr,                     // Generic (human readable) option name
+  PLUGINCODEC_OPTION_VOICE_ACTIVITY_DETECT, // Generic (human readable) option name
   0,                          // Read Only flag
   PluginCodec_MinMerge,       // Merge mode
   "yes",                      // Initial value
@@ -311,7 +309,7 @@ static int set_codec_options(const struct PluginCodec_Definition * defn,
     return 0;
 
   for (option = (const char * const *)parm; *option != NULL; option += 2) {
-    if (_stricmp(option[0], VADStr) == 0)
+    if (_stricmp(option[0], VADOption.m_name) == 0)
       vad = atoi(option[1]) != 0;
   }
 
