@@ -3,9 +3,9 @@
  *
  * Copyright (C) 2004 MX Telecom Ltd.
  *
- * $Revision: 27584 $
+ * $Revision: 33537 $
  * $Author: rjongbloed $
- * $Date: 2012-05-07 00:41:25 -0500 (Mon, 07 May 2012) $
+ * $Date: 2015-04-28 04:23:13 +0100 (Tue, 28 Apr 2015) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -16,7 +16,7 @@
 #include <string.h>
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.h>
@@ -270,11 +270,10 @@ static struct PluginCodec_Option const amrInitialMode =
   "7"                         // Maximum value
 };
 
-static const char VADStr[] = "VAD";
 static struct PluginCodec_Option const amrVAD =
 {
   PluginCodec_BoolOption,     // PluginCodec_OptionTypes
-  VADStr,                     // Generic (human readable) option name
+  PLUGINCODEC_OPTION_VOICE_ACTIVITY_DETECT,                     // Generic (human readable) option name
   0,                          // Read Only flag
   PluginCodec_AndMerge,       // Merge mode
   "1",                        // Initial value
@@ -334,7 +333,7 @@ static int set_codec_options(const struct PluginCodec_Definition * defn,
       if (amr->mode > AMR_BITRATE_1220)
         amr->mode = AMR_BITRATE_1220;
     }
-    else if (STRCMPI(option[0], VADStr) == 0)
+    else if (STRCMPI(option[0], amrVAD.m_name) == 0)
       amr->vad = atoi(option[1]) != 0;
   }
 
@@ -380,6 +379,8 @@ static const struct PluginCodec_H323GenericCodecData amrcap =
   122                             // Must always be this regardless of "Max Bit Rate" option
 };
 
+static const char desc[] = "GSM - Adaptive Multi Rate Codec";
+
 static struct PluginCodec_Definition amrCodecDefn[] = {
   {
     // encoder
@@ -391,7 +392,7 @@ static struct PluginCodec_Definition amrCodecDefn[] = {
     PluginCodec_OutputTypeRaw |             // raw output data
     PluginCodec_RTPTypeDynamic,             // dynamic RTP type
     
-    "GSM-AMR",                              // text decription
+    desc,                                   // text decription
     "L16",                                  // source format
     "GSM-AMR",                              // destination format
     
@@ -400,11 +401,12 @@ static struct PluginCodec_Definition amrCodecDefn[] = {
     8000,                                   // samples per second
     12200,                                  // raw bits per second
     20000,                                  // microseconds per frame
-    L_FRAME,                                // samples per frame
-    33,                                     // bytes per frame
-    
-    1,                                      // recommended number of frames per packet
-    1,                                      // maximum number of frames per packet
+    {{
+      L_FRAME,                              // samples per frame
+      33,                                   // bytes per frame
+      1,                                    // recommended number of frames per packet
+      1,                                    // maximum number of frames per packet
+    }},
     0,                                      // IANA RTP payload code
     "AMR",                                  // RTP payload name
     
@@ -427,7 +429,7 @@ static struct PluginCodec_Definition amrCodecDefn[] = {
     PluginCodec_RTPTypeDynamic |            // dynamic RTP type
     PluginCodec_DecodeSilence,              // Can accept missing (empty) frames and generate silence
 
-    "GSM-AMR",                              // text decription
+    desc,                                   // text decription
     "GSM-AMR",                              // source format
     "L16",                                  // destination format
 
@@ -436,10 +438,12 @@ static struct PluginCodec_Definition amrCodecDefn[] = {
     8000,                                   // samples per second
     12200,                                  // raw bits per second
     30000,                                  // microseconds per frame
-    L_FRAME,                                // samples per frame
-    33,                                     // bytes per frame
-    1,                                      // recommended number of frames per packet
-    1,                                      // maximum number of frames per packet
+    {{
+      L_FRAME,                              // samples per frame
+      33,                                   // bytes per frame
+      1,                                    // recommended number of frames per packet
+      1,                                    // maximum number of frames per packet
+    }},
     0,                                      // IANA RTP payload code
     "AMR",                                  // RTP payload name
 
