@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29871 $
+ * $Revision: 33537 $
  * $Author: rjongbloed $
- * $Date: 2013-06-03 03:11:11 -0500 (Mon, 03 Jun 2013) $
+ * $Date: 2015-04-28 04:23:13 +0100 (Tue, 28 Apr 2015) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -32,14 +32,28 @@
 #include <string.h>
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.h>
 
-#include "iLBC/iLBC_encode.h" 
-#include "iLBC/iLBC_decode.h" 
-#include "iLBC/iLBC_define.h" 
+#ifdef OPAL_SYSTEM_ILBC
+  #include <iLBC_encode.h>
+  #include <iLBC_decode.h>
+  #include <iLBC_define.h>
+#else
+  #include "./iLBC/iLBC_encode.h"
+  #include "./iLBC/iLBC_decode.h"
+  #include "./iLBC/iLBC_define.h"
+#endif
+
+
+#ifndef NO_OF_BYTES_20MS
+  #define NO_OF_BYTES_20MS 38
+#endif
+#ifndef NO_OF_BYTES_30MS
+  #define NO_OF_BYTES_30MS 50
+#endif
 
 #define	BITRATE_30MS	NO_OF_BYTES_30MS*8*8000/BLOCKL_30MS
 #define	BITRATE_20MS	NO_OF_BYTES_20MS*8*8000/BLOCKL_20MS
@@ -83,8 +97,10 @@ static int codec_encoder(const struct PluginCodec_Definition * codec,
                                        unsigned * toLen,
                                    unsigned int * flag)
 {
-  float block[BLOCKL_MAX];
   int i;
+#ifndef OPAL_SYSTEM_ILBC
+  float block[BLOCKL_MAX];
+#endif
 
   struct iLBC_Enc_Inst_t_ * encoder = (struct iLBC_Enc_Inst_t_ *)context;
   const short * sampleBuffer = (const short *)from;
@@ -92,12 +108,16 @@ static int codec_encoder(const struct PluginCodec_Definition * codec,
   if (*fromLen < encoder->blockl*2U)
     return 0;
 
+#ifdef OPAL_SYSTEM_ILBC
+  iLBC_encode(to, (short *)from, encoder);
+#else
   /* convert signal to float */
   for (i = 0; i < encoder->blockl; i++)
     block[i] = (float)sampleBuffer[i];
 
   /* do the actual encoding */
-  iLBC_encode(to, block, encoder);
+  iLBC_encode((unsigned char *)to, block, encoder);
+#endif
 
   // set output lengths
   *toLen = encoder->no_of_bytes;
@@ -116,10 +136,12 @@ static int codec_decoder(const struct PluginCodec_Definition * codec,
                                    unsigned int * flag)
 {
   int i;
+#ifndef OPAL_SYSTEM_ILBC
   float block[BLOCKL_MAX];
+  short * sampleBuffer = (short *)to;
+#endif
 
   struct iLBC_Dec_Inst_t_ * decoder = (struct iLBC_Dec_Inst_t_ *)context;
-  short * sampleBuffer = (short *)to;
 
   // If packet not have integral number of frames for this mode
   if ((*fromLen % decoder->no_of_bytes) != 0) {
@@ -130,20 +152,24 @@ static int codec_decoder(const struct PluginCodec_Definition * codec,
   }
 
   /* do actual decoding of block */ 
+#ifdef OPAL_SYSTEM_ILBC
+  iLBC_decode((short *)to, (WebRtc_UWord16 *)from, decoder, 1);
+#else
   iLBC_decode(block, (unsigned char *)from, decoder, 1);
 
-  if (*toLen < decoder->blockl*2U)
-    return 0;
-
   /* convert to short */     
   for (i = 0; i < decoder->blockl; i++) {
     float tmp = block[i];
-    if (tmp < MIN_SAMPLE)
-      tmp = MIN_SAMPLE;
-    else if (tmp > MAX_SAMPLE)
-      tmp = MAX_SAMPLE;
+    if (tmp < -32767)
+      tmp = -32767;
+    else if (tmp > 32767)
+      tmp = 32767;
     sampleBuffer[i] = (short)tmp;
   }
+#endif
+
+  if (*toLen < decoder->blockl*2U)
+    return 0;
 
   // set output lengths
   *toLen = decoder->blockl*2;
@@ -401,6 +427,8 @@ static struct PluginCodec_information licenseInfo = {
   PluginCodec_License_Freeware                                // codec license code
 };
 
+static const char desc[] = "iLBC - Internet Low Bitrate Audio Codec";
+
 static const char L16Desc[]  = { "L16" };
 
 static const char iLBC13k3[] = { "iLBC-13k3" };
@@ -425,7 +453,7 @@ static struct PluginCodec_H323NonStandardCodecData ilbc13k3Cap =
   EQUIVALENCE_COUNTRY_CODE, 
   EQUIVALENCE_EXTENSION_CODE, 
   EQUIVALENCE_MANUFACTURER_CODE,
-  iLBC13k3, sizeof(iLBC13k3)-1,
+  (unsigned char *)iLBC13k3, sizeof(iLBC13k3)-1,
   NULL
 };
 
@@ -435,7 +463,7 @@ static struct PluginCodec_H323NonStandardCodecData ilbc15k2Cap =
   EQUIVALENCE_COUNTRY_CODE, 
   EQUIVALENCE_EXTENSION_CODE, 
   EQUIVALENCE_MANUFACTURER_CODE,
-  iLBC15k2, sizeof(iLBC15k2)-1,
+  (unsigned char *)iLBC15k2, sizeof(iLBC15k2)-1,
   NULL
 };
 
@@ -452,7 +480,7 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     PluginCodec_RTPTypeShared |         // share RTP code 
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    sdpILBC,                            // text decription
+    desc,                               // text decription
     L16Desc,                            // source format
     sdpILBC,                            // destination format
 
@@ -461,10 +489,12 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     8000,                               // samples per second
     BITRATE_30MS,                       // raw bits per second (note we use highest value here)
     30000,                              // nanoseconds per frame (note we use highest value here)
-    BLOCKL_30MS,                        // samples per frame (note we use highest value here)
-    NO_OF_BYTES_30MS,                   // bytes per frame (note we use highest value here)
-    2,                                  // recommended number of frames per packet
-    5,                                  // maximum number of frames per packe
+    {{
+      BLOCKL_30MS,                      // samples per frame (note we use highest value here)
+      NO_OF_BYTES_30MS,                 // bytes per frame (note we use highest value here)
+      2,                                // recommended number of frames per packet
+      5,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpILBC,                            // RTP payload name
 
@@ -488,7 +518,7 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     PluginCodec_RTPTypeShared |         // share RTP code 
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    sdpILBC,                            // text decription
+    desc,                               // text decription
     sdpILBC,                            // source format
     L16Desc,                            // destination format
 
@@ -497,10 +527,12 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     8000,                               // samples per second
     BITRATE_30MS,                       // raw bits per second (note we use highest value here)
     30000,                              // nanoseconds per frame (note we use highest value here)
-    BLOCKL_30MS,                        // samples per frame (note we use highest value here)
-    NO_OF_BYTES_30MS,                   // bytes per frame (note we use highest value here)
-    2,                                  // recommended number of frames per packet
-    5,                                  // maximum number of frames per packe
+    {{
+      BLOCKL_30MS,                      // samples per frame (note we use highest value here)
+      NO_OF_BYTES_30MS,                 // bytes per frame (note we use highest value here)
+      2,                                // recommended number of frames per packet
+      5,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpILBC,                            // RTP payload name
 
@@ -524,7 +556,7 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     PluginCodec_RTPTypeShared |         // share RTP code 
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    iLBC13k3,                           // text decription
+    desc,                               // text decription
     L16Desc,                            // source format
     iLBC13k3,                           // destination format
 
@@ -533,10 +565,12 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     8000,                               // samples per second
     BITRATE_30MS,                       // raw bits per second
     30000,                              // nanoseconds per frame
-    BLOCKL_30MS,                        // samples per frame
-    NO_OF_BYTES_30MS,                   // bytes per frame
-    1,                                  // recommended number of frames per packet
-    1,                                  // maximum number of frames per packe
+    {{
+      BLOCKL_30MS,                      // samples per frame
+      NO_OF_BYTES_30MS,                 // bytes per frame
+      1,                                // recommended number of frames per packet
+      1,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpILBC,                            // RTP payload name
 
@@ -560,7 +594,7 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     PluginCodec_RTPTypeShared |         // share RTP code 
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    iLBC13k3,                           // text decription
+    desc,                               // text decription
     iLBC13k3,                           // source format
     L16Desc,                            // destination format
 
@@ -569,10 +603,12 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     8000,                               // samples per second
     BITRATE_30MS,                       // raw bits per second
     30000,                              // nanoseconds per frame
-    BLOCKL_30MS,                        // samples per frame
-    NO_OF_BYTES_30MS,                   // bytes per frame
-    1,                                  // recommended number of frames per packet
-    1,                                  // maximum number of frames per packe
+    {{
+      BLOCKL_30MS,                      // samples per frame
+      NO_OF_BYTES_30MS,                 // bytes per frame
+      1,                                // recommended number of frames per packet
+      1,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpILBC,                            // RTP payload name
 
@@ -596,7 +632,7 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     PluginCodec_RTPTypeShared |         // share RTP code 
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    iLBC15k2,                           // text decription
+    desc,                               // text decription
     L16Desc,                            // source format
     iLBC15k2,                           // destination format
 
@@ -605,10 +641,12 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     8000,                               // samples per second
     BITRATE_20MS,                       // raw bits per second
     20000,                              // nanoseconds per frame
-    BLOCKL_20MS,                        // samples per frame
-    NO_OF_BYTES_20MS,                   // bytes per frame
-    1,                                  // recommended number of frames per packet
-    1,                                  // maximum number of frames per packe
+    {{
+      BLOCKL_20MS,                      // samples per frame
+      NO_OF_BYTES_20MS,                 // bytes per frame
+      1,                                // recommended number of frames per packet
+      1,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpILBC,                            // RTP payload name
 
@@ -632,7 +670,7 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     PluginCodec_RTPTypeShared |         // share RTP code 
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    iLBC15k2,                           // text decription
+    desc,                               // text decription
     iLBC15k2,                           // source format
     L16Desc,                            // destination format
 
@@ -641,10 +679,12 @@ static struct PluginCodec_Definition iLBCCodecDefn[] =
     8000,                               // samples per second
     BITRATE_20MS,                       // raw bits per second
     20000,                              // nanoseconds per frame
-    BLOCKL_20MS,                        // samples per frame
-    NO_OF_BYTES_20MS,                   // bytes per frame
-    1,                                  // recommended number of frames per packet
-    1,                                  // maximum number of frames per packe
+    {{
+      BLOCKL_20MS,                      // samples per frame
+      NO_OF_BYTES_20MS,                 // bytes per frame
+      1,                                // recommended number of frames per packet
+      1,                                // maximum number of frames per packe
+    }},
     0,                                  // IANA RTP payload code
     sdpILBC,                            // RTP payload name
 
