@@ -27,11 +27,12 @@
 
 #include <codec/opalplugin.h>
 
-extern "C" {
-#include "spandsp.h"
-};
-
 #include <stdlib.h>
+#include <sstream>
+#include <vector>
+#include <queue>
+#include <map>
+
 
 #if defined(_WIN32) || defined(_WIN32_WCE)
   #define WIN32_LEAN_AND_MEAN
@@ -47,10 +48,9 @@ extern "C" {
   #define DIR_SEPERATORS "/"
 #endif
 
-#include <sstream>
-#include <vector>
-#include <queue>
-#include <map>
+extern "C" {
+#include "spandsp.h"
+};
 
 
 #define LOGGING 1
@@ -859,8 +859,9 @@ class FaxTIFF : public FaxSpanDSP
         if (m_tiffFileName.empty())
           m_tiffFileName = value;
 #if LOGGING
-        else if (*value != '\0' && m_tiffFileName != value)
+        else if (*value != '\0' && m_tiffFileName != value) {
           PTRACE(2, m_tag << " Cannot change filename in mid stream from \"" << m_tiffFileName << "\" to \"" << value << '"');
+        }
 #endif
         return true;
       }
@@ -889,7 +890,7 @@ class FaxTIFF : public FaxSpanDSP
       InitLogging(t30_get_logging_state(t30state), m_tag);
 
       if (m_tiffFileName.empty()) {
-        PTRACE(1, m_tag << " No TIFF file to " << m_receiving ? "receive" : "transmit");
+        PTRACE(1, m_tag << " No TIFF file to " << (m_receiving ? "receive" : "transmit"));
         return false;
       }
 
@@ -1780,7 +1781,7 @@ static PluginCodec_Definition faxCodecDefn[] = {
     PluginCodec_OutputTypeRTP |         // RTP output data
     PluginCodec_RTPTypeExplicit,        // explicit RTP type
 
-    "PCM to T.38 Codec",                // text decription
+    "SpanDSP - PCM to T.38 Codec",      // text decription
     L16Format,                          // source format
     T38Format,                          // destination format
 
@@ -1816,7 +1817,7 @@ static PluginCodec_Definition faxCodecDefn[] = {
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeExplicit,        // explicit RTP type
 
-    "T.38 to PCM Codec",                // text decription
+    "SpanDSP - T.38 to PCM Codec",      // text decription
     T38Format,                          // source format
     L16Format,                          // destination format
 
@@ -1851,7 +1852,7 @@ static PluginCodec_Definition faxCodecDefn[] = {
     PluginCodec_OutputTypeRTP |         // RTP output data
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    "TIFF to T.38 Codec",               // text decription
+    "SpanDSP - TIFF to T.38 Codec",     // text decription
     TIFFFormat,                         // source format
     T38Format,                          // destination format
 
@@ -1886,7 +1887,7 @@ static PluginCodec_Definition faxCodecDefn[] = {
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    "T.38 to TIFF Codec",               // text decription
+    "SpanDSP - T.38 to TIFF Codec",     // text decription
     T38Format,                          // source format
     TIFFFormat,                         // destination format
 
@@ -1921,7 +1922,7 @@ static PluginCodec_Definition faxCodecDefn[] = {
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    "PCM to TIFF Codec",                // text decription
+    "SpanDSP - PCM to TIFF Codec",      // text decription
     L16Format,                          // source format
     TIFFFormat,                         // destination format
 
@@ -1956,7 +1957,7 @@ static PluginCodec_Definition faxCodecDefn[] = {
     PluginCodec_OutputTypeRaw |         // raw output data
     PluginCodec_RTPTypeDynamic,         // dynamic RTP type
 
-    "TIFF to PCM Codec",                // text decription
+    "SpanDSP - TIFF to PCM Codec",      // text decription
     TIFFFormat,                         // source format
     L16Format,                          // destination format
 
