@@ -25,9 +25,9 @@
  * Contributor(s): Michele Piccini (michele@piccini.com)
  *                 Derek Smithies (derek@indranet.co.nz)
  *
- * $Revision: 27898 $
+ * $Revision: 33537 $
  * $Author: rjongbloed $
- * $Date: 2012-06-25 21:05:22 -0500 (Mon, 25 Jun 2012) $
+ * $Date: 2015-04-28 04:23:13 +0100 (Tue, 28 Apr 2015) $
  */
 
 /*
@@ -56,7 +56,7 @@
 #include <algorithm>
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.hpp>
@@ -478,6 +478,7 @@ static struct PluginCodec_information licenseInfo = {
   PluginCodec_License_BSD                                        // codec license code
 };
 
+static const char desc[]         = "ITU-T H.261 - Video Codec (Vic)";
 static const char YUV420PDesc[]  = { "YUV420P" };
 static const char h261Desc[]     = { "H.261" };
 static const char sdpH261[]      = { "h261" };
@@ -615,16 +616,9 @@ static int to_normalised_options(const struct PluginCodec_Definition *, void *,
     frameTime = 3003*cif_mpi;
   }
   else {
-#ifdef DEFAULT_TO_FULL_CAPABILITIES
     maxWidth = 352;
     maxHeight = 288;
     frameTime = 3003*1;
-#else
-    // Default handling according to RFC 4587 (QCIF=1)
-    maxWidth = 176;
-    maxHeight = 144;
-    frameTime = 3003*1;
-#endif
   }
 
   char ** options = (char **)calloc(15, sizeof(char *));
@@ -739,41 +733,8 @@ static int free_codec_options(const struct PluginCodec_Definition *, void *, con
 }
 
 
-static int valid_for_protocol(const struct PluginCodec_Definition *, void *, const char *, void * parm, unsigned * parmLen)
-{
-  if (parmLen == NULL || parm == NULL || *parmLen != sizeof(char *))
-    return 0;
-
-  return (STRCMPI((const char *)parm, "h.323") == 0 ||
-          STRCMPI((const char *)parm, "h323") == 0) ? 1 : 0;
-
-}
-
 PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
 
-static PluginCodec_ControlDefn h323EncoderControls[] = {
-  { PLUGINCODEC_CONTROL_VALID_FOR_PROTOCOL,    valid_for_protocol },
-  { PLUGINCODEC_CONTROL_GET_CODEC_OPTIONS,     get_codec_options },
-  { PLUGINCODEC_CONTROL_TO_NORMALISED_OPTIONS, to_normalised_options },
-  { PLUGINCODEC_CONTROL_TO_CUSTOMISED_OPTIONS, to_customised_options },
-  { PLUGINCODEC_CONTROL_FREE_CODEC_OPTIONS,    free_codec_options },
-  { PLUGINCODEC_CONTROL_SET_CODEC_OPTIONS,     encoder_set_options },
-  { PLUGINCODEC_CONTROL_GET_OUTPUT_DATA_SIZE,  encoder_get_output_data_size },
-  PLUGINCODEC_CONTROL_LOG_FUNCTION_INC
-  { NULL }
-};
-
-static PluginCodec_ControlDefn h323DecoderControls[] = {
-  { PLUGINCODEC_CONTROL_VALID_FOR_PROTOCOL,    valid_for_protocol },
-  { PLUGINCODEC_CONTROL_GET_CODEC_OPTIONS,     get_codec_options },
-  { PLUGINCODEC_CONTROL_TO_NORMALISED_OPTIONS, to_normalised_options },
-  { PLUGINCODEC_CONTROL_TO_CUSTOMISED_OPTIONS, to_customised_options },
-  { PLUGINCODEC_CONTROL_FREE_CODEC_OPTIONS,    free_codec_options },
-  { PLUGINCODEC_CONTROL_SET_CODEC_OPTIONS,     decoder_set_options },
-  { PLUGINCODEC_CONTROL_GET_OUTPUT_DATA_SIZE,  decoder_get_output_data_size },
-  { NULL }
-};
-
 static PluginCodec_ControlDefn EncoderControls[] = {
   { PLUGINCODEC_CONTROL_GET_CODEC_OPTIONS,     get_codec_options },
   { PLUGINCODEC_CONTROL_TO_NORMALISED_OPTIONS, to_normalised_options },
@@ -803,24 +764,6 @@ static struct PluginCodec_Option const maxRxFrameWidth =
 static struct PluginCodec_Option const maxRxFrameHeight =
   { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT, true, PluginCodec_NoMerge, "288", NULL, NULL, 0, "144", "288"  };
 
-static struct PluginCodec_Option const minRxFrameWidthQCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH,  true, PluginCodec_NoMerge, "176", NULL, NULL, 0, "176", "176" };
-static struct PluginCodec_Option const minRxFrameHeightQCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT, true, PluginCodec_NoMerge, "144", NULL, NULL, 0, "144", "144"  };
-static struct PluginCodec_Option const maxRxFrameWidthQCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH,  true, PluginCodec_NoMerge, "176", NULL, NULL, 0, "176", "176"  };
-static struct PluginCodec_Option const maxRxFrameHeightQCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT, true, PluginCodec_NoMerge, "144", NULL, NULL, 0, "144", "144"  };
-
-static struct PluginCodec_Option const minRxFrameWidthCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH,  true, PluginCodec_NoMerge, "352", NULL, NULL, 0, "352", "352" };
-static struct PluginCodec_Option const minRxFrameHeightCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT, true, PluginCodec_NoMerge, "288", NULL, NULL, 0, "288", "288"  };
-static struct PluginCodec_Option const maxRxFrameWidthCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH,  true, PluginCodec_NoMerge, "352", NULL, NULL, 0, "352", "352"  };
-static struct PluginCodec_Option const maxRxFrameHeightCIF =
-  { PluginCodec_IntegerOption, PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT, true, PluginCodec_NoMerge, "288", NULL, NULL, 0, "288", "288"  };
-
 static struct PluginCodec_Option const qcifMPI =
 {
   PluginCodec_IntegerOption,            // Option type
@@ -866,26 +809,6 @@ static struct PluginCodec_Option const annexD =
   "0"                                   // FMTP default value
 };
 
-static struct PluginCodec_Option const * const qcifOptionTable[] = {
-  &qcifMPI,
-  &annexD,
-  &minRxFrameWidthQCIF,
-  &minRxFrameHeightQCIF,
-  &maxRxFrameWidthQCIF,
-  &maxRxFrameHeightQCIF,
-  NULL
-};
-
-static struct PluginCodec_Option const * const cifOptionTable[] = {
-  &cifMPI,
-  &annexD,
-  &minRxFrameWidthCIF,
-  &minRxFrameHeightCIF,
-  &maxRxFrameWidthCIF,
-  &maxRxFrameHeightCIF,
-  NULL
-};
-
 static struct PluginCodec_Option const * const xcifOptionTable[] = {
   &qcifMPI,
   &cifMPI,
@@ -908,7 +831,7 @@ static struct PluginCodec_Definition h261CodecDefn[] =
     PluginCodec_MediaTypeVideo |        // audio codec
     PluginCodec_RTPTypeExplicit,        // specified RTP type
 
-    h261Desc,                           // text decription
+    desc,                               // text decription
     YUV420PDesc,                        // source format
     h261Desc,                           // destination format
 
@@ -944,7 +867,7 @@ static struct PluginCodec_Definition h261CodecDefn[] =
     PluginCodec_MediaTypeVideo |        // audio codec
     PluginCodec_RTPTypeExplicit,        // specified RTP type
 
-    h261Desc,                           // text decription
+    desc,                               // text decription
     h261Desc,                           // source format
     YUV420PDesc,                        // destination format
 
