@@ -40,9 +40,9 @@
  *                 Craig Southeren (craigs@postincrement.com)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 29830 $
+ * $Revision: 33378 $
  * $Author: rjongbloed $
- * $Date: 2013-05-29 21:35:58 -0500 (Wed, 29 May 2013) $
+ * $Date: 2015-02-23 04:38:40 +0000 (Mon, 23 Feb 2015) $
  */
 
 #include "../common/platform.h"
@@ -57,6 +57,12 @@
 #include "rfc2190.h"
 #include "rfc2429.h"
 
+#if defined(_WIN32) || defined(_WIN32_WCE)
+  #define STRCMPI  _strcmpi
+#else
+  #define STRCMPI  strcasecmp
+#endif
+
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -64,7 +70,7 @@ class MY_CODEC { };
 
 PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
 
-static const char MyDescription[] = "H.263 (FFMPEG)";
+static const char MyDescription[] = "ITU-T H.263 - Video Codec (FFMPEG)";
 
 PLUGINCODEC_LICENSE(
   "Matthias Schneider, Craig Southeren"                         // source code author
@@ -313,20 +319,26 @@ public:
   }
 
 
-  virtual bool ToNormalised(OptionMap & original, OptionMap & changed)
+  virtual bool IsValidForProtocol(const char *) const
+	{
+		return true;
+	}
+
+
+  virtual bool ToNormalised(OptionMap & original, OptionMap & changed) const
   {
     return ClampToNormalised(original, changed);
   }
 
 
-  virtual bool ToCustomised(OptionMap & original, OptionMap & changed)
+  virtual bool ToCustomised(OptionMap & original, OptionMap & changed) const
   {
     return ClampToCustomised(original, changed);
   }
 }; 
 
-static H263_PluginMediaFormat MyMediaFormatInfo_RFC2190(H263FormatName,     H263EncodingName,     OptionTable_RFC2190);
-static H263_PluginMediaFormat MyMediaFormatInfo_RFC2429(H263plusFormatName, H263plusEncodingName, OptionTable_RFC22429);
+static H263_PluginMediaFormat const MyMediaFormatInfo_RFC2190(H263FormatName,     H263EncodingName,     OptionTable_RFC2190);
+static H263_PluginMediaFormat const MyMediaFormatInfo_RFC2429(H263plusFormatName, H263plusEncodingName, OptionTable_RFC22429);
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -405,11 +417,11 @@ class H263_Base_Encoder : public PluginVideoEncoder<MY_CODEC>, public FFMPEGCode
   typedef PluginVideoEncoder<MY_CODEC> BaseClass;
 
   public:
-    H263_Base_Encoder(const PluginCodec_Definition * defn, const char * prefix, EncodedFrame * packetizer)
+    H263_Base_Encoder(const PluginCodec_Definition * defn, const char * prefix, OpalPluginFrame * packetizer)
       : BaseClass(defn)
       , FFMPEGCodec(prefix, packetizer)
     { 
-      PTRACE(4, m_prefix, "Created encoder $Revision: 29830 $");
+      PTRACE(4, m_prefix, "Created encoder $Revision: 33378 $");
     }
 
 
@@ -476,18 +488,19 @@ class H263_Base_Encoder : public PluginVideoEncoder<MY_CODEC>, public FFMPEGCode
     }
 
 
+    virtual void ClampResolution()
+    {
+    }
+
+
     virtual bool OnChangedOptions()
     {
       CloseCodec();
 
-      if (!m_fullFrame->SetResolution(m_width, m_height)) {
-        PTRACE(1, m_prefix, "Unable to allocate memory for packet buffer");
-        return false;
-      }
+      ClampResolution();
 
       SetResolution(m_width, m_height);
       SetEncoderOptions(m_frameTime, m_maxBitRate, m_maxRTPSize, m_tsto, m_keyFramePeriod);
-      m_fullFrame->SetMaxPayloadSize(m_context->rtp_payload_size);
       PTRACE(4, m_prefix, "Packetization is " << m_fullFrame->GetName());
 
       #define CODEC_TRACER_FLAG(tracer, flag) \
@@ -525,19 +538,22 @@ class H263_Base_Encoder : public PluginVideoEncoder<MY_CODEC>, public FFMPEGCode
       if (!EncodeVideoPacket(PluginCodec_RTP(fromPtr, fromLen), dstRTP, flags))
         return false;
 
-      toLen = dstRTP.GetHeaderSize() + dstRTP.GetPayloadSize();
+      toLen = (unsigned)(dstRTP.GetHeaderSize() + dstRTP.GetPayloadSize());
       return true;
     }
 
 
-    bool GetStatistics(char * stats, unsigned maxSize)
+    virtual int GetStatistics(char * bufferPtr, unsigned bufferSize)
     {
       if (m_picture == NULL)
-        return false;
+        return -1;
 
-      snprintf(stats, maxSize, "Quality=%i\n", m_picture->quality);
-      stats[maxSize-1] = '\0';
-      return true;
+      size_t len = BaseClass::GetStatistics(bufferPtr, bufferSize);
+
+      if (m_picture->quality >= 0 && len < bufferSize)
+        len += snprintf(bufferPtr+len, bufferSize-len, "Quality=%u\n", m_picture->quality);
+
+      return len;
     }
 };
 
@@ -555,15 +571,13 @@ class H263_RFC2190_Encoder : public H263_Base_Encoder
 
     bool Construct()
     {
-      if (!InitEncoder(CODEC_ID_H263))
+      if (!InitEncoder(AV_CODEC_ID_H263))
         return false;
 
     #if LIBAVCODEC_RTP_MODE
       m_context->rtp_mode = 1;
     #endif
 
-      m_context->rtp_payload_size = PluginCodec_RTP_MaxPayloadSize;
-
     #ifdef CODEC_FLAG_H263P_UMV
       m_context->flags &= ~CODEC_FLAG_H263P_UMV;
     #endif
@@ -582,6 +596,34 @@ class H263_RFC2190_Encoder : public H263_Base_Encoder
 
       return true;
     }
+
+
+    /// Get options that are "active" and may be different from the last SetOptions() call.
+    virtual bool GetActiveOptions(PluginCodec_OptionMap & options)
+    {
+      if (!H263_Base_Encoder::GetActiveOptions(options))
+        return false;
+
+      options.SetUnsigned(this->m_width,  PLUGINCODEC_OPTION_FRAME_WIDTH);
+      options.SetUnsigned(this->m_height, PLUGINCODEC_OPTION_FRAME_HEIGHT);
+      return true;
+    }
+
+
+    virtual void ClampResolution()
+    {
+      int i;
+      for (i = 0; i < StandardResolution::Count; i++) {
+        if (m_width == StandardResolutions[i].width && m_height == StandardResolutions[i].height)
+          return;
+        if (m_width < StandardResolutions[i].width || m_height < StandardResolutions[i].height)
+          break;
+      }
+      if (i > 0)
+        --i;
+      m_width = StandardResolutions[i].width;
+      m_height = StandardResolutions[i].height;
+    }
 };
 
 
@@ -598,7 +640,7 @@ class H263_RFC2429_Encoder : public H263_Base_Encoder
 
     bool Construct()
     {
-      return InitEncoder(CODEC_ID_H263P);
+      return InitEncoder(AV_CODEC_ID_H263P);
     }
 };
 
@@ -610,18 +652,18 @@ class H263_Base_Decoder : public PluginVideoDecoder<MY_CODEC>, public FFMPEGCode
   typedef PluginVideoDecoder<MY_CODEC> BaseClass;
 
   public:
-    H263_Base_Decoder(const PluginCodec_Definition * defn, const char * prefix, EncodedFrame * depacketizer)
+    H263_Base_Decoder(const PluginCodec_Definition * defn, const char * prefix, OpalPluginFrame * depacketizer)
       : BaseClass(defn)
       , FFMPEGCodec(prefix, depacketizer)
     {
-      PTRACE(4, m_prefix, "Created decoder $Revision: 29830 $");
+      PTRACE(4, m_prefix, "Created decoder $Revision: 33378 $");
     }
 
 
     bool Construct()
     {
       m_fullFrame->Reset();
-      return InitDecoder(CODEC_ID_H263) && OpenCodec();
+      return InitDecoder(AV_CODEC_ID_H263) && OpenCodec();
     }
 
 
@@ -656,21 +698,10 @@ class H263_Base_Decoder : public PluginVideoDecoder<MY_CODEC>, public FFMPEGCode
         return true;
 
       PluginCodec_RTP out(toPtr, toLen);
-      toLen = OutputImage(m_picture->data, m_picture->linesize, m_context->width, m_context->height, out, flags);
+      toLen = OutputImage(m_picture->data, m_picture->linesize, PICTURE_WIDTH, PICTURE_HEIGHT, out, flags);
 
       return true;
     }
-
-
-    bool GetStatistics(char * stats, size_t maxSize)
-    {
-      if (m_picture == NULL)
-        return false;
-
-      snprintf(stats, maxSize, "Quality=%i\n", m_picture->quality);
-      stats[maxSize-1] = '\0';
-      return true;
-    }
 };
 
 
