@@ -30,7 +30,7 @@
 class PluginCodec_RTP;
 
 
-class RFC2190EncodedFrame : public FFMPEGCodec::EncodedFrame
+class RFC2190EncodedFrame : public OpalPluginFrame
 {
   public:
     RFC2190EncodedFrame();
@@ -46,8 +46,8 @@ class RFC2190Packetizer : public RFC2190EncodedFrame
   public:
     RFC2190Packetizer();
 
+    virtual void SetMaxPayloadSize(size_t size);
     virtual bool Reset(size_t len = 0);
-    virtual bool SetResolution(unsigned width, unsigned height);
 
     virtual bool GetPacket(PluginCodec_RTP & rtp, unsigned & flags);
     virtual bool AddPacket(const PluginCodec_RTP & rtp, unsigned & flags);
