@@ -22,8 +22,6 @@
 
 #include "rfc2429.h"
 
-#include <codec/opalplugin.hpp>
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -109,7 +107,7 @@ uint32_t Bitstream::PeekBits(uint32_t numBits)
 {
     uint8_t i;
     uint32_t result = 0;
-    uint32_t offset = m_data.pos / 8;
+    size_t offset = m_data.pos / 8;
     uint8_t  offsetBits = (uint8_t)(m_data.pos % 8);
     if (((m_data.len << 3) - m_ebits - m_sbits) < (m_data.pos  + numBits)) {
       PTRACE(2, "RFC2429",
@@ -140,7 +138,7 @@ void Bitstream::PutBits(uint32_t posBits, uint32_t numBits, uint32_t value)
 {
   uint8_t i;
   posBits += m_sbits;
-  uint32_t offset = m_data.pos / 8;
+  size_t offset = m_data.pos / 8;
   uint8_t  offsetBits = (uint8_t)(m_data.pos % 8);
   static const uint8_t maskClear[8] = {
     0x7f, 0xbf, 0xdf, 0xef,
@@ -170,7 +168,7 @@ void Bitstream::SetPos(uint32_t pos)
   m_data.pos = pos;
 }
 
-uint32_t Bitstream::GetPos()
+size_t Bitstream::GetPos()
 {
   return (m_data.pos);
 }
@@ -197,7 +195,7 @@ bool RFC2429Frame::GetPacket(PluginCodec_RTP & frame, unsigned int & flags)
       if (m_buffer[i-1] == 0 && m_buffer[i] == 0)
         m_startCodes.push_back(i);
     }  
-    unsigned requiredPackets = (m_length+m_maxPayloadSize-1)/m_maxPayloadSize;
+    size_t requiredPackets = (m_length+m_maxPayloadSize-1)/m_maxPayloadSize;
     if (m_length > m_maxPayloadSize)
       m_minPayloadSize = m_length/requiredPackets;
     else
@@ -291,7 +289,7 @@ bool RFC2429Frame::AddPacket(const PluginCodec_RTP & packet, unsigned int & flag
     payloadPtr += headerPLEN;
   }
 
-  unsigned remBytes = payloadSize - headerPLEN - (headerV ? 3 : 2);
+  size_t remBytes = payloadSize - headerPLEN - (headerV ? 3 : 2);
 
   if (headerP) {
     PTRACE(6, GetName(), "Adding startcode of 2 bytes to frame of " << remBytes << " bytes");
@@ -318,6 +316,12 @@ bool RFC2429Frame::AddPacket(const PluginCodec_RTP & packet, unsigned int & flag
 }
 
 
+void RFC2429Frame::SetMaxPayloadSize(size_t size)
+{
+  m_maxPayloadSize = size - 2; // Allow for header
+}
+
+
 bool RFC2429Frame::Reset(size_t len)
 {
   m_packetizationOffset = 0;
@@ -325,7 +329,7 @@ bool RFC2429Frame::Reset(size_t len)
   m_customClock = false;
   m_startCodes.clear();
 
-  return FFMPEGCodec::EncodedFrame::Reset(len);
+  return OpalPluginFrame::Reset(len);
 }
 
 
@@ -448,8 +452,9 @@ size_t RFC2429Frame::ParseHeader(uint8_t* headerPtr, size_t headerMaxLen)
 
     if ((PTCODE==3) || (PTCODE==4) || (PTCODE==5)) {
       PTRACE(6, GetName(), "Header\tELNUM: " << headerBits.GetBits(4));
-      if (UFEP==1)
+      if (UFEP==1) {
         PTRACE(6, GetName(), "Header\tRLNUM: " << headerBits.GetBits(4));
+      }
     }
 
     if (RPS) {
@@ -475,9 +480,10 @@ size_t RFC2429Frame::ParseHeader(uint8_t* headerPtr, size_t headerMaxLen)
     }
   }	
 
-  if (PB)
+  if (PB) {
     PTRACE(6, GetName(), "Header\tTRB: " << headerBits.GetBits (3)                     // TRB
                     << " DBQUANT: " << headerBits.GetBits (2));                   // DQUANT
+  }
 
   while (headerBits.GetBits (1)) {                                                // PEI bit
     PTRACE(6, GetName(), "Header\tPSUPP: " << headerBits.GetBits (8));                 // PSPARE bits 
