@@ -36,7 +36,7 @@
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
+#include "plugin_config.h"
 #endif
 
 #include <codec/opalplugin.hpp>
