@@ -21,33 +21,73 @@
  *
  * Contributor(s): Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 30001 $
+ * $Revision: 33861 $
  * $Author: rjongbloed $
- * $Date: 2013-06-19 21:05:15 -0500 (Wed, 19 Jun 2013) $
+ * $Date: 2015-08-09 18:58:22 +0100 (Sun, 09 Aug 2015) $
  */
 
-#ifndef PLUGIN_CODEC_DLL_EXPORTS
-#include "plugin-config.h"
-#endif
+#include "../common/platform.h"
+
+#define MY_CODEC VP8  // Name of codec (use C variable characters)
+
+#define OPAL_SIP 1
+
+#include "../../../src/codec/vp8mf_inc.cxx"
 
 #include <codec/opalplugin.hpp>
-
-#include <vector>
+#include "../common/critsect.h"
 
 #ifdef _MSC_VER
 #pragma warning(disable:4505)
+#define snprintf _snprintf
 #endif
 
+
+/* Building notes for Windows.
+   As of v1.3.0, no prebuilt libraries seem to be available, you must build it
+   from scratch, and to do that you must have Cygwin installed.
+
+   After unpacking the source into opal/plugins/video/VP8-WebM/vpx-vp8, run
+   cygwin and go to that directory. Then, depending on the Visual Studio
+   version you with to compile against, XX=10, XX=11 or XX=12
+
+      mkdir vsXX_32
+      cd vsXX_32
+      ../configure --target=x86-win32-vsXX
+      make
+      ./vpx.sln
+   And finally, batch build vpx Debug and Release.
+
+   And if desired (XX=11 or XX=12 only) again for 64-bit:
+      cd ..
+      mkdir vsXX_64
+      cd vsXX_64
+      ../configure --target=x86_64-win64-vsXX
+      make
+ */
+
 #define VPX_CODEC_DISABLE_COMPAT 1
 #include "vpx/vpx_encoder.h"
 #include "vpx/vp8cx.h"
 #include "vpx/vpx_decoder.h"
 #include "vpx/vp8dx.h"
 
+#include <vector>
+#include <stdio.h>
+
+
+/* Error concelment (handling missing data) does not appear to work. Or we are
+   not using it right. Either way disable it or we eventually get a Error 5 in
+   vpx_codec_decode - Bitstream not supported by this decoder.
+ */
+#ifdef VPX_CODEC_USE_ERROR_CONCEALMENT
+#undef VPX_CODEC_USE_ERROR_CONCEALMENT
+#endif
+
+#define HAS_OUTPUT_PARTITION (defined(VPX_CODEC_CAP_OUTPUT_PARTITION) && defined(VPX_CODEC_USE_OUTPUT_PARTITION))
 
 #define INCLUDE_OM_CUSTOM_PACKETIZATION 1
 
-#define MY_CODEC_LOG "VP8"
 class VP8_CODEC { };
 
 PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
@@ -78,34 +118,32 @@ PLUGINCODEC_LICENSE(
 
 ///////////////////////////////////////////////////////////////////////////////
 
-const unsigned MaxBitRate = 16000000;
-
 static struct PluginCodec_Option const MaxFR =
 {
   PluginCodec_IntegerOption,          // Option type
-  "max-fr",                           // User visible name
+  MaxFrameRateName,                   // User visible name
   false,                              // User Read/Only flag
   PluginCodec_MinMerge,               // Merge mode
-  "30",                               // Initial value
-  "max-fr",                           // FMTP option name
-  "30",                               // FMTP default value
+  STRINGIZE(MAX_FR_SDP),              // Initial value
+  MaxFrameRateSDP,                    // FMTP option name
+  STRINGIZE(MAX_FR_SDP),              // FMTP default value
   0,                                  // H.245 generic capability code and bit mask
-  "1",                                // Minimum value
-  "30"                                // Maximum value
+  STRINGIZE(MIN_FR_SDP),              // Minimum value
+  STRINGIZE(MAX_FR_SDP)               // Maximum value
 };
 
 static struct PluginCodec_Option const MaxFS =
 {
   PluginCodec_IntegerOption,          // Option type
-  "max-fs",                           // User visible name
+  MaxFrameSizeName,                   // User visible name
   false,                              // User Read/Only flag
   PluginCodec_MinMerge,               // Merge mode
-  "0",                                // Initial value
-  "max-fs",                           // FMTP option name
-  "0",                                // FMTP default value
+  STRINGIZE(MAX_FS_SDP),              // Initial value
+  MaxFrameSizeSDP,                    // FMTP option name
+  STRINGIZE(MAX_FS_SDP),              // FMTP default value
   0,                                  // H.245 generic capability code and bit mask
-  "48",                               // Minimum value
-  "65535"                             // Maximum value
+  STRINGIZE(MIN_FS_SDP),              // Minimum value
+  STRINGIZE(MAX_FS_SDP)               // Maximum value
 };
 
 #if INCLUDE_OM_CUSTOM_PACKETIZATION
@@ -138,11 +176,11 @@ static struct PluginCodec_Option const TemporalSpatialTradeOff =
 static struct PluginCodec_Option const SpatialResampling =
 {
   PluginCodec_BoolOption,             // Option type
-  "Spatial Resampling",               // User visible name
+  SpatialResamplingsName,             // User visible name
   false,                              // User Read/Only flag
   PluginCodec_AndMerge,               // Merge mode
   "0",                                // Initial value
-  "dynamicres",                       // FMTP option name
+  SpatialResamplingsSDP,              // FMTP option name
   "0",                                // FMTP default value
   0                                   // H.245 generic capability code and bit mask
 };
@@ -150,7 +188,7 @@ static struct PluginCodec_Option const SpatialResampling =
 static struct PluginCodec_Option const SpatialResamplingUp =
 {
   PluginCodec_IntegerOption,          // Option type
-  "Spatial Resampling Up",            // User visible name
+  SpatialResamplingUpName,            // User visible name
   false,                              // User Read/Only flag
   PluginCodec_AlwaysMerge,            // Merge mode
   "100",                              // Initial value
@@ -164,7 +202,7 @@ static struct PluginCodec_Option const SpatialResamplingUp =
 static struct PluginCodec_Option const SpatialResamplingDown =
 {
   PluginCodec_IntegerOption,          // Option type
-  "Spatial Resampling Down",          // User visible name
+  SpatialResamplingDownName,          // User visible name
   false,                              // User Read/Only flag
   PluginCodec_AlwaysMerge,            // Merge mode
   "0",                                // Initial value
@@ -175,36 +213,40 @@ static struct PluginCodec_Option const SpatialResamplingDown =
   "100"                               // Maximum value
 };
 
-static struct PluginCodec_Option const PictureID =
+static struct PluginCodec_Option const PictureIDSize =
 {
   PluginCodec_EnumOption,             // Option type
-  "Picture ID Size",                  // User visible name
+  PictureIDSizeName,                  // User visible name
   false,                              // User Read/Only flag
   PluginCodec_AlwaysMerge,            // Merge mode
-  "Word",                             // Initial value
+  "None",                             // Initial value
   NULL,                               // FMTP option name
   NULL,                               // FMTP default value
   0,                                  // H.245 generic capability code and bit mask
   "None:Byte:Word"                    // Enumeration
 };
 
+#if HAS_OUTPUT_PARTITION
 static struct PluginCodec_Option const OutputPartition =
 {
   PluginCodec_BoolOption,             // Option type
   "Output Partition",                 // User visible name
   false,                              // User Read/Only flag
   PluginCodec_AndMerge,               // Merge mode
-  "1",                                // Initial value
+  "0",                                // Initial value
   NULL,                               // FMTP option name
   NULL,                               // FMTP default value
   0                                   // H.245 generic capability code and bit mask
 };
+#endif
 
 static struct PluginCodec_Option const * OptionTableRFC[] = {
   &MaxFR,
   &MaxFS,
-  &PictureID,
+  &PictureIDSize,
+#if HAS_OUTPUT_PARTITION
   &OutputPartition,
+#endif
   &TemporalSpatialTradeOff,
   &SpatialResampling,
   &SpatialResamplingUp,
@@ -232,21 +274,13 @@ class VP8Format : public PluginCodec_VideoFormat<VP8_CODEC>
   private:
     typedef PluginCodec_VideoFormat<VP8_CODEC> BaseClass;
 
-  protected:
-    bool m_partition;
-
   public:
     VP8Format(const char * formatName, const char * payloadName, const char * description, OptionsTable options)
       : BaseClass(formatName, payloadName, description, MaxBitRate, options)
-      , m_partition(false)
     {
 #ifdef VPX_CODEC_USE_ERROR_CONCEALMENT
       if ((vpx_codec_get_caps(vpx_codec_vp8_dx()) & VPX_CODEC_CAP_ERROR_CONCEALMENT) != 0)
         m_flags |= PluginCodec_ErrorConcealment; // Prevent video update request on packet loss
-#endif
-#ifdef VPX_CODEC_CAP_OUTPUT_PARTITION
-      if ((vpx_codec_get_caps(vpx_codec_vp8_dx()) & VPX_CODEC_CAP_OUTPUT_PARTITION) != 0)
-        m_partition = true;
 #endif
     }
 };
@@ -256,52 +290,30 @@ class VP8FormatRFC : public VP8Format
 {
   public:
     VP8FormatRFC()
-      : VP8Format("VP8-WebM", "VP8", "VP8 Video Codec (RFC)", OptionTableRFC)
+      : VP8Format(VP8FormatName, VP8EncodingName, "VP8 Video Codec (RFC)", OptionTableRFC)
     {
     }
 
 
-    virtual bool ToNormalised(OptionMap & original, OptionMap & changed)
+    virtual bool IsValidForProtocol(const char * protocol) const
     {
-      OptionMap::iterator it = original.find(MaxFS.m_name);
-      if (it != original.end() && !it->second.empty()) {
-        unsigned maxFrameSize = String2Unsigned(it->second);
-        ClampResolution(m_maxWidth, m_maxHeight, maxFrameSize);
-        Change(maxFrameSize,  original, changed, MaxFS.m_name);
-        ClampMax(m_maxWidth,  original, changed, PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH);
-        ClampMax(m_maxHeight, original, changed, PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);
-        ClampMax(m_maxWidth,  original, changed, PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH);
-        ClampMax(m_maxHeight, original, changed, PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT);
-      }
-
-      it = original.find(MaxFR.m_name);
-      if (it != original.end() && !it->second.empty())
-        ClampMin(PLUGINCODEC_VIDEO_CLOCK/String2Unsigned(it->second), original, changed, PLUGINCODEC_OPTION_FRAME_TIME);
-
-      if (!m_partition)
-        ClampMax(0, original, changed, OutputPartition.m_name);
-
-      return true;
+      return strcasecmp(protocol, PLUGINCODEC_OPTION_PROTOCOL_SIP) == 0;
     }
 
 
-    virtual bool ToCustomised(OptionMap & original, OptionMap & changed)
+    virtual bool ToNormalised(OptionMap & original, OptionMap & changed) const
     {
-      Change(GetMacroBlocks(original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH),
-                            original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT)),
-             original, changed, MaxFS.m_name);
+      return MyToNormalised(original, changed);
+    }
 
-      Change(PLUGINCODEC_VIDEO_CLOCK/original.GetUnsigned(PLUGINCODEC_OPTION_FRAME_TIME),
-             original, changed, MaxFR.m_name);
 
-      if (!m_partition)
-        ClampMax(0, original, changed, OutputPartition.m_name);
-
-      return true;
+    virtual bool ToCustomised(OptionMap & original, OptionMap & changed) const
+    {
+      return MyToCustomised(original, changed);
     }
 };
 
-static VP8FormatRFC VP8MediaFormatInfoRFC;
+static VP8FormatRFC const VP8MediaFormatInfoRFC;
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -317,7 +329,13 @@ class VP8FormatOM : public VP8Format
     }
 
 
-    virtual bool ToNormalised(OptionMap & original, OptionMap & changed)
+    virtual bool IsValidForProtocol(const char * protocol) const
+    {
+      return strcasecmp(protocol, PLUGINCODEC_OPTION_PROTOCOL_SIP) == 0;
+    }
+
+
+    virtual bool ToNormalised(OptionMap & original, OptionMap & changed) const
     {
       OptionMap::iterator it = original.find(MaxFrameSizeOM.m_name);
       if (it != original.end() && !it->second.empty()) {
@@ -338,7 +356,7 @@ class VP8FormatOM : public VP8Format
     }
 
 
-    virtual bool ToCustomised(OptionMap & original, OptionMap & changed)
+    virtual bool ToCustomised(OptionMap & original, OptionMap & changed) const
     {
       unsigned maxWidth = original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH);
       unsigned maxHeight = original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);
@@ -349,7 +367,7 @@ class VP8FormatOM : public VP8Format
     }
 };
 
-static VP8FormatOM VP8MediaFormatInfoOM;
+static VP8FormatOM const VP8MediaFormatInfoOM;
 
 #endif // INCLUDE_OM_CUSTOM_PACKETIZATION
 
@@ -395,6 +413,8 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
     const vpx_codec_cx_pkt_t * m_packet;
     size_t                     m_offset;
 
+    CriticalSection m_mutex;
+
   public:
     VP8Encoder(const PluginCodec_Definition * defn)
       : BaseClass(defn)
@@ -432,7 +452,7 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
       if (!OnChangedOptions())
         return false;
 
-      PTRACE(4, MY_CODEC_LOG, "Encoder opened: " << vpx_codec_version_str() << ", revision $Revision: 30001 $");
+      PTRACE(4, MY_CODEC_LOG, "Encoder opened: " << vpx_codec_version_str() << ", revision $Revision: 33861 $");
       return true;
     }
 
@@ -454,6 +474,8 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
 
     virtual bool OnChangedOptions()
     {
+      WaitAndSignal lock(m_mutex);
+
       m_config.kf_mode = VPX_KF_AUTO;
       if (m_keyFramePeriod != 0)
         m_config.kf_min_dist = m_config.kf_max_dist = m_keyFramePeriod;
@@ -472,6 +494,11 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
       if (m_config.g_w == m_width && m_config.g_h == m_height)
         return !IS_ERROR(vpx_codec_enc_config_set, (&m_codec, &m_config));
 
+      if (((m_width|m_height) & 1) != 0) {
+        PTRACE(1, MY_CODEC_LOG, "Odd width or height provided: " << m_width << 'x' << m_height);
+        return false;
+      }
+
       m_config.g_w = m_width;
       m_config.g_h = m_height;
       vpx_codec_destroy(&m_codec);
@@ -484,6 +511,8 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
       if (m_packet != NULL)
         return false;
 
+      WaitAndSignal lock(m_mutex);
+
       while ((m_packet = vpx_codec_get_cx_data(&m_codec, &m_iterator)) != NULL) {
         if (m_packet->kind == VPX_CODEC_CX_FRAME_PKT)
           return false;
@@ -494,6 +523,21 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
     }
 
 
+    virtual int GetStatistics(char * bufferPtr, unsigned bufferSize)
+    {
+      size_t len = BaseClass::GetStatistics(bufferPtr, bufferSize);
+
+      WaitAndSignal lock(m_mutex);
+
+      int quality = -1;
+      IS_ERROR(vpx_codec_control_VP8E_GET_LAST_QUANTIZER_64,(&m_codec, VP8E_GET_LAST_QUANTIZER_64, &quality));
+      if (quality >= 0 && len < bufferSize)
+        len += snprintf(bufferPtr+len, bufferSize-len, "Quality=%u\n", quality);
+
+      return len;
+    }
+
+
     virtual bool Transcode(const void * fromPtr,
                              unsigned & fromLen,
                                  void * toPtr,
@@ -509,11 +553,14 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
                  m_width << 'x' << m_height << " to " << video->width << 'x' << video->height);
           m_width = video->width;
           m_height = video->height;
-          OnChangedOptions();
+          if (!OnChangedOptions())
+            return false;
         }
 
         vpx_image_t image;
-        vpx_img_wrap(&image, VPX_IMG_FMT_I420, video->width, video->height, 4, srcRTP.GetVideoFrameData());
+        vpx_img_wrap(&image, VPX_IMG_FMT_I420, video->width, video->height, 2, srcRTP.GetVideoFrameData());
+
+        WaitAndSignal lock(m_mutex);
 
         if (IS_ERROR(vpx_codec_encode, (&m_codec, &image,
                                         srcRTP.GetTimestamp(), m_frameTime,
@@ -528,7 +575,7 @@ class VP8Encoder : public PluginVideoEncoder<VP8_CODEC>
 
       PluginCodec_RTP dstRTP(toPtr, toLen);
       Packetise(dstRTP);
-      toLen = dstRTP.GetPacketSize();
+      toLen = (unsigned)dstRTP.GetPacketSize();
 
       if (m_offset >= m_packet->data.frame.sz) {
         flags |= PluginCodec_ReturnCoderLastFrame;
@@ -562,7 +609,7 @@ class VP8EncoderRFC : public VP8Encoder
 
     virtual bool SetOption(const char * optionName, const char * optionValue)
     {
-      if (strcasecmp(optionName, PictureID.m_name) == 0) {
+      if (strcasecmp(optionName, PictureIDSize.m_name) == 0) {
         if (strcasecmp(optionValue, "Byte") == 0) {
           if (m_pictureIdSize != 0x80) {
             m_pictureIdSize = 0x80;
@@ -591,8 +638,9 @@ class VP8EncoderRFC : public VP8Encoder
         return false;
       }
 
-#ifdef VPX_CODEC_USE_OUTPUT_PARTITION
-      if (strcasecmp(optionName, OutputPartition.m_name) == 0)
+#if HAS_OUTPUT_PARTITION
+      if (strcasecmp(optionName, OutputPartition.m_name) == 0 &&
+                    (vpx_codec_get_caps(vpx_codec_vp8_dx()) & VPX_CODEC_CAP_OUTPUT_PARTITION) != 0)
         return SetOptionBit(m_initFlags, VPX_CODEC_USE_OUTPUT_PARTITION, optionValue);
 #endif
 
@@ -609,9 +657,9 @@ class VP8EncoderRFC : public VP8Encoder
       if (m_offset == 0)
         rtp[0] |= 0x10; // Add S bit if start of partition
 
-#ifdef VPX_CODEC_USE_OUTPUT_PARTITION
-      if (m_packet->data.frame.partition_id >= 0)
-        rtp[0] |= (uint8_t)(m_packet->data.frame.partition_id&0x0f);
+#if HAS_OUTPUT_PARTITION
+      if (m_packet->data.frame.partition_id > 0 && m_packet->data.frame.partition_id <= 8)
+        rtp[0] |= (uint8_t)m_packet->data.frame.partition_id;
 #endif
 
       if ((m_packet->data.frame.flags&VPX_FRAME_IS_DROPPABLE) != 0)
@@ -703,8 +751,10 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
     vpx_codec_flags_t    m_flags;
     vpx_codec_iter_t     m_iterator;
     std::vector<uint8_t> m_fullFrame;
+    bool                 m_firstFrame;
     bool                 m_intraFrame;
     bool                 m_ignoreTillKeyFrame;
+    unsigned             m_consecutiveErrors;
 
   public:
     VP8Decoder(const PluginCodec_Definition * defn)
@@ -712,8 +762,10 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
       , m_iface(vpx_codec_vp8_dx())
       , m_flags(0)
       , m_iterator(NULL)
+      , m_firstFrame(true)
       , m_intraFrame(false)
       , m_ignoreTillKeyFrame(false)
+      , m_consecutiveErrors(0)
     {
       memset(&m_codec, 0, sizeof(m_codec));
       m_fullFrame.reserve(10000);
@@ -736,7 +788,7 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
       if (IS_ERROR(vpx_codec_dec_init, (&m_codec, m_iface, NULL, m_flags)))
         return false;
 
-      PTRACE(4, MY_CODEC_LOG, "Decoder opened: " << vpx_codec_version_str() << ", revision $Revision: 30001 $");
+      PTRACE(4, MY_CODEC_LOG, "Decoder opened: " << vpx_codec_version_str() << ", revision $Revision: 33861 $");
       return true;
     }
 
@@ -761,19 +813,19 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
     {
       vpx_image_t * image;
 
-      bool lostPackets = (flags & PluginCodec_CoderPacketLoss) == 0;
+      bool noLostPackets = (flags & PluginCodec_CoderPacketLoss) == 0;
 
       flags = m_intraFrame ? PluginCodec_ReturnCoderIFrame : 0;
 
-      if ((image = vpx_codec_get_frame(&m_codec, &m_iterator)) == NULL) {
-        /* Unless error concealment implementedm decoder has a problems with
-           missing data in the frame and can gets it;s knickers thorougly
+      if (m_firstFrame || (image = vpx_codec_get_frame(&m_codec, &m_iterator)) == NULL) {
+        /* Unless error concealment implemented, decoder has a problems with
+           missing data in the frame and can gets it's knickers thorougly
            twisted, so just ignore everything till next I-Frame. */
         if (BadDecode(flags,
 #ifdef VPX_CODEC_USE_ERROR_CONCEALMENT
                              (m_flags & VPX_CODEC_USE_ERROR_CONCEALMENT) != 0 ||
 #endif
-                             lostPackets))
+                             noLostPackets))
           return true;
 
         PluginCodec_RTP srcRTP(fromPtr, fromLen);
@@ -783,11 +835,18 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
         if (!srcRTP.GetMarker() || m_fullFrame.empty())
           return true;
 
-        vpx_codec_err_t err = vpx_codec_decode(&m_codec, &m_fullFrame[0], m_fullFrame.size(), NULL, 0);
+        vpx_codec_err_t err = vpx_codec_decode(&m_codec, &m_fullFrame[0], (unsigned)m_fullFrame.size(), NULL, 0);
         switch (err) {
           case VPX_CODEC_OK :
+            m_consecutiveErrors = 0;
             break;
 
+          case VPX_CODEC_UNSUP_BITSTREAM :
+            if (m_consecutiveErrors++ > 10) {
+              IsError(err, "vpx_codec_decode");
+              return false;
+            }
+
           // Non fatal errors
           case VPX_CODEC_UNSUP_FEATURE :
           case VPX_CODEC_CORRUPT_FRAME :
@@ -795,7 +854,7 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
             BadDecode(flags, false);
             return true;
 
-          default :
+          default:
             IsError(err, "vpx_codec_decode");
             return false;
         }
@@ -827,6 +886,8 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
         m_iterator = NULL;
         if ((image = vpx_codec_get_frame(&m_codec, &m_iterator)) == NULL)
           return true;
+
+        m_firstFrame = false;
       }
 
       if (image->fmt != VPX_IMG_FMT_I420) {
@@ -835,8 +896,7 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
       }
 
       PluginCodec_RTP dstRTP(toPtr, toLen);
-      OutputImage(image->planes, image->stride, image->d_w, image->d_h, dstRTP, flags);
-      toLen = dstRTP.GetPacketSize();
+      toLen = OutputImage(image->planes, image->stride, image->d_w, image->d_h, dstRTP, flags);
 
       if ((flags & PluginCodec_ReturnCoderLastFrame) != 0)
         m_intraFrame = false;
@@ -863,9 +923,13 @@ class VP8Decoder : public PluginVideoDecoder<VP8_CODEC>
 
 class VP8DecoderRFC : public VP8Decoder
 {
+  protected:
+    unsigned m_partitionID;
+
   public:
     VP8DecoderRFC(const PluginCodec_Definition * defn)
       : VP8Decoder(defn)
+      , m_partitionID(0)
     {
     }
 
@@ -875,7 +939,7 @@ class VP8DecoderRFC : public VP8Decoder
       if (rtp.GetPayloadSize() == 0)
         return true;
 
-      if (rtp.GetPayloadSize() < 3) {
+      if (rtp.GetPayloadSize() < 2) {
         PTRACE(3, MY_CODEC_LOG, "RTP packet far too small.");
         return false;
       }
@@ -902,19 +966,39 @@ class VP8DecoderRFC : public VP8Decoder
         return false;
       }
 
-      if ((rtp[0]&0x10) != 0 && !m_fullFrame.empty()) { // Check S bit
-        PTRACE(3, MY_CODEC_LOG, "Missing start to frame, ignoring till next key frame.");
-        return false;
-      }
-
       if (m_ignoreTillKeyFrame) {
-        // Key frame is S bit == 1, partID == 0 & P bit == 0
-        if ((rtp[0]&0x10) != 0x10 || (rtp[headerSize]&0x01) != 0)
+        // Key frame is S bit == 1, partID == 0 and P bit == 0
+        if ((rtp[0]&0x1f) != 0x10 || (rtp[headerSize]&0x01) != 0)
           return false;
         m_ignoreTillKeyFrame =  false;
         PTRACE(3, MY_CODEC_LOG, "Found next start of key frame.");
       }
 
+      if ((rtp[0]&0x10) != 0) { // Check S bit
+        unsigned partitionID = rtp[0] & 0xf;
+        if (partitionID != 0) {
+          ++m_partitionID;
+          if (m_partitionID != partitionID) {
+            PTRACE(3, MY_CODEC_LOG, "Missing partition "
+                   "(expected " << m_partitionID << " , got " << partitionID << "),"
+                   " ignoring till next key frame.");
+            return false;
+          }
+        }
+        else {
+          m_partitionID = 0;
+          if (!m_fullFrame.empty()) {
+            if ((rtp[headerSize] & 0x01) != 0) {
+              PTRACE(3, MY_CODEC_LOG, "Start bit seen, but not completed previous frame, ignoring till next key frame.");
+              return false;
+            }
+
+            PTRACE(3, MY_CODEC_LOG, "Start bit seen, but not completed previous frame, restarting key frame.");
+            m_fullFrame.clear();
+          }
+        }
+      }
+
       Accumulate(rtp.GetPayloadPtr()+headerSize, rtp.GetPayloadSize()-headerSize);
       return true;
     }
@@ -1038,10 +1122,13 @@ class VP8DecoderOM : public VP8Decoder
           case LandscapeDown :
             planeInfo[p].CopyLandscapeDown();
             break;
+
+          default :
+            return 0;
         }
       }
 
-      return rtp.GetPacketSize();
+      return (unsigned)rtp.GetPacketSize();
     }
 
 
