@@ -39,10 +39,18 @@
  */
 
 #include "ffmpeg.h"
+#include "critsect.h"
 
 #include <stdio.h>
 #include <iomanip>
 
+#ifndef PLUGIN_CODEC_DLL_EXPORTS
+#include "plugin_config.h"
+#endif
+
+
+static CriticalSection g_avcodec_mutex;
+
 
 #if PLUGINCODEC_TRACING
 static void logCallbackFFMPEG(void * avcl, int severity, const char* fmt , va_list arg)
@@ -83,7 +91,7 @@ static void logCallbackFFMPEG(void * avcl, int severity, const char* fmt , va_li
       (len == 2 && isxdigit(buffer[1])))
     level = 6;
 
-  if (avcl != NULL && strcmp((*(AVClass**)avcl)->class_name, "AVCodecContext") == 0)
+  if (avcl != NULL && strcmp((*(AVClass**)avcl)->class_name, "AVCodecContext") == 0 && static_cast<AVCodecContext *>(avcl)->opaque != NULL)
     static_cast<FFMPEGCodec *>(static_cast<AVCodecContext *>(avcl)->opaque)->ErrorCallback(level, buffer);
   else
     PTRACE(level, "FFMPEG", buffer);
@@ -93,24 +101,30 @@ static void logCallbackFFMPEG(void * avcl, int severity, const char* fmt , va_li
 
 ///////////////////////////////////////////////////////////////////////////////
 
-FFMPEGCodec::FFMPEGCodec(const char * prefix, EncodedFrame * fullFrame)
+FFMPEGCodec::FFMPEGCodec(const char * prefix, OpalPluginFrame * fullFrame)
   : m_prefix(prefix)
   , m_codec(NULL)
   , m_context(NULL)
   , m_picture(NULL)
-  , m_alignedInputYUV(NULL)
-  , m_alignedInputSize(0)
   , m_fullFrame(fullFrame)
+  , m_open(false)
   , m_errorCount(0)
+  , m_hadMissingPacket(false)
 {
-  avcodec_register_all();
+  g_avcodec_mutex.Wait();
 
-  av_init_packet(&m_packet);
+  avcodec_register_all();
 
 #if PLUGINCODEC_TRACING
   av_log_set_level(AV_LOG_DEBUG);
   av_log_set_callback(&logCallbackFFMPEG);
 #endif
+
+  g_avcodec_mutex.Signal();
+
+  av_init_packet(&m_packet);
+
+  m_alignedYUV[0] = m_alignedYUV[1] = m_alignedYUV[2] = NULL;
 }
 
 
@@ -120,14 +134,20 @@ FFMPEGCodec::~FFMPEGCodec()
 
   if (m_context != NULL)
     av_free(m_context);
-  if (m_picture != NULL)
+
+  if (m_picture != NULL) {
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(54, 0, 0)
     av_free(m_picture);
-#else
+#elif LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 0, 0)
     avcodec_free_frame(&m_picture);
+#else
+    av_frame_free(&m_picture);
 #endif
-  if (m_alignedInputYUV != NULL)
-    free(m_alignedInputYUV);
+  }
+
+  av_free(m_alignedYUV[0]);
+  av_free(m_alignedYUV[1]);
+  av_free(m_alignedYUV[2]);
 
   delete m_fullFrame;
 
@@ -137,7 +157,7 @@ FFMPEGCodec::~FFMPEGCodec()
 
 bool FFMPEGCodec::InitContext()
 {
-#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 25, 0)
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(53, 0, 0)
   m_context = avcodec_alloc_context2(m_codec->type);
 #else
   m_context = avcodec_alloc_context3(m_codec);
@@ -147,13 +167,17 @@ bool FFMPEGCodec::InitContext()
     return false;
   }
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 0, 0)
   m_picture = avcodec_alloc_frame();
+#else
+  m_picture = av_frame_alloc();
+#endif
   if (m_picture == NULL) {
     PTRACE(1, m_prefix, "Failed to allocate frame for encoder");
     return false;
   }
 
-  m_context->pix_fmt = PIX_FMT_YUV420P;
+  m_picture->format = m_context->pix_fmt = AV_PIX_FMT_YUV420P;
   m_context->workaround_bugs = FF_BUG_AUTODETECT;
 
   // debugging flags
@@ -179,9 +203,9 @@ static void StaticRTPCallBack(AVCodecContext * ctx, void * data, int size, int n
   static_cast<FFMPEGCodec *>(ctx->opaque)->GetEncodedFrame()->RTPCallBack(data, size, numMB);
 }
 
-bool FFMPEGCodec::InitEncoder(CodecID codecId)
+bool FFMPEGCodec::InitEncoder(AVCodecID codecId)
 {
-  PTRACE(5, m_prefix, "Opening encoder");
+  PTRACE(5, m_prefix, "Initialising encoder");
 
   m_codec = avcodec_find_encoder(codecId);
   if (m_codec == NULL) {
@@ -214,7 +238,7 @@ bool FFMPEGCodec::InitEncoder(CodecID codecId)
 }
 
 
-bool FFMPEGCodec::InitDecoder(CodecID codecId)
+bool FFMPEGCodec::InitDecoder(AVCodecID codecId)
 {
   if ((m_codec = avcodec_find_decoder(codecId)) == NULL) {
     PTRACE(1, m_prefix, "Codec not found for decoder");
@@ -226,7 +250,7 @@ bool FFMPEGCodec::InitDecoder(CodecID codecId)
 
   m_picture->quality = -1;
 
-#ifdef FF_ER_AGGRESSIVE
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(53, 0, 0)
   m_context->error_recognition = FF_ER_AGGRESSIVE;
 #endif
   m_context->error_concealment = FF_EC_GUESS_MVS | FF_EC_DEBLOCK;
@@ -242,7 +266,9 @@ bool FFMPEGCodec::OpenCodec()
     return false;
   }
 
-#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 25, 0)
+  g_avcodec_mutex.Wait();
+
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(53, 0, 0)
   int result = avcodec_open(m_context, m_codec);
 #else
   AVDictionary * options = NULL;
@@ -250,26 +276,34 @@ bool FFMPEGCodec::OpenCodec()
   av_dict_free(&options);
 #endif
 
+  g_avcodec_mutex.Signal();
+
   if (result < 0) {
-    PTRACE(1, m_prefix, "Failed to open codec");
+    PTRACE(1, m_prefix, "Failed to open codec \"" << m_codec->long_name << '"');
     return false;
   }
 
-  PTRACE(4, m_prefix, "Codec opened");
+  PTRACE(4, m_prefix, "Codec opened \"" << m_codec->long_name << '"');
+  m_open = true;
   return true;
 }
 
 
 void FFMPEGCodec::CloseCodec()
 {
-  if (m_context != NULL && m_context->codec != NULL)
+  if (m_open) {
+    PTRACE(4, m_prefix, "Closing codec \"" << m_codec->long_name << '"');
+    g_avcodec_mutex.Wait();
     avcodec_close(m_context);
+    g_avcodec_mutex.Signal();
+    m_open = false;
+  }
 }
 
 
 bool FFMPEGCodec::SetResolution(unsigned width, unsigned height)
 {
-  bool wasOpen = m_context->codec != NULL;
+  bool wasOpen = m_open;
   if (wasOpen) {
     PTRACE(3, m_prefix, "Resolution has changed - reopening codec");
     CloseCodec();
@@ -279,16 +313,33 @@ bool FFMPEGCodec::SetResolution(unsigned width, unsigned height)
     if (width > 352)
       m_context->flags &= ~CODEC_FLAG_EMU_EDGE; // Totally bizarre! FFMPEG crashes if on for CIF4
 
-    avcodec_set_dimensions(m_context, width, height);
+    m_context->width = width;
+    m_context->height = height;
+    m_context->coded_width = width;
+    m_context->coded_height = height;
   }
 
   if (m_picture != NULL) {
     // YUV420P input
+    m_picture->width = width;
+    m_picture->height = height;
     m_picture->linesize[0] = width;
     m_picture->linesize[1] = m_picture->linesize[2] = width/2;
+    av_free(m_alignedYUV[0]);
+    av_free(m_alignedYUV[1]);
+    av_free(m_alignedYUV[2]);
+
+    /* Allocate correctly aligned memory using av_malloc(), also pad out length
+       so don't overrrun the end of the allocated buffer.
+    */
+    size_t sz = ((width*height+15)/16)*16;
+    m_picture->data[0] = (uint8_t *)(m_alignedYUV[0] = av_malloc(sz));
+    sz = ((width*height/4+15)/16)*16;
+    m_picture->data[1] = (uint8_t *)(m_alignedYUV[1] = av_malloc(sz));
+    m_picture->data[2] = (uint8_t *)(m_alignedYUV[2] = av_malloc(sz));
   }
 
-  if (m_fullFrame != NULL && !m_fullFrame->SetResolution(width, height)) {
+  if (m_fullFrame != NULL && !m_fullFrame->SetSize(width*height*2)) {
     PTRACE(1, m_prefix, "Frame handler SetResolution failed");
     return false;
   }
@@ -299,7 +350,7 @@ bool FFMPEGCodec::SetResolution(unsigned width, unsigned height)
     return false;
   }
 
-  PTRACE(5, m_prefix, "Resolution set to " << width << 'x' << height);
+  PTRACE(4, m_prefix, "Resolution set to " << width << 'x' << height);
   return true;
 }
 
@@ -325,6 +376,7 @@ void FFMPEGCodec::SetEncoderOptions(unsigned frameTime,
   // no effect.
   m_context->rc_initial_buffer_occupancy = m_context->rc_buffer_size * 1/2;
 
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 0, 0)
   // And this is set to 1.
   // It seems to affect how aggressively the library will raise and lower
   // quantization to keep bandwidth constant. Except it's in reference to
@@ -334,6 +386,7 @@ void FFMPEGCodec::SetEncoderOptions(unsigned frameTime,
 
   // This is set to 0 in ffmpeg.c, the command-line utility.
   m_context->rc_initial_cplx = 0.0f;
+#endif
 
   // FFMPEG requires bit rate tolerance to be at least one frame size
   m_context->bit_rate_tolerance = maxBitRate/10;
@@ -349,11 +402,18 @@ void FFMPEGCodec::SetEncoderOptions(unsigned frameTime,
   m_context->max_qdiff = 10;  // was 3      // max q difference between frames
   m_context->qcompress = 0.5;               // qscale factor between easy & hard scenes (0.0-1.0)
 
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 0, 0)
   // Lagrange multipliers - this is how the context defaults do it:
   m_context->lmin = m_context->qmin * FF_QP2LAMBDA;
   m_context->lmax = m_context->qmax * FF_QP2LAMBDA; 
+#endif
 
-  m_context->rtp_payload_size = maxRTPSize;
+  if (m_fullFrame == NULL)
+    m_context->rtp_payload_size = maxRTPSize;
+  else {
+    m_fullFrame->SetMaxPayloadSize(maxRTPSize);
+    m_context->rtp_payload_size = (int)m_fullFrame->GetMaxPayloadSize(); // Might be adjusted to smaller value
+  }
 
   m_context->qmax = tsto;
   if (m_context->qmax <= m_context->qmin)
@@ -373,7 +433,6 @@ void FFMPEGCodec::SetEncoderOptions(unsigned frameTime,
   PTRACE(5, m_prefix, "qmin set to " << m_context->qmin);
   PTRACE(5, m_prefix, "qmax set to " << m_context->qmax);
   PTRACE(5, m_prefix, "payload size set to " << m_context->rtp_payload_size);
-
 }
 
 
@@ -407,28 +466,13 @@ bool FFMPEGCodec::EncodeVideoPacket(const PluginCodec_RTP & in, PluginCodec_RTP
 
   size_t planeSize = m_context->width*m_context->height;
 
-  // May need to copy to local buffer to guarantee 16 byte alignment
+  // Need to copy to local buffer to guarantee byte alignment
   uint8_t * yuv = OPAL_VIDEO_FRAME_DATA_PTR(header);
-  if ((((intptr_t)yuv)&0xf) != 0) {
-    size_t frameSize = planeSize*3/2;
-    size_t alignedSize = frameSize + 16;
-    if (m_alignedInputSize < alignedSize) {
-      if (m_alignedInputYUV != NULL)
-        free(m_alignedInputYUV);
-      if ((m_alignedInputYUV = (uint8_t *)malloc(alignedSize)) == NULL) {
-        PTRACE(1, m_prefix, "Unable to allocate memory for aligned buffer");
-        return false;
-      }
-      m_alignedInputSize = alignedSize;
-    }
-
-    yuv = m_alignedInputYUV + 16 - (((intptr_t)m_alignedInputYUV) & 0xf);
-    memcpy(yuv, OPAL_VIDEO_FRAME_DATA_PTR(header), frameSize);
-  }
-
-  m_picture->data[0] = yuv;
-  m_picture->data[1] = m_picture->data[0] + planeSize;
-  m_picture->data[2] = m_picture->data[1] + planeSize/4;
+  memcpy(m_picture->data[0], yuv, planeSize);
+  yuv += planeSize;
+  planeSize /= 4;
+  memcpy(m_picture->data[1], yuv, planeSize);
+  memcpy(m_picture->data[2], yuv + planeSize, planeSize);
 
   /*
   m_picture->pts = (int64_t)srcRTP.GetTimestamp()*m_context->time_base.den/m_context->time_base.num/PLUGINCODEC_VIDEO_CLOCK;
@@ -452,7 +496,10 @@ bool FFMPEGCodec::EncodeVideoPacket(const PluginCodec_RTP & in, PluginCodec_RTP
   if (result < 0)
     return false;
 
-  m_fullFrame->Reset(result);
+  if (!m_fullFrame->Reset(result)) {
+    PTRACE(2, m_prefix, "Encoding/Packetisation error: " << result << " bytes");
+    return false;
+  }
 
   if (m_fullFrame->IsIntraFrame())
     flags |= PluginCodec_ReturnCoderIFrame;
@@ -468,7 +515,7 @@ int FFMPEGCodec::EncodeVideoFrame(uint8_t * frame, size_t length, unsigned & fla
   int gotPacket = result > 0;
 #else
   m_packet.data = frame;
-  m_packet.size = length;
+  m_packet.size = (int)length;
   int gotPacket = 0;
   int result = avcodec_encode_video2(m_context, &m_packet, m_picture, &gotPacket);
 #endif
@@ -497,20 +544,38 @@ bool FFMPEGCodec::DecodeVideoPacket(const PluginCodec_RTP & in, unsigned & flags
     return false;
   }
 
+  // Because we cannot trust the decoder not to crash on missing packets, we throw away the whole frame
+  if (!m_hadMissingPacket && (flags & PluginCodec_CoderPacketLoss) != 0) {
+    PTRACE(3, m_prefix, "Decoder throwing away entire video frame due to packet loss");
+    m_hadMissingPacket = true;
+    if (m_fullFrame != NULL)
+        m_fullFrame->Reset();
+  }
+
   flags = 0;
 
+  if (m_hadMissingPacket) {
+    // Ignore packets till end of frame
+    if (in.GetMarker())
+      m_hadMissingPacket = false;
+    return true;
+  }
+
   if (m_fullFrame == NULL)
     return DecodeVideoFrame(in.GetPayloadPtr(), in.GetPayloadSize(), flags);
 
-  if (in.GetMarker())
-    flags |= PluginCodec_ReturnCoderLastFrame;
-
   if (in.GetPayloadSize() > 0 && !m_fullFrame->AddPacket(in, flags))
     return false;
 
-  if ((flags&PluginCodec_ReturnCoderLastFrame) == 0)
+  if (!in.GetMarker())
     return true;
 
+  if (in.GetPayloadSize() == 0 && m_fullFrame->GetLength() == 0) {
+    if (m_picture->data[0] != NULL)
+      flags |= PluginCodec_ReturnCoderLastFrame;
+    return true; // This happens if needed to make buffer bigger, already have frame to return
+  }
+
   bool result = DecodeVideoFrame(m_fullFrame->GetBuffer(), m_fullFrame->GetLength(), flags);
   m_fullFrame->Reset();
   return result;
@@ -523,22 +588,27 @@ bool FFMPEGCodec::DecodeVideoFrame(const uint8_t * frame, size_t length, unsigne
 
   int errorsBefore = m_errorCount;
 #ifdef FFMPEG_HAS_DECODE_ERROR_COUNT
-  errorsBefore += m_context->decode_error_count
+  errorsBefore += m_context->decode_error_count;
 #endif
 
   m_picture->pict_type = AV_PICTURE_TYPE_NONE;
   int gotPicture = 0;
-#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 25, 0)
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 25, 0)
   int bytesDecoded = avcodec_decode_video(m_context, m_picture, &gotPicture, frame, length);
 #else
   m_packet.data = (uint8_t *)frame;
-  m_packet.size = length;
+  m_packet.size = (int)length;
   int bytesDecoded = avcodec_decode_video2(m_context, m_picture, &gotPicture, &m_packet);
 #endif
 
+  if (bytesDecoded < 0) {
+    PTRACE(1, m_prefix, "Decoder failed!");
+    return false;
+  }
+
   int errorsAfter = m_errorCount;
 #ifdef FFMPEG_HAS_DECODE_ERROR_COUNT
-  errorsAfter += m_context->decode_error_count
+  errorsAfter += m_context->decode_error_count;
 #endif
 
   // if error occurred, tell the other end to send another I-frame and hopefully we can resync
@@ -546,6 +616,7 @@ bool FFMPEGCodec::DecodeVideoFrame(const uint8_t * frame, size_t length, unsigne
     flags |= PluginCodec_ReturnCoderRequestIFrame;
 
   if (gotPicture) {
+    flags |= PluginCodec_ReturnCoderLastFrame;
     bool isIntra = m_fullFrame != NULL ? m_fullFrame->IsIntraFrame() : (m_picture->pict_type == AV_PICTURE_TYPE_I);
     if (isIntra)
       flags |= PluginCodec_ReturnCoderIFrame;
@@ -555,8 +626,6 @@ bool FFMPEGCodec::DecodeVideoFrame(const uint8_t * frame, size_t length, unsigne
            (isIntra ? 'I' : 'P') << "-Frame at " << m_context->width << "x" << m_context->height);
   }
   else {
-    flags &= ~PluginCodec_ReturnCoderLastFrame;
-
     PTRACE(4, m_prefix, "Decoded " << bytesDecoded << " of " << length << " bytes without an output frame");
   }
 
@@ -564,73 +633,13 @@ bool FFMPEGCodec::DecodeVideoFrame(const uint8_t * frame, size_t length, unsigne
 }
 
 
-///////////////////////////////////////////////////////////////////////////////
-
-FFMPEGCodec::EncodedFrame::EncodedFrame()
-  : m_length(0)
-  , m_maxSize(0)
-  , m_buffer(NULL)
-  , m_maxPayloadSize(PluginCodec_RTP_MaxPayloadSize)
-{
-}
-
-
-FFMPEGCodec::EncodedFrame::~EncodedFrame()
-{
-  if (m_buffer != NULL)
-    free(m_buffer);
-}
-
-
-bool FFMPEGCodec::EncodedFrame::SetResolution(unsigned width, unsigned height)
-{
-  return SetMaxSize(width*height*2);
-}
-
-
-bool FFMPEGCodec::EncodedFrame::SetMaxSize(size_t newSize)
-{
-  if (newSize <= m_maxSize)
-    return true;
-
-  m_buffer = (uint8_t *)realloc(m_buffer, newSize);
-  if (m_buffer == NULL)
-    return false;
-
-  m_maxSize = newSize;
-  return true;
-}
-
-
-bool FFMPEGCodec::EncodedFrame::Append(const uint8_t * data, size_t len)
-{
-  if (!SetMaxSize(m_length + len))
-    return false;
-
-  memcpy(m_buffer+m_length, data, len);
-  m_length += len;
-  return true;
-}
-
-
-bool FFMPEGCodec::EncodedFrame::Reset(size_t len)
-{
-  if (len > m_maxSize)
-    return false;
-
-  m_length = len;
-  return true;
-}
-
-
-void FFMPEGCodec::EncodedFrame::RTPCallBack(void *, int, int)
-{
-}
-
-
 void FFMPEGCodec::ErrorCallback(unsigned level, const char * msg)
 {
-  PTRACE(level, m_prefix, msg);
+  // This is not really so severe an error, everything decodes fine! Happens with flash, a lot.
+  if (strcmp(msg, "non-existing SPS 32 referenced in buffering period") == 0)
+    level = 4;
+
+  PTRACE(level > 2 ? level > 4 ? 5 : 4 : 3, m_prefix, "FFMPEG(" << level << "): " << msg);
 
   if (level < 2)
     ++m_errorCount;
