@@ -44,10 +44,18 @@
 
   #include <windows.h>
 
-  #ifdef _DEBUG
-    #define OPAL_DLL "OPALd.DLL"
+  #ifdef _WIN64
+    #ifdef _DEBUG
+      #define OPAL_DLL "OPAL64d.DLL"
+    #else
+      #define OPAL_DLL "OPAL64.DLL"
+    #endif
   #else
-    #define OPAL_DLL "OPAL.DLL"
+    #ifdef _DEBUG
+      #define OPAL_DLL "OPALd.DLL"
+    #else
+      #define OPAL_DLL "OPAL.DLL"
+    #endif
   #endif
 
   #define OPEN_LIBRARY(name)             LoadLibrary(name)
@@ -203,8 +211,8 @@ int InitialiseOPAL()
   //command.m_param.m_general.m_audioRecordDevice = "Camera Microphone (2- Logitech";
   command.m_param.m_general.m_autoRxMedia = command.m_param.m_general.m_autoTxMedia = "audio";
 #ifdef STUN_SERVER
-  command.m_param.m_general.m_natMethod = "STUN";
-  command.m_param.m_general.m_natServer = STUN_SERVER;
+  command.m_param.m_general.m_natMethod = "H46019\nSTUN";
+  command.m_param.m_general.m_natServer = "\n"STUN_SERVER;
 #endif
   command.m_param.m_general.m_mediaMask = "RFC4175*";
 
