@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28945 $
+ * $Revision: 33257 $
  * $Author: rjongbloed $
- * $Date: 2013-01-18 21:35:47 -0600 (Fri, 18 Jan 2013) $
+ * $Date: 2015-01-15 04:26:23 +0000 (Thu, 15 Jan 2015) $
  */
 
 #include "precompile.h"
@@ -65,41 +65,44 @@ void CodecTest::Main()
 {
   PArgList & args = GetArguments();
 
-  if (!args.Parse("[Available options are:]"
-                  "-record-driver: audio recorder driver.\n"
-                  "R-record-device: audio recorder device.\n"
-                  "-play-driver: audio player driver.\n"
-                  "P-play-device: audio player device.\n"
-                  "F-audio-frames: audio frames per packet, default 1.\n"
-                  "-play-buffers: audio player buffers, default 8.\n"
-                  "-grab-driver: video grabber driver.\n"
-                  "G-grab-device: video grabber device.\n"
-                  "-grab-format: video grabber format (\"pal\"/\"ntsc\")\n"
-                  "-grab-channel: video grabber channel.\n"
-                  "-display-driver: video display driver to use.\n"
-                  "D-display-device: video display device to use.\n"
-                  "s-frame-size: video frame size (\"qcif\", \"cif\", WxH)\n"
-                  "r-frame-rate: video frame rate (frames/second)\n"
-                  "b-bit-rate: video bit rate (bits/second)\n"
-                  "O-option: set media format option to value, in form  opt=val\n"
-                  "-ext-hdr: Set RTP extension header (RFC5285 one byte value)\n"
-                  "-single-step. video single frame at a time mode\n"
-                  "c-crop. crop rather than scale if resizing\n"
-                  "m-suppress-marker. suppress marker bits to decoder\n"
-                  "M-force-marker. force marker bits to decoder\n"
-                  "p-payload-size: Set size of maximum RTP payload for encoded data\n"
-                  "S-simultanoues: Number of simultaneous encode/decode threads\n"
-                  "T-statistics. output statistics files\n"
-                  "C-rate-control. enable rate control\n"
-                  "d-drop: randomly drop N% of encoded packets\n"
-                  "-count: set number of frames to transcode\n"
-                  "-noprompt. do not prompt for commands, i.e. exit when input closes\n"
-                  "-snr. calculate signal-to-noise ratio between input and output\n"
-                  "i-info. display per-frame info (use multiple times for more info)\n"
-                  "-list. list all available plugin codecs\n"
-                  PTRACE_ARGLIST
-                  "h-help. print this help message.\n"
-                  , false) || args.HasOption('h')) {
+  args.Parse("[Available options are:]"
+             "-record-driver: audio recorder driver.\n"
+             "R-record-device: audio recorder device.\n"
+             "-play-driver: audio player driver.\n"
+             "P-play-device: audio player device.\n"
+             "F-audio-frames: audio frames per packet, default 1.\n"
+             "-play-buffers: audio player buffers, default 8.\n"
+             "-grab-driver: video grabber driver.\n"
+             "G-grab-device: video grabber device.\n"
+             "-grab-format: video grabber format (\"pal\"/\"ntsc\")\n"
+             "-grab-channel: video grabber channel.\n"
+             "-display-driver: video display driver to use.\n"
+             "D-display-device: video display device to use.\n"
+             "s-frame-size: video frame size (\"qcif\", \"cif\", WxH)\n"
+             "r-frame-rate: video frame rate (frames/second)\n"
+             "b-bit-rate: video bit rate (bits/second)\n"
+             "O-option: set media format option to value, in form  opt=val\n"
+             "-ext-hdr: Set RTP extension header (RFC5285 one byte value)\n"
+             "-single-step. video single frame at a time mode\n"
+             "c-crop. crop rather than scale if resizing\n"
+             "m-suppress-marker. suppress marker bits to decoder\n"
+             "M-force-marker. force marker bits to decoder\n"
+             "p-payload-size: Set size of maximum RTP payload for encoded data\n"
+             "S-simultaneous: Number of simultaneous encode/decode threads\n"
+             "T-statistics. output statistics files\n"
+             "C-rate-control. enable rate control\n"
+             "d-drop: randomly drop N% of encoded packets\n"
+             "-count: set number of frames to transcode\n"
+             "-noprompt. do not prompt for commands, i.e. exit when input closes\n"
+             "-snr. calculate signal-to-noise ratio between input and output\n"
+             "i-info. display per-frame info (use multiple times for more info)\n"
+             "-pcap: save encoded packets in a PCAP file\n"
+             "-list. list all available plugin codecs\n"
+             PTRACE_ARGLIST
+             "h-help. print this help message.\n"
+             , false);
+  if (!args.IsParsed() || args.HasOption('h') ||
+              (args.GetCount() == 0 && !args.HasOption("list"))) {
     cerr << "usage: " << GetFile().GetTitle() << " [ options ] fmtname [ fmtname ]\n"
               "  where fmtname is the Media Format Name for the codec(s) to test, up to two\n"
               "  formats (one audio and one video) may be specified.\n";
@@ -110,6 +113,8 @@ void CodecTest::Main()
 
   PTRACE_INITIALISE(args);
 
+  OPAL_REGISTER_RAW_VIDEO();
+
   if (args.HasOption("list")) {
     OpalPluginCodecManager & codecMgr = OpalPluginCodecManager::GetInstance();
     PPluginModuleManager::PluginListType pluginList = codecMgr.GetPluginList();
@@ -213,14 +218,6 @@ void CodecTest::Main()
       if (cmd == "q" || cmd == "x" || cmd == "quit" || cmd == "exit")
         break;
 
-      if (cmd.NumCompare("n") == EqualTo) {
-        int steps = cmd.Mid(1).AsUnsigned();
-        do {
-          test.m_video.m_frameWait.Signal();
-        } while (--steps > 0);
-        continue;
-      }
-
       if (cmd == "vfu") {
         if (test.m_video.m_encoder == NULL)
           cout << "\nNo video encoder running!" << endl;
@@ -260,6 +257,14 @@ void CodecTest::Main()
         continue;
       }
 
+      if (cmd.NumCompare("n") == EqualTo) {
+        int steps = cmd.Mid(1).AsUnsigned();
+        do {
+          test.m_video.m_frameWait.Signal();
+        } while (--steps > 0);
+        continue;
+      }
+
       cout << "Select:\n"
               "  vfu    : Video Fast Update (force I-Frame)\n"
               "  fg     : Flip video grabber top to bottom\n"
@@ -277,7 +282,10 @@ void CodecTest::Main()
 }
 
 
-int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & rawFormat)
+int TranscoderThread::InitialiseCodec(PArgList & args,
+                                      const OpalMediaType & mediaType,
+                                      OpalMediaFormat & mediaFormat,
+                                      OpalMediaFormat & rawFormat)
 {
   if (args.HasOption('m'))
     m_markerHandling = SuppressMarkers;
@@ -289,40 +297,46 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
   if (!s.IsEmpty())
     m_framesToTranscode = s.AsInteger();
 
+  PString pcapFilename = args.GetOptionString("pcap");
+  if (!pcapFilename.IsEmpty() && !m_pcapFile.Open(pcapFilename, PFile::WriteOnly)) {
+    cout << "Could not write to PCAP file \"" << pcapFilename << '"' << endl;
+    return 0;
+  }
+
   m_calcSNR = args.HasOption("snr");
   m_extensionHeader = (BYTE)args.GetOptionString("ext-hdr", "255").AsUnsigned();
 
   for (PINDEX i = 0; i < args.GetCount(); i++) {
-    OpalMediaFormat mediaFormat = args[i];
+    mediaFormat = args[i];
     if (mediaFormat.IsEmpty()) {
-      cout << "Unknown media format name \"" << args[i] << '"' << endl;
+      cout << "Unknown media format name \"" << args[i] << "\", use one of:";
+      OpalMediaFormatList all = OpalMediaFormat::GetAllRegisteredMediaFormats();
+      for (OpalMediaFormatList::iterator it = all.begin(); it != all.end(); ++it) {
+        if (it->IsTransportable())
+          cout << "\n   " << *it;
+      }
+      cout << endl;
       return 0;
     }
 
-    if (mediaFormat.GetMediaType() == rawFormat.GetMediaType()) {
-      if (rawFormat == mediaFormat) {
+    if (mediaFormat.GetMediaType() == mediaType) {
+      if (!mediaFormat.IsTransportable()) {
         m_decoder = NULL;
         m_encoder = NULL;
+        rawFormat = mediaFormat;
       }
       else {
-        OpalMediaFormat adjustedRawFormat = rawFormat;
-        if (rawFormat == OpalPCM16) {
-          if (mediaFormat.GetPayloadType() == RTP_DataFrame::G722)
-            adjustedRawFormat = OpalPCM16_16KHZ;
-          else {
-            PString str = OPAL_PCM16;
-            if (mediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(), 1) == 2)
-              str += 'S';
-            if (mediaFormat.GetClockRate() != 8000)
-              str += '-' + PString(PString::Unsigned, mediaFormat.GetTimeUnits()) + "KHZ";
-            adjustedRawFormat = str;
-          }
+        OpalMediaFormatList rawFormats = OpalTranscoder::GetDestinationFormats(mediaFormat);
+        if (rawFormats.IsEmpty()) {
+          cout << "No transcoders for format name \"" << mediaFormat << '"' << endl;
+          return 0;
+        }
+        rawFormat = rawFormats[0];
 
-          if (args.HasOption('F')) {
-            unsigned fpp = args.GetOptionString('F').AsUnsigned();
-            if (fpp > 0)
-              mediaFormat.SetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), fpp);
-          }
+        if (args.HasOption('F') && mediaType == OpalMediaType::Audio()) {
+          unsigned fpp = args.GetOptionString('F').AsUnsigned();
+          if (fpp > 0)
+            mediaFormat.SetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), fpp);
         }
 
         PStringArray options = args.GetOptionString('O').Lines();
@@ -343,12 +357,12 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
 
         mediaFormat.ToCustomisedOptions();
 
-        if ((m_encoder = OpalTranscoder::Create(adjustedRawFormat, mediaFormat)) == NULL) {
+        if ((m_encoder = OpalTranscoder::Create(rawFormat, mediaFormat)) == NULL) {
           cout << "Could not create encoder for media format \"" << mediaFormat << '"' << endl;
           return false;
         }
 
-        if ((m_decoder = OpalTranscoder::Create(mediaFormat, adjustedRawFormat)) == NULL) {
+        if ((m_decoder = OpalTranscoder::Create(mediaFormat, rawFormat)) == NULL) {
           cout << "Could not create decoder for media format \"" << mediaFormat << '"' << endl;
           return false;
         }
@@ -386,7 +400,8 @@ static double CalcSNR(const BYTE * src1, const BYTE * src2, PINDEX dataLen)
 
 bool AudioThread::Initialise(PArgList & args)
 {
-  switch (InitialiseCodec(args, OpalPCM16)) {
+  OpalMediaFormat mediaFormat, rawFormat;
+  switch (InitialiseCodec(args, OpalMediaType::Audio(), mediaFormat, rawFormat)) {
     case 0 :
       return false;
     case 1 :
@@ -394,17 +409,16 @@ bool AudioThread::Initialise(PArgList & args)
   }
 
   m_readSize = m_encoder != NULL ? m_encoder->GetOptimalDataFrameSize(TRUE) : 480;
-  OpalMediaFormat mediaFormat = m_encoder != NULL ? m_encoder->GetOutputFormat() : OpalPCM16;
 
   cout << "Audio media format set to " << mediaFormat << endl;
 
+  unsigned channels = rawFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption());
+  unsigned sampleRate = rawFormat.GetClockRate();
+
   // Audio recorder
   PString driverName = args.GetOptionString("record-driver");
   PString deviceName = args.GetOptionString("record-device");
-  m_recorder = PSoundChannel::CreateOpenedChannel(driverName, deviceName,
-                                                  PSoundChannel::Recorder,
-                                                  mediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption()),
-                                                  mediaFormat.GetClockRate());
+  m_recorder = PSoundChannel::CreateOpenedChannel(driverName, deviceName, PSoundChannel::Recorder, channels, sampleRate);
   if (m_recorder == NULL) {
     cerr << "Cannot use ";
     if (driverName.IsEmpty() && deviceName.IsEmpty())
@@ -427,7 +441,8 @@ bool AudioThread::Initialise(PArgList & args)
   cout << "Audio Recorder ";
   if (!driverName.IsEmpty())
     cout << "driver \"" << driverName << "\" and ";
-  cout << "device \"" << m_recorder->GetName() << "\" using "
+  cout << "device \"" << m_recorder->GetName() << "\" at "
+       << PString(PString::ScaleSI, m_recorder->GetSampleRate()) << "Hz, using "
        << bufferCount << 'x' << m_readSize << " byte buffers ";
 
   if (!m_recorder->SetBuffers(m_readSize, bufferCount)) {
@@ -440,10 +455,7 @@ bool AudioThread::Initialise(PArgList & args)
   // Audio player
   driverName = args.GetOptionString("play-driver");
   deviceName = args.GetOptionString("play-device");
-  m_player = PSoundChannel::CreateOpenedChannel(driverName, deviceName,
-                                                PSoundChannel::Player,
-                                                mediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption()),
-                                                mediaFormat.GetClockRate());
+  m_player = PSoundChannel::CreateOpenedChannel(driverName, deviceName, PSoundChannel::Player, channels, sampleRate);
   if (m_player == NULL) {
     cerr << "Cannot use ";
     if (driverName.IsEmpty() && deviceName.IsEmpty())
@@ -466,7 +478,8 @@ bool AudioThread::Initialise(PArgList & args)
   cout << "Audio Player ";
   if (!driverName.IsEmpty())
     cout << "driver \"" << driverName << "\" and ";
-  cout << "device \"" << m_player->GetName() << "\" using "
+  cout << "device \"" << m_player->GetName() << "\" at "
+       << PString(PString::ScaleSI, m_player->GetSampleRate()) << "Hz, using "
        << bufferCount << 'x' << m_readSize << " byte buffers ";
 
   if (!m_player->SetBuffers(m_readSize, bufferCount)) {
@@ -474,8 +487,7 @@ bool AudioThread::Initialise(PArgList & args)
     return false;
   }
 
-  if (m_encoder == NULL) 
-    m_frameTime = mediaFormat.GetFrameTime();
+  m_frameTime = mediaFormat.GetFrameTime()*mediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1);
 
   cout << "opened and initialised." << endl;
   m_running = true;
@@ -486,15 +498,14 @@ bool AudioThread::Initialise(PArgList & args)
 
 bool VideoThread::Initialise(PArgList & args)
 {
-  switch (InitialiseCodec(args, OpalYUV420P)) {
+  OpalMediaFormat mediaFormat, rawFormat;
+  switch (InitialiseCodec(args, OpalMediaType::Video(), mediaFormat, rawFormat)) {
     case 0 :
       return false;
     case 1 :
       return true;
   }
 
-  OpalMediaFormat mediaFormat = m_encoder != NULL ? m_encoder->GetOutputFormat() : OpalYUV420P;
-
   cout << "Video media format set to " << mediaFormat << endl;
 
   // Video grabber
@@ -546,6 +557,7 @@ bool VideoThread::Initialise(PArgList & args)
   }
   cout << "Grabber input format set to " << m_grabber->GetVideoFormat() << endl;
 
+
   if (args.HasOption("grab-channel")) {
     int videoInput = args.GetOptionString("grab-channel").AsInteger();
     if (!m_grabber->SetChannel(videoInput)) {
@@ -555,7 +567,56 @@ bool VideoThread::Initialise(PArgList & args)
   }
   cout << "Grabber channel set to " << m_grabber->GetChannel() << endl;
 
-  
+  if (rawFormat != mediaFormat && m_grabber->SetColourFormat(mediaFormat)) {
+    delete m_encoder;
+    m_encoder = NULL;
+  }
+  else if (!m_grabber->SetColourFormatConverter(rawFormat)) {
+    cerr << "Video grabber device could not be set to colour format " << rawFormat << endl;
+    return false;
+  }
+
+  PString nativeColourFormat = m_grabber->PVideoFrameInfo::GetColourFormat();
+  cout << "Grabber colour format set to " << rawFormat << " (";
+  if (nativeColourFormat == rawFormat)
+    cout << "native";
+  else
+    cout << "converted from " << nativeColourFormat;
+  cout << ')' << endl;
+
+
+  unsigned width, height;
+  if (args.HasOption("frame-size")) {
+    PString sizeString = args.GetOptionString("frame-size");
+    if (!PVideoFrameInfo::ParseSize(sizeString, width, height)) {
+      cerr << "Illegal video frame size \"" << sizeString << '"' << endl;
+      return false;
+    }
+  }
+  else
+    m_grabber->GetFrameSize(width, height);
+
+  mediaFormat.SetOptionInteger(OpalVideoFormat::FrameWidthOption(), width);
+  mediaFormat.SetOptionInteger(OpalVideoFormat::FrameHeightOption(), height);
+  mediaFormat.SetOptionInteger(OpalVideoFormat::MaxRxFrameWidthOption(), width);
+  mediaFormat.SetOptionInteger(OpalVideoFormat::MaxRxFrameHeightOption(), height);
+
+  PVideoFrameInfo::ResizeMode resizeMode = args.HasOption("crop") ? PVideoFrameInfo::eCropCentre : PVideoFrameInfo::eScale;
+  if (!m_grabber->SetFrameSizeConverter(width, height, resizeMode)) {
+    cerr << "Video grabber device could not be set to size " << width << 'x' << height << endl;
+    return false;
+  }
+
+  unsigned nativeWidth, nativeHeight;
+  m_grabber->PVideoFrameInfo::GetFrameSize(nativeWidth, nativeHeight);
+  cout << "Grabber frame size set to " << width << 'x' << height << " (";
+  if (nativeWidth == width && nativeHeight == height)
+    cout << "native";
+  else
+    cout << "converted from " << nativeWidth << 'x' << nativeHeight;
+  cout << ')' << endl;
+
+
   if (args.HasOption("frame-rate")) {
     m_frameRate = args.GetOptionString("frame-rate").AsUnsigned();
     if (!m_grabber->SetFrameRate(m_frameRate)) {
@@ -565,9 +626,11 @@ bool VideoThread::Initialise(PArgList & args)
   }
 
   m_frameRate = m_grabber->GetFrameRate();
-  mediaFormat.SetOptionInteger(OpalVideoFormat::FrameTimeOption(), mediaFormat.GetClockRate()/m_frameRate);
+  m_frameTime = mediaFormat.GetClockRate()/m_frameRate;
+  mediaFormat.SetOptionInteger(OpalVideoFormat::FrameTimeOption(), m_frameTime);
   cout << "Grabber frame rate set to " << m_grabber->GetFrameRate() << endl;
 
+
   // Video display
   driverName = args.GetOptionString("display-driver");
   deviceName = args.GetOptionString("display-device");
@@ -595,60 +658,38 @@ bool VideoThread::Initialise(PArgList & args)
   cout << "device \"" << m_display->GetDeviceName() << "\" opened." << endl;
 
   // Configure formats/sizes/speeds
-  cout << "Grabber colour format set to " << m_grabber->GetColourFormat() << " (";
-  if (m_grabber->GetColourFormat() == "YUV420P")
-    cout << "native";
-  else
-    cout << "converted to YUV420P";
-  cout << ')' << endl;
 
-  if (!m_display->SetColourFormatConverter("YUV420P")) {
-    cerr << "Video display device could not be set to colour format YUV420P" << endl;
+  if (rawFormat != mediaFormat && m_display->SetColourFormat(mediaFormat)) {
+    delete m_decoder;
+    m_decoder = NULL;
+  }
+  else if (!m_display->SetColourFormatConverter(rawFormat)) {
+    cerr << "Video display device could not be set to colour format " << rawFormat << endl;
     return false;
   }
 
-  cout << "Display colour format set to " << m_display->GetColourFormat() << " (";
-  if (m_display->GetColourFormat() == "YUV420P")
+  nativeColourFormat = m_display->PVideoFrameInfo::GetColourFormat();
+  cout << "Display colour format set to " << rawFormat << " (";
+  if (nativeColourFormat == rawFormat)
     cout << "native";
   else
-    cout << "converted from YUV420P";
+    cout << "converted to " << nativeColourFormat;
   cout << ')' << endl;
 
 
-  unsigned width, height;
-  if (args.HasOption("frame-size")) {
-    PString sizeString = args.GetOptionString("frame-size");
-    if (!PVideoFrameInfo::ParseSize(sizeString, width, height)) {
-      cerr << "Illegal video frame size \"" << sizeString << '"' << endl;
-      return false;
-    }
-  }
-  else
-    m_grabber->GetFrameSize(width, height);
-
-  mediaFormat.SetOptionInteger(OpalVideoFormat::FrameWidthOption(), width);
-  mediaFormat.SetOptionInteger(OpalVideoFormat::FrameHeightOption(), height);
-  mediaFormat.SetOptionInteger(OpalVideoFormat::MaxRxFrameWidthOption(), width);
-  mediaFormat.SetOptionInteger(OpalVideoFormat::MaxRxFrameHeightOption(), height);
-
-  PVideoFrameInfo::ResizeMode resizeMode = args.HasOption("crop") ? PVideoFrameInfo::eCropCentre : PVideoFrameInfo::eScale;
-  if (!m_grabber->SetFrameSizeConverter(width, height, resizeMode)) {
-    cerr << "Video grabber device could not be set to size " << width << 'x' << height << endl;
-    return false;
-  }
-  cout << "Grabber frame size set to " << m_grabber->GetFrameWidth() << 'x' << m_grabber->GetFrameHeight() << endl;
-
-  if  (!m_display->SetFrameSizeConverter(width, height, resizeMode)) {
+  if (!m_display->SetFrameSizeConverter(width, height, resizeMode)) {
     cerr << "Video display device could not be set to size " << width << 'x' << height << endl;
     return false;
   }
 
-  cout << "Display frame size set to " << m_display->GetFrameWidth() << 'x' << m_display->GetFrameHeight() << endl;
+  m_display->PVideoFrameInfo::GetFrameSize(nativeWidth, nativeHeight);
+  cout << "Display frame size set to " << width << 'x' << height << " (";
+  if (nativeWidth == width && nativeHeight == height)
+    cout << "native";
+  else
+    cout << "converted to " << nativeWidth << 'x' << nativeHeight;
+  cout << ')' << endl;
 
-  if (!m_grabber->SetColourFormatConverter("YUV420P") ) {
-    cerr << "Video grabber device could not be set to colour format YUV420P" << endl;
-    return false;
-  }
 
   if (args.HasOption("bit-rate")) {
     PString str = args.GetOptionString("bit-rate");
@@ -688,7 +729,6 @@ bool VideoThread::Initialise(PArgList & args)
   if (args.HasOption('T'))
     m_frameFilename = "frame_stats.csv";
 
-  m_frameTime = mediaFormat.GetFrameTime();
   if (m_encoder != NULL) {
     if (args.HasOption('p')) {
       unsigned bytes = args.GetOptionString('p').AsUnsigned();
@@ -696,6 +736,13 @@ bool VideoThread::Initialise(PArgList & args)
       mediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), bytes);
     }
     m_encoder->UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
+    mediaFormat = m_encoder->GetOutputFormat();
+    width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption());
+    height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption());
+    if (!m_grabber->SetFrameSizeConverter(width, height, resizeMode)) {
+      cerr << "Video grabber device could not be set to size " << width << 'x' << height << endl;
+      return false;
+    }
   }
 
   m_singleStep = args.HasOption("single-step");
@@ -743,15 +790,20 @@ void TranscoderThread::OnTranscoderCommand(OpalMediaCommand & cmd, P_INT_PTR)
     cout << "decoder lost sync" << endl;
     coutMutex.Signal();
     m_forceIFrame = true;
-    ((OpalVideoTranscoder *)m_encoder)->ForceIFrame();
-  }
-  else {
-    coutMutex.Wait();
-    cout << "unknown decoder command " << cmd.GetName() << endl;
-    coutMutex.Signal();
   }
+
+  m_encoder->ExecuteCommand(cmd);
 }
 
+
+void VideoThread::SaveSNRFrame(const RTP_DataFrame & src)
+{
+  RTP_DataFrame * saved = new RTP_DataFrame(src);
+  saved->MakeUnique();
+  m_snrSourceFrames.push(saved);
+}
+
+
 void VideoThread::CalcSNR(const RTP_DataFrame & dst)
 {
   while (m_snrSourceFrames.size() > 0) {
@@ -890,7 +942,11 @@ void TranscoderThread::Main()
   // main loop
   //
 
-  RTP_DataFrame * srcFramePtr = NULL;
+  RTP_DataFrame srcFrame;
+
+  OpalVideoFormat videoFormat(m_encoder->GetOutputFormat());
+  PBYTEArray videoDetectorContext;
+  bool videoDetectorFailed = false;
 
   while ((m_running && m_framesToTranscode < 0) || (m_framesToTranscode-- > 0)) {
 
@@ -898,10 +954,6 @@ void TranscoderThread::Main()
     //
     //  acquire and format source frame
     //
-    delete srcFramePtr;
-    srcFramePtr = new RTP_DataFrame(0);
-    RTP_DataFrame & srcFrame = *srcFramePtr;
-
     {
       bool state = Read(srcFrame);
       if (oldSrcState != state) {
@@ -915,6 +967,10 @@ void TranscoderThread::Main()
     m_timestamp += m_frameTime;
     ++totalInputFrameCount;
 
+    bool detectorSaysIntra = false;
+    bool encoderSaysIntra = false;
+    bool decoderSaysIntra = false;
+
     //////////////////////////////////////////////
     //
     //  allow the rate controller to skip frames
@@ -949,23 +1005,21 @@ void TranscoderThread::Main()
             coutMutex.Signal();
           }
           if (m_forceIFrame || rateControlForceIFrame)
-            ((OpalVideoTranscoder *)m_encoder)->ForceIFrame();
+            m_encoder->ExecuteCommand(OpalVideoUpdatePicture());
         }
 
         if (m_extensionHeader != 255)
           srcFrame.SetHeaderExtension(1, 1, &m_extensionHeader, RTP_DataFrame::RFC5285_OneByte);
 
+
         bool state = m_encoder->ConvertFrames(srcFrame, encFrames);
         if (oldEncState != state) {
           oldEncState = state;
           cerr << "Encoder " << (state ? "restor" : "fail") << "ed at input frame " << totalInputFrameCount-1 << endl;
           continue;
         }
-        if (isVideo && ((OpalVideoTranscoder *)m_encoder)->WasLastFrameIFrame()) {
-          coutMutex.Wait();
-          cout << "Encoder returned I-Frame at input frame " << totalInputFrameCount-1 << endl;
-          coutMutex.Signal();
-        }
+        if (isVideo)
+          encoderSaysIntra = ((OpalVideoTranscoder *)m_encoder)->WasLastFrameIFrame();
       }
 
       //////////////////////////////////////////////
@@ -991,6 +1045,8 @@ void TranscoderThread::Main()
             break;
         }
 
+        m_pcapFile.WriteRTP(encFrames[i]);
+
         if (g_infoCount > 0) {
           RTP_DataFrame & rtp = encFrames[i];
           coutMutex.Wait();
@@ -1011,10 +1067,8 @@ void TranscoderThread::Main()
 
       totalEncodedPacketCount += encFrames.GetSize();
 
-      if (isVideo && m_calcSNR) {
-        ((VideoThread *)this)->SaveSNRFrame(srcFramePtr);
-        srcFramePtr = NULL;
-      }
+      if (isVideo && m_calcSNR)
+        ((VideoThread *)this)->SaveSNRFrame(srcFrame);
 
       //////////////////////////////////////////////
       //
@@ -1037,14 +1091,10 @@ void TranscoderThread::Main()
       //
       //  push audio/video frames through NULL decoder
       //
-      if (m_encoder == NULL) {
+      if (m_decoder == NULL) {
         totalEncodedByteCount += encodedPayloadSize;
-        RTP_DataFrameList outFrames;
-        outFrames = encFrames;
-        if (outFrames.GetSize() != 1)
-          cerr << "NULL decoder returned != 1 output frame for input frame " << totalInputFrameCount-1 << endl;
-        else {
-          bool state = Write(outFrames[0]);
+        for (PINDEX i = 0; i < encFrames.GetSize(); i++) {
+          bool state = Write(encFrames[i]);
           if (oldOutState != state) {
             oldOutState = state;
             cerr << "Output write " << (state ? "restor" : "fail") << "ed at input frame " << totalInputFrameCount << endl;
@@ -1075,11 +1125,7 @@ void TranscoderThread::Main()
             cerr << "Non rate controlled video decoder returned != 1 output frame for input frame " << totalInputFrameCount-1 << endl;
           else if (outFrames.GetSize() == 1) {
             if (isVideo) {
-              if (((OpalVideoTranscoder *)m_decoder)->WasLastFrameIFrame()) {
-                coutMutex.Wait();
-                cout << "Decoder returned I-Frame at output frame " << totalOutputFrameCount << endl;
-                coutMutex.Signal();
-              } 
+              decoderSaysIntra = ((OpalVideoTranscoder *)m_decoder)->WasLastFrameIFrame();
               if (g_infoCount > 1) {
                 coutMutex.Wait();
                 cout << "Decoder generated payload of size " << outFrames[0].GetPayloadSize() << endl;
@@ -1096,6 +1142,28 @@ void TranscoderThread::Main()
             totalOutputFrameCount++;
           }
         }
+
+        if (isVideo) {
+          bool detectedInter = false;
+          for (PINDEX i = 0; i < encFrames.GetSize(); i++) {
+            switch (videoFormat.GetVideoFrameType(encFrames[i].GetPayloadPtr(), encFrames[i].GetPayloadSize(), videoDetectorContext)) {
+              case OpalVideoFormat::e_IntraFrame :
+                detectorSaysIntra = true;
+                break;
+              case OpalVideoFormat::e_InterFrame :
+                detectedInter = true;
+                break;
+              default :
+                break;
+            }
+          }
+          if (!videoDetectorFailed && !detectorSaysIntra && !detectedInter) {
+            videoDetectorFailed = true;
+            coutMutex.Wait();
+            cout << "Video detector could not determine if I-Frame or P-Frame" << endl;
+            coutMutex.Signal();
+          }
+        }
       }
 
       //////////////////////////////////////////////
@@ -1129,11 +1197,7 @@ void TranscoderThread::Main()
           if (outFrames.GetSize() > 1) 
             cerr << "Rate controlled video decoder returned > 1 output frame for input frame " << totalInputFrameCount-1 << endl;
           else if (outFrames.GetSize() == 1) {
-            if (((OpalVideoTranscoder *)m_decoder)->WasLastFrameIFrame()) {
-              coutMutex.Wait();
-              cout << "Decoder returned I-Frame at output frame " << totalOutputFrameCount << endl;
-              coutMutex.Signal();
-            }
+            decoderSaysIntra = ((OpalVideoTranscoder *)m_decoder)->WasLastFrameIFrame();
             bool state = Write(outFrames[0]);
             if (oldOutState != state) {
               oldOutState = state;
@@ -1147,6 +1211,27 @@ void TranscoderThread::Main()
       }
     }
 
+    if (detectorSaysIntra || encoderSaysIntra || decoderSaysIntra) {
+      coutMutex.Wait();
+      cout << "I-Frame at input frame " << totalInputFrameCount - 1 << " from ";
+      if (detectorSaysIntra) {
+        cout << "detector";
+        if (encoderSaysIntra && decoderSaysIntra)
+          cout << ", ";
+        else if (encoderSaysIntra || decoderSaysIntra)
+          cout << " and ";
+      }
+      if (encoderSaysIntra) {
+        cout << "encoder";
+        if (decoderSaysIntra)
+          cout << " and ";
+      }
+      if (decoderSaysIntra)
+        cout << "decoder";
+      cout << '.' << endl;
+      coutMutex.Signal();
+    }
+
     {
       PTimeInterval duration = PTimer::Tick() - startTick;
       PInt64 msecs = duration.GetMilliSeconds();
@@ -1163,8 +1248,6 @@ void TranscoderThread::Main()
     }
   }
 
-  delete srcFramePtr;
-
   //
   //  end of main loop
   //
@@ -1268,6 +1351,9 @@ bool VideoThread::Read(RTP_DataFrame & data)
 
 bool VideoThread::Write(const RTP_DataFrame & data)
 {
+  if (m_display->GetColourFormat() != "YUV420P")
+    return m_display->SetFrameData(0, 0, 0, 0, data.GetPayloadPtr(), data.GetMarker());
+
   const OpalVideoTranscoder::FrameHeader * frame = (const OpalVideoTranscoder::FrameHeader *)data.GetPayloadPtr();
   m_display->SetFrameSize(frame->width, frame->height);
   return m_display->SetFrameData(frame->x, frame->y,
