@@ -21,14 +21,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28481 $
+ * $Revision: 28641 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:13:17 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2012-12-04 07:37:57 +0000 (Tue, 04 Dec 2012) $
  */
 
 #include <ptlib.h>
 #include <opal/console_mgr.h>
-#include <ep/ivr.h>
+#include <ep/localep.h>
 #include <sip/sipep.h>
 #include <h323/h323ep.h>
 
