@@ -24,41 +24,41 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29850 $
+ * $Revision: 33859 $
  * $Author: rjongbloed $
- * $Date: 2013-05-31 02:34:06 -0500 (Fri, 31 May 2013) $
+ * $Date: 2015-08-08 22:20:16 +0100 (Sat, 08 Aug 2015) $
  */
 
 #ifndef _OpenPhone_MAIN_H
 #define _OpenPhone_MAIN_H
 
-#ifndef _PTLIB_H
-#include <ptlib.h>
+#if WXVER>29
+  #define WXUSINGDLL 1
 #endif
 
-
-#include <wx/wx.h>
-#include <wx/taskbar.h>
-#include <wx/dataobj.h>
-
+#include <ptlib_wx.h>
 #include <opal/manager.h>
 
 #ifndef OPAL_PTLIB_AUDIO
-#error Cannot compile without PTLib sound channel support!
+  #error Cannot compile without PTLib sound channel support!
 #endif
 
-#include <ep/pcss.h>
+#include <wx/taskbar.h>
+#include <wx/dataobj.h>
+
+#include <ptlib/notifier_ext.h>
+#include <ptclib/pssl.h>
 
 #include <h323/h323.h>
 #include <h323/gkclient.h>
 #include <sip/sip.h>
+#include <ep/pcss.h>
 #include <t38/t38proto.h>
 #include <im/im_ep.h>
+#include <h224/h224.h>
+
+// Note, include this after everything else so gets all the conversions
 #include <ptlib/wxstring.h>
-#include <ptlib/notifier_ext.h>
-
-
-#include <list>
 
 
 class MyManager;
@@ -68,12 +68,14 @@ class OpalCapiEndPoint;
 class OpalIVREndPoint;
 class OpalMixerEndPoint;
 class OpalFaxEndPoint;
+class OpalH281Client;
 
 class wxSplitterWindow;
 class wxSplitterEvent;
 class wxSpinCtrl;
 class wxListCtrl;
 class wxListEvent;
+class wxListItem;
 class wxCheckListBox;
 class wxNotebook;
 class wxGrid;
@@ -111,6 +113,8 @@ class MyH323EndPoint : public H323EndPoint
   public:
     MyH323EndPoint(MyManager & manager);
 
+    PStringList m_configuredAliasNames;
+
   private:
     virtual void OnGatekeeperStatus(H323Gatekeeper::RegistrationFailReasons);
 
@@ -153,6 +157,7 @@ class MyFaxEndPoint : public OpalFaxEndPoint
 #endif // OPAL_FAX
 
 
+#if OPAL_HAS_PRESENCE
 class PresenceDialog : public wxDialog
 {
   public:
@@ -167,6 +172,7 @@ class PresenceDialog : public wxDialog
 
     DECLARE_EVENT_TABLE()
 };
+#endif // OPAL_HAS_PRESENCE
 
 
 class AudioDevicesDialog : public wxDialog
@@ -373,7 +379,7 @@ struct StatisticsField
   double CalculateFrameRate(DWORD frames);
   virtual StatisticsField * Clone() const = 0;
   virtual void Update(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics);
-  virtual void GetValue(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics, wxString & value) = 0;
+  virtual void GetValue(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics, PwxString & value) = 0;
 
   const wxChar    * m_name;
   StatisticsPages m_page;
@@ -416,7 +422,7 @@ class StatisticsPage
     void operator=(const StatisticsPage &) { }
 };
 
-class InCallPanel : public CallPanelBase
+class InCallPanel : public CallPanelBase, PAsyncNotifierTarget
 {
   public:
     InCallPanel(MyManager & manager, const PSafePtr<OpalCall> & call, wxWindow * parent);
@@ -446,6 +452,15 @@ class InCallPanel : public CallPanelBase
     void OnUserInputFlash(wxCommandEvent & /*event*/);
     void OnUpdateVU(wxTimerEvent & /*event*/);
 
+    // PAsyncNotifierTarget override
+    virtual void AsyncNotifierSignal();
+    void OnEvtAsyncNotification(wxCommandEvent & /*event*/);
+
+#if OPAL_HAS_H281
+    void OnMouseFECC(wxMouseEvent & /*event*/);
+    PDECLARE_ASYNC_NOTIFIER(OpalH281Client, InCallPanel, OnChangedFECC);
+#endif
+
     void SpeakerVolume(wxScrollEvent & /*event*/);
     void MicrophoneVolume(wxScrollEvent & /*event*/);
     void SetVolume(bool microphone, int value, bool muted);
@@ -458,10 +473,15 @@ class InCallPanel : public CallPanelBase
     wxCheckBox   * m_MicrophoneMute;
     wxSlider     * m_SpeakerVolume;
     wxSlider     * m_MicrophoneVolume;
-    wxGauge      * m_vuSpeaker;
-    wxGauge      * m_vuMicrophone;
+    wxGauge      * m_SpeakerGauge;
+    wxGauge      * m_MicrophoneGauge;
     wxTimer        m_vuTimer;
-    wxStaticText * m_callTime;
+    wxStaticText * m_CallTime;
+
+#if OPAL_HAS_H281
+    wxButton     * m_fecc[PVideoControlInfo::NumTypes][2];
+#endif
+
     unsigned       m_updateStatistics;
     bool           m_FirstTime;
 
@@ -476,7 +496,9 @@ struct SpeedDialInfo
   PwxString m_Name;
   PwxString m_Number;
   PwxString m_Address;
+#if OPAL_HAS_PRESENCE
   PwxString m_Presentity;
+#endif
   PwxString m_Description;
 
   bool operator<(const SpeedDialInfo & info) const { return m_Name < info.m_Name; }
@@ -504,6 +526,25 @@ class SpeedDialDialog : public wxDialog, public SpeedDialInfo
 };
 
 
+class MyMedia
+{
+public:
+  MyMedia();
+  MyMedia(const OpalMediaFormat & format);
+
+  bool operator<(const MyMedia & other) const
+  {
+    return preferenceOrder < other.preferenceOrder;
+  }
+
+  OpalMediaFormat mediaFormat;
+  const wxChar  * validProtocols;
+  int             preferenceOrder;
+};
+
+typedef std::list<MyMedia> MyMediaList;
+
+
 class RegistrationInfo
 {
   public:
@@ -566,6 +607,39 @@ class RegistrationDialog : public wxDialog
 };
 
 
+class NetOptionsDialog : public wxDialog
+{
+  public:
+    NetOptionsDialog(MyManager *manager, OpalRTPEndPoint * ep);
+
+  private:
+    MyManager & m_manager;
+    OpalRTPEndPoint * m_endpoint;
+
+#if OPAL_PTLIB_SSL
+    int              m_SignalingSecurity;
+    wxRadioBox     * m_SignalingSecurityButtons;
+    wxCheckListBox * m_MediaCryptoSuites;
+    wxButton       * m_MediaCryptoSuiteUp;
+    wxButton       * m_MediaCryptoSuiteDown;
+    void InitSecurityFields();
+    void SaveSecurityFields(wxConfigBase * config, const wxChar * securedSignalingKey, const wxChar * mediaCryptoSuitesKey);
+    void SignalingSecurityChanged(wxCommandEvent & /*event*/);
+    void MediaCryptoSuiteChanged(wxCommandEvent & /*event*/);
+    void MediaCryptoSuiteUp(wxCommandEvent & /*event*/);
+    void MediaCryptoSuiteDown(wxCommandEvent & /*event*/);
+    void MediaCryptoSuiteMove(int dir);
+#endif
+
+    wxGrid * m_stringOptions;
+    void SaveOptions(wxConfigBase * config, const wxChar * stringOptionsGroup);
+
+    friend class OptionsDialog;
+
+    DECLARE_EVENT_TABLE()
+};
+
+
 class OptionsDialog : public wxDialog
 {
   public:
@@ -578,18 +652,23 @@ class OptionsDialog : public wxDialog
 
     ////////////////////////////////////////
     // General fields
-    PwxString m_Username;
-    PwxString m_DisplayName;
-    PwxString m_RingSoundDeviceName;
-    PwxString m_RingSoundFileName;
-    bool      m_AutoAnswer;
-    PwxString m_VendorName;
-    PwxString m_ProductName;
-    PwxString m_ProductVersion;
-    bool      m_HideMinimised;
+    PwxString    m_Username;
+    PwxString    m_DisplayName;
+    PwxString    m_RingSoundDeviceName;
+    PwxString    m_RingSoundFileName;
+    bool         m_AutoAnswer;
+    bool         m_OverrideProductInfo;
+    PwxString    m_VendorName;
+    PwxString    m_ProductName;
+    PwxString    m_ProductVersion;
+    wxTextCtrl * m_VendorNameCtrl;
+    wxTextCtrl * m_ProductNameCtrl;
+    wxTextCtrl * m_ProductVersionCtrl;
+    bool         m_HideMinimised;
 
     void BrowseSoundFile(wxCommandEvent & /*event*/);
     void PlaySoundFile(wxCommandEvent & /*event*/);
+    void OnOverrideProductInfo(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // Networking fields
@@ -609,27 +688,30 @@ class OptionsDialog : public wxDialog
     int             m_UDPPortMax;
     int             m_RTPPortBase;
     int             m_RTPPortMax;
-    wxRadioButton * m_NoNATUsedRadio;
-    wxRadioButton * m_NATRouterRadio;
-    wxRadioButton * m_STUNServerRadio;
-    PwxString       m_NATRouter;
-    wxTextCtrl    * m_NATRouterWnd;
-    PwxString       m_STUNServer;
-    wxTextCtrl    * m_STUNServerWnd;
     wxListBox     * m_LocalInterfaces;
     wxChoice      * m_InterfaceProtocol;
     wxComboBox    * m_InterfaceAddress;
     wxTextCtrl    * m_InterfacePort;
     wxButton      * m_AddInterface;
     wxButton      * m_RemoveInterface;
-    void BandwidthClass(wxCommandEvent & /*event*/);
+    wxListBox     * m_NatMethods;
+    int             m_NatMethodSelected;
 
+    struct NatWrapper : wxClientData
+    {
+      PwxString m_method;
+      bool      m_active;
+      PwxString m_server;
+      int       m_priority;
+    } m_NatInfo;
+
+    void BandwidthClass(wxCommandEvent & /*event*/);
 #if OPAL_PTLIB_SSL
     void FindCertificateAuthority(wxCommandEvent & /*event*/);
     void FindLocalCertificate(wxCommandEvent & /*event*/);
     void FindPrivateKey(wxCommandEvent & /*event*/);
 #endif
-    void NATHandling(wxCommandEvent & /*event*/);
+    void NATMethodSelect(wxCommandEvent & /*event*/);
     void SelectedLocalInterface(wxCommandEvent & /*event*/);
     void ChangedInterfaceInfo(wxCommandEvent & /*event*/);
     void AddInterface(wxCommandEvent & /*event*/);
@@ -643,6 +725,7 @@ class OptionsDialog : public wxDialog
 #if OPAL_AEC
     bool      m_EchoCancellation;
 #endif
+    PwxString m_LocalRingbackTone;
     PwxString m_LineInterfaceDevice;
     int       m_AEC;
     PwxString m_Country;
@@ -653,16 +736,24 @@ class OptionsDialog : public wxDialog
     int       m_SignalDeadband;
     int       m_SilenceDeadband;
     bool      m_DisableDetectInBandDTMF;
+    PwxString m_MusicOnHold;
+    PwxString m_AudioOnRing;
 
     wxComboBox * m_soundPlayerCombo;
     wxComboBox * m_soundRecorderCombo;
+    wxComboBox * m_musicOnHoldCombo;
+    wxComboBox * m_audioOnRingCombo;
     wxComboBox * m_selectedLID;
     wxChoice   * m_selectedAEC;
     wxComboBox * m_selectedCountry;
 
     void ChangedSoundPlayer(wxCommandEvent & /*event*/);
     void ChangedSoundRecorder(wxCommandEvent & /*event*/);
+    void TestPlayer(wxCommandEvent & /*event*/);
+    void TestRecorder(wxCommandEvent & /*event*/);
     void SelectedLID(wxCommandEvent & /*event*/);
+    void ChangedMusicOnHold(wxCommandEvent & /*event*/);
+    void ChangedAudioOnRing(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // Video fields
@@ -672,6 +763,7 @@ class OptionsDialog : public wxDialog
     int       m_VideoGrabFrameRate;
     PwxString m_VideoGrabFrameSize;
     bool      m_VideoGrabPreview;
+    PwxString m_VideoWatermarkDevice;
     bool      m_VideoFlipLocal;
     bool      m_VideoAutoTransmit;
     bool      m_VideoAutoReceive;
@@ -680,20 +772,29 @@ class OptionsDialog : public wxDialog
     PwxString m_VideoMaxFrameSize;
     int       m_VideoGrabBitRate;
     int       m_VideoMaxBitRate;
+    PwxString m_VideoOnHold;
+    PwxString m_VideoOnRing;
 
-    wxComboBox * m_videoGrabDeviceCombo;
-    wxChoice   * m_videoSourceChoice;
+    wxComboBox * m_VideoGrabDeviceCtrl;
+    wxComboBox * m_VideoWatermarkDeviceCtrl;
+    wxComboBox * m_VideoOnHoldDeviceCtrl;
+    wxComboBox * m_VideoOnRingDeviceCtrl;
+    wxChoice   * m_VideoGrabSourceCtrl;
 
     wxButton           * m_TestVideoCapture;
     PThread            * m_TestVideoThread;
     PVideoInputDevice  * m_TestVideoGrabber;
     PVideoOutputDevice * m_TestVideoDisplay;
 
-    void AdjustVideoControls(const PwxString & device);
+    void AdjustVideoControls();
     void ChangeVideoGrabDevice(wxCommandEvent & /*event*/);
+    void ChangeVideoWatermarkDevice(wxCommandEvent & /*event*/);
     void TestVideoCapture(wxCommandEvent & /*event*/);
+    void OnTestVideoEnded(wxCommandEvent & /*event*/);
     void TestVideoThreadMain();
     void StopTestVideo();
+    void ChangedVideoOnHold(wxCommandEvent & /*event*/);
+    void ChangedVideoOnRing(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // Fax fields
@@ -712,6 +813,7 @@ class OptionsDialog : public wxDialog
     int       m_VideoRecordingMode;
     PwxString m_VideoRecordingSize;
 
+#if OPAL_HAS_PRESENCE
     ////////////////////////////////////////
     // Presence fields
     wxListCtrl * m_Presentities;
@@ -725,6 +827,7 @@ class OptionsDialog : public wxDialog
     void EditedPresentity(wxListEvent & /*event*/);
     bool FillPresentityAttributes(OpalPresentity * presentity);
     void ChangedPresentityAttribute(wxGridEvent & /*event*/);
+#endif // OPAL_HAS_PRESENCE
 
     ////////////////////////////////////////
     // Codec fields
@@ -732,32 +835,39 @@ class OptionsDialog : public wxDialog
     wxButton     * m_RemoveCodec;
     wxButton     * m_MoveUpCodec;
     wxButton     * m_MoveDownCodec;
-    wxListBox    * m_allCodecs;
+    wxListCtrl   * m_allCodecs;
     wxListBox    * m_selectedCodecs;
     wxListCtrl   * m_codecOptions;
     wxTextCtrl   * m_codecOptionValue;
     wxStaticText * m_CodecOptionValueLabel;
     wxStaticText * m_CodecOptionValueError;
+    wxButton     * m_SetDefaultCodecOption;
 
     void AddCodec(wxCommandEvent & /*event*/);
     void RemoveCodec(wxCommandEvent & /*event*/);
     void MoveUpCodec(wxCommandEvent & /*event*/);
     void MoveDownCodec(wxCommandEvent & /*event*/);
-    void SelectedCodecToAdd(wxCommandEvent & /*event*/);
+    void SelectedCodecToAdd(wxListEvent & /*event*/);
     void SelectedCodec(wxCommandEvent & /*event*/);
     void SelectedCodecOption(wxListEvent & /*event*/);
     void DeselectedCodecOption(wxListEvent & /*event*/);
     void ChangedCodecOptionValue(wxCommandEvent & /*event*/);
+    void SetDefaultCodecOption(wxCommandEvent & /*event*/);
+    void SetCodecOptionValue(bool useDefault, PwxString newValue);
+    MyMedia * GetCodecOptionInfo(wxListItem & item, PwxString & optionName, PwxString & defaultValue);
 
     ////////////////////////////////////////
     // H.323 fields
     int          m_GatekeeperMode;
     PwxString    m_GatekeeperAddress;
+    PwxString    m_GatekeeperInterface;
+    bool         m_GatekeeperSuppressGRQ;
     PwxString    m_GatekeeperIdentifier;
     int          m_GatekeeperTTL;
     PwxString    m_GatekeeperLogin;
     PwxString    m_GatekeeperPassword;
 
+    wxString     m_H323TerminalType;
     int          m_DTMFSendMode;
     int          m_CallIntrusionProtectionLevel;
     bool         m_DisableFastStart;
@@ -777,16 +887,8 @@ class OptionsDialog : public wxDialog
     void AddAlias(wxCommandEvent & /*event*/);
     void RemoveAlias(wxCommandEvent & /*event*/);
 
-#if OPAL_PTLIB_SSL
-    int              m_H323SignalingSecurity;
-    wxCheckListBox * m_H323MediaCryptoSuites;
-    wxButton       * m_H323MediaCryptoSuiteUp;
-    wxButton       * m_H323MediaCryptoSuiteDown;
-    void H323SignalingSecurityChanged(wxCommandEvent & /*event*/);
-    void H323MediaCryptoSuiteChanged(wxCommandEvent & /*event*/);
-    void H323MediaCryptoSuiteUp(wxCommandEvent & /*event*/);
-    void H323MediaCryptoSuiteDown(wxCommandEvent & /*event*/);
-#endif
+    void MoreOptionsH323(wxCommandEvent & /*event*/);
+    NetOptionsDialog m_H323options;
 
     ////////////////////////////////////////
     // SIP fields
@@ -817,16 +919,8 @@ class OptionsDialog : public wxDialog
     void DeselectedRegistration(wxListEvent & /*event*/);
     void ActivateRegistration(wxListEvent & /*event*/);
 
-#if OPAL_PTLIB_SSL
-    int              m_SIPSignalingSecurity;
-    wxCheckListBox * m_SIPMediaCryptoSuites;
-    wxButton       * m_SIPMediaCryptoSuiteUp;
-    wxButton       * m_SIPMediaCryptoSuiteDown;
-    void SIPSignalingSecurityChanged(wxCommandEvent & /*event*/);
-    void SIPMediaCryptoSuiteChanged(wxCommandEvent & /*event*/);
-    void SIPMediaCryptoSuiteUp(wxCommandEvent & /*event*/);
-    void SIPMediaCryptoSuiteDown(wxCommandEvent & /*event*/);
-#endif
+    void MoreOptionsSIP(wxCommandEvent & /*event*/);
+    NetOptionsDialog m_SIPoptions;
 
     ////////////////////////////////////////
     // Routing fields
@@ -877,22 +971,6 @@ class OptionsDialog : public wxDialog
 };
 
 
-class MyMedia
-{
-public:
-  MyMedia();
-  MyMedia(const OpalMediaFormat & format);
-
-  bool operator<(const MyMedia & other) { return preferenceOrder < other.preferenceOrder; }
-
-  OpalMediaFormat mediaFormat;
-  const wxChar  * validProtocols;
-  int             preferenceOrder;
-};
-
-typedef std::list<MyMedia> MyMediaList;
-
-
 class MyTaskBarIcon : public wxTaskBarIcon
 {
   public:
@@ -926,13 +1004,18 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     bool HasSpeedDialNumber(const wxString & number, const wxString & ignore) const;
 
     void MakeCall(const PwxString & address, const PwxString & local = wxEmptyString, OpalConnection::StringOptions * options = NULL);
-    void AnswerCall();
+    bool AnswerCall();
     void RejectCall();
     void HangUpCall();
     void SendUserInput(char tone);
 
     PSafePtr<OpalCall>       GetCall(PSafetyMode mode);
-    PSafePtr<OpalConnection> GetConnection(bool user, PSafetyMode mode);
+    PSafePtr<OpalConnection> GetNetworkConnection(PSafetyMode mode);
+
+    template<class T> bool GetConnection(PSafePtr<T> & ptr, PSafetyMode mode)
+    {
+      return m_activeCall != NULL && (ptr = m_activeCall->GetConnectionAs<T>(0, mode)) != NULL;
+    }
 
     bool HasHandset() const;
 
@@ -950,12 +1033,15 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     MyPCSSEndPoint & GetPCSSEP() { return *pcssEP; }
 
     void PostEvent(
-      const wxEventType & type,
-      unsigned id,
+      const wxCommandEvent & cmdEvent,
       const PString & str = PString::Empty(),
       const void * data = NULL
     );
 
+#if OPAL_PTLIB_SSL
+    void SetSignalingSecurity(OpalEndPoint * ep, int security);
+#endif
+
   private:
     // PAsyncNotifierTarget override
     virtual void AsyncNotifierSignal();
@@ -987,9 +1073,9 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
       const OpalConnection & connection,  ///<  Connection that is about to use formats
       OpalMediaFormatList & mediaFormats  ///<  Media formats to use
     ) const;
-    virtual PBoolean OnOpenMediaStream(
-      OpalConnection & connection,  /// Connection that owns the media stream
-      OpalMediaStream & stream    /// New media stream being opened
+    virtual void OnStartMediaPatch(
+      OpalConnection & connection,  ///< Connection patch is in
+      OpalMediaPatch & patch        ///< Media patch being started
     );
     virtual void OnClosedMediaStream(
       const OpalMediaStream & stream     ///<  Stream being closed
@@ -1022,6 +1108,12 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
       PVideoInputDevice * & device,         ///<  Created device
       PBoolean & autoDelete                     ///<  Flag for auto delete device
     );
+    virtual bool CreateVideoInputDevice(
+      const OpalConnection & connection,    ///<  Connection needing created video device
+      const PVideoDevice::OpenArgs & args,  ///< Device to change to
+      PVideoInputDevice * & device,         ///<  Created device
+      PBoolean & autoDelete                     ///<  Flag for auto delete device
+    );
     virtual PBoolean CreateVideoOutputDevice(
       const OpalConnection & connection,    ///<  Connection needing created video device
       const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
@@ -1084,12 +1176,14 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     void OnSpeedDialRightClick(wxListEvent & /*event*/);
     void OnSpeedDialEndEdit(wxListEvent & /*event*/);
 
+#if OPAL_HAS_PRESENCE
     void OnMyPresence(wxCommandEvent & /*event*/);
 #if OPAL_HAS_IM
     void OnStartIM(wxCommandEvent & /*event*/);
     void OnInCallIM(wxCommandEvent & /*event*/);
     void OnSendIMSpeedDial(wxCommandEvent & /*event*/);
 #endif // OPAL_HAS_IM
+#endif // OPAL_HAS_PRESENCE
 
     void OnEvtRinging(wxCommandEvent & /*event*/ theEvent);
     void OnEvtEstablished(wxCommandEvent & /*event*/ theEvent);
@@ -1098,7 +1192,9 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     void OnSetTrayTipText(wxCommandEvent & /*event*/);
 
     bool CanDoFax() const;
+#if OPAL_HAS_PRESENCE && OPAL_HAS_IM
     bool CanDoIM() const;
+#endif
 
     enum SpeedDialViews {
       e_ViewLarge,
@@ -1155,21 +1251,16 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     OpalLineEndPoint * potsEP;
     void StartLID();
 
-    int       m_NATHandling;
-    PwxString m_NATRouter;
-    PwxString m_STUNServer;
-    void SetNATHandling();
+    void SetNAT(const PwxString & method, bool active, const PwxString & server, int priority);
 
     vector<PwxString> m_LocalInterfaces;
     void StartAllListeners();
-#if OPAL_PTLIB_SSL
-    void SetSignalingSecurity(OpalEndPoint * ep, int security, const char * unsecure, const char * secure);
-#endif
 
 #if OPAL_H323
     MyH323EndPoint * h323EP;
     int              m_gatekeeperMode;
     PwxString        m_gatekeeperAddress;
+    PwxString        m_gatekeeperInterface;
     PwxString        m_gatekeeperIdentifier;
 #endif
     bool StartGatekeeper();
@@ -1208,17 +1299,30 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     PwxString m_LastDialed;
     PwxString m_LastReceived;
 
+    bool            m_OverrideProductInfo;
+    OpalProductInfo m_overriddenProductInfo;
+
     PwxString m_ForwardingAddress;
     int       m_ForwardingTimeout;
     PTimer    m_ForwardingTimer;
     PDECLARE_NOTIFIER(PTimer, MyManager, OnForwardingTimeout);
 
-    bool m_VideoGrabPreview;
-    int  m_localVideoFrameX;
-    int  m_localVideoFrameY;
-    int  m_remoteVideoFrameX;
-    int  m_remoteVideoFrameY;
+    struct VideoWindowInfo
+    {
+      int x, y, size;
 
+      VideoWindowInfo() : x(wxDefaultCoord), y(wxDefaultCoord), size(0)
+      {
+      }
+      VideoWindowInfo(const PVideoOutputDevice * device);
+
+      bool operator==(const VideoWindowInfo & info) const;
+      bool ReadConfig(wxConfigBase * config, const wxString & keyBase);
+      void WriteConfig(wxConfigBase * config, const wxString & keyBase);
+    };
+    VideoWindowInfo m_videoWindowInfo[2][OpalVideoFormat::NumContentRole];
+
+    bool                m_VideoGrabPreview;
     PVideoInputDevice * m_primaryVideoGrabber;
 
     PVideoDevice::OpenArgs m_SecondaryVideoGrabber;
@@ -1232,6 +1336,8 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     int       m_VideoTargetBitRate;
     bool AdjustVideoFormats();
 
+    PString m_VideoWatermarkDevice;
+
     enum ExtendedVideoRoles {
       e_DisabledExtendedVideoRoles,
       e_ExtendedVideoRolePerOption,
@@ -1287,9 +1393,11 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     PwxString                  m_lastRecordFile;
     PwxString                  m_lastPlayFile;
 
+#if OPAL_HAS_PRESENCE
     // Presence
     bool MonitorPresence(const PString & presentity, const PString & uri, bool start);
     PDECLARE_ASYNC_PresenceChangeNotifier(MyManager, OnPresenceChange);
+#endif
 
 #if OPAL_HAS_IM
     // Instant messaging
@@ -1300,6 +1408,20 @@ class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarge
     IMDialogMap m_imDialogMap;
 #endif
 
+#if OPAL_PTLIB_SSL
+    virtual bool ApplySSLCredentials(
+      const OpalEndPoint & ep,  ///< Endpoint transport is based on.
+      PSSLContext & context,    ///< Context on which to set certificates
+      bool create               ///< Create self signed cert/key if required
+    ) const;
+
+    PDECLARE_SSLPasswordNotifier(MyManager, OnSSLPassword);
+    void OnEvtGetSSLPassword(wxCommandEvent & /*event*/);
+
+    PSyncPoint m_gotSSLPassword;
+    PwxString m_SSLPassword;
+#endif // OPAL_PTLIB_SSL
+
     DECLARE_EVENT_TABLE()
 
   friend class OptionsDialog;
