@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26798 $
+ * $Revision: 33840 $
  * $Author: rjongbloed $
- * $Date: 2011-12-14 02:30:18 -0600 (Wed, 14 Dec 2011) $
+ * $Date: 2015-08-06 20:16:18 +0100 (Thu, 06 Aug 2015) $
  */
 
 #ifndef _PlayRTP_MAIN_H
@@ -53,7 +53,7 @@ class PlayRTP : public PProcess
     bool m_extendedInfo;
     unsigned m_rotateExtensionId;
     bool m_noDelay;
-    bool m_writeEventLog;
+    PTimeInterval m_jitterLimit;
 
     PFile     m_payloadFile;
     PTextFile m_eventLog;
