@@ -23,14 +23,18 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29992 $
+ * $Revision: 31170 $
  * $Author: rjongbloed $
- * $Date: 2013-06-19 00:11:09 -0500 (Wed, 19 Jun 2013) $
+ * $Date: 2014-01-10 02:57:39 +0000 (Fri, 10 Jan 2014) $
  */
 
 #include <ptlib.h>
+#include <ptclib/podbc.h>
 
-#include <opal/manager.h>
+#include <opal/console_mgr.h>
+#include <opal/patch.h>
+#include <h323/gkclient.h>
+#include <h323/gkserver.h>
 
 #if OPAL_PTLIB_SSL
 #include <ptclib/shttpsvc.h>
@@ -40,15 +44,5 @@ typedef PSecureHTTPServiceProcess MyProcessAncestor;
 typedef PHTTPServiceProcess MyProcessAncestor;
 #endif
 
-#include <h323/h323.h>
-#include <h323/gkclient.h>
-#include <h323/gkserver.h>
-#include <sip/sip.h>
-#include <lids/lidep.h>
-#include <lids/capi_ep.h>
-#include <ep/ivr.h>
-#include <ep/opalmixer.h>
-#include <opal/patch.h>
-
 
 // End of File ///////////////////////////////////////////////////////////////
