@@ -23,14 +23,14 @@
 *
 * Contributor(s): ______________________________________.
 *
-* $Revision: 28977 $
+* $Revision: 33537 $
 * $Author: rjongbloed $
-* $Date: 2013-01-23 22:03:40 -0600 (Wed, 23 Jan 2013) $
+* $Date: 2015-04-28 04:23:13 +0100 (Tue, 28 Apr 2015) $
 */
 
 #ifndef SBC_DISABLE_PTLIB
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #endif
 
 #include <codec/g711a1_plc.h>
@@ -43,7 +43,6 @@
 static const int CORR_DECIMATION_RATE=4000;     /**< decimation to a rate of [Hz] */
 static const int corr_len_ms=20;       /**< Correlation length [ms] */
 
-static const int OVERLAPADD=4;             /**< length of the overlapadd windows [ms] */
 static const double CORR_MINPOWER=250./80.; /**< minimum power */
 
 static const int CONCEAL_ATTENUATION_PERIOD1=10;      /**< first 10 ms are not faded out */
@@ -56,11 +55,8 @@ static const int TRANSITION_START=10;    /**< after 10ms of loss, the transition
 static const double TRANSITION_RATIO=0.4;  /**< for each 10ms of loss, the transition period will be prolonged by 4ms (ITU G.711 I.2.7) */
 static const int TRANSITION_MAX=10;      /**< the transition period will have a maximal length of 10ms (ITU G.711 I.2.7 [ms] */
 
-static const double PITCH_LOW=66.6;/**< minimum allowed channel[c].pitch. default 66 Hz [Hz] */
-static const double PITCH_HIGH=200;/**< maximum allowed channel[c].pitch. default 200 Hz [Hz] */
 
-
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && _MSC_VER < 1800
 __inline double round(const double & value)
 {
   return value < 0 ? floor(value-0.5) : ceil(value+0.5);
@@ -194,7 +190,7 @@ void OpalG711_PLC::scalespeech(short *inout, int c, int size, bool decay) const
     }
     else {
       if(g<1)
-	inout[index] = short(round(inout[index] * g));
+        inout[index] = short(round(inout[index] * g));
       if(decay)
         g -= attenincr;
     }
