@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21988 $
+ * $Revision: 30368 $
  * $Author: rjongbloed $
- * $Date: 2009-02-04 06:28:03 +0000 (Wed, 04 Feb 2009) $
+ * $Date: 2013-08-23 03:43:58 +0100 (Fri, 23 Aug 2013) $
  */
 
 #include <ptlib.h>
@@ -33,7 +33,7 @@
 #pragma implementation "g711codec.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <codec/g711codec.h>
 
