@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27734 $
+ * $Revision: 32253 $
  * $Author: rjongbloed $
- * $Date: 2012-05-29 02:43:41 -0500 (Tue, 29 May 2012) $
+ * $Date: 2014-06-28 08:33:35 +0100 (Sat, 28 Jun 2014) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/mediafmt.h>
 #include <h323/h323caps.h>
@@ -65,6 +65,7 @@
   static const char G726_##rate##K_EncodingName[] = "G726-" #rate; \
   const OpalAudioFormat & GetOpalG726_##rate##K() \
   { \
+    static OpalAudioFormat const plugin(G726_##rate##K_FormatName); if (plugin.IsValid()) return plugin; \
     static const OpalAudioFormat format(G726_##rate##K_FormatName, RTP_DataFrame::DynamicBase, \
                                         G726_##rate##K_EncodingName, bits, 8, 240, 30, 256, 8000); \
     CAPABILITY(rate); \
