@@ -24,15 +24,17 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 32253 $
+ * $Author: rjongbloed $
+ * $Date: 2014-06-28 08:33:35 +0100 (Sat, 28 Jun 2014) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/mediafmt.h>
+#include <h323/h323caps.h>
+#include <asn/h245.h>
 
 
 #define new PNEW
@@ -40,10 +42,41 @@
 
 /////////////////////////////////////////////////////////////////////////////
 
+#if OPAL_H323
+class H323_G728Capability : public H323AudioCapability
+{
+  public:
+    virtual PObject * Clone() const
+    {
+      return new H323_G728Capability(*this);
+    }
+
+    virtual unsigned GetSubType() const
+    {
+      return H245_AudioCapability::e_g728;
+    }
+
+    virtual PString GetFormatName() const
+    {
+      return OpalG728;
+    }
+};
+#endif // OPAL_H323
+
+
 const OpalAudioFormat & GetOpalG728()
 {
-  static const OpalAudioFormat G728(OPAL_G728, RTP_DataFrame::G728,  "G728", 5, 20, 100, 10, 256, 8000);
-  return G728;
+  static OpalAudioFormat const plugin(OPAL_G728);
+  if (plugin.IsValid())
+    return plugin;
+
+  static const OpalAudioFormat G728_Format(OPAL_G728, RTP_DataFrame::G728,  "G728", 5, 20, 100, 10, 256, 8000);
+
+#if OPAL_H323
+  static H323CapabilityFactory::Worker<H323_G728Capability> G728_Factory(OPAL_G728, true);
+#endif // OPAL_H323
+
+  return G728_Format;
 }
 
 
