@@ -24,15 +24,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28413 $
+ * $Revision: 34175 $
  * $Author: rjongbloed $
- * $Date: 2012-09-26 21:38:00 -0500 (Wed, 26 Sep 2012) $
+ * $Date: 2015-10-10 11:27:32 +0100 (Sat, 10 Oct 2015) $
  */
 
 #include <codec/opalplugin.hpp>
 #include <codec/known.h>
 
 #include <stdio.h>
+#include <vector>
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -46,6 +47,7 @@
 static const char H264_Mode0_FormatName[] = OPAL_H264_MODE0;
 static const char H264_Mode1_FormatName[] = OPAL_H264_MODE1;
 static const char H264_High_FormatName[]  = OPAL_H264_High;
+static const char H264_Flash_FormatName[] = OPAL_H264_Flash;
 
 static const char H264EncodingName[] = "H264";
 
@@ -53,7 +55,7 @@ static const char H264EncodingName[] = "H264";
 #define DefaultProfileStr          H264_PROFILE_STR_BASELINE
 #define DefaultProfileInt          H264_PROFILE_INT_BASELINE
 #define DefaultProfileH241         64
-#define DefaultLevelStr            H264_LEVEL_STR_3_1
+#define DefaultLevelStr            H264_LEVEL_STR_4
 #define DefaultLevelInt            31
 #define DefaultLevelH241           71
 #define DefaultSDPProfileAndLevel  "42801f"
@@ -88,6 +90,7 @@ static const char H264EncodingName[] = "H264";
 #define H264_LEVEL_STR_4_2  "4.2"
 #define H264_LEVEL_STR_5    "5"
 #define H264_LEVEL_STR_5_1  "5.1"
+#define H264_LEVEL_STR_5_2  "5.2"
 
 
 enum
@@ -107,9 +110,9 @@ enum
 };
 
 
-static char const ProfileName[] = "Profile";
-static char const LevelName[] = "Level";
-static char const MaxNALUSizeName[] = "Max NALU Size";
+static char const ProfileName[] = PLUGINCODEC_OPTION_PROFILE;
+static char const LevelName[] = PLUGINCODEC_OPTION_LEVEL;
+static char const MaxNALUSizeName[] = PLUGINCODEC_OPTION_MAX_NALU_SIZE;
 
 static char const H241ProfilesName[] = "H.241 Profile Mask";
 static char const H241LevelName[] = "H.241 Level";
@@ -117,6 +120,7 @@ static char const MaxMBPS_H241_Name[] = "H.241 Max MBPS";
 static char const MaxSMBPS_H241_Name[] = "H.241 Max SMBPS";
 static char const MaxFS_H241_Name[] = "H.241 Max FS";
 static char const MaxBR_H241_Name[] = "H.241 Max BR";
+static char const H241ForcedName[] = "H.241 Forced";
 
 static char const SDPProfileAndLevelName[] = "SIP/SDP Profile & Level";
 static char const SDPProfileAndLevelFMTPName[] = "profile-level-id";
@@ -129,11 +133,29 @@ static char const MaxFS_SDP_Name[] = "SIP/SDP Max FS";
 static char const MaxFS_FMTPName[] = "max-fs";
 static char const MaxBR_SDP_Name[] = "SIP/SDP Max BR";
 static char const MaxBR_FMTPName[] = "max-br";
+static char const SDPForcedName[] = "SDP Forced";
+
 static char const MaxNALUSizeFMTPName[] = "max-rcmd-nalu-size";
-static const char PacketizationModeName[] = "Packetization Mode";
+static const char PacketizationModeName[] = PLUGINCODEC_OPTION_H264_PACKET_MODE;
 static const char PacketizationFMTPName[] = "packetization-mode";
 
 
+/* Need to make missing MBPS parameter larger than all possible values so when
+   "min merge" is done a real value always take precedence. Thus the max
+   value defined here is actually one larger than the real max value */
+#define MAX_MBPS_SDP  983041
+#define MAX_MBPS_H241 1967
+#define MAX_FS_SDP    36865
+#define MAX_FS_H241   145
+#define MAX_BR_SDP    240001
+#define MAX_BR_H241   9601
+
+#define SCALE_MBPS_H241 500
+#define SCALE_FS_H241   256
+#define SCALE_BR_H241   30000
+#define SCALE_BR_SDP    1000
+
+
 static struct {
   char     m_Name[9];
   unsigned m_H264;
@@ -157,26 +179,31 @@ static struct LevelInfoStruct {
   unsigned m_MaxBitRate;
 } const LevelInfo[] = {
   // Table A-1 from H.264 specification
-  //                     Lev  Con  H241    FS   W/H    MBPS         BR
-  { H264_LEVEL_STR_1,    10, 0x00,  15,    99,  448,   1485,     64000 },
-  { H264_LEVEL_STR_1_b,  11, 0x10,  19,    99,  448,   1485,    128000 },
-  { H264_LEVEL_STR_1_1,  11, 0x00,  22,   396,  896,   3000,    192000 },
-  { H264_LEVEL_STR_1_2,  12, 0x00,  29,   396,  896,   6000,    384000 },
-  { H264_LEVEL_STR_1_3,  13, 0x00,  36,   396,  896,  11880,    768000 },
-  { H264_LEVEL_STR_2,    20, 0x00,  43,   396,  896,  11880,   2000000 },
-  { H264_LEVEL_STR_2_1,  21, 0x00,  50,   792, 1264,  19800,   4000000 },
-  { H264_LEVEL_STR_2_2,  22, 0x00,  57,  1620, 1808,  20250,   4000000 },
-  { H264_LEVEL_STR_3,    30, 0x00,  64,  1620, 1808,  40500,  10000000 },
-  { H264_LEVEL_STR_3_1,  31, 0x00,  71,  3600, 2704, 108000,  14000000 },
-  { H264_LEVEL_STR_3_2,  32, 0x00,  78,  5120, 3232, 216000,  20000000 },
-  { H264_LEVEL_STR_4,    40, 0x00,  85,  8192, 4096, 245760,  25000000 },
-  { H264_LEVEL_STR_4_1,  41, 0x00,  92,  8292, 4112, 245760,  62500000 },
-  { H264_LEVEL_STR_4_2,  42, 0x00,  99,  8704, 4208, 522340,  62500000 },
-  { H264_LEVEL_STR_5,    50, 0x00, 106, 22080, 6720, 589824, 135000000 },
-  { H264_LEVEL_STR_5_1,  51, 0x00, 113, 36864, 8320, 983040, 240000000 }
+  //                     Lev  Con  H241    FS   W/H     MBPS             BR
+  { H264_LEVEL_STR_1,    10, 0x00,  15,    99,  448,    1485,     64 * 1200 },
+  { H264_LEVEL_STR_1_b,  11, 0x10,  19,    99,  448,    1485,    128 * 1200 },
+  { H264_LEVEL_STR_1_1,  11, 0x00,  22,   396,  896,    3000,    192 * 1200 },
+  { H264_LEVEL_STR_1_2,  12, 0x00,  29,   396,  896,    6000,    384 * 1200 },
+  { H264_LEVEL_STR_1_3,  13, 0x00,  36,   396,  896,   11880,    768 * 1200 },
+  { H264_LEVEL_STR_2,    20, 0x00,  43,   396,  896,   11880,   2000 * 1200 },
+  { H264_LEVEL_STR_2_1,  21, 0x00,  50,   792, 1264,   19800,   4000 * 1200 },
+  { H264_LEVEL_STR_2_2,  22, 0x00,  57,  1620, 1808,   20250,   4000 * 1200 },
+  { H264_LEVEL_STR_3,    30, 0x00,  64,  1620, 1808,   40500,  10000 * 1200 },
+  { H264_LEVEL_STR_3_1,  31, 0x00,  71,  3600, 2704,  108000,  14000 * 1200 },
+  { H264_LEVEL_STR_3_2,  32, 0x00,  78,  5120, 3232,  216000,  20000 * 1200 },
+  { H264_LEVEL_STR_4,    40, 0x00,  85,  8192, 4096,  245760,  20000 * 1200 },
+  { H264_LEVEL_STR_4_1,  41, 0x00,  92,  8192, 4096,  245760,  50000 * 1200 },
+  { H264_LEVEL_STR_4_2,  42, 0x00,  99,  8704, 4208,  522240,  50000 * 1200 },
+  { H264_LEVEL_STR_5,    50, 0x00, 106, 22080, 6720,  589824, 135000 * 1200 },
+  { H264_LEVEL_STR_5_1,  51, 0x00, 113, 36864, 8320,  983040, 240000 * 1200 },
+  { H264_LEVEL_STR_5_2,  52, 0x00, 120, 36864, 8320, 2073600, 240000 * 1200 },
 };
 
 
+static const size_t FlashHeaderSize = 5;
+static const uint8_t FlashSPS_PPS[FlashHeaderSize] = { 0x17, 0, 0, 0, 0 };
+
+
 static int hexdigit(char ch)
 {
   if (ch < '0')
@@ -201,25 +228,6 @@ static unsigned hexbyte(const char * hex)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-static void ClampSizes(unsigned maxWidth,
-                       unsigned maxHeight,
-                       unsigned & maxFrameSize,
-                       PluginCodec_OptionMap & original,
-                       PluginCodec_OptionMap & changed)
-{
-  if (PluginCodec_Utilities::ClampResolution(maxWidth, maxHeight, maxFrameSize)) {
-    PTRACE(5, MY_CODEC_LOG, "Reduced max resolution to " << maxWidth << 'x' << maxHeight);
-  }
-
-  PluginCodec_Utilities::ClampMax(maxWidth,  original, changed, PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH);
-  PluginCodec_Utilities::ClampMax(maxHeight, original, changed, PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT);
-  PluginCodec_Utilities::ClampMax(maxWidth,  original, changed, PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH);
-  PluginCodec_Utilities::ClampMax(maxHeight, original, changed, PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);
-  PluginCodec_Utilities::ClampMax(maxWidth,  original, changed, PLUGINCODEC_OPTION_FRAME_WIDTH);
-  PluginCodec_Utilities::ClampMax(maxHeight, original, changed, PLUGINCODEC_OPTION_FRAME_HEIGHT);
-}
-
-
 static bool MyToNormalised(PluginCodec_OptionMap & original, PluginCodec_OptionMap & changed)
 {
   size_t levelIndex = 0;
@@ -242,15 +250,19 @@ static bool MyToNormalised(PluginCodec_OptionMap & original, PluginCodec_OptionM
         break;
     }
 
-    maxMBPS = original.GetUnsigned(MaxMBPS_H241_Name)*500;
-    maxSMBPS = original.GetUnsigned(MaxSMBPS_H241_Name)*500;
-    maxFrameSizeInMB = original.GetUnsigned(MaxFS_H241_Name)*256;
-    maxBitRate = original.GetUnsigned(MaxBR_H241_Name)*25000;
+    maxMBPS = (original.GetUnsigned(MaxMBPS_H241_Name)%MAX_MBPS_H241)*SCALE_MBPS_H241;
+    maxSMBPS = (original.GetUnsigned(MaxSMBPS_H241_Name)%MAX_MBPS_H241)*SCALE_MBPS_H241;
+    maxFrameSizeInMB = (original.GetUnsigned(MaxFS_H241_Name)%MAX_FS_H241)*SCALE_FS_H241;
+    maxBitRate = (original.GetUnsigned(MaxBR_H241_Name)%MAX_BR_H241)*SCALE_BR_H241;
 
-    PluginCodec_Utilities::Change(maxMBPS, original, changed, MaxMBPS_SDP_Name); 
-    PluginCodec_Utilities::Change(maxSMBPS, original, changed, MaxSMBPS_SDP_Name); 
-    PluginCodec_Utilities::Change(maxFrameSizeInMB, original, changed, MaxFS_SDP_Name); 
-    PluginCodec_Utilities::Change((maxBitRate+999)/1000, original, changed, MaxBR_SDP_Name); 
+    if (maxMBPS > 0)
+      PluginCodec_Utilities::Change(maxMBPS, original, changed, MaxMBPS_SDP_Name); 
+    if (maxSMBPS > 0)
+      PluginCodec_Utilities::Change(maxSMBPS, original, changed, MaxSMBPS_SDP_Name);
+    if (maxFrameSizeInMB > 0)
+      PluginCodec_Utilities::Change(maxFrameSizeInMB, original, changed, MaxFS_SDP_Name);
+    if (maxBitRate > 0)
+      PluginCodec_Utilities::Change((maxBitRate+SCALE_BR_SDP-1)/SCALE_BR_SDP, original, changed, MaxBR_SDP_Name); 
   }
   else if (original[PLUGINCODEC_OPTION_PROTOCOL] == PLUGINCODEC_OPTION_PROTOCOL_SIP) {
     std::string sdpProfLevel = original[SDPProfileAndLevelName];
@@ -283,15 +295,19 @@ static bool MyToNormalised(PluginCodec_OptionMap & original, PluginCodec_OptionM
       if ((sdpConstraints & 0x10) == LevelInfo[levelIndex].m_constraints)
         break;
     }
-    maxMBPS = original.GetUnsigned(MaxMBPS_SDP_Name);
-    maxSMBPS = original.GetUnsigned(MaxSMBPS_SDP_Name);
-    maxFrameSizeInMB = original.GetUnsigned(MaxFS_SDP_Name);
-    maxBitRate = original.GetUnsigned(MaxBR_SDP_Name)*1000;
+    maxMBPS = original.GetUnsigned(MaxMBPS_SDP_Name)%MAX_MBPS_SDP;
+    maxSMBPS = original.GetUnsigned(MaxSMBPS_SDP_Name)%MAX_MBPS_SDP;
+    maxFrameSizeInMB = original.GetUnsigned(MaxFS_SDP_Name)%MAX_FS_SDP;
+    maxBitRate = (original.GetUnsigned(MaxBR_SDP_Name)%MAX_BR_SDP)*SCALE_BR_SDP;
 
-    PluginCodec_Utilities::Change((maxMBPS+499)/500, original, changed, MaxMBPS_H241_Name); 
-    PluginCodec_Utilities::Change((maxSMBPS+499)/500, original, changed, MaxSMBPS_H241_Name); 
-    PluginCodec_Utilities::Change((maxFrameSizeInMB+255)/256, original, changed, MaxFS_H241_Name); 
-    PluginCodec_Utilities::Change((maxBitRate+24999)/25000, original, changed, MaxBR_H241_Name); 
+    if (maxMBPS > 0)
+      PluginCodec_Utilities::Change((maxMBPS+SCALE_MBPS_H241-1)/SCALE_MBPS_H241, original, changed, MaxMBPS_H241_Name); 
+    if (maxSMBPS > 0)
+      PluginCodec_Utilities::Change((maxSMBPS+SCALE_MBPS_H241-1)/SCALE_MBPS_H241, original, changed, MaxSMBPS_H241_Name);
+    if (maxFrameSizeInMB > 0)
+      PluginCodec_Utilities::Change((maxFrameSizeInMB+SCALE_FS_H241-1)/SCALE_FS_H241, original, changed, MaxFS_H241_Name); 
+    if (maxBitRate > 0)
+      PluginCodec_Utilities::Change((maxBitRate+SCALE_BR_H241-1)/SCALE_BR_H241, original, changed, MaxBR_H241_Name); 
   }
   else {
     std::string profileName = original[ProfileName];
@@ -316,10 +332,10 @@ static bool MyToNormalised(PluginCodec_OptionMap & original, PluginCodec_OptionM
 
   if (maxFrameSizeInMB < LevelInfo[levelIndex].m_MaxFrameSize)
     maxFrameSizeInMB = LevelInfo[levelIndex].m_MaxFrameSize;
-  ClampSizes(original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH),
-             original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT),
-             maxFrameSizeInMB,
-             original, changed);
+  PluginCodec_Utilities::ClampResolution(original, changed,
+																			   original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH),
+																				 original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT),
+																				 maxFrameSizeInMB);
 
   // Frame rate
   if (maxMBPS < LevelInfo[levelIndex].m_MaxMBPS)
@@ -339,6 +355,12 @@ static bool MyToNormalised(PluginCodec_OptionMap & original, PluginCodec_OptionM
 
 static bool MyToCustomised(PluginCodec_OptionMap & original, PluginCodec_OptionMap & changed)
 {
+  if ((original.GetUnsigned(SDPForcedName)  != 0 && original[PLUGINCODEC_OPTION_PROTOCOL] == PLUGINCODEC_OPTION_PROTOCOL_SIP) ||
+      (original.GetUnsigned(H241ForcedName) != 0 && original[PLUGINCODEC_OPTION_PROTOCOL] == PLUGINCODEC_OPTION_PROTOCOL_H323)) {
+    PTRACE(4, MY_CODEC_LOG, "Forcing " << original[PLUGINCODEC_OPTION_PROTOCOL] << " parameters, not customising");
+    return true;
+  }
+
   // Determine the profile
   std::string str = original[ProfileName];
   if (str.empty())
@@ -357,26 +379,32 @@ static bool MyToCustomised(PluginCodec_OptionMap & original, PluginCodec_OptionM
   if (str.empty())
     str = H264_LEVEL_STR_1_3;
 
-  size_t levelIndex = sizeof(LevelInfo)/sizeof(LevelInfo[0]);
-  while (--levelIndex > 0) {
-    if (str == LevelInfo[levelIndex].m_Name)
-      break;
-  }
-  PTRACE(5, MY_CODEC_LOG, "Level \"" << str << "\" selected index " << levelIndex);
+  size_t levelIndex = sizeof(LevelInfo)/sizeof(LevelInfo[0])-1;
+  while (levelIndex > 0 && str != LevelInfo[levelIndex].m_Name)
+    --levelIndex;
 
   /* While we have selected the desired level, we may need to adjust it
       further due to resolution restrictions. This is due to the fact that
-      we have no other mechnism to prevent the remote from sending, say
+      we have no other mechnism to prevent the remote from sending, say,
       CIF when we only support QCIF.
     */
   unsigned maxWidth = original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH);
   unsigned maxHeight = original.GetUnsigned(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);
-  unsigned maxFrameSizeInMB = PluginCodec_Utilities::GetMacroBlocks(maxWidth, maxHeight);
-  if (maxFrameSizeInMB > 0) {
-    while (levelIndex > 0 && maxFrameSizeInMB < LevelInfo[levelIndex].m_MaxFrameSize)
+  unsigned frameTime = original.GetUnsigned(PLUGINCODEC_OPTION_FRAME_TIME);
+  unsigned maxMacroBlocks = PluginCodec_Utilities::GetMacroBlocks(maxWidth, maxHeight);
+  unsigned macroBlocksPerSecond = maxMacroBlocks*PLUGINCODEC_VIDEO_CLOCK/frameTime;
+  if (maxMacroBlocks > 0) {
+    while (levelIndex > 0 &&
+            (LevelInfo[levelIndex].m_MaxFrameSize > maxMacroBlocks ||
+             LevelInfo[levelIndex].m_MaxMBPS > macroBlocksPerSecond))
       --levelIndex;
+    PTRACE(4, MY_CODEC_LOG, "Max resolution " << maxWidth << 'x' << maxHeight
+           << " and frame rate " << frameTime << " (" << (PLUGINCODEC_VIDEO_CLOCK/frameTime) << "fps)"
+              " selected index " << levelIndex);
+  }
+  else {
+    PTRACE(4, MY_CODEC_LOG, "Level \"" << str << "\" selected index " << levelIndex);
   }
-  PTRACE(5, MY_CODEC_LOG, "Max resolution " << maxWidth << 'x' << maxHeight << " selected index " << levelIndex);
 
   // set the new level
   PluginCodec_Utilities::Change(LevelInfo[levelIndex].m_H241, original, changed, H241LevelName);
@@ -390,42 +418,177 @@ static bool MyToCustomised(PluginCodec_OptionMap & original, PluginCodec_OptionM
   PluginCodec_Utilities::Change(sdpProfLevel, original, changed, SDPProfileAndLevelName);
 
   // Clamp other variables (width/height etc) according to the adjusted profile/level
-  ClampSizes(maxWidth, maxHeight, maxFrameSizeInMB, original, changed);
+  PluginCodec_Utilities::ClampResolution(original, changed, maxWidth, maxHeight, maxMacroBlocks);
 
   // Do this afer the clamping, maxFrameSizeInMB may change
-  if (maxFrameSizeInMB > LevelInfo[levelIndex].m_MaxFrameSize) {
-    PluginCodec_Utilities::ClampMax(maxFrameSizeInMB, original, changed, MaxFS_SDP_Name, true);
-    PluginCodec_Utilities::ClampMax((maxFrameSizeInMB+255)/256, original, changed, MaxFS_H241_Name, true);
+  if (maxMacroBlocks > LevelInfo[levelIndex].m_MaxFrameSize) {
+    PluginCodec_Utilities::ClampMax(maxMacroBlocks, original, changed, MaxFS_SDP_Name, true);
+    PluginCodec_Utilities::ClampMax((maxMacroBlocks+SCALE_FS_H241-1)/SCALE_FS_H241, original, changed, MaxFS_H241_Name, true);
   }
   else {
-    PluginCodec_Utilities::Change(0U, original, changed, MaxFS_SDP_Name);
-    PluginCodec_Utilities::Change(0U, original, changed, MaxFS_H241_Name);
+    PluginCodec_Utilities::Change(MAX_FS_SDP, original, changed, MaxFS_SDP_Name);
+    PluginCodec_Utilities::Change(MAX_FS_H241, original, changed, MaxFS_H241_Name);
   }
 
   // Set exception to bit rate if necessary
   unsigned bitRate = original.GetUnsigned(PLUGINCODEC_OPTION_MAX_BIT_RATE);
   if (bitRate > LevelInfo[levelIndex].m_MaxBitRate) {
-    PluginCodec_Utilities::ClampMax((bitRate+999)/1000, original, changed, MaxBR_SDP_Name, true);
-    PluginCodec_Utilities::ClampMax((bitRate+24999)/25000, original, changed, MaxBR_H241_Name, true);
+    PluginCodec_Utilities::ClampMax((bitRate+SCALE_BR_SDP-1)/SCALE_BR_SDP, original, changed, MaxBR_SDP_Name, true);
+    PluginCodec_Utilities::ClampMax((bitRate+SCALE_BR_H241-1)/SCALE_BR_H241, original, changed, MaxBR_H241_Name, true);
   }
   else {
-    PluginCodec_Utilities::Change(0U, original, changed, MaxBR_SDP_Name);
-    PluginCodec_Utilities::Change(0U, original, changed, MaxBR_H241_Name);
+    PluginCodec_Utilities::Change(MAX_BR_SDP, original, changed, MaxBR_SDP_Name);
+    PluginCodec_Utilities::Change(MAX_BR_H241, original, changed, MaxBR_H241_Name);
   }
 
   // Set exception to frame rate if necessary
-  unsigned mbps = maxFrameSizeInMB*PLUGINCODEC_VIDEO_CLOCK/original.GetUnsigned(PLUGINCODEC_OPTION_FRAME_TIME);
-  if (mbps > LevelInfo[levelIndex].m_MaxMBPS) {
-    PluginCodec_Utilities::ClampMax(mbps, original, changed, MaxMBPS_SDP_Name, true);
-    PluginCodec_Utilities::ClampMax((mbps+499)/500, original, changed, MaxMBPS_H241_Name, true);
+  if (macroBlocksPerSecond > LevelInfo[levelIndex].m_MaxMBPS) {
+    PluginCodec_Utilities::ClampMax(macroBlocksPerSecond, original, changed, MaxMBPS_SDP_Name, true);
+    PluginCodec_Utilities::ClampMax((macroBlocksPerSecond+SCALE_MBPS_H241-1)/SCALE_MBPS_H241, original, changed, MaxMBPS_H241_Name, true);
   }
   else {
-    PluginCodec_Utilities::Change(0U, original, changed, MaxMBPS_SDP_Name);
-    PluginCodec_Utilities::Change(0U, original, changed, MaxMBPS_H241_Name);
+    PluginCodec_Utilities::Change(MAX_MBPS_SDP, original, changed, MaxMBPS_SDP_Name);
+    PluginCodec_Utilities::Change(MAX_MBPS_H241, original, changed, MaxMBPS_H241_Name);
   }
 
   return true;
 }
 
 
+class H264FlashPacketizer
+{
+  protected:
+    std::vector<unsigned char> m_naluBuffer;
+    bool m_firstFrame;
+
+    H264FlashPacketizer()
+      : m_firstFrame(true)
+    {
+    }
+
+    virtual ~H264FlashPacketizer()
+    {
+    }
+
+
+    virtual bool GetNALU(const void * fromPtr, unsigned & fromLen, const uint8_t * & naluPtr, unsigned & naluLen, unsigned & flags) = 0;
+
+    static const size_t HeaderSize = 5+5+6+4;
+
+    bool FlashTranscode(const void * fromPtr,
+                          unsigned & fromLen,
+                              void * toPtr,
+                          unsigned & toLen,
+                          unsigned & flags)
+    {
+      PluginCodec_RTP rtp(toPtr, toLen);
+      uint8_t * pBuffer = rtp.GetPayloadPtr();
+
+      const uint8_t * naluPtr;
+      unsigned naluLen;
+      if (!GetNALU(fromPtr, fromLen, naluPtr, naluLen, flags))
+        return false;
+
+      bool bKey = (flags&PluginCodec_ReturnCoderIFrame) != 0;
+
+      if (bKey && (naluPtr[0] & 0x1f) == 0x07) { // SPS
+        toLen -= 5+5+3+3;
+        if (naluLen > toLen)
+          return false;
+
+        /* Pre-amble */
+        *pBuffer++ = 0x17;
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x00;
+
+        /* Start header */
+        *pBuffer++ = 0x01; /* version */
+        *pBuffer++ = naluPtr[1]; /* profile */
+        *pBuffer++ = naluPtr[2]; /* profile compat */
+        *pBuffer++ = naluPtr[3]; /* level */
+        *pBuffer++ = 0xFF; /* 6 bits reserved (111111) + 2 bits nal size length - 1 (11) */
+
+        /* Write SPSs */
+        *pBuffer++ = 0xE1; // Only 1 for now
+        *pBuffer++ = (uint8_t)(naluLen >> 8);
+        *pBuffer++ = (uint8_t) naluLen;
+        memcpy(pBuffer, naluPtr, naluLen);
+        pBuffer += naluLen;
+        toLen -= naluLen;
+
+        // We assume next thing is SPS
+        if (!GetNALU(fromPtr, fromLen, naluPtr, naluLen, flags))
+            return false;
+
+        if (naluLen > toLen)
+          return false;
+
+        /* Write PPSs */
+        *pBuffer++ = 0x01; // Only 1 for now
+        *pBuffer++ = (uint8_t)(naluLen >> 8);
+        *pBuffer++ = (uint8_t) naluLen;
+        memcpy(pBuffer, naluPtr, naluLen);
+        pBuffer += naluLen;
+      }
+      else {
+        toLen -= HeaderSize;
+        if (naluLen > toLen)
+          return false;
+
+        /* Pre-amble */
+        *pBuffer++ = bKey ? 0x17 : 0x27;
+        *pBuffer++ = 0x01;
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x00;
+
+        if (bKey && !m_firstFrame) {
+          /* End of previous GOP */
+          *pBuffer++ = 0x00;
+          *pBuffer++ = 0x00;
+          *pBuffer++ = 0x00;
+          *pBuffer++ = 0x01;
+          *pBuffer++ = 0x0A;
+        }
+
+        /* Access unit delimiter */
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x00;
+        *pBuffer++ = 0x02;
+        *pBuffer++ = 0x09;
+        *pBuffer++ = bKey ? 0x10 : 0x30;
+
+        for (;;) {
+          *pBuffer++ = (uint8_t)(naluLen >> 24);
+          *pBuffer++ = (uint8_t)(naluLen >> 16);
+          *pBuffer++ = (uint8_t)(naluLen >>  8);
+          *pBuffer++ = (uint8_t) naluLen;
+          memcpy(pBuffer, naluPtr, naluLen);
+          pBuffer += naluLen;
+          toLen -= naluLen+4;
+
+          if ((flags&PluginCodec_ReturnCoderLastFrame) != 0)
+            break;
+
+          if (!GetNALU(fromPtr, fromLen, naluPtr, naluLen, flags))
+            return false;
+          if (naluLen+4 > toLen)
+            return false;
+        }
+
+        m_firstFrame = false;
+      }
+
+      if (!rtp.SetPayloadSize(pBuffer - rtp.GetPayloadPtr()))
+        return false;
+
+      toLen = rtp.GetPacketSize();
+      return true;
+    }
+};
+
+
 // End of File ///////////////////////////////////////////////////////////////
