@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27745 $
+ * $Revision: 32253 $
  * $Author: rjongbloed $
- * $Date: 2012-06-01 02:50:13 -0500 (Fri, 01 Jun 2012) $
+ * $Date: 2014-06-28 08:33:35 +0100 (Sat, 28 Jun 2014) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/mediafmt.h>
 #include <codec/opalplugin.h>
@@ -116,6 +116,10 @@ extern const char iLBC_Identifier[] = OpalPluginCodec_Identifer_iLBC;
 
 const OpalAudioFormat & GetOpaliLBC()
 {
+  static OpalAudioFormat const plugin(OPAL_iLBC);
+  if (plugin.IsValid())
+    return plugin;
+
   static OpalAudioFormat const iLBC_Format(new OpaliLBCFormat);
 
 #if OPAL_H323
