@@ -24,9 +24,9 @@
  *
  * Contributor(s): Post Increment
  *
- * $Revision: 27415 $
+ * $Revision: 30368 $
  * $Author: rjongbloed $
- * $Date: 2012-04-02 22:20:52 -0500 (Mon, 02 Apr 2012) $
+ * $Date: 2013-08-23 03:43:58 +0100 (Fri, 23 Aug 2013) $
  */
 
 #include <ptlib.h>
@@ -37,7 +37,7 @@ using namespace std;
 #pragma implementation "ratectl.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #include <codec/ratectl.h>
 
 #if OPAL_VIDEO
