@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29937 $
+ * $Revision: 34197 $
  * $Author: rjongbloed $
- * $Date: 2013-06-12 21:03:10 -0500 (Wed, 12 Jun 2013) $
+ * $Date: 2015-10-18 15:06:39 +0100 (Sun, 18 Oct 2015) $
  */
 
 #include <ptlib.h>
@@ -34,10 +34,13 @@
 #pragma implementation "localep.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <ep/localep.h>
 #include <opal/call.h>
+#include <im/rfc4103.h>
+#include <h224/h224handler.h>
+#include <h224/h281handler.h>
 
 
 #define new PNEW
@@ -45,14 +48,21 @@
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalLocalEndPoint::OpalLocalEndPoint(OpalManager & mgr, const char * prefix)
-  : OpalEndPoint(mgr, prefix, CanTerminateCall)
+OpalLocalEndPoint::OpalLocalEndPoint(OpalManager & mgr, const char * prefix, bool useCallbacks)
+  : OpalEndPoint(mgr, prefix, NoAttributes)
   , m_deferredAlerting(false)
   , m_deferredAnswer(false)
+  , m_pauseTransmitMediaOnHold(true)
   , m_defaultAudioSynchronicity(e_Synchronous)
   , m_defaultVideoSourceSynchronicity(e_Synchronous)
 {
   PTRACE(3, "LocalEP\tCreated endpoint.");
+
+  if (useCallbacks) {
+    OpalMediaTypeList mediaTypes = OpalMediaType::GetList();
+    for (OpalMediaTypeList::iterator it = mediaTypes.begin(); it != mediaTypes.end(); ++it)
+      m_useCallback[*it] = UseSourceCallback|UseSinkCallback;
+  }
 }
 
 
@@ -78,7 +88,7 @@ PSafePtr<OpalConnection> OpalLocalEndPoint::MakeConnection(OpalCall & call,
 }
 
 
-OpalLocalConnection * OpalLocalEndPoint::CreateConnection(OpalCall & call, 
+OpalLocalConnection * OpalLocalEndPoint::CreateConnection(OpalCall & call,
                                                               void * userData,
                                                             unsigned options,
                                      OpalConnection::StringOptions * stringOptions)
@@ -93,9 +103,9 @@ bool OpalLocalEndPoint::OnOutgoingSetUp(const OpalLocalConnection & /*connection
 }
 
 
-bool OpalLocalEndPoint::OnOutgoingCall(const OpalLocalConnection & /*connection*/)
+bool OpalLocalEndPoint::OnOutgoingCall(const OpalLocalConnection & connection)
 {
-  return true;
+  return manager.OnLocalOutgoingCall(connection);
 }
 
 
@@ -103,7 +113,7 @@ bool OpalLocalEndPoint::OnIncomingCall(OpalLocalConnection & connection)
 {
   if (!m_deferredAnswer)
     connection.AcceptIncoming();
-  return true;
+  return manager.OnLocalIncomingCall(connection);
 }
 
 
@@ -161,6 +171,23 @@ bool OpalLocalEndPoint::OnUserInput(const OpalLocalConnection &, const PString &
 }
 
 
+bool OpalLocalEndPoint::UseCallback(const OpalMediaFormat & mediaFormat, bool isSource) const
+{
+  OpalLocalEndPoint::CallbackMap::const_iterator it = m_useCallback.find(mediaFormat.GetMediaType());
+  return it != m_useCallback.end() && (it->second & (isSource ? UseSourceCallback : UseSinkCallback));
+}
+
+
+bool OpalLocalEndPoint::SetCallbackUsage(const OpalMediaType & mediaType, CallbackUsage usage)
+{
+  if (mediaType.empty())
+    return false;
+
+  m_useCallback[mediaType] |= usage;
+  return true;
+}
+
+
 bool OpalLocalEndPoint::OnReadMediaFrame(const OpalLocalConnection & /*connection*/,
                                          const OpalMediaStream & /*mediaStream*/,
                                          RTP_DataFrame & /*frame*/)
@@ -200,15 +227,40 @@ bool OpalLocalEndPoint::OnWriteMediaData(const OpalLocalConnection & /*connectio
 }
 
 
+#if OPAL_VIDEO
+
+bool OpalLocalEndPoint::CreateVideoInputDevice(const OpalConnection & connection,
+                                               const OpalMediaFormat & mediaFormat,
+                                               PVideoInputDevice * & device,
+                                               bool & autoDelete)
+{
+  return manager.CreateVideoInputDevice(connection, mediaFormat, device, autoDelete);
+}
+
+
+bool OpalLocalEndPoint::CreateVideoOutputDevice(const OpalConnection & connection,
+                                                const OpalMediaFormat & mediaFormat,
+                                                bool preview,
+                                                PVideoOutputDevice * & device,
+                                                bool & autoDelete)
+{
+  return manager.CreateVideoOutputDevice(connection, mediaFormat, preview, device, autoDelete);
+}
+
+#endif // OPAL_VIDEO
+
+
 OpalLocalEndPoint::Synchronicity
-OpalLocalEndPoint::GetSynchronicity(const OpalMediaFormat & mediaFormat,
-                                    bool isSource) const
+        OpalLocalEndPoint::GetSynchronicity(const OpalMediaFormat & mediaFormat,
+                                            bool isSource) const
 {
   if (mediaFormat.GetMediaType() == OpalMediaType::Audio())
     return m_defaultAudioSynchronicity;
 
-  if (isSource && mediaFormat.GetMediaType() == OpalMediaType::Audio())
+#if OPAL_VIDEO
+  if (isSource && mediaFormat.GetMediaType() == OpalMediaType::Video())
     return m_defaultVideoSourceSynchronicity;
+#endif
 
   return e_Asynchronous;
 }
@@ -223,18 +275,35 @@ OpalLocalConnection::OpalLocalConnection(OpalCall & call,
                     OpalConnection::StringOptions * stringOptions,
                                                char tokenPrefix)
   : OpalConnection(call, ep, ep.GetManager().GetNextToken(tokenPrefix), options, stringOptions)
-  , endpoint(ep)
+  , m_endpoint(ep)
   , m_userData(userData)
+#if OPAL_HAS_H224
+  , m_h224Handler(new OpalH224Handler)
+#endif
 {
 #if OPAL_PTLIB_DTMF
   m_sendInBandDTMF = m_detectInBandDTMF = false;
 #endif
+
+#if OPAL_HAS_H281
+  m_farEndCameraControl = new OpalFarEndCameraControl;
+  m_farEndCameraControl->SetCapabilityChangedNotifier(ep.GetFarEndCameraCapabilityChangedNotifier());
+  m_farEndCameraControl->SetOnActionNotifier(ep.GetFarEndCameraActionNotifier());
+  m_h224Handler->AddClient(*m_farEndCameraControl);
+#endif
+
   PTRACE(4, "LocalCon\tCreated connection with token \"" << callToken << '"');
 }
 
 
 OpalLocalConnection::~OpalLocalConnection()
 {
+#if OPAL_HAS_H281
+  delete m_farEndCameraControl;
+#endif
+#if OPAL_HAS_H224
+  delete m_h224Handler;
+#endif
   PTRACE(4, "LocalCon\tDeleted connection.");
 }
 
@@ -267,12 +336,15 @@ PBoolean OpalLocalConnection::OnIncomingConnection(unsigned int options, OpalCon
 
 PBoolean OpalLocalConnection::SetUpConnection()
 {
-  bool notIncoming = ownerCall.GetConnection(0) == this || ownerCall.IsEstablished();
+  if (ownerCall.IsEstablished())
+    return OpalConnection::SetUpConnection();
+
+  InternalSetAsOriginating();
 
   if (!OpalConnection::SetUpConnection())
     return false;
 
-  if (notIncoming)
+  if (ownerCall.GetConnection(0) == this)
     return true;
 
   if (!OnIncoming()) {
@@ -281,7 +353,7 @@ PBoolean OpalLocalConnection::SetUpConnection()
     return false;
   }
 
-  if (!endpoint.IsDeferredAlerting())
+  if (!m_endpoint.IsDeferredAlerting() && m_endpoint.IsDeferredAnswer())
     AlertingIncoming();
 
   return true;
@@ -293,7 +365,7 @@ PBoolean OpalLocalConnection::SetAlerting(const PString & calleeName, PBoolean)
   PTRACE(3, "LocalCon\tSetAlerting(" << calleeName << ')');
   SetPhase(AlertingPhase);
   remotePartyName = calleeName;
-  return endpoint.OnOutgoingCall(*this);
+  return m_endpoint.OnOutgoingCall(*this);
 }
 
 
@@ -307,53 +379,112 @@ PBoolean OpalLocalConnection::SetConnected()
   return OpalConnection::SetConnected();
 }
 
+
+bool OpalLocalConnection::HoldRemote(bool placeOnHold)
+{
+  if (m_endpoint.WillPauseTransmitMediaOnHold()) {
+    OpalMediaStreamPtr stream;
+    while ((stream = GetMediaStream(OpalMediaType(), true, stream)) != NULL)
+      stream->InternalSetPaused(placeOnHold, false, false);
+  }
+
+  return true;
+}
+
+
 OpalMediaStream * OpalLocalConnection::CreateMediaStream(const OpalMediaFormat & mediaFormat,
                                                          unsigned sessionID,
                                                          PBoolean isSource)
 {
-  return new OpalLocalMediaStream(*this, mediaFormat, sessionID, isSource,
-                                  endpoint.GetSynchronicity(mediaFormat, isSource));
+  if (m_endpoint.UseCallback(mediaFormat, isSource))
+    return new OpalLocalMediaStream(*this, mediaFormat, sessionID, isSource, GetSynchronicity(mediaFormat, isSource));
+
+#if OPAL_VIDEO
+  if (mediaFormat.GetMediaType() == OpalMediaType::Video()) {
+    if (isSource) {
+      PVideoInputDevice * videoDevice;
+      PBoolean autoDeleteGrabber;
+      if (CreateVideoInputDevice(mediaFormat, videoDevice, autoDeleteGrabber))
+        PTRACE(4, "OpalCon\tCreated video input device \"" << videoDevice->GetDeviceName() << '"');
+      else {
+        PTRACE(3, "OpalCon\tCreating fake text video input");
+        PVideoDevice::OpenArgs args;
+        args.deviceName = P_FAKE_VIDEO_TEXT "=No Video Input";
+        mediaFormat.AdjustVideoArgs(args);
+        videoDevice = PVideoInputDevice::CreateOpenedDevice(args, false);
+      }
+
+      PVideoOutputDevice * previewDevice;
+      PBoolean autoDeletePreview;
+      if (CreateVideoOutputDevice(mediaFormat, true, previewDevice, autoDeletePreview))
+        PTRACE(4, "OpalCon\tCreated preview device \"" << previewDevice->GetDeviceName() << '"');
+      else
+        previewDevice = NULL;
+
+      return new OpalVideoMediaStream(*this, mediaFormat, sessionID, videoDevice, previewDevice, autoDeleteGrabber, autoDeletePreview);
+    }
+    else {
+      PVideoOutputDevice * videoDevice;
+      PBoolean autoDelete;
+      if (CreateVideoOutputDevice(mediaFormat, false, videoDevice, autoDelete)) {
+        PTRACE(4, "OpalCon\tCreated display device \"" << videoDevice->GetDeviceName() << '"');
+        return new OpalVideoMediaStream(*this, mediaFormat, sessionID, NULL, videoDevice, false, autoDelete);
+      }
+      PTRACE(2, "OpalCon\tCould not create video output device");
+    }
+
+    return NULL;
+  }
+#endif // OPAL_VIDEO
+
+#if OPAL_HAS_H224
+  if (mediaFormat.GetMediaType() == OpalH224MediaType())
+    return new OpalH224MediaStream(*this, *m_h224Handler, mediaFormat, sessionID, isSource);
+#endif
+
+#if OPAL_HAS_RFC4103
+  if (mediaFormat.GetMediaType() == OpalT140.GetMediaType())
+    return new OpalT140MediaStream(*this, mediaFormat, sessionID, isSource);
+#endif
+
+  return OpalConnection::CreateMediaStream(mediaFormat, sessionID, isSource);
 }
 
 
-OpalMediaStreamPtr OpalLocalConnection::OpenMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, bool isSource)
+void OpalLocalConnection::OnClosedMediaStream(const OpalMediaStream & stream)
 {
-#if OPAL_VIDEO
-  if ( isSource &&
-       mediaFormat.GetMediaType() == OpalMediaType::Video() &&
-      !ownerCall.IsEstablished() &&
-      !endpoint.GetManager().CanAutoStartTransmitVideo()) {
-    PTRACE(3, "LocalCon\tOpenMediaStream auto start disabled, refusing video open");
-    return NULL;
-  }
+#if OPAL_HAS_H281
+  const OpalVideoMediaStream * video = dynamic_cast<const OpalVideoMediaStream *>(&stream);
+  if (video != NULL)
+    m_farEndCameraControl->Detach(video->GetVideoInputDevice());
 #endif
 
-  return OpalConnection::OpenMediaStream(mediaFormat, sessionID, isSource);
+  OpalConnection::OnClosedMediaStream(stream);
 }
 
 
 bool OpalLocalConnection::SendUserInputString(const PString & value)
 {
   PTRACE(3, "LocalCon\tSendUserInputString(" << value << ')');
-  return endpoint.OnUserInput(*this, value);
+  return m_endpoint.OnUserInput(*this, value);
 }
 
 
 bool OpalLocalConnection::OnOutgoingSetUp()
 {
-  return endpoint.OnOutgoingSetUp(*this);
+  return m_endpoint.OnOutgoingSetUp(*this);
 }
 
 
 bool OpalLocalConnection::OnOutgoing()
 {
-  return endpoint.OnOutgoingCall(*this);
+  return m_endpoint.OnOutgoingCall(*this);
 }
 
 
 bool OpalLocalConnection::OnIncoming()
 {
-  return endpoint.OnIncomingCall(*this);
+  return m_endpoint.OnIncomingCall(*this);
 }
 
 
@@ -361,16 +492,16 @@ void OpalLocalConnection::AlertingIncoming(bool withMedia)
 {
   if (LockReadWrite()) {
     if (GetPhase() < AlertingPhase) {
-      SetPhase(AlertingPhase);
+      PTRACE(3, "LocalCon\tAlertingIncoming " << (withMedia ? "with" : "sans") << " media");
 
-      if (withMedia) {
-        PSafePtr<OpalConnection> conn = GetOtherPartyConnection();
-        if (conn != NULL)
-          conn->AutoStartMediaStreams();
+      if (withMedia)
         AutoStartMediaStreams();
-      }
 
-      OnAlerting();
+      SetPhase(AlertingPhase);
+      OnAlerting(withMedia);
+
+      if (withMedia)
+        StartMediaStreams();
     }
     UnlockReadWrite();
   }
@@ -379,6 +510,7 @@ void OpalLocalConnection::AlertingIncoming(bool withMedia)
 
 void OpalLocalConnection::AcceptIncoming()
 {
+  PTRACE(3, "LocalEP\tAccepting incoming call " << *this);
   GetEndPoint().GetManager().QueueDecoupledEvent(
         new PSafeWorkNoArg<OpalLocalConnection>(this, &OpalLocalConnection::InternalAcceptIncoming));
 }
@@ -386,17 +518,137 @@ void OpalLocalConnection::AcceptIncoming()
 
 void OpalLocalConnection::InternalAcceptIncoming()
 {
+  PTRACE(4, "LocalEP\tAccepting (internal) incoming call " << *this);
   PThread::Sleep(100);
-  
+
   if (LockReadWrite()) {
     AlertingIncoming();
-    OnConnectedInternal();
+    InternalOnConnected();
     AutoStartMediaStreams();
     UnlockReadWrite();
   }
 }
 
 
+bool OpalLocalConnection::OnReadMediaFrame(const OpalMediaStream & mediaStream, RTP_DataFrame & frame)
+{
+  return m_endpoint.OnReadMediaFrame(*this, mediaStream, frame);
+}
+
+
+bool OpalLocalConnection::OnWriteMediaFrame(const OpalMediaStream & mediaStream, RTP_DataFrame & frame)
+{
+  return m_endpoint.OnWriteMediaFrame(*this, mediaStream, frame);
+}
+
+
+bool OpalLocalConnection::OnReadMediaData(const OpalMediaStream & mediaStream, void * data, PINDEX size, PINDEX & length)
+{
+  return m_endpoint.OnReadMediaData(*this, mediaStream, data, size, length);
+}
+
+
+bool OpalLocalConnection::OnWriteMediaData(const OpalMediaStream & mediaStream, const void * data, PINDEX length, PINDEX & written)
+{
+  return m_endpoint.OnWriteMediaData(*this, mediaStream, data, length, written);
+}
+
+
+OpalLocalEndPoint::Synchronicity OpalLocalConnection::GetSynchronicity(const OpalMediaFormat & mediaFormat, bool isSource) const
+{
+  return m_endpoint.GetSynchronicity(mediaFormat, isSource);
+}
+
+
+#if OPAL_VIDEO
+
+bool OpalLocalConnection::CreateVideoInputDevice(const OpalMediaFormat & mediaFormat,
+                                                 PVideoInputDevice * & device,
+                                                 bool & autoDelete)
+{
+  if (!m_endpoint.CreateVideoInputDevice(*this, mediaFormat, device, autoDelete))
+    return false;
+
+#if OPAL_HAS_H281
+  m_farEndCameraControl->Attach(device);
+#endif
+  return true;
+}
+
+
+bool OpalLocalConnection::CreateVideoOutputDevice(const OpalMediaFormat & mediaFormat,
+                                                  bool preview,
+                                                  PVideoOutputDevice * & device,
+                                                  bool & autoDelete)
+{
+  return m_endpoint.CreateVideoOutputDevice(*this, mediaFormat, preview, device, autoDelete);
+}
+
+
+bool OpalLocalConnection::ChangeVideoInputDevice(const PVideoDevice::OpenArgs & deviceArgs, unsigned sessionID)
+{
+  PSafePtr<OpalVideoMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalVideoMediaStream>(
+                      sessionID != 0 ? GetMediaStream(sessionID, true) : GetMediaStream(OpalMediaType::Video(), true));
+  if (stream == NULL)
+    return false;
+
+  PVideoInputDevice * newDevice;
+  bool autoDelete;
+  if (!m_endpoint.GetManager().CreateVideoInputDevice(*this, deviceArgs, newDevice, autoDelete)) {
+    PTRACE(2, "OpalCon\tCould not open video device \"" << deviceArgs.deviceName << '"');
+    return false;
+  }
+
+  PTRACE_CONTEXT_ID_TO(newDevice);
+  stream->SetVideoInputDevice(newDevice, autoDelete);
+#if OPAL_HAS_H281
+  m_farEndCameraControl->Attach(newDevice);
+#endif
+  return true;
+}
+
+
+bool OpalLocalConnection::ChangeVideoOutputDevice(const PVideoDevice::OpenArgs & deviceArgs, unsigned sessionID, bool preview)
+{
+  PSafePtr<OpalVideoMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalVideoMediaStream>(
+                      sessionID != 0 ? GetMediaStream(sessionID, true) : GetMediaStream(OpalMediaType::Video(), preview));
+  if (stream == NULL)
+    return false;
+
+  PVideoOutputDevice * newDevice = PVideoOutputDevice::CreateOpenedDevice(deviceArgs, false);
+  if (newDevice == NULL) {
+    PTRACE(2, "OpalCon\tCould not open video device \"" << deviceArgs.deviceName << '"');
+    return false;
+  }
+
+  PTRACE_CONTEXT_ID_TO(newDevice);
+  stream->SetVideoOutputDevice(newDevice);
+  return true;
+}
+
+#endif // OPAL_VIDEO
+
+
+#if OPAL_HAS_H281
+bool OpalLocalConnection::FarEndCameraControl(PVideoControlInfo::Types what, int direction, const PTimeInterval & duration)
+{
+  return m_farEndCameraControl->Action(what, direction, duration);
+}
+
+
+void OpalLocalConnection::SetFarEndCameraCapabilityChangedNotifier(const PNotifier & notifier)
+{
+  m_farEndCameraControl->SetCapabilityChangedNotifier(notifier);
+}
+
+
+void OpalLocalConnection::SetFarEndCameraActionNotifier(const PNotifier & notifier)
+{
+  m_farEndCameraControl->SetOnActionNotifier(notifier);
+}
+#endif // OPAL_HAS_H281
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalLocalMediaStream::OpalLocalMediaStream(OpalLocalConnection & connection,
@@ -406,6 +658,7 @@ OpalLocalMediaStream::OpalLocalMediaStream(OpalLocalConnection & connection,
                                            OpalLocalEndPoint::Synchronicity synchronicity)
   : OpalMediaStream(connection, mediaFormat, sessionID, isSource)
   , OpalMediaStreamPacing(mediaFormat)
+  , m_connection(connection)
   , m_synchronicity(synchronicity)
 {
 }
@@ -416,12 +669,15 @@ PBoolean OpalLocalMediaStream::ReadPacket(RTP_DataFrame & frame)
   if (!IsOpen())
     return false;
 
-  OpalLocalEndPoint & ep = dynamic_cast<OpalLocalEndPoint &>(connection.GetEndPoint());
-  OpalLocalConnection & conn = dynamic_cast<OpalLocalConnection &>(connection);
-  if (ep.OnReadMediaFrame(conn, *this, frame))
-    return true;
+  if (!m_connection.OnReadMediaFrame(*this, frame))
+    return OpalMediaStream::ReadPacket(frame);
 
-  return OpalMediaStream::ReadPacket(frame);
+  marker = frame.GetMarker();
+  timestamp = frame.GetTimestamp();
+
+  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSynchronous)
+    Pace(false, frame.GetPayloadSize(), marker);
+  return true;
 }
 
 
@@ -430,36 +686,44 @@ PBoolean OpalLocalMediaStream::WritePacket(RTP_DataFrame & frame)
   if (!IsOpen())
     return false;
 
-  OpalLocalEndPoint & ep = dynamic_cast<OpalLocalEndPoint &>(connection.GetEndPoint());
-  OpalLocalConnection & conn = dynamic_cast<OpalLocalConnection &>(connection);
-  if (ep.OnWriteMediaFrame(conn, *this, frame))
-    return true;
+  if (!m_connection.OnWriteMediaFrame(*this, frame))
+    return OpalMediaStream::WritePacket(frame);
 
-  return OpalMediaStream::WritePacket(frame);
+  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSynchronous)
+    Pace(false, frame.GetPayloadSize(), marker);
+  return true;
 }
 
 
 PBoolean OpalLocalMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & length)
 {
-  OpalLocalEndPoint & ep = dynamic_cast<OpalLocalEndPoint &>(connection.GetEndPoint());
-  OpalLocalConnection & conn = dynamic_cast<OpalLocalConnection &>(connection);
-  if (!ep.OnReadMediaData(conn, *this, data, size, length))
+  if (!m_connection.OnReadMediaData(*this, data, size, length))
     return false;
 
-  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSyncronous)
-    Pace(true, size, marker);
+  timestamp += OpalMediaStream::m_frameTime;
+
+  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSynchronous)
+    Pace(false, size, marker);
   return true;
 }
 
 
 PBoolean OpalLocalMediaStream::WriteData(const BYTE * data, PINDEX length, PINDEX & written)
 {
-  OpalLocalEndPoint & ep = dynamic_cast<OpalLocalEndPoint &>(connection.GetEndPoint());
-  OpalLocalConnection & conn = dynamic_cast<OpalLocalConnection &>(connection);
-  if (!ep.OnWriteMediaData(conn, *this, data, length, written))
+  if (GetMediaFormat().GetName().NumCompare(OPAL_PCM16) == EqualTo) {
+    if (data != NULL && length != 0)
+      m_silence.SetMinSize(length);
+    else {
+      length = m_silence.GetSize();
+      data = m_silence;
+      PTRACE(6, "Media\tPlaying silence " << length << " bytes");
+    }
+  }
+
+  if (!m_connection.OnWriteMediaData(*this, data, length, written))
     return false;
 
-  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSyncronous)
+  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSynchronous)
     Pace(false, written, marker);
   return true;
 }
