@@ -25,9 +25,9 @@
  * Contributor(s): Craig Southeren (craigs@postincrement.com)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 29917 $
+ * $Revision: 34362 $
  * $Author: rjongbloed $
- * $Date: 2013-06-11 21:13:58 -0500 (Tue, 11 Jun 2013) $
+ * $Date: 2015-11-28 12:10:23 +0000 (Sat, 28 Nov 2015) $
  */
 
 #include <ptlib.h>
@@ -36,10 +36,6 @@
 
 #if OPAL_HAS_MIXER
 
-  #ifdef _MSC_VER
-    #pragma message("Mixer (MCU) support enabled")
-  #endif
-
 #include <opal/patch.h>
 #include <rtp/rtp.h>
 #include <rtp/jitter.h>
@@ -86,6 +82,8 @@ public:
 
 /////////////////////////////////////////////////////////////////////////////
 
+#define PTraceModule() "Mixer"
+
 OpalBaseMixer::OpalBaseMixer(bool pushThread, unsigned periodMS, unsigned periodTS)
   : m_pushThread(pushThread)
   , m_periodMS(periodMS)
@@ -112,7 +110,7 @@ void OpalBaseMixer::RemoveStream(const Key_T & key)
   if (iter != m_inputStreams.end()) {
     delete iter->second;
     m_inputStreams.erase(iter);
-    PTRACE(4, "Mixer\tRemoved stream at key " << key);
+    PTRACE(4, "Removed stream at key " << key);
   }
 
   if (m_inputStreams.empty())
@@ -124,7 +122,7 @@ void OpalBaseMixer::RemoveStream(const Key_T & key)
 
 void OpalBaseMixer::RemoveAllStreams()
 {
-  PTRACE(4, "Mixer\tRemoving all streams");
+  PTRACE(4, "Removing all streams");
 
   m_mutex.Wait();
 
@@ -150,7 +148,7 @@ bool OpalBaseMixer::AddStream(const Key_T & key)
 
   m_inputStreams[key] = stream;
   PTRACE_CONTEXT_ID_SET(*stream, key);
-  PTRACE(4, "Mixer\tAdded input stream " << stream << " at key " << key);
+  PTRACE(4, "Added input stream " << stream << " at key " << key);
 
   StartPushThread();
   return true;
@@ -235,8 +233,8 @@ void OpalBaseMixer::StopPushThread(bool lock)
   m_mutex.Signal();
 
   if (thread != NULL) {
-    PTRACE(4, "Mixer\tWaiting for push thread to terminate");
-    PAssert(thread->WaitForTermination(5000), "Mixer worker thread took too long to terminate.");
+    PTRACE(4, "Waiting for push thread " << *thread << " to terminate");
+    PAssert(thread->WaitForTermination(5000), PSTRSTRM("Mixer worker thread " << *thread << " took too long to terminate."));
     delete thread;
   }
 }
@@ -244,12 +242,12 @@ void OpalBaseMixer::StopPushThread(bool lock)
 
 void OpalBaseMixer::PushThreadMain()
 {
-  PTRACE(4, "Mixer\tPushThread start " << m_periodMS << " ms");
-  PAdaptiveDelay delay;
+  PTRACE(4, "PushThread start " << m_periodMS << " ms");
+  PAdaptiveDelay delay(500);
   while (m_threadRunning && OnPush())
     delay.Delay(m_periodMS);
 
-  PTRACE(4, "Mixer\tPushThread end");
+  PTRACE(4, "PushThread end");
 }
 
 
@@ -296,7 +294,7 @@ OpalBaseMixer::Stream * OpalAudioMixer::CreateStream()
     else if (m_right == NULL)
       m_right = stream;
     else {
-      PTRACE(2, "Mixer\tCannot have more than two streams for stereo mode!");
+      PTRACE(2, "Cannot have more than two streams for stereo mode!");
       delete stream;
       return NULL;
     }
@@ -342,7 +340,7 @@ bool OpalAudioMixer::SetSampleRate(unsigned rate)
   m_mixedAudio.resize(m_periodTS);
   for (StreamMap_T::iterator iter = m_inputStreams.begin(); iter != m_inputStreams.end(); ++iter)
     ((AudioStream *)iter->second)->m_cacheSamples.SetSize(m_periodTS);
-  PTRACE(4, "Mixer\tSample rate set to " << rate);
+  PTRACE(4, "Sample rate set to " << rate);
   return true;
 }
 
@@ -360,21 +358,22 @@ bool OpalAudioMixer::SetJitterBufferSize(const Key_T & key, const OpalJitterBuff
     return false;
 
   OpalJitterBuffer * & jitter = audioStream->m_jitter;
-  if (jitter != NULL) {
-    if (init.m_maxJitterDelay > 0)
-      jitter->SetDelay(init);
-    else {
-      PTRACE(4, "AudioMix\tJitter buffer disabled");
+
+  if (init.m_maxJitterDelay == 0) {
+    if (jitter != NULL) {
+      PTRACE(4, "Jitter buffer disabled");
       delete jitter;
       jitter = NULL;
     }
+    return false;
   }
+
+  if (jitter != NULL)
+    jitter->SetDelay(init);
   else {
-    if (init.m_maxJitterDelay > 0) {
-      PTRACE(4, "AudioMix\tJitter buffer enabled");
-      jitter = new OpalJitterBuffer(init);
-      PTRACE_CONTEXT_ID_SET(*jitter, audioStream);
-    }
+    PTRACE(4, "Jitter buffer enabled");
+    jitter = OpalJitterBuffer::Create(OpalMediaType::Audio(), init);
+    PTRACE_CONTEXT_ID_SET(*jitter, audioStream);
   }
 
   return true;
@@ -564,7 +563,7 @@ bool OpalVideoMixer::SetFrameRate(unsigned rate)
   m_mutex.Wait();
   m_periodMS = 1000/rate;
   m_periodTS = OpalMediaFormat::VideoClockRate/rate;
-  PTRACE(4, "Mixer\tPushThread period adjusted to " << m_periodMS << " ms");
+  PTRACE(4, "PushThread period adjusted to " << m_periodMS << " ms");
   m_mutex.Signal();
 
   return true;
@@ -593,39 +592,74 @@ OpalBaseMixer::Stream * OpalVideoMixer::CreateStream()
 
 
 bool OpalVideoMixer::MixStreams(RTP_DataFrame & frame)
+{
+  if (!MixVideo())
+    return false;
+
+  frame.SetPayloadSize(GetOutputSize());
+  PluginCodec_Video_FrameHeader * video = (PluginCodec_Video_FrameHeader *)frame.GetPayloadPtr();
+  video->width = m_width;
+  video->height = m_height;
+  memcpy(OPAL_VIDEO_FRAME_DATA_PTR(video), m_frameStore, m_frameStore.GetSize());
+
+  return true;
+}
+
+
+bool OpalVideoMixer::MixVideo()
 {
   // create output frame
-  unsigned left, x, y, w, h;
+  unsigned x, y, w, h, left;
+  if (!StartMix(x, y, w, h, left))
+    return false;
+
+  // This makes sure subimage are on 32 bit boundary, some parts of the
+  // system can get mightily upset if this is not the case.
+  w &= 0xfffffffc;
+  h &= 0xfffffffc;
+
+  for (StreamMap_T::iterator iter = m_inputStreams.begin(); iter != m_inputStreams.end(); ++iter) {
+    InsertVideoFrame(iter, x, y, w, h);
+    if (!NextMix(x, y, w, h, left))
+      break;
+  }
+
+  return true;
+}
+
+
+bool OpalVideoMixer::StartMix(unsigned & x, unsigned & y, unsigned & w, unsigned & h, unsigned & left)
+{
   switch (m_style) {
-    case eSideBySideLetterbox :
+    case eSideBySideLetterbox:
       x = left = 0;
-      y = m_height/4;
-      w = m_width/2;
-      h = m_height/2;
+      y = m_height / 4;
+      w = m_width / 2;
+      h = m_height / 2;
       break;
 
-    case eSideBySideScaled :
+    case eSideBySideScaled:
       x = left = 0;
       y = 0;
-      w = m_width/2;
+      w = m_width / 2;
       h = m_height;
       break;
 
-    case eStackedPillarbox :
-      x = left = m_width/4;
+    case eStackedPillarbox:
+      x = left = m_width / 4;
       y = 0;
-      w = m_width/2;
-      h = m_height/2;
+      w = m_width / 2;
+      h = m_height / 2;
       break;
 
-    case eStackedScaled :
+    case eStackedScaled:
       x = left = 0;
       y = 0;
       w = m_width;
-      h = m_height/2;
+      h = m_height / 2;
       break;
 
-    case eGrid :
+    case eGrid:
       x = left = 0;
       y = 0;
       if (m_lastStreamCount != m_inputStreams.size()) {
@@ -635,69 +669,66 @@ bool OpalVideoMixer::MixStreams(RTP_DataFrame & frame)
         m_lastStreamCount = m_inputStreams.size();
       }
       switch (m_lastStreamCount) {
-        case 0 :
-        case 1 :
+        case 0:
+        case 1:
           w = m_width;
           h = m_height;
           break;
 
-        case 2 :
-          y = m_height/4;
+        case 2:
+          y = m_height / 4;
           // Fall into next case
 
-        case 3 :
-        case 4 :
-          w = m_width/2;
-          h = m_height/2;
+        case 3:
+        case 4:
+          w = m_width / 2;
+          h = m_height / 2;
           break;
 
-        case 5 :
-        case 6 :
-        case 7 :
-        case 8 :
-        case 9 :
-          w = m_width/3;
-          h = m_height/3;
+        case 5:
+        case 6:
+        case 7:
+        case 8:
+        case 9:
+          w = m_width / 3;
+          h = m_height / 3;
           break;
 
-        default :
-          w = m_width/4;
-          h = m_height/4;
+        default:
+          w = m_width / 4;
+          h = m_height / 4;
           break;
       }
       break;
 
-    default :
+    default:
       return false;
   }
 
-  // This makes sure subimage are on 32 bit boundary, some parts of the
-  // system can get mightily upset if this is not the case.
-  w &= 0xfffffffc;
-  h &= 0xfffffffc;
-
-  for (StreamMap_T::iterator iter = m_inputStreams.begin(); iter != m_inputStreams.end(); ++iter) {
-    ((VideoStream *)iter->second)->InsertVideoFrame(x, y, w, h);
-
-    x += w;
-    if (x+w > m_width) {
-      x = left;
-      y += h;
-      if (y+h > m_height)
-        break;
-    }
-  }
-
-  frame.SetPayloadSize(GetOutputSize());
-  PluginCodec_Video_FrameHeader * video = (PluginCodec_Video_FrameHeader *)frame.GetPayloadPtr();
-  video->width = m_width;
-  video->height = m_height;
-  memcpy(OPAL_VIDEO_FRAME_DATA_PTR(video), m_frameStore, m_frameStore.GetSize());
-
   return true;
 }
 
 
+bool OpalVideoMixer::NextMix(unsigned & x, unsigned & y, unsigned & w, unsigned & h, unsigned & left)
+{
+  x += w;
+  if (x + w <= m_width)
+    return true;
+
+  x = left;
+  y += h;
+  return y + h <= m_height;
+}
+
+
+void OpalVideoMixer::InsertVideoFrame(const StreamMap_T::iterator & it, unsigned x, unsigned y, unsigned w, unsigned h)
+{
+  VideoStream * vid = dynamic_cast<VideoStream *>(it->second);
+  if (vid != NULL)
+    vid->InsertVideoFrame(x, y, w, h);
+}
+
+
 size_t OpalVideoMixer::GetOutputSize() const
 {
   return m_frameStore.GetSize() + sizeof(PluginCodec_Video_FrameHeader);
@@ -723,7 +754,7 @@ void OpalVideoMixer::VideoStream::InsertVideoFrame(unsigned x, unsigned y, unsig
 
   PluginCodec_Video_FrameHeader * header = (PluginCodec_Video_FrameHeader *)m_queue.front().GetPayloadPtr();
 
-  PTRACE(DETAIL_LOG_LEVEL, "Mixer\tCopying video: " << header->width << 'x' << header->height
+  PTRACE(DETAIL_LOG_LEVEL, "Copying video: " << header->width << 'x' << header->height
          << " -> " << x << ',' << y << '/' << w << 'x' << h);
 
   PColourConverter::CopyYUV420P(0, 0, header->width, header->height,
@@ -746,26 +777,30 @@ void OpalVideoMixer::VideoStream::InsertVideoFrame(unsigned x, unsigned y, unsig
 
 ///////////////////////////////////////////////////////////////////////////////
 
+#undef  PTraceModule
+#define PTraceModule() "MixerEP"
+
 OpalMixerEndPoint::OpalMixerEndPoint(OpalManager & manager, const char * prefix)
-  : OpalLocalEndPoint(manager, prefix)
+  : OpalLocalEndPoint(manager, prefix, false)
   , OpalMixerNodeManager(manager)
   , m_adHocNodeInfo(NULL)
   , m_factoryNodeInfo(NULL)
+  , m_factoryIndex(0)
 {
-  PTRACE(4, "MixerEP\tConstructed");
+  PTRACE(4, "Constructed");
 }
 
 
 OpalMixerEndPoint::~OpalMixerEndPoint()
 {
   delete m_adHocNodeInfo;
-  PTRACE(4, "MixerEP\tDestroyed");
+  PTRACE(4, "Destroyed");
 }
 
 
 void OpalMixerEndPoint::ShutDown()
 {
-  PTRACE(4, "MixerEP\tShutting down");
+  PTRACE(4, "Shutting down");
 
   OpalMixerNodeManager::ShutDown();
 
@@ -790,7 +825,7 @@ PSafePtr<OpalConnection> OpalMixerEndPoint::MakeConnection(OpalCall & call,
                                                          unsigned int options,
                                       OpalConnection::StringOptions * stringOptions)
 {
-  PTRACE(4, "MixerEP\tMaking connection to \"" << party << '"');
+  PTRACE(4, "Making connection to \"" << party << '"');
 
   PSafePtr<OpalMixerNode> node;
 
@@ -808,7 +843,7 @@ PSafePtr<OpalConnection> OpalMixerEndPoint::MakeConnection(OpalCall & call,
     OpalMixerNodeInfo * info = m_factoryNodeInfo->Clone();
     info->m_name = GetNewFactoryName();
     if (info->m_name.IsEmpty() || (node = AddNode(info)) == NULL) {
-      PTRACE(2, "MixerEP\tCannot make factory node.");
+      PTRACE(2, "Cannot make factory node.");
       return NULL;
     }
 
@@ -822,7 +857,7 @@ PSafePtr<OpalConnection> OpalMixerEndPoint::MakeConnection(OpalCall & call,
 
   if (name.IsEmpty() || name == "*") {
     if (m_adHocNodeInfo == NULL || m_adHocNodeInfo->m_name.IsEmpty()) {
-      PTRACE(2, "MixerEP\tCannot make ad-hoc node for default alias");
+      PTRACE(2, "Cannot make ad-hoc node for default alias");
       return NULL;
     }
     name = m_adHocNodeInfo->m_name;
@@ -836,7 +871,7 @@ PSafePtr<OpalConnection> OpalMixerEndPoint::MakeConnection(OpalCall & call,
   }
 
   if (node == NULL) {
-    PTRACE(2, "MixerEP\tNode alias \"" << party << "\" does not exist and cannot make ad-hoc node.");
+    PTRACE(2, "Node alias \"" << party << "\" does not exist and cannot make ad-hoc node.");
     return NULL;
   }
 
@@ -963,6 +998,9 @@ void OpalMixerEndPoint::OnNodeStatusChanged(const OpalMixerNode & node, OpalConf
 
 ///////////////////////////////////////////////////////////////////////////////
 
+#undef  PTraceModule
+#define PTraceModule() "MixerCon"
+
 OpalMixerConnection::OpalMixerConnection(PSafePtr<OpalMixerNode> node,
                                                       OpalCall & call,
                                              OpalMixerEndPoint & ep,
@@ -982,13 +1020,13 @@ OpalMixerConnection::OpalMixerConnection(PSafePtr<OpalMixerNode> node,
   else
     localPartyName = *names.begin();
 
-  PTRACE(4, "MixerCon\tConstructed");
+  PTRACE(4, "Constructed");
 }
 
 
 OpalMixerConnection::~OpalMixerConnection()
 {
-  PTRACE(4, "MixerCon\tDestroyed");
+  PTRACE(4, "Destroyed");
 }
 
 
@@ -1067,7 +1105,7 @@ bool OpalMixerConnection::GetConferenceState(OpalConferenceState * state) const
 
 void OpalMixerConnection::SetListenOnly(bool listenOnly)
 {
-  PTRACE(3, "MixerCon\tSet listen only mode to " << (listenOnly ? "ON" : "OFF"));
+  PTRACE(3, "Set listen only mode to " << (listenOnly ? "ON" : "OFF"));
 
   m_listenOnly = listenOnly;
 
@@ -1096,7 +1134,8 @@ OpalMixerMediaStream::OpalMixerMediaStream(OpalConnection & conn,
   , m_node(node)
   , m_listenOnly(listenOnly)
 #if OPAL_VIDEO
-  , m_video(mediaFormat.GetMediaType() == OpalMediaType::Video())
+  , m_mixedVideoWidth(0)
+  , m_mixedVideoHeight(0)
 #endif
 {
   /* We are a bit sneaky here. OpalCall::OpenSourceMediaStream will have
@@ -1108,7 +1147,7 @@ OpalMixerMediaStream::OpalMixerMediaStream(OpalConnection & conn,
      connections. */
   if (IsSink()) {
 #if OPAL_VIDEO
-    if (m_video)
+    if (mediaFormat.GetMediaType() == OpalMediaType::Video())
       mediaFormat = OpalYUV420P;
     else
 #endif
@@ -1133,16 +1172,25 @@ PBoolean OpalMixerMediaStream::Open()
    && mediaFormat.GetMediaType() != OpalMediaType::Video()
 #endif
   ) {
-    PTRACE(3, "MixerStrm\tCannot open media stream of type " << mediaFormat.GetMediaType());
+    PTRACE(3, "Cannot open media stream of type " << mediaFormat.GetMediaType());
     return false;
   }
 
   SetPaused(IsSink() && m_listenOnly);
 
-  if (!IsPaused() && !m_node->AttachStream(this))
+  return OpalMediaStream::Open();
+}
+
+
+PBoolean OpalMixerMediaStream::Start()
+{
+  if (!OpalMediaStream::Start())
     return false;
 
-  return OpalMediaStream::Open();
+  if (IsSink() && m_listenOnly)
+    return true;
+
+  return m_node->AttachStream(this);
 }
 
 
@@ -1154,12 +1202,7 @@ void OpalMixerMediaStream::InternalClose()
 
 PBoolean OpalMixerMediaStream::WritePacket(RTP_DataFrame & packet)
 {
-#if OPAL_VIDEO
-  if (m_video)
-    return m_node->WriteVideo(GetID(), packet);
-#endif
-
-  return m_node->WriteAudio(GetID(), packet);
+  return IsOpen() && m_node->WritePacket(*this, packet);
 }
 
 
@@ -1175,22 +1218,33 @@ PBoolean OpalMixerMediaStream::RequiresPatchThread() const
 }
 
 
-bool OpalMixerMediaStream::InternalSetJitterBuffer(const OpalJitterBuffer::Init & init) const
+bool OpalMixerMediaStream::InternalSetJitterBuffer(const OpalJitterBuffer::Init & init)
 {
   return IsSink() && m_node->SetJitterBufferSize(GetID(), init);
 }
 
 
+#if OPAL_VIDEO
+bool OpalMixerMediaStream::CheckMixedVideoSize(unsigned width, unsigned height)
+{
+  bool different = m_mixedVideoWidth != width || m_mixedVideoHeight != height;
+  m_mixedVideoWidth = width;
+  m_mixedVideoHeight = height;
+  return different;
+}
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 
+#undef  PTraceModule
+#define PTraceModule() "MixerNode"
+
 OpalMixerNode::OpalMixerNode(OpalMixerNodeManager & manager,
                                 OpalMixerNodeInfo * info)
   : m_manager(manager)
   , m_info(info != NULL ? info : new OpalMixerNodeInfo)
-  , m_audioMixer(*m_info)
-#if OPAL_VIDEO
-  , m_videoMixer(*m_info)
-#endif
+  , m_shuttingDown(false)
+  , m_audioMixer(manager.CreateAudioMixer(*m_info))
 {
   PTRACE_CONTEXT_ID_NEW();
 
@@ -1198,7 +1252,7 @@ OpalMixerNode::OpalMixerNode(OpalMixerNodeManager & manager,
 
   AddName(m_info->m_name);
 
-  PTRACE(4, "MixerNode\tConstructed " << *this);
+  PTRACE(4, "Constructed " << *this);
 }
 
 
@@ -1206,17 +1260,19 @@ OpalMixerNode::~OpalMixerNode()
 {
   ShutDown(); // Fail safe
 
+  delete m_audioMixer;
   delete m_info;
-  PTRACE(4, "MixerNode\tDestroyed " << *this);
+
+  PTRACE(4, "Destroyed " << *this);
 }
 
 
 void OpalMixerNode::ShutDown()
 {
-  if (m_shuttingDown.TestAndSet(true))
+  if (m_shuttingDown.exchange(true))
     return;
 
-  PTRACE(4, "MixerNode\tShutting down " << *this);
+  PTRACE(4, "Shutting down " << *this);
 
   m_manager.OnNodeStatusChanged(*this, OpalConferenceState::Destroyed);
 
@@ -1226,14 +1282,20 @@ void OpalMixerNode::ShutDown()
   while (GetConnectionCount() > 0)
     PThread::Sleep(100);
 
-  m_audioMixer.RemoveAllStreams();
+  if (LockReadWrite()) {
+    m_audioMixer->RemoveAllStreams();
 #if OPAL_VIDEO
-  m_videoMixer.RemoveAllStreams();
+    for (VideoMixerMap::iterator it = m_videoMixers.begin(); it != m_videoMixers.end(); ++it)
+      delete it->second;
+    m_videoMixers.clear();
 #endif
-  m_manager.RemoveNodeNames(GetNames());
-  m_names.RemoveAll();
+    m_manager.RemoveNodeNames(GetNames());
+    m_names.RemoveAll();
 
-  m_manager.RemoveNode(*this);
+    m_manager.RemoveNode(*this);
+
+    UnlockReadWrite();
+  }
 }
 
 
@@ -1244,19 +1306,23 @@ void OpalMixerNode::PrintOn(ostream & strm) const
 }
 
 
-void OpalMixerNode::AddName(const PString & name)
+bool OpalMixerNode::AddName(const PString & name)
 {
   if (name.IsEmpty())
-    return;
+    return false;
 
-  if (m_names.Contains(name)) {
-    PTRACE(4, "MixerNode\tName \"" << name << "\" already added to " << *this);
-    return;
+  PSafeLockReadWrite mutex(*this);
+  if (!mutex.IsLocked())
+    return false;
+
+  if (!m_manager.AddNodeName(name, this)) {
+    PTRACE(4, "Name \"" << name << "\" already added to " << *this);
+    return false;
   }
 
-  PTRACE(4, "MixerNode\tAdding name \"" << name << "\" to " << *this);
+  PTRACE(4, "Added name \"" << name << "\" to " << *this);
   m_names += name;
-  m_manager.AddNodeName(name, this);
+  return true;
 }
 
 
@@ -1265,11 +1331,15 @@ void OpalMixerNode::RemoveName(const PString & name)
   if (name.IsEmpty())
     return;
 
+  PSafeLockReadWrite mutex(*this);
+  if (!mutex.IsLocked())
+    return;
+
   PStringSet::iterator it = m_names.find(name);
   if (it == m_names.end())
-    PTRACE(4, "MixerNode\tName \"" << name << "\" not present in " << *this);
+    PTRACE(4, "Name \"" << name << "\" not present in " << *this);
   else {
-    PTRACE(4, "MixerNode\tRemoving name \"" << name << "\" from " << *this);
+    PTRACE(4, "Removing name \"" << name << "\" from " << *this);
     m_names.erase(it);
     m_manager.RemoveNodeName(name);
   }
@@ -1314,47 +1384,69 @@ void OpalMixerNode::DetachConnection(OpalConnection * connection)
 
 bool OpalMixerNode::AttachStream(OpalMixerMediaStream * stream)
 {
-  PTRACE(4, "MixerNode\tAttaching " << stream->GetMediaFormat()
+  PString id = stream->GetID();
+
+  PTRACE(4, "Attaching " << stream->GetMediaFormat()
          << ' ' << (stream->IsSource() ? "source" : "sink")
-         << " stream with id " << stream->GetID() << " to " << *this);
+         << " stream with id " << id << " to " << *this);
 
 #if OPAL_VIDEO
   if (stream->GetMediaFormat().GetMediaType() == OpalMediaType::Video()) {
+    OpalVideoFormat::ContentRole role = stream->GetMediaFormat().GetOptionEnum(OpalVideoFormat::ContentRoleOption(), OpalVideoFormat::eNoRole);
+    OpalVideoStreamMixer * videoMixer;
+    VideoMixerMap::iterator it = m_videoMixers.find(role);
+    if (it != m_videoMixers.end())
+      videoMixer = it->second;
+    else {
+      videoMixer = m_manager.CreateVideoMixer(*m_info);
+      m_videoMixers[role] = videoMixer;
+    }
+
+    m_mixerById[id] = videoMixer;
+
     if (stream->IsSink())
-      return m_videoMixer.AddStream(stream->GetID());
-    m_videoMixer.m_outputStreams.Append(stream);
+      return videoMixer->AddStream(id);
+
+    videoMixer->Append(stream);
     return true;
   }
-#endif
+#endif // OPAL_VIDEO
+
+  m_mixerById[id] = m_audioMixer;
 
   if (stream->IsSink())
-    return m_audioMixer.AddStream(stream->GetID());
+    return m_audioMixer->AddStream(id);
 
-  m_audioMixer.m_outputStreams.Append(stream);
+  m_audioMixer->Append(stream);
   return true;
 }
 
 
 void OpalMixerNode::DetachStream(OpalMixerMediaStream * stream)
 {
-  PTRACE(4, "MixerNode\tDetaching " << stream->GetMediaFormat()
+  PString id = stream->GetID();
+
+  PTRACE(4, "Detaching " << stream->GetMediaFormat()
          << ' ' << (stream->IsSource() ? "source" : "sink")
-         << " stream with id " << stream->GetID() << " from " << *this);
+         << " stream with id " << id << " from " << *this);
 
 #if OPAL_VIDEO
   if (stream->GetMediaFormat().GetMediaType() == OpalMediaType::Video()) {
+    VideoMixerMap::iterator it = m_videoMixers.find(stream->GetMediaFormat().GetOptionEnum(OpalVideoFormat::ContentRoleOption(), OpalVideoFormat::eNoRole));
+    if (it == m_videoMixers.end())
+      return;
     if (stream->IsSource())
-      m_videoMixer.m_outputStreams.Remove(stream);
+      it->second->Remove(stream);
     else
-      m_videoMixer.RemoveStream(stream->GetID());
+      it->second->RemoveStream(stream->GetID());
     return;
   }
 #endif
 
   if (stream->IsSource())
-    m_audioMixer.m_outputStreams.Remove(stream);
+    m_audioMixer->Remove(stream);
   else
-    m_audioMixer.RemoveStream(stream->GetID());
+    m_audioMixer->RemoveStream(stream->GetID());
 }
 
 
@@ -1392,6 +1484,20 @@ void OpalMixerNode::UseMediaPassThrough(unsigned sessionID, OpalConnection * con
 }
 
 
+bool OpalMixerNode::SetJitterBufferSize(const OpalBaseMixer::Key_T & key, const OpalJitterBuffer::Init & init)
+{
+  return m_audioMixer != NULL && m_audioMixer->SetJitterBufferSize(key, init);
+}
+
+
+bool OpalMixerNode::WritePacket(const OpalMixerMediaStream & stream, const RTP_DataFrame & input)
+{
+  PString id = stream.GetID();
+  MixerByIdMap::iterator it = m_mixerById.find(id);
+  return it == m_mixerById.end() || it->second->WriteStream(id, input);
+}
+
+
 void OpalMixerNode::BroadcastUserInput(const OpalConnection * connection, const PString & value)
 {
   for (PSafePtr<OpalConnection> conn(m_connections, PSafeReference); conn != NULL; ++conn) {
@@ -1425,6 +1531,8 @@ void OpalMixerNode::GetConferenceState(OpalConferenceState & state) const
     PSafePtr<OpalConnection> other = conn->GetOtherPartyConnection();
     if (other != NULL && other->IsNetworkConnection()) {
       OpalConferenceState::User user;
+      /* Should readlly use readonly lock here, but can cause deadlock, and at
+         this point inprocessing the remote party info should be static. */
       user.m_uri = other->GetRemotePartyURL();
       user.m_displayText = other->GetRemotePartyName();
       user.m_roles += "participant";
@@ -1569,20 +1677,30 @@ public:
   }
 };
 
-static SIPEventPackageFactory::Worker<SIPConferenceEventPackageHandler> conferenceEventPackageHandler(SIPSubscribe::Conference);
+PFACTORY_CREATE(SIPEventPackageFactory, SIPConferenceEventPackageHandler, SIPSubscribe::Conference);
 
 #endif // OPAL_SIP
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
-OpalMixerNode::MediaMixer::MediaMixer()
+OpalMediaStreamMixer::OpalMediaStreamMixer()
 {
   m_outputStreams.DisallowDeleteObjects();
 }
 
 
-void OpalMixerNode::MediaMixer::CloseOne(const PSafePtr<OpalMixerMediaStream> & stream)
+void OpalMediaStreamMixer::Append(const PSafePtr<OpalMixerMediaStream> & stream)
+{
+  for (PSafePtr<OpalMixerMediaStream> s(m_outputStreams); s != NULL; ++s) {
+    if (s == stream)
+      return;
+  }
+  m_outputStreams.Append(stream);
+}
+
+
+void OpalMediaStreamMixer::CloseOne(const PSafePtr<OpalMixerMediaStream> & stream)
 {
   stream->GetConnection().GetEndPoint().GetManager().QueueDecoupledEvent(
                             new PSafeWorkNoArg<OpalMixerMediaStream, bool>(stream, &OpalMediaStream::Close));
@@ -1592,7 +1710,7 @@ void OpalMixerNode::MediaMixer::CloseOne(const PSafePtr<OpalMixerMediaStream> &
 
 ///////////////////////////////////////////////////////////////////////////////
 
-OpalMixerNode::AudioMixer::AudioMixer(const OpalMixerNodeInfo & info)
+OpalAudioStreamMixer::OpalAudioStreamMixer(const OpalMixerNodeInfo & info)
   : OpalAudioMixer(false, info.m_sampleRate)
 #if OPAL_MIXER_AUDIO_DEBUG
   , m_audioDebug(new PAudioMixerDebug(info.m_name))
@@ -1601,15 +1719,15 @@ OpalMixerNode::AudioMixer::AudioMixer(const OpalMixerNodeInfo & info)
 }
 
 
-OpalMixerNode::AudioMixer::~AudioMixer()
+OpalAudioStreamMixer::~OpalAudioStreamMixer()
 {
   StopPushThread();
 }
 
 
-void OpalMixerNode::AudioMixer::PushOne(PSafePtr<OpalMixerMediaStream> & stream,
-                                        CachedAudio & cache,
-                                        const short * audioToSubtract)
+void OpalAudioStreamMixer::PushOne(PSafePtr<OpalMixerMediaStream> & stream,
+                                   CachedAudio & cache,
+                                   const short * audioToSubtract)
 {
   MIXER_DEBUG_OUT(stream->GetID() << ',');
 
@@ -1631,6 +1749,8 @@ void OpalMixerNode::AudioMixer::PushOne(PSafePtr<OpalMixerMediaStream> & stream,
           << cache.m_encoded.GetTimestamp() << ','
           << cache.m_encoded.GetPayloadSize() << ',');
       stream.SetSafetyMode(PSafeReference); // OpalMediaStream::PushPacket might block
+      PTRACE(6, "Pushing cached encoded packet: pt=" << cache.m_encoded.GetPayloadType()
+             << " ts=" << cache.m_encoded.GetTimestamp() << " sz=" << cache.m_encoded.GetPayloadSize());
       stream->PushPacket(cache.m_encoded);
       stream.SetSafetyMode(PSafeReadOnly); // restore lock
       return;
@@ -1651,6 +1771,7 @@ void OpalMixerNode::AudioMixer::PushOne(PSafePtr<OpalMixerMediaStream> & stream,
         << cache.m_raw.GetPayloadSize() << ',');
     MIXER_DEBUG_WAV(stream->GetID(), cache.m_raw);
     stream.SetSafetyMode(PSafeReference); // OpalMediaStream::PushPacket might block
+    PTRACE(6, "Pushing raw packet: ts=" << cache.m_raw.GetTimestamp() << " sz=" << cache.m_raw.GetPayloadSize());
     stream->PushPacket(cache.m_raw);
     stream.SetSafetyMode(PSafeReadOnly); // restore lock
     return;
@@ -1659,11 +1780,12 @@ void OpalMixerNode::AudioMixer::PushOne(PSafePtr<OpalMixerMediaStream> & stream,
   if (cache.m_transcoder == NULL) {
     cache.m_transcoder = OpalTranscoder::Create(OpalPCM16, mediaFormat);
     if (cache.m_transcoder == NULL) {
-      PTRACE(2, "MixerNode\tCould not create transcoder to "
+      PTRACE(2, "Could not create transcoder to "
              << mediaFormat << " for stream id " << stream->GetID());
       CloseOne(stream);
       return;
     }
+    PTRACE(3, "Created transcoder to " << mediaFormat << " for stream id " << stream->GetID());
   }
 
   if (cache.m_raw.GetPayloadSize() < cache.m_transcoder->GetOptimalDataFrameSize(true)) {
@@ -1683,18 +1805,19 @@ void OpalMixerNode::AudioMixer::PushOne(PSafePtr<OpalMixerMediaStream> & stream,
         << cache.m_encoded.GetPayloadSize() << ',');
     MIXER_DEBUG_WAV(stream->GetID(), cache.m_raw);
     stream.SetSafetyMode(PSafeReference); // OpalMediaStream::PushPacket might block
+    PTRACE(6, "Pushing new packet: pt=" << cache.m_encoded.GetPayloadType()
+            << " ts=" << cache.m_encoded.GetTimestamp() << " sz=" << cache.m_encoded.GetPayloadSize());
     stream->PushPacket(cache.m_encoded);
     stream.SetSafetyMode(PSafeReadOnly); // restore lock
   }
   else {
-    PTRACE(2, "MixerNode\tCould not convert audio to "
-           << mediaFormat << " for stream id " << stream->GetID());
+    PTRACE(2, "Could not convert audio to " << mediaFormat << " for stream id " << stream->GetID());
     CloseOne(stream);
   }
 }
 
 
-bool OpalMixerNode::AudioMixer::OnPush()
+bool OpalAudioStreamMixer::OnPush()
 {
   MIXER_DEBUG_OUT(PTimer::Tick().GetMilliSeconds() << ',' << m_outputTimestamp << ',');
 
@@ -1742,14 +1865,14 @@ bool OpalMixerNode::AudioMixer::OnPush()
 }
 
 
-OpalMixerNode::AudioMixer::CachedAudio::CachedAudio()
+OpalAudioStreamMixer::CachedAudio::CachedAudio()
   : m_state(Collecting)
   , m_transcoder(NULL)
 {
 }
 
 
-OpalMixerNode::AudioMixer::CachedAudio::~CachedAudio()
+OpalAudioStreamMixer::CachedAudio::~CachedAudio()
 {
   delete m_transcoder;
 }
@@ -1758,24 +1881,43 @@ OpalMixerNode::AudioMixer::CachedAudio::~CachedAudio()
 ///////////////////////////////////////////////////////////////////////////////
 
 #if OPAL_VIDEO
-OpalMixerNode::VideoMixer::VideoMixer(const OpalMixerNodeInfo & info)
+OpalVideoStreamMixer::OpalVideoStreamMixer(const OpalMixerNodeInfo & info)
   : OpalVideoMixer(info.m_style, info.m_width, info.m_height, info.m_rate)
 {
 }
 
 
-OpalMixerNode::VideoMixer::~VideoMixer()
+OpalVideoStreamMixer::~OpalVideoStreamMixer()
 {
   StopPushThread();
 }
 
 
-bool OpalMixerNode::VideoMixer::OnMixed(RTP_DataFrame * & output)
+bool OpalVideoStreamMixer::SetFrameRate(unsigned rate)
 {
-  std::map<PString, RTP_DataFrameList> cachedVideo;
-  std::map<PString, RTP_DataFrame>     cachedFrameStore;
+  if (!OpalVideoMixer::SetFrameRate(rate))
+    return false;
+
+  for (TranscoderMap::iterator it = m_transcoders.begin(); it != m_transcoders.end(); ++it) {
+    OpalMediaFormat mediaFormat;
+    mediaFormat.SetOptionInteger(OpalMediaFormat::FrameTimeOption(), m_periodTS);
+    it->second.UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
+  }
+  return true;
+}
+
+
+bool OpalVideoStreamMixer::OnMixed(RTP_DataFrame * & output)
+{
+  typedef std::map<PString, RTP_DataFrameList> CachedPackets;
+  CachedPackets cachedPackets;
+  typedef std::map<unsigned, RTP_DataFrame> CachedFrameStore;
+  CachedFrameStore cachedFrameStore;
 
   for (PSafePtr<OpalMixerMediaStream> stream(m_outputStreams, PSafeReadOnly); stream != NULL; ++stream) {
+    if (stream->IsPaused())
+      continue;
+
     OpalMediaFormat mediaFormat = stream->GetMediaFormat();
     if (mediaFormat == OpalYUV420P) {
       stream.SetSafetyMode(PSafeReference); // OpalMediaStream::PushPacket might block
@@ -1783,59 +1925,88 @@ bool OpalMixerNode::VideoMixer::OnMixed(RTP_DataFrame * & output)
       stream.SetSafetyMode(PSafeReadOnly); // restore lock
     }
     else {
-      unsigned width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption());
-      unsigned height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption());
+      unsigned width, height;
+      const OpalVideoTranscoder::FrameHeader * header = (const OpalVideoTranscoder::FrameHeader *)output->GetPayloadPtr();
+      if (stream->CheckMixedVideoSize(header->width, header->height)) {
+        // Try and set outgoing video to same size as mixed frame store
+        mediaFormat.SetOptionInteger(OpalVideoFormat::FrameWidthOption(), header->width);
+        mediaFormat.SetOptionInteger(OpalVideoFormat::FrameHeightOption(), header->height);
+        if (!stream->UpdateMediaFormat(mediaFormat, true)) {
+          PTRACE(2, "Could not adjust media format to " << header->width << 'x' << header->height);
+          continue;
+        }
+        mediaFormat = stream->GetMediaFormat();
+        width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption());
+        height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption());
+        PTRACE(4, "Output of " << mediaFormat << " started at " << width << 'x' << height
+               << " (" << header->width << 'x' << header->height << ")"
+                  " to stream id " << stream->GetID());
+      }
+      else {
+        width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption());
+        height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption());
+      }
 
-      PStringStream key;
-      key << mediaFormat << width << height;
+      PStringStream keyPackets;
+      keyPackets << mediaFormat << ' ' << width << 'x' << height;
 
-      if (cachedVideo.find(key) == cachedVideo.end()) {
-        OpalTranscoder * transcoder = m_transcoders.GetAt(key);
+      CachedPackets::iterator itPackets = cachedPackets.find(keyPackets);
+      if (itPackets == cachedPackets.end()) {
+        OpalTranscoder * transcoder = m_transcoders.GetAt(keyPackets);
         if (transcoder == NULL) {
+          mediaFormat.SetOptionInteger(OpalMediaFormat::FrameTimeOption(), m_periodTS);
           transcoder = OpalTranscoder::Create(OpalYUV420P, mediaFormat);
           if (transcoder == NULL) {
-            PTRACE(2, "MixerNode\tCould not create transcoder to "
-                   << mediaFormat << " for stream id " << stream->GetID());
+            PTRACE(2, "Could not create transcoder to " << mediaFormat << " for stream id " << stream->GetID());
             CloseOne(stream);
             continue;
           }
-          PTRACE(3, "MixerNode\tCreated transcoder to "
-                 << mediaFormat << " for stream id " << stream->GetID());
-          m_transcoders.SetAt(key, transcoder);
+          PTRACE(3, "Created transcoder to " << mediaFormat << ' '
+                 << width << 'x' << height << " for stream id " << stream->GetID());
+          m_transcoders.SetAt(keyPackets, transcoder);
         }
 
-        const RTP_DataFrame * rawRTP;
-        const OpalVideoTranscoder::FrameHeader * header = (const OpalVideoTranscoder::FrameHeader *)output->GetPayloadPtr();
-        if (header->width == width && header->height == height)
+        RTP_DataFrame * rawRTP;
+        if (header->width == width && header->height == height) {
+          PTRACE(5, "Using mixer video frame: " << width << 'x' << height);
           rawRTP = output;
+        }
         else {
-          if (cachedFrameStore.find(key) == cachedFrameStore.end()) {
-            RTP_DataFrame & store = cachedFrameStore[key];
-            store.SetPayloadSize(width*height*3/2+sizeof(OpalVideoTranscoder::FrameHeader));
-            OpalVideoTranscoder::FrameHeader * resized = (OpalVideoTranscoder::FrameHeader *)store.GetPayloadPtr();
+          unsigned frameStoreKey = width + height*65536;
+          CachedFrameStore::iterator itFrameStore = cachedFrameStore.find(frameStoreKey);
+          if (itFrameStore != cachedFrameStore.end()) {
+            PTRACE(5, "Using cached video frame: " << header->width << 'x' << header->height << " to " << width << 'x' << height);
+            rawRTP = &itFrameStore->second;
+          }
+          else {
+            PTRACE(5, "Scaling video frame: " << header->width << 'x' << header->height << " to " << width << 'x' << height);
+            rawRTP = &cachedFrameStore[frameStoreKey];
+            rawRTP->CopyHeader(*output);
+            rawRTP->SetPayloadSize(width*height*3/2+sizeof(OpalVideoTranscoder::FrameHeader));
+            OpalVideoTranscoder::FrameHeader * resized = (OpalVideoTranscoder::FrameHeader *)rawRTP->GetPayloadPtr();
             resized->width = width;
             resized->height = height;
             PColourConverter::CopyYUV420P(0, 0, header->width, header->height,
-                                            header->width, header->height, OPAL_VIDEO_FRAME_DATA_PTR(header),
+                                          header->width, header->height, OPAL_VIDEO_FRAME_DATA_PTR(header),
                                           0, 0, width, height,
                                           width, height, OPAL_VIDEO_FRAME_DATA_PTR(resized),
                                           PVideoFrameInfo::eScale);
           }
-          rawRTP = &cachedFrameStore[key];
         }
 
-        if (!transcoder->ConvertFrames(*rawRTP, cachedVideo[key])) {
-          PTRACE(2, "MixerNode\tCould not convert video to "
-                 << mediaFormat << " for stream id " << stream->GetID());
+        RTP_DataFrameList packets;
+        if (!transcoder->ConvertFrames(*rawRTP, packets)) {
+          PTRACE(2, "Could not convert video to " << mediaFormat << " for stream id " << stream->GetID());
           CloseOne(stream);
           continue;
         }
+
+        itPackets = cachedPackets.insert(CachedPackets::value_type(keyPackets, packets)).first;
       }
 
       stream.SetSafetyMode(PSafeReference); // OpalMediaStream::PushPacket might block
 
-      RTP_DataFrameList & list = cachedVideo[key];
-      for (RTP_DataFrameList::iterator frame = list.begin(); frame != list.end(); ++frame)
+      for (RTP_DataFrameList::iterator frame = itPackets->second.begin(); frame != itPackets->second.end(); ++frame)
         stream->PushPacket(*frame);
 
       stream.SetSafetyMode(PSafeReadOnly); // restore lock
@@ -1865,7 +2036,7 @@ OpalMixerNodeManager::~OpalMixerNodeManager()
 
 void OpalMixerNodeManager::ShutDown()
 {
-  PTRACE(4, "Mixer\tDestroying " << m_nodesByUID.GetSize() << '/' << m_nodesByName.GetSize() << " nodes");
+  PTRACE(4, "Destroying " << m_nodesByUID.GetSize() << '/' << m_nodesByName.GetSize() << " nodes");
 
   PSafePtr<OpalMixerNode> node;
   while ((node = m_nodesByUID.GetAt(0)) != NULL)
@@ -1894,7 +2065,7 @@ PSafePtr<OpalMixerNode> OpalMixerNodeManager::AddNode(OpalMixerNodeInfo * info)
     delete info;
   else {
     m_nodesByUID.SetAt(node->GetGUID(), node);
-    PTRACE(3, "MixerEP\tAdded new node, id=" << node->GetGUID());
+    PTRACE(3, "Added new node, id=" << node->GetGUID());
     OnNodeStatusChanged(*node, OpalConferenceState::Created);
   }
 
@@ -1923,10 +2094,13 @@ void OpalMixerNodeManager::RemoveNode(OpalMixerNode & node)
 }
 
 
-void OpalMixerNodeManager::AddNodeName(PString name,
-                               OpalMixerNode * node)
+bool OpalMixerNodeManager::AddNodeName(PString name, OpalMixerNode * node)
 {
+  if (m_nodesByName.FindWithLock(name, PSafeReference) != NULL)
+    return false;
+
   m_nodesByName.SetAt(name, node);
+  return true;
 }
 
 
@@ -1954,13 +2128,16 @@ void OpalMixerNodeManager::OnNodeStatusChanged(const OpalMixerNode &, OpalConfer
 }
 
 
-#else
+OpalAudioStreamMixer * OpalMixerNodeManager::CreateAudioMixer(const OpalMixerNodeInfo & info)
+{
+  return new OpalAudioStreamMixer(info);
+}
 
-  #ifdef _MSC_VER
-    #pragma message("Mixer (MCU) support DISABLED")
-  #endif
 
+#if OPAL_VIDEO
+OpalVideoStreamMixer * OpalMixerNodeManager::CreateVideoMixer(const OpalMixerNodeInfo & info)
+{
+  return new OpalVideoStreamMixer(info);
+}
+#endif
 #endif // OPAL_HAS_MIXER
-
-
-//////////////////////////////////////////////////////////////////////////////
