@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29960 $
+ * $Revision: 33766 $
  * $Author: rjongbloed $
- * $Date: 2013-06-13 23:26:53 -0500 (Thu, 13 Jun 2013) $
+ * $Date: 2015-07-27 21:27:47 +0100 (Mon, 27 Jul 2015) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -53,6 +53,7 @@
 #include <h323/h323rtp.h>
 #include <h323/transaddr.h>
 #include <h323/h323pdu.h>
+#include <asn/h235_srtp.h>
 
 
 #define new PNEW
@@ -144,6 +145,7 @@ H323Channel::H323Channel(H323Connection & conn, const H323Capability & cap)
   , connection(conn)
   , capability((H323Capability *)cap.Clone())
   , opened(false)
+  , m_terminating(false)
   , m_bandwidthUsed(0)
 {
   PTRACE_CONTEXT_ID_FROM(conn);
@@ -184,20 +186,18 @@ PBoolean H323Channel::GetMediaTransportAddress(OpalTransportAddress & /*data*/,
 
 void H323Channel::Close()
 {
-  if (m_terminating++ == 0)
+  if (!m_terminating.exchange(true))
     InternalClose();
 }
 
 
 void H323Channel::InternalClose()
 {
-  if (opened) {
-    // Signal to the connection that this channel is on the way out
+  // Signal to the connection that this channel is on the way out
+  if (opened)
     connection.OnClosedLogicalChannel(*this);
-
-    connection.SetBandwidthUsed(GetDirection() == IsReceiver ? OpalBandwidth::Rx : OpalBandwidth::Tx, m_bandwidthUsed, 0);
-  }
   
+  connection.SetBandwidthUsed(GetDirection() == IsReceiver ? OpalBandwidth::Rx : OpalBandwidth::Tx, m_bandwidthUsed, 0);
   PTRACE(4, "LogChan\tCleaned up " << number);
 }
 
@@ -226,6 +226,12 @@ PBoolean H323Channel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck & /*pdu*
 }
 
 
+PBoolean H323Channel::OnSendingPDU(H245_OpenLogicalChannel & open) const
+{
+  return capability->OnSendingPDU(open.m_forwardLogicalChannelParameters.m_dataType);
+}
+
+
 void H323Channel::OnSendOpenAck(const H245_OpenLogicalChannel & /*pdu*/,
                                 H245_OpenLogicalChannelAck & /* pdu*/) const
 {
@@ -238,35 +244,32 @@ void H323Channel::OnFlowControl(long bitRateRestriction)
 
   OpalMediaStreamPtr stream = GetMediaStream();
   if (stream != NULL)
-    stream->ExecuteCommand(OpalMediaFlowControl(bitRateRestriction*100));
+    connection.ExecuteMediaCommand(OpalMediaFlowControl(bitRateRestriction*100, stream->GetMediaFormat().GetMediaType(), GetSessionID()));
 }
 
 
 void H323Channel::OnMiscellaneousCommand(const H245_MiscellaneousCommand_type & type)
 {
   PTRACE(3, "LogChan\tOnMiscellaneousCommand: chan=" << number << ", type=" << type.GetTagName());
-  OpalMediaStreamPtr mediaStream = GetMediaStream();
-  if (mediaStream == NULL)
-    return;
 
 #if OPAL_VIDEO
   switch (type.GetTag())
   {
     case H245_MiscellaneousCommand_type::e_videoFreezePicture :
-      mediaStream->ExecuteCommand(OpalVideoFreezePicture());
+      connection.ExecuteMediaCommand(OpalVideoFreezePicture(GetSessionID()));
       break;
 
     case H245_MiscellaneousCommand_type::e_videoFastUpdatePicture:
-      mediaStream->ExecuteCommand(OpalVideoUpdatePicture());
+      connection.ExecuteMediaCommand(OpalVideoUpdatePicture(GetSessionID()));
       break;
 
     case H245_MiscellaneousCommand_type::e_videoFastUpdateGOB :
     case H245_MiscellaneousCommand_type::e_videoFastUpdateMB :
-      mediaStream->ExecuteCommand(OpalVideoPictureLoss());
+      connection.ExecuteMediaCommand(OpalVideoPictureLoss(0, 0, GetSessionID()));
       break;
 
     case H245_MiscellaneousCommand_type::e_videoTemporalSpatialTradeOff :
-      mediaStream->ExecuteCommand(OpalTemporalSpatialTradeOff((const PASN_Integer &)type));
+      connection.ExecuteMediaCommand(OpalTemporalSpatialTradeOff((const PASN_Integer &)type, GetSessionID()));
       break;
 
     default:
@@ -298,16 +301,28 @@ PBoolean H323Channel::SetBandwidthUsed(OpalBandwidth bandwidth)
 {
   if (!connection.SetBandwidthUsed(GetDirection() == IsReceiver ? OpalBandwidth::Rx : OpalBandwidth::Tx, m_bandwidthUsed, bandwidth)) {
     m_bandwidthUsed = 0;
-    PTRACE(2, "LogChan\tFailed bandwidth request=" << bandwidth << ", used=" << m_bandwidthUsed);
+    PTRACE(2, "LogChan\tFailed bandwidth request=" << bandwidth << ", currently using=" << m_bandwidthUsed);
     return false;
   }
 
-  PTRACE(3, "LogChan\tBandwidth requested=" << bandwidth << ", used=" << m_bandwidthUsed);
+  PTRACE(3, "LogChan\tBandwidth requested=" << bandwidth << ", currently using=" << m_bandwidthUsed);
   m_bandwidthUsed = bandwidth;
   return true;
 }
 
 
+OpalMediaFormat H323Channel::GetMediaFormat() const
+{
+  return capability->GetMediaFormat();
+}
+
+
+bool H323Channel::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
+{
+  return capability->UpdateMediaFormat(mediaFormat);
+}
+
+
 bool H323Channel::PreOpen()
 {
   return Open();
@@ -339,7 +354,6 @@ H323UnidirectionalChannel::H323UnidirectionalChannel(H323Connection & conn,
                                                      Directions direction)
   : H323Channel(conn, cap)
   , receiver(direction == IsReceiver)
-  , m_mediaFormat(capability->GetMediaFormat())
 {
 }
 
@@ -357,8 +371,18 @@ H323Channel::Directions H323UnidirectionalChannel::GetDirection() const
 PBoolean H323UnidirectionalChannel::SetInitialBandwidth()
 {
   OpalBandwidth bandwidth;
-  if (GetDirection() == IsTransmitter)
-    bandwidth = capability->GetMediaFormat().GetOptionInteger(OpalMediaFormat::TargetBitRateOption());
+  if (GetDirection() == IsTransmitter) {
+    OpalMediaFormat mediaFormat = GetMediaFormat();
+    bandwidth = mediaFormat.GetOptionInteger(OpalMediaFormat::TargetBitRateOption());
+    OpalBandwidth available = connection.GetBandwidthAvailable(OpalBandwidth::Tx);
+    if (bandwidth > available) {
+      PTRACE(3, "LogChan", "Adjusting " << GetNumber() << ' ' << capability->GetMediaFormat()
+             << " target bandwidth from " << bandwidth << " to " << available);
+      bandwidth = available;
+      mediaFormat.SetOptionInteger(OpalMediaFormat::TargetBitRateOption(), bandwidth);
+      UpdateMediaFormat(mediaFormat);
+    }
+  }
   if (bandwidth == 0)
     bandwidth = capability->GetMediaFormat().GetOptionInteger(OpalMediaFormat::MaxBitRateOption());
   return SetBandwidthUsed(bandwidth);
@@ -367,34 +391,35 @@ PBoolean H323UnidirectionalChannel::SetInitialBandwidth()
 
 bool H323UnidirectionalChannel::PreOpen()
 {
-  if (m_mediaStream != NULL)
-    return true;
-
-  m_mediaStream = connection.CreateMediaStream(m_mediaFormat, GetSessionID(), receiver);
-
-  OpalCall & call = connection.GetCall();
-  OpalMediaType mediaType = m_mediaFormat.GetMediaType();
-
-  if (GetDirection() == IsReceiver) {
-    if (!call.OpenSourceMediaStreams(connection, mediaType, GetSessionID(), m_mediaFormat)) {
-      PTRACE(1, "LogChan\tReceive OpenSourceMediaStreams failed");
+  if (m_mediaStream == NULL) {
+    OpalMediaFormat mediaFormat = GetMediaFormat();
+    m_mediaStream = connection.CreateMediaStream(mediaFormat, GetSessionID(), receiver);
+    if (m_mediaStream == NULL)
       return false;
+
+    OpalCall & call = connection.GetCall();
+    OpalMediaType mediaType = mediaFormat.GetMediaType();
+
+    if (GetDirection() == IsReceiver) {
+      if (!call.OpenSourceMediaStreams(connection, mediaType, GetSessionID(), mediaFormat)) {
+        PTRACE(1, "LogChan\tReceive OpenSourceMediaStreams failed");
+        return false;
+      }
     }
-  }
-  else {
-    PSafePtr<OpalConnection> otherConnection = call.GetOtherPartyConnection(connection);
-    if (otherConnection == NULL) {
-      PTRACE(1, "LogChan\tTransmit failed, no other connection");
-      return false;
-    }
-    if (!call.OpenSourceMediaStreams(*otherConnection, mediaType, GetSessionID(), m_mediaFormat)) {
-      PTRACE(1, "LogChan\tTransmit OpenSourceMediaStreams failed");
-      return false;
+    else {
+      PSafePtr<OpalConnection> otherConnection = call.GetOtherPartyConnection(connection);
+      if (otherConnection == NULL) {
+        PTRACE(1, "LogChan\tTransmit failed, no other connection");
+        return false;
+      }
+      if (!call.OpenSourceMediaStreams(*otherConnection, mediaType, GetSessionID(), mediaFormat)) {
+        PTRACE(1, "LogChan\tTransmit OpenSourceMediaStreams failed");
+        return false;
+      }
     }
   }
 
-  m_mediaFormat = m_mediaStream->GetMediaFormat();
-  capability->UpdateMediaFormat(m_mediaFormat);
+  capability->UpdateMediaFormat(m_mediaStream->GetMediaFormat());
   return true;
 }
 
@@ -410,7 +435,7 @@ void H323UnidirectionalChannel::InternalClose()
   PTRACE(4, "H323RTP\tCleaning up media stream on " << number);
 
   // If we have source media stream close it
-  if (m_mediaStream != NULL) {
+  if (m_mediaStream != NULL && connection.GetPhase() < OpalConnection::ReleasingPhase) {
     if (!m_mediaStream->Close())
       connection.RemoveMediaStream(*m_mediaStream);
     m_mediaStream.SetNULL();
@@ -420,6 +445,13 @@ void H323UnidirectionalChannel::InternalClose()
 }
 
 
+bool H323UnidirectionalChannel::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
+{
+  return H323Channel::UpdateMediaFormat(mediaFormat) &&
+          (m_mediaStream == NULL || m_mediaStream->UpdateMediaFormat(mediaFormat));
+}
+
+
 OpalMediaStreamPtr H323UnidirectionalChannel::GetMediaStream() const
 {
   return m_mediaStream;
@@ -429,7 +461,13 @@ OpalMediaStreamPtr H323UnidirectionalChannel::GetMediaStream() const
 void H323UnidirectionalChannel::SetMediaStream(OpalMediaStreamPtr mediaStream)
 {
   m_mediaStream = mediaStream;
-  m_mediaFormat = m_mediaStream->GetMediaFormat();
+  if (mediaStream != NULL) {
+    capability->UpdateMediaFormat(m_mediaStream->GetMediaFormat());
+    if (!m_mediaStream->SetMediaFormat(capability->GetMediaFormat())) {
+      m_mediaStream->Close();
+      m_mediaStream.SetNULL();
+    }
+  }
 }
 
 
@@ -462,8 +500,18 @@ PBoolean H323_RealTimeChannel::OnSendingPDU(H245_OpenLogicalChannel & open) cons
 {
   PTRACE(3, "H323RTP\tOnSendingPDU");
 
+  if (!H323Channel::OnSendingPDU(open))
+    return false;
+
   open.m_forwardLogicalChannelNumber = (unsigned)number;
 
+#if OPAL_H235_6 || OPAL_H235_8
+  if (capability->OnSendingPDU(open.m_encryptionSync, connection, GetSessionID(), false)) {
+    open.IncludeOptionalField(H245_OpenLogicalChannel::e_encryptionSync);
+    open.m_encryptionSync.m_synchFlag = GetDynamicRTPPayloadType();
+  }
+#endif // OPAL_H235_6 || OPAL_H235_8
+
   if (open.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)) {
     open.m_reverseLogicalChannelParameters.IncludeOptionalField(
             H245_OpenLogicalChannel_reverseLogicalChannelParameters::e_multiplexParameters);
@@ -502,12 +550,15 @@ PBoolean H323_RealTimeChannel::OnSendingPDU(H245_H2250LogicalChannelParameters &
 }
 
 
-void H323_RealTimeChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & /*param*/) const
+void H323_RealTimeChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & param) const
 {
+  // set session ID
+  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
+  param.m_sessionID = GetSessionID();
 }
 
 
-void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
+void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & /*open*/,
                                          H245_OpenLogicalChannelAck & ack) const
 {
   PTRACE(3, "H323RTP\tOnSendOpenAck");
@@ -520,18 +571,16 @@ void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
       H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters);
 
   // get H225 parms
-  H245_H2250LogicalChannelAckParameters & param = ack.m_forwardMultiplexAckParameters;
+  OnSendOpenAck(ack.m_forwardMultiplexAckParameters);
 
-  // set session ID
-  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
-  const H245_H2250LogicalChannelParameters & openparam =
-                          open.m_forwardLogicalChannelParameters.m_multiplexParameters;
-  unsigned sessionID = openparam.m_sessionID;
-  param.m_sessionID = sessionID;
+#if OPAL_H235_6 || OPAL_H235_8
+  if (capability->OnSendingPDU(ack.m_encryptionSync, connection, GetSessionID(), true)) {
+    ack.IncludeOptionalField(H245_OpenLogicalChannelAck::e_encryptionSync);
+    ack.m_encryptionSync.m_synchFlag = GetDynamicRTPPayloadType();
+  }
+#endif // OPAL_H235_6 || OPAL_H235_8
 
-  OnSendOpenAck(param);
-
-  PTRACE(3, "H323RTP\tSending open logical channel ACK: sessionID=" << sessionID);
+  PTRACE(3, "H323RTP\tSending open logical channel ACK: sessionID=" << GetSessionID());
 }
 
 
@@ -553,6 +602,11 @@ PBoolean H323_RealTimeChannel::OnReceivedPDU(const H245_OpenLogicalChannel & ope
     return false;
   }
 
+#if OPAL_H235_6 || OPAL_H235_8
+  if (open.HasOptionalField(H245_OpenLogicalChannel::e_encryptionSync))
+    capability->OnReceivedPDU(open.m_encryptionSync, connection, GetSessionID(), true);
+#endif // OPAL_H235_6 || OPAL_H235_8
+
   if (reverse) {
     if (open.m_reverseLogicalChannelParameters.m_multiplexParameters.GetTag() ==
              H245_OpenLogicalChannel_reverseLogicalChannelParameters_multiplexParameters::e_h2250LogicalChannelParameters)
@@ -576,6 +630,7 @@ PBoolean H323_RealTimeChannel::OnReceivedPDU(const H245_H2250LogicalChannelParam
   unsigned sessionID = param.m_sessionID;
 
   if (connection.IsH245Master()) {
+    PAssert(GetSessionID() < 256, PLogicError);
     if (sessionID == 0)
       return true;
   }
@@ -609,8 +664,13 @@ PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck
     return false;
   }
 
-  if (ack.HasOptionalField(H245_OpenLogicalChannel::e_genericInformation))
-    OnReceivedAckAltPDU(ack.m_genericInformation);
+  if (ack.HasOptionalField(H245_OpenLogicalChannelAck::e_genericInformation))
+    connection.OnReceiveOLCGenericInformation(GetSessionID(), ack.m_genericInformation, true);
+
+#if OPAL_H235_6 || OPAL_H235_8
+  if (ack.HasOptionalField(H245_OpenLogicalChannelAck::e_encryptionSync))
+    capability->OnReceivedPDU(ack.m_encryptionSync, connection, GetSessionID(), false);
+#endif // OPAL_H235_6 || OPAL_H235_8
 
   return OnReceivedAckPDU(ack.m_forwardMultiplexAckParameters);
 }
@@ -622,7 +682,7 @@ PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAc
   if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID))
     sessionID = param.m_sessionID;
 
-  if (connection.IsH245Master() || sessionID <= H323Capability::DefaultDataSessionID) {
+  if (connection.IsH245Master() || sessionID < H323Capability::DefaultDataSessionID) {
     PTRACE_IF(2, sessionID != 0 && sessionID != GetSessionID(),
               "LogChan\tAck contains invalid session ID " << param.m_sessionID << ", ignoring");
     return true;
@@ -634,9 +694,9 @@ PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAc
   }
 
   if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_dynamicRTPPayloadType)) {
-    m_mediaFormat.SetPayloadType((RTP_DataFrame::PayloadTypes)param.m_dynamicRTPPayloadType.GetValue());
-    if (m_mediaStream != NULL)
-      m_mediaStream->UpdateMediaFormat(m_mediaFormat);
+    OpalMediaFormat mediaFormat = GetMediaFormat();
+    mediaFormat.SetPayloadType((RTP_DataFrame::PayloadTypes)param.m_dynamicRTPPayloadType.GetValue());
+    UpdateMediaFormat(mediaFormat);
   }
 
   return SetSessionID(sessionID);
@@ -702,6 +762,9 @@ PBoolean H323DataChannel::OnSendingPDU(H245_OpenLogicalChannel & open) const
 {
   PTRACE(3, "LogChan\tOnSendingPDU for channel: " << number);
 
+  if (!H323Channel::OnSendingPDU(open))
+    return false;
+
   open.m_forwardLogicalChannelNumber = (unsigned)number;
 
   open.m_forwardLogicalChannelParameters.m_multiplexParameters.SetTag(
