@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28483 $
+ * $Revision: 31784 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2014-04-16 10:27:05 +0100 (Wed, 16 Apr 2014) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -47,12 +47,9 @@
 #include <h323/h323pdu.h>
 #include <h323/h235auth.h>
 #include <asn/h248.h>
-
+#include <h460/h460.h>
 #include <ptclib/random.h>
 
-#if OPAL_H460
-#include <h460/h460.h>
-#endif
 
 #define new PNEW
 
@@ -382,67 +379,51 @@ PBoolean H225_RAS::OnReceiveRequestInProgress(const H225_RequestInProgress & /*r
 }
 
 
-template <typename PDUType>
-static void SendGenericData(const H225_RAS * ras, unsigned code, PDUType & pdu)
-{
-  H225_FeatureSet fs;
-  if (!ras->OnSendFeatureSet(code,fs))
-    return;
-
-  if (fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
-    pdu.IncludeOptionalField(PDUType::e_genericData);
-
-    H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
-    H225_ArrayOf_GenericData & data = pdu.m_genericData;
-
-    for (PINDEX i=0; i < fsn.GetSize(); i++) {
-      PINDEX lastPos = data.GetSize();
-      data.SetSize(lastPos+1);
-      data[lastPos] = fsn[i];
-    }
-  }
-}
-
-
-template <typename PDUType>
-static void SendFeatureSet(const H225_RAS * ras, unsigned code, PDUType & pdu)
-{
-  H225_FeatureSet fs;
-  if (!ras->OnSendFeatureSet(code,fs))
-    return;
-
-  switch (code) {
 #if OPAL_H460
-    case H460_MessageType::e_gatekeeperRequest:
-    case H460_MessageType::e_gatekeeperConfirm:
-    case H460_MessageType::e_gatekeeperReject:
-    case H460_MessageType::e_registrationRequest:
-    case H460_MessageType::e_registrationConfirm: 
-    case H460_MessageType::e_registrationReject:
-    case H460_MessageType::e_setup:                    
-    case H460_MessageType::e_callProceeding:
-      pdu.IncludeOptionalField(PDUType::e_featureSet);
-      pdu.m_featureSet = fs;
-      break;
-#endif
-    default:
-      if (fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
-        pdu.IncludeOptionalField(PDUType::e_genericData);
+/// For some completely silly reason the ITU decided to send/receive H460 Messages in two places,
+/// for some messages it is included in the messsage body FeatureSet (to Advertise it) and others 
+/// in the genericData field (as actual information). Even though a Feature is generic data. It is beyond
+/// me to determine Why it is so. Anyway if a message is to be carried in the genericData field it is given
+/// the default category of supported.
 
-        H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
-        H225_ArrayOf_GenericData & data = pdu.m_genericData;
-
-        for (PINDEX i=0; i < fsn.GetSize(); i++) {
-          PINDEX lastPos = data.GetSize();
-          data.SetSize(lastPos+1);
-          data[lastPos] = fsn[i];
-        }
-      }
-      break;
-   }
+template <typename PDUType>
+static void SendGenericData(const H225_RAS * ras, H460_MessageType pduType, PDUType & pdu)
+{
+  H225_FeatureSet fs;
+  if (ras->OnSendFeatureSet(pduType, fs) && H460_FeatureSet::Copy(pdu.m_genericData, fs))
+    pdu.IncludeOptionalField(PDUType::e_genericData);
 }
 
 
+template <typename PDUType>
+static void SendFeatureSet(const H225_RAS * ras, H460_MessageType pduType, PDUType & pdu)
+{
+  if (ras->OnSendFeatureSet(pduType, pdu.m_featureSet))
+    pdu.IncludeOptionalField(PDUType::e_featureSet);
+}
+
+
+template <typename PDUType>
+static void ReceiveGenericData(const H225_RAS * ras, H460_MessageType pduType, const PDUType & pdu)
+{
+  if (!pdu.HasOptionalField(PDUType::e_genericData))
+    return;
+
+  H225_FeatureSet fs;
+  if (H460_FeatureSet::Copy(fs, pdu.m_genericData))
+    ras->OnReceiveFeatureSet(pduType, fs);
+}
+
+
+template <typename PDUType>
+static void ReceiveFeatureSet(const H225_RAS * ras, H460_MessageType pduType, const PDUType & pdu)
+{
+  if (pdu.HasOptionalField(PDUType::e_featureSet))
+    ras->OnReceiveFeatureSet(pduType, pdu.m_featureSet);
+}
+#endif // OPAL_H460
+
+
 void H225_RAS::OnSendGatekeeperRequest(H323RasPDU &, H225_GatekeeperRequest & grq)
 {
   // This function is never called during sending GRQ
@@ -458,7 +439,7 @@ void H225_RAS::OnSendGatekeeperRequest(H323RasPDU &, H225_GatekeeperRequest & gr
 void H225_RAS::OnSendGatekeeperRequest(H225_GatekeeperRequest & grq)
 {
 #if OPAL_H460
-  SendFeatureSet<H225_GatekeeperRequest>(this, H460_MessageType::e_gatekeeperRequest, grq);
+  SendFeatureSet(this, H460_MessageType::e_gatekeeperRequest, grq);
 #endif
 }
 
@@ -471,7 +452,7 @@ void H225_RAS::OnSendGatekeeperConfirm(H323RasPDU &, H225_GatekeeperConfirm & gc
   }
 
 #if OPAL_H460
-  SendFeatureSet<H225_GatekeeperConfirm>(this, H460_MessageType::e_gatekeeperConfirm, gcf);
+  SendFeatureSet(this, H460_MessageType::e_gatekeeperConfirm, gcf);
 #endif
 
   OnSendGatekeeperConfirm(gcf);
@@ -491,7 +472,7 @@ void H225_RAS::OnSendGatekeeperReject(H323RasPDU &, H225_GatekeeperReject & grj)
   }
     
 #if OPAL_H460
-  SendFeatureSet<H225_GatekeeperReject>(this, H460_MessageType::e_gatekeeperReject, grj);
+  SendFeatureSet(this, H460_MessageType::e_gatekeeperReject, grj);
 #endif
 
   OnSendGatekeeperReject(grj);
@@ -503,45 +484,10 @@ void H225_RAS::OnSendGatekeeperReject(H225_GatekeeperReject & /*grj*/)
 }
 
 
-template <typename PDUType>
-static void ProcessFeatureSet(const H225_RAS * ras, unsigned code, const PDUType & pdu)
-{
-  if (pdu.HasOptionalField(PDUType::e_genericData)) {
-    H225_FeatureSet fs;
-    fs.IncludeOptionalField(H225_FeatureSet::e_supportedFeatures);
-    H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
-    const H225_ArrayOf_GenericData & data = pdu.m_genericData;
-    for (PINDEX i=0; i < data.GetSize(); i++) {
-      PINDEX lastPos = fsn.GetSize();
-      fsn.SetSize(lastPos+1);
-      fsn[lastPos] = (H225_FeatureDescriptor &)data[i];
-    }
-    ras->OnReceiveFeatureSet(code, fs);
-  }
-}
-
-
-template <typename PDUType>
-static void ReceiveGenericData(const H225_RAS * ras, unsigned code, const PDUType & pdu)
-{
-  ProcessFeatureSet<PDUType>(ras,code,pdu);
-}
-
-
-template <typename PDUType>
-static void ReceiveFeatureSet(const H225_RAS * ras, unsigned code, const PDUType & pdu)
-{
-  if (pdu.HasOptionalField(PDUType::e_featureSet))
-    ras->OnReceiveFeatureSet(code, pdu.m_featureSet);
-
-  ProcessFeatureSet<PDUType>(ras,code,pdu);
-}
-
-
 PBoolean H225_RAS::OnReceiveGatekeeperRequest(const H323RasPDU &, const H225_GatekeeperRequest & grq)
 {
 #if OPAL_H460
-  ReceiveFeatureSet<H225_GatekeeperRequest>(this, H460_MessageType::e_gatekeeperRequest, grq);
+  ReceiveFeatureSet(this, H460_MessageType::e_gatekeeperRequest, grq);
 #endif
 
   return OnReceiveGatekeeperRequest(grq);
@@ -574,7 +520,7 @@ PBoolean H225_RAS::OnReceiveGatekeeperConfirm(const H323RasPDU &, const H225_Gat
 
 #if OPAL_H460
   if (gcf.HasOptionalField(H225_GatekeeperConfirm::e_featureSet))
-    ReceiveFeatureSet<H225_GatekeeperConfirm>(this, H460_MessageType::e_gatekeeperConfirm, gcf);
+    ReceiveFeatureSet(this, H460_MessageType::e_gatekeeperConfirm, gcf);
   else
     DisableFeatureSet();
 #endif
@@ -595,7 +541,7 @@ PBoolean H225_RAS::OnReceiveGatekeeperReject(const H323RasPDU &, const H225_Gate
     return false;
     
 #if OPAL_H460
-  ReceiveFeatureSet<H225_GatekeeperReject>(this, H460_MessageType::e_gatekeeperReject, grj);
+  ReceiveFeatureSet(this, H460_MessageType::e_gatekeeperReject, grj);
 #endif
 
   return OnReceiveGatekeeperReject(grj);
@@ -613,11 +559,13 @@ void H225_RAS::OnSendRegistrationRequest(H323RasPDU & pdu, H225_RegistrationRequ
   OnSendRegistrationRequest(rrq);
     
 #if OPAL_H460
-  SendFeatureSet<H225_RegistrationRequest>(this, H460_MessageType::e_registrationRequest, rrq);
+  SendFeatureSet(this,
+                 rrq.m_keepAlive.GetValue() ? H460_MessageType::e_lightweightRegistrationRequest
+                                            : H460_MessageType::e_registrationRequest,
+                 rrq);
 #endif
     
-  pdu.Prepare(rrq.m_tokens, H225_RegistrationRequest::e_tokens,
-              rrq.m_cryptoTokens, H225_RegistrationRequest::e_cryptoTokens);
+  pdu.Prepare(rrq);
 }
 
 
@@ -636,11 +584,10 @@ void H225_RAS::OnSendRegistrationConfirm(H323RasPDU & pdu, H225_RegistrationConf
   OnSendRegistrationConfirm(rcf);
 
 #if OPAL_H460
-  SendFeatureSet<H225_RegistrationConfirm>(this, H460_MessageType::e_registrationConfirm, rcf);
+  SendFeatureSet(this, H460_MessageType::e_registrationConfirm, rcf);
 #endif
 
-  pdu.Prepare(rcf.m_tokens, H225_RegistrationConfirm::e_tokens,
-              rcf.m_cryptoTokens, H225_RegistrationConfirm::e_cryptoTokens);
+  pdu.Prepare(rcf);
 }
 
 
@@ -659,11 +606,10 @@ void H225_RAS::OnSendRegistrationReject(H323RasPDU & pdu, H225_RegistrationRejec
   OnSendRegistrationReject(rrj);
     
 #if OPAL_H460
-  SendFeatureSet<H225_RegistrationReject>(this, H460_MessageType::e_registrationReject, rrj);
+  SendFeatureSet(this, H460_MessageType::e_registrationReject, rrj);
 #endif
 
-  pdu.Prepare(rrj.m_tokens, H225_RegistrationReject::e_tokens,
-              rrj.m_cryptoTokens, H225_RegistrationReject::e_cryptoTokens);
+  pdu.Prepare(rrj);
 }
 
 
@@ -675,7 +621,7 @@ void H225_RAS::OnSendRegistrationReject(H225_RegistrationReject & /*rrj*/)
 PBoolean H225_RAS::OnReceiveRegistrationRequest(const H323RasPDU &, const H225_RegistrationRequest & rrq)
 {
 #if OPAL_H460
-  ReceiveFeatureSet<H225_RegistrationRequest>(this, H460_MessageType::e_registrationRequest, rrq);
+  ReceiveFeatureSet(this, H460_MessageType::e_registrationRequest, rrq);
 #endif
     
   return OnReceiveRegistrationRequest(rrq);
@@ -695,27 +641,25 @@ PBoolean H225_RAS::OnReceiveRegistrationConfirm(const H323RasPDU & pdu, const H2
 
   if (lastRequest != NULL) {
     PString endpointIdentifier = rcf.m_endpointIdentifier;
-    // NOTE this diff here might not have been needed.. needs review
-    const H235Authenticators & authenticators = lastRequest->requestPDU.GetAuthenticators();
-    for (PINDEX i = 0; i < authenticators.GetSize(); i++) {
-      H235Authenticator & authenticator = authenticators[i];
-      if (authenticator.UseGkAndEpIdentifiers())
-        authenticator.SetLocalId(endpointIdentifier);
+    H235Authenticators & authenticators = lastRequest->requestPDU.GetAuthenticators();
+    for (H235Authenticators::iterator it = authenticators.begin(); it != authenticators.end(); ++it) {
+      if (it->UseGkAndEpIdentifiers())
+        it->SetLocalId(endpointIdentifier);
     }
   }
 
-  if (!CheckCryptoTokens(pdu,
-                         rcf.m_tokens, H225_RegistrationConfirm::e_tokens,
-                         rcf.m_cryptoTokens, H225_RegistrationConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, rcf))
     return false;
 
 #if OPAL_H460
   if (rcf.HasOptionalField(H225_RegistrationConfirm::e_featureSet))
-    ReceiveFeatureSet<H225_RegistrationConfirm>(this, H460_MessageType::e_registrationConfirm, rcf);
+    ReceiveFeatureSet(this, H460_MessageType::e_registrationConfirm, rcf);
   else
     DisableFeatureSet();
 #endif
 
+  if (rcf.HasOptionalField(H225_RegistrationConfirm::e_gatekeeperIdentifier))
+    gatekeeperIdentifier = rcf.m_gatekeeperIdentifier;
   return OnReceiveRegistrationConfirm(rcf);
 }
 
@@ -731,13 +675,11 @@ PBoolean H225_RAS::OnReceiveRegistrationReject(const H323RasPDU & pdu, const H22
   if (!CheckForResponse(H225_RasMessage::e_registrationRequest, rrj.m_requestSeqNum, &rrj.m_rejectReason))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         rrj.m_tokens, H225_RegistrationReject::e_tokens,
-                         rrj.m_cryptoTokens, H225_RegistrationReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, rrj))
     return false;
   
 #if OPAL_H460
-  ReceiveFeatureSet<H225_RegistrationReject>(this, H460_MessageType::e_registrationReject, rrj);
+  ReceiveFeatureSet(this, H460_MessageType::e_registrationReject, rrj);
 #endif
 
   return OnReceiveRegistrationReject(rrj);
@@ -753,8 +695,7 @@ PBoolean H225_RAS::OnReceiveRegistrationReject(const H225_RegistrationReject & /
 void H225_RAS::OnSendUnregistrationRequest(H323RasPDU & pdu, H225_UnregistrationRequest & urq)
 {
   OnSendUnregistrationRequest(urq);
-  pdu.Prepare(urq.m_tokens, H225_UnregistrationRequest::e_tokens,
-              urq.m_cryptoTokens, H225_UnregistrationRequest::e_cryptoTokens);
+  pdu.Prepare(urq);
 }
 
 
@@ -766,8 +707,7 @@ void H225_RAS::OnSendUnregistrationRequest(H225_UnregistrationRequest & /*urq*/)
 void H225_RAS::OnSendUnregistrationConfirm(H323RasPDU & pdu, H225_UnregistrationConfirm & ucf)
 {
   OnSendUnregistrationConfirm(ucf);
-  pdu.Prepare(ucf.m_tokens, H225_UnregistrationConfirm::e_tokens,
-              ucf.m_cryptoTokens, H225_UnregistrationConfirm::e_cryptoTokens);
+  pdu.Prepare(ucf);
 }
 
 
@@ -779,8 +719,7 @@ void H225_RAS::OnSendUnregistrationConfirm(H225_UnregistrationConfirm & /*ucf*/)
 void H225_RAS::OnSendUnregistrationReject(H323RasPDU & pdu, H225_UnregistrationReject & urj)
 {
   OnSendUnregistrationReject(urj);
-  pdu.Prepare(urj.m_tokens, H225_UnregistrationReject::e_tokens,
-              urj.m_cryptoTokens, H225_UnregistrationReject::e_cryptoTokens);
+  pdu.Prepare(urj);
 }
 
 
@@ -791,13 +730,11 @@ void H225_RAS::OnSendUnregistrationReject(H225_UnregistrationReject & /*urj*/)
 
 PBoolean H225_RAS::OnReceiveUnregistrationRequest(const H323RasPDU & pdu, const H225_UnregistrationRequest & urq)
 {
-  if (!CheckCryptoTokens(pdu,
-                         urq.m_tokens, H225_UnregistrationRequest::e_tokens,
-                         urq.m_cryptoTokens, H225_UnregistrationRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, urq))
     return false;
 
 #if OPAL_H460
-  ProcessFeatureSet<H225_UnregistrationRequest>(this, H460_MessageType::e_unregistrationRequest, urq);
+  ReceiveGenericData(this, H460_MessageType::e_unregistrationRequest, urq);
 #endif
 
   return OnReceiveUnregistrationRequest(urq);
@@ -815,9 +752,7 @@ PBoolean H225_RAS::OnReceiveUnregistrationConfirm(const H323RasPDU & pdu, const
   if (!CheckForResponse(H225_RasMessage::e_unregistrationRequest, ucf.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         ucf.m_tokens, H225_UnregistrationConfirm::e_tokens,
-                         ucf.m_cryptoTokens, H225_UnregistrationConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, ucf))
     return false;
 
   return OnReceiveUnregistrationConfirm(ucf);
@@ -835,9 +770,7 @@ PBoolean H225_RAS::OnReceiveUnregistrationReject(const H323RasPDU & pdu, const H
   if (!CheckForResponse(H225_RasMessage::e_unregistrationRequest, urj.m_requestSeqNum, &urj.m_rejectReason))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         urj.m_tokens, H225_UnregistrationReject::e_tokens,
-                         urj.m_cryptoTokens, H225_UnregistrationReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, urj))
     return false;
 
   return OnReceiveUnregistrationReject(urj);
@@ -855,11 +788,10 @@ void H225_RAS::OnSendAdmissionRequest(H323RasPDU & pdu, H225_AdmissionRequest &
   OnSendAdmissionRequest(arq);
     
 #if OPAL_H460
-  SendFeatureSet<H225_AdmissionRequest>(this, H460_MessageType::e_admissionRequest, arq);
+  SendGenericData(this, H460_MessageType::e_admissionRequest, arq);
 #endif
     
-  pdu.Prepare(arq.m_tokens, H225_AdmissionRequest::e_tokens,
-              arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens);
+  pdu.Prepare(arq);
 }
 
 
@@ -873,11 +805,10 @@ void H225_RAS::OnSendAdmissionConfirm(H323RasPDU & pdu, H225_AdmissionConfirm &
   OnSendAdmissionConfirm(acf);
     
 #if OPAL_H460
-  SendFeatureSet<H225_AdmissionConfirm>(this, H460_MessageType::e_admissionConfirm, acf);
+  SendGenericData(this, H460_MessageType::e_admissionConfirm, acf);
 #endif
     
-  pdu.Prepare(acf.m_tokens, H225_AdmissionConfirm::e_tokens,
-              acf.m_cryptoTokens, H225_AdmissionConfirm::e_cryptoTokens);
+  pdu.Prepare(acf);
 }
 
 
@@ -891,11 +822,10 @@ void H225_RAS::OnSendAdmissionReject(H323RasPDU & pdu, H225_AdmissionReject & ar
   OnSendAdmissionReject(arj);
     
 #if OPAL_H460
-    SendFeatureSet<H225_AdmissionReject>(this, H460_MessageType::e_admissionReject, arj);
+    SendGenericData(this, H460_MessageType::e_admissionReject, arj);
 #endif
     
-  pdu.Prepare(arj.m_tokens, H225_AdmissionReject::e_tokens,
-              arj.m_cryptoTokens, H225_AdmissionReject::e_cryptoTokens);
+  pdu.Prepare(arj);
 }
 
 
@@ -906,13 +836,11 @@ void H225_RAS::OnSendAdmissionReject(H225_AdmissionReject & /*arj*/)
 
 PBoolean H225_RAS::OnReceiveAdmissionRequest(const H323RasPDU & pdu, const H225_AdmissionRequest & arq)
 {
-  if (!CheckCryptoTokens(pdu,
-                         arq.m_tokens, H225_AdmissionRequest::e_tokens,
-                         arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, arq))
     return false;
     
 #if OPAL_H460
-  ReceiveFeatureSet<H225_AdmissionRequest>(this, H460_MessageType::e_admissionRequest, arq);
+  ReceiveGenericData(this, H460_MessageType::e_admissionRequest, arq);
 #endif
 
   return OnReceiveAdmissionRequest(arq);
@@ -930,13 +858,11 @@ PBoolean H225_RAS::OnReceiveAdmissionConfirm(const H323RasPDU & pdu, const H225_
   if (!CheckForResponse(H225_RasMessage::e_admissionRequest, acf.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         acf.m_tokens, H225_AdmissionConfirm::e_tokens,
-                         acf.m_cryptoTokens, H225_AdmissionConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, acf))
     return false;
   
 #if OPAL_H460
-  ReceiveFeatureSet<H225_AdmissionConfirm>(this, H460_MessageType::e_admissionConfirm, acf);
+  ReceiveGenericData(this, H460_MessageType::e_admissionConfirm, acf);
 #endif
 
   return OnReceiveAdmissionConfirm(acf);
@@ -954,13 +880,11 @@ PBoolean H225_RAS::OnReceiveAdmissionReject(const H323RasPDU & pdu, const H225_A
   if (!CheckForResponse(H225_RasMessage::e_admissionRequest, arj.m_requestSeqNum, &arj.m_rejectReason))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         arj.m_tokens, H225_AdmissionReject::e_tokens,
-                         arj.m_cryptoTokens, H225_AdmissionReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, arj))
     return false;
   
 #if OPAL_H460
-  ReceiveFeatureSet<H225_AdmissionReject>(this, H460_MessageType::e_admissionReject, arj);
+  ReceiveGenericData(this, H460_MessageType::e_admissionReject, arj);
 #endif
 
   return OnReceiveAdmissionReject(arj);
@@ -976,8 +900,7 @@ PBoolean H225_RAS::OnReceiveAdmissionReject(const H225_AdmissionReject & /*arj*/
 void H225_RAS::OnSendBandwidthRequest(H323RasPDU & pdu, H225_BandwidthRequest & brq)
 {
   OnSendBandwidthRequest(brq);
-  pdu.Prepare(brq.m_tokens, H225_BandwidthRequest::e_tokens,
-              brq.m_cryptoTokens, H225_BandwidthRequest::e_cryptoTokens);
+  pdu.Prepare(brq);
 }
 
 
@@ -988,9 +911,7 @@ void H225_RAS::OnSendBandwidthRequest(H225_BandwidthRequest & /*brq*/)
 
 PBoolean H225_RAS::OnReceiveBandwidthRequest(const H323RasPDU & pdu, const H225_BandwidthRequest & brq)
 {
-  if (!CheckCryptoTokens(pdu,
-                         brq.m_tokens, H225_BandwidthRequest::e_tokens,
-                         brq.m_cryptoTokens, H225_BandwidthRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, brq))
     return false;
 
   return OnReceiveBandwidthRequest(brq);
@@ -1006,8 +927,7 @@ PBoolean H225_RAS::OnReceiveBandwidthRequest(const H225_BandwidthRequest & /*brq
 void H225_RAS::OnSendBandwidthConfirm(H323RasPDU & pdu, H225_BandwidthConfirm & bcf)
 {
   OnSendBandwidthConfirm(bcf);
-  pdu.Prepare(bcf.m_tokens, H225_BandwidthConfirm::e_tokens,
-              bcf.m_cryptoTokens, H225_BandwidthConfirm::e_cryptoTokens);
+  pdu.Prepare(bcf);
 }
 
 
@@ -1021,9 +941,7 @@ PBoolean H225_RAS::OnReceiveBandwidthConfirm(const H323RasPDU & pdu, const H225_
   if (!CheckForResponse(H225_RasMessage::e_bandwidthRequest, bcf.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         bcf.m_tokens, H225_BandwidthConfirm::e_tokens,
-                         bcf.m_cryptoTokens, H225_BandwidthConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, bcf))
     return false;
 
   return OnReceiveBandwidthConfirm(bcf);
@@ -1039,8 +957,7 @@ PBoolean H225_RAS::OnReceiveBandwidthConfirm(const H225_BandwidthConfirm & /*bcf
 void H225_RAS::OnSendBandwidthReject(H323RasPDU & pdu, H225_BandwidthReject & brj)
 {
   OnSendBandwidthReject(brj);
-  pdu.Prepare(brj.m_tokens, H225_BandwidthReject::e_tokens,
-              brj.m_cryptoTokens, H225_BandwidthReject::e_cryptoTokens);
+  pdu.Prepare(brj);
 }
 
 
@@ -1054,9 +971,7 @@ PBoolean H225_RAS::OnReceiveBandwidthReject(const H323RasPDU & pdu, const H225_B
   if (!CheckForResponse(H225_RasMessage::e_bandwidthRequest, brj.m_requestSeqNum, &brj.m_rejectReason))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         brj.m_tokens, H225_BandwidthReject::e_tokens,
-                         brj.m_cryptoTokens, H225_BandwidthReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, brj))
     return false;
 
   return OnReceiveBandwidthReject(brj);
@@ -1072,11 +987,10 @@ PBoolean H225_RAS::OnReceiveBandwidthReject(const H225_BandwidthReject & /*brj*/
 void H225_RAS::OnSendDisengageRequest(H323RasPDU & pdu, H225_DisengageRequest & drq)
 {
   OnSendDisengageRequest(drq);
-  pdu.Prepare(drq.m_tokens, H225_DisengageRequest::e_tokens,
-              drq.m_cryptoTokens, H225_DisengageRequest::e_cryptoTokens);
+  pdu.Prepare(drq);
 
 #if OPAL_H460
-  SendGenericData<H225_DisengageRequest>(this, H460_MessageType::e_disengagerequest, drq);
+  SendGenericData(this, H460_MessageType::e_disengagerequest, drq);
 #endif
 }
 
@@ -1088,13 +1002,11 @@ void H225_RAS::OnSendDisengageRequest(H225_DisengageRequest & /*drq*/)
 
 PBoolean H225_RAS::OnReceiveDisengageRequest(const H323RasPDU & pdu, const H225_DisengageRequest & drq)
 {
-  if (!CheckCryptoTokens(pdu,
-                         drq.m_tokens, H225_DisengageRequest::e_tokens,
-                         drq.m_cryptoTokens, H225_DisengageRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, drq))
     return false;
 
 #if OPAL_H460
-  ReceiveGenericData<H225_DisengageRequest>(this, H460_MessageType::e_disengagerequest, drq);
+  ReceiveGenericData(this, H460_MessageType::e_disengagerequest, drq);
 #endif
 
   return OnReceiveDisengageRequest(drq);
@@ -1110,11 +1022,10 @@ PBoolean H225_RAS::OnReceiveDisengageRequest(const H225_DisengageRequest & /*drq
 void H225_RAS::OnSendDisengageConfirm(H323RasPDU & pdu, H225_DisengageConfirm & dcf)
 {
   OnSendDisengageConfirm(dcf);
-  pdu.Prepare(dcf.m_tokens, H225_DisengageConfirm::e_tokens,
-              dcf.m_cryptoTokens, H225_DisengageConfirm::e_cryptoTokens);
+  pdu.Prepare(dcf);
 
 #if OPAL_H460
-  SendGenericData<H225_DisengageConfirm>(this, H460_MessageType::e_disengageconfirm, dcf);
+  SendGenericData(this, H460_MessageType::e_disengageconfirm, dcf);
 #endif
 }
 
@@ -1129,13 +1040,11 @@ PBoolean H225_RAS::OnReceiveDisengageConfirm(const H323RasPDU & pdu, const H225_
   if (!CheckForResponse(H225_RasMessage::e_disengageRequest, dcf.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         dcf.m_tokens, H225_DisengageConfirm::e_tokens,
-                         dcf.m_cryptoTokens, H225_DisengageConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, dcf))
     return false;
 
 #if OPAL_H460
-  ReceiveGenericData<H225_DisengageConfirm>(this, H460_MessageType::e_disengageconfirm, dcf);
+  ReceiveGenericData(this, H460_MessageType::e_disengageconfirm, dcf);
 #endif
 
   return OnReceiveDisengageConfirm(dcf);
@@ -1151,8 +1060,7 @@ PBoolean H225_RAS::OnReceiveDisengageConfirm(const H225_DisengageConfirm & /*dcf
 void H225_RAS::OnSendDisengageReject(H323RasPDU & pdu, H225_DisengageReject & drj)
 {
   OnSendDisengageReject(drj);
-  pdu.Prepare(drj.m_tokens, H225_DisengageReject::e_tokens,
-              drj.m_cryptoTokens, H225_DisengageReject::e_cryptoTokens);
+  pdu.Prepare(drj);
 }
 
 
@@ -1166,9 +1074,7 @@ PBoolean H225_RAS::OnReceiveDisengageReject(const H323RasPDU & pdu, const H225_D
   if (!CheckForResponse(H225_RasMessage::e_disengageRequest, drj.m_requestSeqNum, &drj.m_rejectReason))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         drj.m_tokens, H225_DisengageReject::e_tokens,
-                         drj.m_cryptoTokens, H225_DisengageReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, drj))
     return false;
 
   return OnReceiveDisengageReject(drj);
@@ -1186,11 +1092,10 @@ void H225_RAS::OnSendLocationRequest(H323RasPDU & pdu, H225_LocationRequest & lr
   OnSendLocationRequest(lrq);
     
 #if OPAL_H460
-  SendFeatureSet<H225_LocationRequest>(this, H460_MessageType::e_locationRequest, lrq);
+  SendGenericData(this, H460_MessageType::e_locationRequest, lrq);
 #endif
     
-  pdu.Prepare(lrq.m_tokens, H225_LocationRequest::e_tokens,
-              lrq.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens);
+  pdu.Prepare(lrq);
 }
 
 
@@ -1201,13 +1106,11 @@ void H225_RAS::OnSendLocationRequest(H225_LocationRequest & /*lrq*/)
 
 PBoolean H225_RAS::OnReceiveLocationRequest(const H323RasPDU & pdu, const H225_LocationRequest & lrq)
 {
-  if (!CheckCryptoTokens(pdu,
-                         lrq.m_tokens, H225_LocationRequest::e_tokens,
-                         lrq.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, lrq))
     return false;
     
 #if OPAL_H460
-  ReceiveFeatureSet<H225_LocationRequest>(this, H460_MessageType::e_locationRequest, lrq);
+  ReceiveGenericData(this, H460_MessageType::e_locationRequest, lrq);
 #endif
 
   return OnReceiveLocationRequest(lrq);
@@ -1225,11 +1128,10 @@ void H225_RAS::OnSendLocationConfirm(H323RasPDU & pdu, H225_LocationConfirm & lc
   OnSendLocationConfirm(lcf);
     
 #if OPAL_H460
-  SendFeatureSet<H225_LocationConfirm>(this, H460_MessageType::e_locationConfirm, lcf);
+  SendGenericData(this, H460_MessageType::e_locationConfirm, lcf);
 #endif
     
-  pdu.Prepare(lcf.m_tokens, H225_LocationRequest::e_tokens,
-              lcf.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens);
+  pdu.Prepare(lcf);
 }
 
 
@@ -1249,7 +1151,7 @@ PBoolean H225_RAS::OnReceiveLocationConfirm(const H323RasPDU &, const H225_Locat
   }
   
 #if OPAL_H460
-  ReceiveFeatureSet<H225_LocationConfirm>(this, H460_MessageType::e_locationConfirm, lcf);
+  ReceiveGenericData(this, H460_MessageType::e_locationConfirm, lcf);
 #endif
 
   return OnReceiveLocationConfirm(lcf);
@@ -1267,11 +1169,10 @@ void H225_RAS::OnSendLocationReject(H323RasPDU & pdu, H225_LocationReject & lrj)
   OnSendLocationReject(lrj);
     
 #if OPAL_H460
-  SendFeatureSet<H225_LocationReject>(this, H460_MessageType::e_locationReject, lrj);
+  SendGenericData(this, H460_MessageType::e_locationReject, lrj);
 #endif
     
-  pdu.Prepare(lrj.m_tokens, H225_LocationReject::e_tokens,
-              lrj.m_cryptoTokens, H225_LocationReject::e_cryptoTokens);
+  pdu.Prepare(lrj);
 }
 
 
@@ -1285,13 +1186,11 @@ PBoolean H225_RAS::OnReceiveLocationReject(const H323RasPDU & pdu, const H225_Lo
   if (!CheckForResponse(H225_RasMessage::e_locationRequest, lrj.m_requestSeqNum, &lrj.m_rejectReason))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         lrj.m_tokens, H225_LocationReject::e_tokens,
-                         lrj.m_cryptoTokens, H225_LocationReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, lrj))
     return false;
   
 #if OPAL_H460
-  ReceiveFeatureSet<H225_LocationReject>(this, H460_MessageType::e_locationReject, lrj);
+  ReceiveGenericData(this, H460_MessageType::e_locationReject, lrj);
 #endif
 
   return OnReceiveLocationReject(lrj);
@@ -1307,11 +1206,10 @@ PBoolean H225_RAS::OnReceiveLocationReject(const H225_LocationReject & /*lrj*/)
 void H225_RAS::OnSendInfoRequest(H323RasPDU & pdu, H225_InfoRequest & irq)
 {
   OnSendInfoRequest(irq);
-  pdu.Prepare(irq.m_tokens, H225_InfoRequest::e_tokens,
-              irq.m_cryptoTokens, H225_InfoRequest::e_cryptoTokens);
+  pdu.Prepare(irq);
 
 #if OPAL_H460
-  SendGenericData<H225_InfoRequest>(this, H460_MessageType::e_inforequest, irq);
+  SendGenericData(this, H460_MessageType::e_inforequest, irq);
 #endif
 }
 
@@ -1323,13 +1221,11 @@ void H225_RAS::OnSendInfoRequest(H225_InfoRequest & /*irq*/)
 
 PBoolean H225_RAS::OnReceiveInfoRequest(const H323RasPDU & pdu, const H225_InfoRequest & irq)
 {
-  if (!CheckCryptoTokens(pdu,
-                         irq.m_tokens, H225_InfoRequest::e_tokens,
-                         irq.m_cryptoTokens, H225_InfoRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, irq))
     return false;
 
 #if OPAL_H460
-  ReceiveGenericData<H225_InfoRequest>(this, H460_MessageType::e_inforequest, irq);
+  ReceiveGenericData(this, H460_MessageType::e_inforequest, irq);
 #endif
 
   return OnReceiveInfoRequest(irq);
@@ -1345,11 +1241,10 @@ PBoolean H225_RAS::OnReceiveInfoRequest(const H225_InfoRequest & /*irq*/)
 void H225_RAS::OnSendInfoRequestResponse(H323RasPDU & pdu, H225_InfoRequestResponse & irr)
 {
   OnSendInfoRequestResponse(irr);
-  pdu.Prepare(irr.m_tokens, H225_InfoRequestResponse::e_tokens,
-              irr.m_cryptoTokens, H225_InfoRequestResponse::e_cryptoTokens);
+  pdu.Prepare(irr);
 
 #if OPAL_H460
-  SendGenericData<H225_InfoRequestResponse>(this, H460_MessageType::e_inforequestresponse, irr);
+  SendGenericData(this, H460_MessageType::e_inforequestresponse, irr);
 #endif
 }
 
@@ -1364,13 +1259,11 @@ PBoolean H225_RAS::OnReceiveInfoRequestResponse(const H323RasPDU & pdu, const H2
   if (!CheckForResponse(H225_RasMessage::e_infoRequest, irr.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         irr.m_tokens, H225_InfoRequestResponse::e_tokens,
-                         irr.m_cryptoTokens, H225_InfoRequestResponse::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, irr))
     return false;
 
 #if OPAL_H460
-  ReceiveGenericData<H225_InfoRequestResponse>(this, H460_MessageType::e_inforequestresponse, irr);
+  ReceiveGenericData(this, H460_MessageType::e_inforequestresponse, irr);
 #endif
 
   return OnReceiveInfoRequestResponse(irr);
@@ -1388,11 +1281,10 @@ void H225_RAS::OnSendNonStandardMessage(H323RasPDU & pdu, H225_NonStandardMessag
   OnSendNonStandardMessage(nsm);
     
 #if OPAL_H460
-  SendFeatureSet<H225_NonStandardMessage>(this, H460_MessageType::e_nonStandardMessage, nsm);
+  SendGenericData(this, H460_MessageType::e_nonStandardMessage, nsm);
 #endif
     
-  pdu.Prepare(nsm.m_tokens, H225_InfoRequestResponse::e_tokens,
-              nsm.m_cryptoTokens, H225_InfoRequestResponse::e_cryptoTokens);
+  pdu.Prepare(nsm);
 }
 
 
@@ -1403,13 +1295,11 @@ void H225_RAS::OnSendNonStandardMessage(H225_NonStandardMessage & /*nsm*/)
 
 PBoolean H225_RAS::OnReceiveNonStandardMessage(const H323RasPDU & pdu, const H225_NonStandardMessage & nsm)
 {
-  if (!CheckCryptoTokens(pdu,
-                         nsm.m_tokens, H225_NonStandardMessage::e_tokens,
-                         nsm.m_cryptoTokens, H225_NonStandardMessage::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, nsm))
     return false;
     
 #if OPAL_H460
-  ReceiveFeatureSet<H225_NonStandardMessage>(this, H460_MessageType::e_nonStandardMessage, nsm);
+  ReceiveGenericData(this, H460_MessageType::e_nonStandardMessage, nsm);
 #endif
 
   return OnReceiveNonStandardMessage(nsm);
@@ -1425,8 +1315,7 @@ PBoolean H225_RAS::OnReceiveNonStandardMessage(const H225_NonStandardMessage & /
 void H225_RAS::OnSendUnknownMessageResponse(H323RasPDU & pdu, H225_UnknownMessageResponse & umr)
 {
   OnSendUnknownMessageResponse(umr);
-  pdu.Prepare(umr.m_tokens, H225_UnknownMessageResponse::e_tokens,
-              umr.m_cryptoTokens, H225_UnknownMessageResponse::e_cryptoTokens);
+  pdu.Prepare(umr);
 }
 
 
@@ -1437,9 +1326,7 @@ void H225_RAS::OnSendUnknownMessageResponse(H225_UnknownMessageResponse & /*umr*
 
 PBoolean H225_RAS::OnReceiveUnknownMessageResponse(const H323RasPDU & pdu, const H225_UnknownMessageResponse & umr)
 {
-  if (!CheckCryptoTokens(pdu,
-                         umr.m_tokens, H225_UnknownMessageResponse::e_tokens,
-                         umr.m_cryptoTokens, H225_UnknownMessageResponse::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, umr))
     return false;
 
   return OnReceiveUnknownMessageResponse(umr);
@@ -1455,8 +1342,7 @@ PBoolean H225_RAS::OnReceiveUnknownMessageResponse(const H225_UnknownMessageResp
 void H225_RAS::OnSendRequestInProgress(H323RasPDU & pdu, H225_RequestInProgress & rip)
 {
   OnSendRequestInProgress(rip);
-  pdu.Prepare(rip.m_tokens, H225_RequestInProgress::e_tokens,
-              rip.m_cryptoTokens, H225_RequestInProgress::e_cryptoTokens);
+  pdu.Prepare(rip);
 }
 
 
@@ -1468,8 +1354,7 @@ void H225_RAS::OnSendRequestInProgress(H225_RequestInProgress & /*rip*/)
 void H225_RAS::OnSendResourcesAvailableIndicate(H323RasPDU & pdu, H225_ResourcesAvailableIndicate & rai)
 {
   OnSendResourcesAvailableIndicate(rai);
-  pdu.Prepare(rai.m_tokens, H225_ResourcesAvailableIndicate::e_tokens,
-              rai.m_cryptoTokens, H225_ResourcesAvailableIndicate::e_cryptoTokens);
+  pdu.Prepare(rai);
 }
 
 
@@ -1480,9 +1365,7 @@ void H225_RAS::OnSendResourcesAvailableIndicate(H225_ResourcesAvailableIndicate
 
 PBoolean H225_RAS::OnReceiveResourcesAvailableIndicate(const H323RasPDU & pdu, const H225_ResourcesAvailableIndicate & rai)
 {
-  if (!CheckCryptoTokens(pdu,
-                         rai.m_tokens, H225_ResourcesAvailableIndicate::e_tokens,
-                         rai.m_cryptoTokens, H225_ResourcesAvailableIndicate::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, rai))
     return false;
 
   return OnReceiveResourcesAvailableIndicate(rai);
@@ -1498,8 +1381,7 @@ PBoolean H225_RAS::OnReceiveResourcesAvailableIndicate(const H225_ResourcesAvail
 void H225_RAS::OnSendResourcesAvailableConfirm(H323RasPDU & pdu, H225_ResourcesAvailableConfirm & rac)
 {
   OnSendResourcesAvailableConfirm(rac);
-  pdu.Prepare(rac.m_tokens, H225_ResourcesAvailableConfirm::e_tokens,
-              rac.m_cryptoTokens, H225_ResourcesAvailableConfirm::e_cryptoTokens);
+  pdu.Prepare(rac);
 }
 
 
@@ -1513,9 +1395,7 @@ PBoolean H225_RAS::OnReceiveResourcesAvailableConfirm(const H323RasPDU & pdu, co
   if (!CheckForResponse(H225_RasMessage::e_resourcesAvailableIndicate, rac.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         rac.m_tokens, H225_ResourcesAvailableConfirm::e_tokens,
-                         rac.m_cryptoTokens, H225_ResourcesAvailableConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, rac))
     return false;
 
   return OnReceiveResourcesAvailableConfirm(rac);
@@ -1533,11 +1413,10 @@ void H225_RAS::OnSendServiceControlIndication(H323RasPDU & pdu, H225_ServiceCont
   OnSendServiceControlIndication(sci);
     
 #if OPAL_H460
-  SendFeatureSet<H225_ServiceControlIndication>(this, H460_MessageType::e_serviceControlIndication, sci);
+  SendGenericData(this, H460_MessageType::e_serviceControlIndication, sci);
 #endif
     
-  pdu.Prepare(sci.m_tokens, H225_ServiceControlIndication::e_tokens,
-              sci.m_cryptoTokens, H225_ServiceControlIndication::e_cryptoTokens);
+  pdu.Prepare(sci);
 }
 
 
@@ -1548,13 +1427,11 @@ void H225_RAS::OnSendServiceControlIndication(H225_ServiceControlIndication & /*
 
 PBoolean H225_RAS::OnReceiveServiceControlIndication(const H323RasPDU & pdu, const H225_ServiceControlIndication & sci)
 {
-  if (!CheckCryptoTokens(pdu,
-                         sci.m_tokens, H225_ServiceControlIndication::e_tokens,
-                         sci.m_cryptoTokens, H225_ServiceControlIndication::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, sci))
     return false;
     
 #if OPAL_H460
-  ReceiveFeatureSet<H225_ServiceControlIndication>(this, H460_MessageType::e_serviceControlIndication, sci);
+  ReceiveGenericData(this, H460_MessageType::e_serviceControlIndication, sci);
 #endif
 
   return OnReceiveServiceControlIndication(sci);
@@ -1572,11 +1449,10 @@ void H225_RAS::OnSendServiceControlResponse(H323RasPDU & pdu, H225_ServiceContro
   OnSendServiceControlResponse(scr);
     
 #if OPAL_H460
-  SendFeatureSet<H225_ServiceControlResponse>(this, H460_MessageType::e_serviceControlResponse, scr);
+  SendGenericData(this, H460_MessageType::e_serviceControlResponse, scr);
 #endif
     
-  pdu.Prepare(scr.m_tokens, H225_ResourcesAvailableConfirm::e_tokens,
-              scr.m_cryptoTokens, H225_ResourcesAvailableConfirm::e_cryptoTokens);
+  pdu.Prepare(scr);
 }
 
 
@@ -1590,13 +1466,11 @@ PBoolean H225_RAS::OnReceiveServiceControlResponse(const H323RasPDU & pdu, const
   if (!CheckForResponse(H225_RasMessage::e_serviceControlIndication, scr.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         scr.m_tokens, H225_ServiceControlResponse::e_tokens,
-                         scr.m_cryptoTokens, H225_ServiceControlResponse::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, scr))
     return false;
   
 #if OPAL_H460
-  ReceiveFeatureSet<H225_ServiceControlResponse>(this, H460_MessageType::e_serviceControlResponse, scr);
+  ReceiveGenericData(this, H460_MessageType::e_serviceControlResponse, scr);
 #endif
 
   return OnReceiveServiceControlResponse(scr);
@@ -1612,8 +1486,7 @@ PBoolean H225_RAS::OnReceiveServiceControlResponse(const H225_ServiceControlResp
 void H225_RAS::OnSendInfoRequestAck(H323RasPDU & pdu, H225_InfoRequestAck & iack)
 {
   OnSendInfoRequestAck(iack);
-  pdu.Prepare(iack.m_tokens, H225_InfoRequestAck::e_tokens,
-              iack.m_cryptoTokens, H225_InfoRequestAck::e_cryptoTokens);
+  pdu.Prepare(iack);
 }
 
 
@@ -1627,9 +1500,7 @@ PBoolean H225_RAS::OnReceiveInfoRequestAck(const H323RasPDU & pdu, const H225_In
   if (!CheckForResponse(H225_RasMessage::e_infoRequestResponse, iack.m_requestSeqNum))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         iack.m_tokens, H225_InfoRequestAck::e_tokens,
-                         iack.m_cryptoTokens, H225_InfoRequestAck::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, iack))
     return false;
 
   return OnReceiveInfoRequestAck(iack);
@@ -1645,8 +1516,7 @@ PBoolean H225_RAS::OnReceiveInfoRequestAck(const H225_InfoRequestAck & /*iack*/)
 void H225_RAS::OnSendInfoRequestNak(H323RasPDU & pdu, H225_InfoRequestNak & inak)
 {
   OnSendInfoRequestNak(inak);
-  pdu.Prepare(inak.m_tokens, H225_InfoRequestAck::e_tokens,
-              inak.m_cryptoTokens, H225_InfoRequestAck::e_cryptoTokens);
+  pdu.Prepare(inak);
 }
 
 
@@ -1660,9 +1530,7 @@ PBoolean H225_RAS::OnReceiveInfoRequestNak(const H323RasPDU & pdu, const H225_In
   if (!CheckForResponse(H225_RasMessage::e_infoRequestResponse, inak.m_requestSeqNum, &inak.m_nakReason))
     return false;
 
-  if (!CheckCryptoTokens(pdu,
-                         inak.m_tokens, H225_InfoRequestNak::e_tokens,
-                         inak.m_cryptoTokens, H225_InfoRequestNak::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu, inak))
     return false;
 
   return OnReceiveInfoRequestNak(inak);
