@@ -23,14 +23,14 @@
  *
  * Contributor(s): __________________________________
  *
- * $Revision: 28483 $
+ * $Revision: 32977 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2014-10-10 12:17:47 +0100 (Fri, 10 Oct 2014) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -52,7 +52,7 @@
 
 H235Authenticator::H235Authenticator()
 {
-  enabled = true;
+  m_enabled = true;
   sentRandomSequenceNumber = PRandom::Number()&INT_MAX;
   lastRandomSequenceNumber = 0;
   lastTimestamp = 0;
@@ -66,27 +66,24 @@ void H235Authenticator::PrintOn(ostream & strm) const
   PWaitAndSignal m(mutex);
 
   strm << GetName() << '<';
-  if (IsActive())
-    strm << "active";
-  else if (!enabled)
+  if (!IsEnabled())
     strm << "disabled";
   else if (password.IsEmpty())
     strm << "no-pwd";
   else
-    strm << "inactive";
+    strm << "active";
   strm << '>';
 }
 
 
-PBoolean H235Authenticator::PrepareTokens(PASN_Array & clearTokens,
-                                          PASN_Array & cryptoTokens)
+PBoolean H235Authenticator::PrepareTokens(PASN_Array & clearTokens, PASN_Array & cryptoTokens, unsigned rasPDU)
 {
   PWaitAndSignal m(mutex);
 
-  if (!IsActive())
+  if (!IsEnabled() || !IsSecuredPDU(rasPDU, false))
     return false;
 
-  H235_ClearToken * clearToken = CreateClearToken();
+  H235_ClearToken * clearToken = CreateClearToken(rasPDU);
   if (clearToken != NULL) {
     // Check if already have a token of thsi type and overwrite it
     for (PINDEX i = 0; i < clearTokens.GetSize(); i++) {
@@ -104,22 +101,34 @@ PBoolean H235Authenticator::PrepareTokens(PASN_Array & clearTokens,
   }
 
   H225_CryptoH323Token * cryptoToken;
-  if ((cryptoToken = CreateCryptoToken(false)) != NULL)
+  if ((cryptoToken = CreateCryptoToken(false, rasPDU)) != NULL)
     cryptoTokens.Append(cryptoToken);
 
-  if ((cryptoToken = CreateCryptoToken(true)) != NULL)
+  if ((cryptoToken = CreateCryptoToken(true, rasPDU)) != NULL)
     cryptoTokens.Append(cryptoToken);
 
   return true;
 }
 
 
+H235_ClearToken * H235Authenticator::CreateClearToken(unsigned)
+{
+  return CreateClearToken();
+}
+
+
 H235_ClearToken * H235Authenticator::CreateClearToken()
 {
   return NULL;
 }
 
 
+H225_CryptoH323Token * H235Authenticator::CreateCryptoToken(bool digits, unsigned)
+{
+  return CreateCryptoToken(digits);
+}
+
+
 H225_CryptoH323Token * H235Authenticator::CreateCryptoToken(bool)
 {
   return NULL;
@@ -139,7 +148,7 @@ H235Authenticator::ValidationResult H235Authenticator::ValidateTokens(
 {
   PWaitAndSignal m(mutex);
 
-  if (!IsActive())
+  if (!IsEnabled())
     return e_Disabled;
 
   PINDEX i;
@@ -186,37 +195,20 @@ PBoolean H235Authenticator::IsSecuredPDU(unsigned, PBoolean) const
 }
 
 
-PBoolean H235Authenticator::IsActive() const
-{
-  return enabled && !password;
-}
-
-
-PBoolean H235Authenticator::AddCapability(unsigned mechanism,
-                                      const PString & oid,
-                                      H225_ArrayOf_AuthenticationMechanism & mechanisms,
-                                      H225_ArrayOf_PASN_ObjectId & algorithmOIDs)
+PINDEX H235Authenticator::AddCapabilityIfNeeded(unsigned mechanism,
+                                                const PString & oid,
+                                                H225_ArrayOf_AuthenticationMechanism & mechanisms,
+                                                H225_ArrayOf_PASN_ObjectId & algorithmOIDs)
 {
   PWaitAndSignal m(mutex);
 
-  if (!IsActive()) {
-    PTRACE(2, "RAS\tAuthenticator " << *this
-            << " not active during GRQ SetCapability negotiation");
-    return false;
+  if (!IsEnabled()) {
+    PTRACE(3, "RAS\tAuthenticator " << *this << " not enabled during GRQ SetCapability negotiation");
+    return P_MAX_INDEX;
   }
 
   PINDEX i;
-  PINDEX size = mechanisms.GetSize();
-  for (i = 0; i < size; i++) {
-    if (mechanisms[i].GetTag() == mechanism)
-      break;
-  }
-  if (i >= size) {
-    mechanisms.SetSize(size+1);
-    mechanisms[size].SetTag(mechanism);
-  }
-
-  size = algorithmOIDs.GetSize();
+  PINDEX size = algorithmOIDs.GetSize();
   for (i = 0; i < size; i++) {
     if (algorithmOIDs[i] == oid)
       break;
@@ -226,17 +218,25 @@ PBoolean H235Authenticator::AddCapability(unsigned mechanism,
     algorithmOIDs[size] = oid;
   }
 
-  return true;
+  size = mechanisms.GetSize();
+  for (i = 0; i < size; i++) {
+    if (mechanisms[i].GetTag() == mechanism)
+      return i;
+  }
+
+  mechanisms.SetSize(size + 1);
+  mechanisms[size].SetTag(mechanism);
+  return size;
 }
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
-void H235Authenticators::PreparePDU(H323TransactionPDU & pdu,
-                                    PASN_Array & clearTokens,
-                                    unsigned clearOptionalField,
-                                    PASN_Array & cryptoTokens,
-                                    unsigned cryptoOptionalField)
+void H235Authenticators::InternalPreparePDU(H323TransactionPDU & pdu,
+                                            PASN_Array & clearTokens,
+                                            unsigned clearOptionalField,
+                                            PASN_Array & cryptoTokens,
+                                            unsigned cryptoOptionalField)
 {
   // Clean out any crypto tokens in case this is a retry message
   // and we are regenerating the tokens due to possible timestamp
@@ -244,9 +244,9 @@ void H235Authenticators::PreparePDU(H323TransactionPDU & pdu,
   // other endpoints and should be passed through unchanged.
   cryptoTokens.RemoveAll();
 
+  unsigned pduTag = pdu.GetChoice().GetTag();
   for (iterator iterAuth = begin(); iterAuth != end(); ++iterAuth) {
-    if (iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), false) &&
-        iterAuth->PrepareTokens(clearTokens, cryptoTokens)) {
+    if (iterAuth->IsEnabled() && iterAuth->PrepareTokens(clearTokens, cryptoTokens, pduTag)) {
       PTRACE(4, "H235RAS\tPrepared PDU with authenticator " << *iterAuth);
     }
   }
@@ -261,16 +261,18 @@ void H235Authenticators::PreparePDU(H323TransactionPDU & pdu,
 
 
 H235Authenticator::ValidationResult
-       H235Authenticators::ValidatePDU(const H323TransactionPDU & pdu,
-                                       const PASN_Array & clearTokens,
-                                       unsigned clearOptionalField,
-                                       const PASN_Array & cryptoTokens,
-                                       unsigned cryptoOptionalField,
-                                       const PBYTEArray & rawPDU)
+       H235Authenticators::InternalValidatePDU(const H323TransactionPDU & pdu,
+                                               const PASN_Array & clearTokens,
+                                               unsigned clearOptionalField,
+                                               const PASN_Array & cryptoTokens,
+                                               unsigned cryptoOptionalField,
+                                               const PBYTEArray & rawPDU)
 {
+  unsigned pduTag = pdu.GetChoice().GetTag();
+
   PBoolean noneActive = true;
   for (iterator iterAuth = begin(); iterAuth != end(); ++iterAuth) {
-    if (iterAuth->IsActive() && iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), true)) {
+    if (iterAuth->IsEnabled() && iterAuth->IsSecuredPDU(pduTag, true)) {
       noneActive = false;
       break;
     }
@@ -289,7 +291,7 @@ H235Authenticator::ValidationResult
   }
 
   for (iterator iterAuth = begin(); iterAuth != end(); ++iterAuth) {
-    if (iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), true)) {
+    if (iterAuth->IsSecuredPDU(pduTag, true)) {
       H235Authenticator::ValidationResult result = iterAuth->ValidateTokens(clearTokens, cryptoTokens, rawPDU);
       switch (result) {
         case H235Authenticator::e_OK :
@@ -318,10 +320,11 @@ H235Authenticator::ValidationResult
 
 ///////////////////////////////////////////////////////////////////////////////
 
-PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthSimpleMD5, "SimpleMD5", false);
-
+static const char Name_MD5[] = "MD5";
 static const char OID_MD5[] = "1.2.840.113549.2.5";
 
+PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthSimpleMD5, Name_MD5, false);
+
 H235AuthSimpleMD5::H235AuthSimpleMD5()
 {
   usage = AnyApplication;
@@ -336,13 +339,13 @@ PObject * H235AuthSimpleMD5::Clone() const
 
 const char * H235AuthSimpleMD5::GetName() const
 {
-  return "MD5";
+  return Name_MD5;
 }
 
 
 H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken(bool digits)
 {
-  if (!IsActive())
+  if (!IsEnabled())
     return NULL;
 
   if (localId.IsEmpty()) {
@@ -371,7 +374,9 @@ H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken(bool digits)
   }
   else {
     cryptoEPPwdHash.m_alias.SetTag(H225_AliasAddress::e_h323_ID);
-    (PASN_BMPString &)cryptoEPPwdHash.m_alias = localId;
+    /* Avaya ECS Gatekeeper needs a trailing NULL character.
+       Awaiting compatibility errors with other gatekeepers now. */
+    (PASN_BMPString &)cryptoEPPwdHash.m_alias = localId + '\0';
   }
 
   // Use SetValueRaw to make sure trailing NULL is included
@@ -408,7 +413,7 @@ H235Authenticator::ValidationResult H235AuthSimpleMD5::ValidateCryptoToken(
                                              const H225_CryptoH323Token & cryptoToken,
                                              const PBYTEArray &)
 {
-  if (!IsActive())
+  if (!IsEnabled())
     return e_Disabled;
 
   // verify the token is of correct type
@@ -468,12 +473,15 @@ PBoolean H235AuthSimpleMD5::IsCapability(const H235_AuthenticationMechanism & me
 PBoolean H235AuthSimpleMD5::SetCapability(H225_ArrayOf_AuthenticationMechanism & mechanisms,
                                       H225_ArrayOf_PASN_ObjectId & algorithmOIDs)
 {
-  return AddCapability(H235_AuthenticationMechanism::e_pwdHash, OID_MD5, mechanisms, algorithmOIDs);
+  return AddCapabilityIfNeeded(H235_AuthenticationMechanism::e_pwdHash, OID_MD5, mechanisms, algorithmOIDs) != P_MAX_INDEX;
 }
 
 
 PBoolean H235AuthSimpleMD5::IsSecuredPDU(unsigned rasPDU, PBoolean received) const
 {
+  if (password.IsEmpty())
+    return false;
+
   switch (rasPDU) {
     case H225_RasMessage::e_registrationRequest :
     case H225_RasMessage::e_unregistrationRequest :
@@ -491,10 +499,11 @@ PBoolean H235AuthSimpleMD5::IsSecuredPDU(unsigned rasPDU, PBoolean received) con
 
 ///////////////////////////////////////////////////////////////////////////////
 
-PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthCAT, "SimpleCAT", false);
-
+static const char Name_CAT[] = "CAT";
 static const char OID_CAT[] = "1.2.840.113548.10.1.2.1";
 
+PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthCAT, Name_CAT, false);
+
 H235AuthCAT::H235AuthCAT()
 {
   usage = GKAdmission;
@@ -509,15 +518,20 @@ PObject * H235AuthCAT::Clone() const
 
 const char * H235AuthCAT::GetName() const
 {
-  return "CAT";
+  return Name_CAT;
 }
 
 
 H235_ClearToken * H235AuthCAT::CreateClearToken()
 {
-  if (!IsActive())
+  if (!IsEnabled())
     return NULL;
 
+  if (password.IsEmpty()) {
+    PTRACE(4, "H235RAS\tH235AuthCAT requires password.");
+    return NULL;
+  }
+
   if (localId.IsEmpty()) {
     PTRACE(2, "H235RAS\tH235AuthCAT requires local ID for encoding.");
     return NULL;
@@ -554,12 +568,17 @@ H235_ClearToken * H235AuthCAT::CreateClearToken()
 }
 
 
-H235Authenticator::ValidationResult
-        H235AuthCAT::ValidateClearToken(const H235_ClearToken & clearToken)
+H235Authenticator::ValidationResult H235AuthCAT::ValidateClearToken(const H235_ClearToken & clearToken)
 {
-  if (!IsActive())
+  if (!IsEnabled())
     return e_Disabled;
 
+  if (password.IsEmpty()) {
+    PTRACE(4, "H235RAS\tH235AuthCAT requires password.");
+    return e_BadPassword;
+  }
+
+
   if (clearToken.m_tokenOID != OID_CAT)
     return e_Absent;
 
@@ -644,11 +663,11 @@ PBoolean H235AuthCAT::IsCapability(const H235_AuthenticationMechanism & mechanis
 PBoolean H235AuthCAT::SetCapability(H225_ArrayOf_AuthenticationMechanism & mechanisms,
                                 H225_ArrayOf_PASN_ObjectId & algorithmOIDs)
 {
-  if (!AddCapability(H235_AuthenticationMechanism::e_authenticationBES, OID_CAT,
-                     mechanisms, algorithmOIDs))
+  PINDEX idx = AddCapabilityIfNeeded(H235_AuthenticationMechanism::e_authenticationBES, OID_CAT, mechanisms, algorithmOIDs);
+  if (idx == P_MAX_INDEX)
     return false;
 
-  H235_AuthenticationBES & bes = mechanisms[mechanisms.GetSize()-1];
+  H235_AuthenticationBES & bes = mechanisms[idx];
   bes.SetTag(H235_AuthenticationBES::e_radius);
   return true;
 }
@@ -656,6 +675,9 @@ PBoolean H235AuthCAT::SetCapability(H225_ArrayOf_AuthenticationMechanism & mecha
 
 PBoolean H235AuthCAT::IsSecuredPDU(unsigned rasPDU, PBoolean received) const
 {
+  if (password.IsEmpty())
+    return false;
+
   switch (rasPDU) {
     case H225_RasMessage::e_registrationRequest :
     case H225_RasMessage::e_admissionRequest :
