@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29729 $
+ * $Revision: 31954 $
  * $Author: rjongbloed $
- * $Date: 2013-05-16 21:20:35 -0500 (Thu, 16 May 2013) $
+ * $Date: 2014-05-26 00:35:23 +0100 (Mon, 26 May 2014) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -80,16 +80,17 @@ void H245Negotiator::HandleTimeout()
 H245NegMasterSlaveDetermination::H245NegMasterSlaveDetermination(H323EndPoint & end,
                                                                  H323Connection & conn)
   : H245Negotiator(end, conn)
+  , m_state(e_Idle)
+  , m_determinationNumber(0)
+  , m_retryCount(1)
+  , m_status(e_Indeterminate)
 {
-  retryCount = 1;
-  state = e_Idle;
-  status = e_Indeterminate;
 }
 
 
 PBoolean H245NegMasterSlaveDetermination::Start(PBoolean renegotiate)
 {
-  if (state != e_Idle) {
+  if (m_state != e_Idle) {
     PTRACE(3, "H245\tMasterSlaveDetermination already in progress");
     return true;
   }
@@ -97,7 +98,7 @@ PBoolean H245NegMasterSlaveDetermination::Start(PBoolean renegotiate)
   if (!renegotiate && IsDetermined())
     return true;
 
-  retryCount = 1;
+  m_retryCount = 1;
   return Restart();
 }
 
@@ -107,35 +108,35 @@ PBoolean H245NegMasterSlaveDetermination::Restart()
   PTRACE(3, "H245\tSending MasterSlaveDetermination");
 
   // Begin the Master/Slave determination procedure
-  determinationNumber = PRandom::Number()%16777216;
+  m_determinationNumber = PRandom::Number() % 16777216;
   replyTimer = endpoint.GetMasterSlaveDeterminationTimeout();
-  state = e_Outgoing;
+  m_state = e_Outgoing;
 
   H323ControlPDU pdu;
-  pdu.BuildMasterSlaveDetermination(endpoint.GetTerminalType(), determinationNumber);
+  pdu.BuildMasterSlaveDetermination(endpoint.GetTerminalType(), m_determinationNumber);
   return connection.WriteControlPDU(pdu);
 }
 
 
 void H245NegMasterSlaveDetermination::Stop()
 {
-  PTRACE(3, "H245\tStopping MasterSlaveDetermination: state=" << state);
+  PTRACE(3, "H245\tStopping MasterSlaveDetermination: state=" << m_state);
 
-  if (state == e_Idle)
+  if (m_state == e_Idle)
     return;
 
   replyTimer.Stop(false);
-  state = e_Idle;
+  m_state = e_Idle;
 }
 
 
 PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveDetermination & pdu)
 {
-  PTRACE(3, "H245\tReceived MasterSlaveDetermination: state=" << state);
+  PTRACE(3, "H245\tReceived MasterSlaveDetermination: state=" << m_state);
 
-  if (state == e_Incoming) {
+  if (m_state == e_Incoming) {
     replyTimer.Stop(false);
-    state = e_Idle;
+    m_state = e_Idle;
     return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                              "Duplicate MasterSlaveDetermination");
   }
@@ -149,7 +150,7 @@ PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveD
   else if (pdu.m_terminalType > (unsigned)endpoint.GetTerminalType())
     newStatus = e_DeterminedSlave;
   else {
-    DWORD moduloDiff = (pdu.m_statusDeterminationNumber - determinationNumber)&0xffffff;
+    DWORD moduloDiff = (pdu.m_statusDeterminationNumber - m_determinationNumber) & 0xffffff;
     if (moduloDiff == 0 || moduloDiff == 0x800000)
       newStatus = e_Indeterminate;
     else if (moduloDiff < 0x800000)
@@ -164,16 +165,15 @@ PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveD
     PTRACE(3, "H245\tMasterSlaveDetermination: local is "
                   << (newStatus == e_DeterminedMaster ? "master" : "slave"));
     reply.BuildMasterSlaveDeterminationAck(newStatus == e_DeterminedMaster);
-    state = state == e_Outgoing ? e_Incoming : e_Idle;
-    status = newStatus;
+    m_state = m_state == e_Outgoing ? e_Incoming : e_Idle;
+    m_status = newStatus;
   }
-  else if (state == e_Outgoing) {
-    retryCount++;
-    if (retryCount < endpoint.GetMasterSlaveDeterminationRetries())
+  else if (m_state == e_Outgoing) {
+    if (++m_retryCount < endpoint.GetMasterSlaveDeterminationRetries())
       return Restart(); // Try again
 
     replyTimer.Stop(false);
-    state = e_Idle;
+    m_state = e_Idle;
     return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                              "Retries exceeded");
   }
@@ -187,9 +187,9 @@ PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveD
 
 PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterminationAck & pdu)
 {
-  PTRACE(3, "H245\tReceived MasterSlaveDeterminationAck: state=" << state);
+  PTRACE(3, "H245\tReceived MasterSlaveDeterminationAck: state=" << m_state);
 
-  if (state == e_Idle)
+  if (m_state == e_Idle)
     return true;
 
   replyTimer = endpoint.GetMasterSlaveDeterminationTimeout();
@@ -201,8 +201,8 @@ PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterm
     newStatus = e_DeterminedSlave;
 
   H323ControlPDU reply;
-  if (state == e_Outgoing) {
-    status = newStatus;
+  if (m_state == e_Outgoing) {
+    m_status = newStatus;
     PTRACE(3, "H245\tMasterSlaveDetermination: remote is "
                   << (newStatus == e_DeterminedSlave ? "master" : "slave"));
     reply.BuildMasterSlaveDeterminationAck(newStatus == e_DeterminedMaster);
@@ -211,9 +211,9 @@ PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterm
   }
 
   replyTimer.Stop(false);
-  state = e_Idle;
+  m_state = e_Idle;
 
-  if (status != newStatus)
+  if (m_status != newStatus)
     return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                              "Master/Slave mismatch");
 
@@ -223,16 +223,15 @@ PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterm
 
 PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDeterminationReject & pdu)
 {
-  PTRACE(3, "H245\tReceived MasterSlaveDeterminationReject: state=" << state);
+  PTRACE(3, "H245\tReceived MasterSlaveDeterminationReject: state=" << m_state);
 
-  switch (state) {
+  switch (m_state) {
     case e_Idle :
       return true;
 
     case e_Outgoing :
       if (pdu.m_cause.GetTag() == H245_MasterSlaveDeterminationReject_cause::e_identicalNumbers) {
-        retryCount++;
-        if (retryCount < endpoint.GetMasterSlaveDeterminationRetries())
+        if (++m_retryCount < endpoint.GetMasterSlaveDeterminationRetries())
           return Restart();
       }
 
@@ -241,7 +240,7 @@ PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDet
   }
 
   replyTimer.Stop(false);
-  state = e_Idle;
+  m_state = e_Idle;
 
   return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                            "Retries exceeded");
@@ -250,13 +249,13 @@ PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDet
 
 PBoolean H245NegMasterSlaveDetermination::HandleRelease(const H245_MasterSlaveDeterminationRelease & /*pdu*/)
 {
-  PTRACE(3, "H245\tReceived MasterSlaveDeterminationRelease: state=" << state);
+  PTRACE(3, "H245\tReceived MasterSlaveDeterminationRelease: state=" << m_state);
 
-  if (state == e_Idle)
+  if (m_state == e_Idle)
     return true;
 
   replyTimer.Stop(false);
-  state = e_Idle;
+  m_state = e_Idle;
 
   return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                            "Aborted");
@@ -265,44 +264,24 @@ PBoolean H245NegMasterSlaveDetermination::HandleRelease(const H245_MasterSlaveDe
 
 void H245NegMasterSlaveDetermination::HandleTimeout()
 {
-  if (state == e_Idle)
+  if (m_state == e_Idle)
     return;
 
-  PTRACE(3, "H245\tTimeout on MasterSlaveDetermination: state=" << state);
+  PTRACE(3, "H245\tTimeout on MasterSlaveDetermination: state=" << m_state);
 
-  if (state == e_Outgoing) {
+  if (m_state == e_Outgoing) {
     H323ControlPDU reply;
     reply.Build(H245_IndicationMessage::e_masterSlaveDeterminationRelease);
     connection.WriteControlPDU(reply);
   }
 
-  state = e_Idle;
+  m_state = e_Idle;
 
   connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                     "Timeout");
 }
 
 
-#if PTRACING
-const char * H245NegMasterSlaveDetermination::GetStateName(States s)
-{
-  static const char * const names[] = {
-    "Idle", "Outgoing", "Incoming"
-  };
-  return s < PARRAYSIZE(names) ? names[s] : "<Unknown>";
-}
-
-
-const char * H245NegMasterSlaveDetermination::GetStatusName(MasterSlaveStatus s)
-{
-  static const char * const names[] = {
-    "Indeterminate", "DeterminedMaster", "DeterminedSlave"
-  };
-  return s < PARRAYSIZE(names) ? names[s] : "<Unknown>";
-}
-#endif
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 H245NegTerminalCapabilitySet::H245NegTerminalCapabilitySet(H323EndPoint & end,
@@ -462,17 +441,6 @@ void H245NegTerminalCapabilitySet::HandleTimeout()
 }
 
 
-#if PTRACING
-const char * H245NegTerminalCapabilitySet::GetStateName(States s)
-{
-  static const char * const names[] = {
-    "Idle", "InProgress", "Sent"
-  };
-  return s < PARRAYSIZE(names) ? names[s] : "<Unknown>";
-}
-#endif
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 H245NegLogicalChannel::H245NegLogicalChannel(H323EndPoint & end,
@@ -508,44 +476,46 @@ H245NegLogicalChannel::~H245NegLogicalChannel()
 
 PBoolean H245NegLogicalChannel::Open(const H323Capability & capability,
                                  unsigned sessionID,
-                                 unsigned replacementFor)
+                                 unsigned replacementFor,
+                                 OpalMediaStreamPtr mediaStream)
 {
   if (state != e_Released && state != e_AwaitingRelease) {
     PTRACE(2, "H245\tOpen of channel currently in negotiations: " << channelNumber);
     return false;
   }
 
+  state = e_AwaitingEstablishment;
+
   PTRACE(3, "H245\tOpening channel: " << channelNumber);
 
   if (channel != NULL) {
     channel->Close();
     delete channel;
-    channel = NULL;
-  }
-
-  state = e_AwaitingEstablishment;
-
-  H323ControlPDU pdu;
-  H245_OpenLogicalChannel & open = pdu.BuildOpenLogicalChannel(channelNumber);
-
-  if (!capability.OnSendingPDU(open.m_forwardLogicalChannelParameters.m_dataType)) {
-    PTRACE(1, "H245\tOpening channel: " << channelNumber
-           << ", capability.OnSendingPDU() failed");
-    return false;
   }
 
   channel = capability.CreateChannel(connection, H323Channel::IsTransmitter, sessionID, NULL);
   if (channel == NULL) {
-    PTRACE(1, "H245\tOpening channel: " << channelNumber
-           << ", capability.CreateChannel() failed");
+    PTRACE(1, "H245\tOpen channel: " << channelNumber << ", capability.CreateChannel() failed");
     return false;
   }
 
   channel->SetNumber(channelNumber);
+  channel->SetMediaStream(mediaStream);
 
+  if (!channel->PreOpen()) {
+    PTRACE(2, "H245\tOpen channel: " << channelNumber << ", pre-open failed");
+    return false;
+  }
+
+  if (!channel->SetInitialBandwidth()) {
+    PTRACE(2, "H245\tOpen channel: " << channelNumber << ", Insufficient bandwidth");
+    return false;
+  }
+
+  H323ControlPDU pdu;
+  H245_OpenLogicalChannel & open = pdu.BuildOpenLogicalChannel(channelNumber);
   if (!channel->OnSendingPDU(open)) {
-    PTRACE(1, "H245\tOpening channel: " << channelNumber
-           << ", channel->OnSendingPDU() failed");
+    PTRACE(1, "H245\tOpen channel: " << channelNumber << ", channel->OnSendingPDU() failed");
     return false;
   }
 
@@ -560,14 +530,6 @@ PBoolean H245NegLogicalChannel::Open(const H323Capability & capability,
     }
   }
 
-  if (!channel->PreOpen())
-    return false;
-
-  if (!channel->SetInitialBandwidth()) {
-    PTRACE(2, "H245\tOpening channel: " << channelNumber << ", Insufficient bandwidth");
-    return false;
-  }
-
   replyTimer = endpoint.GetLogicalChannelTimeout();
 
   return connection.WriteControlPDU(pdu);
@@ -609,18 +571,26 @@ PBoolean H245NegLogicalChannel::Close()
 
 PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
 {
-  PTRACE(3, "H245\tReceived open channel: " << channelNumber << ", state=" << state);
+  H323ControlPDU reply;
+  H245_OpenLogicalChannelAck & ack = reply.BuildOpenLogicalChannelAck(channelNumber);
 
   if (channel != NULL) {
+    if (state == e_Established) {
+      PTRACE(3, "H245\tReceived duplicate open channel: " << channelNumber << ", state=" << state);
+      // Really should check that the codec has not changed, later maybe
+      channel->OnSendOpenAck(pdu, ack);
+      return connection.WriteControlPDU(reply);
+    }
+
+    PTRACE(3, "H245\tClosing channel due to received open channel: " << channelNumber << ", state=" << state);
     channel->Close();
     delete channel;
     channel = NULL;
   }
 
-  state = e_Establishing;
+  PTRACE(3, "H245\tReceived open channel: " << channelNumber << ", state=" << state);
 
-  H323ControlPDU reply;
-  H245_OpenLogicalChannelAck & ack = reply.BuildOpenLogicalChannelAck(channelNumber);
+  state = e_Establishing;
 
   PBoolean ok = false;
 
@@ -653,10 +623,13 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
 
   if (ok) {
     OpalMediaStreamPtr mediaStream = channel->GetMediaStream();
-    if (mediaStream != NULL)
+    if (mediaStream != NULL && connection.OnOpenMediaStream(*mediaStream))
       mediaStream->Start();
+    else
+      ok = false;
   }
-  else {
+
+  if (!ok) {
     reply.BuildOpenLogicalChannelReject(channelNumber, cause);
     Release();
   }
@@ -695,8 +668,12 @@ PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck &
       // somthing other than an asymmetric codec conflict, so close it down.
       if (channel->Open()) {
         OpalMediaStreamPtr mediaStream = channel->GetMediaStream();
-        if (mediaStream != NULL)
-          mediaStream->Start();
+        if (mediaStream != NULL) {
+          if (connection.OnOpenMediaStream(*mediaStream))
+            mediaStream->Start();
+          else
+            return Close();
+        }
       }
       else
         return Close();
@@ -738,7 +715,8 @@ PBoolean H245NegLogicalChannel::HandleOpenConfirm(const H245_OpenLogicalChannelC
 
 PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject & pdu)
 {
-  PTRACE(3, "H245\tReceived open channel reject: " << channelNumber << ", state=" << state);
+  PTRACE(3, "H245\tReceived open channel reject: " << channelNumber
+         << ", cause=" << pdu.m_cause.GetTagName() << ", state=" << state);
 
   switch (state) {
     case e_Released :
@@ -750,8 +728,10 @@ PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject
                                                "Reject established channel");
     case e_AwaitingEstablishment :
       // Master rejected our attempt to open, so try something else.
-      if (pdu.m_cause.GetTag() == H245_OpenLogicalChannelReject_cause::e_masterSlaveConflict)
-        connection.OnConflictingLogicalChannel(*channel);
+      if (pdu.m_cause.GetTag() == H245_OpenLogicalChannelReject_cause::e_masterSlaveConflict) {
+        if (!connection.OnConflictingLogicalChannel(*channel))
+          return false;
+      }
       // Do next case
 
     case e_AwaitingRelease :
@@ -769,15 +749,8 @@ PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject
 PBoolean H245NegLogicalChannel::HandleClose(const H245_CloseLogicalChannel & /*pdu*/)
 {
   PTRACE(3, "H245\tReceived close channel: " << channelNumber << ", state=" << state);
-
-  //if (pdu.m_source.GetTag() == H245_CloseLogicalChannel_source::e_user)
-
-  H323ControlPDU reply;
-  reply.BuildCloseLogicalChannelAck(channelNumber);
-
   Release();
-
-  return connection.WriteControlPDU(reply);
+  return true;
 }
 
 
@@ -919,22 +892,6 @@ H323Channel * H245NegLogicalChannel::GetChannel() const
 }
 
 
-#if PTRACING
-const char * H245NegLogicalChannel::GetStateName(States s)
-{
-  static const char * const names[] = {
-    "Released",
-    "AwaitingEstablishment",
-    "Established",
-    "AwaitingRelease",
-    "AwatingConfirmation",
-    "AwaitingResponse"
-  };
-  return s < PARRAYSIZE(names) ? names[s] : "<Unknown>";
-}
-#endif
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 H245NegLogicalChannels::H245NegLogicalChannels(H323EndPoint & end,
@@ -953,14 +910,15 @@ void H245NegLogicalChannels::Add(H323Channel & channel)
 
 PBoolean H245NegLogicalChannels::Open(const H323Capability & capability,
                                   unsigned sessionID,
-                                  unsigned replacementFor)
+                                  unsigned replacementFor,
+                                  OpalMediaStreamPtr mediaStream)
 {
   lastChannelNumber++;
 
   H245NegLogicalChannel * negChan = new H245NegLogicalChannel(endpoint, connection, lastChannelNumber);
   channels.SetAt(lastChannelNumber, negChan);
 
-  return negChan->Open(capability, sessionID, replacementFor);
+  return negChan->Open(capability, sessionID, replacementFor, mediaStream);
 }
 
 
@@ -1027,11 +985,18 @@ PBoolean H245NegLogicalChannels::HandleReject(const H245_OpenLogicalChannelRejec
 PBoolean H245NegLogicalChannels::HandleClose(const H245_CloseLogicalChannel & pdu)
 {
   H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, true);
-  if (chan != NULL)
-    return chan->HandleClose(pdu);
+  if (chan != NULL) {
+    if (!chan->HandleClose(pdu))
+      return false;
+  }
+  else {
+    if (!connection.OnControlProtocolError(H323Connection::e_LogicalChannel, "Close unknown"))
+      return false;
+  }
 
-  return connection.OnControlProtocolError(H323Connection::e_LogicalChannel,
-                                           "Close unknown");
+  H323ControlPDU reply;
+  reply.BuildCloseLogicalChannelAck(pdu.m_forwardLogicalChannelNumber);
+  return connection.WriteControlPDU(reply);
 }
 
 
@@ -1052,8 +1017,12 @@ PBoolean H245NegLogicalChannels::HandleRequestClose(const H245_RequestChannelClo
   if (chan != NULL)
     return chan->HandleRequestClose(pdu);
 
-  return connection.OnControlProtocolError(H323Connection::e_LogicalChannel,
-                                           "Request Close unknown");
+  if (!connection.OnControlProtocolError(H323Connection::e_LogicalChannel, "Request Close unknown"))
+    return false;
+
+  H323ControlPDU reply;
+  reply.BuildRequestChannelCloseReject(pdu.m_forwardLogicalChannelNumber);
+  return connection.WriteControlPDU(reply);
 }
 
 
@@ -1125,8 +1094,8 @@ H323Channel * H245NegLogicalChannels::FindChannelBySession(unsigned rtpSessionId
     H245NegLogicalChannel & logChan = it->second;
     if (logChan.IsAwaitingEstablishment() || logChan.IsEstablished()) {
       H323Channel * channel = logChan.GetChannel();
-      if (channel != NULL && channel->GetSessionID() == rtpSessionId &&
-                             channel->GetDirection() == desiredDirection)
+      if (channel != NULL && (rtpSessionId == 0 || channel->GetSessionID() == rtpSessionId) &&
+                                                   channel->GetDirection() == desiredDirection)
         return channel;
     }
   }
