@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29736 $
+ * $Revision: 33102 $
  * $Author: rjongbloed $
- * $Date: 2013-05-17 03:23:30 -0500 (Fri, 17 May 2013) $
+ * $Date: 2014-11-12 01:02:56 +0000 (Wed, 12 Nov 2014) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -47,10 +47,8 @@
 #include <h323/transaddr.h>
 #include <h323/h225ras.h>
 #include <h323/h235auth.h>
-
-#if OPAL_H460
 #include <h460/h460.h>
-#endif
+
 
 #define new PNEW
 
@@ -77,7 +75,7 @@ void H323TraceDumpPDU(const char * proto,
   if (!PTrace::CanTrace(3))
     return;
 
-  ostream & trace = PTrace::Begin(PTrace::CanTrace(4) ? 4 : 3, __FILE__, __LINE__);
+  ostream & trace = PTRACE_BEGIN(PTrace::CanTrace(4) ? 4 : 3);
   trace << proto << '\t' << (writing ? "Send" : "Receiv") << "ing PDU:";
 
   if (PTrace::CanTrace(4)) {
@@ -554,14 +552,22 @@ unsigned H323GetGenericParameterInteger(const H245_ArrayOf_GenericParameter & pa
 }
 
 
-H245_ParameterValue * H323AddGenericParameter(H245_ArrayOf_GenericParameter & params, unsigned ordinal)
+bool H323GetGenericParameterObject(const H245_ArrayOf_GenericParameter & params, unsigned ordinal, PASN_Object & object)
+{
+  const H245_ParameterValue * param = H323GetGenericParameter(params, ordinal);
+  return param != NULL && param->GetTag() == H245_ParameterValue::e_octetString && ((const PASN_OctetString *)param)->DecodeSubType(object);
+}
+
+
+H245_ParameterValue & H323AddGenericParameter(H245_ArrayOf_GenericParameter & params, unsigned ordinal, H245_ParameterValue::Choices tag)
 {
   PINDEX size = params.GetSize();
   params.SetSize(size+1);
   H245_GenericParameter & param = params[size];
   param.m_parameterIdentifier.SetTag(H245_ParameterIdentifier::e_standard);
   (PASN_Integer &)param.m_parameterIdentifier = ordinal;
-  return &param.m_parameterValue;
+  param.m_parameterValue.SetTag(tag);
+  return param.m_parameterValue;
 }
 
 
@@ -569,7 +575,7 @@ void H323AddGenericParameterBoolean(H245_ArrayOf_GenericParameter & params, unsi
 {
   // Do not include a logical at all if it is false
   if (value)
-    H323AddGenericParameter(params, ordinal)->SetTag(H245_ParameterValue::e_logical);
+    H323AddGenericParameter(params, ordinal, H245_ParameterValue::e_logical);
 }
 
 
@@ -578,89 +584,50 @@ void H323AddGenericParameterInteger(H245_ArrayOf_GenericParameter & params,
                                     unsigned value,
                                     H245_ParameterValue::Choices subType)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
-  param->SetTag(subType);
-  (PASN_Integer &)*param = value;
+  H323AddGenericParameterAs<PASN_Integer>(params, ordinal, subType) = value;
 }
 
 
 void H323AddGenericParameterString(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PString & value)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
-  param->SetTag(H245_ParameterValue::e_octetString);
-  (PASN_OctetString &)*param = value;
+  H323AddGenericParameterAs<PASN_OctetString>(params, ordinal, H245_ParameterValue::e_octetString) = value;
 }
 
 
 void H323AddGenericParameterOctets(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PBYTEArray & value)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
-  param->SetTag(H245_ParameterValue::e_octetString);
-  (PASN_OctetString &)*param = value;
+  H323AddGenericParameterAs<PASN_OctetString>(params, ordinal, H245_ParameterValue::e_octetString) = value;
+}
+
+
+void H323AddGenericParameterObject(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PASN_Object & object)
+{
+  H323AddGenericParameterAs<PASN_OctetString>(params, ordinal, H245_ParameterValue::e_octetString).EncodeSubType(object);
 }
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
 #if OPAL_H460
-static void SendSetupFeatureSet(const H323Connection * connection, H225_Setup_UUIE & pdu)
-{
-  H225_FeatureSet fs;
-  
-  if(!connection->OnSendFeatureSet(H460_MessageType::e_setup, fs)) {
-    return;
-  }
-  
-  if(fs.HasOptionalField(H225_FeatureSet::e_neededFeatures)) {
-    pdu.IncludeOptionalField(H225_Setup_UUIE::e_neededFeatures);
-    H225_ArrayOf_FeatureDescriptor & fsn = pdu.m_neededFeatures;
-    fsn = fs.m_neededFeatures;
-  }
-  
-  if(fs.HasOptionalField(H225_FeatureSet::e_desiredFeatures)) {
-    pdu.IncludeOptionalField(H225_Setup_UUIE::e_desiredFeatures);
-    H225_ArrayOf_FeatureDescriptor & fsn = pdu.m_desiredFeatures;
-    fsn = fs.m_desiredFeatures;
-  }
-  
-  if(fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
-    pdu.IncludeOptionalField(H225_Setup_UUIE::e_supportedFeatures);
-    H225_ArrayOf_FeatureDescriptor & fsn = pdu.m_supportedFeatures;
-    fsn = fs.m_supportedFeatures;
-  }
-}
-
-
 template <typename PDUType>
-static void SendFeatureSet(const H323Connection * connection, unsigned code, H225_H323_UU_PDU & msg, PDUType & pdu)
+static void SendFeatureSet(const H323Connection & connection, H460_MessageType pduType, H225_H323_UU_PDU & msg, PDUType & pdu)
 {
   H225_FeatureSet fs;
-  if (!connection->OnSendFeatureSet(code,fs))
+  if (!connection.OnSendFeatureSet(pduType, fs))
     return;
 
-  if (code == H460_MessageType::e_callProceeding ||
-      code == H460_MessageType::e_alerting ||
-      code == H460_MessageType::e_connect) {
+  if (pduType == H460_MessageType::e_callProceeding ||
+      pduType == H460_MessageType::e_alerting ||
+      pduType == H460_MessageType::e_connect) {
     pdu.IncludeOptionalField(PDUType::e_featureSet);
     pdu.m_featureSet = fs;
     return;
   }
 
-  if (!fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures))
-    return;
-
-  msg.IncludeOptionalField(H225_H323_UU_PDU::e_genericData);
-
-  H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
-  H225_ArrayOf_GenericData & data = msg.m_genericData;
-
-  for (PINDEX i=0; i < fsn.GetSize(); i++) {
-    PINDEX lastPos = data.GetSize();
-    data.SetSize(lastPos+1);
-    data[lastPos] = fsn[i];
-  }
+  if (H460_FeatureSet::Copy(msg.m_genericData, fs))
+    msg.IncludeOptionalField(H225_H323_UU_PDU::e_genericData);
 }
-#endif
+#endif // OPAL_H460
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -729,22 +696,22 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
 
   endpoint.SetEndpointTypeInfo(setup.m_sourceInfo);
   
-#if OPAL_H460
-  SendSetupFeatureSet(&connection, setup);
+#if OPAL_PTLIB_SSL
+  if (connection.GetSignallingChannel()->GetProtoPrefix() == OpalTransportAddress::TlsPrefix()) {
+    setup.IncludeOptionalField(H225_Setup_UUIE::e_h245SecurityCapability);
+    setup.m_h245SecurityCapability.SetSize(1);
+    setup.m_h245SecurityCapability[0].SetTag(H225_H245Security::e_tls);
+    H225_SecurityCapabilities & secCap = setup.m_h245SecurityCapability[0];
+    secCap.m_encryption.SetTag(H225_SecurityServiceMode::e_default);
+    secCap.m_authenticaton.SetTag(H225_SecurityServiceMode::e_default);
+    secCap.m_integrity.SetTag(H225_SecurityServiceMode::e_default);
+  }
 #endif
 
   return setup;
 }
 
 
-#if OPAL_H460
-void H323SignalPDU::InsertH460Setup(const H323Connection & connection, H225_Setup_UUIE & setup)
-{
-   SendSetupFeatureSet(&connection, setup);
-}
-#endif
-
-
 H225_CallProceeding_UUIE &
         H323SignalPDU::BuildCallProceeding(const H323Connection & connection)
 {
@@ -763,7 +730,12 @@ H225_CallProceeding_UUIE &
   connection.GetEndPoint().SetEndpointTypeInfo(proceeding.m_destinationInfo);
   
 #if OPAL_H460
-  SendFeatureSet<H225_CallProceeding_UUIE>(&connection, H460_MessageType::e_callProceeding, m_h323_uu_pdu, proceeding);
+  SendFeatureSet(connection, H460_MessageType::e_callProceeding, m_h323_uu_pdu, proceeding);
+#endif
+
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(proceeding.m_tokens))
+    proceeding.IncludeOptionalField(H225_CallProceeding_UUIE::e_tokens);
 #endif
 
   return proceeding;
@@ -788,25 +760,13 @@ H225_Connect_UUIE & H323SignalPDU::BuildConnect(const H323Connection & connectio
   connection.GetEndPoint().SetEndpointTypeInfo(connect.m_destinationInfo);
   
 #if OPAL_H460
-  SendFeatureSet<H225_Connect_UUIE>(&connection, H460_MessageType::e_connect, m_h323_uu_pdu, connect);
+  SendFeatureSet(connection, H460_MessageType::e_connect, m_h323_uu_pdu, connect);
 #endif
 
-  return connect;
-}
-
-
-H225_Connect_UUIE & H323SignalPDU::BuildConnect(const H323Connection & connection,
-                                                const PIPSocket::Address & h245Address,
-                                                WORD port)
-{
-  H225_Connect_UUIE & connect = BuildConnect(connection);
-
-  // indicate we are including the optional H245 address in the PDU
-  connect.IncludeOptionalField(H225_Connect_UUIE::e_h245Address);
-
-  // convert IP address into the correct H245 type
-  H323TransportAddress transAddr(h245Address, port);
-  transAddr.SetPDU(connect.m_h245Address);
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(connect.m_tokens))
+    connect.IncludeOptionalField(H225_Connect_UUIE::e_tokens);
+#endif
 
   return connect;
 }
@@ -829,7 +789,12 @@ H225_Alerting_UUIE & H323SignalPDU::BuildAlerting(const H323Connection & connect
   connection.GetEndPoint().SetEndpointTypeInfo(alerting.m_destinationInfo);
   
 #if OPAL_H460
-  SendFeatureSet<H225_Alerting_UUIE>(&connection, H460_MessageType::e_alerting, m_h323_uu_pdu, alerting);
+  SendFeatureSet(connection, H460_MessageType::e_alerting, m_h323_uu_pdu, alerting);
+#endif
+
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(alerting.m_tokens))
+    alerting.IncludeOptionalField(H225_Alerting_UUIE::e_tokens);
 #endif
 
   return alerting;
@@ -1000,7 +965,7 @@ H225_ReleaseComplete_UUIE &
     release.IncludeOptionalField(H225_ReleaseComplete_UUIE::e_reason);
 
 #if OPAL_H460
-  SendFeatureSet<H225_ReleaseComplete_UUIE>(&connection, H460_MessageType::e_releaseComplete, m_h323_uu_pdu, release);
+  SendFeatureSet(connection, H460_MessageType::e_releaseComplete, m_h323_uu_pdu, release);
 #endif
 
   return release;
@@ -1024,13 +989,35 @@ H225_Facility_UUIE * H323SignalPDU::BuildFacility(const H323Connection & connect
   
 #if OPAL_H460
   if (reason == H225_FacilityReason::e_featureSetUpdate)
-    SendFeatureSet<H225_Facility_UUIE>(&connection, H460_MessageType::e_facility, m_h323_uu_pdu, fac);
+    SendFeatureSet(connection, H460_MessageType::e_facility, m_h323_uu_pdu, fac);
+#endif
+
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(fac.m_tokens))
+    fac.IncludeOptionalField(H225_Facility_UUIE::e_tokens);
 #endif
 
   return &fac;
 }
 
 
+H225_Facility_UUIE & H323SignalPDU::BuildFacility(const OpalGloballyUniqueID & callId)
+{
+  // Build facility msg
+  q931pdu.BuildFacility(0, false);
+
+  // Build the UUIE
+  m_h323_uu_pdu.m_h323_message_body.SetTag(H225_H323_UU_PDU_h323_message_body::e_facility);
+  H225_Facility_UUIE & fac = m_h323_uu_pdu.m_h323_message_body;
+
+  fac.m_protocolIdentifier.SetValue("0.0.8.2250.0." + PString(H225_PROTOCOL_VERSION));
+  fac.m_reason.SetTag(H225_FacilityReason::e_undefinedReason);
+  fac.IncludeOptionalField(H225_Facility_UUIE::e_callIdentifier);
+  fac.m_callIdentifier.m_guid = callId;
+  return fac;
+}
+
+
 H225_Progress_UUIE & H323SignalPDU::BuildProgress(const H323Connection & connection)
 {
   q931pdu.BuildProgress(connection.GetCallReference(), connection.HadAnsweredCall(), Q931::ProgressInbandInformationAvailable);
@@ -1043,6 +1030,11 @@ H225_Progress_UUIE & H323SignalPDU::BuildProgress(const H323Connection & connect
   progress.m_callIdentifier.m_guid = connection.GetCallIdentifier();
   connection.GetEndPoint().SetEndpointTypeInfo(progress.m_destinationInfo);
 
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(progress.m_tokens))
+    progress.IncludeOptionalField(H225_Progress_UUIE::e_tokens);
+#endif
+
   return progress;
 }
 
@@ -1057,6 +1049,11 @@ H225_Status_UUIE & H323SignalPDU::BuildStatus(const H323Connection & connection)
   SetH225Version(connection, status.m_protocolIdentifier);
   status.m_callIdentifier.m_guid = connection.GetCallIdentifier();
 
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(status.m_tokens))
+    status.IncludeOptionalField(H225_Status_UUIE::e_tokens);
+#endif
+
   return status;
 }
 
@@ -1070,6 +1067,11 @@ H225_StatusInquiry_UUIE & H323SignalPDU::BuildStatusInquiry(const H323Connection
   SetH225Version(connection, inquiry.m_protocolIdentifier);
   inquiry.m_callIdentifier.m_guid = connection.GetCallIdentifier();
 
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(inquiry.m_tokens))
+    inquiry.IncludeOptionalField(H225_StatusInquiry_UUIE::e_tokens);
+#endif
+
   return inquiry;
 }
 
@@ -1098,6 +1100,11 @@ H225_Notify_UUIE & H323SignalPDU::BuildNotify(const H323Connection & connection)
   SetH225Version(connection, notify.m_protocolIdentifier);
   notify.m_callIdentifier.m_guid = connection.GetCallIdentifier();
 
+#if OPAL_H235_6
+  if (connection.GetDiffieHellman().ToTokens(notify.m_tokens))
+    notify.IncludeOptionalField(H225_Notify_UUIE::e_tokens);
+#endif
+
   return notify;
 }
 
@@ -2310,6 +2317,39 @@ H225_ServiceControlResponse & H323RasPDU::BuildServiceControlResponse(unsigned s
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+
+OpalBandwidth::OpalBandwidth(const H225_BandWidth & bw)
+  : m_bps(bw.GetValue()*100)
+{
+}
+
+
+OpalBandwidth & OpalBandwidth::operator=(const H225_BandWidth & bw)
+{
+  m_bps = bw.GetValue()*100;
+  return *this;
+}
+
+
+void OpalBandwidth::SetH225(H225_BandWidth & bw) const
+{
+  bw.SetValue((m_bps+99)/100);
+}
+
+
+void OpalBandwidth::SetH245(PASN_Integer & bitRate) const
+{
+  bitRate.SetValue((m_bps+99)/100);
+}
+
+
+void OpalBandwidth::FromH245(const PASN_Integer & bitRate)
+{
+  m_bps = bitRate.GetValue()*100;
+}
+
+
 #endif // OPAL_H323
 
 /////////////////////////////////////////////////////////////////////////////
