@@ -23,14 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29865 $
+ * $Revision: 32592 $
  * $Author: rjongbloed $
- * $Date: 2013-06-02 21:15:04 -0500 (Sun, 02 Jun 2013) $
+ * $Date: 2014-08-21 07:49:04 +0100 (Thu, 21 Aug 2014) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -40,6 +40,7 @@
 #include <h323/h323rtp.h>
 
 #include <opal/manager.h>
+#include <codec/silencedetect.h>
 #include <h323/h323ep.h>
 #include <h323/h323con.h>
 #include <h323/h323pdu.h>
@@ -58,7 +59,7 @@ H323_RTPChannel::H323_RTPChannel(H323Connection & conn,
                                  Directions direction,
                                  OpalMediaSession & session)
   : H323_RealTimeChannel(conn, cap, direction)
-  , m_session(session)
+  , m_session(&session)
 {
   PTRACE(3, "H323RTP\t" << (receiver ? "Receiver" : "Transmitter")
          << " created using session " << GetSessionID());
@@ -72,7 +73,7 @@ H323_RTPChannel::~H323_RTPChannel()
 
 unsigned H323_RTPChannel::GetSessionID() const
 {
-  return m_session.GetSessionID();
+  return m_session->GetSessionID();
 }
 
 
@@ -89,9 +90,8 @@ bool H323_RTPChannel::SetSessionID(unsigned sessionID)
 PBoolean H323_RTPChannel::GetMediaTransportAddress(OpalTransportAddress & data,
                                                    OpalTransportAddress & control) const
 {
-  const OpalRTPSession & rtpSession = dynamic_cast<const OpalRTPSession &>(m_session);
-  data = rtpSession.GetRemoteAddress(true);
-  control = rtpSession.GetRemoteAddress(false);
+  data = m_session->GetRemoteAddress(true);
+  control = m_session->GetRemoteAddress(false);
   return true;
 }
 
@@ -104,24 +104,29 @@ PBoolean H323_RTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & para
   param.m_mediaGuaranteedDelivery = false;
 
   // unicast must have mediaControlChannel
-  H323TransportAddress mediaControlAddress(m_session.GetLocalAddress(false));
-  param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel);
-  mediaControlAddress.SetPDU(param.m_mediaControlChannel);
+  H323TransportAddress mediaControlAddress(m_session->GetLocalAddress(false));
+  if (mediaControlAddress.SetPDU(param.m_mediaControlChannel))
+    param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel);
 
-  if (GetDirection() == H323Channel::IsReceiver) {
-    // set mediaChannel
-    H323TransportAddress mediaAddress(m_session.GetLocalAddress(true));
-    param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
-    mediaAddress.SetPDU(param.m_mediaChannel);
-  }
-
-  if (GetDirection() != H323Channel::IsReceiver) {
+  if (GetDirection() == H323Channel::IsTransmitter) {
     // Set flag for we are going to stop sending audio on silence
-    if (m_mediaFormat.GetMediaType() == OpalMediaType::Audio()) {
-      param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_silenceSuppression);
-      param.m_silenceSuppression = (connection.GetEndPoint().GetManager().GetSilenceDetectParams().m_mode != OpalSilenceDetector::NoSilenceDetection);
+    if (m_session->GetMediaType() == OpalMediaType::Audio()) {
+      PSafePtr<OpalConnection> otherConnection = connection.GetOtherPartyConnection();
+      if (otherConnection != NULL) {
+        OpalSilenceDetector * silenceDetector = otherConnection->GetSilenceDetector();
+        if (silenceDetector != NULL && silenceDetector->GetStatus(NULL, NULL) != OpalSilenceDetector::NoSilenceDetection) {
+          param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_silenceSuppression);
+          param.m_silenceSuppression = true;
+        }
+      }
     }
   }
+  else {
+    // Set mediaChannel on receiver
+    H323TransportAddress mediaAddress(m_session->GetLocalAddress(true));
+    if (mediaAddress.SetPDU(param.m_mediaChannel))
+      param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
+  }
 
   // Set dynamic payload type, if is one
   RTP_DataFrame::PayloadTypes rtpPayloadType = GetDynamicRTPPayloadType();
@@ -132,7 +137,7 @@ PBoolean H323_RTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & para
 
   // Set the media packetization field if have an option to describe it.
   param.m_mediaPacketization.SetTag(H245_H2250LogicalChannelParameters_mediaPacketization::e_rtpPayloadType);
-  if (H323SetRTPPacketization(param.m_mediaPacketization, m_mediaFormat, rtpPayloadType))
+  if (H323SetRTPPacketization(param.m_mediaPacketization, GetMediaFormat(), rtpPayloadType))
     param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaPacketization);
 
   return H323_RealTimeChannel::OnSendingPDU(param);
@@ -141,7 +146,7 @@ PBoolean H323_RTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & para
 
 PBoolean H323_RTPChannel::OnSendingAltPDU(H245_ArrayOf_GenericInformation & alternate) const
 {
-  PTRACE ( 4, "H323RTP\tOnSendingAltPDU");
+  PTRACE(4, "H323RTP\tOnSendingAltPDU");
 #if OPAL_H460
   return connection.OnSendingOLCGenericInformation(GetSessionID(), alternate, false);
 #else
@@ -155,14 +160,14 @@ void H323_RTPChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & para
   PTRACE(3, "RTP\tOnSendingAckPDU");
 
   // set mediaControlChannel
-  H323TransportAddress mediaControlAddress(m_session.GetLocalAddress(false));
-  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel);
-  mediaControlAddress.SetPDU(param.m_mediaControlChannel);
+  H323TransportAddress mediaControlAddress(m_session->GetLocalAddress(false));
+  if (mediaControlAddress.SetPDU(param.m_mediaControlChannel))
+    param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel);
 
   // set mediaChannel
-  H323TransportAddress mediaAddress(m_session.GetLocalAddress(true));
-  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
-  mediaAddress.SetPDU(param.m_mediaChannel);
+  H323TransportAddress mediaAddress(m_session->GetLocalAddress(true));
+  if (mediaAddress.SetPDU(param.m_mediaChannel))
+    param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
 
   // Set dynamic payload type, if is one
   int rtpPayloadType = GetDynamicRTPPayloadType();
@@ -178,37 +183,48 @@ void H323_RTPChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & para
 PBoolean H323_RTPChannel::OnReceivedPDU(const H245_H2250LogicalChannelParameters & param,
                                     unsigned & errorCode)
 {
-  bool ok = false;
+  bool explicitMedia = false;
+  bool explicitControl = false;
 
   if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel)) {
-    if (!ExtractTransport(param.m_mediaControlChannel, false, errorCode)) {
-      PTRACE(1, "RTP_UDP\tFailed to extract mediaControl transport for " << *this);
-      return false;
-    }
-    ok = true;
+    explicitControl = ExtractTransport(param.m_mediaControlChannel, false, errorCode);
+    PTRACE_IF(1, !explicitControl, "RTP_UDP\tFailed to extract mediaControl transport for " << *this);
   }
 
   if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaChannel)) {
-    if (ok && GetDirection() == H323Channel::IsReceiver)
+    if (GetDirection() == H323Channel::IsReceiver)
       PTRACE(2, "RTP_UDP\tIgnoring media transport for " << *this);
-    else if (!ExtractTransport(param.m_mediaChannel, true, errorCode)) {
-      PTRACE(1, "RTP_UDP\tFailed to extract media transport for " << *this);
-      return false;
+    else {
+      explicitMedia = ExtractTransport(param.m_mediaChannel, true, errorCode);
+      PTRACE_IF(1, !explicitMedia, "RTP_UDP\tFailed to extract media transport for " << *this);
     }
-    ok = true;
   }
 
-  if (!ok) {
-    PTRACE(1, "RTP_UDP\tNo mediaChannel or mediaControlChannel specified for " << *this);
+  if (!explicitMedia && !explicitControl) {
+    PTRACE(2, "RTP_UDP\tNo mediaChannel or mediaControlChannel specified for " << *this);
 
-    if (GetSessionID() != H323Capability::DefaultDataSessionID) {
+    if (GetSessionID() < H323Capability::DefaultDataSessionID) {
+      // Audio/video session must have it
       errorCode = H245_OpenLogicalChannelReject_cause::e_unspecified;
       return false;
     }
   }
 
+  if (!explicitMedia && explicitControl) {
+    OpalTransportAddress controlAddress = m_session->GetRemoteAddress(false);
+    PIPSocket::AddressAndPort ap;
+    if (controlAddress.GetIpAndPort(ap)) {
+      ap.SetPort(ap.GetPort() & 0xfffe);
+      OpalTransportAddress mediaAddress(ap, controlAddress.GetProto());
+      m_session->SetRemoteAddress(mediaAddress);
+      PTRACE(3, "RTP_UDP\tNo mediaChannel, using " << mediaAddress << " for " << *this);
+    }
+  }
+
+  OpalMediaFormat mediaFormat = GetMediaFormat();
+
   // Override default payload type if indicated by remote
-  RTP_DataFrame::PayloadTypes rtpPayloadType = m_mediaFormat.GetPayloadType();
+  RTP_DataFrame::PayloadTypes rtpPayloadType = mediaFormat.GetPayloadType();
   if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_dynamicRTPPayloadType))
     rtpPayloadType = (RTP_DataFrame::PayloadTypes)param.m_dynamicRTPPayloadType.GetValue();
 
@@ -219,13 +235,12 @@ PBoolean H323_RTPChannel::OnReceivedPDU(const H245_H2250LogicalChannelParameters
     mediaPacketization = H323GetRTPPacketization(param.m_mediaPacketization);
 
   // Hack for H.263 compatibility, default to RFC2190 if not told otherwise
-  if (mediaPacketization.IsEmpty() && m_mediaFormat == OPAL_H263)
+  if (mediaPacketization.IsEmpty() && mediaFormat == OPAL_H263)
     mediaPacketization = "RFC2190";
 
-  m_mediaFormat.SetPayloadType(rtpPayloadType);
-  m_mediaFormat.SetMediaPacketizations(mediaPacketization);
-  if (m_mediaStream != NULL)
-    m_mediaStream->UpdateMediaFormat(m_mediaFormat);
+  mediaFormat.SetPayloadType(rtpPayloadType);
+  mediaFormat.SetMediaPacketizations(mediaPacketization);
+  UpdateMediaFormat(mediaFormat);
 
   return H323_RealTimeChannel::OnReceivedPDU(param, errorCode);
 }
@@ -258,16 +273,6 @@ PBoolean H323_RTPChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAckPara
 }
 
 
-PBoolean H323_RTPChannel::OnReceivedAckAltPDU(const H245_ArrayOf_GenericInformation & alternate)
-{ 
-#if OPAL_H460
-  return connection.OnReceiveOLCGenericInformation(GetSessionID(), alternate);
-#else
-  return false;
-#endif
-}
-
-
 bool H323_RTPChannel::ExtractTransport(const H245_TransportAddress & pdu,
                                     bool isDataPort,
                                     unsigned & errorCode)
@@ -278,12 +283,12 @@ bool H323_RTPChannel::ExtractTransport(const H245_TransportAddress & pdu,
     return false;
   }
 
-  if (!isDataPort && !m_session.GetRemoteAddress().IsEmpty())
+  if (!isDataPort && !m_session->GetRemoteAddress().IsEmpty())
     return true; // Ignore
 
   H323TransportAddress transAddr = pdu;
 
-  if (m_session.SetRemoteAddress(transAddr, isDataPort))
+  if (m_session->SetRemoteAddress(transAddr, isDataPort))
     return true;
 
   PTRACE(1, "RTP_UDP\tIllegal IP address/port in transport address.");
@@ -293,22 +298,22 @@ bool H323_RTPChannel::ExtractTransport(const H245_TransportAddress & pdu,
 
 void H323_RTPChannel::OnSendRasInfo(H225_RTPSession & info)
 {
-  info.m_sessionId = m_session.GetSessionID();
+  info.m_sessionId = m_session->GetSessionID();
 
-  OpalRTPSession * rtpSession = dynamic_cast<OpalRTPSession *>(&m_session);
+  OpalRTPSession * rtpSession = dynamic_cast<OpalRTPSession *>(m_session);
   if (rtpSession != NULL) {
     info.m_ssrc = rtpSession->GetSyncSourceOut();
     info.m_cname = rtpSession->GetCanonicalName();
   }
 
-  H323TransportAddress lda(m_session.GetLocalAddress(true));
+  H323TransportAddress lda(m_session->GetLocalAddress(true));
   lda.SetPDU(info.m_rtpAddress.m_recvAddress);
-  H323TransportAddress rda(m_session.GetRemoteAddress(true));
+  H323TransportAddress rda(m_session->GetRemoteAddress(true));
   rda.SetPDU(info.m_rtpAddress.m_sendAddress);
 
-  H323TransportAddress lca(m_session.GetLocalAddress(false));
+  H323TransportAddress lca(m_session->GetLocalAddress(false));
   lca.SetPDU(info.m_rtcpAddress.m_recvAddress);
-  H323TransportAddress rca(m_session.GetRemoteAddress(false));
+  H323TransportAddress rca(m_session->GetRemoteAddress(false));
   rca.SetPDU(info.m_rtcpAddress.m_sendAddress);
 }
 
