@@ -23,14 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29415 $
+ * $Revision: 33712 $
  * $Author: rjongbloed $
- * $Date: 2013-04-02 16:41:13 -0500 (Tue, 02 Apr 2013) $
+ * $Date: 2015-07-05 22:45:46 +0100 (Sun, 05 Jul 2015) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -145,6 +145,7 @@ H323Transactor::H323Transactor(H323EndPoint & ep,
     transport = trans;
   else
     transport = new H323TransportUDP(ep, PIPSocket::GetDefaultIpAny(), local_port);
+  PTRACE_CONTEXT_ID_TO(transport);
 
   Construct();
 }
@@ -164,6 +165,7 @@ H323Transactor::H323Transactor(H323EndPoint & ep,
     PIPSocket::Address addr;
     PAssert(iface.GetIpAndPort(addr, local_port), "Cannot parse address");
     transport = new H323TransportUDP(ep, addr, local_port);
+    PTRACE_CONTEXT_ID_TO(transport);
   }
 
   Construct();
@@ -172,7 +174,7 @@ H323Transactor::H323Transactor(H323EndPoint & ep,
 
 void H323Transactor::Construct()
 {
-  nextSequenceNumber = PRandom::Number()%65536;
+  m_nextSequenceNumber = PRandom::Number()%65536;
   checkResponseCryptoTokens = true;
   lastRequest = NULL;
 
@@ -191,7 +193,7 @@ void H323Transactor::PrintOn(ostream & strm) const
   if (transport == NULL)
     strm << "<<no-transport>>";
   else
-    strm << transport->GetRemoteAddress();
+    strm << transport->GetRemoteAddress().GetHostName(true);
 }
 
 
@@ -217,17 +219,9 @@ PBoolean H323Transactor::SetTransport(const H323TransportAddress & iface)
   }
 
   transport = new H323TransportUDP(endpoint, addr, port);
+  PTRACE_CONTEXT_ID_TO(transport);
   transport->SetPromiscuous(H323Transport::AcceptFromAny);
-  return StartChannel();;
-}
-
-
-H323TransportAddressArray H323Transactor::GetInterfaceAddresses(bool excludeLocalHost)
-{
-  if (transport == NULL)
-    return H323TransportAddressArray();
-  else
-    return endpoint.GetInterfaceAddresses(excludeLocalHost, transport);
+  return StartChannel();
 }
 
 
@@ -267,6 +261,8 @@ void H323Transactor::HandleTransactions(PThread &, P_INT_PTR)
     PTRACE(5, "Trans\tReading PDU");
     H323TransactionPDU * response = CreateTransactionPDU();
     if (response->Read(*transport)) {
+      if (transport->GetInterface().IsEmpty())
+        transport->SetInterface(transport->GetLastReceivedInterface());
       consecutiveErrors = 0;
       lastRequest = NULL;
       if (HandleTransaction(response->GetPDU()))
@@ -312,21 +308,18 @@ PBoolean H323Transactor::SetUpCallSignalAddresses(H225_ArrayOf_TransportAddress
   if (PAssertNULL(transport) == NULL)
     return false;
 
-  H323SetTransportAddresses(*transport,
-                            endpoint.GetInterfaceAddresses(false, transport),
-                            addresses);
-
+  H323SetTransportAddresses(*transport, endpoint.GetInterfaceAddresses(transport), addresses);
   return addresses.GetSize() > 0;
 }
 
 
 unsigned H323Transactor::GetNextSequenceNumber()
 {
-  PWaitAndSignal mutex(nextSequenceNumberMutex);
-  nextSequenceNumber++;
-  if (nextSequenceNumber >= 65536)
-    nextSequenceNumber = 1;
-  return nextSequenceNumber;
+  unsigned sn;
+  do {
+    sn = ++m_nextSequenceNumber;
+  }  while (sn == 0);
+  return sn;
 }
 
 
@@ -478,11 +471,7 @@ PBoolean H323Transactor::HandleRequestInProgress(const H323TransactionPDU & pdu,
 }
 
 
-PBoolean H323Transactor::CheckCryptoTokens(const H323TransactionPDU & pdu,
-                                       const PASN_Array & clearTokens,
-                                       unsigned clearOptionalField,
-                                       const PASN_Array & cryptoTokens,
-                                       unsigned cryptoOptionalField)
+bool H323Transactor::CheckCryptoTokens1(const H323TransactionPDU & pdu)
 {
   // If cypto token checking disabled, just return true.
   if (!GetCheckResponseCryptoTokens())
@@ -494,44 +483,41 @@ PBoolean H323Transactor::CheckCryptoTokens(const H323TransactionPDU & pdu,
            << setfill(',') << pdu.GetAuthenticators() << setfill(' '));
   }
 
-  if (pdu.Validate(clearTokens, clearOptionalField,
-                   cryptoTokens, cryptoOptionalField) == H235Authenticator::e_OK)
-    return true;
+  // Need futher checking
+  return false;
+}
+
+
+bool H323Transactor::CheckCryptoTokens2()
+{
+  if (lastRequest == NULL)
+    return false; // or true?
 
   /* Note that a crypto tokens error is flagged to the requestor in the
      responseResult field but the other thread is NOT signalled. This is so
      it can wait for the full timeout for any other packets that might have
      the correct tokens, preventing a possible DOS attack.
    */
-  if (lastRequest != NULL) {
-    lastRequest->responseResult = Request::BadCryptoTokens;
-    lastRequest->responseHandled.Signal();
-    lastRequest->responseMutex.Signal();
-    lastRequest = NULL;
-  }
-
+  lastRequest->responseResult = Request::BadCryptoTokens;
+  lastRequest->responseHandled.Signal();
+  lastRequest->responseMutex.Signal();
+  lastRequest = NULL;
   return false;
 }
 
 
 /////////////////////////////////////////////////////////////////////////////
 
-H323Transactor::Request::Request(unsigned seqNum, H323TransactionPDU & pdu)
-  : requestPDU(pdu)
-{
-  sequenceNumber = seqNum;
-  responseInfo   = NULL;
-}
-
-
 H323Transactor::Request::Request(unsigned seqNum,
                                  H323TransactionPDU & pdu,
                                  const H323TransportAddressArray & addresses)
-  : requestAddresses(addresses),
-    requestPDU(pdu)
+  : sequenceNumber(seqNum)
+  , requestPDU(pdu)
+  , requestAddresses(addresses)
+  , rejectReason(0)
+  , responseInfo(NULL)
+  , responseResult(NoResponseReceived)
 {
-  sequenceNumber = seqNum;
-  responseInfo   = NULL;
 }
 
 
@@ -571,17 +557,17 @@ PBoolean H323Transactor::Request::Poll(H323Transactor & rasChannel, unsigned num
         case ConfirmReceived :
           return true;
 
-        case RejectReceived :
-        case TryAlternate :
-          return false;
-
         case BadCryptoTokens :
           PTRACE(1, "Trans\tResponse to seqnum=" << requestPDU.GetSequenceNumber()
                  << " had invalid crypto tokens.");
           return false;
 
-        default : // RequestInProgress
+        case RequestInProgress :
           responseResult = AwaitingResponse; // Keep waiting
+          break;
+
+        default :
+          return false;
       }
 
       PTRACE_IF(3, responseResult == AwaitingResponse,
@@ -615,33 +601,6 @@ void H323Transactor::Request::CheckResponse(unsigned reqTag, const PASN_Choice *
          << " rejected: " << reason->GetTagName());
   responseResult = RejectReceived;
   rejectReason = reason->GetTag();
-
-  switch(reqTag) {
-    case H225_RasMessage::e_admissionRequest:
-      if (rejectReason == H225_AdmissionRejectReason::e_callerNotRegistered)
-        responseResult = TryAlternate;
-      break;
-
-    case H225_RasMessage::e_gatekeeperRequest:
-      if (rejectReason == H225_GatekeeperRejectReason::e_resourceUnavailable)
-        responseResult = TryAlternate;
-      break;
-
-    case H225_RasMessage::e_disengageRequest:
-      if (rejectReason == H225_DisengageRejectReason::e_notRegistered)
-        responseResult = TryAlternate;
-      break;
-
-    case H225_RasMessage::e_registrationRequest:
-      if (rejectReason == H225_RegistrationRejectReason::e_resourceUnavailable)
-        responseResult = TryAlternate;
-      break;
-
-    case H225_RasMessage::e_infoRequestResponse:
-      if (rejectReason == H225_InfoRequestNakReason::e_notRegistered)
-        responseResult = TryAlternate;
-      break;
-  }
 }
 
 
@@ -739,8 +698,10 @@ PBoolean H323Transaction::HandlePDU()
       return false;
 
     case Confirm :
-      if (confirm != NULL)
+      if (confirm != NULL) {
+        PrepareConfirm();
         WritePDU(*confirm);
+      }
       return false;
 
     case Reject :
@@ -788,17 +749,16 @@ PBoolean H323Transaction::WritePDU(H323TransactionPDU & pdu)
 }
 
 
-PBoolean H323Transaction::CheckCryptoTokens(const H235Authenticators & auth)
+bool H323Transaction::CheckCryptoTokens()
 {
-  authenticators = auth;
-
   request->SetAuthenticators(authenticators);
 
   authenticatorResult = ValidatePDU();
-
   if (authenticatorResult == H235Authenticator::e_OK)
     return true;
 
+  SetRejectReason(GetSecurityRejectTag());
+
   PINDEX i;
   for (i = 0; (authenticatorStrings[i].code >= 0) && (authenticatorStrings[i].code != authenticatorResult); ++i) 
     ;
@@ -843,7 +803,7 @@ PBoolean H323TransactionServer::AddListeners(const H323TransportAddressArray & i
       }
     }
     if (remove) {
-      PTRACE(3, "Trans\tRemoving listener " << *iterListener);
+      PTRACE(3, "Trans\tRemoving existing listener " << *iterListener);
       listeners.erase(iterListener++);
     }
     else
@@ -878,14 +838,14 @@ PBoolean H323TransactionServer::AddListener(const H323TransportAddress & interfa
     return AddListener(interfaceName.CreateTransport(ownerEndPoint));
 
   if (!addr.IsAny())
-    return AddListener(new H323TransportUDP(ownerEndPoint, addr, port));
+    return AddListener(new H323TransportUDP(ownerEndPoint, addr, port, false, true));
 
   PIPSocket::InterfaceTable interfaces;
   if (!PIPSocket::GetInterfaceTable(interfaces)) {
     PTRACE(1, "Trans\tNo interfaces on system!");
     if (!PIPSocket::GetHostAddress(addr))
       return false;
-    return AddListener(new H323TransportUDP(ownerEndPoint, addr, port));
+    return AddListener(new H323TransportUDP(ownerEndPoint, addr, port, false, true));
   }
 
   PTRACE(4, "Trans\tAdding interfaces:\n" << setfill('\n') << interfaces << setfill(' '));
@@ -894,7 +854,7 @@ PBoolean H323TransactionServer::AddListener(const H323TransportAddress & interfa
 
   for (i = 0; i < interfaces.GetSize(); i++) {
     addr = interfaces[i].GetAddress();
-    if (addr != 0) {
+    if (addr.IsValid()) {
       if (AddListener(new H323TransportUDP(ownerEndPoint, addr, port, false, true)))
         atLeastOne = true;
     }
@@ -906,10 +866,15 @@ PBoolean H323TransactionServer::AddListener(const H323TransportAddress & interfa
 
 PBoolean H323TransactionServer::AddListener(H323Transport * transport)
 {
-  if (transport == NULL)
+  if (transport == NULL) {
+    PTRACE(2, "Trans\tTried to listen on NULL transport");
     return false;
+  }
+
+  PTRACE_CONTEXT_ID_TO(transport);
 
   if (!transport->IsOpen()) {
+    PTRACE(3, "Trans\tTransport is not open");
     delete transport;
     return false;
   }
@@ -923,7 +888,7 @@ PBoolean H323TransactionServer::AddListener(H323Transactor * listener)
   if (listener == NULL)
     return false;
 
-  PTRACE(3, "Trans\tStarted listener " << *listener);
+  PTRACE(3, "Trans\tStarted listener \"" << *listener << '"');
 
   mutex.Wait();
   listeners.Append(listener);
