@@ -23,14 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28937 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 29536 $
+ * $Author: rjongbloed $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 #if OPAL_H450
 
