@@ -23,14 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28937 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 33607 $
+ * $Author: rjongbloed $
+ * $Date: 2015-05-18 05:05:31 +0100 (Mon, 18 May 2015) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_H501
 
@@ -75,8 +75,7 @@ H323TransactionPDU * H501Transaction::CreateRIP(unsigned sequenceNumber,
 
 H235Authenticator::ValidationResult H501Transaction::ValidatePDU() const
 {
-  return request->Validate(requestCommon.m_tokens, H501_MessageCommonInfo::e_tokens,
-                           requestCommon.m_cryptoTokens, H501_MessageCommonInfo::e_cryptoTokens);
+  return request->Validate(requestCommon);
 }
 
 
@@ -180,15 +179,13 @@ H323Transaction::Response H501AccessRequest::OnHandlePDU()
 ////////////////////////////////////////////////////////////////
 
 H323PeerElement::H323PeerElement(H323EndPoint & ep, H323Transport * trans)
-  : H323_AnnexG(ep, trans),
-    requestMutex(1, 1)
+  : H323_AnnexG(ep, trans)
 {
   Construct();
 }
 
 H323PeerElement::H323PeerElement(H323EndPoint & ep, const H323TransportAddress & addr)
-  : H323_AnnexG(ep, addr),
-    requestMutex(1, 1)
+  : H323_AnnexG(ep, addr)
 {
   Construct();
 }
@@ -482,7 +479,7 @@ H323PeerElement::Error H323PeerElement::ServiceRequestByAddr(const H323Transport
 
   // build the service request
   H501PDU pdu;
-  H323TransportAddressArray interfaces = GetInterfaceAddresses();
+  H323TransportAddressArray interfaces = endpoint.GetInterfaceAddresses(transport);
   H501_ServiceRequest & body = pdu.BuildServiceRequest(GetNextSequenceNumber(), interfaces);
 
   // include the element indentifier
@@ -1113,7 +1110,7 @@ H323PeerElement::Error H323PeerElement::SendUpdateDescriptor(H501PDU & pdu,
   H501_DescriptorUpdate & body = pdu.m_body;
 
   // put in sender address
-  H323TransportAddressArray addrs = GetInterfaceAddresses();
+  H323TransportAddressArray addrs = endpoint.GetInterfaceAddresses(transport);
   PAssert(addrs.GetSize() > 0, "No interface addresses");
   H323SetAliasAddress(addrs[0], body.m_sender, H225_AliasAddress::e_transportID);
 
@@ -1458,10 +1455,8 @@ PBoolean H323PeerElement::OnReceiveAccessRejection(const H501PDU & pdu, const H5
 
 PBoolean H323PeerElement::MakeRequest(Request & request)
 {
-  requestMutex.Wait();
-  PBoolean stat = H323_AnnexG::MakeRequest(request);
-  requestMutex.Signal();
-  return stat;
+  PWaitAndSignal muyex(m_requestMutex);
+  return H323_AnnexG::MakeRequest(request);
 }
 
 //////////////////////////////////////////////////////////////////////////////
