@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28483 $
+ * $Revision: 32829 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2014-09-26 05:06:48 +0100 (Fri, 26 Sep 2014) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #if OPAL_H323
 
 #ifdef __GNUC__
@@ -368,7 +368,7 @@ PBoolean Q931::Encode(PBYTEArray & data) const
   if (fromDestination)
     data[2] |= 0x80;
   data[3] = (BYTE)callReference;
-  PAssert(messageType < 256, PInvalidParameter);
+  PAssert((PINDEX)messageType < 256, PInvalidParameter);
   data[4] = (BYTE)messageType;
 
   // The following assures disciminators are in ascending value order
@@ -520,13 +520,13 @@ PString Q931::GetMessageTypeName() const
 }
 
 
+static atomic<unsigned> s_lastCallReference;
+
 unsigned Q931::GenerateCallReference()
 {
-  static PAtomicInteger lastCallReference;
-
-  unsigned ref = ++lastCallReference & 0x7fff;
+  unsigned ref = ++s_lastCallReference & 0x7fff;
   if (ref == 0)
-    ref = ++lastCallReference & 0x7fff;
+    ref = ++s_lastCallReference & 0x7fff;
 
   return ref;
 }
@@ -595,6 +595,14 @@ void Q931::SetBearerCapabilities(const PString & str)
         capability = TransferSpeech;
       else if (caps[0] *= "digital")
         capability = TransferUnrestrictedDigital;
+      else if (caps[0] *= "audio")
+        capability = Transfer3_1kHzAudio;
+      else if (caps[0] *= "video")
+        capability = TransferVideo;
+      else if (caps[0] *= "digital-tones")
+        capability = TransferUnrestrictedDigitalWithTones;
+      else if (caps[0] *= "restricted")
+        capability = TransferRestrictedDigital;
       else
         capability = (Q931::InformationTransferCapability)caps[0].AsUnsigned();
     case 0 :
@@ -661,7 +669,7 @@ void Q931::SetBearerCapabilities(InformationTransferCapability capability,
 PBoolean Q931::GetBearerCapabilities(InformationTransferCapability & capability,
                                  unsigned & transferRate,
                                  unsigned * codingStandard,
-                                 unsigned * userInfoLayer1)
+                                 unsigned * userInfoLayer1) const
 {
   if (!HasIE(BearerCapabilityIE))
     return false;
@@ -1007,6 +1015,15 @@ PBoolean Q931::GetCallingPartyNumber(PString  & number,
 }
 
 
+PString Q931::GetCallingPartyNumber() const
+{
+  PString number;
+  if (GetCallingPartyNumber(number))
+    return number;
+  return PString::Empty();
+}
+
+
 void Q931::SetCalledPartyNumber(const PString & number, unsigned plan, unsigned type)
 {
   SetIE(CalledPartyNumberIE,
@@ -1021,6 +1038,15 @@ PBoolean Q931::GetCalledPartyNumber(PString & number, unsigned * plan, unsigned
 }
 
 
+PString Q931::GetCalledPartyNumber() const
+{
+  PString number;
+  if (GetCalledPartyNumber(number))
+    return number;
+  return PString::Empty();
+}
+
+
 void Q931::SetRedirectingNumber(const PString & number,
                                 unsigned plan,
                                 unsigned type,
