@@ -33,14 +33,14 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 29536 $
+ * $Author: rjongbloed $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <h323/h323pdu.h>
 #include <h460/h460p.h>
