@@ -24,15 +24,15 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 28937 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 33270 $
+ * $Author: rjongbloed $
+ * $Date: 2015-01-22 05:15:09 +0000 (Thu, 22 Jan 2015) $
  */
 
 
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
@@ -72,6 +72,10 @@
 
 IAX2CallProcessor::IAX2CallProcessor(IAX2EndPoint &ep)
   : IAX2Processor(ep)
+  , audioFramesSent(0)
+  , audioFramesRcvd(0)
+  , videoFramesSent(0)
+  , videoFramesRcvd(0)
 {
   callStatus = 0;
   
@@ -186,7 +190,7 @@ void IAX2CallProcessor::RemoteNodeHasAnswered()
   
   SetCallAnswered();
   PTRACE(3, "Processor\tRemote node has answered");
-  con->OnConnectedInternal();
+  con->InternalOnConnected();
 }
 
 void IAX2CallProcessor::Hangup(PString dieMessage)
