@@ -25,13 +25,13 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 29059 $
+ * $Revision: 34003 $
  * $Author: rjongbloed $
- * $Date: 2013-01-30 21:12:49 -0600 (Wed, 30 Jan 2013) $
+ * $Date: 2015-09-11 19:39:03 +0100 (Fri, 11 Sep 2015) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
@@ -66,7 +66,7 @@ IAX2Connection::IAX2Connection(OpalCall & call,               /* Owner call for
   : OpalConnection(call, ep, token)
   , endpoint(ep)
   , iax2Processor(*new IAX2CallProcessor(ep))
-  , jitterBuffer(OpalJitterBuffer::Init(400, 2000))
+  , m_jitterBuffer(OpalJitterBuffer::Create(OpalMediaType::Audio(), OpalJitterBuffer::Init(ep.GetManager(), 8)))
 {  
   opalPayloadType = RTP_DataFrame::IllegalPayloadType;
 
@@ -101,6 +101,7 @@ IAX2Connection::~IAX2Connection()
   PTRACE(3, "connection has terminated");
 
   delete & iax2Processor;
+  delete m_jitterBuffer;
 }
 
 void IAX2Connection::StartOperation()
@@ -175,8 +176,7 @@ PBoolean IAX2Connection::SetConnected()
     if (otherParty != NULL)
       ownerCall.OpenSourceMediaStreams(*otherParty, OpalMediaType::Audio(), 1);
 
-    jitterBuffer.SetDelay(OpalJitterBuffer::Init(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
-                                                 endpoint.GetManager().GetMaxAudioJitterDelay() * 8));
+    m_jitterBuffer->SetDelay(OpalJitterBuffer::Init(endpoint.GetManager(), 8));
     PTRACE(5, "Iax2Con\t Start jitter buffer");
   }  
   return OpalConnection::SetConnected();
@@ -214,9 +214,7 @@ void IAX2Connection::OnConnected()
     if (otherParty != NULL)
       ownerCall.OpenSourceMediaStreams(*otherParty, OpalMediaType::Audio(), 1);
 
-
-    jitterBuffer.SetDelay(OpalJitterBuffer::Init(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
-			                         endpoint.GetManager().GetMaxAudioJitterDelay() * 8));
+    m_jitterBuffer->SetDelay(OpalJitterBuffer::Init(endpoint.GetManager(), 8));
     PTRACE(5, "Iax2Con\t Start jitter buffer");
   }
 
@@ -381,22 +379,16 @@ unsigned int IAX2Connection::ChooseCodec()
   return 0;
 }
 
-PBoolean IAX2Connection::IsOnHold(bool fromRemote)
+PBoolean IAX2Connection::IsOnHold(bool fromRemote) const
 {
   return fromRemote ? remote_hold : local_hold;
 }
 
-bool IAX2Connection::Hold(bool fromRemote, bool placeOnHold)
+bool IAX2Connection::HoldRemote(bool placeOnHold)
 {
-  if (IsOnHold(fromRemote))
-    return true;
-
-  if (fromRemote)
-    return false;
-
   local_hold = placeOnHold;
   PauseMediaStreams(placeOnHold);
-  OnHold(fromRemote, placeOnHold);
+  OnHold(false, placeOnHold);
 
   if (placeOnHold)
     iax2Processor.SendHold();
@@ -470,9 +462,8 @@ void IAX2Connection::ReceivedSoundPacketFromNetwork(IAX2Frame *soundFrame)
     mediaFrame.SetMarker(false);
     mediaFrame.SetPayloadType(opalPayloadType);
 
-    mediaFrame.SetSize(mediaFrame.GetPayloadSize() + mediaFrame.GetHeaderSize());
-    memcpy(mediaFrame.GetPayloadPtr(), soundFrame->GetMediaDataPointer(), soundFrame->GetMediaDataSize());
-    jitterBuffer.WriteData(mediaFrame);
+    mediaFrame.SetPayload(soundFrame->GetMediaDataPointer(), soundFrame->GetMediaDataSize());
+    m_jitterBuffer->WriteData(mediaFrame);
     PTRACE(5, "RTP\tIAX2 frame now on jitter buffer (As a RTP frame)");
     delete soundFrame;
 }
@@ -483,7 +474,7 @@ PBoolean IAX2Connection::ReadSoundPacket(RTP_DataFrame & packet)
     return false;   /* Sending sound now is meaningless */
   
   PTRACE(6, "Iax2Con\t Start read from  jitter buffer"); 
-  if (!jitterBuffer.ReadData(packet)) {
+  if (!m_jitterBuffer->ReadData(packet)) {
     PINDEX zeroBytes = packet.GetSize() - packet.GetHeaderSize();
     memset(packet.GetPayloadPtr() + packet.GetHeaderSize(), 0, zeroBytes);
     PTRACE(5, "Iax2Con\t faulty  read from  jitter buffer"); 
