@@ -25,20 +25,16 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 29658 $
+ * $Revision: 32828 $
  * $Author: rjongbloed $
- * $Date: 2013-05-07 21:16:26 -0500 (Tue, 07 May 2013) $
+ * $Date: 2014-09-26 04:52:33 +0100 (Fri, 26 Sep 2014) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
-  #ifdef _MSC_VER
-    #pragma message("IAX2 support enabled")
-  #endif
-
 #ifdef P_USE_PRAGMA
 #pragma implementation "iax2ep.h"
 #endif
@@ -56,7 +52,8 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 IAX2EndPoint::IAX2EndPoint(OpalManager & mgr)
-  : OpalEndPoint(mgr, "iax2", CanTerminateCall|SupportsE164)
+  : OpalEndPoint(mgr, "iax2", IsNetworkEndPoint | SupportsE164)
+  , callsEstablished(0)
 {
   localUserName = mgr.GetDefaultUserName();
   localNumber   = "1234";
@@ -67,7 +64,7 @@ IAX2EndPoint::IAX2EndPoint(OpalManager & mgr)
   transmitter = NULL;
   receiver = NULL;
   sock = NULL;
-  callsEstablished.SetValue(0);
+  callsEstablished = 0;
 
   IAX2IeCallToken::InitialiseKey();
   //We handle the deletion of regProcessor objects.
@@ -854,24 +851,4 @@ void IAX2IncomingEthernetFrames::Main()
   PTRACE(3, "Distribute\tEnd of thread - Do no more work now");
   return;
 }
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("IAX2 support DISABLED")
-  #endif
-
 #endif // OPAL_IAX2
-
-/* The comment below is magic for those who use emacs to edit this file. 
- * With the comment below, the tab key does auto indent to 2 spaces.     
- *
- * Local Variables:
- * mode:c
- * c-basic-offset:2
- * End:
- */
-
-
-
