@@ -24,13 +24,13 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 28483 $
+ * $Revision: 30574 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2013-09-26 04:26:16 +0100 (Thu, 26 Sep 2013) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_IAX2
 
@@ -76,24 +76,6 @@ OpalIAX2MediaStream::~OpalIAX2MediaStream()
   PTRACE(6, "Media\tDestroy OpalIAX2MediaStream");
 }
  
-PBoolean OpalIAX2MediaStream::Open()
-{
-  if (IsOpen())
-    return true;
-
-  PBoolean res = OpalMediaStream::Open();
-  PTRACE(3, "Media\t" << *this << " is now open");
- 
-  return res;
-}
- 
-PBoolean OpalIAX2MediaStream::Start()
-{
-  PTRACE(2, "Media\t" << *this << " Run ::Start");
-
-  return OpalMediaStream::Start();
-}
-  
  
 PBoolean OpalIAX2MediaStream::ReadPacket(RTP_DataFrame & packet)
 {
@@ -118,6 +100,8 @@ PBoolean OpalIAX2MediaStream::IsSynchronous() const
 {
   return IsSource();
 }
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 // This routine takes data from the source (eg mic) and sends the data to the remote host.
