@@ -24,13 +24,13 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 28483 $
+ * $Revision: 29536 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2013-04-19 09:55:15 +0100 (Fri, 19 Apr 2013) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 #include <ptclib/random.h>
 
 #if OPAL_IAX2
