@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29861 $
+ * $Revision: 33270 $
  * $Author: rjongbloed $
- * $Date: 2013-06-02 18:40:55 -0500 (Sun, 02 Jun 2013) $
+ * $Date: 2015-01-22 05:15:09 +0000 (Thu, 22 Jan 2015) $
  */
 
 #ifdef __GNUC__
@@ -62,10 +62,11 @@ OpalIM::OpalIM()
 }
 
 
-PAtomicInteger::IntegerType OpalIM::GetNextMessageId()
+static atomic<OpalIM::MessageID> s_messageIdCounter;
+
+OpalIM::MessageID OpalIM::GetNextMessageId()
 {
-  static PAtomicInteger messageIdCounter;
-  return ++messageIdCounter;
+  return ++s_messageIdCounter;
 }
 
 
@@ -76,6 +77,7 @@ OpalIMContext::OpalIMContext()
   , m_weStartedCall(false)
   , m_currentOutgoingMessage(NULL)
 { 
+  PTRACE_CONTEXT_ID_NEW();
   m_conversationId = OpalGloballyUniqueID().AsString();
   PTRACE(5, "OpalIM\tContext '" << m_conversationId << "' created");
 }
@@ -330,7 +332,7 @@ void OpalIMContext::SetCompositionIndicationNotifier(const CompositionIndication
 const PCaselessString & OpalIMEndPoint::Prefix() { static PConstCaselessString s("im"); return s; }
 
 OpalIMEndPoint::OpalIMEndPoint(OpalManager & manager)
-  : OpalEndPoint(manager, Prefix(), 0)
+  : OpalEndPoint(manager, Prefix(), NoAttributes)
   , m_manager(manager)
   , m_deleting(false)
 {
@@ -608,6 +610,7 @@ OpalIMContext::MessageDisposition
                                                PString & errorInfo)
 {
   PSafePtr<OpalIMContext> context = FindContextForMessageWithLock(message, connection);
+  PTRACE_CONTEXT_ID_PUSH_THREAD(context);
 
   // if context found, add message to it
   if (context == NULL) {
@@ -663,7 +666,7 @@ PBoolean OpalIMConnection::OnSetUpConnection()
   if (!LockReadWrite())
     return false;
 
-  OnConnectedInternal();
+  InternalOnConnected();
   UnlockReadWrite();
   return true;
 }
