@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29743 $
+ * $Revision: 33766 $
  * $Author: rjongbloed $
- * $Date: 2013-05-19 22:43:02 -0500 (Sun, 19 May 2013) $
+ * $Date: 2015-07-27 21:27:47 +0100 (Mon, 27 Jul 2015) $
  */
 
 #ifdef __GNUC__
@@ -46,46 +46,36 @@
 #include <opal/connection.h>
 #include <opal/endpoint.h>
 #include <im/t140.h>
-
-#if OPAL_SIP
-#include <sip/sdp.h>
-#endif
+#include <sdp/sdp.h>
 
 
 #define CRLF "\r\n"
 
 const PCaselessString & OpalMSRPMediaSession::TCP_MSRP() { static const PConstCaselessString s("TCP/MSRP"); return s; }
 
-static OpalMediaSessionFactory::Worker<OpalMSRPMediaSession> tcp_msrp_session(OpalMSRPMediaSession::TCP_MSRP());
+PFACTORY_CREATE(OpalMediaSessionFactory, OpalMSRPMediaSession, OpalMSRPMediaSession::TCP_MSRP());
 
-class OpalMSRPMediaType : public OpalMediaTypeDefinition 
+class OpalMSRPMediaDefinition : public OpalMediaTypeDefinition 
 {
   public:
     static const char * Name() { return OPAL_IM_MEDIA_TYPE_PREFIX"msrp"; }
 
-    OpalMSRPMediaType()
+    OpalMSRPMediaDefinition()
       : OpalMediaTypeDefinition(Name(), OpalMSRPMediaSession::TCP_MSRP())
     {
     }
 
-#if OPAL_SIP
-    static const PCaselessString & GetSDPMediaType();
-    static const PCaselessString & GetSDPTransportType();
-
-    virtual bool MatchesSDP(
-      const PCaselessString & sdpMediaType,
-      const PCaselessString & sdpTransport,
-      const PStringArray & sdpLines,
-      PINDEX index
-    );
+#if OPAL_SDP
+    virtual PString GetSDPMediaType() const { static PConstCaselessString const s("message"); return s; }
+    virtual PString GetSDPTransportType() const { return OpalMSRPMediaSession::TCP_MSRP(); }
 
     virtual SDPMediaDescription * CreateSDPMediaDescription(
       const OpalTransportAddress & localAddress
     ) const;
-#endif // OPAL_SIP
+#endif // OPAL_SDP
 };
 
-OPAL_INSTANTIATE_MEDIATYPE(OpalMSRPMediaType);
+OPAL_MEDIATYPE(OpalMSRPMedia);
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -94,7 +84,7 @@ OPAL_INSTANTIATE_MEDIATYPE(OpalMSRPMediaType);
 class IM##title##OpalMSRPEncoding : public OpalMSRPEncoding \
 { \
 }; \
-static PFactory<OpalMSRPEncoding>::Worker<IM##title##OpalMSRPEncoding> worker_##IM##title##OpalMSRPEncoding(encoding, true); \
+PFACTORY_CREATE(PFactory<OpalMSRPEncoding>, IM##title##OpalMSRPEncoding, encoding, true)
 
 
 DECLARE_MSRP_ENCODING(Text, "text/plain");
@@ -107,15 +97,17 @@ const OpalMediaFormat & GetOpalMSRP()
     public: 
       IMMSRPMediaFormat() 
         : OpalMediaFormat(OPAL_MSRP, 
-                          OpalMSRPMediaType::Name(),
+                          OpalMSRPMediaType(),
                           RTP_DataFrame::MaxPayloadType, 
                           "+", 
                           false,  
                           1440, 
                           512, 
                           0, 
-                          1000)            // as defined in RFC 4103 - good as anything else   
+                          1000)
       { 
+        SetOptionString(OpalMediaFormat::DescriptionOption(), "RFC 4975 (MSRP) Instant Message Relay");
+
         PFactory<OpalMSRPEncoding>::KeyList_T types = PFactory<OpalMSRPEncoding>::GetKeyList();
         PFactory<OpalMSRPEncoding>::KeyList_T::iterator r;
 
@@ -139,7 +131,7 @@ const OpalMediaFormat & GetOpalMSRP()
 } 
 
 
-#if OPAL_SIP
+#if OPAL_SDP
 
 /////////////////////////////////////////////////////////
 //
@@ -160,16 +152,6 @@ class SDPMSRPMediaDescription : public SDPMediaDescription
     SDPMSRPMediaDescription(const OpalTransportAddress & address);
     SDPMSRPMediaDescription(const OpalTransportAddress & address, const PString & url);
 
-    PCaselessString GetSDPTransportType() const
-    {
-      return OpalMSRPMediaType::GetSDPTransportType();
-    }
-
-    virtual PString GetSDPMediaType() const 
-    {
-      return OpalMSRPMediaType::GetSDPMediaType();
-    }
-
     virtual void CreateSDPMediaFormats(const PStringArray &);
     virtual void OutputAttributes(ostream & strm) const;
     virtual void SetAttribute(const PString & attr, const PString & value);
@@ -179,31 +161,28 @@ class SDPMSRPMediaDescription : public SDPMediaDescription
     virtual OpalMediaFormatList GetMediaFormats() const;
 
     // CreateSDPMediaFormat is used for processing format lists. MSRP always contains only "*"
-    virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & ) { return NULL; }
+    virtual SDPMediaFormat * CreateSDPMediaFormat();
 
     // FindFormat is used only for rtpmap and fmtp, neither of which are used for MSRP
     virtual SDPMediaFormat * FindFormat(PString &) const { return NULL; }
 
   protected:
+    class Format : public SDPMediaFormat
+    {
+      public:
+        Format(SDPMSRPMediaDescription & parent, const OpalMediaFormat & mediaFormat)
+          : SDPMediaFormat(parent)
+        {
+          FromMediaFormat(mediaFormat);
+        }
+    };
+
     PString path;
     PString types;
 };
 
-////////////////////////////////////////////////////////////////////////////////////////////
 
-const PCaselessString & OpalMSRPMediaType::GetSDPMediaType()     { static PConstCaselessString const s("message"); return s; }
-const PCaselessString & OpalMSRPMediaType::GetSDPTransportType() { static PConstCaselessString const s("tcp/msrp"); return s; }
-
-bool OpalMSRPMediaType::MatchesSDP(const PCaselessString & sdpMediaType,
-                                   const PCaselessString & sdpTransport,
-                                   const PStringArray & /*sdpLines*/,
-                                   PINDEX /*index*/)
-{
-  return sdpMediaType == GetSDPMediaType() && sdpTransport == GetSDPTransportType();
-}
-
-
-SDPMediaDescription * OpalMSRPMediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const
+SDPMediaDescription * OpalMSRPMediaDefinition::CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const
 {
   return new SDPMSRPMediaDescription(localAddress, PURL());
 }
@@ -217,21 +196,29 @@ SDPMediaDescription * OpalMSRPMediaSession::CreateSDPMediaDescription()
 ///////////////////////////////////////////////////////////////////////////////////////////
 
 SDPMSRPMediaDescription::SDPMSRPMediaDescription(const OpalTransportAddress & address)
-  : SDPMediaDescription(address, OpalMSRPMediaType::Name())
+  : SDPMediaDescription(address, OpalMSRPMediaType())
 {
   SetDirection(SDPMediaDescription::SendRecv);
 }
 
+
 SDPMSRPMediaDescription::SDPMSRPMediaDescription(const OpalTransportAddress & address, const PString & _path)
-  : SDPMediaDescription(address, OpalMSRPMediaType::Name())
+  : SDPMediaDescription(address, OpalMSRPMediaType())
   , path(_path)
 {
   SetDirection(SDPMediaDescription::SendRecv);
 }
 
+
 void SDPMSRPMediaDescription::CreateSDPMediaFormats(const PStringArray &)
 {
-  formats.Append(new SDPMediaFormat(*this, RTP_DataFrame::MaxPayloadType, OpalMSRP));
+  m_formats.Append(CreateSDPMediaFormat());
+}
+
+
+SDPMediaFormat * SDPMSRPMediaDescription::CreateSDPMediaFormat()
+{
+  return new Format(*this, OpalMSRP);
 }
 
 
@@ -255,7 +242,7 @@ void SDPMSRPMediaDescription::SetAttribute(const PString & attr, const PString &
 
 void SDPMSRPMediaDescription::ProcessMediaOptions(SDPMediaFormat & /*sdpFormat*/, const OpalMediaFormat & mediaFormat)
 {
-  if (mediaFormat.GetMediaType() == OpalMSRPMediaType::Name()) 
+  if (mediaFormat.GetMediaType() == OpalMSRPMediaType()) 
     types = mediaFormat.GetOptionString("Accept Types").Trim();
 }
 
@@ -274,18 +261,16 @@ OpalMediaFormatList SDPMSRPMediaDescription::GetMediaFormats() const
 
 void SDPMSRPMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != OpalMSRPMediaType::Name()) {
+  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != OpalMSRPMediaType()) {
     PTRACE(4, "MSRP\tSDP not including " << mediaFormat << " as it is not a valid MSRP format");
     return;
   }
 
-  SDPMediaFormat * sdpFormat = new SDPMediaFormat(*this, mediaFormat);
-  ProcessMediaOptions(*sdpFormat, mediaFormat);
-  AddSDPMediaFormat(sdpFormat);
+  AddSDPMediaFormat(new Format(*this, mediaFormat));
 }
 
 
-#endif // OPAL_SIP
+#endif // OPAL_SDP
 
 
 ////////////////////////////////////////////////////////////////////////////////////////////
@@ -336,14 +321,12 @@ OpalMSRPMediaSession::~OpalMSRPMediaSession()
 }
 
 
-bool OpalMSRPMediaSession::Open(const PString & localInterface, const OpalTransportAddress & remoteAddress, bool isMediaAddress)
+bool OpalMSRPMediaSession::Open(const PString & localInterface, const OpalTransportAddress & remoteAddress)
 {
-  if (!OpalMediaSession::Open(localInterface, remoteAddress, isMediaAddress))
-    return false;
-
   m_isOriginating = m_connection.IsOriginating();
   m_localMSRPSessionId = m_manager.CreateSessionID();
   m_localUrl = m_manager.SessionIDToURL(localInterface, m_localMSRPSessionId);
+  m_remoteAddress = remoteAddress;
   return true;
 }
 
@@ -520,7 +503,9 @@ void OpalMSRPMediaStream::OnReceiveMSRP(OpalMSRPManager &, OpalMSRPManager::Inco
 ////////////////////////////////////////////////////////////////////////////////////////////
 
 OpalMSRPManager::OpalMSRPManager(OpalManager & _opalManager, WORD _port)
-  : opalManager(_opalManager), m_listenerPort(_port), m_listenerThread(NULL)
+  : opalManager(_opalManager)
+  , m_listenerPort(_port)
+  , m_listenerThread(NULL)
 {
   if (m_listenerSocket.Listen(5, m_listenerPort, PSocket::CanReuseAddress)) 
     m_listenerThread = new PThreadObj<OpalMSRPManager>(*this, &OpalMSRPManager::ListenerThread);
@@ -603,34 +588,18 @@ void OpalMSRPManager::ListenerThread()
 
 PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL & localURL, const PURL & remoteURL)
 {
-  // get hostname of remote 
-  PIPSocket::Address ip(remoteURL.GetHostName());
-  WORD port = remoteURL.GetPort();
-  if (!ip.IsValid()) {
-    if (remoteURL.GetPortSupplied()) {
-      if (!PIPSocket::GetHostAddress(remoteURL.GetHostName(), ip)) {
-        PTRACE(2, "MSRP\tUnable to resolve MSRP URL '" << remoteURL << "' with explicit port");
-        return NULL;
-      }
-    }
-    else {
-#if P_DNS_RESOLVER
-      PIPSocketAddressAndPortVector addresses;
-      if (PDNS::LookupSRV(remoteURL.GetHostName(), "_im._msrp", remoteURL.GetPort(), addresses) && !addresses.empty()) {
-        ip   = addresses[0].GetAddress(); // Only use first entry
-        port = addresses[0].GetPort();
-      } 
-      else if (!PIPSocket::GetHostAddress(remoteURL.GetHostName(), ip)) {
-        PTRACE(2, "MSRP\tUnable to resolve MSRP URL hostname '" << remoteURL << "' ");
-        return NULL;
-      }
-#else
-      return NULL;
-#endif // P_DNS_RESOLVER
-    }
-  }
+  PIPSocket::AddressAndPort ap;
 
-  PString connectionKey(ip.AsString() + ":" + PString(PString::Unsigned, port));
+  // get hostname of remote 
+#if P_DNS_RESOLVER
+  PIPSocketAddressAndPortVector addresses;
+  if (PDNS::LookupSRV(remoteURL.GetHostName(), "_im._msrp", remoteURL.GetPort(), addresses) && !addresses.empty())
+    ap = addresses[0];
+  else
+#endif // P_DNS_RESOLVER
+    ap.Parse(remoteURL.GetHostPort());
+
+  PString connectionKey(ap.AsString());
 
   PSafePtr<Connection> connectionPtr = NULL;
 
@@ -640,7 +609,7 @@ PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL
     PWaitAndSignal m(m_connectionInfoMapAddMutex);
     ConnectionInfoMapType::iterator r = m_connectionInfoMap.find(connectionKey);
     if (r != m_connectionInfoMap.end()) {
-      PTRACE(2, "MSRP\tReusing existing connection to " << ip << ":" << port);
+      PTRACE(2, "MSRP\tReusing existing connection to " << ap);
       connectionPtr = r->second;
       connectionPtr.SetSafetyMode(PSafeReadWrite);
       ++connectionPtr->m_refCount;
@@ -656,15 +625,15 @@ PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL
   // create a connection to the remote
   // if cannot, remove it from connection map
   connectionPtr->m_protocol->SetReadTimeout(2000);
-  if (!connectionPtr->m_protocol->Connect(ip, port)) {
-    PTRACE(2, "MSRP\tUnable to make new connection to " << ip << ":" << port);
+  if (!connectionPtr->m_protocol->Connect(ap.GetAddress(), ap.GetPort())) {
+    PTRACE(2, "MSRP\tUnable to make new connection to " << ap);
     PWaitAndSignal m(m_connectionInfoMapAddMutex);
     m_connectionInfoMap.erase(connectionKey);
     connectionPtr.SetNULL();
     return NULL;
   }
 
-  PTRACE(2, "MSRP\tConnection established to to " << ip << ":" << port);
+  PTRACE(2, "MSRP\tConnection established to to " << ap);
 
 
   PString uid;
@@ -730,12 +699,12 @@ OpalMSRPManager::Connection::Connection(OpalMSRPManager & manager, const std::st
   , m_protocol(protocol)
   , m_running(true)
   , m_handlerThread(NULL)
+  , m_refCount(1)
 
 {
   PTRACE(3, "MSRP\tCreating connection");
   if (m_protocol == NULL)
     m_protocol = new MSRPProtocol();
-  m_refCount.SetValue(1);
 }
 
 void OpalMSRPManager::Connection::StartHandler()
@@ -917,14 +886,20 @@ bool MSRPProtocol::SendREPORT(const PString & chunkId,
   *this << "-------" << chunkId << "$" << CRLF;
   flush();
 
-  {
-    PStringStream str; str << ::setfill('\r') << mime.PrintContents(str);
-    PTRACE(4, "Sending MSRP REPORT\n" << "MSRP " << chunkId << " " << MSRPCommands[REPORT] << CRLF 
-                                                 << "To-Path: " << toUrl << CRLF 
-                                                 << "From-Path: "<< fromUrl << CRLF 
-                                                 << str << CRLF
-                                                 << "-------" << chunkId << "$");
+#if PTRACING
+  static const unsigned Level = 4;
+  if (PTrace::CanTrace(Level)) {
+    ostream & trace = PTRACE_BEGIN(Level, "MSRP");
+    trace << "Sending MSRP REPORT\n"
+             "MSRP " << chunkId << " " << MSRPCommands[REPORT] << "\n"
+             "To-Path: " << toUrl << "\n"
+             "From-Path: " << fromUrl << '\n'
+             << ::setfill('\r');
+    mime.PrintContents(trace);
+    trace << "-------" << chunkId << '$'
+          << PTrace::End;
   }
+#endif
 
   return true;
 }
@@ -981,17 +956,14 @@ bool MSRPProtocol::ReadMessage(int & command,
   body.MakeEmpty();
 
   // read MIME until empty line or terminator
-  bool terminated = false;
   {
     mime.RemoveAll();
     PString line;
     while (ReadLine(line, false)) {
       if (line.IsEmpty())
         break;
-      if (line.Find(terminator) == 0) {
-        terminated = true;
+      if (line.Find(terminator) == 0)
         break;
-      }
       mime.AddMIME(line);
     }
   }
