@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29850 $
+ * $Revision: 33853 $
  * $Author: rjongbloed $
- * $Date: 2013-05-31 02:34:06 -0500 (Fri, 31 May 2013) $
+ * $Date: 2015-08-08 12:30:34 +0100 (Sat, 08 Aug 2015) $
  */
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #ifdef __GNUC__
 #pragma implementation "sipim.h"
@@ -43,7 +43,7 @@
 #include <im/t140.h>
 #include <sip/sipep.h>
 #include <sip/sipcon.h>
-#include <sip/sdp.h>
+#include <sdp/sdp.h>
 
 
 #if OPAL_HAS_SIPIM
@@ -53,30 +53,23 @@ static PConstCaselessString const DispositionMimeType("message/imdn+xml");
 static char const ConversationIdSeparator = ' ';
 
 
-class OpalSIPIMMediaType : public OpalRTPAVPMediaType
+class OpalSIPIMMediaDefinition : public OpalRTPAVPMediaDefinition
 {
   public:
     static const char * Name() { return OPAL_IM_MEDIA_TYPE_PREFIX"sip"; } // RFC 3428
 
-    OpalSIPIMMediaType()
-      : OpalRTPAVPMediaType(Name())
+    OpalSIPIMMediaDefinition()
+      : OpalRTPAVPMediaDefinition(Name())
     {
     }
 
-    static const PCaselessString & GetSDPMediaType();
-    static const PCaselessString & GetSDPTransportType();
-
-    virtual bool MatchesSDP(
-      const PCaselessString & sdpMediaType,
-      const PCaselessString & sdpTransport,
-      const PStringArray & /*sdpLines*/,
-      PINDEX /*index*/
-    );
+    virtual PString GetSDPMediaType() const { static PConstCaselessString const s("message"); return s; }
+    virtual PString GetSDPTransportType() const { static PConstCaselessString const s("sip"); return s; }
 
     SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const;
 };
 
-OPAL_INSTANTIATE_MEDIATYPE(OpalSIPIMMediaType);
+OPAL_MEDIATYPE(OpalSIPIMMedia);
 
 
 /////////////////////////////////////////////////////////
@@ -90,88 +83,77 @@ OPAL_INSTANTIATE_MEDIATYPE(OpalSIPIMMediaType);
 //  - the format list is a SIP URL
 //
 
-class SDPSIPIMMediaDescription : public SDPMediaDescription
+class OpalSIPIMSDPMediaDescription : public SDPMediaDescription
 {
-  PCLASSINFO(SDPSIPIMMediaDescription, SDPMediaDescription);
+  PCLASSINFO(OpalSIPIMSDPMediaDescription, SDPMediaDescription);
   public:
-    SDPSIPIMMediaDescription(const OpalTransportAddress & address);
-    SDPSIPIMMediaDescription(
+    OpalSIPIMSDPMediaDescription(const OpalTransportAddress & address);
+    OpalSIPIMSDPMediaDescription(
       const OpalTransportAddress & address,
       const OpalTransportAddress & transportAddr,
       const PString & fromURL
     );
 
-    PCaselessString GetSDPTransportType() const
-    {
-      return OpalSIPIMMediaType::GetSDPTransportType();
-    }
-
     virtual SDPMediaDescription * CreateEmpty() const
     {
-      return new SDPSIPIMMediaDescription(OpalTransportAddress());
-    }
-
-    virtual PString GetSDPMediaType() const 
-    {
-      return OpalSIPIMMediaType::GetSDPMediaType();
+      return new OpalSIPIMSDPMediaDescription(OpalTransportAddress());
     }
 
     virtual PString GetSDPPortList() const;
 
     virtual void CreateSDPMediaFormats(const PStringArray &);
     virtual void SetAttribute(const PString & attr, const PString & value);
-    virtual void ProcessMediaOptions(SDPMediaFormat & sdpFormat, const OpalMediaFormat & mediaFormat);
     virtual void AddMediaFormat(const OpalMediaFormat & mediaFormat);
 
     virtual OpalMediaFormatList GetMediaFormats() const;
 
     // CreateSDPMediaFormat is used for processing format lists. MSRP always contains only "*"
-    virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & ) { return NULL; }
+    virtual SDPMediaFormat * CreateSDPMediaFormat();
 
     // FindFormat is used only for rtpmap and fmtp, neither of which are used for MSRP
     virtual SDPMediaFormat * FindFormat(PString &) const { return NULL; }
 
   protected:
+    class Format : public SDPMediaFormat
+    {
+      public:
+        Format(OpalSIPIMSDPMediaDescription & parent, const OpalMediaFormat & mediaFormat)
+          : SDPMediaFormat(parent)
+        {
+          FromMediaFormat(mediaFormat);
+        }
+    };
+
     OpalTransportAddress m_transportAddress;
     PString              m_fromURL;
 };
 
 
-////////////////////////////////////////////////////////////////////////////
-
-const PCaselessString & OpalSIPIMMediaType::GetSDPMediaType()     { static PConstCaselessString const s("message"); return s; }
-const PCaselessString & OpalSIPIMMediaType::GetSDPTransportType() { static PConstCaselessString const s("sip"); return s; }
-
-bool OpalSIPIMMediaType::MatchesSDP(const PCaselessString & sdpMediaType,
-                                    const PCaselessString & sdpTransport,
-                                    const PStringArray & /*sdpLines*/,
-                                    PINDEX /*index*/)
+SDPMediaDescription * OpalSIPIMMediaDefinition::CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const
 {
-  return sdpMediaType == GetSDPMediaType() && sdpTransport == GetSDPTransportType();
-}
-
-SDPMediaDescription * OpalSIPIMMediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const
-{
-  return new SDPSIPIMMediaDescription(localAddress);
+  return new OpalSIPIMSDPMediaDescription(localAddress);
 }
 
 ////////////////////////////////////////////////////////////////////////////
 
 const OpalMediaFormat & GetOpalSIPIM() 
 { 
-  static class IMSIPMediaFormat : public OpalMediaFormat { 
+  static class OpalSIPIMMediaFormat : public OpalMediaFormat
+  { 
     public: 
-      IMSIPMediaFormat() 
+      OpalSIPIMMediaFormat() 
         : OpalMediaFormat(OPAL_SIPIM, 
-                          OpalSIPIMMediaType::Name(),
+                          OpalSIPIMMediaType(),
                           RTP_DataFrame::MaxPayloadType, 
                           "+", 
                           false,  
                           1440, 
                           512, 
                           0, 
-                          1000)     // as defined in RFC 4103 - good as anything else
+                          1000)
       { 
+        SetOptionString(OpalMediaFormat::DescriptionOption(), "RFC 3428 (SIP) Instant Messaging");
+
         OpalMediaOptionString * option = new OpalMediaOptionString("URL", false, "");
         option->SetMerge(OpalMediaOption::NoMerge);
         AddOption(option);
@@ -183,17 +165,17 @@ const OpalMediaFormat & GetOpalSIPIM()
 
 ///////////////////////////////////////////////////////////////////////////////////////////
 
-SDPSIPIMMediaDescription::SDPSIPIMMediaDescription(const OpalTransportAddress & address)
-  : SDPMediaDescription(address, OpalSIPIMMediaType::Name())
+OpalSIPIMSDPMediaDescription::OpalSIPIMSDPMediaDescription(const OpalTransportAddress & address)
+  : SDPMediaDescription(address, OpalSIPIMMediaType())
 {
   SetDirection(SDPMediaDescription::SendRecv);
 }
 
 
-SDPSIPIMMediaDescription::SDPSIPIMMediaDescription(const OpalTransportAddress & address,
+OpalSIPIMSDPMediaDescription::OpalSIPIMSDPMediaDescription(const OpalTransportAddress & address,
                                                    const OpalTransportAddress & transportAddr,
                                                    const PString & fromURL)
-  : SDPMediaDescription(address, OpalSIPIMMediaType::Name())
+  : SDPMediaDescription(address, OpalSIPIMMediaType())
   , m_transportAddress(transportAddr)
   , m_fromURL(fromURL)
 {
@@ -201,13 +183,19 @@ SDPSIPIMMediaDescription::SDPSIPIMMediaDescription(const OpalTransportAddress &
 }
 
 
-void SDPSIPIMMediaDescription::CreateSDPMediaFormats(const PStringArray &)
+void OpalSIPIMSDPMediaDescription::CreateSDPMediaFormats(const PStringArray &)
 {
-  formats.Append(new SDPMediaFormat(*this, OpalSIPIM));
+  m_formats.Append(CreateSDPMediaFormat());
 }
 
 
-PString SDPSIPIMMediaDescription::GetSDPPortList() const
+SDPMediaFormat * OpalSIPIMSDPMediaDescription::CreateSDPMediaFormat()
+{
+  return new Format(*this, OpalSIPIM);
+}
+
+
+PString OpalSIPIMSDPMediaDescription::GetSDPPortList() const
 { 
   PString str = m_fromURL;
   if (str.Find('@') == P_MAX_INDEX)
@@ -216,17 +204,12 @@ PString SDPSIPIMMediaDescription::GetSDPPortList() const
 }
 
 
-void SDPSIPIMMediaDescription::SetAttribute(const PString & /*attr*/, const PString & /*value*/)
+void OpalSIPIMSDPMediaDescription::SetAttribute(const PString & /*attr*/, const PString & /*value*/)
 {
 }
 
 
-void SDPSIPIMMediaDescription::ProcessMediaOptions(SDPMediaFormat & /*sdpFormat*/, const OpalMediaFormat & /*mediaFormat*/)
-{
-}
-
-
-OpalMediaFormatList SDPSIPIMMediaDescription::GetMediaFormats() const
+OpalMediaFormatList OpalSIPIMSDPMediaDescription::GetMediaFormats() const
 {
   OpalMediaFormat sipim(OpalSIPIM);
   sipim.SetOptionString("URL", m_fromURL);
@@ -238,22 +221,20 @@ OpalMediaFormatList SDPSIPIMMediaDescription::GetMediaFormats() const
   return fmts;
 }
 
-void SDPSIPIMMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat)
+void OpalSIPIMSDPMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != OpalSIPIMMediaType::Name()) {
+  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != OpalSIPIMMediaType()) {
     PTRACE(4, "SIPIM\tSDP not including " << mediaFormat << " as it is not a valid SIPIM format");
     return;
   }
 
-  SDPMediaFormat * sdpFormat = new SDPMediaFormat(*this, mediaFormat);
-  ProcessMediaOptions(*sdpFormat, mediaFormat);
-  AddSDPMediaFormat(sdpFormat);
+  AddSDPMediaFormat(new Format(*this, mediaFormat));
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
-static PFactory<OpalIMContext>::Worker<OpalSIPIMContext> static_OpalSIPContext("sip");
+PFACTORY_CREATE(PFactory<OpalIMContext>, OpalSIPIMContext, "sip");
 
 OpalSIPIMContext::OpalSIPIMContext()
 {
@@ -367,19 +348,14 @@ void OpalSIPIMContext::OnReceivedMESSAGE(SIPEndPoint & endpoint,
                                          SIP_PDU & request)
 {
   // RFC3428
-  OpalIMEndPoint * imEP = endpoint.GetManager().FindEndPointAs<OpalIMEndPoint>(OpalIMEndPoint::Prefix());
-  if (imEP == NULL) {
-    PTRACE2(2, &endpoint, "SIPIM\tCannot find IM endpoint");
-    return;
-  }
 
-  SIPMIMEInfo & mime  = request.GetMIME();
-
-  OpalIMContext::MessageDisposition status;
+  OpalIMContext::MessageDisposition status = OpalIMContext::DeliveryOK;
   PString errorInfo;
   {
     OpalIM message;
 
+    SIPMIMEInfo & mime  = request.GetMIME();
+
     SIPURL to(mime.GetTo());
     to.SetTag(); // If not present
     message.m_conversationId = mime.GetCallID() + ConversationIdSeparator + to.GetTag();
@@ -396,11 +372,24 @@ void OpalSIPIMContext::OnReceivedMESSAGE(SIPEndPoint & endpoint,
     message.m_toAddr   = request.GetTransport()->GetLastReceivedAddress();
     message.m_fromAddr = request.GetTransport()->GetRemoteAddress();
 
-    message.m_bodies.SetAt(mime.GetContentType(), request.GetEntityBody());
-    status = imEP->OnRawMessageReceived(message, connection, errorInfo);
+    if (mime.GetContentType().NumCompare("multipart/") == PObject::EqualTo &&
+        mime.DecodeMultiPartList(message.m_bodyParts, request.GetEntityBody())) {
+      for (PMultiPartList::iterator it = message.m_bodyParts.begin(); it != message.m_bodyParts.end(); ++it)
+        message.m_bodies.SetAt(it->m_mime.GetString(PMIMEInfo::ContentTypeTag), it->m_textBody);
+    }
+    else
+      message.m_bodies.SetAt(mime.GetContentType(), request.GetEntityBody());
 
-    to.SetTag(message.m_conversationId.Mid(message.m_conversationId.Find(ConversationIdSeparator)+1));
-    mime.SetTo(to);
+    OpalIMEndPoint * imEP = endpoint.GetManager().FindEndPointAs<OpalIMEndPoint>(OpalIMEndPoint::Prefix());
+    if (imEP != NULL)
+      status = imEP->OnRawMessageReceived(message, connection, errorInfo);
+    else
+      endpoint.OnMessageReceived(message);
+
+    if (!message.m_conversationId.IsEmpty()) {
+      to.SetTag(message.m_conversationId.Mid(message.m_conversationId.Find(ConversationIdSeparator) + 1));
+      mime.SetTo(to);
+    }
   }
 
   SIPResponse * response = new SIPResponse(endpoint, request, SIP_PDU::Failure_BadRequest);
@@ -430,8 +419,7 @@ void OpalSIPIMContext::OnReceivedMESSAGE(SIPEndPoint & endpoint,
 void OpalSIPIMContext::PopulateParams(SIPMessage::Params & params, const OpalIM & message)
 {
   PString tag;
-  if (!message.m_conversationId.Split(ConversationIdSeparator, params.m_id, tag))
-    params.m_id = message.m_conversationId;
+  message.m_conversationId.Split(ConversationIdSeparator, params.m_id, tag, PString::SplitTrim|PString::SplitDefaultToBefore);
 
   SIPURL from(message.m_from);
   from.SetDisplayName(m_localName);
