@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28937 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 33270 $
+ * $Author: rjongbloed $
+ * $Date: 2015-01-22 05:15:09 +0000 (Thu, 22 Jan 2015) $
  */
 
 #include <ptlib.h>
@@ -33,10 +33,6 @@
 
 #if OPAL_CAPI
 
-  #ifdef _MSC_VER
-    #pragma message("CAPI ISDN support enabled")
-  #endif
-
 #ifdef _WIN32
 #include "capi_win32.h"
 #else
@@ -522,7 +518,7 @@ static struct {
 ///////////////////////////////////////////////////////////////////////
 
 OpalCapiEndPoint::OpalCapiEndPoint(OpalManager & manager)
-  : OpalEndPoint(manager, "isdn", CanTerminateCall|SupportsE164)
+  : OpalEndPoint(manager, "isdn", IsNetworkEndPoint | SupportsE164)
   , m_capi(new OpalCapiFunctions)
   , m_thread(NULL)
   , m_applicationId(0)
@@ -530,7 +526,7 @@ OpalCapiEndPoint::OpalCapiEndPoint(OpalManager & manager)
   PTRACE(4, "CAPI\tOpalCapiEndPoint created");
 
   OpalCapiFunctions::ApplID id = 0;
-  DWORD err = m_capi->REGISTER(MaxLineCount*MaxBlockCount*MaxBlockSize+1024,
+  OpalCapiFunctions::Result err = m_capi->REGISTER(MaxLineCount*MaxBlockCount*MaxBlockSize+1024,
                                MaxLineCount, MaxBlockCount, MaxBlockSize, &id);
   switch (err) {
     case CapiNoError :
@@ -634,11 +630,11 @@ PString OpalCapiEndPoint::GetDriverInfo() const
     return PString::Empty();
 
   OpalCapiProfile profile;
-  DWORD capiversion_min;
-  DWORD capiversion_maj;
-  DWORD manufacturerversion_min;
-  DWORD manufacturerversion_maj;
-  DWORD InstalledControllers;
+  OpalCapiFunctions::UInt capiversion_min;
+  OpalCapiFunctions::UInt capiversion_maj;
+  OpalCapiFunctions::UInt manufacturerversion_min;
+  OpalCapiFunctions::UInt manufacturerversion_maj;
+  OpalCapiFunctions::UInt installedControllers;
   char szBuffer[64] = "";
   PStringStream str;
 
@@ -649,11 +645,11 @@ PString OpalCapiEndPoint::GetDriverInfo() const
   str << "CAPI_version=" << capiversion_maj << '.' << capiversion_min << '(' << manufacturerversion_maj << '.' << manufacturerversion_min << ")\n";
 
   m_capi->GET_PROFILE(&profile, 0);
-  InstalledControllers = profile.m_NumControllers;
-  if (InstalledControllers > 30)
-    InstalledControllers = 1;
-  str << "ISDN_controllers=" << InstalledControllers << '\n';
-  for (DWORD controller = 1; controller <= InstalledControllers; ++controller) {
+  installedControllers = profile.m_NumControllers;
+  if (installedControllers > 30)
+    installedControllers = 1;
+  str << "ISDN_controllers=" << installedControllers << '\n';
+  for (OpalCapiFunctions::UInt controller = 1; controller <= installedControllers; ++controller) {
     m_capi->GET_PROFILE(&profile, controller);
     str << "Controller#_" << controller << "_NumB=" << profile.m_NumBChannels << '\n';
   }
@@ -795,7 +791,7 @@ void OpalCapiEndPoint::ProcessMessages(PThread &, P_INT_PTR)
 }
 
 
-bool OpalCapiEndPoint::IdToConnMap::Forward(const OpalCapiMessage & message, DWORD id)
+bool OpalCapiEndPoint::IdToConnMap::Forward(const OpalCapiMessage & message, unsigned id)
 {
   PSafePtr<OpalCapiConnection> connection;
 
@@ -891,7 +887,7 @@ bool OpalCapiEndPoint::PutMessage(OpalCapiMessage & message)
 {
   PTRACE(message.header.m_Command == CAPI_DATA_B3 ? 6 : 4, "CAPI\tSending message " << message);
   message.header.m_ApplId = (WORD)m_applicationId;
-  DWORD err = m_capi->PUT_MESSAGE(m_applicationId, &message);
+  OpalCapiFunctions::UInt err = m_capi->PUT_MESSAGE(m_applicationId, &message);
   PTRACE_IF(2, err != CapiNoError, "CAPI\tPUT_MESSAGE error 0x" << hex << err);
   return err == CapiNoError;
 }
@@ -1116,7 +1112,7 @@ void OpalCapiConnection::ProcessMessage(const OpalCapiMessage & message)
         case 0 :
           m_NCCI = message.param.connect_b3_conf.m_NCCI;
           AutoStartMediaStreams();
-          OnConnectedInternal();
+          InternalOnConnected();
           break;
 
         case 0x2004 :
@@ -1338,15 +1334,4 @@ PBoolean OpalCapiMediaStream::IsSynchronous() const
 {
   return true;
 }
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("CAPI ISDN support DISABLED")
-  #endif
-
 #endif // OPAL_CAPI
-
-
-// End of File ///////////////////////////////////////////////////////////////
