@@ -34,7 +34,7 @@
 #pragma implementation "dahdi_ep.cxx"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <lids/dahdi_ep.h>
 
@@ -99,8 +99,6 @@ bool DahdiLineInterfaceDevice::Open(const PString &)
         if (ioctl(fd, DAHDI_GET_PARAMS, &chanInfo))
           continue;
 
-        bool ok = true;
-
         ChannelInfo * channelInfo = NULL;
 
         switch (chanInfo.sigcap & (__DAHDI_SIG_FXO | __DAHDI_SIG_FXS)) {
@@ -141,7 +139,7 @@ bool DahdiLineInterfaceDevice::Open(const PString &)
     baseChannel += spanInfo.totalchans;
   }
 
-  PTRACE(3, "DAHDI\t" << m_channelInfoList.size() << " channels defined" << endl;
+  PTRACE(3, "DAHDI\t" << m_channelInfoList.size() << " channels defined");
 
   // build the list of FDs to poll for events
   BuildPollFDs();
