@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28483 $
+ * $Revision: 34469 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2015-12-31 08:30:24 +0000 (Thu, 31 Dec 2015) $
  */
 
 #include <ptlib.h>
@@ -37,7 +37,7 @@
 #pragma implementation "ixjlid.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <lids/ixjlid.h>
 
@@ -226,7 +226,7 @@ void OpalIxJDevice::SignalHandler(int sig)
           if (newHookState != info.hookState) {
             timeval now;
             gettimeofday(&now, NULL);
-            long diff = (now.tv_sec - info.lastHookChange.tv_sec) * 1000000;
+            long long diff = (now.tv_sec - info.lastHookChange.tv_sec) * 1000000ULL;
             diff += now.tv_usec - info.lastHookChange.tv_usec;
             diff = (diff + 500) / 1000;
             if (newHookState && (diff < FLASH_TIME))
