@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28937 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 33270 $
+ * $Author: rjongbloed $
+ * $Date: 2015-01-22 05:15:09 +0000 (Thu, 22 Jan 2015) $
  */
 
 #include <ptlib.h>
@@ -38,10 +38,6 @@
 
 #if OPAL_LID
 
-  #ifdef _MSC_VER
-    #pragma message("Line Interface Device support enabled")
-  #endif
-
 #include <opal/manager.h>
 #include <opal/call.h>
 #include <opal/patch.h>
@@ -56,7 +52,7 @@ static const char PrefixPOTS[] = "pots";
 /////////////////////////////////////////////////////////////////////////////
 
 OpalLineEndPoint::OpalLineEndPoint(OpalManager & mgr)
-  : OpalEndPoint(mgr, PrefixPOTS, CanTerminateCall|SupportsE164),
+  : OpalEndPoint(mgr, PrefixPOTS, SupportsE164),
     defaultLine("*")
 {
   PTRACE(4, "LID EP\tOpalLineEndPoint created");
@@ -245,6 +241,8 @@ PBoolean OpalLineEndPoint::AddLinesFromDevice(OpalLineInterfaceDevice & device)
       linesMutex.Wait();
       lines.Append(newLine);
       linesMutex.Signal();
+      if (!device.IsLineTerminal(line))
+        m_attributes |= IsNetworkEndPoint;
       PTRACE(3, "LID EP\tAdded line  " << line << ", " << (device.IsLineTerminal(line) ? "terminal" : "network"));
     }
     else {
@@ -287,9 +285,7 @@ PBoolean OpalLineEndPoint::AddDeviceNames(const PStringArray & descriptors)
 const OpalLineInterfaceDevice * OpalLineEndPoint::GetDeviceByName(const PString & descriptor)
 {
   PString deviceType, deviceName;
-  descriptor.Split(':', deviceType, deviceName);
-
-  if (deviceType.IsEmpty() || deviceName.IsEmpty()) {
+  if (!descriptor.Split(':', deviceType, deviceName, PString::SplitTrim|PString::SplitNonEmpty)) {
     PTRACE(1, "LID EP\tInvalid device description \"" << descriptor << '"');
     return NULL;
   }
@@ -714,7 +710,7 @@ void OpalLineConnection::Monitor()
       else {
         // If we are in alerting state then we are B-Party
         AutoStartMediaStreams();
-        OnConnectedInternal();
+        InternalOnConnected();
       }
     }
   }
@@ -852,7 +848,7 @@ PBoolean OpalLineConnection::SetUpConnection()
 
     PTRACE(3, "LID Con\tNo remote party indicated, going off hook without dialing.");
     AutoStartMediaStreams();
-    OnConnectedInternal();
+    InternalOnConnected();
     return true;
   }
 
@@ -883,7 +879,7 @@ PBoolean OpalLineConnection::SetUpConnection()
 
   // Wait for connection
   if (m_dialParams.m_progressTimeout == 0) {
-    OnConnectedInternal();
+    InternalOnConnected();
     return true;
   }
 
@@ -894,7 +890,7 @@ PBoolean OpalLineConnection::SetUpConnection()
       return false;
 
     if (line.IsConnected()) {
-      OnConnectedInternal();
+      InternalOnConnected();
       return true;
     }
 
@@ -916,7 +912,7 @@ PBoolean OpalLineConnection::SetUpConnection()
   }
 
   // Connect anyway
-  OnConnectedInternal();
+  InternalOnConnected();
   return true;
 }
 
@@ -983,7 +979,7 @@ PBoolean OpalLineMediaStream::ReadPacket(RTP_DataFrame & packet)
   if (notUsingRTP)
     return OpalMediaStream::ReadPacket(packet);
 
-  if (!packet.SetMinSize(RTP_DataFrame::MinHeaderSize+defaultDataSize))
+  if (!packet.SetMinSize(RTP_DataFrame::MinHeaderSize+GetDataSize()))
     return false;
 
   PINDEX count = packet.GetSize();
@@ -1001,7 +997,7 @@ PBoolean OpalLineMediaStream::WritePacket(RTP_DataFrame & packet)
     return OpalMediaStream::WritePacket(packet);
 
   PINDEX written = 0;
-  return line.WriteFrame(packet.GetPointer(), packet.GetHeaderSize()+packet.GetPayloadSize(), written);
+  return line.WriteFrame(packet.GetPointer(), packet.GetPacketSize(), written);
 }
 
 
@@ -1187,15 +1183,4 @@ unsigned OpalLineSilenceDetector::GetAverageSignalLevel(const BYTE *, PINDEX)
 {
   return line.GetAverageSignalLevel(true);
 }
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("Line Interface Device support DISABLED")
-  #endif
-
 #endif // OPAL_LID
-
-
-/////////////////////////////////////////////////////////////////////////////
