@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28937 $
- * $Author: csoutheren $
- * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
+ * $Revision: 31420 $
+ * $Author: rjongbloed $
+ * $Date: 2014-02-10 00:36:04 +0000 (Mon, 10 Feb 2014) $
  */
 
 #ifdef __GNUC__
@@ -35,7 +35,9 @@
 
 #include <ptlib.h>
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
+
+#if OPAL_LID
 
 #include <lids/lidpluginmgr.h>
 #include <ptclib/dtmf.h>
@@ -1160,3 +1162,4 @@ PStringList OpalPluginLID::GetCountryCodeNameList() const
   }
 }
 
+#endif //OPAL_LID
