@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29960 $
+ * $Revision: 34275 $
  * $Author: rjongbloed $
- * $Date: 2013-06-13 23:26:53 -0500 (Thu, 13 Jun 2013) $
+ * $Date: 2015-11-04 22:16:57 +0000 (Wed, 04 Nov 2015) $
  */
 
 #include <ptlib.h>
@@ -35,7 +35,7 @@
 #pragma implementation "call.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/call.h>
 
@@ -47,6 +47,8 @@
 
 #define new PNEW
 
+#define PTraceModule() "Call"
+
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -57,6 +59,8 @@ OpalCall::OpalCall(OpalManager & mgr)
   , m_establishedTime(0)
   , m_isEstablished(false)
   , m_isClearing(false)
+  , m_handlingHold(false)
+  , m_isCleared(false)
   , callEndReason(OpalConnection::NumCallEndReasons)
 #if OPAL_HAS_MIXER
   , m_recordManager(NULL)
@@ -81,7 +85,7 @@ OpalCall::OpalCall(OpalManager & mgr)
   }
 #endif
 
-  PTRACE(3, "Call\tCreated " << *this);
+  PTRACE(3, "Created " << *this << " ptr=" << this);
 }
 
 
@@ -99,7 +103,7 @@ OpalCall::~OpalCall()
   }
 #endif
 
-  PTRACE(3, "Call\tDestroyed " << *this);
+  PTRACE(3, "Destroyed " << *this << " ptr=" << this);
 }
 
 
@@ -111,7 +115,7 @@ void OpalCall::PrintOn(ostream & strm) const
 
 void OpalCall::OnEstablishedCall()
 {
-  PTRACE(3, "Call\tEstablished " << *this);
+  PTRACE(3, "Established " << *this);
   manager.OnEstablishedCall(*this);
 }
 
@@ -129,7 +133,7 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 {
   PTRACE_CONTEXT_ID_PUSH_THREAD(this);
 
-  PTRACE(3, "Call\tClearing " << (sync != NULL ? "(sync) " : "") << *this << " reason=" << reason);
+  PTRACE(3, "Clearing " << (sync != NULL ? "(sync) " : "") << *this << " reason=" << reason);
 
   {
     PSafeLockReadWrite lock(*this);
@@ -146,25 +150,11 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
     if (sync != NULL)
       m_endCallSyncPoint.push_back(sync);
 
-    switch (connectionsActive.GetSize()) {
-      case 0 :
-        break;
-
-      case 1 :
-        {
-          PSafePtr<OpalConnection> connection = connectionsActive.GetAt(0, PSafeReference);
-          if (connection != NULL)
-            connection->Release(reason);
-        }
-        break;
-
-      default :
-        // Release all but A-Party, it gets done in the B-Party OnReleased thread.
-        for (PINDEX i = 1; i < connectionsActive.GetSize(); ++i) {
-          PSafePtr<OpalConnection> connection = connectionsActive.GetAt(i, PSafeReference);
-          if (connection != NULL)
-            connection->Release(reason);
-        }
+    PINDEX i = connectionsActive.GetSize();
+    while (i > 0) {
+      PSafePtr<OpalConnection> connection = connectionsActive.GetAt(--i, PSafeReference);
+      if (connection != NULL)
+        connection->Release(reason);
     }
   }
 
@@ -173,12 +163,27 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 }
 
 
+void OpalCall::OnReleased(OpalConnection & connection)
+{
+  PTRACE(3, "OnReleased " << connection);
+  connectionsActive.Remove(&connection);
+
+  PSafePtr<OpalConnection> other = connectionsActive.GetAt(0, PSafeReference);
+  if (other != NULL && other->GetPhase() == OpalConnection::ReleasingPhase) {
+    PTRACE(4, "Follow on OnReleased to " << *other);
+    other->OnReleased();
+  }
+
+  InternalOnClear();
+}
+
+
 void OpalCall::InternalOnClear()
 {
   if (!connectionsActive.IsEmpty())
     return;
 
-  if (m_isCleared.TestAndSet(true))
+  if (m_isCleared.exchange(true))
     return;
 
   OnCleared();
@@ -189,7 +194,7 @@ void OpalCall::InternalOnClear()
 
   if (LockReadWrite()) {
     while (!m_endCallSyncPoint.empty()) {
-      PTRACE(5, "Call\tSignalling end call.");
+      PTRACE(5, "Signalling end call.");
       m_endCallSyncPoint.front()->Signal();
       m_endCallSyncPoint.pop_front();
     }
@@ -218,7 +223,7 @@ void OpalCall::OnNewConnection(OpalConnection & connection)
 
 PBoolean OpalCall::OnSetUp(OpalConnection & connection)
 {
-  PTRACE(3, "Call\tOnSetUp " << connection);
+  PTRACE(3, "OnSetUp " << connection);
 
   if (m_isClearing)
     return false;
@@ -239,43 +244,45 @@ PBoolean OpalCall::OnSetUp(OpalConnection & connection)
 
 void OpalCall::OnProceeding(OpalConnection & PTRACE_PARAM(connection))
 {
-  PTRACE(3, "Call\tOnProceeding " << connection);
+  PTRACE(3, "OnProceeding " << connection);
+}
+
+
+void OpalCall::OnAlerting(OpalConnection & connection, bool withMedia)
+{
+  PTRACE(3, "OnAlerting " << connection);
+
+  if (m_isClearing)
+    return;
+
+  PSafePtr<OpalConnection> otherConnection;
+  while (EnumerateConnections(otherConnection, PSafeReadWrite, &connection)) {
+    if (!otherConnection->SetAlerting(connection.GetRemotePartyName(), withMedia))
+      return;
+  }
+
+  SetPartyNames();
 }
 
 
 PBoolean OpalCall::OnAlerting(OpalConnection & connection)
 {
-  PTRACE(3, "Call\tOnAlerting " << connection);
-
-  if (m_isClearing)
-    return false;
-
-  PBoolean hasMedia = connection.GetMediaStream(OpalMediaType::Audio(), true) != NULL;
-
-  bool ok = false;
-
-  PSafePtr<OpalConnection> otherConnection;
-  while (EnumerateConnections(otherConnection, PSafeReadWrite, &connection)) {
-    if (otherConnection->SetAlerting(connection.GetRemotePartyName(), hasMedia))
-      ok = true;
-  }
-
-  SetPartyNames();
-
-  return ok;
+  connection.OnAlerting(connection.GetMediaStream(OpalMediaType(), true) != NULL);
+  return true;
 }
 
+
 OpalConnection::AnswerCallResponse OpalCall::OnAnswerCall(OpalConnection & PTRACE_PARAM(connection),
                                                           const PString & PTRACE_PARAM(caller))
 {
-  PTRACE(3, "Call\tOnAnswerCall " << connection << " caller \"" << caller << '"');
+  PTRACE(3, "OnAnswerCall " << connection << " caller \"" << caller << '"');
   return OpalConnection::AnswerCallDeferred;
 }
 
 
 PBoolean OpalCall::OnConnected(OpalConnection & connection)
 {
-  PTRACE(3, "Call\tOnConnected " << connection);
+  PTRACE(3, "OnConnected " << connection);
 
   if (m_isClearing || !LockReadOnly())
     return false;
@@ -309,7 +316,7 @@ PBoolean OpalCall::OnConnected(OpalConnection & connection)
 
 PBoolean OpalCall::OnEstablished(OpalConnection & connection)
 {
-  PTRACE(3, "Call\tOnEstablished " << connection);
+  PTRACE(3, "OnEstablished " << connection);
 
   PSafeLockReadWrite lock(*this);
   if (m_isClearing || !lock.IsLocked())
@@ -338,7 +345,7 @@ PBoolean OpalCall::OnEstablished(OpalConnection & connection)
 
 PSafePtr<OpalConnection> OpalCall::GetOtherPartyConnection(const OpalConnection & connection) const
 {
-  PTRACE(4, "Call\tGetOtherPartyConnection " << connection);
+  PTRACE(4, "GetOtherPartyConnection " << connection);
 
   PSafePtr<OpalConnection> otherConnection;
   EnumerateConnections(otherConnection, PSafeReference, &connection);
@@ -346,17 +353,21 @@ PSafePtr<OpalConnection> OpalCall::GetOtherPartyConnection(const OpalConnection
 }
 
 
-bool OpalCall::Hold()
+bool OpalCall::Hold(bool placeOnHold)
 {
   PTRACE_CONTEXT_ID_PUSH_THREAD(this);
 
-  PTRACE(3, "Call\tSetting to On Hold");
+  PTRACE(3, (placeOnHold ? "Setting to" : "Retrieving from") << " On Hold");
 
   bool ok = false;
 
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite)) {
-    if (connection->IsNetworkConnection() && connection->Hold(false, true))
+    if (!connection->IsNetworkConnection())
+      connection->HoldRemote(placeOnHold);
+  }
+  while (EnumerateConnections(connection, PSafeReadWrite)) {
+    if (connection->IsNetworkConnection() && connection->HoldRemote(placeOnHold))
       ok = true;
   }
 
@@ -364,29 +375,11 @@ bool OpalCall::Hold()
 }
 
 
-bool OpalCall::Retrieve()
-{
-  PTRACE_CONTEXT_ID_PUSH_THREAD(this);
-
-  PTRACE(3, "Call\tRetrieve from On Hold");
-
-  bool ok = false;
-
-  PSafePtr<OpalConnection> connection;
-  while (EnumerateConnections(connection, PSafeReadWrite)) {
-    if (connection->IsNetworkConnection() && connection->Hold(false, false))
-      ok = true;
-  }
-
-  return ok;
-}
-
-
-bool OpalCall::IsOnHold() const
+bool OpalCall::IsOnHold(bool fromRemote) const
 {
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadOnly)) {
-    if (connection->IsNetworkConnection() && connection->IsOnHold(false))
+    if (connection->IsNetworkConnection() && connection->IsOnHold(fromRemote))
       return true;
   }
 
@@ -409,12 +402,12 @@ bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
         return conn->TransferConnection(newAddress);
     }
 
-    PTRACE(2, "Call\tUnable to resolve transfer to \"" << newAddress << '"');
+    PTRACE(2, "Unable to resolve transfer to \"" << newAddress << '"');
     return false;
   }
 
   if (connection->IsReleased()) {
-    PTRACE(2, "Call\tCannot transfer to released connection " << *connection);
+    PTRACE(2, "Cannot transfer to released connection " << *connection);
     return false;
   }
 
@@ -424,7 +417,7 @@ bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
   if (prefix.IsEmpty() || prefix == connection->GetPrefixName() || manager.HasCall(newAddress))
     return connection->TransferConnection(newAddress);
 
-  PTRACE(3, "Call\tTransferring " << *connection << " to \"" << newAddress << '"');
+  PTRACE(3, "Transferring " << *connection << " to \"" << newAddress << '"');
 
   PSafePtr<OpalConnection> connectionToKeep = GetOtherPartyConnection(*connection);
   if (connectionToKeep == NULL)
@@ -441,7 +434,8 @@ bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
   if (newConnection->SetUpConnection() && newConnection->OnSetUpConnection()) {
     connectionToKeep->AutoStartMediaStreams(true);
     connection->Release(OpalConnection::EndedByCallForwarded, true);
-    StartMediaStreams(); // Just in case ...
+    newConnection->StartMediaStreams();
+    connectionToKeep->StartMediaStreams();
     return true;
   }
 
@@ -493,7 +487,7 @@ OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection)
 
   connection.AdjustMediaFormats(true, NULL, commonFormats);
 
-  PTRACE(4, "Call\tGetMediaFormats for " << connection << "\n    "
+  PTRACE(4, "GetMediaFormats for " << connection << "\n    "
          << setfill(',') << commonFormats << setfill(' '));
 
   return commonFormats;
@@ -510,34 +504,39 @@ void OpalCall::AdjustMediaFormats(bool local, const OpalConnection & connection,
 
 PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection, 
                                      const OpalMediaType & mediaType,
-                                                  unsigned sessionID, 
+                                                  unsigned requestedSessionID, 
                                    const OpalMediaFormat & preselectedFormat,
 #if OPAL_VIDEO
                               OpalVideoFormat::ContentRole contentRole,
 #endif
-                                                      bool transfer)
+                                                      bool transfer,
+                                                      bool startPaused)
 {
   PTRACE_CONTEXT_ID_PUSH_THREAD(this);
 
-  PSafeLockReadOnly lock(*this);
+  PSafeLockReadWrite lock(*this);
   if (m_isClearing || !lock.IsLocked())
     return false;
 
+  unsigned sessionID = requestedSessionID > 0 ? requestedSessionID : connection.GetNextSessionID(mediaType, true);
+
 #if PTRACING
   PStringStream traceText;
   if (PTrace::CanTrace(2)) {
-    traceText << " for " << mediaType << " session " << sessionID;
+    traceText << " for ";
+    if (sessionID == 0)
+      traceText << "unallocated ";
+    else if (requestedSessionID == 0)
+      traceText << "allocated ";
+    traceText << mediaType << " session";
+    if (sessionID != 0)
+      traceText << ' ' << sessionID;
     if (preselectedFormat.IsValid())
       traceText << " (" << preselectedFormat << ')';
     traceText << " on " << connection;
   }
 #endif
 
-  if (IsOnHold()) {
-    PTRACE(3, "Call\tOpenSourceMediaStreams (call on hold)" << traceText);
-    return false;
-  }
-
   // Check if already done
   OpalMediaStreamPtr sinkStream;
   OpalMediaStreamPtr sourceStream = connection.GetMediaStream(sessionID, true);
@@ -550,23 +549,21 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
         (sinkStream != NULL && sinkStream->GetMediaFormat() == preselectedFormat)) {
       if (sourceStream->IsPaused()) {
         sourceStream->SetPaused(false);
-        PTRACE(3, "Call\tOpenSourceMediaStreams (un-pausing)" << traceText);
+        PTRACE(3, "OpenSourceMediaStreams (un-pausing)" << traceText);
       }
       else {
-        PTRACE(3, "Call\tOpenSourceMediaStreams (already opened)" << traceText);
+        PTRACE(3, "OpenSourceMediaStreams (already opened)" << traceText);
       }
       return true;
     }
     if (sinkStream == NULL && sourceStream->IsOpen()) {
-      PTRACE(3, "Call\tOpenSourceMediaStreams (is opening)" << traceText);
+      PTRACE(3, "OpenSourceMediaStreams (is opening)" << traceText);
       return true;
     }
   }
 
-  if (sessionID == 0)
-    sessionID = connection.GetNextSessionID(mediaType, true);
-
-  PTRACE(3, "Call\tOpenSourceMediaStreams " << (sourceStream != NULL ? "replace" : "open") << traceText);
+  PTRACE(3, "OpenSourceMediaStreams " << (sourceStream != NULL ? "replace" : "open")
+         << (startPaused ? " (paused)" : "") << (transfer ? " (transfer)" : "") << traceText);
   sourceStream.SetNULL();
 
   // Create the sinks and patch if needed
@@ -586,7 +583,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 
     OpalMediaFormatList sinkMediaFormats = otherConnection->GetMediaFormats();
     if (sinkMediaFormats.IsEmpty()) {
-      PTRACE(2, "Call\tOpenSourceMediaStreams failed with no sink formats" << traceText);
+      PTRACE(2, "OpenSourceMediaStreams failed with no sink formats" << traceText);
       return false;
     }
 
@@ -601,7 +598,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     else {
       sourceMediaFormats = connection.GetMediaFormats();
       if (sourceMediaFormats.IsEmpty()) {
-        PTRACE(2, "Call\tOpenSourceMediaStreams failed with no source formats" << traceText);
+        PTRACE(2, "OpenSourceMediaStreams failed with no source formats" << traceText);
         return false;
       }
 
@@ -624,7 +621,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     if (contentRole == OpalVideoFormat::eNoRole && preselectedFormat.IsValid())
       contentRole = preselectedFormat.GetOptionEnum(OpalVideoFormat::ContentRoleOption(), OpalVideoFormat::eNoRole);
     if (contentRole != OpalVideoFormat::eNoRole) {
-      PTRACE(4, "Call\tContent Role " << contentRole << traceText);
+      PTRACE(4, "Content Role " << contentRole << traceText);
 
       bool noDefaultMainRole = contentRole != OpalVideoFormat::eMainRole;
 
@@ -652,7 +649,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 
       if (noDefaultMainRole) {
         if (newLists[0].IsEmpty() || newLists[1].IsEmpty()) {
-          PTRACE(2, "Call\tUnsupported Content Role " << contentRole << traceText);
+          PTRACE(2, "Unsupported Content Role " << contentRole << traceText);
           return false;
         }
 
@@ -660,17 +657,16 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
         sinkMediaFormats = newLists[1];
       }
       else {
-        PTRACE(4, "Call\tDefault main content role used " << traceText);
+        PTRACE(4, "Default main content role used " << traceText);
         contentRole = OpalVideoFormat::eNoRole;
       }
     }
 #endif
 
-    OpalMediaFormatList localMediaFormats = OpalMediaFormat::GetAllRegisteredMediaFormats();
     if (!SelectMediaFormats(mediaType,
                             sourceMediaFormats,
                             sinkMediaFormats,
-                            localMediaFormats,
+                            connection.GetLocalMediaFormats(),
                             sourceFormat,
                             sinkFormat))
       return false;
@@ -682,7 +678,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 
     if (sessionID == 0) {
       sessionID = otherConnection->GetNextSessionID(mediaType, false);
-      PTRACE(3, "Call\tOpenSourceMediaStreams using session " << sessionID << " on " << connection);
+      PTRACE(3, "OpenSourceMediaStreams using session " << sessionID << " on " << connection);
     }
 
     // Finally have the negotiated formats, open the streams
@@ -703,8 +699,10 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
       if (patch == NULL) {
         patch = manager.CreateMediaPatch(*sourceStream, sinkStream->RequiresPatchThread(sourceStream) &&
                                                         sourceStream->RequiresPatchThread(sinkStream));
-        if (patch == NULL)
+        if (patch == NULL) {
+          PTRACE(3, "OpenSourceMediaStreams could not patch session " << sessionID << " on " << connection);
           return false;
+        }
       }
 
       sourceStream.SetSafetyMode(PSafeReference);
@@ -719,23 +717,29 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
   if (!startedOne) {
     if (sourceStream != NULL)
       sourceStream->Close();
+    PTRACE(3, "OpenSourceMediaStreams did not start session " << sessionID << " on " << connection);
     return false;
   }
 
-  // if a patch was created, make sure the callback is called, just once per
-  // connection. Note must lock connection before stream or we can deadlock.
-  while (EnumerateConnections(otherConnection, PSafeReadWrite)) {
-    if (!sourceStream.SetSafetyMode(PSafeReadOnly))
-      return false;
+  if (!sourceStream.SetSafetyMode(PSafeReadOnly))
+    return false;
 
-    OpalMediaPatchPtr patch = sourceStream->GetPatch();
-    if (patch == NULL)
-      return false;
-
-    otherConnection->OnPatchMediaStream(otherConnection == &connection, *patch);
-    sourceStream.SetSafetyMode(PSafeReference);
+  if (startPaused) {
+    sourceStream->InternalSetPaused(true, false, false);
+    sinkStream->InternalSetPaused(true, false, false);
   }
 
+  OpalMediaPatchPtr patch = sourceStream->GetPatch();
+  if (patch == NULL) {
+    PTRACE(3, "OpenSourceMediaStreams patch failed session " << sessionID << " on " << connection);
+    return false;
+  }
+
+  // If a patch was created, make sure the callback is called once per connection.
+  while (EnumerateConnections(otherConnection, PSafeReadWrite))
+    otherConnection->OnPatchMediaStream(otherConnection == &connection, *patch);
+
+  PTRACE(4, "OpenSourceMediaStreams completed session " << sessionID << " on " << connection);
   return true;
 }
 
@@ -743,30 +747,26 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 bool OpalCall::SelectMediaFormats(const OpalMediaType & mediaType,
                                   const OpalMediaFormatList & srcFormats,
                                   const OpalMediaFormatList & dstFormats,
-                                  const OpalMediaFormatList & allFormats,
+                                  const OpalMediaFormatList & masterFormats,
                                   OpalMediaFormat & srcFormat,
                                   OpalMediaFormat & dstFormat) const
 {
-  if (OpalTranscoder::SelectFormats(mediaType, srcFormats, dstFormats, allFormats, srcFormat, dstFormat)) {
-    PTRACE(3, "Call\tSelected media formats " << srcFormat << " -> " << dstFormat);
+  if (OpalTranscoder::SelectFormats(mediaType, srcFormats, dstFormats, masterFormats, srcFormat, dstFormat)) {
+    PTRACE(4, "SelectMediaFormats:\n"
+              "  source formats=" << setfill(',') << srcFormats << "\n"
+              "   sink  formats=" << dstFormats << "\n"
+              "  master formats=" << masterFormats << setfill(' '));
+    PTRACE(3, "Selected media formats " << srcFormat << " -> " << dstFormat);
     return true;
   }
 
-  PTRACE(2, "Call\tSelectMediaFormats could not find compatible " << mediaType << " format:\n"
+  PTRACE(2, "SelectMediaFormats could not find compatible " << mediaType << " format:\n"
             "  source formats=" << setfill(',') << srcFormats << "\n"
             "   sink  formats=" << dstFormats << setfill(' '));
   return false;
 }
 
 
-void OpalCall::StartMediaStreams()
-{
-  PSafePtr<OpalConnection> connection;
-  while (EnumerateConnections(connection, PSafeReadWrite))
-    connection->StartMediaStreams();
-}
-
-
 void OpalCall::CloseMediaStreams()
 {
   PSafePtr<OpalConnection> connection;
@@ -799,38 +799,28 @@ void OpalCall::OnUserInputTone(OpalConnection & connection,
       reprocess = false;
   }
 
+  PTRACE(4, "OnUserInputTone: '" << tone << "', duration=" << duration << ", " << (reprocess ? "reprocessing as string" : "handled"));
+
   if (reprocess)
     connection.OnUserInputString(tone);
 }
 
 
-void OpalCall::OnReleased(OpalConnection & connection)
+void OpalCall::OnHold(OpalConnection & connection, bool fromRemote, bool onHold)
 {
-  PTRACE(3, "Call\tOnReleased " << connection);
-
-  SetCallEndReason(connection.GetCallEndReason());
-
-  connectionsActive.Remove(&connection);
-
-  // A call will evaporate when one connection left, at some point this is
-  // to be changes so can have "parked" connections. 
-  if(connectionsActive.GetSize() == 1)
-  {
-    PSafePtr<OpalConnection> last = connectionsActive.GetAt(0, PSafeReference);
-    if (last != NULL) {
-      PTRACE(4, "Call", "Releasing last connection in call");
-      last->Release(connection.GetCallEndReason(), true);
+  if (m_handlingHold)
+    manager.OnHold(connection, fromRemote, onHold);
+  else {
+    m_handlingHold = true;
+    PSafePtr<OpalConnection> otherConnection;
+    while (EnumerateConnections(otherConnection, PSafeReadWrite, &connection)) {
+      if (!otherConnection->IsNetworkConnection())
+        otherConnection->OnHold(fromRemote, onHold);
+      else if (fromRemote)
+        otherConnection->HoldRemote(onHold);
     }
+    m_handlingHold = false;
   }
-
-  InternalOnClear();
-}
-
-
-void OpalCall::OnHold(OpalConnection & /*connection*/,
-                      bool /*fromRemote*/, 
-                      bool /*onHold*/)
-{
 }
 
 #if OPAL_HAS_MIXER
@@ -841,7 +831,7 @@ bool OpalCall::StartRecording(const PFilePath & fn, const OpalRecordManager::Opt
 
   OpalRecordManager * newManager = OpalRecordManager::Factory::CreateInstance(fn.GetType());
   if (newManager == NULL) {
-    PTRACE(2, "OPAL\tCannot record to file type " << fn);
+    PTRACE(2, "Cannot record to file type " << fn);
     return false;
   }
 
