@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 30003 $
+ * $Revision: 34451 $
  * $Author: rjongbloed $
- * $Date: 2013-06-19 22:38:44 -0500 (Wed, 19 Jun 2013) $
+ * $Date: 2015-12-30 09:48:47 +0000 (Wed, 30 Dec 2015) $
  */
 
 #include <ptlib.h>
@@ -35,7 +35,7 @@
 #pragma implementation "endpoint.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/endpoint.h>
 #include <opal/manager.h>
@@ -46,15 +46,17 @@ static const OpalBandwidth DefaultInitialBandwidth = 4000000; // 4Mb/s
 
 #define new PNEW
 
+#define PTraceModule() "OpalEP"
+
 
 /////////////////////////////////////////////////////////////////////////////
 
 OpalEndPoint::OpalEndPoint(OpalManager & mgr,
                            const PCaselessString & prefix,
-                           unsigned attributes)
+                           Attributes attributes)
   : manager(mgr)
   , prefixName(prefix)
-  , attributeBits(attributes)
+  , m_attributes(attributes)
   , m_maxSizeUDP(4096)
   , productInfo(mgr.GetProductInfo())
   , defaultLocalPartyName(manager.GetDefaultUserName())
@@ -68,19 +70,19 @@ OpalEndPoint::OpalEndPoint(OpalManager & mgr,
   if (defaultLocalPartyName.IsEmpty())
     defaultLocalPartyName = PProcess::Current().GetName() & "User";
 
-  PTRACE(4, "OpalEP\tCreated endpoint: " << prefixName);
+  PTRACE(4, "Created endpoint: " << prefixName);
 }
 
 
 OpalEndPoint::~OpalEndPoint()
 {
-  PTRACE(4, "OpalEP\t" << prefixName << " endpoint destroyed.");
+  PTRACE(4, prefixName << " endpoint destroyed.");
 }
 
 
 void OpalEndPoint::ShutDown()
 {
-  PTRACE(3, "OpalEP\t" << prefixName << " endpoint shutting down.");
+  PTRACE(3, prefixName << " endpoint shutting down.");
 
   // Shut down the listeners as soon as possible to avoid race conditions
   listeners.RemoveAll();
@@ -106,23 +108,31 @@ OpalBandwidth OpalEndPoint::GetInitialBandwidth(OpalBandwidth::Direction dir) co
 }
 
 
-void OpalEndPoint::SetInitialBandwidth(OpalBandwidth::Direction dir, OpalBandwidth bandwidth)
+bool OpalEndPoint::SetInitialBandwidth(OpalBandwidth::Direction dir, OpalBandwidth bandwidth)
 {
   switch (dir) {
     case OpalBandwidth::Rx :
+      if (bandwidth < 64000)
+        return false;
       m_initialRxBandwidth = bandwidth;
       break;
 
     case OpalBandwidth::Tx :
+      if (bandwidth < 64000)
+        return false;
       m_initialTxBandwidth = bandwidth;
       break;
 
     default :
       OpalBandwidth rx = (PUInt64)(unsigned)bandwidth*m_initialRxBandwidth/(m_initialRxBandwidth+m_initialTxBandwidth);
       OpalBandwidth tx = (PUInt64)(unsigned)bandwidth*m_initialTxBandwidth/(m_initialRxBandwidth+m_initialTxBandwidth);
+      if (rx < 64000 || tx < 64000)
+        return false;
       m_initialRxBandwidth = rx;
       m_initialTxBandwidth = tx;
   }
+
+  return true;
 }
 
 
@@ -142,8 +152,10 @@ bool OpalEndPoint::StartListeners(const PStringArray & listenerAddresses, bool a
   OpalTransportAddressArray interfaces;
   if (listenerAddresses.IsEmpty()) {
     interfaces = GetDefaultListeners();
-    if (interfaces.IsEmpty())
+    if (interfaces.IsEmpty()) {
+      PTRACE(1, "No default listener interfaces specified for " << GetPrefixName());
       return false;
+    }
   }
   else {
     for (PINDEX i = 0; i < listenerAddresses.GetSize(); i++) {
@@ -159,7 +171,7 @@ bool OpalEndPoint::StartListeners(const PStringArray & listenerAddresses, bool a
             port = (WORD)transport.Mid(colon+1).AsUnsigned();
             transport.Delete(colon, P_MAX_INDEX);
           }
-          interfaces.AppendAddress(OpalTransportAddress(interfaces[i], port, transport));
+          interfaces.AppendAddress(OpalTransportAddress(listenerAddresses[i], port, transport));
         }
       }
     }
@@ -202,36 +214,34 @@ PBoolean OpalEndPoint::StartListener(const OpalTransportAddress & listenerAddres
 
   if (iface.IsEmpty()) {
     PStringArray interfaces = GetDefaultListeners();
-    if (interfaces.IsEmpty())
+    if (interfaces.IsEmpty()) {
+      PTRACE(1, "No default listener interfaces specified for " << GetPrefixName());
       return false;
+    }
     iface = OpalTransportAddress(interfaces[0], GetDefaultSignalPort());
   }
 
   // Check for already listening
   for (OpalListenerList::iterator it = listeners.begin(); it != listeners.end(); ++it) {
     if (it->GetLocalAddress().IsEquivalent(iface)) {
-      PTRACE(4, "OpalEP\tAlready listening on " << iface);
+      PTRACE(4, "Already listening on " << iface);
       return true;
     }
   }
 
   listener = iface.CreateListener(*this, OpalTransportAddress::FullTSAP);
   if (listener == NULL) {
-    PTRACE(1, "OpalEP\tCould not create listener: " << iface);
+    PTRACE(1, "Could not create listener: " << iface);
     return false;
   }
 
-  if (StartListener(listener))
-    return true;
-
-  PTRACE(1, "OpalEP\tCould not start listener: " << iface);
-  return false;
+  return StartListener(listener);
 }
 
 
 PBoolean OpalEndPoint::StartListener(OpalListener * listener)
 {
-  if (listener == NULL)
+  if (PAssertNULL(listener) == NULL)
     return false;
 
   OpalListenerUDP * udpListener = dynamic_cast<OpalListenerUDP *>(listener);
@@ -242,6 +252,7 @@ PBoolean OpalEndPoint::StartListener(OpalListener * listener)
   // stopping the listener thread. This is good - it means that the 
   // listener Close function will appear to have stopped the thread
   if (!listener->Open(PCREATE_NOTIFIER(NewIncomingConnection))) {
+    PTRACE(1, "Could not start listener: " << *listener);
     delete listener;
     return false;
   }
@@ -276,14 +287,10 @@ PStringArray OpalEndPoint::GetDefaultListeners() const
       transProto.Delete(pos, P_MAX_INDEX);
     }
     if (port != 0) {
-      if (transProto == OpalTransportAddress::UdpPrefix())
-        listenerAddresses += transProto + psprintf("*:%u", port);
-      else {
-        listenerAddresses += OpalTransportAddress(PIPSocket::Address::GetAny(4), port, transProto);
+      listenerAddresses += OpalTransportAddress(PIPSocket::Address::GetAny(4), port, transProto);
 #if OPAL_PTLIB_IPV6
-        listenerAddresses += OpalTransportAddress(PIPSocket::Address::GetAny(6), port, transProto);
+      listenerAddresses += OpalTransportAddress(PIPSocket::Address::GetAny(6), port, transProto);
 #endif
-      }
     }
   }
   return listenerAddresses;
@@ -318,133 +325,71 @@ PBoolean OpalEndPoint::RemoveListener(OpalListener * listener)
 
 
 static void AddTransportAddress(OpalTransportAddressArray & interfaceAddresses,
-                                const PIPSocket::Address & natInterfaceIP,
-                                const PIPSocket::Address & natExternalIP,
-                                const PIPSocket::Address & ip,
-                                WORD port,
-                                const PString & proto)
+                                const OpalTransportAddress & listenerAddress)
 {
-  if (ip != natExternalIP && (natInterfaceIP.IsAny() || ip == natInterfaceIP))
-    AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, natExternalIP, port, proto);
-
-  OpalTransportAddress addr(ip, port, proto);
-  if (interfaceAddresses.GetValuesIndex(addr) == P_MAX_INDEX)
-    interfaceAddresses.Append(new OpalTransportAddress(addr));
+  if (interfaceAddresses.GetValuesIndex(listenerAddress) == P_MAX_INDEX)
+    interfaceAddresses.Append(new OpalTransportAddress(listenerAddress));
 }
 
 
 static void AddTransportAddresses(OpalTransportAddressArray & interfaceAddresses,
-                                  bool excludeLocalHost,
-                                  const PIPSocket::Address & natInterfaceIP,
-                                  const PIPSocket::Address & natExternalIP,
-                                  const OpalTransportAddress & preferredAddress,
-                                  const OpalTransportAddress & localAddress,
-                                  bool includeProto)
+                                  const OpalListenerList & listeners,
+                                  const OpalTransportAddress & remoteAddress,
+                                  const OpalTransportAddress & interfaceAddress)
 {
-  PCaselessString proto = localAddress.GetProtoPrefix();
-  if (includeProto && proto != preferredAddress.GetProto(true))
-    return;
-
-  if (!preferredAddress.IsEmpty() && !preferredAddress.IsEquivalent(localAddress, true))
-    return;
-
-  PIPSocket::Address localIP;
-  WORD port = 0;
-  if (!localAddress.GetIpAndPort(localIP, port))
-    return;
-
-  PIPSocket::InterfaceTable interfaces;
-  if (!localIP.IsAny() || !PIPSocket::GetInterfaceTable(interfaces)) {
-    AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, localIP, port, proto);
-    return;
-  }
-
-
-  PIPSocket::Address preferredIP;
-  if (preferredAddress.GetIpAddress(preferredIP) && !preferredIP.IsAny()) {
-    for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
-      PIPSocket::Address ip = interfaces[i].GetAddress();
-      if (ip == preferredIP)
-        AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, ip, port, proto);
-    }
-  }
-
-  for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
-    PIPSocket::Address ip = interfaces[i].GetAddress();
-    if (ip.GetVersion() == localIP.GetVersion() && !(excludeLocalHost && ip.IsLoopback()))
-      AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, ip, port, proto);
+  for (OpalListenerList::const_iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
+    if (listener->GetLocalAddress().IsEquivalent(interfaceAddress, true))
+      AddTransportAddress(interfaceAddresses, listener->GetLocalAddress(remoteAddress, interfaceAddress));
   }
 }
 
 
-OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLocalHost,
-                                                              const OpalTransport * associatedTransport) const
+OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(const OpalTransport * associatedTransport) const
 {
   OpalTransportAddressArray interfaceAddresses;
 
-  OpalTransportAddress associatedLocalAddress, associatedRemoteAddress, associatedWildcard;
-  PIPSocket::Address natInterfaceIP = PIPSocket::GetInvalidAddress();
-  PIPSocket::Address natExternalIP = PIPSocket::GetInvalidAddress();
   if (associatedTransport != NULL) {
-    associatedLocalAddress = associatedTransport->GetLocalAddress();
-    associatedRemoteAddress = associatedTransport->GetRemoteAddress();
+    OpalTransportAddress remoteAddress = associatedTransport->GetRemoteAddress();
+    PIPSocket::Address associatedInterfaceIP(associatedTransport->GetInterface());
+    if (!associatedInterfaceIP.IsValid())
+      associatedTransport->GetLocalAddress().GetIpAddress(associatedInterfaceIP);
+    AddTransportAddresses(interfaceAddresses, listeners, remoteAddress,
+                          OpalTransportAddress(associatedInterfaceIP, 65535, remoteAddress.GetProtoPrefix()));
+    AddTransportAddresses(interfaceAddresses, listeners, remoteAddress,
+                          OpalTransportAddress(associatedInterfaceIP, 65535, OpalTransportAddress::IpPrefix()));
+  }
 
-    PIPSocket::Address remoteIP;
-    associatedRemoteAddress.GetIpAddress(remoteIP);
-
-    associatedWildcard = OpalTransportAddress(PIPSocket::Address::GetAny(remoteIP.GetVersion()), 0,
-                                              associatedLocalAddress.GetProtoPrefix());
-
-#if P_NAT
-    PNatMethod * natMethod = manager.GetNatMethod(remoteIP);
-    if (natMethod != NULL) {
-      natMethod->GetInterfaceAddress(natInterfaceIP);
-      natMethod->GetExternalAddress(natExternalIP);
+  PIPSocket::InterfaceTable interfaceTable;
+  for (OpalListenerList::const_iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
+    OpalTransportAddress listenerInterface = listener->GetLocalAddress();
+    PIPSocket::Address listenerIP;
+    WORD listenerPort = 0;
+    if (!listenerInterface.GetIpAndPort(listenerIP, listenerPort) || !listenerIP.IsAny())
+      AddTransportAddress(interfaceAddresses, listenerInterface);
+    else {
+      if (interfaceTable.IsEmpty())
+        PIPSocket::GetInterfaceTable(interfaceTable);
+      for (PINDEX i = 0; i < interfaceTable.GetSize(); ++i) {
+        if (!interfaceTable[i].GetAddress().IsLoopback())
+          AddTransportAddress(interfaceAddresses, OpalTransportAddress(interfaceTable[i].GetAddress(), listenerPort));
+      }
     }
+  }
+
+#if PTRACING
+  if (PTrace::CanTrace(4) && !interfaceAddresses.IsEmpty()) {
+    ostream & trace = PTRACE_BEGIN(4, "OpalMan");
+    trace << "Listener interfaces: ";
+    if (associatedTransport == NULL)
+      trace << "no ";
+    trace << "associated transport";
+    if (associatedTransport != NULL)
+      trace << *associatedTransport;
+    trace << "\n    " << setfill(',') << interfaceAddresses
+          << PTrace::End;
+  }
 #endif
-  }
 
-  OpalListenerList::const_iterator listener;
-
-  if (!associatedLocalAddress.IsEmpty()) {
-    for (listener = listeners.begin(); listener != listeners.end(); ++listener)
-      AddTransportAddresses(interfaceAddresses,
-                            excludeLocalHost,
-                            natInterfaceIP,
-                            natExternalIP,
-                            associatedLocalAddress,
-                            listener->GetLocalAddress(associatedRemoteAddress),
-                            true);
-    for (listener = listeners.begin(); listener != listeners.end(); ++listener)
-      AddTransportAddresses(interfaceAddresses,
-                            excludeLocalHost,
-                            natInterfaceIP,
-                            natExternalIP,
-                            associatedWildcard,
-                            listener->GetLocalAddress(associatedRemoteAddress),
-                            true);
-    for (listener = listeners.begin(); listener != listeners.end(); ++listener)
-      AddTransportAddresses(interfaceAddresses,
-                            excludeLocalHost,
-                            natInterfaceIP,
-                            natExternalIP,
-                            associatedLocalAddress,
-                            listener->GetLocalAddress(associatedRemoteAddress),
-                            false);
-  }
-
-  for (listener = listeners.begin(); listener != listeners.end(); ++listener)
-    AddTransportAddresses(interfaceAddresses,
-                          excludeLocalHost,
-                          natInterfaceIP,
-                          natExternalIP,
-                          OpalTransportAddress(),
-                          listener->GetLocalAddress(),
-                          false);
-
-  PTRACE(4, "OpalMan\tListener interfaces: associated transport="
-         << (associatedTransport != NULL ? (const char *)associatedLocalAddress : "None")
-         << "\n    " << setfill(',') << interfaceAddresses);
   return interfaceAddresses;
 }
 
@@ -456,8 +401,8 @@ void OpalEndPoint::NewIncomingConnection(OpalListener & /*listener*/, const Opal
 
 PSafePtr<OpalConnection> OpalEndPoint::GetConnectionWithLock(const PString & token, PSafetyMode mode) const
 {
-  if (token.IsEmpty())
-    return NULL;
+  if (token.IsEmpty() || token == "*")
+    return PSafePtr<OpalConnection>(connectionsActive, mode);
 
   PSafePtr<OpalConnection> connection = connectionsActive.FindWithLock(token, mode);
   if (connection != NULL)
@@ -506,11 +451,18 @@ OpalConnection * OpalEndPoint::AddConnection(OpalConnection * connection)
   if (connection == NULL)
     return NULL;
 
+  PString token = connection->GetToken();
+  if (connectionsActive.Contains(token)) {
+    PTRACE(2, "Cannot add connection, duplicate token: " << token);
+    delete connection;
+    return NULL;
+  }
+
   connection->SetStringOptions(m_defaultStringOptions, false);
 
-  OnNewConnection(connection->GetCall(), *connection);
+  connectionsActive.SetAt(token, connection);
 
-  connectionsActive.SetAt(connection->GetToken(), connection);
+  OnNewConnection(connection->GetCall(), *connection);
 
   return connection;
 }
@@ -524,7 +476,7 @@ void OpalEndPoint::DestroyConnection(OpalConnection * connection)
 
 PBoolean OpalEndPoint::OnSetUpConnection(OpalConnection & PTRACE_PARAM(connection))
 {
-  PTRACE(3, "OpalEP\tOnSetUpConnection " << connection);
+  PTRACE(3, "OnSetUpConnection " << connection);
   return true;
 }
 
@@ -540,6 +492,13 @@ void OpalEndPoint::OnProceeding(OpalConnection & connection)
   manager.OnProceeding(connection);
 }
 
+
+void OpalEndPoint::OnAlerting(OpalConnection & connection, bool withMedia)
+{
+  manager.OnAlerting(connection, withMedia);
+}
+
+
 void OpalEndPoint::OnAlerting(OpalConnection & connection)
 {
   manager.OnAlerting(connection);
@@ -566,7 +525,7 @@ void OpalEndPoint::OnEstablished(OpalConnection & connection)
 
 void OpalEndPoint::OnReleased(OpalConnection & connection)
 {
-  PTRACE(4, "OpalEP\tOnReleased " << connection);
+  PTRACE(4, "OnReleased " << connection);
   connectionsActive.RemoveAt(connection.GetToken());
   manager.OnReleased(connection);
 }
@@ -574,7 +533,7 @@ void OpalEndPoint::OnReleased(OpalConnection & connection)
 
 void OpalEndPoint::OnHold(OpalConnection & connection, bool fromRemote, bool onHold)
 {
-  manager.OnHold(connection, fromRemote, onHold);
+  connection.GetCall().OnHold(connection, fromRemote, onHold);
 }
 
 
@@ -587,14 +546,22 @@ void OpalEndPoint::OnHold(OpalConnection & connection)
 PBoolean OpalEndPoint::OnForwarded(OpalConnection & connection,
 			       const PString & forwardParty)
 {
-  PTRACE(4, "OpalEP\tOnForwarded " << connection);
+  PTRACE(4, "OnForwarded " << connection);
   return manager.OnForwarded(connection, forwardParty);
 }
 
 
-bool OpalEndPoint::OnTransferNotify(OpalConnection & connection, const PStringToString & info)
+bool OpalEndPoint::OnTransferNotify(OpalConnection & connection, const PStringToString & info, const OpalConnection * transferringConnection)
 {
-  return manager.OnTransferNotify(connection, info);
+  if (&connection != transferringConnection)
+    return false;
+
+  bool stayConnected = false;
+  PSafePtr<OpalConnection> otherConnection = connection.GetOtherPartyConnection();
+  if (otherConnection != NULL)
+    stayConnected = otherConnection->OnTransferNotify(info, transferringConnection);
+
+  return manager.OnTransferNotify(connection, info) || stayConnected;
 }
 
 
@@ -625,7 +592,7 @@ PBoolean OpalEndPoint::ClearCallSynchronous(const PString & token,
   if (!ClearCall(token, reason, sync))
     return false;
 
-  PTRACE(5, "OpalCon\tSynchronous wait for " << token);
+  PTRACE(5, "Synchronous wait for " << token);
   sync->Wait();
   return true;
 }
@@ -645,6 +612,16 @@ void OpalEndPoint::AdjustMediaFormats(bool local,
 }
 
 
+bool OpalEndPoint::GetMediaTransportAddresses(const OpalConnection & provider,
+                                              const OpalConnection & consumer,
+                                                          unsigned   sessionId,
+                                               const OpalMediaType & mediaType,
+                                         OpalTransportAddressArray & transports) const
+{
+  return manager.GetMediaTransportAddresses(provider, consumer, sessionId, mediaType, transports);
+}
+
+
 PBoolean OpalEndPoint::OnOpenMediaStream(OpalConnection & connection,
                                      OpalMediaStream & stream)
 {
@@ -658,6 +635,12 @@ void OpalEndPoint::OnClosedMediaStream(const OpalMediaStream & stream)
 }
 
 
+void OpalEndPoint::OnFailedMediaStream(OpalConnection & connection, bool fromRemote, const PString & reason)
+{
+  manager.OnFailedMediaStream(connection, fromRemote, reason);
+}
+
+
 void OpalEndPoint::SetMediaCryptoSuites(const PStringArray & security)
 {
   m_mediaCryptoSuites = security;
@@ -694,40 +677,6 @@ PStringArray OpalEndPoint::GetAllMediaCryptoSuites() const
 }
 
 
-#if P_NAT
-PNatStrategy & OpalEndPoint::GetNatMethods() const
-{
-  return manager.GetNatMethods();
-}
-
-
-PNatMethod * OpalEndPoint::GetNatMethod(const PIPSocket::Address & remoteAddress) const
-{
-  return manager.GetNatMethod(remoteAddress);
-}
-#endif
-
-
-#if OPAL_VIDEO
-PBoolean OpalEndPoint::CreateVideoInputDevice(const OpalConnection & connection,
-                                          const OpalMediaFormat & mediaFormat,
-                                          PVideoInputDevice * & device,
-                                          PBoolean & autoDelete)
-{
-  return manager.CreateVideoInputDevice(connection, mediaFormat, device, autoDelete);
-}
-
-PBoolean OpalEndPoint::CreateVideoOutputDevice(const OpalConnection & connection,
-                                           const OpalMediaFormat & mediaFormat,
-                                           PBoolean preview,
-                                           PVideoOutputDevice * & device,
-                                           PBoolean & autoDelete)
-{
-  return manager.CreateVideoOutputDevice(connection, mediaFormat, preview, device, autoDelete);
-}
-#endif
-
-
 void OpalEndPoint::OnUserInputString(OpalConnection & connection,
                                      const PString & value)
 {
@@ -858,6 +807,43 @@ void OpalEndPoint::OnMessageReceived(const OpalIM & message)
 #endif // OPAL_HAS_IM
 
 
+PStringList OpalEndPoint::GetAvailableStringOptions() const
+{
+  static char const * const StringOpts[] = {
+    OPAL_OPT_AUTO_START,
+    OPAL_OPT_CALL_IDENTIFIER,
+    OPAL_OPT_CALLING_PARTY_URL,
+    OPAL_OPT_CALLING_PARTY_NUMBER,
+    OPAL_OPT_CALLING_PARTY_NAME,
+    OPAL_OPT_CALLING_PARTY_DOMAIN,
+    OPAL_OPT_CALLING_DISPLAY_NAME,
+    OPAL_OPT_CALLED_PARTY_NAME,
+    OPAL_OPT_CALLED_DISPLAY_NAME,
+    OPAL_OPT_REDIRECTING_PARTY,
+    OPAL_OPT_PRESENTATION_BLOCK,
+    OPAL_OPT_ORIGINATOR_ADDRESS,
+    OPAL_OPT_INTERFACE,
+    OPAL_OPT_USER_INPUT_MODE,
+    OPAL_OPT_ENABLE_INBAND_DTMF,
+    OPAL_OPT_ENABLE_INBAND_DTMF,
+    OPAL_OPT_DETECT_INBAND_DTMF,
+    OPAL_OPT_SEND_INBAND_DTMF,
+    OPAL_OPT_DTMF_MULT,
+    OPAL_OPT_DTMF_DIV,
+    OPAL_OPT_DISABLE_JITTER,
+    OPAL_OPT_MAX_JITTER,
+    OPAL_OPT_MIN_JITTER,
+    OPAL_OPT_RECORD_AUDIO,
+    OPAL_OPT_ALERTING_TYPE,
+    OPAL_OPT_REMOVE_CODEC,
+    OPAL_OPT_SILENCE_DETECT_MODE,
+    OPAL_OPT_VIDUP_METHODS
+  };
+
+  return PStringList(PARRAYSIZE(StringOpts), StringOpts, true);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 bool OpalIsE164(const PString & number, bool strict)
