@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28483 $
+ * $Revision: 33929 $
  * $Author: rjongbloed $
- * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
+ * $Date: 2015-08-27 13:45:19 +0100 (Thu, 27 Aug 2015) $
  */
 
 #include <ptlib.h>
@@ -34,12 +34,13 @@
 #pragma implementation "transcoders.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <opal/transcoders.h>
 
 
 #define new PNEW
+#define PTraceModule() "Transcoder"
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -83,6 +84,8 @@ OpalTranscoder::OpalTranscoder(const OpalMediaFormat & inputMediaFormat,
   , acceptOtherPayloads(false)
   , m_inClockRate(inputMediaFormat.GetClockRate())
   , m_outClockRate(outputMediaFormat.GetClockRate())
+  , m_lastPayloadType(RTP_DataFrame::IllegalPayloadType)
+  , m_consecutivePayloadTypeMismatches(0)
 {
 }
 
@@ -106,11 +109,11 @@ PBoolean OpalTranscoder::ExecuteCommand(const OpalMediaCommand & command)
   if (outputMediaFormat.IsTransportable()) {
     const OpalMediaFlowControl * flow = dynamic_cast<const OpalMediaFlowControl *>(&command);
     if (flow != NULL) {
-      unsigned bitsPerSecond = flow->GetMaxBitRate();
-      outputMediaFormat.SetOptionInteger(OpalMediaFormat::MaxBitRateOption(), bitsPerSecond);
-      if ((unsigned)outputMediaFormat.GetOptionInteger(OpalMediaFormat::TargetBitRateOption()) > bitsPerSecond)
-        outputMediaFormat.SetOptionInteger(OpalMediaFormat::TargetBitRateOption(), bitsPerSecond);
-      UpdateMediaFormats(OpalMediaFormat(), outputMediaFormat);
+      unsigned bitRate = std::min(flow->GetMaxBitRate(), outputMediaFormat.GetMaxBandwidth());
+      if ((unsigned)outputMediaFormat.GetOptionInteger(OpalMediaFormat::TargetBitRateOption()) != bitRate) {
+        outputMediaFormat.SetOptionInteger(OpalMediaFormat::TargetBitRateOption(), bitRate);
+        UpdateMediaFormats(OpalMediaFormat(), outputMediaFormat);
+      }
       return true;
     }
   }
@@ -119,12 +122,24 @@ PBoolean OpalTranscoder::ExecuteCommand(const OpalMediaCommand & command)
 }
 
 
+void OpalTranscoder::SetMaxOutputSize(PINDEX size)
+{
+  maxOutputSize = size;
+
+  if (outputMediaFormat.GetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption()) > (int)maxOutputSize) {
+    PTRACE(4, "Media\tReducing \"" << OpalMediaFormat::MaxTxPacketSizeOption() << "\" to " << maxOutputSize);
+    outputMediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), maxOutputSize);
+    UpdateMediaFormats(inputMediaFormat, outputMediaFormat);
+  }
+}
+
+
 void OpalTranscoder::NotifyCommand(const OpalMediaCommand & command) const
 {
   if (commandNotifier != PNotifier())
     commandNotifier(const_cast<OpalMediaCommand &>(command), m_sessionID);
   else
-    PTRACE(4, "Opal\tNo command notifier available for transcoder " << this);
+    PTRACE(4, "No command notifier available for transcoder " << this);
 }
 
 
@@ -172,25 +187,47 @@ PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFram
       output.RemoveTail();
   }
 
-  output.front().CopyHeader(input);
+  RTP_DataFrame & outframe = output.front();
+  outframe.SetPayloadSize(0);
+  outframe.CopyHeader(input);
 
   // set the output timestamp and marker bit
-  CopyTimestamp(output.front(), input, true);
+  CopyTimestamp(outframe, input, true);
 
   // set the output payload type directly from the output media format
   // and the input payload directly from the input media format
-  output.front().SetPayloadType(GetPayloadType(false));
+  outframe.SetPayloadType(GetPayloadType(false));
 
-  // do not transcode if no match
-  RTP_DataFrame::PayloadTypes packetPayloadType = input.GetPayloadType();
-  RTP_DataFrame::PayloadTypes formatPayloadType = inputMediaFormat.GetPayloadType();
-  if (formatPayloadType != RTP_DataFrame::MaxPayloadType && packetPayloadType != formatPayloadType && input.GetPayloadSize() > 0) {
-    PTRACE(2, "Opal\tExpected payload type " << formatPayloadType << ", but received " << packetPayloadType << ". Ignoring packet");
-    output.RemoveAll();
+  // Check for if we handle empty payload packets, if not just return empty payload packet
+  if (!AcceptEmptyPayload() && input.GetPayloadSize() == 0)
+    return true;
+
+  RTP_DataFrame::PayloadTypes pt = input.GetPayloadType();
+
+  // Check for if we can handle comfort noise, if not just return empty payload packet
+  if (!AcceptComfortNoise() && (pt == RTP_DataFrame::CN || pt == RTP_DataFrame::Cisco_CN)) {
+    PTRACE(5, "Removing comfort noise frame with payload type " << pt);
     return true;
   }
 
-  return Convert(input, output.front());
+  // Check if we can handle different payload types
+  if (AcceptOtherPayloads() || pt == GetPayloadType(true))
+    return Convert(input, outframe);
+
+  // If not see if we get a lot of consecutive ones
+  if (pt != m_lastPayloadType) {
+    m_consecutivePayloadTypeMismatches = 0;
+    m_lastPayloadType = pt;
+  }
+  else if (++m_consecutivePayloadTypeMismatches > 10) {
+    // OK, we give in, you are really sending this payload type. Hopefully the actual codec is right!
+    PTRACE(2, "Consecutive mismatched payload type, expected "  << GetPayloadType(true) << ", now using " << pt);
+    inputMediaFormat.SetPayloadType(pt);
+    return Convert(input, outframe);
+  }
+
+  PTRACE(4, "Removing frame with mismatched payload type " << pt << " - should be " << GetPayloadType(true));
+  return true;
 }
 
 
@@ -199,9 +236,9 @@ OpalTranscoder * OpalTranscoder::Create(const OpalMediaFormat & srcFormat,
                                                    const BYTE * instance,
                                                        unsigned instanceLen)
 {
-  OpalTranscoder * transcoder = OpalTranscoderFactory::CreateInstance(OpalTranscoderKey(srcFormat, destFormat));
+  OpalTranscoder * transcoder = OpalTranscoderFactory::CreateInstance(OpalTranscoderKey(srcFormat.GetName(), destFormat.GetName()));
   if (transcoder == NULL) {
-    PTRACE2(2, NULL, "Opal\tCould not create transcoder instance from " << srcFormat << " to " << destFormat);
+    PTRACE2(2, NULL, "Could not create transcoder instance from " << srcFormat << " to " << destFormat);
     return NULL;
   }
 
@@ -218,7 +255,7 @@ OpalTranscoder * OpalTranscoder::Create(const OpalMediaFormat & srcFormat,
   if (transcoder->UpdateMediaFormats(srcFormat, destFormat))
     return transcoder;
 
-  PTRACE2(2, transcoder, "Opal\tError creating transcoder instance from " << srcFormat << " to " << destFormat);
+  PTRACE2(2, transcoder, "Error creating transcoder instance from " << srcFormat << " to " << destFormat);
   delete transcoder;
   return NULL;
 }
@@ -291,27 +328,27 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
   OpalMediaFormatList::const_iterator masterFormat = masterFormats.FindFormat(srcCapability);
   if (masterFormat == masterFormats.end()) {
     srcFormat = srcCapability;
-    PTRACE(5, "Opal\tInitial source format from capabilities:\n" << setw(-1) << srcFormat);
+    PTRACE(5, "Initial source format from capabilities:\n" << setw(-1) << srcFormat);
   }
   else {
     srcFormat = *masterFormat;
-    PTRACE(5, "Opal\tInitial source format from master:\n" << setw(-1) << srcFormat
+    PTRACE(5, "Initial source format from master:\n" << setw(-1) << srcFormat
                             << "Merging with capability\n" << setw(-1) << srcCapability);
-    if (!srcFormat.Update(srcCapability)) // This includes the PayloadType, a normal Merge does not.
+    if (!srcFormat.Merge(srcCapability, true)) // Include the PayloadType.
       return false;
   }
 
   masterFormat = masterFormats.FindFormat(dstCapability);
   if (masterFormat == masterFormats.end()) {
     dstFormat = dstCapability;
-    PTRACE(5, "Opal\tInitial destination format from capabilities:\n" << setw(-1) << dstFormat);
+    PTRACE(5, "Initial destination format from capabilities:\n" << setw(-1) << dstFormat);
   }
   else {
     dstFormat = *masterFormat;
-    PTRACE(5, "Opal\tInitial destination format from master:\n" << setw(-1) << dstFormat
+    PTRACE(5, "Initial destination format from master:\n" << setw(-1) << dstFormat
                                  << "Merging with capability\n" << setw(-1) << dstCapability);
 
-    if (!dstFormat.Update(dstCapability)) // This includes the PayloadType, a normal Merge does not.
+    if (!dstFormat.Merge(dstCapability, true)) // Include the PayloadType.
       return false;
   }
 
@@ -328,7 +365,7 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
 bool OpalTranscoder::SelectFormats(const OpalMediaType & mediaType,
                                    const OpalMediaFormatList & srcFormats,
                                    const OpalMediaFormatList & dstFormats,
-                                   const OpalMediaFormatList & allFormats,
+                                   const OpalMediaFormatList & masterFormats,
                                    OpalMediaFormat & srcFormat,
                                    OpalMediaFormat & dstFormat)
 {
@@ -338,7 +375,7 @@ bool OpalTranscoder::SelectFormats(const OpalMediaType & mediaType,
   // directly from the given format to a possible one with no transcoders.
   for (d = dstFormats.begin(); d != dstFormats.end(); ++d) {
     for (s = srcFormats.begin(); s != srcFormats.end(); ++s) {
-      if (*s == *d && s->GetMediaType() == mediaType && MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
+      if (*s == *d && s->GetMediaType() == mediaType && MergeFormats(masterFormats, *s, *d, srcFormat, dstFormat))
         return true;
     }
   }
@@ -347,10 +384,10 @@ bool OpalTranscoder::SelectFormats(const OpalMediaType & mediaType,
   for (d = dstFormats.begin(); d != dstFormats.end(); ++d) {
     for (s = srcFormats.begin(); s != srcFormats.end(); ++s) {
       if (s->GetMediaType() == mediaType || d->GetMediaType() == mediaType) {
-        OpalTranscoderKey search(*s, *d);
+        OpalTranscoderKey search(s->GetName(), d->GetName());
         OpalTranscoderList availableTranscoders = OpalTranscoderFactory::GetKeyList();
         for (OpalTranscoderIterator i = availableTranscoders.begin(); i != availableTranscoders.end(); ++i) {
-          if (search == *i && MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
+          if (search == *i && MergeFormats(masterFormats, *s, *d, srcFormat, dstFormat))
             return true;
         }
       }
@@ -363,7 +400,7 @@ bool OpalTranscoder::SelectFormats(const OpalMediaType & mediaType,
       if (s->GetMediaType() == mediaType || d->GetMediaType() == mediaType) {
         OpalMediaFormat intermediateFormat;
         if (FindIntermediateFormat(*s, *d, intermediateFormat) &&
-            MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
+            MergeFormats(masterFormats, *s, *d, srcFormat, dstFormat))
           return true;
       }
     }
@@ -432,10 +469,12 @@ OpalMediaFormatList OpalTranscoder::GetPossibleFormats(const OpalMediaFormatList
 {
   OpalMediaFormatList possibleFormats;
 
-  // Run through the formats connection can do directly and calculate all of
-  // the possible formats, including ones via a transcoder
-  for (OpalMediaFormatList::const_iterator f = formats.begin(); f != formats.end(); ++f) {
+  // Start possible formats with the formats connection can do directly
+  for (OpalMediaFormatList::const_iterator f = formats.begin(); f != formats.end(); ++f)
     possibleFormats += *f;
+
+  // Now calculate all of the possible formats via a transcoder
+  for (OpalMediaFormatList::const_iterator f = formats.begin(); f != formats.end(); ++f) {
     OpalMediaFormatList srcFormats = GetSourceFormats(*f);
     for (OpalMediaFormatList::iterator s = srcFormats.begin(); s != srcFormats.end(); ++s) {
       OpalMediaFormatList dstFormats = GetDestinationFormats(*s);
@@ -484,24 +523,18 @@ void OpalFramedTranscoder::CalculateSizes()
 {
   unsigned framesPerPacket = outputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(),
                               inputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1));
-  unsigned channels = outputMediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(),
-                       inputMediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(), 1));
+  // Use num channels from raw side, the network side is sometimes not actually what is used, e.g. Opus.
   unsigned inFrameSize = inputMediaFormat.GetFrameSize();
   unsigned outFrameSize = outputMediaFormat.GetFrameSize();
-  unsigned inFrameTime = inputMediaFormat.GetFrameTime();
-  unsigned outFrameTime = outputMediaFormat.GetFrameTime();
+  unsigned inFrameTime = inputMediaFormat.GetFrameTime()*1000000/m_inClockRate;   // microseconds
+  unsigned outFrameTime = outputMediaFormat.GetFrameTime()*1000000/m_outClockRate; // microseconds
   unsigned leastCommonMultiple = inFrameTime*outFrameTime/GreatestCommonDivisor(inFrameTime, outFrameTime);
-  inputBytesPerFrame = leastCommonMultiple/inFrameTime*inFrameSize*framesPerPacket*channels;
-  outputBytesPerFrame = leastCommonMultiple/outFrameTime*outFrameSize*framesPerPacket*channels;
+  inputBytesPerFrame = leastCommonMultiple/inFrameTime*inFrameSize*framesPerPacket;
+  outputBytesPerFrame = leastCommonMultiple/outFrameTime*outFrameSize*framesPerPacket;
 
-  PINDEX inMaxTimePerFrame  = inputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption()) * 
-                              inputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption());
-  PINDEX outMaxTimePerFrame = outputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption()) * 
-                              outputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption());
-
-  PINDEX maxPacketTime = PMAX(inMaxTimePerFrame, outMaxTimePerFrame);
-
-  maxOutputDataSize = outputBytesPerFrame * (maxPacketTime / outputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption()));
+  unsigned inMaxTimePerFrame  = inFrameTime*inputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption(), 1);
+  unsigned outMaxTimePerFrame = outFrameTime*outputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption(), 1);
+  maxOutputDataSize = outputBytesPerFrame*std::max(inMaxTimePerFrame, outMaxTimePerFrame)/outFrameTime;
 }
 
 
@@ -521,7 +554,7 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
     PINDEX inLen;
     if (inputIsRTP) {
       inputPtr = (const BYTE *)input;
-      inLen    = input.GetHeaderSize() + input.GetPayloadSize(); 
+      inLen = input.GetPacketSize();
     }
     else
     {
