@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28020 $
+ * $Revision: 34543 $
  * $Author: rjongbloed $
- * $Date: 2012-07-13 00:39:15 -0500 (Fri, 13 Jul 2012) $
+ * $Date: 2016-01-23 13:51:06 +0000 (Sat, 23 Jan 2016) $
  */
 
 #ifdef __GNUC__
@@ -40,8 +40,6 @@
 #include <math.h>
 
 
-const unsigned SecondsFrom1900to1970 = (70*365+17)*24*60*60U;
-
 class RTP_DataFrame;
 
 RTCP_XR_Metrics::RTCP_XR_Metrics(float    Ie,
@@ -88,8 +86,8 @@ RTCP_XR_Metrics::~RTCP_XR_Metrics()
 {
   PTRACE_IF(3, m_packetsReceived != 0 || m_packetsLost != 0,
             "VoIP Metrics\tRTCP_XR_Metrics final statistics:\n"
-            "   R Factor = "<< EndOfCallRFactor() << "\n"
-            "   MOS = "<< EndOfCallMOS());
+            "   R Factor = "<< GetEndOfCallRFactor() << "\n"
+            "   MOS = "<< GetEndOfCallMOS());
 }
 
 
@@ -125,72 +123,60 @@ RTCP_XR_Metrics * RTCP_XR_Metrics::Create(const RTP_DataFrame & frame)
       return new RTCP_XR_Metrics(20, 10, 5, 33, 13000);
 
     default:
-      PTRACE(3, NULL, "VoIP Metrics", "No Ie and Bpl data for payload type " << frame.GetPayloadType() <<
+      PTRACE(3, "VoIP Metrics", "No Ie and Bpl data for payload type " << frame.GetPayloadType() <<
                 ", unable to calculate R Factor and MOS score.");
       return NULL;
   }
 }
 
 
-void RTCP_XR_Metrics::SetJitterDelay(DWORD delay)
+void RTCP_XR_Metrics::SetJitterDelay(unsigned delay)
 {
   m_jitterDelay = delay;
 
   /* If the Id factor has changed, create a new Id period */
-  if (fabs(IdFactor() - m_lastId) < 1e-14) {
+  if (fabs(GetIdFactor() - m_lastId) < 1e-14) {
     PTime now;
-    createIdPeriod(m_lastJitterBufferChangeTimestamp, now);
+    CreateIdPeriod(m_lastJitterBufferChangeTimestamp, now);
     m_lastJitterBufferChangeTimestamp = now;
-    m_lastId = IdFactor();
+    m_lastId = GetIdFactor();
   }
 }
 
 
-BYTE RTCP_XR_Metrics::GetLossRate()
+unsigned RTCP_XR_Metrics::GetLossRate() const
 {
   unsigned packets = m_packetsReceived + m_packetsLost + m_packetsDiscarded;
-  if (packets == 0)
-    return 0;
-
-  return (BYTE)(256 * (float)m_packetsLost/packets);
+  return packets > 0 ? m_packetsLost*256/packets : 0;
 }
 
 
-BYTE RTCP_XR_Metrics::GetDiscardRate()
+unsigned RTCP_XR_Metrics::GetDiscardRate() const
 {
   unsigned packets = m_packetsReceived + m_packetsLost + m_packetsDiscarded;
-  if (packets == 0)
-    return 0;
-
-  return (BYTE)(256 * (float)m_packetsDiscarded/packets);
+  return packets > 0 ? m_packetsDiscarded*256/packets : 0;
 }
 
 
-BYTE RTCP_XR_Metrics::GetBurstDensity()
+unsigned RTCP_XR_Metrics::GetBurstDensity() const
 {
   unsigned packets = m_packetsReceivedInBurst + m_packetsLostInBurst;
-  if (packets == 0)
-    return 0;
-
-  return (BYTE)(ceil(256 * ((float)m_packetsLostInBurst)/packets));
+  return packets > 0 ? (m_packetsLostInBurst*256+packets-1)/packets : 0;
 }
 
 
-BYTE RTCP_XR_Metrics::GetGapDensity()
+unsigned RTCP_XR_Metrics::GetGapDensity() const
 {
   unsigned packets = m_packetsReceivedInGap + m_packetsLostInGap + m_packetsSinceLastLoss;
-  if (packets == 0) {
-    return 0;
-  }
-  return (BYTE)(ceil(256 * ((float)m_packetsLostInGap)/packets));
+  return packets > 0 ? (m_packetsLostInGap*256+packets-1)/packets : 0;
 }
 
 
-PUInt16b RTCP_XR_Metrics::GetBurstDuration()
+unsigned RTCP_XR_Metrics::GetBurstDuration() const
 {
   unsigned averageDuration = 0;
-  PUInt64  totalDuration = 0;
-  DWORD    count = 0;
+  uint64_t totalDuration = 0;
+  unsigned count = 0;
 
   /* If we are within a burst account for it */
   if (m_currentPeriodType == BURST) {
@@ -199,22 +185,22 @@ PUInt16b RTCP_XR_Metrics::GetBurstDuration()
   }
 
   /* Iterate over the list of periods to calculate a mean duration */
-  for (list<TimePeriod>::iterator period = m_timePeriods.begin(); period != m_timePeriods.end(); period++) {
+  for (list<TimePeriod>::const_iterator period = m_timePeriods.begin(); period != m_timePeriods.end(); period++) {
     if ((*period).type == BURST) {
       totalDuration = totalDuration + (*period).duration.GetMilliSeconds();
       count++;
     }
   }
 
-  return (WORD)(count > 0 ? (totalDuration/count) : averageDuration);
+  return (unsigned)(count > 0 ? (totalDuration/count) : averageDuration);
 }
 
 
-PUInt16b RTCP_XR_Metrics::GetGapDuration()
+unsigned RTCP_XR_Metrics::GetGapDuration() const
 {
   unsigned averageDuration = 0;
-  PUInt64  totalDuration = 0;
-  DWORD    count = 0;
+  uint64_t totalDuration = 0;
+  unsigned count = 0;
 
   /* If we are within a burst, the last received packets after the last loss are assumed to be in gap */
   PTime now;
@@ -229,18 +215,18 @@ PUInt16b RTCP_XR_Metrics::GetGapDuration()
   }
 
   /* Iterate over the list of periods to calculate a mean duration */
-  for (list<TimePeriod>::iterator period = m_timePeriods.begin(); period != m_timePeriods.end(); period++) {
+  for (list<TimePeriod>::const_iterator period = m_timePeriods.begin(); period != m_timePeriods.end(); period++) {
     if ((*period).type == GAP) {
       totalDuration = totalDuration + (*period).duration.GetMilliSeconds();
       count++;
     }
   }
 
-  return (WORD)(count > 0 ? (totalDuration/count) : averageDuration);
+  return (unsigned)(count > 0 ? (totalDuration/count) : averageDuration);
 }
 
 
-PUInt16b RTCP_XR_Metrics::GetRoundTripDelay()
+unsigned RTCP_XR_Metrics::GetRoundTripDelay() const
 {
   if (m_srPacketsReceived == 0 || !m_lsrTime.IsValid() || m_dlsrTime == 0)
     return 0;
@@ -254,7 +240,7 @@ PUInt16b RTCP_XR_Metrics::GetRoundTripDelay()
 }
 
 
-PUInt16b RTCP_XR_Metrics::GetEndSystemDelay()
+unsigned RTCP_XR_Metrics::GetEndSystemDelay() const
 {
   if (m_payloadBitrate == 0)
     return 0;
@@ -264,14 +250,14 @@ PUInt16b RTCP_XR_Metrics::GetEndSystemDelay()
 }
 
 
-BYTE RTCP_XR_Metrics::RFactor()
+unsigned RTCP_XR_Metrics::GetRFactor() const
 {
   /* The reported R factor is for conversational listening quality */
-  return RFactor(CQ);
+  return GetRFactor(CQ);
 }
 
 
-BYTE RTCP_XR_Metrics::RFactor(QualityType qt)
+unsigned RTCP_XR_Metrics::GetRFactor(QualityType qt) const
 {
   if (m_payloadBitrate == 0)
     return 127;
@@ -293,51 +279,51 @@ BYTE RTCP_XR_Metrics::RFactor(QualityType qt)
       R = 127;
   }
 
-  return (BYTE)ceil(R);
+  return (unsigned)ceil(R);
 }
 
 
-BYTE RTCP_XR_Metrics::EndOfCallRFactor()
+unsigned RTCP_XR_Metrics::GetEndOfCallRFactor() const
 {
   if (m_payloadBitrate == 0)
     return 127;
 
   /* Compute end of call R factor, according to the extended E-Model */
   double R = 93.4 - GetPonderateId() - GetEndOfCallIe();
-  return (BYTE)ceil(R);
+  return (unsigned)ceil(R);
 }
 
 
-BYTE RTCP_XR_Metrics::MOS_LQ()
+unsigned RTCP_XR_Metrics::GetMOS_LQ() const
 {
   if (m_payloadBitrate == 0)
     return 127;
 
   /* RTCP-XR requires MOS score in the range of 10 to 50 */
-  return (BYTE)ceil(MOS(LQ)*10);
+  return (unsigned)ceil(GetMOS(LQ)*10);
 }
 
 
-BYTE RTCP_XR_Metrics::MOS_CQ()
+unsigned RTCP_XR_Metrics::GetMOS_CQ() const
 {
   if (m_payloadBitrate == 0)
     return 127;
 
   /* RTCP-XR requires MOS score in the range of 10 to 50 */
-  return (BYTE)ceil(MOS(CQ)*10);
+  return (unsigned)ceil(GetMOS(CQ)*10);
 }
 
 
-float RTCP_XR_Metrics::MOS(QualityType qt)
+float RTCP_XR_Metrics::GetMOS(QualityType qt) const
 {
-  float R = RFactor(qt);
+  double R = GetRFactor(qt);
 
   /* Compute MOS, according to ITU-T G.107 */
   if (R <= 6.5153)
     return 1;
 
   if ((6.5153 < R) && (R < 100.))
-    return 1.0f + (0.035f * R) + (R * (R - 60.0f) * (100.0f - R) * 7.0f * pow(10.0f, -6.0f));
+    return (float)(1.0 + (0.035 * R) + (R * (R - 60.0) * (100.0 - R) * 7.0 * pow(10.0, -6.0)));
 
   if (R >= 100)
     return 4.5;
@@ -346,16 +332,16 @@ float RTCP_XR_Metrics::MOS(QualityType qt)
 }
 
 
-float RTCP_XR_Metrics::EndOfCallMOS()
+float RTCP_XR_Metrics::GetEndOfCallMOS() const
 {
-  float R = EndOfCallRFactor();
+  double R = GetEndOfCallRFactor();
 
   /* Compute MOS, according to ITU-T G.107 */
   if (R <= 6.5153)
     return 1;
 
   if ((6.5153 < R) && (R < 100.))
-    return 1.0f + (0.035f * R) + (R * (R - 60.0f) * (100.0f - R) * 7.0f * pow(10.0f, -6.0f));
+    return (float)(1.0 + (0.035 * R) + (R * (R - 60.0) * (100.0 - R) * 7.0 * pow(10.0, -6.0)));
 
   if (R >= 100.)
     return 4.5;
@@ -364,7 +350,7 @@ float RTCP_XR_Metrics::EndOfCallMOS()
 }
 
 
-float RTCP_XR_Metrics::IdFactor()
+float RTCP_XR_Metrics::GetIdFactor() const
 {
   float delay = (float) GetEndSystemDelay();
 
@@ -387,7 +373,7 @@ float RTCP_XR_Metrics::IdFactor()
 }
 
 
-float RTCP_XR_Metrics::GetPonderateId()
+float RTCP_XR_Metrics::GetPonderateId() const
 {
   float   ponderateId = 0;
   float   sumId = 0;
@@ -396,12 +382,12 @@ float RTCP_XR_Metrics::GetPonderateId()
 
   /* Account for the current time and Id value */
   PTime now;
-  sumId = IdFactor() * (now - m_lastJitterBufferChangeTimestamp).GetMilliSeconds();
+  sumId = GetIdFactor() * (now - m_lastJitterBufferChangeTimestamp).GetMilliSeconds();
   sumDuration = sumDuration + (now - m_lastJitterBufferChangeTimestamp).GetMilliSeconds();
   count++;
 
   /* Iterate over the list of periods to calculate an average Id */
-  for (list<IdPeriod>::iterator period = m_idPeriods.begin(); period != m_idPeriods.end(); period++) {
+  for (list<IdPeriod>::const_iterator period = m_idPeriods.begin(); period != m_idPeriods.end(); period++) {
     sumId = sumId + (*period).Id * (*period).duration.GetMilliSeconds();
     sumDuration = sumDuration + (*period).duration.GetMilliSeconds();
     count++;
@@ -414,7 +400,7 @@ float RTCP_XR_Metrics::GetPonderateId()
 }
 
 
-float RTCP_XR_Metrics::Ieff(PeriodType type)
+float RTCP_XR_Metrics::GetIeff(PeriodType type) const
 {
   float Ppl = 0;
 
@@ -441,7 +427,7 @@ float RTCP_XR_Metrics::Ieff(PeriodType type)
 }
 
 
-float RTCP_XR_Metrics::GetEndOfCallIe()
+float RTCP_XR_Metrics::GetEndOfCallIe() const
 {
   /* Calculate the time since the last burst period */
   PTime now;
@@ -452,7 +438,7 @@ float RTCP_XR_Metrics::GetEndOfCallIe()
 }
 
 
-float RTCP_XR_Metrics::GetPonderateIe()
+float RTCP_XR_Metrics::GetPonderateIe() const
 {
   float   ponderateIe = 0;
   float   sumIe = 0;
@@ -461,12 +447,12 @@ float RTCP_XR_Metrics::GetPonderateIe()
 
   /* Account for the current time and Ie value */
   PTime now;
-  ponderateIe = Ieff(m_currentPeriodType) * (now - m_periodBeginTimestamp).GetMilliSeconds();
+  ponderateIe = GetIeff(m_currentPeriodType) * (now - m_periodBeginTimestamp).GetMilliSeconds();
   sumDuration = sumDuration + (now - m_periodBeginTimestamp).GetMilliSeconds();
   count++;
 
   /* Iterate over the list of periods to calculate an average Ie */
-  for (list<IePeriod>::iterator period = m_iePeriods.begin(); period != m_iePeriods.end(); period++) {
+  for (list<IePeriod>::const_iterator period = m_iePeriods.begin(); period != m_iePeriods.end(); period++) {
     sumIe = (*period).Ieff * (*period).duration.GetMilliSeconds();
     sumDuration = sumDuration + (*period).duration.GetMilliSeconds();
     count++;
@@ -479,7 +465,7 @@ float RTCP_XR_Metrics::GetPonderateIe()
 }
 
 
-RTCP_XR_Metrics::TimePeriod RTCP_XR_Metrics::createTimePeriod(PeriodType type, PTime beginTimestamp, PTime endTimestamp)
+RTCP_XR_Metrics::TimePeriod RTCP_XR_Metrics::CreateTimePeriod(PeriodType type, PTime beginTimestamp, PTime endTimestamp)
 {
   TimePeriod newPeriod;
 
@@ -492,12 +478,12 @@ RTCP_XR_Metrics::TimePeriod RTCP_XR_Metrics::createTimePeriod(PeriodType type, P
 }
 
 
-RTCP_XR_Metrics::IdPeriod RTCP_XR_Metrics::createIdPeriod(PTime beginTimestamp, PTime endTimestamp)
+RTCP_XR_Metrics::IdPeriod RTCP_XR_Metrics::CreateIdPeriod(PTime beginTimestamp, PTime endTimestamp)
 {
   IdPeriod newPeriod;
 
   /* Get the current Id value */
-  newPeriod.Id = IdFactor();
+  newPeriod.Id = GetIdFactor();
   newPeriod.duration = endTimestamp - beginTimestamp;
 
   m_idPeriods.push_back(newPeriod);
@@ -506,7 +492,7 @@ RTCP_XR_Metrics::IdPeriod RTCP_XR_Metrics::createIdPeriod(PTime beginTimestamp,
 }
 
 
-RTCP_XR_Metrics::IePeriod RTCP_XR_Metrics::createIePeriod(RTCP_XR_Metrics::TimePeriod timePeriod)
+RTCP_XR_Metrics::IePeriod RTCP_XR_Metrics::CreateIePeriod(RTCP_XR_Metrics::TimePeriod timePeriod)
 {
   /* Calculate a perceptual Ie average value, according to the extended E-Model, presented by Alan Clark */
   float Ieg = 0;
@@ -521,7 +507,7 @@ RTCP_XR_Metrics::IePeriod RTCP_XR_Metrics::createIePeriod(RTCP_XR_Metrics::TimeP
 
   newPeriod.type = timePeriod.type;
   newPeriod.duration = timePeriod.duration;
-  newPeriod.Ieff = Ieff(newPeriod.type);
+  newPeriod.Ieff = GetIeff(newPeriod.type);
 
   if (newPeriod.type == BURST)
   {
@@ -566,62 +552,48 @@ void RTCP_XR_Metrics::OnPacketReceived()
 {
   m_packetsReceived++;
   m_packetsSinceLastLoss++;
-  markov(PACKET_RECEIVED);
+  Markov(PACKET_RECEIVED);
 }
 
 
 void RTCP_XR_Metrics::OnPacketDiscarded()
 {
   m_packetsDiscarded++;
-  markov(PACKET_DISCARDED);
+  Markov(PACKET_DISCARDED);
 }
 
 
 void RTCP_XR_Metrics::OnPacketLost()
 {
   m_packetsLost++;
-  markov(PACKET_LOST);
+  Markov(PACKET_LOST);
 }
 
 
-void RTCP_XR_Metrics::OnPacketLost(DWORD dropped)
+void RTCP_XR_Metrics::OnPacketLost(unsigned dropped)
 {
   for (DWORD i = 0; i < dropped; i++)
     OnPacketLost();
 }
 
 
-void RTCP_XR_Metrics::OnRxSenderReport(PUInt32b lsr, PUInt32b dlsr)
+void RTCP_XR_Metrics::OnRxSenderReport(const PTime & lastTimestamp, const PTimeInterval & delay)
 {
-  PTime now;
+  m_arrivalTime.SetCurrentTime();
 
-  /* Convert the arrival time to NTP timestamp */
-  PUInt32b arrival_ntp_sec  = (DWORD)(now.GetTimeInSeconds()+SecondsFrom1900to1970); // Convert from 1970 to 1900
-  PUInt32b arrival_ntp_frac = now.GetMicrosecond()*4294; // Scale microseconds to "fraction" from 0 to 2^32
+  /* Get the timestamp of the last SR */
+  if (lastTimestamp.IsValid())
+    m_lsrTime = lastTimestamp;
 
-  /* Take the middle 32 bits of NTP timestamp and break in seconds and microseconds */
-  arrival_ntp_sec = ((arrival_ntp_sec) & 0x0000FFFF);
-  arrival_ntp_frac = ((arrival_ntp_frac) & 0xFFFF0000);
-
-  m_arrivalTime = PTime(arrival_ntp_sec - SecondsFrom1900to1970, arrival_ntp_frac/4294);
-
-  if (lsr != 0) {
-    /* Get the timestamp of the last SR */
-    PUInt32b lsr_ntp_sec = ((lsr >> 16) & 0x0000FFFF);
-    PUInt32b lsr_ntp_frac = ((lsr << 16) & 0xFFFF0000);
-    m_lsrTime = PTime(lsr_ntp_sec - SecondsFrom1900to1970, lsr_ntp_frac/4294);
-  }
-
-  if (dlsr != 0) {
-    /* Get the delay since last SR */
-    m_dlsrTime.SetInterval (dlsr*1000/65536);
-  }
+  /* Get the delay since last SR */
+  if (delay != 0)
+    m_dlsrTime = delay;
 
   m_srPacketsReceived++;
 }
 
 
-void RTCP_XR_Metrics::markov(RTCP_XR_Metrics::PacketEvent event)
+void RTCP_XR_Metrics::Markov(RTCP_XR_Metrics::PacketEvent event)
 {
   if (m_packetsReceived == 0) {
     m_periodBeginTimestamp.SetCurrentTime();
@@ -637,7 +609,7 @@ void RTCP_XR_Metrics::markov(RTCP_XR_Metrics::PacketEvent event)
 
         if (m_currentPeriodType == BURST) {
           /* The burst period ended with the last packet loss */
-          createIePeriod(createTimePeriod(m_currentPeriodType, m_periodBeginTimestamp, m_lastLossTimestamp));
+          CreateIePeriod(CreateTimePeriod(m_currentPeriodType, m_periodBeginTimestamp, m_lastLossTimestamp));
           /* mark as a gap */
           m_currentPeriodType = GAP;
           m_periodBeginTimestamp = m_lastLossTimestamp;
@@ -661,7 +633,7 @@ void RTCP_XR_Metrics::markov(RTCP_XR_Metrics::PacketEvent event)
     case PACKET_RECEIVED :
       if (m_currentPeriodType == GAP) {
         /* The burst period ended with the packet received before the last loss */
-        createIePeriod(createTimePeriod(m_currentPeriodType, m_periodBeginTimestamp, m_lastLossTimestamp));
+        CreateIePeriod(CreateTimePeriod(m_currentPeriodType, m_periodBeginTimestamp, m_lastLossTimestamp));
          /* mark as a burst */
         m_currentPeriodType = BURST;
         m_periodBeginTimestamp = m_lastLossTimestamp;
@@ -694,68 +666,67 @@ void RTCP_XR_Metrics::markov(RTCP_XR_Metrics::PacketEvent event)
 }
 
 
-void RTCP_XR_Metrics::InsertExtendedReportPacket(unsigned PTRACE_PARAM(sessionID),
-                                                 DWORD syncSourceOut,
-                                                 OpalRTPSession::JitterBufferPtr jitter,
-                                                 RTP_ControlFrame & report)
+void RTCP_XR_Metrics::InsertMetricsReport(RTP_ControlFrame & report,
+                                          const OpalRTPSession & PTRACE_PARAM(session),
+                                          RTP_SyncSourceId syncSourceOut,
+                                          OpalJitterBuffer * jitter)
 {
-  report.StartNewPacket();
-  report.SetPayloadType(RTP_ControlFrame::e_ExtendedReport);
-  report.SetPayloadSize(sizeof(PUInt32b) + sizeof(RTP_ControlFrame::ExtendedReport));  // length is SSRC of packet sender plus XR
+  report.StartNewPacket(RTP_ControlFrame::e_ExtendedReport);
+  report.SetPayloadSize(sizeof(PUInt32b) + sizeof(RTP_ControlFrame::MetricsReport));  // length is SSRC of packet sender plus MR
   report.SetCount(1);
   BYTE * payload = report.GetPayloadPtr();
 
   // add the SSRC to the start of the payload
   *(PUInt32b *)payload = syncSourceOut;
   
-  RTP_ControlFrame::ExtendedReport & xr = *(RTP_ControlFrame::ExtendedReport *)(payload+4);
+  RTP_ControlFrame::MetricsReport & xr = *(RTP_ControlFrame::MetricsReport *)(payload+4);
   
-  xr.bt = 0x07;
+  xr.bt = 0x07; // Code for Metrics Report
   xr.type_specific = 0x00;
-  xr.length = 0x08;
+  xr.length = (sizeof(RTP_ControlFrame::MetricsReport)-sizeof(RTP_ControlFrame::ExtendedReport)+3)/4;
   xr.ssrc = syncSourceOut;
   
-  xr.loss_rate = GetLossRate();
-  xr.discard_rate = GetDiscardRate();
-  xr.burst_density = GetBurstDensity();
-  xr.gap_density = GetGapDensity();
-  xr.burst_duration = GetBurstDuration();
-  xr.gap_duration = GetGapDuration();
-  xr.round_trip_delay = GetRoundTripDelay();
-  xr.end_system_delay = GetEndSystemDelay();
+  xr.loss_rate = (uint8_t)GetLossRate();
+  xr.discard_rate = (uint8_t)GetDiscardRate();
+  xr.burst_density = (uint8_t)GetBurstDensity();
+  xr.gap_density = (uint8_t)GetGapDensity();
+  xr.burst_duration = (uint16_t)GetBurstDuration();
+  xr.gap_duration = (uint16_t)GetGapDuration();
+  xr.round_trip_delay = (uint16_t)GetRoundTripDelay();
+  xr.end_system_delay = (uint16_t)GetEndSystemDelay();
   xr.signal_level = 0x7F;
   xr.noise_level = 0x7F;
   xr.rerl = 0x7F;	
   xr.gmin = 16;
-  xr.r_factor = RFactor();
+  xr.r_factor = (uint8_t)GetRFactor();
   xr.ext_r_factor = 0x7F;
-  xr.mos_lq = MOS_LQ();
-  xr.mos_cq = MOS_CQ();
+  xr.mos_lq = (uint8_t)GetMOS_LQ();
+  xr.mos_cq = (uint8_t)GetMOS_CQ();
   xr.rx_config = 0x00;
   xr.reserved = 0x00;
 
   if (jitter != NULL) {
-    xr.jb_nominal = (WORD)(jitter->GetMinJitterDelay()/jitter->GetTimeUnits());
-    xr.jb_maximum = (WORD)(jitter->GetCurrentJitterDelay()/jitter->GetTimeUnits());
-    xr.jb_absolute = (WORD)(jitter->GetMaxJitterDelay()/jitter->GetTimeUnits());
+    xr.jb_nominal = (uint16_t)(jitter->GetMinJitterDelay()/jitter->GetTimeUnits());
+    xr.jb_maximum = (uint16_t)(jitter->GetCurrentJitterDelay()/jitter->GetTimeUnits());
+    xr.jb_absolute = (uint16_t)(jitter->GetMaxJitterDelay()/jitter->GetTimeUnits());
   }
   
   report.EndPacket();
   
-  PTRACE(3, "RTP\tSession " << sessionID << ", SentExtendedReport:"
-            " ssrc=" << xr.ssrc
-         << " loss_rate=" << (PUInt32b) xr.loss_rate
-         << " discard_rate=" << (PUInt32b) xr.discard_rate
-         << " burst_density=" << (PUInt32b) xr.burst_density
-         << " gap_density=" << (PUInt32b) xr.gap_density
+  PTRACE(3, "RTP", session << "SentExtendedReport:"
+            " ssrc=" << RTP_TRACE_SRC(xr.ssrc)
+         << " loss_rate=" << (unsigned)xr.loss_rate
+         << " discard_rate=" << (unsigned)xr.discard_rate
+         << " burst_density=" << (unsigned)xr.burst_density
+         << " gap_density=" << (unsigned)xr.gap_density
          << " burst_duration=" << xr.burst_duration
          << " gap_duration=" << xr.gap_duration
          << " round_trip_delay="<< xr.round_trip_delay
          << " end_system_delay="<< xr.end_system_delay
-         << " gmin="<< (PUInt32b) xr.gmin
-         << " r_factor="<< (PUInt32b) xr.r_factor
-         << " mos_lq="<< (PUInt32b) xr.mos_lq
-         << " mos_cq="<< (PUInt32b) xr.mos_cq
+         << " gmin="<< (unsigned)xr.gmin
+         << " r_factor="<< (unsigned)xr.r_factor
+         << " mos_lq="<< (unsigned)xr.mos_lq
+         << " mos_cq="<< (unsigned)xr.mos_cq
          << " jb_nominal_delay="<< xr.jb_nominal
          << " jb_maximum_delay="<< xr.jb_maximum
          << " jb_absolute_delay="<< xr.jb_absolute);
@@ -763,50 +734,33 @@ void RTCP_XR_Metrics::InsertExtendedReportPacket(unsigned PTRACE_PARAM(sessionID
 }
 
 
-OpalRTPSession::ExtendedReportArray
-RTCP_XR_Metrics::BuildExtendedReportArray(const RTP_ControlFrame & frame, PINDEX offset)
+RTP_MetricsReport::RTP_MetricsReport(const RTP_ControlFrame::MetricsReport & mr)
+  : sourceIdentifier(mr.ssrc)
+  , lossRate(mr.loss_rate)
+  , discardRate(mr.discard_rate)
+  , burstDensity(mr.burst_density)
+  , gapDensity(mr.gap_density)
+  , roundTripDelay(mr.round_trip_delay)
+  , RFactor(mr.r_factor)
+  , mosLQ(mr.mos_lq)
+  , mosCQ(mr.mos_cq)
+  , jbNominal(mr.jb_nominal)
+  , jbMaximum(mr.jb_maximum)
+  , jbAbsolute(mr.jb_absolute)
 {
-  OpalRTPSession::ExtendedReportArray reports;
-
-  const RTP_ControlFrame::ExtendedReport * rr = (const RTP_ControlFrame::ExtendedReport *)(frame.GetPayloadPtr()+offset);
-  for (PINDEX repIdx = 0; repIdx < (PINDEX)frame.GetCount(); repIdx++) {
-    OpalRTPSession::ExtendedReport * report = new OpalRTPSession::ExtendedReport;
-    report->sourceIdentifier = rr->ssrc;
-    report->lossRate = rr->loss_rate;
-    report->discardRate = rr->discard_rate;
-    report->burstDensity = rr->burst_density;
-    report->gapDensity = rr->gap_density;
-    report->roundTripDelay = rr->round_trip_delay;
-    report->RFactor = rr->r_factor;
-    report->mosLQ = rr->mos_lq;
-    report->mosCQ = rr->mos_cq;
-    report->jbNominal = rr->jb_nominal;
-    report->jbMaximum = rr->jb_maximum;
-    report->jbAbsolute = rr->jb_absolute;
-    reports.SetAt(repIdx, report);
-    rr++;
-  }
-  return reports;
 }
 
 
-void OpalRTPSession::OnRxExtendedReport(DWORD PTRACE_PARAM(src), const ExtendedReportArray & PTRACE_PARAM(reports))
+void OpalRTPSession::OnRxMetricsReport(RTP_SyncSourceId PTRACE_PARAM(src), const RTP_MetricsReport & PTRACE_PARAM(report))
 {
+  PTRACE(4, "RTP", *this << "OnMetricsReport: ssrc=" << RTP_TRACE_SRC(src) << ' ' << report);
+}
+
+
 #if PTRACING
-  if (PTrace::CanTrace(3)) {
-    ostream & strm = PTrace::Begin(3, __FILE__, __LINE__, this);
-    strm << "RTP\tSession " << m_sessionId << ", OnExtendedReport: ssrc=" << src << '\n';
-    for (PINDEX i = 0; i < reports.GetSize(); i++)
-      strm << "  XR: " << reports[i] << '\n';
-    strm << PTrace::End;
-  }
-#endif
-}
-
-
-void OpalRTPSession::ExtendedReport::PrintOn(ostream & strm) const
+void RTP_MetricsReport::PrintOn(ostream & strm) const
 {
-  strm << "ssrc=" << sourceIdentifier
+  strm << "ssrc=" << RTP_TRACE_SRC(sourceIdentifier)
        << " loss_rate=" << lossRate
        << " discard_rate=" << discardRate
        << " burst_density=" << burstDensity
@@ -819,5 +773,7 @@ void OpalRTPSession::ExtendedReport::PrintOn(ostream & strm) const
        << " jb_maximum=" << jbMaximum
        << " jb_absolute=" << jbAbsolute;
 }
+#endif // PTRACING
+
 
 #endif // OPAL_RTCP_XR
