@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 29837 $
+ * $Revision: 34054 $
  * $Author: rjongbloed $
- * $Date: 2013-05-30 02:31:47 -0500 (Thu, 30 May 2013) $
+ * $Date: 2015-09-18 16:04:49 +0100 (Fri, 18 Sep 2015) $
  */
 
 #include <ptlib.h>
@@ -34,7 +34,7 @@
 #pragma implementation "rtpconn.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <rtp/rtpconn.h>
 #include <rtp/rtpep.h>
@@ -49,10 +49,13 @@
 
 #include <algorithm>
 
+#define PTraceModule() "RTPCon"
 
 #define new PNEW
 
 
+///////////////////////////////////////////////////////////////////////////////
+
 OpalRTPConnection::OpalRTPConnection(OpalCall & call,
                              OpalRTPEndPoint  & ep,
                                 const PString & token,
@@ -81,13 +84,17 @@ OpalRTPConnection::~OpalRTPConnection()
 
 void OpalRTPConnection::OnReleased()
 {
+  for (SessionMap::iterator it = m_sessions.begin(); it != m_sessions.end(); ++it)
+    it->second->Close();
+
   OpalConnection::OnReleased();
 
-  for (SessionMap::iterator it = m_sessions.begin(); it != m_sessions.end(); ++it) {
-    OpalRTPSession * rtp = dynamic_cast<OpalRTPSession * >(&*it->second);
-    if (rtp != NULL && (rtp->GetPacketsSent() != 0 || rtp->GetPacketsReceived() != 0))
-      rtp->SendBYE();
-  }
+  if (m_rfc2833Handler != NULL)
+    m_rfc2833Handler->UseRTPSession(false, NULL);
+#if OPAL_T38_CAPABILITY
+  if (m_ciscoNSEHandler != NULL)
+    m_ciscoNSEHandler->UseRTPSession(false, NULL);
+#endif
 
   m_sessions.RemoveAll();
 }
@@ -119,59 +126,53 @@ unsigned OpalRTPConnection::GetNextSessionID(const OpalMediaType & mediaType, bo
 }
 
 
-vector<bool> OpalRTPConnection::CreateAllMediaSessions(CreateMediaSessionsSecurity security)
+OpalMediaCryptoSuite::KeyExchangeModes OpalRTPConnection::GetMediaCryptoKeyExchangeModes() const
 {
-  OpalMediaTypeList allMediaTypes = m_localMediaFormats.GetMediaTypes();
-  OpalMediaTypeList::iterator iterMediaType;
+  return OpalMediaCryptoSuite::KeyExchangeModes::All();
+}
 
-#if OPAL_VIDEO
-  // For maximum compatibility, make sure audio/video are first
-  iterMediaType = std::find(allMediaTypes.begin(), allMediaTypes.end(), OpalMediaType::Video());
-  if (iterMediaType != allMediaTypes.end() && iterMediaType != allMediaTypes.begin()) {
-    allMediaTypes.erase(iterMediaType);
-    allMediaTypes.insert(allMediaTypes.begin(), OpalMediaType::Video());
-  }
-#endif
-  iterMediaType = std::find(allMediaTypes.begin(), allMediaTypes.end(), OpalMediaType::Audio());
-  if (iterMediaType != allMediaTypes.end() && iterMediaType != allMediaTypes.begin()) {
-    allMediaTypes.erase(iterMediaType);
-    allMediaTypes.insert(allMediaTypes.begin(), OpalMediaType::Audio());
-  }
 
-  const PStringArray cryptoSuites = endpoint.GetMediaCryptoSuites();
+vector<bool> OpalRTPConnection::CreateAllMediaSessions()
+{
+  OpalMediaTypeList allMediaTypes = GetLocalMediaFormats().GetMediaTypes();
+  allMediaTypes.PrioritiseAudioVideo();
+
+  OpalMediaCryptoSuite::KeyExchangeModes keyExchangeModes = GetMediaCryptoKeyExchangeModes();
+  const PStringArray cryptoSuites = GetMediaCryptoSuites();
+  PTRACE(4, "Creating media sessions: media=" << setfill(',') << allMediaTypes << " - crypto=" << cryptoSuites);
 
   vector<bool> openedMediaSessions(allMediaTypes.size()*cryptoSuites.GetSize()+1);
 
-  for (iterMediaType = allMediaTypes.begin(); iterMediaType != allMediaTypes.end(); ++iterMediaType) {
+  for (OpalMediaTypeList::iterator iterMediaType = allMediaTypes.begin(); iterMediaType != allMediaTypes.end(); ++iterMediaType) {
     const OpalMediaType & mediaType = *iterMediaType;
     if (!PAssert(mediaType.GetDefinition() != NULL, PLogicError))
       continue;
 
     if (GetAutoStart(mediaType) == OpalMediaType::DontOffer) {
-      PTRACE(4, "RTPCon\tNot offerring " << mediaType);
-      continue;
-    }
-
-    // See if any media formats of this session id, so don't create unused RTP session
-    if (!m_localMediaFormats.HasType(mediaType)) {
-      PTRACE(3, "RTPCon\tNo media formats of type " << mediaType << ", not creating media session");
+      PTRACE(4, "Not offering " << mediaType);
       continue;
     }
 
     for (PINDEX csIdx = 0; csIdx < cryptoSuites.GetSize(); ++csIdx) {
       PCaselessString cryptoSuiteName = cryptoSuites[csIdx];
-      if (security == (cryptoSuiteName != OpalMediaCryptoSuite::ClearText() ? e_ClearMediaSession : e_SecureMediaSession))
+      if (cryptoSuiteName == OpalMediaCryptoSuite::ClearText() && !(keyExchangeModes & OpalMediaCryptoSuite::e_AllowClear)) {
+        PTRACE(4, "Skipping " << cryptoSuiteName << " as secure media required.");
         continue;
+      }
+      if (cryptoSuiteName != OpalMediaCryptoSuite::ClearText() && keyExchangeModes == OpalMediaCryptoSuite::e_AllowClear) {
+        PTRACE(4, "Skipping " << cryptoSuiteName << " as non-secure media required.");
+        continue;
+      }
 
       OpalMediaCryptoSuite * cryptoSuite = OpalMediaCryptoSuiteFactory::CreateInstance(cryptoSuiteName);
       if (cryptoSuite == NULL) {
-        PTRACE(1, "RTPCon\tCannot create crypto suite for " << cryptoSuiteName);
+        PTRACE(1, "Cannot create crypto suite for " << cryptoSuiteName);
         continue;
       }
 
       PCaselessString sessionType = mediaType->GetMediaSessionType();
-      if (!cryptoSuite->ChangeSessionType(sessionType)) {
-        PTRACE(3, "RTPCon\tCannot use crypto suite " << cryptoSuiteName << " with media type " << mediaType);
+      if (!cryptoSuite->ChangeSessionType(sessionType, keyExchangeModes)) {
+        PTRACE(3, "Cannot use crypto suite " << cryptoSuiteName << " with media type " << mediaType);
         continue;
       }
 
@@ -181,6 +182,8 @@ vector<bool> OpalRTPConnection::CreateAllMediaSessions(CreateMediaSessionsSecuri
       for (SessionMap::const_iterator it = m_sessions.begin(); it != m_sessions.end(); ++it) {
         if (it->second->GetMediaType() == mediaType && it->second->GetSessionType() == sessionType) {
           session = it->second;
+          PTRACE_IF(4, !openedMediaSessions[session->GetSessionID()],
+                    "Session " << session->GetSessionID() << " for " << mediaType << " and " << sessionType << " already created.");
           break;
         }
         if (nextSessionId <= it->first)
@@ -188,14 +191,12 @@ vector<bool> OpalRTPConnection::CreateAllMediaSessions(CreateMediaSessionsSecuri
       }
 
       if (session == NULL && (session = UseMediaSession(nextSessionId, mediaType, sessionType)) == NULL) {
-        PTRACE(1, "RTPCon\tCould not create RTP session for media type " << mediaType);
+        PTRACE(1, "Could not create RTP session for media type " << mediaType);
         continue;
       }
 
       session->OfferCryptoSuite(cryptoSuiteName);
 
-      CheckForMediaBypass(*session);
-
       openedMediaSessions[session->GetSessionID()] = true;
     }
   }
@@ -211,6 +212,16 @@ OpalMediaSession * OpalRTPConnection::GetMediaSession(unsigned sessionID) const
 }
 
 
+OpalMediaSession * OpalRTPConnection::FindSessionByMediaType(const OpalMediaType & mediaType) const
+{
+  for (SessionMap::const_iterator it = m_sessions.begin(); it != m_sessions.end(); ++it) {
+    if (it->second->GetMediaType() == mediaType)
+      return &*it->second;
+  }
+  return NULL;
+}
+
+
 OpalMediaSession * OpalRTPConnection::FindSessionByLocalPort(WORD port) const
 {
   for (SessionMap::const_iterator it = m_sessions.begin(); it != m_sessions.end(); ++it) {
@@ -229,7 +240,7 @@ OpalMediaSession * OpalRTPConnection::UseMediaSession(unsigned sessionId,
 {
   SessionMap::iterator it = m_sessions.find(sessionId);
   if (it != m_sessions.end()) {
-    PTRACE(4, "RTPCon\tFound existing " << mediaType << " session " << sessionId);
+    PTRACE(4, "Found existing " << it->second->GetMediaType() << " session " << sessionId);
     return it->second;
   }
 
@@ -245,62 +256,125 @@ OpalMediaSession * OpalRTPConnection::CreateMediaSession(unsigned sessionId,
                                                          const OpalMediaType & mediaType,
                                                          const PString & sessionType)
 {
+  OpalMediaSession::Init init(*this, sessionId, mediaType, m_remoteBehindNAT);
+
+  {
+    OpalTransportAddressArray transports;
+    PSafePtr<OpalConnection> otherConnection = GetOtherPartyConnection();
+    if (otherConnection != NULL && otherConnection->GetMediaTransportAddresses(*this, sessionId, mediaType, transports)) {
+      // Make sure we do not include any transcoded format combinations
+      m_localMediaFormats.Remove(PString('@')+mediaType);
+      m_localMediaFormats += otherConnection->GetMediaFormats();
+      PTRACE(4, "Created dummy " << mediaType << " session " << sessionId
+             << " using formats " << setfill(',') << m_localMediaFormats << " on " << *this);
+      return new OpalDummySession(init, transports);
+    }
+  }
+
   PString actualSessionType = sessionType;
   if (actualSessionType.IsEmpty())
     actualSessionType = mediaType->GetMediaSessionType();
 
-  OpalMediaSession * session = OpalMediaSessionFactory::CreateInstance(actualSessionType,
-                                     OpalMediaSession::Init(*this, sessionId, mediaType, m_remoteBehindNAT));
+  OpalMediaSession * session = OpalMediaSessionFactory::CreateInstance(actualSessionType, init);
   if (session == NULL) {
-    PTRACE(1, "RTPCon\tCannot create session for " << actualSessionType);
+    PTRACE(1, "Cannot create session for " << actualSessionType << " on " << *this);
     return NULL;
   }
 
-  PTRACE(4, "RTPCon\tCreated " << mediaType << " session " << sessionId << " using " << actualSessionType);
-  CheckForMediaBypass(*session);
+  session->SetStringOptions(GetStringOptions());
 
+  PTRACE(4, "Created " << mediaType << " session " << sessionId << " using " << actualSessionType << " on " << *this);
   return session;
 }
 
 
-bool OpalRTPConnection::GetMediaTransportAddresses(const OpalMediaType & mediaType,
-                                             OpalTransportAddressArray & transports) const
+bool OpalRTPConnection::GetMediaTransportAddresses(OpalConnection & otherConnection,
+                                                         unsigned   sessionId,
+                                              const OpalMediaType & mediaType,
+                                        OpalTransportAddressArray & transports) const
 {
-  for (SessionMap::const_iterator session = m_sessions.begin(); session != m_sessions.end(); ++session) {
-    if (session->second->GetMediaType() == mediaType) {
-      OpalTransportAddress address = session->second->GetRemoteAddress();
-      if (!address.IsEmpty()) {
-        transports.AppendAddress(address);
-        PTRACE(3, "RTPCon\tGetMediaTransport for " << mediaType << " found " << setfill(',') << address);
-        return true;
+  if (!OpalConnection::GetMediaTransportAddresses(otherConnection, sessionId, mediaType, transports))
+    return false;
+
+  if (!transports.IsEmpty())
+    return true;
+
+  SessionMap::const_iterator session = m_sessions.find(sessionId);
+  if (session == m_sessions.end() || session->second->GetMediaType() != mediaType) {
+    for (session = m_sessions.begin(); session != m_sessions.end(); ++session) {
+      if (session->second->GetMediaType() == mediaType)
+        break;
+    }
+    if (session == m_sessions.end()) {
+      PTRACE(4, "GetMediaTransportAddresses of " << mediaType << " had no session for " << otherConnection << " on " << *this);
+      return true;
+    }
+  }
+
+  if (transports.SetAddressPair(session->second->GetRemoteAddress(true), session->second->GetRemoteAddress(false)))
+    PTRACE(3, "GetMediaTransportAddresses of " << mediaType << " found session addresses "
+           << setfill(',') << transports << " for " << otherConnection << " on " << *this);
+  else
+    PTRACE(4, "GetMediaTransportAddresses of " << mediaType << " had no transports in sessions for " << otherConnection << " on " << *this);
+
+  return true;
+}
+
+
+#if OPAL_RTP_FEC
+OpalMediaFormatList OpalRTPConnection::NegotiateFECMediaFormats(OpalMediaSession & session)
+{
+  OpalMediaFormatList fecFormats;
+
+  OpalRTPSession * rtpSession = dynamic_cast<OpalRTPSession *>(&session);
+  if (rtpSession != NULL) {
+    OpalMediaFormatList remoteFormats = GetMediaFormats();
+
+    for (OpalMediaFormatList::iterator it = m_localMediaFormats.begin(); it != m_localMediaFormats.end(); ++it) {
+      if (it->GetMediaType() == OpalFEC::MediaType() && it->GetOptionString(OpalFEC::MediaTypeOption()) == session.GetMediaType()) {
+        OpalMediaFormatList::const_iterator fmt = remoteFormats.FindFormat(*it);
+        if (fmt != remoteFormats.end()) {
+          fecFormats += *fmt;
+
+          if (fmt->GetName().NumCompare(OPAL_REDUNDANT_PREFIX) == EqualTo)
+            rtpSession->SetRedundencyPayloadType(fmt->GetPayloadType());
+          else if (fmt->GetName().NumCompare(OPAL_ULP_FEC_PREFIX) == EqualTo)
+            rtpSession->SetUlpFecPayloadType(fmt->GetPayloadType());
+
+          PTRACE(4, "Accepted redundant format " << *fmt << ", pt=" << fmt->GetPayloadType());
+        }
       }
     }
   }
 
-  return OpalConnection::GetMediaTransportAddresses(mediaType, transports);
+  return fecFormats;
 }
+#endif // OPAL_RTP_FEC
 
 
 bool OpalRTPConnection::ChangeSessionID(unsigned fromSessionID, unsigned toSessionID)
 {
-  if (m_sessions.find(toSessionID) != m_sessions.end()) {
-    PTRACE(2, "RTPCon\tAttempt to renumber session " << fromSessionID << " to existing session ID " << toSessionID);
+  SessionMap::iterator from = m_sessions.find(fromSessionID);
+  if (from == m_sessions.end()) {
+    PTRACE(2, "Attempt to renumber unknown session " << fromSessionID << " to session ID " << toSessionID);
     return false;
   }
 
-  SessionMap::iterator it = m_sessions.find(fromSessionID);
-  if (it == m_sessions.end()) {
-    PTRACE(2, "RTPCon\tAttempt to renumber unknown session " << fromSessionID << " to session ID " << toSessionID);
-    return false;
+  SessionMap::iterator to = m_sessions.find(toSessionID);
+  if (to != m_sessions.end()) {
+    PTRACE(2, "Attempt to renumber session " << fromSessionID << " to existing session ID " << toSessionID);
+
+    m_sessions.erase(from);
   }
+  else {
+    PTRACE(3, "Changing session ID " << fromSessionID << " to " << toSessionID);
 
-  PTRACE(3, "RTPCon\tChanging session ID " << fromSessionID << " to " << toSessionID);
-
-  PSafePtr<OpalMediaSession> session = it->second;
-  m_sessions.DisallowDeleteObjects();
-  m_sessions.erase(it);
-  m_sessions.AllowDeleteObjects();
-  m_sessions.SetAt(toSessionID, session);
+    PSafePtr<OpalMediaSession> session = from->second;
+    m_sessions.DisallowDeleteObjects();
+    m_sessions.erase(from);
+    m_sessions.AllowDeleteObjects();
+    m_sessions.SetAt(toSessionID, session);
+  }
 
   for (OpalMediaStreamPtr stream(mediaStreams, PSafeReference); stream != NULL; ++stream) {
     if (stream->GetSessionID() == fromSessionID) {
@@ -324,7 +398,7 @@ void OpalRTPConnection::ReplaceMediaSession(unsigned sessionId, OpalMediaSession
   {
     SessionMap::iterator it = m_sessions.find(sessionId);
     if (it == m_sessions.end()) {
-      PTRACE(4, "RTPCon\tReplacing empty session " << sessionId);
+      PTRACE(4, "Replacing empty session " << sessionId);
       m_sessions.SetAt(sessionId, mediaSession);
       return;
     }
@@ -332,27 +406,25 @@ void OpalRTPConnection::ReplaceMediaSession(unsigned sessionId, OpalMediaSession
     OpalTransportAddress remoteMedia = it->second->GetRemoteAddress(true);
     OpalTransportAddress remoteCtrl = it->second->GetRemoteAddress(false);
 
-    OpalMediaSession::Transport transport = it->second->DetachTransport();
-    mediaSession->AttachTransport(transport);
+    mediaSession->AttachTransport(it->second->DetachTransport());
 
     mediaSession->SetRemoteAddress(remoteMedia, true);
     mediaSession->SetRemoteAddress(remoteCtrl, false);
-    PTRACE(4, "RTPCon\tReplacing session " << sessionId << " media=" << remoteMedia << " ctrl=" << remoteCtrl);
+    PTRACE(4, "Replacing session " << sessionId << " media=" << remoteMedia << " ctrl=" << remoteCtrl);
+  }
+
+  OpalRTPSession * rtpSession = dynamic_cast<OpalRTPSession *>(mediaSession);
+  if (rtpSession != NULL && rtpSession->IsAudio()) {
+    if (m_rfc2833Handler != NULL)
+      m_rfc2833Handler->UseRTPSession(false, rtpSession);
+
+#if OPAL_T38_CAPABILITY
+    if (m_ciscoNSEHandler != NULL)
+      m_ciscoNSEHandler->UseRTPSession(false, rtpSession);
+#endif
   }
 
   m_sessions.SetAt(sessionId, mediaSession);
-
-  OpalRTPSession * rtpSession = dynamic_cast<OpalRTPSession *>(mediaSession);
-  if (rtpSession == NULL || !rtpSession->IsAudio())
-    return;
-
-  if (m_rfc2833Handler != NULL)
-    m_rfc2833Handler->UseRTPSession(false, rtpSession);
-
-#if OPAL_T38_CAPABILITY
-  if (m_ciscoNSEHandler != NULL)
-    m_ciscoNSEHandler->UseRTPSession(false, rtpSession);
-#endif
 }
 
 
@@ -360,10 +432,12 @@ void OpalRTPConnection::ReleaseMediaSession(unsigned sessionID)
 {
   SessionMap::iterator it = m_sessions.find(sessionID);
   if (it == m_sessions.end()) {
-    PTRACE(2, "RTPCon\tAttempt to release unknown session " << sessionID);
+    PTRACE(2, "Attempt to release unknown session " << sessionID);
     return;
   }
 
+  it->second->Close();
+
   OpalRTPSession * rtpSession = dynamic_cast<OpalRTPSession *>(&*it->second);
   if (rtpSession != NULL && rtpSession->IsAudio()) {
     if (m_rfc2833Handler != NULL)
@@ -379,22 +453,54 @@ void OpalRTPConnection::ReleaseMediaSession(unsigned sessionID)
 }
 
 
+#if OPAL_VIDEO
+void OpalRTPConnection::AddAudioVideoGroup(const PString & id)
+{
+  for (SessionMap::iterator it = m_sessions.begin(); it != m_sessions.end(); ++it) {
+    if (it->second->GetMediaType() == OpalMediaType::Audio() || it->second->GetMediaType() == OpalMediaType::Video())
+      it->second->AddGroup(id, it->second->GetMediaType(), false);
+  }
+}
+#endif // OPAL_VIDEO
+
+
 bool OpalRTPConnection::SetSessionQoS(OpalRTPSession * /*session*/)
 {
   return true;
 }
 
 
-void OpalRTPConnection::DetermineRTPNAT(const PIPSocket::Address & localAddr, 
-                                        const PIPSocket::Address & peerAddr,
-                                        const PIPSocket::Address & sigAddr)
+void OpalRTPConnection::DetermineRTPNAT(const OpalTransport & transport, const OpalTransportAddress & signalAddr)
 {
-  m_remoteBehindNAT = dynamic_cast<OpalRTPEndPoint &>(endpoint).IsRTPNATEnabled(*this, localAddr, peerAddr, sigAddr, !IsOriginating());
+  if (m_remoteBehindNAT) {
+    PTRACE(4, "Already determined is behind NAT");
+    return;
+  }
+
+  PIPSocket::Address localAddr, peerAddr, sigAddr;
+
+  transport.GetLocalAddress().GetIpAddress(localAddr);
+  transport.GetRemoteAddress().GetIpAddress(peerAddr);
+  signalAddr.GetIpAddress(sigAddr);
+
+  if (dynamic_cast<OpalRTPEndPoint &>(endpoint).IsRTPNATEnabled(*this, localAddr, peerAddr, sigAddr, !IsOriginating())) {
+    PTRACE(4, "Determined is behind NAT");
+    m_remoteBehindNAT = true;
+    for (SessionMap::const_iterator it = m_sessions.begin(); it != m_sessions.end(); ++it)
+      it->second->SetRemoteBehindNAT();
+  }
 }
 
 
 bool OpalRTPConnection::OnMediaCommand(OpalMediaStream & stream, const OpalMediaCommand & command)
 {
+  if (stream.IsSource() != (&stream.GetConnection() == this)) {
+    PTRACE(5, "OnMediaCommand \"" << command << "\" on " << stream << " not processed for " << *this);
+    return OpalConnection::OnMediaCommand(stream, command);
+  }
+
+  PTRACE(5, "OnMediaCommand \"" << command << "\" on " << stream << " processed for " << *this);
+
   unsigned sessionID = stream.GetSessionID();
   OpalRTPSession * session = dynamic_cast<OpalRTPSession *>(GetMediaSession(sessionID));
   if (session == NULL)
@@ -402,62 +508,24 @@ bool OpalRTPConnection::OnMediaCommand(OpalMediaStream & stream, const OpalMedia
 
 #if OPAL_VIDEO
 
-  OpalVideoFormat::RTCPFeedback rtcp_fb = stream.GetMediaFormat().GetOptionEnum(OpalVideoFormat::RTCPFeedbackOption(),
-                                                                                OpalVideoFormat::e_NoRTCPFb);
-  if (rtcp_fb == OpalVideoFormat::e_NoRTCPFb) {
-    OpalMediaStreamPtr source = GetMediaStream(sessionID, true);
-    if (source != NULL)
-      rtcp_fb = source->GetMediaFormat().GetOptionEnum(OpalVideoFormat::RTCPFeedbackOption(), OpalVideoFormat::e_NoRTCPFb);
-  }
-
   const OpalMediaFlowControl * flow = dynamic_cast<const OpalMediaFlowControl *>(&command);
-  if (flow != NULL) {
-    if (rtcp_fb & OpalVideoFormat::e_TMMBR) {
-      session->SendFlowControl(flow->GetMaxBitRate());
-      return true;
-    }
-    PTRACE(3, "RTPCon\tRemote not capable of flow control (TMMBR)");
-    return OpalConnection::OnMediaCommand(stream, command);
-  }
+  if (flow != NULL)
+    return session->SendFlowControl(flow->GetMaxBitRate());
 
   const OpalTemporalSpatialTradeOff * tsto = dynamic_cast<const OpalTemporalSpatialTradeOff *>(&command);
-  if (tsto != NULL) {
-    if (rtcp_fb & OpalVideoFormat::e_TSTR) {
-      session->SendTemporalSpatialTradeOff(tsto->GetTradeOff());
-      return true;
+  if (tsto != NULL)
+    return session->SendTemporalSpatialTradeOff(tsto->GetTradeOff());
+
+  const OpalVideoUpdatePicture * vup = dynamic_cast<const OpalVideoUpdatePicture *>(&command);
+  if (vup != NULL) {
+    unsigned options = m_stringOptions.GetInteger(OPAL_OPT_VIDUP_METHODS, OPAL_OPT_VIDUP_METHOD_DEFAULT);
+    if ((options&(OPAL_OPT_VIDUP_METHOD_RTCP | OPAL_OPT_VIDUP_METHOD_PLI | OPAL_OPT_VIDUP_METHOD_FIR)) != 0) {
+      if (PIsDescendant(&command, OpalVideoPictureLoss))
+        options |= OPAL_OPT_VIDUP_METHOD_PREFER_PLI;
+      return session->SendIntraFrameRequest(options, vup->GetSyncSource());
     }
-    PTRACE(3, "RTPCon\tRemote not capable of Temporal/Spatial Tradeoff (TSTR)");
-    return OpalConnection::OnMediaCommand(stream, command);
-  }
 
-  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    unsigned mask = m_stringOptions.GetInteger(OPAL_OPT_VIDUP_METHODS, OPAL_OPT_VIDUP_METHOD_DEFAULT);
-    if ((mask&(OPAL_OPT_VIDUP_METHOD_RTCP|OPAL_OPT_VIDUP_METHOD_PLI|OPAL_OPT_VIDUP_METHOD_FIR)) != 0) {
-      if (m_rtcpIntraFrameRequestTimer.IsRunning()) {
-        PTRACE(4, "RTPCon\tRecent RTCP FIR was sent, not sending another");
-        return true;
-      }
-
-      bool has_AVPF_PLI = (rtcp_fb & OpalVideoFormat::e_PLI) || (mask & OPAL_OPT_VIDUP_METHOD_PLI);
-      bool has_AVPF_FIR = (rtcp_fb & OpalVideoFormat::e_FIR) || (mask & OPAL_OPT_VIDUP_METHOD_FIR);
-
-      if (has_AVPF_PLI && has_AVPF_FIR)
-        session->SendIntraFrameRequest(false, PIsDescendant(&command, OpalVideoPictureLoss));
-      else if (has_AVPF_PLI)
-        session->SendIntraFrameRequest(false, true);  // More common, use RFC4585 PLI
-      else if (has_AVPF_FIR)
-        session->SendIntraFrameRequest(false, false); // Unusual, but possible, use RFC5104 FIR
-      else
-        session->SendIntraFrameRequest(true, false);  // Fall back to RFC2032
-
-      m_rtcpIntraFrameRequestTimer.SetInterval(0, 1);
-
-#if OPAL_STATISTICS
-      m_VideoUpdateRequestsSent++;
-#endif
-
-      return true;
-    }
+    PTRACE(4, "RTCP Intra-Frame Request disabled in string options");
   }
 #endif // OPAL_VIDEO
 
@@ -475,7 +543,7 @@ PBoolean OpalRTPConnection::SendUserInputTone(char tone, unsigned duration)
          m_rfc2833Handler->SendToneAsync(tone, duration))
       return true;
 
-    PTRACE(2, "RTPCon\tCould not send tone '" << tone << "' via RFC2833.");
+    PTRACE(2, "Could not send tone '" << tone << "' via RFC2833.");
   }
 
   return OpalConnection::SendUserInputTone(tone, duration);
@@ -490,35 +558,11 @@ OpalMediaStream * OpalRTPConnection::CreateMediaStream(const OpalMediaFormat & m
   if (mediaSession != NULL)
     return mediaSession->CreateMediaStream(mediaFormat, sessionID, isSource);
 
-  PTRACE(1, "RTPCon\tUnable to create media stream for session " << sessionID);
+  PTRACE(1, "Unable to create media stream for session " << sessionID);
   return NULL;
 }
 
 
-void OpalRTPConnection::CheckForMediaBypass(OpalMediaSession & session)
-{
-  if (session.IsExternalTransport())
-    return;
-
-  PSafePtr<OpalConnection> otherConnection = GetOtherPartyConnection();
-  if (otherConnection == NULL)
-    return;
-
-  const OpalMediaType & mediaType = session.GetMediaType();
-  if (endpoint.GetManager().GetMediaTransferMode(*this, *otherConnection, mediaType) != OpalManager::MediaTransferBypass)
-    return;
-
-  OpalTransportAddressArray transports;
-  if (!otherConnection->GetMediaTransportAddresses(mediaType, transports))
-    return;
-
-  // Make sure we do not include any transcoded format combinations
-  m_localMediaFormats.Remove(PString('@')+mediaType);
-  m_localMediaFormats += otherConnection->GetMediaFormats();
-  session.SetExternalTransport(transports);
-}
-
-
 void OpalRTPConnection::AdjustMediaFormats(bool   local,
                            const OpalConnection * otherConnection,
                             OpalMediaFormatList & mediaFormats) const
@@ -531,6 +575,16 @@ void OpalRTPConnection::AdjustMediaFormats(bool   local,
       else
         mediaFormats -= *fmt++;
     }
+#if OPAL_RTP_FEC
+    if (!mediaFormats.IsEmpty()) {
+      mediaFormats += OpalRedundantAudio;
+      mediaFormats += OpalUlpFecAudio;
+ #if OPAL_VIDEO
+      mediaFormats += OpalRedundantVideo;
+      mediaFormats += OpalUlpFecVideo;
+ #endif
+    }
+#endif
   }
 
   OpalConnection::AdjustMediaFormats(local, otherConnection, mediaFormats);
@@ -557,15 +611,13 @@ void OpalRTPConnection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & p
 #endif
 }
 
-void OpalRTPConnection::OnUserInputInlineRFC2833(OpalRFC2833Info & info, P_INT_PTR type)
+void OpalRTPConnection::OnUserInputInlineRFC2833(OpalRFC2833Info & info, P_INT_PTR)
 {
-  // trigger on start of tone only
-  if (type == 0)
-    GetEndPoint().GetManager().QueueDecoupledEvent(
+  GetEndPoint().GetManager().QueueDecoupledEvent(
           new PSafeWorkArg2<OpalConnection, char, unsigned>(
-                this, info.GetTone(),
-                info.GetDuration() > 0 ? info.GetDuration()/8 : 100,
-                &OpalConnection::OnUserInputTone));
+                 this, info.GetTone(),
+                 info.GetDuration(),
+                 &OpalConnection::OnUserInputTone));
 }
 
 
@@ -575,17 +627,4 @@ bool OpalRTPConnection::GarbageCollection()
 }
 
 
-void OpalRTPConnection::SessionMap::MoveFrom(SessionMap & other)
-{
-  *this = other;
-  AllowDeleteObjects(); // We now own the objects, need to put this back on
-
-  // Remove from other without deleting them
-  bool del = other.deleteObjects;
-  other.DisallowDeleteObjects();
-  other.RemoveAll();
-  other.AllowDeleteObjects(del);
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
