@@ -47,7 +47,7 @@
 
 
 #include <ptlib.h>
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #if OPAL_SIP_PRESENCE
 
@@ -58,7 +58,7 @@
 
 
 PFACTORY_CREATE(PFactory<OpalPresentity>, SIP_Presentity, "sip", false);
-static bool Synonym_for_pres_URL = PFactory<OpalPresentity>::RegisterAs("pres", "sip");
+PFACTORY_SYNONYM(PFactory<OpalPresentity>, SIP_Presentity, pres, "pres");
 
 const PCaselessString & SIP_Presentity::PIDFEntityKey()    { static const PConstCaselessString s("PIDF-Entity");    return s; }
 const PCaselessString & SIP_Presentity::SubProtocolKey()   { static const PConstCaselessString s("Sub-Protocol");   return s; }
@@ -90,13 +90,15 @@ static struct SIPAttributeInfo {
   /* 12 */ { OpalPresentity::TimeToLiveKey(),    "Integer\n1,999999999\n300" }
 };
 
-OPAL_DEFINE_COMMAND(OpalSetLocalPresenceCommand,     SIP_Presentity, Internal_SendLocalPresence);
-OPAL_DEFINE_COMMAND(OpalSubscribeToPresenceCommand,  SIP_Presentity, Internal_SubscribeToPresence);
-OPAL_DEFINE_COMMAND(SIPWatcherInfoCommand,           SIP_Presentity, Internal_SubscribeToWatcherInfo);
-OPAL_DEFINE_COMMAND(OpalAuthorisationRequestCommand, SIP_Presentity, Internal_AuthorisationRequest);
+OPAL_PRESENTITY_COMMAND(OpalSetLocalPresenceCommand,     SIP_Presentity, Internal_SendLocalPresence);
+OPAL_PRESENTITY_COMMAND(OpalSubscribeToPresenceCommand,  SIP_Presentity, Internal_SubscribeToPresence);
+OPAL_PRESENTITY_COMMAND(SIPWatcherInfoCommand,           SIP_Presentity, Internal_SubscribeToWatcherInfo);
+OPAL_PRESENTITY_COMMAND(OpalAuthorisationRequestCommand, SIP_Presentity, Internal_AuthorisationRequest);
 
 static const char * const AuthNames[OpalPresentity::NumAuthorisations] = { "allow", "block", "polite-block", "confirm", "remove" };
 
+static atomic<unsigned> NextRuleId(PRandom::Number());
+
 
 //////////////////////////////////////////////////////////////////////////////////////
 
@@ -254,9 +256,6 @@ bool SIP_Presentity::Close()
     m_endpoint->Unsubscribe(SIPSubscribe::Presence, subs->second, true);
   }
 
-  if (!m_publishedTupleId.IsEmpty() && m_subProtocol != e_PeerToPeer)
-    m_endpoint->Publish(m_aor.AsString(), PString::Empty(), 0);
-
   PTRACE(4, "SIPPres\t'" << m_aor << "' awaiting unsubscriptions to complete.");
   while (m_endpoint->IsSubscribed(SIPSubscribe::Presence | SIPSubscribe::Watcher, watcherSubscriptionAOR, true))
     PThread::Sleep(100);
@@ -556,24 +555,29 @@ void SIP_Presentity::OnReceivedWatcherStatus(PXMLElement * watcher)
 }
 
 
+static atomic<uint32_t> g_idNumber;
+
 void SIP_Presentity::Internal_SendLocalPresence(const OpalSetLocalPresenceCommand & cmd)
 {
   PTRACE(3, "SIPPres\t'" << m_aor << "' sending own presence " << cmd.m_state << "/" << cmd.m_note);
 
   SIPPresenceInfo sipPresence(cmd);
 
-  static PAtomicInteger::IntegerType g_idNumber;
   sipPresence.m_personId = PString(++g_idNumber);
   SetPIDFEntity(sipPresence.m_entity);
   sipPresence.m_contact =  m_aor;  // As required by OMA-TS-Presence_SIMPLE-V2_0-20090917-C
   sipPresence.m_presenceAgent = m_presenceAgentURL.AsString();
 
-  if (m_publishedTupleId.IsEmpty())
-    m_publishedTupleId = PGloballyUniqueID().AsString();
-  sipPresence.m_service = m_publishedTupleId;
+  if (!sipPresence.m_service.IsEmpty())
+    m_publishedTupleId = sipPresence.m_service;
+  else {
+    if (m_publishedTupleId.IsEmpty())
+      m_publishedTupleId = PGloballyUniqueID().AsString();
+    sipPresence.m_service = m_publishedTupleId;
+  }
 
   if (m_subProtocol != e_PeerToPeer)
-    m_endpoint->PublishPresence(sipPresence, GetExpiryTime());
+    m_endpoint->PublishPresence(sipPresence, m_open ? GetExpiryTime() : 0);
   else
     m_endpoint->Notify(m_aor, SIPSubscribe::Presence, sipPresence.AsXML());
 }
@@ -756,7 +760,6 @@ void SIP_Presentity::Internal_AuthorisationRequest(const OpalAuthorisationReques
   }
 
   // Create new rule with id as per http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCName
-  static PAtomicInteger NextRuleId(PRandom::Number());
   PString newRuleId(PString::Printf, "wp_prs%s_one_%lu",
                     cmd.m_authorisation == AuthorisationPermitted ? "_allow" : "",
                     ++NextRuleId);
