@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 28250 $
+ * $Revision: 32719 $
  * $Author: rjongbloed $
- * $Date: 2012-08-26 20:02:15 -0500 (Sun, 26 Aug 2012) $
+ * $Date: 2014-09-09 08:22:51 +0100 (Tue, 09 Sep 2014) $
  */
 
 /*
@@ -55,32 +55,47 @@ Either, or both, can be used in a call
 #pragma implementation "sipt38.h"
 #endif
 
-#include <opal/buildopts.h>
+#include <opal_config.h>
 
 #include <t38/sipt38.h>
 
 
-#if OPAL_SIP
+#if OPAL_SDP
 #if OPAL_T38_CAPABILITY
 
 #include <t38/t38proto.h>
 
+static const char * const T38OptionsInit[] = {
+  OPAL_T38FaxRateManagement,
+  OPAL_T38FaxVersion,
+  OPAL_T38MaxBitRate,
+  OPAL_T38FaxMaxBuffer,
+  OPAL_T38FaxMaxDatagram,
+  OPAL_T38FaxUdpEC,
+  OPAL_T38FaxFillBitRemoval,
+  OPAL_T38FaxTranscodingMMR,
+  OPAL_T38FaxTranscodingJBIG
+};
+
+static PStringSet const T38Options(PARRAYSIZE(T38OptionsInit), T38OptionsInit);
+
 
 /////////////////////////////////////////////////////////////////////////////
 
-const PCaselessString & OpalFaxMediaType::GetSDPMediaType() { static PConstCaselessString const s("image"); return s; }
-const PString & OpalFaxMediaType::GetSDPTransportType() { return OpalFaxMediaType::UDPTL(); }
-
-bool OpalFaxMediaType::MatchesSDP(const PCaselessString & sdpMediaType,
-                                  const PCaselessString & sdpTransport,
-                                  const PStringArray & /*sdpLines*/,
-                                  PINDEX /*index*/)
+PString OpalFaxMediaDefinition::GetSDPMediaType() const
 {
-  return sdpMediaType == GetSDPMediaType() && sdpTransport == OpalFaxMediaType::UDPTL();
+  static PConstCaselessString const s("image");
+  return s;
 }
 
 
-SDPMediaDescription * OpalFaxMediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const
+PString OpalFaxMediaDefinition::GetSDPTransportType() const
+{
+  return OpalFaxMediaDefinition::UDPTL();
+}
+
+
+SDPMediaDescription * OpalFaxMediaDefinition::CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const
 {
   return new SDPFaxMediaDescription(localAddress);
 }
@@ -90,39 +105,32 @@ SDPMediaDescription * OpalFaxMediaType::CreateSDPMediaDescription(const OpalTran
 SDPFaxMediaDescription::SDPFaxMediaDescription(const OpalTransportAddress & address)
   : SDPMediaDescription(address, OpalMediaType::Fax())
 {
-  t38Attributes.SetAt("T38FaxRateManagement", "transferredTCF");
-  t38Attributes.SetAt("T38FaxVersion",        "0");
-  //t38Attributes.SetAt("T38MaxBitRate",        "9600");
-  //t38Attributes.SetAt("T38FaxMaxBuffer",      "200");
 }
 
-PCaselessString SDPFaxMediaDescription::GetSDPTransportType() const
-{ 
-  return OpalFaxMediaType::GetSDPTransportType();
-}
 
-PString SDPFaxMediaDescription::GetSDPMediaType() const 
-{ 
-  return OpalFaxMediaType::GetSDPMediaType(); 
-}
-
-SDPMediaFormat * SDPFaxMediaDescription::CreateSDPMediaFormat(const PString & portString)
+bool SDPFaxMediaDescription::Format::FromSDP(const PString & portString)
 {
   const OpalMediaFormat mediaFormat(RTP_DataFrame::DynamicBase, 0, portString, "sip");
   if (mediaFormat.IsEmpty()) {
     PTRACE(2, "SDPFax\tCould not find media format for " << portString);
-    return NULL;
+    return false;
   }
 
   PTRACE(3, "SDPFax\tUsing RTP payload " << mediaFormat.GetPayloadType() << " for " << portString);
+  FromMediaFormat(mediaFormat);
+  return true;
+}
 
-  return new SDPMediaFormat(*this, mediaFormat);
+
+SDPMediaFormat * SDPFaxMediaDescription::CreateSDPMediaFormat()
+{
+  return new Format(*this);
 }
 
 
 PString SDPFaxMediaDescription::GetSDPPortList() const
 {
-  if (formats.IsEmpty())
+  if (m_formats.IsEmpty())
     return OpalT38.GetEncodingName(); // Have to have SOMETHING
 
   return SDPMediaDescription::GetSDPPortList();
@@ -139,6 +147,7 @@ void SDPFaxMediaDescription::OutputAttributes(ostream & strm) const
     strm << "a=" << it->first << ":" << it->second << "\r\n";
 }
 
+
 void SDPFaxMediaDescription::SetAttribute(const PString & attr, const PString & value)
 {
   if (attr.Left(3) *= OpalT38.GetEncodingName()) {
@@ -149,16 +158,14 @@ void SDPFaxMediaDescription::SetAttribute(const PString & attr, const PString &
   return SDPMediaDescription::SetAttribute(attr, value);
 }
 
+
 void SDPFaxMediaDescription::ProcessMediaOptions(SDPMediaFormat & /*sdpFormat*/, const OpalMediaFormat & mediaFormat)
 {
   if (mediaFormat.GetMediaType() == OpalMediaType::Fax()) {
     for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); ++i) {
       const OpalMediaOption & option = mediaFormat.GetOption(i);
-      if (option.GetName().Left(3) *= OpalT38.GetEncodingName()) {
-        PString value = option.AsString();
-        if (value != option.GetFMTPDefault())
-          t38Attributes.SetAt(option.GetName(), option.AsString());
-      }
+      if (T38Options.Contains(option.GetName()))
+        t38Attributes.SetAt(option.GetName(), option.AsString());
     }
   }
 }
@@ -169,7 +176,7 @@ bool SDPFaxMediaDescription::PostDecode(const OpalMediaFormatList & mediaFormats
   if (!SDPMediaDescription::PostDecode(mediaFormats))
     return false;
 
-  for (SDPMediaFormatList::iterator format = formats.begin(); format != formats.end(); ++format) {
+  for (SDPMediaFormatList::iterator format = m_formats.begin(); format != m_formats.end(); ++format) {
     OpalMediaFormat & mediaFormat = format->GetWritableMediaFormat();
     if (mediaFormat.GetMediaType() == OpalMediaType::Fax()) {
       for (PINDEX i = 0; i < t38Attributes.GetSize(); ++i) {
@@ -188,5 +195,5 @@ bool SDPFaxMediaDescription::PostDecode(const OpalMediaFormatList & mediaFormats
 
 
 #endif // OPAL_T38_CAPABILITY
-#endif // OPAL_SIP
+#endif // OPAL_SDP
 
