@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 29743 $
+ * $Revision: 33766 $
  * $Author: rjongbloed $
- * $Date: 2013-05-19 22:43:02 -0500 (Sun, 19 May 2013) $
+ * $Date: 2015-07-27 21:27:47 +0100 (Mon, 27 Jul 2015) $
  */
 
 #include <ptlib.h>
@@ -38,10 +38,6 @@
 
 #if OPAL_FAX
 
-#ifdef _MSC_VER
-  #pragma message("T.38 Fax (spandsp) support enabled")
-#endif
-
 #include <asn/t38.h>
 #include <opal/patch.h>
 #include <codec/opalpluginmgr.h>
@@ -49,14 +45,14 @@
 
 /////////////////////////////////////////////////////////////////////////////
 
-OPAL_DEFINE_MEDIA_COMMAND(OpalFaxTerminate, PLUGINCODEC_CONTROL_TERMINATE_CODEC);
+OPAL_DEFINE_MEDIA_COMMAND(OpalFaxTerminate, PLUGINCODEC_CONTROL_TERMINATE_CODEC, OpalMediaType::Fax());
 
 #define new PNEW
 
 
 static const char TIFF_File_FormatName[] = OPAL_FAX_TIFF_FILE;
-const PCaselessString & OpalFaxSession::UDPTL() { return OpalFaxMediaType::UDPTL(); }
-static OpalMediaSessionFactory::Worker<OpalFaxSession> udptl_session(OpalFaxMediaType::UDPTL());
+const PCaselessString & OpalFaxSession::UDPTL() { return OpalFaxMediaDefinition::UDPTL(); }
+PFACTORY_CREATE(OpalMediaSessionFactory, OpalFaxSession, OpalFaxMediaDefinition::UDPTL());
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -124,12 +120,17 @@ void OpalFaxMediaStream::InternalClose()
 }
 
 
+void OpalFaxMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
+{
+  m_session.GetStatistics(statistics, IsSource());
+  OpalMediaStream::GetStatistics(statistics, fromPatch);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalFaxSession::OpalFaxSession(const Init & init)
   : OpalMediaSession(init)
-  , m_dataSocket(NULL)
-  , m_shuttingDown(false)
   , m_rawUDPTL(false)
   , m_datagramSize(528)
   , m_consecutiveBadPackets(0)
@@ -139,6 +140,11 @@ OpalFaxSession::OpalFaxSession(const Init & init)
   , m_secondaryPacket(-1)
   , m_optimiseOnRetransmit(false) // not optimise udptl packets on retransmit
   , m_sentPacket(new T38_UDPTLPacket)
+  , m_txBytes(0)
+  , m_txPackets(0)
+  , m_rxBytes(0)
+  , m_rxPackets(0)
+  , m_missingPackets(0)
 {
   m_timerWriteDataIdle.SetNotifier(PCREATE_NOTIFIER(OnWriteDataIdle));
 
@@ -152,8 +158,6 @@ OpalFaxSession::OpalFaxSession(const Init & init)
 OpalFaxSession::~OpalFaxSession()
 {
   m_timerWriteDataIdle.Stop();
-  if (m_dataSocket != NULL && m_savedTransport.GetObjectsIndex(m_dataSocket) == P_MAX_INDEX)
-    delete m_dataSocket;
   delete m_receivedPacket;
   delete m_sentPacket;
 }
@@ -170,13 +174,13 @@ void OpalFaxSession::ApplyMediaOptions(const OpalMediaFormat & mediaFormat)
 
     for (PINDEX i = 0 ; i < value.GetSize() ; i++) {
       PString size, redundancy;
-      if (value[i].Split(':', size, redundancy))
+      if (value[i].Split(':', size, redundancy, PString::SplitTrim|PString::SplitBeforeNonEmpty|PString::SplitAfterNonEmpty))
         m_redundancy[size.AsInteger()] = redundancy.AsInteger();
     }
 
 #if PTRACING
     if (PTrace::CanTrace(3)) {
-      ostream & trace = PTrace::Begin(3, __FILE__, __LINE__, this);
+      ostream & trace = PTRACE_BEGIN(3);
       trace << "UDPTL\tUse redundancy \"";
       for (std::map<int, int>::iterator it = m_redundancy.begin() ; it != m_redundancy.end() ; it++) {
         if (it != m_redundancy.begin())
@@ -225,33 +229,25 @@ void OpalFaxSession::ApplyMediaOptions(const OpalMediaFormat & mediaFormat)
 }
 
 
-bool OpalFaxSession::Open(const PString & localInterface,
-                          const OpalTransportAddress & remoteAddress,
-                          bool isMediaAddress)
+bool OpalFaxSession::Open(const PString & localInterface, const OpalTransportAddress & remoteAddress)
 {
-  m_shuttingDown = false;
-
   if (IsOpen())
     return true;
 
-  if (!isMediaAddress) {
-    PTRACE(2, "UDPTL\tDoes not have control transport.");
-    return false;
-  }
-
   // T.38 over TCP is half baked. One day someone might want it enough for it to be finished
   if (remoteAddress.IsEmpty() || remoteAddress.GetProto(true) == OpalTransportAddress::UdpPrefix())
-    m_dataSocket = new PUDPSocket();
+    m_transport = new OpalUDPMediaTransport("T.38-UDP");
   else
-    m_dataSocket = new PTCPSocket();
+    m_transport = new OpalTCPMediaTransport("T.38-TCP");
+  PTRACE_CONTEXT_ID_TO(m_transport);
 
   PIPSocket::Address localIP(localInterface);
-  if (!localIP.IsValid() || !m_dataSocket->Listen(localIP)) {
+  if (!localIP.IsValid() || !m_transport->Open(*this, 1, localIP, remoteAddress)) {
     PTRACE(2, "UDPTL\tCould listen on interface=\"" << localInterface << '"');
     return false;
   }
 
-  if (!remoteAddress.IsEmpty() && !m_dataSocket->Connect(remoteAddress.GetHostName(true))) {
+  if (!remoteAddress.IsEmpty() && !m_transport->SetRemoteAddress(remoteAddress.GetHostName(true))) {
     PTRACE(2, "UDPTL\tCould conect to " << remoteAddress);
     return false;
   }
@@ -263,115 +259,18 @@ bool OpalFaxSession::Open(const PString & localInterface,
 
 bool OpalFaxSession::IsOpen() const
 {
-  return m_dataSocket != NULL && m_dataSocket->IsOpen();
+  return m_transport != NULL && m_transport->IsOpen();
 }
 
 
 bool OpalFaxSession::Close()
 {
-  if (m_dataSocket == NULL)
+  if (m_transport == NULL)
     return false;
 
-  m_shuttingDown = true;
-
-  if (m_savedTransport.GetObjectsIndex(m_dataSocket) == P_MAX_INDEX)
-    return m_dataSocket->Close();
-
-  PUDPSocket *udp = dynamic_cast<PUDPSocket *>(m_dataSocket);
-  if (udp != NULL) {
-    PTRACE(4, "UDPTL\tBreaking UDPTL session read block.");
-    PIPSocketAddressAndPort addrPort;
-    udp->GetLocalAddress(addrPort);
-    udp->WriteTo("", 1, addrPort);
-  }
-
-  return true;
-}
-
-
-OpalTransportAddress OpalFaxSession::GetLocalAddress(bool isMediaAddress) const
-{
-  OpalTransportAddress address;
-
-  if (isMediaAddress && m_dataSocket != NULL) {
-    PIPSocket::Address ip;
-    WORD port;
-    if (m_dataSocket->GetLocalAddress(ip, port))
-      address = OpalTransportAddress(ip, port,
-                  dynamic_cast<PUDPSocket *>(m_dataSocket) != NULL ? "udp" : "tcp");
-  }
-
-  return address;
-}
-
-
-OpalTransportAddress OpalFaxSession::GetRemoteAddress(bool isMediaAddress) const
-{
-  PIPSocket * socket = NULL;
-
-  if (isMediaAddress)
-    socket = m_dataSocket;
-  else if (m_savedTransport.GetSize() > 1)
-    socket = dynamic_cast<PIPSocket *>(&m_savedTransport.back());
-
-  if (socket == NULL)
-    return OpalMediaSession::GetRemoteAddress(isMediaAddress);
-
-  PIPSocket::Address ip;
-  WORD port;
-  const char * proto;
-
-  PUDPSocket * udp = dynamic_cast<PUDPSocket *>(socket);
-  if (udp != NULL) {
-    udp->GetSendAddress(ip, port);
-    proto = "udp";
-  }
-  else {
-    socket->GetPeerAddress(ip, port);
-    proto = "tcp";
-  }
-
-  return OpalTransportAddress(ip, port, proto);
-}
-
-
-bool OpalFaxSession::SetRemoteAddress(const OpalTransportAddress & remoteAddress, bool isMediaAddress)
-{
-  if (!isMediaAddress || m_dataSocket == NULL)
-    return false;
-
-  PIPSocketAddressAndPort addrPort;
-  if (!remoteAddress.GetIpAndPort(addrPort))
-    return false;
-
-  PUDPSocket *udp = dynamic_cast<PUDPSocket *>(m_dataSocket);
-  if (udp == NULL)
-    return false;
-
-  udp->SetSendAddress(addrPort);
-  return true;
-}
-
-
-void OpalFaxSession::AttachTransport(Transport & transport)
-{
-  if (transport.IsEmpty()) {
-    PTRACE(2, "UDPTL\tTried to attach empty transport list.");
-    return;
-  }
-
-  m_savedTransport = transport;
-  m_dataSocket = dynamic_cast<PIPSocket *>(&transport.front());
-}
-
-
-OpalMediaSession::Transport OpalFaxSession::DetachTransport()
-{
-  Transport temp = m_savedTransport;
-  m_savedTransport = Transport(); // Detaches reference, do not use RemoveAll()
-  m_dataSocket = NULL;
-  PTRACE_IF(2, temp.IsEmpty(), "UDPTL\tDetaching transport from closed session.");
-  return temp;
+  m_transport->RemoveReadNotifier(this);
+  m_readQueue.Close(false);
+  return OpalMediaSession::Close();
 }
 
 
@@ -383,12 +282,12 @@ OpalMediaStream * OpalFaxSession::CreateMediaStream(const OpalMediaFormat & medi
 
 bool OpalFaxSession::WriteData(RTP_DataFrame & frame)
 {
-  if (m_dataSocket == NULL)
+  if (m_transport == NULL)
     return false;
 
   if (m_rawUDPTL) {
-    PTRACE(4, "UDPTL\tSending raw UDPTL of size " << frame.GetPayloadSize());
-    return m_dataSocket->Write(frame.GetPayloadPtr(), frame.GetPayloadSize());
+    PTRACE(5, "UDPTL\tSending raw UDPTL, size=" << frame.GetPayloadSize());
+    return m_transport->Write(frame.GetPayloadPtr(), frame.GetPayloadSize());
   }
 
   PINDEX plLen = frame.GetPayloadSize();
@@ -480,7 +379,7 @@ void OpalFaxSession::DecrementSentPacketRedundancy(bool stripRedundancy)
   m_sentPacketRedundancy.resize(iMax + 1);
 
   if (stripRedundancy) {
-    T38_UDPTLPacket_error_recovery &recovery = m_sentPacket->m_error_recovery;
+    T38_UDPTLPacket_error_recovery & recovery = m_sentPacket->m_error_recovery;
 
     if (recovery.GetTag() == T38_UDPTLPacket_error_recovery::e_secondary_ifp_packets) {
       T38_UDPTLPacket_error_recovery_secondary_ifp_packets &secondary = recovery;
@@ -494,50 +393,55 @@ void OpalFaxSession::DecrementSentPacketRedundancy(bool stripRedundancy)
 
 bool OpalFaxSession::WriteUDPTL()
 {
-  if (m_dataSocket == NULL || !m_dataSocket->IsOpen()) {
+  if (m_transport == NULL || !m_transport->IsOpen()) {
     PTRACE(2, "UDPTL\tWrite failed, data socket not open");
     return false;
   }
 
-  PTRACE(5, "UDPTL\tEncoded transmitted UDPTL data :\n  " << setprecision(2) << *m_sentPacket);
-
   PPER_Stream rawData;
   m_sentPacket->Encode(rawData);
   rawData.CompleteEncoding();
 
-  PTRACE(4, "UDPTL\tSending UDPTL of size " << rawData.GetSize());
+  PTRACE(5, "UDPTL\tEncoded transmitted UDPTL data, size=" << rawData.GetSize() << " :\n  " << setprecision(2) << *m_sentPacket);
 
-  return m_dataSocket->Write(rawData.GetPointer(), rawData.GetSize());
+  m_txBytes += rawData.GetSize();
+  ++m_txPackets;
+
+  return m_transport->Write(rawData.GetPointer(), rawData.GetSize());
 }
 
 
 void OpalFaxSession::SetFrameFromIFP(RTP_DataFrame & frame, const PASN_OctetString & ifp, unsigned sequenceNumber)
 {
-  frame.SetPayloadSize(ifp.GetDataLength());
-  memcpy(frame.GetPayloadPtr(), (const BYTE *)ifp, ifp.GetDataLength());
+  frame.SetPayload(ifp, ifp.GetDataLength());
   frame.SetSequenceNumber((WORD)(sequenceNumber & 0xffff));
   if (m_secondaryPacket <= 0)
     m_expectedSequenceNumber = sequenceNumber+1;
 }
 
 
+void OpalFaxSession::OnReadPacket(OpalMediaTransport &, PBYTEArray data)
+{
+  m_readQueue.Enqueue(data);
+}
+
+
 bool OpalFaxSession::ReadData(RTP_DataFrame & frame)
 {
-  if (m_dataSocket == NULL)
+  PBYTEArray rawData;
+  if (!m_readQueue.Dequeue(rawData))
     return false;
 
+  if (rawData.GetSize() >= m_datagramSize) {
+    PTRACE(4, "UDPTL\tProbable RTP packet");
+    return true;
+  }
+
   if (m_rawUDPTL) {
-    frame.SetPayloadSize(m_datagramSize);
-    if (!m_dataSocket->Read(frame.GetPayloadPtr(), m_datagramSize))
-      return false;
-
-    if (m_shuttingDown) {
-      PTRACE(4, "UDPTL\tRead raw UDPTL shutting down");
-      return false;
-    }
-
-    frame.SetPayloadSize(m_dataSocket->GetLastReadCount());
-    PTRACE(4, "UDPTL\tRead raw UDPTL of size " << frame.GetPayloadSize());
+    frame.SetPayload(rawData, rawData.GetSize());
+    m_rxBytes += frame.GetPayloadSize();
+    ++m_rxPackets;
+    PTRACE(5, "UDPTL\tRead raw UDPTL, size " << frame.GetPayloadSize());
     return true;
   }
 
@@ -552,28 +456,18 @@ bool OpalFaxSession::ReadData(RTP_DataFrame & frame)
     return true;
   }
 
-  PBYTEArray thisUDPTL(m_datagramSize);
-  if (!m_dataSocket->Read(thisUDPTL.GetPointer(), m_datagramSize)) {
-    if (m_dataSocket->GetErrorCode(PChannel::LastReadError) == PChannel::BufferTooSmall) {
-      PTRACE(4, "UDPTL\tProbable RTP packet");
-      return true;
-    }
-    PTRACE(2, "UDPTL\tRead socket failed: " << m_dataSocket->GetErrorText(PChannel::LastReadError));
-    return false;
-  }
-
-  if (m_shuttingDown) {
-    PTRACE(4, "UDPTL\tRead UDPTL shutting down");
-    return false;
-  }
-
-  PINDEX pduSize = m_dataSocket->GetLastReadCount();
-      
-  PPER_Stream rawData(thisUDPTL);
-
+  PPER_Stream per(rawData);
   // Decode the PDU, but not if still receiving RTP
-  if (!m_receivedPacket->Decode(rawData) || rawData.GetPosition() < pduSize ||
-            (m_awaitingGoodPacket && m_receivedPacket->m_seq_number >= 32768)) {
+  if (  !m_receivedPacket->Decode(per) ||
+         per.GetPosition() < per.GetSize() ||
+        (m_awaitingGoodPacket &&
+          (
+            m_receivedPacket->m_primary_ifp_packet.GetSize() == 0 ||
+            m_receivedPacket->m_seq_number >= 32768
+          )
+        )
+     )
+  {
     if (++m_consecutiveBadPackets > 1000) {
       PTRACE(1, "T38_UDPTL\tRaw data decode failed 1000 times, remote probably not switched from audio, aborting!");
       return false;
@@ -582,14 +476,15 @@ bool OpalFaxSession::ReadData(RTP_DataFrame & frame)
 #if PTRACING
     const unsigned Level = m_awaitingGoodPacket ? 4 : 2;
     if (PTrace::CanTrace(Level)) {
-      ostream & trace = PTrace::Begin(Level, __FILE__, __LINE__);
+      ostream & trace = PTRACE_BEGIN(Level);
       trace << "UDPTL\t";
       if (m_awaitingGoodPacket)
         trace << "Probable RTP packet: " << rawData.GetSize() << " bytes.";
-      else
+      else {
         trace << "Raw data decode failure:\n  "
               << setprecision(2) << rawData << "\n  UDPTL = "
               << setprecision(2) << m_receivedPacket;
+      }
       trace << PTrace::End;
     }
 #endif
@@ -621,14 +516,26 @@ bool OpalFaxSession::ReadData(RTP_DataFrame & frame)
   SetFrameFromIFP(frame, m_receivedPacket->m_primary_ifp_packet, m_receivedPacket->m_seq_number);
   m_expectedSequenceNumber = m_receivedPacket->m_seq_number+1;
 
+  m_rxBytes += rawData.GetSize();
+  ++m_rxPackets;
+  m_missingPackets += missing;
+
   return true;
 }
 
 
+void OpalFaxSession::GetStatistics(OpalMediaStatistics & statistics, bool receiver) const
+{
+  statistics.m_totalBytes = receiver ? m_rxBytes : m_txBytes;
+  statistics.m_totalPackets = receiver ? m_rxPackets : m_txPackets;
+  statistics.m_packetsLost = receiver ? m_missingPackets : 0;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalFaxEndPoint::OpalFaxEndPoint(OpalManager & mgr, const char * g711Prefix, const char * t38Prefix)
-  : OpalLocalEndPoint(mgr, g711Prefix)
+  : OpalLocalEndPoint(mgr, g711Prefix, false)
   , m_t38Prefix(t38Prefix)
   , m_defaultDirectory(".")
 {
@@ -733,10 +640,10 @@ OpalMediaFormatList OpalFaxEndPoint::GetMediaFormats() const
 }
 
 
-void OpalFaxEndPoint::OnFaxCompleted(OpalFaxConnection & connection, bool PTRACE_PARAM(failed))
+void OpalFaxEndPoint::OnFaxCompleted(OpalFaxConnection & connection, bool failed)
 {
   PTRACE(3, "FAX\tFax " << (failed ? "failed" : "completed") << " on connection: " << connection);
-  connection.Release();
+  connection.Release(failed ? OpalConnection::EndedByMediaFailed : OpalConnection::EndedByLocalUser);
 }
 
 
@@ -755,6 +662,7 @@ OpalFaxConnection::OpalFaxConnection(OpalCall        & call,
   , m_disableT38(disableT38)
   , m_switchTime(0)
   , m_tiffFileFormat(TIFF_File_FormatName)
+  , m_completed(false)
 {
   SetFaxMediaFormatOptions(m_tiffFileFormat);
 
@@ -887,33 +795,13 @@ OpalMediaStream * OpalFaxConnection::CreateMediaStream(const OpalMediaFormat & m
 }
 
 
-void OpalFaxConnection::OnStartMediaPatch(OpalMediaPatch & patch)
-{
-  if (m_switchTime != 0)
-    patch.SetPaused(true);
-
-  OpalLocalConnection::OnStartMediaPatch(patch);
-}
-
-
 void OpalFaxConnection::OnClosedMediaStream(const OpalMediaStream & stream)
 {
   OpalLocalConnection::OnClosedMediaStream(stream);
 
-  bool bothClosed = false;
-  OpalMediaStreamPtr other;
-  if (stream.IsSource()) {
-    other = GetMediaStream(stream.GetID(), false);
-    bothClosed = other == NULL || !other->IsOpen();
-  }
-  else {
-    other = GetMediaStream(stream.GetID(), true);
-    bothClosed = other == NULL || !other->IsOpen();
-
+  if (stream.IsSink()) {
     PTRACE(4, "FAX\tTerminating fax in closed media stream id=" << stream.GetID());
-    OpalMediaPatchPtr patch = stream.GetPatch();
-    if (patch != NULL)
-      patch->ExecuteCommand(OpalFaxTerminate(), false);
+    stream.ExecuteCommand(OpalFaxTerminate());
 
     if (m_finalStatistics.m_fax.m_result < 0) {
       stream.GetStatistics(m_finalStatistics);
@@ -922,7 +810,8 @@ void OpalFaxConnection::OnClosedMediaStream(const OpalMediaStream & stream)
     }
   }
 
-  if (bothClosed) {
+  OpalMediaStreamPtr other = GetMediaStream(stream.GetID(), stream.IsSink());
+  if (other == NULL || !other->IsOpen()) {
     if (    m_finalStatistics.m_fax.m_result == 0 /* success!*/ ||
           !(m_finalStatistics.m_fax.m_result < 0  /* in progress */ || IsReleased() || ownerCall.IsSwitchingT38()))
       InternalOnFaxCompleted();
@@ -1062,20 +951,11 @@ void OpalFaxConnection::InternalOpenFaxStreams()
 
 void OpalFaxConnection::InternalOnFaxCompleted()
 {
-  if (m_completed.TestAndSet(true))
+  if (m_completed.exchange(true))
     return;
 
   PTRACE(4, "FAX", "OnFaxCompleted, result=" << m_finalStatistics.m_fax.m_result);
 
   OnFaxCompleted(m_finalStatistics.m_fax.m_result != OpalMediaStatistics::FaxSuccessful);
 }
-
-
-#else
-
-  #ifdef _MSC_VER
-    #pragma message("T.38 Fax (spandsp) support DISABLED")
-  #endif
-
 #endif // OPAL_FAX
-
