@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27092 $
+ * $Revision: 31564 $
  * $Author: rjongbloed $
- * $Date: 2012-03-01 21:21:33 -0600 (Thu, 01 Mar 2012) $
+ * $Date: 2014-03-12 02:19:57 +0000 (Wed, 12 Mar 2014) $
  */
 
 #include <ptlib.h>
@@ -40,9 +40,21 @@
 #include <opal/recording.h>
 
 
+#if OPAL_JAVA
+  // Include symbol hook so links in all the SWIG interface.
+  extern int opal_java_swig_wrapper_link;
+  static int const * const force_java_swig_wrapper_link = &opal_java_swig_wrapper_link;
+#endif
+
+#if OPAL_CSHARP
+  // Include symbol hook so links in all the SWIG interface.
+  extern int opal_csharp_swig_wrapper_link;
+  static int const * const force_csharp_swig_wrapper_link = &opal_csharp_swig_wrapper_link;
+#endif
+
 #if OPAL_RUBY
-extern "C" void Init_opal();
-static void (*dummy)() = Init_opal;
+  extern "C" void Init_opal();
+  static void (*dummy)() = Init_opal;
 #endif
 
 
